@@ -35,48 +35,16 @@ except ValueError:
 
 from qgis.core import QgsFeature, QgsGeometry
 
-try:
-    # Add a __nonzero__ method onto QPyNullVariant so we can check for null values easier.
-    #   >>> value = QPyNullVariant("int")
-    #   >>> if value:
-    #   >>>	  print "Not a null value"
-    from types import MethodType
-    from PyQt4.QtCore import QPyNullVariant
-
-    def __nonzero__(self):
-        return False
-
-    def __repr__(self):
-        return 'NULL'
-
-    def __eq__(self, other):
-        return isinstance(other, QPyNullVariant) or other is None
-
-    def __ne__(self, other):
-        return not isinstance(other, QPyNullVariant) and other is not None
-
-    QPyNullVariant.__nonzero__ = MethodType(__nonzero__, None, QPyNullVariant)
-    QPyNullVariant.__repr__ = MethodType(__repr__, None, QPyNullVariant)
-    QPyNullVariant.__eq__= MethodType(__eq__, None, QPyNullVariant)
-    QPyNullVariant.__ne__= MethodType(__ne__, None, QPyNullVariant)
-
-    # define a dummy QPyNullVariant instance NULL in qgis.core
-    # this is mainly used to compare against
-    # so one can write if feat['attr'] == NULL:
-    from qgis import core
-    core.NULL = QPyNullVariant( int )
-except ImportError:
-    pass
-
 
 def mapping_feature(feature):
     geom = feature.geometry()
     properties = {}
     fields = [field.name() for field in feature.fields()]
     properties = dict(zip(fields, feature.attributes()))
-    return {'type' : 'Feature',
-            'properties' : properties,
-            'geometry' : geom.__geo_interface__}
+    return {'type': 'Feature',
+            'properties': properties,
+            'geometry': geom.__geo_interface__}
+
 
 def mapping_geometry(geometry):
     geo = geometry.exportToGeoJSON()
