@@ -51,14 +51,17 @@ from MetaSearch.dialogs.newconnectiondialog import NewConnectionDialog
 from MetaSearch.dialogs.recorddialog import RecordDialog
 from MetaSearch.dialogs.xmldialog import XMLDialog
 from MetaSearch.util import (get_connections_from_file, get_ui_class,
-                             highlight_xml, normalize_text, open_url,
-                             render_template, StaticContext)
+                             get_help_url, highlight_xml, normalize_text,
+                             open_url, render_template, serialize_string,
+                             StaticContext)
 
 BASE_CLASS = get_ui_class('maindialog.ui')
 
 
 class MetaSearchDialog(QDialog, BASE_CLASS):
+
     """main dialogue"""
+
     def __init__(self, iface):
         """init window"""
 
@@ -72,6 +75,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.catalog_url = None
         self.context = StaticContext()
 
+        version = self.context.metadata.get('general', 'version')
+        self.setWindowTitle('MetaSearch %s' % version)
+
         self.rubber_band = QgsRubberBand(self.map, True)  # True = a polygon
         self.rubber_band.setColor(QColor(255, 0, 0, 75))
         self.rubber_band.setWidth(5)
@@ -121,6 +127,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.btnAddToWcs.clicked.connect(self.add_to_ows)
         self.btnShowXml.clicked.connect(self.show_xml)
 
+        # settings
+        self.radioTitleAsk.clicked.connect(self.set_ows_save_title_ask)
+        self.radioTitleNoAsk.clicked.connect(self.set_ows_save_title_no_ask)
+        self.radioTempName.clicked.connect(self.set_ows_save_temp_name)
+
         self.manageGui()
 
     def manageGui(self):
@@ -139,6 +150,16 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         self.reset_buttons()
 
+        # get preferred connection save strategy from settings and set it
+        save_strat = self.settings.value('/MetaSearch/ows_save_strategy',
+                                         'title_ask')
+        if save_strat == 'temp_name':
+            self.radioTempName.setChecked(True)
+        elif save_strat == 'title_no_ask':
+            self.radioTitleNoAsk.setChecked(True)
+        else:
+            self.radioTitleAsk.setChecked(True)
+
         # install proxy handler if specified in QGIS settings
         self.install_proxy()
 
@@ -376,10 +397,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             maxx = extent.xMaximum()
             maxy = extent.yMaximum()
 
-        self.leNorth.setText(str(maxy)[0:9])
-        self.leSouth.setText(str(miny)[0:9])
-        self.leWest.setText(str(minx)[0:9])
-        self.leEast.setText(str(maxx)[0:9])
+        self.leNorth.setText(unicode(maxy)[0:9])
+        self.leSouth.setText(unicode(miny)[0:9])
+        self.leWest.setText(unicode(minx)[0:9])
+        self.leEast.setText(unicode(maxx)[0:9])
 
     def set_bbox_global(self):
         """set global bounding box"""
@@ -447,12 +468,12 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         try:
             self.catalog.getrecords2(constraints=self.constraints,
                                      maxrecords=self.maxrecords, esn='full')
-        except ExceptionReport, err:
+        except ExceptionReport as err:
             QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Search error'),
                                 self.tr('Search error: %s') % err)
             return
-        except Exception, err:
+        except Exception as err:
             QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('Connection error'),
                                 self.tr('Connection error: %s') % err)
@@ -520,7 +541,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         identifier = get_item_data(item, 'identifier')
         try:
             record = self.catalog.records[identifier]
-        except KeyError, err:
+        except KeyError as err:
             QMessageBox.warning(self,
                                 self.tr('Record parsing error'),
                                 'Unable to locate record identifier')
@@ -537,11 +558,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
                     ctr = QgsCoordinateTransform(src, dst)
                     try:
                         geom.transform(ctr)
-                    except Exception, err:
+                    except Exception as err:
                         QMessageBox.warning(
                             self,
                             self.tr('Coordinate Transformation Error'),
-                            str(err))
+                            unicode(err))
                 self.rubber_band.setToGeometry(geom, None)
 
         # figure out if the data is interactive and can be operated on
@@ -623,9 +644,20 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
 
-        self.catalog.getrecords2(constraints=self.constraints,
-                                 maxrecords=self.maxrecords,
-                                 startposition=self.startfrom, esn='full')
+        try:
+            self.catalog.getrecords2(constraints=self.constraints,
+                                     maxrecords=self.maxrecords,
+                                     startposition=self.startfrom, esn='full')
+        except ExceptionReport as err:
+            QApplication.restoreOverrideCursor()
+            QMessageBox.warning(self, self.tr('Search error'),
+                                self.tr('Search error: %s') % err)
+            return
+        except Exception as err:
+            QApplication.restoreOverrideCursor()
+            QMessageBox.warning(self, self.tr('Connection error'),
+                                self.tr('Connection error: %s') % err)
+            return
 
         QApplication.restoreOverrideCursor()
 
@@ -634,6 +666,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
     def add_to_ows(self):
         """add to OWS provider connection list"""
 
+        conn_name_matches = []
+
         item = self.treeRecords.currentItem()
 
         if not item:
@@ -664,13 +698,24 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         keys = self.settings.childGroups()
         self.settings.endGroup()
 
+        for key in keys:
+            if key.startswith(sname):
+                conn_name_matches.append(key)
+        if conn_name_matches:
+            sname = conn_name_matches[-1]
+
         # check for duplicates
-        if sname in keys:
-            msg = self.tr('Connection %s exists. Overwrite?') % sname
-            res = QMessageBox.warning(self, self.tr('Saving server'), msg,
-                                      QMessageBox.Yes | QMessageBox.No)
-            if res != QMessageBox.Yes:
-                return
+        if sname in keys:  # duplicate found
+            if self.radioTitleAsk.isChecked():  # ask to overwrite
+                msg = self.tr('Connection %s exists. Overwrite?') % sname
+                res = QMessageBox.warning(self, self.tr('Saving server'), msg,
+                                          QMessageBox.Yes | QMessageBox.No)
+                if res != QMessageBox.Yes:  # assign new name with serial
+                    sname = serialize_string(sname)
+            elif self.radioTitleNoAsk.isChecked():  # don't ask to overwrite
+                pass
+            elif self.radioTempName.isChecked():  # use temp name
+                sname = serialize_string(sname)
 
         # no dups detected or overwrite is allowed
         self.settings.beginGroup('/Qgis/connections-%s' % stype[1])
@@ -735,12 +780,12 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout)
             cat.getrecordbyid(
                 [self.catalog.records[identifier].identifier])
-        except ExceptionReport, err:
+        except ExceptionReport as err:
             QApplication.restoreOverrideCursor()
             QMessageBox.warning(self, self.tr('GetRecords error'),
                                 self.tr('Error getting response: %s') % err)
             return
-        except KeyError, err:
+        except KeyError as err:
             QMessageBox.warning(self,
                                 self.tr('Record parsing error'),
                                 'Unable to locate record identifier')
@@ -796,7 +841,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
     def help(self):
         """launch help"""
 
-        open_url(self.context.metadata.get('general', 'homepage'))
+        open_url(get_help_url())
 
     def reject(self):
         """back out of dialogue"""
@@ -813,11 +858,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             self.catalog = CatalogueServiceWeb(self.catalog_url,
                                                timeout=self.timeout)
             return True
-        except ExceptionReport, err:
+        except ExceptionReport as err:
             msg = self.tr('Error connecting to service: %s') % err
-        except ValueError, err:
+        except ValueError as err:
             msg = self.tr('Value Error: %s') % err
-        except Exception, err:
+        except Exception as err:
             msg = self.tr('Unknown Error: %s') % err
 
         QMessageBox.warning(self, self.tr('CSW Connection error'), msg)
