@@ -24,6 +24,10 @@
 #
 ###############################################################################
 
+#avoid PendingDeprecationWarning from PyQt4.uic
+import warnings
+warnings.filterwarnings("ignore", category=PendingDeprecationWarning)
+
 import ConfigParser
 from gettext import gettext, ngettext
 import logging
@@ -36,13 +40,18 @@ from jinja2 import Environment, FileSystemLoader
 from pygments import highlight
 from pygments.lexers import XmlLexer
 from pygments.formatters import HtmlFormatter
+from PyQt4.QtCore import QSettings
 from PyQt4.QtGui import QMessageBox
 from PyQt4.uic import loadUiType
 
+from qgis.core import QGis
+
+
 LOGGER = logging.getLogger('MetaSearch')
 
 
 class StaticContext(object):
+
     """base configuration / scaffolding"""
 
     def __init__(self):
@@ -77,17 +86,16 @@ def get_connections_from_file(parent, filename):
     error = 0
     try:
         doc = etree.parse(filename).getroot()
-    except etree.ParseError, err:
+        if doc.tag != 'qgsCSWConnections':
+            error = 1
+            msg = parent.tr('Invalid CSW connections XML.')
+    except etree.ParseError as err:
         error = 1
         msg = parent.tr('Cannot parse XML file: %s' % err)
-    except IOError, err:
+    except IOError as err:
         error = 1
         msg = parent.tr('Cannot open file: %s' % err)
 
-    if doc.tag != 'qgsCSWConnections':
-        error = 1
-        msg = parent.tr('Invalid CSW connections XML.')
-
     if error == 1:
         QMessageBox.information(parent, parent.tr('Loading Connections'), msg)
         return
@@ -121,6 +129,18 @@ def highlight_xml(context, xml):
     return template.render(css=css, body=body)
 
 
+def get_help_url():
+    """return QGIS MetaSearch help documentation link"""
+
+    locale_name = QSettings().value('locale/userLocale')[0:2]
+    version = QGis.QGIS_VERSION[:3]
+
+    path = '%s/%s/docs/user_manual/plugins/plugins_metasearch.html' % \
+           (version, locale_name)
+
+    return '/'.join(['http://docs.qgis.org', path])
+
+
 def open_url(url):
     """open URL in web browser"""
 
@@ -131,3 +151,19 @@ def normalize_text(text):
     """tidy up string"""
 
     return text.replace('\n', '')
+
+
+def serialize_string(input_string):
+    """apply a serial counter to a string"""
+
+    s = input_string.strip().split()
+
+    last_token = s[-1]
+    all_other_tokens_as_string = input_string.replace(last_token, '')
+
+    if last_token.isdigit():
+        value = '%s%s' % (all_other_tokens_as_string, int(last_token) + 1)
+    else:
+        value = '%s 1' % input_string
+
+    return value
