@@ -30,7 +30,9 @@ import subprocess
 import platform
 from PyQt4.QtCore import QSettings
 from qgis.core import QgsApplication, QgsVectorFileWriter
+from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.ProcessingLog import ProcessingLog
+from processing.tools.system import isWindows, isMac
 
 try:
     from osgeo import gdal
@@ -41,6 +43,8 @@ except:
 
 class GdalUtils:
 
+    GDAL_HELP_PATH = 'GDAL_HELP_PATH'
+
     supportedRasters = None
 
     @staticmethod
@@ -163,3 +167,21 @@ class GdalUtils:
                 escaped = s
             joined += escaped + ' '
         return joined.strip()
+
+    @staticmethod
+    def gdalHelpPath():
+        helpPath = ProcessingConfig.getSetting(GdalUtils.GDAL_HELP_PATH)
+
+        if helpPath is None:
+            if isWindows():
+                pass
+            elif isMac():
+                pass
+            else:
+                searchPaths = ['/usr/share/doc/libgdal-doc/gdal']
+                for path in searchPaths:
+                    if os.path.exists(path):
+                        helpPath = os.path.abspath(path)
+                        break
+
+        return helpPath if helpPath is not None else 'http://www.gdal.org/'
