@@ -40,11 +40,14 @@ class ExtractProjection(GdalAlgorithm):
     PRJ_FILE = 'PRJ_FILE'
 
     def defineCharacteristics(self):
-        self.name = 'Extract projection'
-        self.group = '[GDAL] Projections'
+        self.name, self.i18n_name = self.trAlgorithm('Extract projection')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Projections')
         self.addParameter(ParameterRaster(self.INPUT, self.tr('Input file')))
         self.addParameter(ParameterBoolean(self.PRJ_FILE,
-            self.tr('Create also .prj file'), False))
+                                           self.tr('Create also .prj file'), False))
+
+    def getConsoleCommands(self):
+        return ["extractprojection"]
 
     def processAlgorithm(self, progress):
         rasterPath = self.getParameterValue(self.INPUT)
@@ -74,7 +77,7 @@ class ExtractProjection(GdalAlgorithm):
         wld.write('%0.8f\n' % geotransform[2])
         wld.write('%0.8f\n' % geotransform[5])
         wld.write('%0.8f\n' % (geotransform[0] + 0.5 * geotransform[1] + 0.5
-                  * geotransform[2]))
+                               * geotransform[2]))
         wld.write('%0.8f\n' % (geotransform[3] + 0.5 * geotransform[4] + 0.5
-                  * geotransform[5]))
+                               * geotransform[5]))
         wld.close()
