@@ -41,21 +41,21 @@ class pct2rgb(GdalAlgorithm):
     NBAND = 'NBAND'
 
     def defineCharacteristics(self):
-        self.name = 'PCT to RGB'
-        self.group = '[GDAL] Conversion'
+        self.name, self.i18n_name = self.trAlgorithm('PCT to RGB')
+        self.group, self.i18n_group = self.trAlgorithm('[GDAL] Conversion')
         self.addParameter(ParameterRaster(pct2rgb.INPUT,
-            self.tr('Input layer'), False))
+                                          self.tr('Input layer'), False))
         options = []
         for i in range(25):
-            options.append(str(i + 1))
+            options.append(unicode(i + 1))
         self.addParameter(ParameterSelection(pct2rgb.NBAND,
-            self.tr('Band to convert'), options))
+                                             self.tr('Band to convert'), options))
         self.addOutput(OutputRaster(pct2rgb.OUTPUT, self.tr('PCT to RGB')))
 
-    def processAlgorithm(self, progress):
+    def getConsoleCommands(self):
         arguments = []
         arguments.append('-b')
-        arguments.append(str(self.getParameterValue(pct2rgb.NBAND) + 1))
+        arguments.append(unicode(self.getParameterValue(pct2rgb.NBAND) + 1))
         arguments.append('-of')
         out = self.getOutputValue(pct2rgb.OUTPUT)
         arguments.append(GdalUtils.getFormatShortNameFromFilename(out))
@@ -69,4 +69,4 @@ class pct2rgb(GdalAlgorithm):
         else:
             commands = ['pct2rgb.py', GdalUtils.escapeAndJoin(arguments)]
 
-        GdalUtils.runGdal(commands, progress)
+        return commands
