@@ -31,6 +31,7 @@ import uuid
 import importlib
 import re
 
+from PyQt4.QtCore import QCoreApplication
 from PyQt4.QtGui import QIcon
 
 from qgis.core import QgsRasterLayer
@@ -48,6 +49,9 @@ from GrassUtils import GrassUtils
 
 from processing.tools import dataobjects, system
 
+pluginPath = os.path.normpath(os.path.join(
+    os.path.split(os.path.dirname(__file__))[0], os.pardir))
+
 
 class GrassAlgorithm(GeoAlgorithm):
 
@@ -62,24 +66,28 @@ class GrassAlgorithm(GeoAlgorithm):
 
     def __init__(self, descriptionfile):
         GeoAlgorithm.__init__(self)
+        self.hardcodedStrings = []
         self.descriptionFile = descriptionfile
         self.defineCharacteristicsFromFile()
         self.numExportedLayers = 0
 
-        # GRASS console output, needed to do postprocessing in case GRASS
-        # dumps results to the console
-        self.consoleOutput = []
-
     def getCopy(self):
         newone = GrassAlgorithm(self.descriptionFile)
         newone.provider = self.provider
         return newone
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/grass.png')
+        return QIcon(os.path.join(pluginPath, 'images', 'grass.svg'))
 
     def help(self):
-        return False, 'http://grass.osgeo.org/grass64/manuals/' + self.grassName + '.html'
+        helpPath = GrassUtils.grassHelpPath()
+        if helpPath == '':
+            return False, None
+
+        if os.path.exists(helpPath):
+            return False, QUrl.fromLocalFile(os.path.join(helpPath, '{}.html'.format(self.grassName))).toString()
+        else:
+            return False, helpPath + '{}.html'.format(self.grassName)
 
     def getParameterDescriptions(self):
         descs = {}
@@ -107,8 +115,13 @@ class GrassAlgorithm(GeoAlgorithm):
         self.grassName = line
         line = lines.readline().strip('\n').strip()
         self.name = line
+        self.i18n_name = QCoreApplication.translate("GrassAlgorithm", line)
+        if " - " not in self.name:
+            self.name = self.grassName + " - " + self.name
+            self.i18n_name = self.grassName + " - " + self.i18n_name
         line = lines.readline().strip('\n').strip()
         self.group = line
+        self.i18n_group = QCoreApplication.translate("GrassAlgorithm", line)
         hasRasterOutput = False
         hasVectorInput = False
         vectorOutputs = 0
@@ -116,7 +129,9 @@ class GrassAlgorithm(GeoAlgorithm):
         while line != '':
             try:
                 line = line.strip('\n').strip()
-                if line.startswith('Parameter'):
+                if line.startswith('Hardcoded'):
+                    self.hardcodedStrings.append(line[len('Hardcoded|'):])
+                elif line.startswith('Parameter'):
                     parameter = getParameterFromString(line)
                     self.addParameter(parameter)
                     if isinstance(parameter, ParameterVector):
@@ -135,15 +150,21 @@ class GrassAlgorithm(GeoAlgorithm):
                         hasRasterOutput = True
                     elif isinstance(output, OutputVector):
                         vectorOutputs += 1
+                    if isinstance(output, OutputHTML):
+                        self.addOutput(OutputFile("rawoutput", output.description +
+                                                  " (raw output)", "txt"))
                 line = lines.readline().strip('\n').strip()
-            except Exception, e:
-                ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
+            except Exception as e:
+                ProcessingLog.addToLog(
+                    ProcessingLog.LOG_ERROR,
                     self.tr('Could not open GRASS algorithm: %s.\n%s' % (self.descriptionFile, line)))
                 raise e
         lines.close()
 
-        self.addParameter(ParameterExtent(self.GRASS_REGION_EXTENT_PARAMETER,
-            self.tr('GRASS region extent')))
+        self.addParameter(ParameterExtent(
+            self.GRASS_REGION_EXTENT_PARAMETER,
+            self.tr('GRASS region extent'))
+        )
         if hasRasterOutput:
             self.addParameter(ParameterNumber(
                 self.GRASS_REGION_CELLSIZE_PARAMETER,
@@ -176,7 +197,7 @@ class GrassAlgorithm(GeoAlgorithm):
                     else:
                         layer = dataobjects.getObjectFromUri(param.value)
                     cellsize = max(cellsize, (layer.extent().xMaximum()
-                                   - layer.extent().xMinimum())
+                                              - layer.extent().xMinimum())
                                    / layer.width())
                 elif isinstance(param, ParameterMultipleInput):
 
@@ -187,10 +208,11 @@ class GrassAlgorithm(GeoAlgorithm):
                             cellsize = max(cellsize, (
                                 layer.extent().xMaximum()
                                 - layer.extent().xMinimum())
-                                / layer.width())
+                                / layer.width()
+                            )
 
         if cellsize == 0:
-            cellsize = 1
+            cellsize = 100
         return cellsize
 
     def processAlgorithm(self, progress):
@@ -253,7 +275,10 @@ class GrassAlgorithm(GeoAlgorithm):
                         else:
                             self.setSessionProjectionFromLayer(layer, commands)
                             commands.append(self.exportRasterLayer(layer))
-                elif param.datatype == ParameterMultipleInput.TYPE_VECTOR_ANY:
+                elif param.datatype in [ParameterMultipleInput.TYPE_VECTOR_ANY,
+                                        ParameterMultipleInput.TYPE_VECTOR_LINE,
+                                        ParameterMultipleInput.TYPE_VECTOR_POLYGON,
+                                        ParameterMultipleInput.TYPE_VECTOR_POINT]:
                     for layer in layers:
                         if layer in self.exportedLayers.keys():
                             continue
@@ -264,18 +289,18 @@ class GrassAlgorithm(GeoAlgorithm):
         self.setSessionProjectionFromProject(commands)
 
         region = \
-            str(self.getParameterValue(self.GRASS_REGION_EXTENT_PARAMETER))
+            unicode(self.getParameterValue(self.GRASS_REGION_EXTENT_PARAMETER))
         regionCoords = region.split(',')
         command = 'g.region'
-        command += ' n=' + str(regionCoords[3])
-        command += ' s=' + str(regionCoords[2])
-        command += ' e=' + str(regionCoords[1])
-        command += ' w=' + str(regionCoords[0])
+        command += ' n=' + unicode(regionCoords[3])
+        command += ' s=' + unicode(regionCoords[2])
+        command += ' e=' + unicode(regionCoords[1])
+        command += ' w=' + unicode(regionCoords[0])
         cellsize = self.getParameterValue(self.GRASS_REGION_CELLSIZE_PARAMETER)
         if cellsize:
-            command += ' res=' + str(cellsize)
+            command += ' res=' + unicode(cellsize)
         else:
-            command += ' res=' + str(self.getDefaultCellsize())
+            command += ' res=' + unicode(self.getDefaultCellsize())
         alignToResolution = \
             self.getParameterValue(self.GRASS_REGION_ALIGN_TO_RESOLUTION)
         if alignToResolution:
@@ -285,49 +310,45 @@ class GrassAlgorithm(GeoAlgorithm):
         # 2: Set parameters and outputs
 
         command = self.grassName
+        command += ' ' + ' '.join(self.hardcodedStrings)
+
         for param in self.parameters:
             if param.value is None or param.value == '':
                 continue
-            if param.name in [ self.GRASS_REGION_CELLSIZE_PARAMETER, self.GRASS_REGION_EXTENT_PARAMETER, self.GRASS_MIN_AREA_PARAMETER, self.GRASS_SNAP_TOLERANCE_PARAMETER, self.GRASS_OUTPUT_TYPE_PARAMETER, self.GRASS_REGION_ALIGN_TO_RESOLUTION ]:
+            if param.name in [self.GRASS_REGION_CELLSIZE_PARAMETER, self.GRASS_REGION_EXTENT_PARAMETER,
+                              self.GRASS_MIN_AREA_PARAMETER, self.GRASS_SNAP_TOLERANCE_PARAMETER,
+                              self.GRASS_OUTPUT_TYPE_PARAMETER, self.GRASS_REGION_ALIGN_TO_RESOLUTION]:
                 continue
             if isinstance(param, (ParameterRaster, ParameterVector)):
                 value = param.value
                 if value in self.exportedLayers.keys():
-                    command += ' ' + param.name + '=' \
-                        + self.exportedLayers[value]
+                    command += ' %s="%s"' % (param.name, self.exportedLayers[value])
                 else:
-                    command += ' ' + param.name + '=' + value
+                    command += ' %s="%s"' % (param.name, value)
             elif isinstance(param, ParameterMultipleInput):
                 s = param.value
                 for layer in self.exportedLayers.keys():
                     s = s.replace(layer, self.exportedLayers[layer])
                 s = s.replace(';', ',')
-                command += ' ' + param.name + '=' + s
+                command += ' %s="%s"' % (param.name, s)
             elif isinstance(param, ParameterBoolean):
                 if param.value:
                     command += ' ' + param.name
             elif isinstance(param, ParameterSelection):
                 idx = int(param.value)
-                command += ' ' + param.name + '=' + str(param.options[idx])
+                command += ' ' + param.name + '=' + unicode(param.options[idx])
             elif isinstance(param, ParameterString):
-                command += ' ' + param.name + '="' + str(param.value) + '"'
+                command += ' ' + param.name + '="' + unicode(param.value) + '"'
             else:
-                command += ' ' + param.name + '="' + str(param.value) + '"'
+                command += ' ' + param.name + '="' + unicode(param.value) + '"'
 
-        uniqueSufix = str(uuid.uuid4()).replace('-', '')
+        uniqueSufix = unicode(uuid.uuid4()).replace('-', '')
         for out in self.outputs:
             if isinstance(out, OutputFile):
-                if out.name == 'outputtext':
-                    # The 'outputtext' file is generated by piping output
-                    # from GRASS, is not an actual grass command
-                    command += ' > ' + out.value
-                else:
-                    command += ' ' + out.name + '="' + out.value + '"'
+                command += ' > ' + out.value
             elif not isinstance(out, OutputHTML):
-                # Html files are not generated by GRASS, only by us to
-                # decorate GRASS output, so we skip them. An output name
-                # to make sure it is unique if the session uses this
-                # algorithm several times.
+                # We add an output name to make sure it is unique if the session
+                # uses this algorithm several times.
                 uniqueOutputName = out.name + uniqueSufix
                 command += ' ' + out.name + '=' + uniqueOutputName
 
@@ -342,7 +363,7 @@ class GrassAlgorithm(GeoAlgorithm):
 
         for out in self.outputs:
             if isinstance(out, OutputRaster):
-                filename = out.value
+                filename = out.getCompatibleFileName(self)
 
                 # Raster layer output: adjust region to layer before
                 # exporting
@@ -369,11 +390,11 @@ class GrassAlgorithm(GeoAlgorithm):
                     outputCommands.append(command)
 
             if isinstance(out, OutputVector):
-                filename = out.value
+                filename = out.getCompatibleFileName(self)
                 command = 'v.out.ogr -s -c -e -z input=' + out.name + uniqueSufix
-                command += ' dsn="' + os.path.dirname(out.value) + '"'
+                command += ' dsn="' + os.path.dirname(filename) + '"'
                 command += ' format=ESRI_Shapefile'
-                command += ' olayer=' + os.path.basename(out.value)[:-4]
+                command += ' olayer="%s"' % os.path.splitext(os.path.basename(filename))[0]
                 typeidx = \
                     self.getParameterValue(self.GRASS_OUTPUT_TYPE_PARAMETER)
                 outtype = ('auto' if typeidx
@@ -391,9 +412,14 @@ class GrassAlgorithm(GeoAlgorithm):
             loglines.append(line)
         if ProcessingConfig.getSetting(GrassUtils.GRASS_LOG_COMMANDS):
             ProcessingLog.addToLog(ProcessingLog.LOG_INFO, loglines)
-        self.consoleOutput = GrassUtils.executeGrass(commands, progress,
-                outputCommands)
-        self.postProcessResults()
+        GrassUtils.executeGrass(commands, progress, outputCommands)
+
+        for out in self.outputs:
+            if isinstance(out, OutputHTML):
+                with open(self.getOutputFromName("rawoutput").value) as f:
+                    rawOutput = "".join(f.readlines())
+                with open(out.value, "w") as f:
+                    f.write("<pre>%s</pre>" % rawOutput)
 
         # If the session has been created outside of this algorithm, add
         # the new GRASS layers to it otherwise finish the session
@@ -402,16 +428,6 @@ class GrassAlgorithm(GeoAlgorithm):
         else:
             GrassUtils.endGrassSession()
 
-    def postProcessResults(self):
-        name = self.commandLineName().replace('.', '_')[len('grass:'):]
-        try:
-            module = importlib.import_module('processing.algs.grass.ext.' + name)
-        except ImportError:
-            return
-        if hasattr(module, 'postProcessResults'):
-            func = getattr(module, 'postProcessResults')
-            func(self)
-
     def exportVectorLayer(self, orgFilename):
 
         # TODO: improve this. We are now exporting if it is not a shapefile,
@@ -436,18 +452,18 @@ class GrassAlgorithm(GeoAlgorithm):
         self.exportedLayers[orgFilename] = destFilename
         command = 'v.in.ogr'
         min_area = self.getParameterValue(self.GRASS_MIN_AREA_PARAMETER)
-        command += ' min_area=' + str(min_area)
+        command += ' min_area=' + unicode(min_area)
         snap = self.getParameterValue(self.GRASS_SNAP_TOLERANCE_PARAMETER)
-        command += ' snap=' + str(snap)
-        command += ' dsn="' + os.path.dirname(filename) + '"'
-        command += ' layer=' + os.path.basename(filename)[:-4]
+        command += ' snap=' + unicode(snap)
+        command += ' dsn="%s"' % os.path.dirname(filename)
+        command += ' layer="%s"' % os.path.splitext(os.path.basename(filename)[:-4])[0]
         command += ' output=' + destFilename
         command += ' --overwrite -o'
         return command
 
     def setSessionProjectionFromProject(self, commands):
         if not GrassUtils.projectionSet:
-            proj4 = iface.mapCanvas().mapRenderer().destinationCrs().toProj4()
+            proj4 = iface.mapCanvas().mapSettings().destinationCrs().toProj4()
             command = 'g.proj'
             command += ' -c'
             command += ' proj4="' + proj4 + '"'
@@ -458,7 +474,7 @@ class GrassAlgorithm(GeoAlgorithm):
         if not GrassUtils.projectionSet:
             qGisLayer = dataobjects.getObjectFromUri(layer)
             if qGisLayer:
-                proj4 = str(qGisLayer.crs().toProj4())
+                proj4 = unicode(qGisLayer.crs().toProj4())
                 command = 'g.proj'
                 command += ' -c'
                 command += ' proj4="' + proj4 + '"'
@@ -479,23 +495,15 @@ class GrassAlgorithm(GeoAlgorithm):
         return command
 
     def getTempFilename(self):
-        filename = 'tmp' + str(time.time()).replace('.', '') \
-            + str(system.getNumExportedLayers())
+        filename = 'tmp' + unicode(time.time()).replace('.', '') \
+            + unicode(system.getNumExportedLayers())
         return filename
 
     def commandLineName(self):
         return 'grass:' + self.name[:self.name.find(' ')]
 
     def checkBeforeOpeningParametersDialog(self):
-        msg = GrassUtils.checkGrassIsInstalled()
-        if msg is not None:
-            html = self.tr(
-                '<p>This algorithm requires GRASS to be run. Unfortunately, '
-                'it seems that GRASS is not installed in your system, or it '
-                'is not correctly configured to be used from QGIS</p>'
-                '<p><a href="http://docs.qgis.org/testing/en/docs/user_manual/processing/3rdParty.html">Click here</a> '
-                'to know more about how to install and configure GRASS to be used with QGIS</p>')
-            return html
+        return GrassUtils.checkGrassIsInstalled()
 
     def checkParameterValuesBeforeExecuting(self):
         name = self.commandLineName().replace('.', '_')[len('grass:'):]
@@ -506,21 +514,3 @@ class GrassAlgorithm(GeoAlgorithm):
         if hasattr(module, 'checkParameterValuesBeforeExecuting'):
             func = getattr(module, 'checkParameterValuesBeforeExecuting')
             return func(self)
-
-    def getPostProcessingErrorMessage(self, wrongLayers):
-        html = GeoAlgorithm.getPostProcessingErrorMessage(self, wrongLayers)
-        msg = GrassUtils.checkGrassIsInstalled(True)
-        html += self.tr(
-            '<p>This algorithm requires GRASS to be run. A test to check '
-            'if GRASS is correctly installed and configured in your system '
-            'has been performed, with the following result:</p><ul><i>')
-        if msg is None:
-            html += self.tr('GRASS seems to be correctly installed and '
-                            'configured</i></li></ul>')
-        else:
-            html += msg + '</i></li></ul>'
-            html += self.tr(
-                '<p><a href="http://docs.qgis.org/testing/en/docs/user_manual/processing/3rdParty.html">Click here</a> '
-                'to know more about how to install and configure GRASS to be used with QGIS</p>')
-
-        return html
