@@ -30,23 +30,27 @@ import time
 import uuid
 import importlib
 
+from PyQt4.QtCore import QCoreApplication, QUrl
+from PyQt4.QtGui import QIcon
+
 from qgis.core import QgsRasterLayer
 from qgis.utils import iface
 
-from PyQt4.QtGui import QIcon
-
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.ProcessingLog import ProcessingLog
 from processing.core.GeoAlgorithmExecutionException import GeoAlgorithmExecutionException
 
 from processing.core.parameters import getParameterFromString, ParameterVector, ParameterMultipleInput, ParameterExtent, ParameterNumber, ParameterSelection, ParameterRaster, ParameterTable, ParameterBoolean, ParameterString
-from processing.core.outputs import getOutputFromString, OutputRaster, OutputVector, OutputHTML, OutputFile
+from processing.core.outputs import getOutputFromString, OutputRaster, OutputVector, OutputFile, OutputHTML
 
 from Grass7Utils import Grass7Utils
 
 from processing.tools import dataobjects, system
 
+pluginPath = os.path.normpath(os.path.join(
+    os.path.split(os.path.dirname(__file__))[0], os.pardir))
+
 
 class Grass7Algorithm(GeoAlgorithm):
 
@@ -61,13 +65,18 @@ class Grass7Algorithm(GeoAlgorithm):
 
     def __init__(self, descriptionfile):
         GeoAlgorithm.__init__(self)
+        self.hardcodedStrings = []
         self.descriptionFile = descriptionfile
         self.defineCharacteristicsFromFile()
         self.numExportedLayers = 0
+        self.uniqueSufix = unicode(uuid.uuid4()).replace('-', '')
 
-        # GRASS GIS 7 console output, needed to do postprocessing in case GRASS
-        # dumps results to the console
-        self.consoleOutput = []
+        # Use the ext mechanism
+        name = self.commandLineName().replace('.', '_')[len('grass7:'):]
+        try:
+            self.module = importlib.import_module('processing.algs.grass7.ext.' + name)
+        except ImportError:
+            self.module = None
 
     def getCopy(self):
         newone = Grass7Algorithm(self.descriptionFile)
@@ -75,11 +84,17 @@ class Grass7Algorithm(GeoAlgorithm):
         return newone
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/grass.png')
+        return QIcon(os.path.join(pluginPath, 'images', 'grass.svg'))
 
     def help(self):
-        return False, 'http://grass.osgeo.org/grass70/manuals/' + self.grass7Name \
-            + '.html'
+        helpPath = Grass7Utils.grassHelpPath()
+        if helpPath == '':
+            return False, None
+
+        if os.path.exists(helpPath):
+            return False, QUrl.fromLocalFile(os.path.join(helpPath, '{}.html'.format(self.grass7Name))).toString()
+        else:
+            return False, helpPath + '{}.html'.format(self.grass7Name)
 
     def getParameterDescriptions(self):
         descs = {}
@@ -107,8 +122,13 @@ class Grass7Algorithm(GeoAlgorithm):
         self.grass7Name = line
         line = lines.readline().strip('\n').strip()
         self.name = line
+        self.i18n_name = QCoreApplication.translate("GrassAlgorithm", line)
+        if " - " not in self.name:
+            self.name = self.grass7Name + " - " + self.name
+            self.i18n_name = self.grass7Name + " - " + self.i18n_name
         line = lines.readline().strip('\n').strip()
         self.group = line
+        self.i18n_group = QCoreApplication.translate("GrassAlgorithm", line)
         hasRasterOutput = False
         hasVectorInput = False
         vectorOutputs = 0
@@ -116,7 +136,9 @@ class Grass7Algorithm(GeoAlgorithm):
         while line != '':
             try:
                 line = line.strip('\n').strip()
-                if line.startswith('Parameter'):
+                if line.startswith('Hardcoded'):
+                    self.hardcodedStrings.append(line[len('Hardcoded|'):])
+                elif line.startswith('Parameter'):
                     parameter = getParameterFromString(line)
                     self.addParameter(parameter)
                     if isinstance(parameter, ParameterVector):
@@ -135,8 +157,11 @@ class Grass7Algorithm(GeoAlgorithm):
                         hasRasterOutput = True
                     elif isinstance(output, OutputVector):
                         vectorOutputs += 1
+                    if isinstance(output, OutputHTML):
+                        self.addOutput(OutputFile("rawoutput", output.description +
+                                                  " (raw output)", "txt"))
                 line = lines.readline().strip('\n').strip()
-            except Exception, e:
+            except Exception as e:
                 ProcessingLog.addToLog(
                     ProcessingLog.LOG_ERROR,
                     self.tr('Could not open GRASS GIS 7 algorithm: %s\n%s' % (self.descriptionFile, line)))
@@ -179,7 +204,7 @@ class Grass7Algorithm(GeoAlgorithm):
                     else:
                         layer = dataobjects.getObjectFromUri(param.value)
                     cellsize = max(cellsize, (layer.extent().xMaximum()
-                                   - layer.extent().xMinimum())
+                                              - layer.extent().xMinimum())
                                    / layer.width())
                 elif isinstance(param, ParameterMultipleInput):
 
@@ -194,7 +219,7 @@ class Grass7Algorithm(GeoAlgorithm):
                             )
 
         if cellsize == 0:
-            cellsize = 1
+            cellsize = 100
         return cellsize
 
     def processAlgorithm(self, progress):
@@ -205,9 +230,10 @@ class Grass7Algorithm(GeoAlgorithm):
                     self.tr('GRASS GIS 7 folder is not configured. Please '
                             'configure it before running GRASS GIS 7 algorithms.'))
 
-        commands = []
+        # Create brand new commands lists
+        self.commands = []
+        self.outputCommands = []
         self.exportedLayers = {}
-        outputCommands = []
 
         # If GRASS session has been created outside of this algorithm then
         # get the list of layers loaded in GRASS otherwise start a new
@@ -218,8 +244,57 @@ class Grass7Algorithm(GeoAlgorithm):
         else:
             Grass7Utils.startGrass7Session()
 
-        # 1: Export layer to grass mapset
+        # Handle ext functions for inputs/command/outputs
+        if self.module:
+            if hasattr(self.module, 'processInputs'):
+                func = getattr(self.module, 'processInputs')
+                func(self)
+            else:
+                self.processInputs()
 
+            if hasattr(self.module, 'processCommand'):
+                func = getattr(self.module, 'processCommand')
+                func(self)
+            else:
+                self.processCommand()
+
+            if hasattr(self.module, 'processOutputs'):
+                func = getattr(self.module, 'processOutputs')
+                func(self)
+            else:
+                self.processOutputs()
+        else:
+            self.processInputs()
+            self.processCommand()
+            self.processOutputs()
+
+        # Run GRASS
+        loglines = []
+        loglines.append(self.tr('GRASS GIS 7 execution commands'))
+        for line in self.commands:
+            progress.setCommand(line)
+            loglines.append(line)
+        if ProcessingConfig.getSetting(Grass7Utils.GRASS_LOG_COMMANDS):
+            ProcessingLog.addToLog(ProcessingLog.LOG_INFO, loglines)
+
+        Grass7Utils.executeGrass7(self.commands, progress, self.outputCommands)
+
+        for out in self.outputs:
+            if isinstance(out, OutputHTML):
+                with open(self.getOutputFromName("rawoutput").value) as f:
+                    rawOutput = "".join(f.readlines())
+                with open(out.value, "w") as f:
+                    f.write("<pre>%s</pre>" % rawOutput)
+
+        # If the session has been created outside of this algorithm, add
+        # the new GRASS GIS 7 layers to it otherwise finish the session
+        if existingSession:
+            Grass7Utils.addSessionLayers(self.exportedLayers)
+        else:
+            Grass7Utils.endGrass7Session()
+
+    def processInputs(self):
+        """Prepare the GRASS import commands"""
         for param in self.parameters:
             if isinstance(param, ParameterRaster):
                 if param.value is None:
@@ -231,8 +306,8 @@ class Grass7Algorithm(GeoAlgorithm):
                 if value in self.exportedLayers.keys():
                     continue
                 else:
-                    self.setSessionProjectionFromLayer(value, commands)
-                    commands.append(self.exportRasterLayer(value))
+                    self.setSessionProjectionFromLayer(value, self.commands)
+                    self.commands.append(self.exportRasterLayer(value))
             if isinstance(param, ParameterVector):
                 if param.value is None:
                     continue
@@ -240,8 +315,8 @@ class Grass7Algorithm(GeoAlgorithm):
                 if value in self.exportedLayers.keys():
                     continue
                 else:
-                    self.setSessionProjectionFromLayer(value, commands)
-                    commands.append(self.exportVectorLayer(value))
+                    self.setSessionProjectionFromLayer(value, self.commands)
+                    self.commands.append(self.exportVectorLayer(value))
             if isinstance(param, ParameterTable):
                 pass
             if isinstance(param, ParameterMultipleInput):
@@ -255,49 +330,51 @@ class Grass7Algorithm(GeoAlgorithm):
                         if layer in self.exportedLayers.keys():
                             continue
                         else:
-                            self.setSessionProjectionFromLayer(layer, commands)
-                            commands.append(self.exportRasterLayer(layer))
-                elif param.datatype == ParameterMultipleInput.TYPE_VECTOR_ANY:
+                            self.setSessionProjectionFromLayer(layer, self.commands)
+                            self.commands.append(self.exportRasterLayer(layer))
+                elif param.datatype in [ParameterMultipleInput.TYPE_VECTOR_ANY,
+                                        ParameterMultipleInput.TYPE_VECTOR_LINE,
+                                        ParameterMultipleInput.TYPE_VECTOR_POLYGON,
+                                        ParameterMultipleInput.TYPE_VECTOR_POINT]:
                     for layer in layers:
                         if layer in self.exportedLayers.keys():
                             continue
                         else:
-                            self.setSessionProjectionFromLayer(layer, commands)
-                            commands.append(self.exportVectorLayer(layer))
+                            self.setSessionProjectionFromLayer(layer, self.commands)
+                            self.commands.append(self.exportVectorLayer(layer))
 
-        self.setSessionProjectionFromProject(commands)
+        self.setSessionProjectionFromProject(self.commands)
 
-        region = str(self.getParameterValue(self.GRASS_REGION_EXTENT_PARAMETER))
+        region = \
+            unicode(self.getParameterValue(self.GRASS_REGION_EXTENT_PARAMETER))
         regionCoords = region.split(',')
         command = 'g.region'
         command += ' -a'
-        command += ' n=' + str(regionCoords[3])
-        command += ' s=' + str(regionCoords[2])
-        command += ' e=' + str(regionCoords[1])
-        command += ' w=' + str(regionCoords[0])
+        command += ' n=' + unicode(regionCoords[3])
+        command += ' s=' + unicode(regionCoords[2])
+        command += ' e=' + unicode(regionCoords[1])
+        command += ' w=' + unicode(regionCoords[0])
         cellsize = self.getParameterValue(self.GRASS_REGION_CELLSIZE_PARAMETER)
         if cellsize:
-            command += ' res=' + str(cellsize)
+            command += ' res=' + unicode(cellsize)
         else:
-            command += ' res=' + str(self.getDefaultCellsize())
+            command += ' res=' + unicode(self.getDefaultCellsize())
         alignToResolution = \
             self.getParameterValue(self.GRASS_REGION_ALIGN_TO_RESOLUTION)
         if alignToResolution:
             command += ' -a'
-        commands.append(command)
-
-        # 2: Set parameters and outputs
+        self.commands.append(command)
 
+    def processCommand(self):
+        """Prepare the GRASS algorithm command"""
         command = self.grass7Name
+        command += ' ' + ' '.join(self.hardcodedStrings)
+
+        # Add algorithm command
         for param in self.parameters:
             if param.value is None or param.value == '':
                 continue
-            if param.name == self.GRASS_REGION_CELLSIZE_PARAMETER \
-               or param.name == self.GRASS_REGION_EXTENT_PARAMETER \
-               or param.name == self.GRASS_MIN_AREA_PARAMETER \
-               or param.name == self.GRASS_SNAP_TOLERANCE_PARAMETER \
-               or param.name == self.GRASS_OUTPUT_TYPE_PARAMETER \
-               or param.name == self.GRASS_REGION_ALIGN_TO_RESOLUTION:
+            if param.name in [self.GRASS_REGION_CELLSIZE_PARAMETER, self.GRASS_REGION_EXTENT_PARAMETER, self.GRASS_MIN_AREA_PARAMETER, self.GRASS_SNAP_TOLERANCE_PARAMETER, self.GRASS_OUTPUT_TYPE_PARAMETER, self.GRASS_REGION_ALIGN_TO_RESOLUTION]:
                 continue
             if isinstance(param, (ParameterRaster, ParameterVector)):
                 value = param.value
@@ -317,27 +394,19 @@ class Grass7Algorithm(GeoAlgorithm):
                     command += ' ' + param.name
             elif isinstance(param, ParameterSelection):
                 idx = int(param.value)
-                command += ' ' + param.name + '=' + str(param.options[idx])
+                command += ' ' + param.name + '=' + unicode(param.options[idx])
             elif isinstance(param, ParameterString):
-                command += ' ' + param.name + '="' + str(param.value) + '"'
+                command += ' ' + param.name + '="' + unicode(param.value) + '"'
             else:
-                command += ' ' + param.name + '="' + str(param.value) + '"'
+                command += ' ' + param.name + '="' + unicode(param.value) + '"'
 
-        uniqueSufix = str(uuid.uuid4()).replace('-', '')
         for out in self.outputs:
             if isinstance(out, OutputFile):
-                if out.name == 'outputtext':
-                    # The 'outputtext' file is generated by piping output
-                    # from GRASS, is not an actual grass command
-                    command += ' > ' + out.value
-                else:
-                    command += ' ' + out.name + '="' + out.value + '"'
+                command += ' > ' + out.value
             elif not isinstance(out, OutputHTML):
-                # Html files are not generated by GRASS, only by us to
-                # decorate GRASS output, so we skip them. An output name
-                # to make sure it is unique if the session uses this
-                # algorithm several times.
-                uniqueOutputName = out.name + uniqueSufix
+                # We add an output name to make sure it is unique if the session
+                # uses this algorithm several times.
+                uniqueOutputName = out.name + self.uniqueSufix
                 command += ' ' + out.name + '=' + uniqueOutputName
 
                 # Add output file to exported layers, to indicate that
@@ -345,82 +414,80 @@ class Grass7Algorithm(GeoAlgorithm):
                 self.exportedLayers[out.value] = uniqueOutputName
 
         command += ' --overwrite'
-        commands.append(command)
-
-        # 3: Export resulting layers to a format that qgis can read
+        self.commands.append(command)
 
+    def processOutputs(self):
+        """Prepare the GRASS v.out.ogr commands"""
         for out in self.outputs:
             if isinstance(out, OutputRaster):
                 filename = out.value
 
                 # Raster layer output: adjust region to layer before
                 # exporting
-                commands.append('g.region raster=' + out.name + uniqueSufix)
-                outputCommands.append('g.region raster=' + out.name
-                                      + uniqueSufix)
-                if self.grass7Name == 'r.composite':
-                    command = 'r.out.tiff -t --verbose' # FIXME r.out.tiff deprecated, use r.out.gdal
+                self.commands.append('g.region raster=' + out.name + self.uniqueSufix)
+                self.outputCommands.append('g.region raster=' + out.name
+                                           + self.uniqueSufix)
+
+                if self.grass7Name == 'r.statistics':
+                    # r.statistics saves its results in a non-qgis compatible
+                    # way. Post-process them with r.mapcalc.
+                    calcExpression = 'correctedoutput' + self.uniqueSufix
+                    calcExpression += '=@' + out.name + self.uniqueSufix
+                    command = 'r.mapcalc expression="' + calcExpression + '"'
+                    self.commands.append(command)
+                    self.outputCommands.append(command)
+
+                    command = 'r.out.gdal --overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
                     command += ' input='
-                    command += out.name + uniqueSufix
+                    command += 'correctedoutput' + self.uniqueSufix
+                    command += ' output="' + filename + '"'
+                elif self.grass7Name == 'r.composite':
+                    command = 'r.out.gdal --overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
+                    command += ' input='
+                    command += 'correctedoutput' + self.uniqueSufix
                     command += ' output="' + filename + '"'
-                    commands.append(command)
-                    outputCommands.append(command)
                 else:
-                    command = 'r.out.gdal -c createopt="TFW=YES,COMPRESS=LZW"'
+                    command = 'r.out.gdal --overwrite -c createopt="TFW=YES,COMPRESS=LZW"'
                     command += ' input='
 
                 if self.grass7Name == 'r.horizon':
-                    command += out.name + uniqueSufix + '_0'
+                    command += out.name + self.uniqueSufix + '_0'
+                elif self.grass7Name == 'r.composite':
+                    self.commands.append(command)
+                    self.outputCommands.append(command)
+                elif self.grass7Name == 'r.statistics':
+                    self.commands.append(command)
+                    self.outputCommands.append(command)
                 else:
-                    command += out.name + uniqueSufix
+                    command += out.name + self.uniqueSufix
                     command += ' output="' + filename + '"'
-                    commands.append(command)
-                    outputCommands.append(command)
+                    self.commands.append(command)
+                    self.outputCommands.append(command)
 
             if isinstance(out, OutputVector):
                 filename = out.value
-                # FIXME: check if needed: -c   Also export features without category (not labeled). Otherwise only features with category are exported.
-                command = 'v.out.ogr -s -e input=' + out.name + uniqueSufix
-                command += ' output="' + os.path.dirname(out.value) + '"'
-                command += ' format=ESRI_Shapefile'
-                command += ' olayer=' + os.path.basename(out.value)[:-4]
-                typeidx = \
-                    self.getParameterValue(self.GRASS_OUTPUT_TYPE_PARAMETER)
+                typeidx = self.getParameterValue(self.GRASS_OUTPUT_TYPE_PARAMETER)
                 outtype = ('auto' if typeidx
                            is None else self.OUTPUT_TYPES[typeidx])
-                command += ' type=' + outtype
-                commands.append(command)
-                outputCommands.append(command)
-
-        # 4: Run GRASS
-
-        loglines = []
-        loglines.append(self.tr('GRASS GIS 7 execution commands'))
-        for line in commands:
-            progress.setCommand(line)
-            loglines.append(line)
-        if ProcessingConfig.getSetting(Grass7Utils.GRASS_LOG_COMMANDS):
-            ProcessingLog.addToLog(ProcessingLog.LOG_INFO, loglines)
-        self.consoleOutput = Grass7Utils.executeGrass7(commands, progress,
-                                                       outputCommands)
-        self.postProcessResults()
-
-        # If the session has been created outside of this algorithm, add
-        # the new GRASS GIS 7 layers to it otherwise finish the session
-        if existingSession:
-            Grass7Utils.addSessionLayers(self.exportedLayers)
-        else:
-            Grass7Utils.endGrass7Session()
-
-    def postProcessResults(self):
-        name = self.commandLineName().replace('.', '_')[len('grass7:'):]
-        try:
-            module = importlib.import_module('processing.algs.grass7.ext.' + name)
-        except ImportError:
-            return
-        if hasattr(module, 'postProcessResults'):
-            func = getattr(module, 'postProcessResults')
-            func(self)
+                if self.grass7Name == 'r.flow':
+                    command = 'v.out.ogr type=line layer=0 -s -e input=' + out.name + self.uniqueSufix
+                elif self.grass7Name == 'v.voronoi':
+                    if '-l' in self.commands[-1]:
+                        command = 'v.out.ogr type=line layer=0 -s -e input=' + out.name + self.uniqueSufix
+                    else:
+                        command = 'v.out.ogr -s -e input=' + out.name + self.uniqueSufix
+                        command += ' type=' + outtype
+                elif self.grass7Name == 'v.sample':
+                    command = 'v.out.ogr type=point -s -e input=' + out.name + self.uniqueSufix
+                else:
+                    command = 'v.out.ogr -s -e input=' + out.name + self.uniqueSufix
+                    command += ' type=' + outtype
+                command += ' output="' + os.path.dirname(out.value) + '"'
+                command += ' format=ESRI_Shapefile'
+                command += ' output_layer=' + os.path.basename(out.value)[:-4]
+                command += ' --overwrite'
+                self.commands.append(command)
+                self.outputCommands.append(command)
 
     def exportVectorLayer(self, orgFilename):
 
@@ -446,9 +513,9 @@ class Grass7Algorithm(GeoAlgorithm):
         self.exportedLayers[orgFilename] = destFilename
         command = 'v.in.ogr'
         min_area = self.getParameterValue(self.GRASS_MIN_AREA_PARAMETER)
-        command += ' min_area=' + str(min_area)
+        command += ' min_area=' + unicode(min_area)
         snap = self.getParameterValue(self.GRASS_SNAP_TOLERANCE_PARAMETER)
-        command += ' snap=' + str(snap)
+        command += ' snap=' + unicode(snap)
         command += ' input="' + os.path.dirname(filename) + '"'
         command += ' layer=' + os.path.basename(filename)[:-4]
         command += ' output=' + destFilename
@@ -457,22 +524,22 @@ class Grass7Algorithm(GeoAlgorithm):
 
     def setSessionProjectionFromProject(self, commands):
         if not Grass7Utils.projectionSet:
-            proj4 = iface.mapCanvas().mapRenderer().destinationCrs().toProj4()
+            proj4 = iface.mapCanvas().mapSettings().destinationCrs().toProj4()
             command = 'g.proj'
             command += ' -c'
             command += ' proj4="' + proj4 + '"'
-            commands.append(command)
+            self.commands.append(command)
             Grass7Utils.projectionSet = True
 
     def setSessionProjectionFromLayer(self, layer, commands):
         if not Grass7Utils.projectionSet:
             qGisLayer = dataobjects.getObjectFromUri(layer)
             if qGisLayer:
-                proj4 = str(qGisLayer.crs().toProj4())
+                proj4 = unicode(qGisLayer.crs().toProj4())
                 command = 'g.proj'
                 command += ' -c'
                 command += ' proj4="' + proj4 + '"'
-                commands.append(command)
+                self.commands.append(command)
                 Grass7Utils.projectionSet = True
 
     def exportRasterLayer(self, layer):
@@ -486,49 +553,19 @@ class Grass7Algorithm(GeoAlgorithm):
         return command
 
     def getTempFilename(self):
-        filename = 'tmp' + str(time.time()).replace('.', '') \
-            + str(system.getNumExportedLayers())
+        filename = 'tmp' + unicode(time.time()).replace('.', '') \
+            + unicode(system.getNumExportedLayers())
         return filename
 
     def commandLineName(self):
         return 'grass7:' + self.name[:self.name.find(' ')]
 
     def checkBeforeOpeningParametersDialog(self):
-        msg = Grass7Utils.checkGrass7IsInstalled()
-        if msg is not None:
-            html = self.tr(
-                '<p>This algorithm requires GRASS GIS 7 to be run. '
-                'Unfortunately, it seems that GRASS GIS 7 is not installed in '
-                'your system, or it is not correctly configured to be used '
-                'from QGIS</p>'
-                '<p><a href="http://docs.qgis.org/testing/en/docs/user_manual/processing/3rdParty.html">Click here</a> '
-                'to know more about how to install and configure GRASS GIS 7 to be used with QGIS</p>') # FIXME update URL or page
-            return html
+        return Grass7Utils.checkGrass7IsInstalled()
 
     def checkParameterValuesBeforeExecuting(self):
-        name = self.commandLineName().replace('.', '_')[len('grass7:'):]
-        try:
-            module = importlib.import_module('processing.algs.grass7.ext.' + name)
-        except ImportError:
-            return
-        if hasattr(module, 'checkParameterValuesBeforeExecuting'):
-            func = getattr(module, 'checkParameterValuesBeforeExecuting')
-            return func(self)
-
-    def getPostProcessingErrorMessage(self, wrongLayers):
-        html = GeoAlgorithm.getPostProcessingErrorMessage(self, wrongLayers)
-        msg = Grass7Utils.checkGrass7IsInstalled(True)
-        html += self.tr(
-            '<p>This algorithm requires GRASS GIS 7 to be run. A test '
-            'to check if GRASS GIS 7 is correctly installed and configured in '
-            'your system has been performed, with the following result:</p><ul><i>')
-        if msg is None:
-            html += self.tr(
-                'GRASS GIS 7 seems to be correctly installed and configured</i></li></ul>')
-        else:
-            html += msg + '</i></li></ul>'
-            html += self.tr(
-                '<p><a href="http://docs.qgis.org/testing/en/docs/user_manual/processing/3rdParty.html">Click here</a> '
-                'to know more about how to install and configure GRASS GIS 7 to be used with QGIS</p>')
-
-        return html
+        if self.module:
+            if hasattr(self.module, 'checkParameterValuesBeforeExecuting'):
+                func = getattr(self.module, 'checkParameterValuesBeforeExecuting')
+                return func(self)
+        return
