@@ -35,11 +35,14 @@ from Grass7Algorithm import Grass7Algorithm
 from processing.tools.system import isWindows, isMac
 from nviz7 import nviz7
 
+pluginPath = os.path.normpath(os.path.join(
+    os.path.split(os.path.dirname(__file__))[0], os.pardir))
+
+
 class Grass7AlgorithmProvider(AlgorithmProvider):
 
     def __init__(self):
         AlgorithmProvider.__init__(self)
-        self.activate = False
         self.createAlgsList()
 
     def initializeSettings(self):
@@ -48,11 +51,7 @@ class Grass7AlgorithmProvider(AlgorithmProvider):
             ProcessingConfig.addSetting(Setting(
                 self.getDescription(),
                 Grass7Utils.GRASS_FOLDER, self.tr('GRASS7 folder'),
-                Grass7Utils.grassPath()))
-            ProcessingConfig.addSetting(Setting(
-                self.getDescription(),
-                Grass7Utils.GRASS_WIN_SHELL, self.tr('Msys folder'),
-                Grass7Utils.grassWinShell()))
+                Grass7Utils.grassPath(), valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(
             self.getDescription(),
             Grass7Utils.GRASS_LOG_COMMANDS,
@@ -61,14 +60,19 @@ class Grass7AlgorithmProvider(AlgorithmProvider):
             self.getDescription(),
             Grass7Utils.GRASS_LOG_CONSOLE,
             self.tr('Log console output'), False))
+        ProcessingConfig.addSetting(Setting(
+            self.getDescription(),
+            Grass7Utils.GRASS_HELP_PATH,
+            self.tr('Location of GRASS docs'),
+            Grass7Utils.grassHelpPath()))
 
     def unload(self):
         AlgorithmProvider.unload(self)
         if isWindows() or isMac():
             ProcessingConfig.removeSetting(Grass7Utils.GRASS_FOLDER)
-            ProcessingConfig.removeSetting(Grass7Utils.GRASS_WIN_SHELL)
         ProcessingConfig.removeSetting(Grass7Utils.GRASS_LOG_COMMANDS)
         ProcessingConfig.removeSetting(Grass7Utils.GRASS_LOG_CONSOLE)
+        ProcessingConfig.removeSetting(Grass7Utils.GRASS_HELP_PATH)
 
     def createAlgsList(self):
         self.preloadedAlgs = []
@@ -83,7 +87,7 @@ class Grass7AlgorithmProvider(AlgorithmProvider):
                         ProcessingLog.addToLog(
                             ProcessingLog.LOG_ERROR,
                             self.tr('Could not open GRASS GIS 7 algorithm: %s' % descriptionFile))
-                except Exception, e:
+                except Exception as e:
                     ProcessingLog.addToLog(
                         ProcessingLog.LOG_ERROR,
                         self.tr('Could not open GRASS GIS 7 algorithm: %s' % descriptionFile))
@@ -99,10 +103,13 @@ class Grass7AlgorithmProvider(AlgorithmProvider):
         return 'grass70'
 
     def getIcon(self):
-        return QIcon(os.path.dirname(__file__) + '/../../images/grass.png')
+        return QIcon(os.path.join(pluginPath, 'images', 'grass.svg'))
 
     def getSupportedOutputVectorLayerExtensions(self):
         return ['shp']
 
     def getSupportedOutputRasterLayerExtensions(self):
         return ['tif']
+
+    def canBeActivated(self):
+        return not bool(Grass7Utils.checkGrass7IsInstalled())
