@@ -31,6 +31,7 @@ from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterSelection
 from processing.core.parameters import ParameterNumber
 
+
 class hugeFileClassify(LAStoolsAlgorithm):
 
     TILE_SIZE = "TILE_SIZE"
@@ -42,21 +43,21 @@ class hugeFileClassify(LAStoolsAlgorithm):
     GRANULARITIES = ["coarse", "default", "fine", "extra_fine", "ultra_fine"]
 
     def defineCharacteristics(self):
-        self.name = "hugeFileClassify"
-        self.group = "LAStools Pipelines"
+        self.name, self.i18n_name = self.trAlgorithm('hugeFileClassify')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools Pipelines')
         self.addParametersPointInputGUI()
         self.addParameter(ParameterNumber(hugeFileClassify.TILE_SIZE,
-            self.tr("tile size (side length of square tile)"),
-            0, None, 1000.0))
+                                          self.tr("tile size (side length of square tile)"),
+                                          0, None, 1000.0))
         self.addParameter(ParameterNumber(hugeFileClassify.BUFFER,
-            self.tr("buffer around each tile (avoids edge artifacts)"),
-            0, None, 25.0))
+                                          self.tr("buffer around each tile (avoids edge artifacts)"),
+                                          0, None, 25.0))
         self.addParameter(ParameterBoolean(hugeFileClassify.AIRBORNE,
-            self.tr("airborne LiDAR"), True))
+                                           self.tr("airborne LiDAR"), True))
         self.addParameter(ParameterSelection(hugeFileClassify.TERRAIN,
-            self.tr("terrain type"), hugeFileClassify.TERRAINS, 1))
+                                             self.tr("terrain type"), hugeFileClassify.TERRAINS, 1))
         self.addParameter(ParameterSelection(hugeFileClassify.GRANULARITY,
-            self.tr("preprocessing"), hugeFileClassify.GRANULARITIES, 1))
+                                             self.tr("preprocessing"), hugeFileClassify.GRANULARITIES, 1))
         self.addParametersTemporaryDirectoryGUI()
         self.addParametersPointOutputGUI()
         self.addParametersCoresGUI()
@@ -69,11 +70,11 @@ class hugeFileClassify(LAStoolsAlgorithm):
         self.addParametersPointInputCommands(commands)
         tile_size = self.getParameterValue(hugeFileClassify.TILE_SIZE)
         commands.append("-tile_size")
-        commands.append(str(tile_size))
+        commands.append(unicode(tile_size))
         buffer = self.getParameterValue(hugeFileClassify.BUFFER)
         if buffer != 0.0:
             commands.append("-buffer")
-            commands.append(str(buffer))
+            commands.append(unicode(buffer))
         commands.append("-reversible")
         self.addParametersTemporaryDirectoryAsOutputDirectoryCommands(commands)
         commands.append("-o")
