@@ -31,6 +31,7 @@ from processing.core.parameters import ParameterBoolean
 from processing.core.parameters import ParameterSelection
 from processing.core.parameters import ParameterNumber
 
+
 class hugeFileGroundClassify(LAStoolsAlgorithm):
 
     TILE_SIZE = "TILE_SIZE"
@@ -42,22 +43,22 @@ class hugeFileGroundClassify(LAStoolsAlgorithm):
     GRANULARITIES = ["coarse", "default", "fine", "extra_fine", "ultra_fine"]
 
     def defineCharacteristics(self):
-        self.name = "hugeFileGroundClassify"
-        self.group = "LAStools Pipelines"
+        self.name, self.i18n_name = self.trAlgorithm('hugeFileGroundClassify')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools Pipelines')
         self.addParametersPointInputGUI()
         self.addParameter(ParameterNumber(
             hugeFileGroundClassify.TILE_SIZE,
             self.tr("tile size (side length of square tile)"),
             0, None, 1000.0))
         self.addParameter(ParameterNumber(hugeFileGroundClassify.BUFFER,
-            self.tr("buffer around each tile (avoids edge artifacts)"),
-            0, None, 25.0))
+                                          self.tr("buffer around each tile (avoids edge artifacts)"),
+                                          0, None, 25.0))
         self.addParameter(ParameterBoolean(hugeFileGroundClassify.AIRBORNE,
-            self.tr("airborne LiDAR"), True))
+                                           self.tr("airborne LiDAR"), True))
         self.addParameter(ParameterSelection(hugeFileGroundClassify.TERRAIN,
-            self.tr("terrain type"), hugeFileGroundClassify.TERRAINS, 1))
+                                             self.tr("terrain type"), hugeFileGroundClassify.TERRAINS, 1))
         self.addParameter(ParameterSelection(hugeFileGroundClassify.GRANULARITY,
-            self.tr("preprocessing"), hugeFileGroundClassify.GRANULARITIES, 1))
+                                             self.tr("preprocessing"), hugeFileGroundClassify.GRANULARITIES, 1))
         self.addParametersTemporaryDirectoryGUI()
         self.addParametersPointOutputGUI()
         self.addParametersCoresGUI()
@@ -71,11 +72,11 @@ class hugeFileGroundClassify(LAStoolsAlgorithm):
         self.addParametersPointInputCommands(commands)
         tile_size = self.getParameterValue(hugeFileGroundClassify.TILE_SIZE)
         commands.append("-tile_size")
-        commands.append(str(tile_size))
+        commands.append(unicode(tile_size))
         buffer = self.getParameterValue(hugeFileGroundClassify.BUFFER)
         if buffer != 0.0:
             commands.append("-buffer")
-            commands.append(str(buffer))
+            commands.append(unicode(buffer))
         commands.append("-reversible")
         self.addParametersTemporaryDirectoryAsOutputDirectoryCommands(commands)
         commands.append("-o")
