@@ -30,18 +30,19 @@ from LAStoolsAlgorithm import LAStoolsAlgorithm
 from processing.core.parameters import ParameterString
 from processing.core.outputs import OutputFile
 
+
 class las2txt(LAStoolsAlgorithm):
 
     PARSE = "PARSE"
     OUTPUT = "OUTPUT"
 
     def defineCharacteristics(self):
-        self.name = "las2txt"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('las2txt')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParameter(ParameterString(las2txt.PARSE,
-            self.tr("parse string"), "xyz"))
+                                          self.tr("parse string"), "xyz"))
         self.addOutput(OutputFile(las2txt.OUTPUT, self.tr("Output ASCII file")))
         self.addParametersAdditionalGUI()
 
