@@ -33,13 +33,14 @@ from processing.core.parameters import ParameterExtent
 from LAStoolsAlgorithm import LAStoolsAlgorithm
 from qgis.core import QgsMapLayer, QgsMapLayerRegistry
 
+
 class lasquery(LAStoolsAlgorithm):
 
     AOI = "AOI"
 
     def defineCharacteristics(self):
-        self.name = "lasquery"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('lasquery')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParameter(ParameterExtent(self.AOI, self.tr('area of interest')))
         self.addParametersAdditionalGUI()
@@ -50,14 +51,14 @@ class lasquery(LAStoolsAlgorithm):
         self.addParametersVerboseCommands(commands)
 
     # get area-of-interest
-        aoi = str(self.getParameterValue(self.AOI))
+        aoi = unicode(self.getParameterValue(self.AOI))
         aoiCoords = aoi.split(',')
 
         # get layers
         layers = QgsMapLayerRegistry.instance().mapLayers()
 
         # loop over layers
-        for name,layer in layers.iteritems():
+        for name, layer in layers.iteritems():
             layerType = layer.type()
             if layerType == QgsMapLayer.VectorLayer:
                 shp_file_name = layer.source()
