@@ -29,6 +29,7 @@ from LAStoolsAlgorithm import LAStoolsAlgorithm
 
 from processing.core.parameters import ParameterBoolean
 
+
 class laszip(LAStoolsAlgorithm):
 
     REPORT_SIZE = "REPORT_SIZE"
@@ -36,16 +37,16 @@ class laszip(LAStoolsAlgorithm):
     APPEND_LAX = "APPEND_LAX"
 
     def defineCharacteristics(self):
-        self.name = "laszip"
-        self.group = "LAStools"
+        self.name, self.i18n_name = self.trAlgorithm('laszip')
+        self.group, self.i18n_group = self.trAlgorithm('LAStools')
         self.addParametersVerboseGUI()
         self.addParametersPointInputGUI()
         self.addParameter(ParameterBoolean(laszip.REPORT_SIZE,
-            self.tr("only report size"), False))
+                                           self.tr("only report size"), False))
         self.addParameter(ParameterBoolean(laszip.CREATE_LAX,
-            self.tr("create spatial indexing file (*.lax)"), False))
+                                           self.tr("create spatial indexing file (*.lax)"), False))
         self.addParameter(ParameterBoolean(laszip.APPEND_LAX,
-            self.tr("append *.lax into *.laz file"), False))
+                                           self.tr("append *.lax into *.laz file"), False))
         self.addParametersPointOutputGUI()
         self.addParametersAdditionalGUI()
 
