@@ -29,20 +29,22 @@ __copyright__ = '(C) 2012, Victor Olaya'
 __revision__ = '$Format:%H$'
 
 import os
-import PyQt4.QtGui
+from PyQt4.QtGui import QIcon
 from processing.core.AlgorithmProvider import AlgorithmProvider
 from processing.core.ProcessingConfig import ProcessingConfig, Setting
-from OTBUtils import OTBUtils
+import OTBUtils
 from OTBAlgorithm import OTBAlgorithm
 from processing.core.ProcessingLog import ProcessingLog
 
+pluginPath = os.path.normpath(os.path.join(
+    os.path.split(os.path.dirname(__file__))[0], os.pardir))
+
+
 class OTBAlgorithmProvider(AlgorithmProvider):
 
     def __init__(self):
         AlgorithmProvider.__init__(self)
         self.activate = True
-        self.createAlgsList()
-
 
     def getDescription(self):
         return self.tr("Orfeo Toolbox (Image analysis)")
@@ -51,45 +53,57 @@ class OTBAlgorithmProvider(AlgorithmProvider):
         return "otb"
 
     def getIcon(self):
-        return PyQt4.QtGui.QIcon(os.path.dirname(__file__) + "/../../images/otb.png")
+        return QIcon(os.path.join(pluginPath, 'images', 'otb.png'))
 
     def _loadAlgorithms(self):
-        self.algs = self.preloadedAlgs
+        self.algs = []
+
+        version = OTBUtils.getInstalledVersion(True)
+        if version is None:
+            ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
+                                   self.tr('Problem with OTB installation: OTB was not found or is not correctly installed'))
+            return
+
+        folder = OTBUtils.compatibleDescriptionPath(version)
+        if folder is None:
+            ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
+                                   self.tr('Problem with OTB installation: installed OTB version (%s) is not supported' % version))
+            return
 
-    def createAlgsList(self):
-        self.preloadedAlgs = []
-        folder = OTBUtils.otbDescriptionPath()
         for descriptionFile in os.listdir(folder):
             if descriptionFile.endswith("xml"):
                 try:
                     alg = OTBAlgorithm(os.path.join(folder, descriptionFile))
 
                     if alg.name.strip() != "":
-                        self.preloadedAlgs.append(alg)
+                        self.algs.append(alg)
                     else:
                         ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
-                            self.tr("Could not open OTB algorithm: %s" % descriptionFile))
-                except Exception, e:
+                                               self.tr("Could not open OTB algorithm: %s" % descriptionFile))
+                except Exception as e:
                     ProcessingLog.addToLog(ProcessingLog.LOG_ERROR,
-                        self.tr("Could not open OTB algorithm: %s" % descriptionFile))
-
+                                           self.tr("Could not open OTB algorithm: %s" % descriptionFile))
 
     def initializeSettings(self):
         AlgorithmProvider.initializeSettings(self)
         if OTBUtils.findOtbPath() is None:
             ProcessingConfig.addSetting(Setting(self.getDescription(),
-                OTBUtils.OTB_FOLDER,
-                self.tr("OTB command line tools folder"), OTBUtils.otbPath()))
+                                                OTBUtils.OTB_FOLDER,
+                                                self.tr("OTB command line tools folder"), OTBUtils.otbPath(),
+                                                valuetype=Setting.FOLDER))
         if OTBUtils.findOtbLibPath() is None:
             ProcessingConfig.addSetting(Setting(self.getDescription(),
-                OTBUtils.OTB_LIB_FOLDER,
-                self.tr("OTB applications folder"), OTBUtils.otbLibPath()))
+                                                OTBUtils.OTB_LIB_FOLDER,
+                                                self.tr("OTB applications folder"), OTBUtils.otbLibPath(),
+                                                valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            OTBUtils.OTB_SRTM_FOLDER,
-            self.tr("SRTM tiles folder"), OTBUtils.otbSRTMPath()))
+                                            OTBUtils.OTB_SRTM_FOLDER,
+                                            self.tr("SRTM tiles folder"), OTBUtils.otbSRTMPath(),
+                                            valuetype=Setting.FOLDER))
         ProcessingConfig.addSetting(Setting(self.getDescription(),
-            OTBUtils.OTB_GEOID_FILE,
-            self.tr("Geoid file"), OTBUtils.otbGeoidPath()))
+                                            OTBUtils.OTB_GEOID_FILE,
+                                            self.tr("Geoid file"), OTBUtils.otbGeoidPath(),
+                                            valuetype=Setting.FOLDER))
 
     def unload(self):
         AlgorithmProvider.unload(self)
