@@ -39,117 +39,148 @@ from processing.tools.system import isMac, isWindows
 import logging
 import xml.etree.ElementTree as ET
 import traceback
+from processing.gui.SilentProgress import SilentProgress
 
 
-class OTBUtils:
+OTB_FOLDER = "OTB_FOLDER"
+OTB_LIB_FOLDER = "OTB_LIB_FOLDER"
+OTB_SRTM_FOLDER = "OTB_SRTM_FOLDER"
+OTB_GEOID_FILE = "OTB_GEOID_FILE"
 
-    OTB_FOLDER = "OTB_FOLDER"
-    OTB_LIB_FOLDER = "OTB_LIB_FOLDER"
-    OTB_SRTM_FOLDER = "OTB_SRTM_FOLDER"
-    OTB_GEOID_FILE = "OTB_GEOID_FILE"
 
-    @staticmethod
-    def findOtbPath():
-        folder = None
-        #try to configure the path automatically
-        if isMac():
-            testfolder = os.path.join(str(QgsApplication.prefixPath()), "bin")
+def findOtbPath():
+    folder = None
+    #try to configure the path automatically
+    if isMac():
+        testfolder = os.path.join(unicode(QgsApplication.prefixPath()), "bin")
+        if os.path.exists(os.path.join(testfolder, "otbcli")):
+            folder = testfolder
+        else:
+            testfolder = "/usr/local/bin"
             if os.path.exists(os.path.join(testfolder, "otbcli")):
                 folder = testfolder
-            else:
-                testfolder = "/usr/local/bin"
-                if os.path.exists(os.path.join(testfolder, "otbcli")):
-                    folder = testfolder
-        elif isWindows():
-            testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()),
-                                      os.pardir, "bin")
-            if os.path.exists(os.path.join(testfolder, "otbcli.bat")):
-                folder = testfolder
+    elif isWindows():
+        testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()),
+                                  os.pardir, "bin")
+        if os.path.exists(os.path.join(testfolder, "otbcli.bat")):
+            folder = testfolder
+    else:
+        testfolder = "/usr/bin"
+        if os.path.exists(os.path.join(testfolder, "otbcli")):
+            folder = testfolder
+    return folder
+
+
+def otbPath():
+    folder = findOtbPath()
+    if folder is None:
+        folder = ProcessingConfig.getSetting(OTB_FOLDER)
+    return folder
+
+
+def findOtbLibPath():
+    folder = None
+    #try to configure the path automatically
+    if isMac():
+        testfolder = os.path.join(unicode(QgsApplication.prefixPath()), "lib/otb/applications")
+        if os.path.exists(testfolder):
+            folder = testfolder
         else:
-            testfolder = "/usr/bin"
-            if os.path.exists(os.path.join(testfolder, "otbcli")):
-                folder = testfolder
-        return folder
-
-    @staticmethod
-    def otbPath():
-        folder = OTBUtils.findOtbPath()
-        if folder is None:
-            folder = ProcessingConfig.getSetting(OTBUtils.OTB_FOLDER)
-        return folder
-
-    @staticmethod
-    def findOtbLibPath():
-        folder = None
-        #try to configure the path automatically
-        if isMac():
-            testfolder = os.path.join(str(QgsApplication.prefixPath()), "lib/otb/applications")
-            if os.path.exists(testfolder):
-                folder = testfolder
-            else:
-                testfolder = "/usr/local/lib/otb/applications"
-                if os.path.exists(testfolder):
-                    folder = testfolder
-        elif isWindows():
-            testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()), "orfeotoolbox", "applications")
+            testfolder = "/usr/local/lib/otb/applications"
             if os.path.exists(testfolder):
                 folder = testfolder
+    elif isWindows():
+        testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()), "orfeotoolbox", "applications")
+        if os.path.exists(testfolder):
+            folder = testfolder
+    else:
+        testfolder = "/usr/lib/otb/applications"
+        if os.path.exists(testfolder):
+            folder = testfolder
+    return folder
+
+
+def otbLibPath():
+    return ProcessingConfig.getSetting(OTB_LIB_FOLDER) or ''
+
+
+def otbSRTMPath():
+    return ProcessingConfig.getSetting(OTB_SRTM_FOLDER) or ''
+
+
+def otbGeoidPath():
+    return ProcessingConfig.getSetting(OTB_GEOID_FILE) or ''
+
+
+def otbDescriptionPath():
+    return os.path.join(os.path.dirname(__file__), "description")
+
+_installedVersion = None
+_installedVersionFound = False
+
+
+def getInstalledVersion(runOtb=False):
+    global _installedVersion
+    global _installedVersionFound
+
+    if _installedVersionFound and not runOtb:
+        return _installedVersion
+
+    if otbPath() is None:
+        _installedVersionFound = False
+        return None
+    commands = [os.path.join(otbPath(), "otbcli_Smoothing")]
+    progress = SilentProgress()
+    out = executeOtb(commands, progress, False)
+    for line in out:
+        if "version" in line:
+            _installedVersionFound = True
+            _installedVersion = line.split("version")[-1].strip()
+            break
+    return _installedVersion
+
+
+def compatibleDescriptionPath(version):
+    supportedVersions = {"5.0.0": "5.0.0"}
+    if version is None:
+        return None
+    if version not in supportedVersions:
+        lastVersion = sorted(supportedVersions.keys())[-1]
+        if version > lastVersion:
+            version = lastVersion
         else:
-            testfolder = "/usr/lib/otb/applications"
-            if os.path.exists(testfolder):
-                folder = testfolder
-        return folder
+            return None
 
-    @staticmethod
-    def otbLibPath():
-        folder = OTBUtils.findOtbLibPath()
-        if folder is None:
-            folder = ProcessingConfig.getSetting(OTBUtils.OTB_LIB_FOLDER)
-        return folder
+    return os.path.join(otbDescriptionPath(), supportedVersions[version])
 
 
-    @staticmethod
-    def otbSRTMPath():
-        folder = ProcessingConfig.getSetting(OTBUtils.OTB_SRTM_FOLDER)
-        if folder is None:
-            folder = ""
-        return folder
-
-    @staticmethod
-    def otbGeoidPath():
-        filepath = ProcessingConfig.getSetting(OTBUtils.OTB_GEOID_FILE)
-        if filepath is None:
-            filepath = ""
-        return filepath
-
-    @staticmethod
-    def otbDescriptionPath():
-        return os.path.join(os.path.dirname(__file__), "description")
-
-    @staticmethod
-    def executeOtb(commands, progress):
-        loglines = []
-        loglines.append(OTBUtils.tr("OTB execution console output"))
-        os.putenv('ITK_AUTOLOAD_PATH', OTBUtils.otbLibPath())
-        fused_command = ''.join(['"%s" ' % re.sub(r'^"|"$', '', c) for c in commands])
-        proc = subprocess.Popen(fused_command, shell=True, stdout=subprocess.PIPE, stdin=open(os.devnull),stderr=subprocess.STDOUT, universal_newlines=True).stdout
-        for line in iter(proc.readline, ""):
-            if "[*" in line:
-                idx = line.find("[*")
-                perc = int(line[idx-4:idx-2].strip(" "))
-                if perc != 0:
-                    progress.setPercentage(perc)
-            else:
-                loglines.append(line)
-                progress.setConsoleInfo(line)
+def executeOtb(commands, progress, addToLog=True):
+    loglines = []
+    loglines.append(tr("OTB execution console output"))
+    os.putenv('ITK_AUTOLOAD_PATH', otbLibPath())
+    fused_command = ''.join(['"%s" ' % re.sub(r'^"|"$', '', c) for c in commands])
+    proc = subprocess.Popen(fused_command, shell=True, stdout=subprocess.PIPE, stdin=open(os.devnull), stderr=subprocess.STDOUT, universal_newlines=True).stdout
+    for line in iter(proc.readline, ""):
+        if "[*" in line:
+            idx = line.find("[*")
+            perc = int(line[idx - 4:idx - 2].strip(" "))
+            if perc != 0:
+                progress.setPercentage(perc)
+        else:
+            loglines.append(line)
+            progress.setConsoleInfo(line)
 
+    if addToLog:
         ProcessingLog.addToLog(ProcessingLog.LOG_INFO, loglines)
 
-    @staticmethod
-    def tr(string, context=''):
-        if context == '':
-            context = 'OTBUtils'
-        return QCoreApplication.translate(context, string)
+    return loglines
+
+
+def tr(string, context=''):
+    if context == '':
+        context = 'OTBUtils'
+    return QCoreApplication.translate(context, string)
+
 
 def get_choices_of(doc, parameter):
     choices = []
@@ -161,7 +192,8 @@ def get_choices_of(doc, parameter):
         logger.warning(traceback.format_exc())
     return choices
 
-def remove_dependant_choices(doc, parameter, choice):
+
+def remove_dependent_choices(doc, parameter, choice):
     choices = get_choices_of(doc, parameter)
     choices.remove(choice)
     for a_choice in choices:
@@ -169,13 +201,14 @@ def remove_dependant_choices(doc, parameter, choice):
         for t5 in t4:
             doc.remove(t5)
 
+
 def renameValueField(doc, textitem, field, newValue):
     t4 = [item for item in doc.findall('.//parameter') if item.find('key').text == textitem]
     for t5 in t4:
         t5.find(field).text = newValue
 
 
-def remove_independant_choices(doc, parameter, choice):
+def remove_independent_choices(doc, parameter, choice):
     choices = []
     choices.append(choice)
     for a_choice in choices:
@@ -183,11 +216,13 @@ def remove_independant_choices(doc, parameter, choice):
         for t5 in t4:
             doc.remove(t5)
 
+
 def remove_parameter_by_key(doc, parameter):
     t4 = [item for item in doc.findall('.//parameter') if item.find('key').text == parameter]
     for t5 in t4:
         doc.remove(t5)
 
+
 def remove_other_choices(doc, parameter, choice):
     t5 = [item for item in doc.findall('.//parameter') if item.find('key').text == parameter]
     if len(t5) > 0:
@@ -196,6 +231,7 @@ def remove_other_choices(doc, parameter, choice):
         for a_choice in choices:
             choice_root.remove(a_choice)
 
+
 def remove_choice(doc, parameter, choice):
     t5 = [item for item in doc.findall('.//parameter') if item.find('key').text == parameter]
     if len(t5) > 0:
@@ -204,6 +240,7 @@ def remove_choice(doc, parameter, choice):
         for a_choice in choices:
             choice_root.remove(a_choice)
 
+
 def split_by_choice(doc, parameter):
     """
     splits the given doc into several docs according to the given parameter
@@ -215,7 +252,7 @@ def split_by_choice(doc, parameter):
     for choice in choices:
         #creates a new copy of the document
         working_copy = copy.deepcopy(doc)
-        remove_dependant_choices(working_copy, parameter, choice)
+        remove_dependent_choices(working_copy, parameter, choice)
         #remove all other choices except the current one
         remove_other_choices(working_copy, parameter, choice)
         #set a new name according to the choice
@@ -226,17 +263,20 @@ def split_by_choice(doc, parameter):
         result[choice] = working_copy
     return result
 
+
 def remove_parameter_by_criteria(doc, criteria):
     t4 = [item for item in doc.findall('./parameter') if criteria(item)]
     for t5 in t4:
         doc.getroot().remove(t5)
 
+
 def defaultWrite(available_app, original_dom_document):
     fh = open("description/%s.xml" % available_app, "w")
     the_root = original_dom_document
     ET.ElementTree(the_root).write(fh)
     fh.close()
 
+
 def defaultSplit(available_app, original_dom_document, parameter):
     the_root = original_dom_document
     split = split_by_choice(the_root, parameter)
