@@ -29,38 +29,45 @@ import shelve
 
 try:
     import processing
-except ImportError, e:
+except ImportError as e:
     raise Exception("Processing must be installed and available in PYTHONPATH")
 
 try:
     import otbApplication
-except ImportError, e:
+except ImportError as e:
     raise Exception("OTB python plugins must be installed and available in PYTHONPATH")
 
 from processing.algs.otb.OTBHelper import get_OTB_log, create_xml_descriptors
 from processing.algs.otb.OTBTester import MakefileParser
 
+
 class Alarm(Exception):
     pass
 
+
 def alarm_handler(signum, frame):
     raise Alarm
 
+
 class AlgoTestCase(unittest.TestCase):
+
     def setUp(self):
         self.logger = get_OTB_log()
-        self.the_files = [os.path.join(os.path.join(os.path.abspath(os.curdir), 'description'),each) for each in os.listdir(os.path.join(os.path.abspath(os.curdir), 'description')) if '.xml' in each]
+        self.the_files = [os.path.join(os.path.join(os.path.abspath(os.curdir), 'description'), each) for each in os.listdir(os.path.join(os.path.abspath(os.curdir), 'description')) if '.xml' in each]
 
     def tearDown(self):
         self.logger = None
 
+
 class TestSequence(unittest.TestCase):
+
     def setUp(self):
-        self.data = shelve.open("tests.shelve",writeback=True)
+        self.data = shelve.open("tests.shelve", writeback=True)
 
     def tearDown(self):
         self.data.close()
 
+
 def ut_generator(test_name, a_tuple):
     def test(self):
         logger = get_OTB_log()
@@ -74,7 +81,7 @@ def ut_generator(test_name, a_tuple):
 
         if needs_update:
             signal.signal(signal.SIGALRM, alarm_handler)
-            signal.alarm(6*60)  # 6 minutes
+            signal.alarm(6 * 60)  # 6 minutes
 
             black_list = []
 
@@ -106,7 +113,7 @@ def ut_generator(test_name, a_tuple):
                 logger.info(pout)
 
             if (len(ut_command_validation) > 0) and not failed:
-                new_ut_command_validation = ut_command_validation + " Execute " +  ut_command
+                new_ut_command_validation = ut_command_validation + " Execute " + ut_command
 
                 logger.info("Running Unit test [%s]" % new_ut_command_validation)
                 argz = shlex.split(new_ut_command_validation)
@@ -124,13 +131,13 @@ def ut_generator(test_name, a_tuple):
                     logger.info(qout)
 
             signal.alarm(0)
-            self.data[test_name] = [ a_tuple[0], a_tuple[1], failed ]
+            self.data[test_name] = [a_tuple[0], a_tuple[1], failed]
         else:
             logger.info("Passed test: %s" % test_name)
 
-
     return test
 
+
 def get_client_apps():
     app_clients = []
     for available_app in otbApplication.Registry.GetAvailableApplications():
@@ -140,6 +147,7 @@ def get_client_apps():
         app_clients.append(ct)
     return app_clients
 
+
 def unfiltered_processing_mapping():
     mkf = MakefileParser()
     the_tests = mkf.test_algos()
@@ -158,6 +166,7 @@ def unfiltered_processing_mapping():
     suite = unittest.TestLoader().loadTestsFromTestCase(TestSequence)
     unittest.TextTestRunner(verbosity=2).run(suite)
 
+
 def test_processing_mapping():
     mkf = MakefileParser()
     the_tests = mkf.test_algos()
@@ -185,6 +194,7 @@ def test_processing_mapping():
     suite = unittest.TestLoader().loadTestsFromTestCase(TestSequence)
     unittest.TextTestRunner(verbosity=2).run(suite)
 
+
 def test_xml_generation():
     create_xml_descriptors()
 
