@@ -38,6 +38,12 @@ class PostGISExecuteSQL(GeoAlgorithm):
     DATABASE = 'DATABASE'
     SQL = 'SQL'
 
+    def defineCharacteristics(self):
+        self.name, self.i18n_name = self.trAlgorithm('PostGIS execute SQL')
+        self.group, self.i18n_group = self.trAlgorithm('Database')
+        self.addParameter(ParameterString(self.DATABASE, self.tr('Database')))
+        self.addParameter(ParameterString(self.SQL, self.tr('SQL query'), '', True))
+
     def processAlgorithm(self, progress):
         connection = self.getParameterValue(self.DATABASE)
         settings = QSettings()
@@ -48,25 +54,19 @@ class PostGISExecuteSQL(GeoAlgorithm):
             host = settings.value(mySettings + '/host')
             port = settings.value(mySettings + '/port', type=int)
             password = settings.value(mySettings + '/password')
-        except Exception, e:
+        except Exception as e:
             raise GeoAlgorithmExecutionException(
                 self.tr('Wrong database connection name: %s' % connection))
         try:
             self.db = postgis_utils.GeoDB(host=host, port=port,
-                    dbname=database, user=username, passwd=password)
-        except postgis_utils.DbError, e:
+                                          dbname=database, user=username, passwd=password)
+        except postgis_utils.DbError as e:
             raise GeoAlgorithmExecutionException(
                 self.tr("Couldn't connect to database:\n%s" % e.message))
 
         sql = self.getParameterValue(self.SQL).replace('\n', ' ')
         try:
-            self.db._exec_sql_and_commit(str(sql))
-        except postgis_utils.DbError, e:
+            self.db._exec_sql_and_commit(unicode(sql))
+        except postgis_utils.DbError as e:
             raise GeoAlgorithmExecutionException(
                 self.tr('Error executing SQL:\n%s' % e.message))
-
-    def defineCharacteristics(self):
-        self.name = 'PostGIS execute SQL'
-        self.group = 'Database'
-        self.addParameter(ParameterString(self.DATABASE, self.tr('Database')))
-        self.addParameter(ParameterString(self.SQL, self.tr('SQL query'), '', True))
