@@ -28,7 +28,8 @@ __revision__ = '$Format:%H$'
 import random
 
 from PyQt4.QtCore import QVariant
-from qgis.core import QGis, QgsGeometry, QgsFields, QgsField, QgsSpatialIndex, QgsPoint, QgsFeature, QgsFeatureRequest
+from qgis.core import (QGis, QgsGeometry, QgsFields, QgsField, QgsSpatialIndex,
+                       QgsPoint, QgsFeature, QgsFeatureRequest)
 
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.core.ProcessingLog import ProcessingLog
@@ -46,14 +47,14 @@ class RandomPointsLayer(GeoAlgorithm):
     OUTPUT = 'OUTPUT'
 
     def defineCharacteristics(self):
-        self.name = 'Random points in layer bounds'
-        self.group = 'Vector creation tools'
+        self.name, self.i18n_name = self.trAlgorithm('Random points in layer bounds')
+        self.group, self.i18n_group = self.trAlgorithm('Vector creation tools')
         self.addParameter(ParameterVector(self.VECTOR,
-            self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_POLYGON]))
+                                          self.tr('Input layer'), [ParameterVector.VECTOR_TYPE_POLYGON]))
         self.addParameter(ParameterNumber(self.POINT_NUMBER,
-            self.tr('Points number'), 1, 9999999, 1))
+                                          self.tr('Points number'), 1, None, 1))
         self.addParameter(ParameterNumber(self.MIN_DISTANCE,
-            self.tr('Minimum distance'), 0.0, 9999999, 0.0))
+                                          self.tr('Minimum distance'), 0.0, None, 0.0))
 
         self.addOutput(OutputVector(self.OUTPUT, self.tr('Random points')))
 
@@ -69,7 +70,7 @@ class RandomPointsLayer(GeoAlgorithm):
         fields = QgsFields()
         fields.append(QgsField('id', QVariant.Int, '', 10, 0))
         writer = self.getOutputFromName(self.OUTPUT).getVectorWriter(
-            fields, QGis.WKBPoint, layer.dataProvider().crs())
+            fields, QGis.WKBPoint, layer.crs())
 
         nPoints = 0
         nIterations = 0
@@ -109,8 +110,8 @@ class RandomPointsLayer(GeoAlgorithm):
             nIterations += 1
 
         if nPoints < pointCount:
-             ProcessingLog.addToLog(ProcessingLog.LOG_INFO,
-                 self.tr('Can not generate requested number of random points. '
-                         'Maximum number of attempts exceeded.'))
+            ProcessingLog.addToLog(ProcessingLog.LOG_INFO,
+                                   self.tr('Can not generate requested number of random points. '
+                                           'Maximum number of attempts exceeded.'))
 
         del writer
