@@ -48,17 +48,16 @@ class RasterLayerHistogram(GeoAlgorithm):
     BINS = 'BINS'
 
     def defineCharacteristics(self):
-        self.name = 'Raster layer histogram'
-        self.group = 'Graphics'
+        self.name, self.i18n_name = self.trAlgorithm('Raster layer histogram')
+        self.group, self.i18n_group = self.trAlgorithm('Graphics')
 
         self.addParameter(ParameterRaster(self.INPUT,
-            self.tr('Input layer')))
+                                          self.tr('Input layer')))
         self.addParameter(ParameterNumber(self.BINS,
-           self.tr('Number of bins'), 2, None, 10))
-
-        self.addOutput(OutputHTML(self.PLOT, self.tr('Output plot')))
-        self.addOutput(OutputTable(self.TABLE, self.tr('Output table')))
+                                          self.tr('Number of bins'), 2, None, 10))
 
+        self.addOutput(OutputHTML(self.PLOT, self.tr('Histogram')))
+        self.addOutput(OutputTable(self.TABLE, self.tr('Table')))
 
     def processAlgorithm(self, progress):
         layer = dataobjects.getObjectFromUri(
@@ -82,10 +81,10 @@ class RasterLayerHistogram(GeoAlgorithm):
                   QgsField('NUM_ELEM', QVariant.Double)]
         writer = outputtable.getTableWriter(fields)
         for i in xrange(len(values)):
-            writer.addRecord([str(bins[i]) + '-' + str(bins[i + 1]), n[i]])
+            writer.addRecord([unicode(bins[i]) + '-' + unicode(bins[i + 1]), n[i]])
 
         plotFilename = outputplot + '.png'
         lab.savefig(plotFilename)
         f = open(outputplot, 'w')
-        f.write('<img src="' + plotFilename + '"/>')
+        f.write('<html><img src="' + plotFilename + '"/></html>')
         f.close()
