@@ -25,9 +25,11 @@ __copyright__ = '(C) 2014, Arnaud Morvan'
 
 __revision__ = '$Format:%H$'
 
+import os
 
 from collections import OrderedDict
 
+from PyQt4 import uic
 from PyQt4 import QtCore, QtGui
 
 from qgis.core import QgsExpression
@@ -35,7 +37,9 @@ from qgis.gui import QgsFieldExpressionWidget
 
 from processing.tools import dataobjects
 
-from .ui_widgetFieldsMapping import Ui_Form
+pluginPath = os.path.dirname(__file__)
+WIDGET, BASE = uic.loadUiType(
+    os.path.join(pluginPath, 'widgetFieldsMapping.ui'))
 
 
 class FieldsMappingModel(QtCore.QAbstractTableModel):
@@ -44,6 +48,8 @@ class FieldsMappingModel(QtCore.QAbstractTableModel):
         (QtCore.QVariant.Int, "Integer"),
         (QtCore.QVariant.Double, "Double"),
         (QtCore.QVariant.String, "String"),
+        (QtCore.QVariant.DateTime, "Date"),
+        (QtCore.QVariant.LongLong, "Double"),
         (QtCore.QVariant.Date, "Date")])
 
     columns = [
@@ -211,7 +217,7 @@ class FieldsMappingModel(QtCore.QAbstractTableModel):
                 'type': field.type(),
                 'length': field.length(),
                 'precision': field.precision(),
-                'expression': field.name()}
+                'expression': QgsExpression.quotedColumnRef(field.name())}
 
     def loadLayerFields(self, layer):
         self.beginResetModel()
@@ -283,7 +289,10 @@ class FieldDelegate(QtGui.QStyledItemDelegate):
 
         elif fieldType == QgsExpression:
             (value, isExpression, isValid) = editor.currentField()
-            model.setData(index, value)
+            if isExpression is True:
+                model.setData(index, value)
+            else:
+                model.setData(index, QgsExpression.quotedColumnRef(value))
 
         else:
             QtGui.QStyledItemDelegate.setModelData(self, editor, model, index)
@@ -295,10 +304,10 @@ class FieldDelegate(QtGui.QStyledItemDelegate):
         self.commitData.emit(self.sender())
 
 
-class FieldsMappingPanel(QtGui.QWidget, Ui_Form):
+class FieldsMappingPanel(BASE, WIDGET):
 
     def __init__(self, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        super(FieldsMappingPanel, self).__init__(parent)
         self.setupUi(self)
 
         self.addButton.setIcon(
@@ -346,10 +355,10 @@ class FieldsMappingPanel(QtGui.QWidget, Ui_Form):
         self.model.insertRows(rowCount, 1)
         index = self.model.index(rowCount, 0)
         self.fieldsView.selectionModel().select(index,
-            QtGui.QItemSelectionModel.SelectionFlags(QtGui.QItemSelectionModel.Clear
-                                                     + QtGui.QItemSelectionModel.Select
-                                                     + QtGui.QItemSelectionModel.Current
-                                                     + QtGui.QItemSelectionModel.Rows))
+                                                QtGui.QItemSelectionModel.SelectionFlags(QtGui.QItemSelectionModel.Clear
+                                                                                         + QtGui.QItemSelectionModel.Select
+                                                                                         + QtGui.QItemSelectionModel.Current
+                                                                                         + QtGui.QItemSelectionModel.Rows))
         self.fieldsView.scrollTo(index)
         self.fieldsView.scrollTo(index)
 
