@@ -33,7 +33,7 @@ import subprocess
 from PyQt4.QtCore import QSettings, QCoreApplication
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.ProcessingLog import ProcessingLog
-from processing.tools.system import userFolder, isWindows, mkdir
+from processing.tools.system import userFolder, isWindows, mkdir, getTempFilenameInTempFolder
 
 
 class RUtils:
@@ -41,6 +41,9 @@ class RUtils:
     RSCRIPTS_FOLDER = 'R_SCRIPTS_FOLDER'
     R_FOLDER = 'R_FOLDER'
     R_USE64 = 'R_USE64'
+    R_LIBS_USER = 'R_LIBS_USER'
+
+    rscriptfilename = userFolder() + os.sep + 'processing_script.r'
 
     @staticmethod
     def RFolder():
@@ -50,12 +53,28 @@ class RUtils:
 
         return os.path.abspath(unicode(folder))
 
+    @staticmethod
+    def RLibs():
+        folder = ProcessingConfig.getSetting(RUtils.R_LIBS_USER)
+        if folder is None:
+            folder = unicode(os.path.join(userFolder(), 'rlibs'))
+        try:
+            mkdir(folder)
+        except:
+            folder = unicode(os.path.join(userFolder(), 'rlibs'))
+            mkdir(folder)
+        return os.path.abspath(unicode(folder))
+
     @staticmethod
     def RScriptsFolder():
         folder = ProcessingConfig.getSetting(RUtils.RSCRIPTS_FOLDER)
         if folder is None:
             folder = unicode(os.path.join(userFolder(), 'rscripts'))
-        mkdir(folder)
+        try:
+            mkdir(folder)
+        except:
+            folder = unicode(os.path.join(userFolder(), 'rscripts'))
+            mkdir(folder)
 
         return os.path.abspath(folder)
 
@@ -68,7 +87,7 @@ class RUtils:
 
     @staticmethod
     def getRScriptFilename():
-        return userFolder() + os.sep + 'processing_script.r'
+        return RUtils.rscriptfilename
 
     @staticmethod
     def getConsoleOutputFilename():
@@ -76,6 +95,9 @@ class RUtils:
 
     @staticmethod
     def executeRAlgorithm(alg, progress):
+        # generate new R script file name in a temp folder
+        RUtils.rscriptfilename = getTempFilenameInTempFolder('processing_script.r')
+        # run commands
         RUtils.verboseCommands = alg.getVerboseCommands()
         RUtils.createRScriptFromRCommands(alg.getFullSetOfRCommands())
         if isWindows():
@@ -90,8 +112,9 @@ class RUtils:
                 'BATCH',
                 '--vanilla',
                 RUtils.getRScriptFilename(),
-                RUtils.getConsoleOutputFilename(),
+                RUtils.getConsoleOutputFilename()
             ]
+
         else:
             os.chmod(RUtils.getRScriptFilename(), stat.S_IEXEC | stat.S_IREAD
                      | stat.S_IWRITE)
@@ -190,7 +213,7 @@ class RUtils:
 
     @staticmethod
     def getRequiredPackages(code):
-        regex = re.compile('library\("?(.*?)"?\)')
+        regex = re.compile('[^#]library\("?(.*?)"?\)')
         return regex.findall(code)
 
     @staticmethod
