@@ -33,6 +33,9 @@ from processing.core.outputs import OutputRaster
 from processing.tools.system import getTempFilename, isWindows
 import SagaUtils
 
+pluginPath = os.path.normpath(os.path.join(
+    os.path.split(os.path.dirname(__file__))[0], os.pardir))
+
 
 class SplitRGBBands(GeoAlgorithm):
 
@@ -42,19 +45,19 @@ class SplitRGBBands(GeoAlgorithm):
     B = 'B'
 
     def getIcon(self):
-        return QtGui.QIcon(os.path.dirname(__file__) + '/../../images/saga.png')
+        return QtGui.QIcon(os.path.join(pluginPath, 'images', 'saga.png'))
 
     def defineCharacteristics(self):
-        self.name = 'Split RGB bands'
-        self.group = 'Grid - Tools'
+        self.name, self.i18n_name = self.trAlgorithm('Split RGB bands')
+        self.group, self.i18n_group = self.trAlgorithm('Image tools')
         self.addParameter(ParameterRaster(SplitRGBBands.INPUT,
-            self.tr('Input layer'), False))
+                                          self.tr('Input layer'), False))
         self.addOutput(OutputRaster(SplitRGBBands.R,
-            self.tr('Output R band layer')))
+                                    self.tr('Output R band layer')))
         self.addOutput(OutputRaster(SplitRGBBands.G,
-            self.tr('Output G band layer')))
+                                    self.tr('Output G band layer')))
         self.addOutput(OutputRaster(SplitRGBBands.B,
-            self.tr('Output B band layer')))
+                                    self.tr('Output B band layer')))
 
     def processAlgorithm(self, progress):
         # TODO: check correct num of bands
@@ -70,30 +73,17 @@ class SplitRGBBands(GeoAlgorithm):
         g = self.getOutputValue(SplitRGBBands.G)
         b = self.getOutputValue(SplitRGBBands.B)
         commands = []
-        if isWindows():
-            commands.append('io_gdal 0 -GRIDS "' + temp + '" -FILES "' + input
-                            + '"')
-            commands.append('io_gdal 1 -GRIDS "' + temp
-                            + '_0001.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + r + '"'
-                            )
-            commands.append('io_gdal 1 -GRIDS "' + temp
-                            + '_0002.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + g + '"'
-                            )
-            commands.append('io_gdal 1 -GRIDS "' + temp
-                            + '_0003.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + b + '"'
-                            )
-        else:
-            commands.append('libio_gdal 0 -GRIDS "' + temp + '" -FILES "'
-                            + input + '"')
-            commands.append('libio_gdal 1 -GRIDS "' + temp
-                            + '_0001.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + r + '"'
-                            )
-            commands.append('libio_gdal 1 -GRIDS "' + temp
-                            + '_0002.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + g + '"'
-                            )
-            commands.append('libio_gdal 1 -GRIDS "' + temp
-                            + '_0003.sgrd" -FORMAT 1 -TYPE 0 -FILE "' + b + '"'
-                            )
+        version = SagaUtils.getSagaInstalledVersion(True)
+        trailing = ""
+        lib = ""
+        commands.append('%sio_gdal 0 -GRIDS "%s" -FILES "%s"' % (lib, temp, input)
+                        )
+        commands.append('%sio_gdal 1 -GRIDS "%s_%s1.sgrd" -FORMAT 1 -TYPE 0 -FILE "%s"' % (lib, temp, trailing, r)
+                        )
+        commands.append('%sio_gdal 1 -GRIDS "%s_%s2.sgrd" -FORMAT 1 -TYPE 0 -FILE "%s"' % (lib, temp, trailing, g)
+                        )
+        commands.append('%sio_gdal 1 -GRIDS "%s_%s3.sgrd" -FORMAT 1 -TYPE 0 -FILE "%s"' % (lib, temp, trailing, b)
+                        )
 
         SagaUtils.createSagaBatchJobFileFromSagaCommands(commands)
         SagaUtils.executeSaga(progress)
