@@ -17,7 +17,6 @@
 ***************************************************************************
 """
 
-
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
@@ -26,63 +25,101 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 __revision__ = '$Format:%H$'
 
-from PyQt4.QtCore import Qt, QSettings, QCoreApplication
-from PyQt4.QtGui import QDockWidget, QMenu, QAction, QTreeWidgetItem
+import os
+
+from PyQt4 import uic
+from PyQt4.QtCore import Qt, QSettings, QCoreApplication, SIGNAL
+from PyQt4.QtGui import QMenu, QAction, QTreeWidgetItem, QLabel, QMessageBox
 from qgis.utils import iface
-from processing.modeler.ModelerUtils import ModelerUtils
+
 from processing.core.Processing import Processing
 from processing.core.ProcessingLog import ProcessingLog
 from processing.core.ProcessingConfig import ProcessingConfig
 from processing.core.GeoAlgorithm import GeoAlgorithm
 from processing.gui.MessageDialog import MessageDialog
-from processing.gui.AlgorithmClassification import AlgorithmDecorator
+from processing.gui import AlgorithmClassification
 from processing.gui.AlgorithmDialog import AlgorithmDialog
 from processing.gui.BatchAlgorithmDialog import BatchAlgorithmDialog
 from processing.gui.EditRenderingStylesDialog import EditRenderingStylesDialog
-
-from processing.ui.ui_ProcessingToolbox import Ui_ProcessingToolbox
+from processing.gui.ConfigDialog import ConfigDialog
 
 
-class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
+pluginPath = os.path.split(os.path.dirname(__file__))[0]
+WIDGET, BASE = uic.loadUiType(
+    os.path.join(pluginPath, 'ui', 'ProcessingToolbox.ui'))
 
-    USE_CATEGORIES = '/Processing/UseSimplifiedInterface'
+
+class ProcessingToolbox(BASE, WIDGET):
+
+    updateAlgList = True
 
     def __init__(self):
-        QDockWidget.__init__(self, None)
+        super(ProcessingToolbox, self).__init__(None)
+        self.tipWasClosed = False
         self.setupUi(self)
         self.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
 
-        self.modeComboBox.clear()
-        self.modeComboBox.addItems(['Simplified interface',
-                                   'Advanced interface'])
-        settings = QSettings()
-        if not settings.contains(self.USE_CATEGORIES):
-            settings.setValue(self.USE_CATEGORIES, True)
-        useCategories = settings.value(self.USE_CATEGORIES, type=bool)
-        if useCategories:
-            self.modeComboBox.setCurrentIndex(0)
-        else:
-            self.modeComboBox.setCurrentIndex(1)
-        self.modeComboBox.currentIndexChanged.connect(self.modeHasChanged)
-
         self.searchBox.textChanged.connect(self.textChanged)
         self.algorithmTree.customContextMenuRequested.connect(
             self.showPopupMenu)
         self.algorithmTree.doubleClicked.connect(self.executeAlgorithm)
+        self.txtDisabled.setVisible(False)
+        self.txtTip.setVisible(self.disabledProviders())
+        self.txtDisabled.linkActivated.connect(self.showDisabled)
 
+        def openSettings(url):
+            if url == "close":
+                self.txtTip.setVisible(False)
+                self.tipWasClosed = True
+            else:
+                dlg = ConfigDialog(self)
+                dlg.exec_()
+                self.txtTip.setVisible(self.disabledProviders())
+        self.txtTip.linkActivated.connect(openSettings)
         if hasattr(self.searchBox, 'setPlaceholderText'):
             self.searchBox.setPlaceholderText(self.tr('Search...'))
 
         self.fillTree()
 
+    def showDisabled(self):
+        self.txtDisabled.setVisible(False)
+        for providerName in self.disabledWithMatchingAlgs:
+            self.disabledProviderItems[providerName].setHidden(False)
+        self.algorithmTree.expandAll()
+
+    def disabledProviders(self):
+        showTip = ProcessingConfig.getSetting(ProcessingConfig.SHOW_PROVIDERS_TOOLTIP)
+        if not showTip or self.tipWasClosed:
+            return False
+
+        for providerName in Processing.algs.keys():
+            name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
+            if not ProcessingConfig.getSetting(name):
+                return True
+        return False
+
     def textChanged(self):
         text = self.searchBox.text().strip(' ').lower()
+        for item in self.disabledProviderItems.values():
+            item.setHidden(True)
         self._filterItem(self.algorithmTree.invisibleRootItem(), text)
         if text:
             self.algorithmTree.expandAll()
+            self.disabledWithMatchingAlgs = []
+            for providerName, provider in Processing.algs.iteritems():
+                name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
+                if not ProcessingConfig.getSetting(name):
+                    for alg in provider.values():
+                        if text in alg.name:
+                            self.disabledWithMatchingAlgs.append(providerName)
+                            break
+            showTip = ProcessingConfig.getSetting(ProcessingConfig.SHOW_PROVIDERS_TOOLTIP)
+            if showTip:
+                self.txtDisabled.setVisible(bool(self.disabledWithMatchingAlgs))
         else:
             self.algorithmTree.collapseAll()
             self.algorithmTree.invisibleRootItem().child(0).setExpanded(True)
+            self.txtDisabled.setVisible(False)
 
     def _filterItem(self, item, text):
         if (item.childCount() > 0):
@@ -90,35 +127,40 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
             for i in xrange(item.childCount()):
                 child = item.child(i)
                 showChild = self._filterItem(child, text)
-                show = showChild or show
+                show = (showChild or show) and not item in self.disabledProviderItems.values()
             item.setHidden(not show)
             return show
         elif isinstance(item, (TreeAlgorithmItem, TreeActionItem)):
             hide = bool(text) and (text not in item.text(0).lower())
+            if isinstance(item, TreeAlgorithmItem):
+                hide = hide and (text not in item.alg.commandLineName())
             item.setHidden(hide)
             return not hide
         else:
             item.setHidden(True)
             return False
 
-
-    def modeHasChanged(self):
-        idx = self.modeComboBox.currentIndex()
-        settings = QSettings()
-        if idx == 0:
-            # Simplified
-            settings.setValue(self.USE_CATEGORIES, True)
-        else:
-            settings.setValue(self.USE_CATEGORIES, False)
-
+    def activateProvider(self, providerName):
+        name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
+        ProcessingConfig.setSettingValue(name, True)
         self.fillTree()
+        self.textChanged()
+        self.showDisabled()
+        provider = Processing.getProviderFromName(providerName)
+        if not provider.canBeActivated():
+            QMessageBox.warning(self, "Activate provider",
+                                "The provider has been activated, but it might need additional configuration.")
 
     def algsListHasChanged(self):
-        self.fillTree()
+        if self.updateAlgList:
+            self.fillTree()
+            self.textChanged()
 
     def updateProvider(self, providerName, updateAlgsList=True):
         if updateAlgsList:
+            self.updateAlgList = False
             Processing.updateAlgsList()
+            self.updateAlgList = True
         for i in xrange(self.algorithmTree.invisibleRootItem().childCount()):
             child = self.algorithmTree.invisibleRootItem().child(i)
             if isinstance(child, TreeProviderItem):
@@ -129,6 +171,7 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
                     for i in xrange(child.childCount()):
                         child.child(i).sortChildren(0, Qt.AscendingOrder)
                     break
+        self.addRecentAlgorithms(True)
 
     def showPopupMenu(self, point):
         item = self.algorithmTree.itemAt(point)
@@ -159,7 +202,7 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
                 action.setData(alg, self)
                 if action.isEnabled():
                     contextMenuAction = QAction(action.name,
-                            self.algorithmTree)
+                                                self.algorithmTree)
                     contextMenuAction.triggered.connect(action.execute)
                     popupmenu.addAction(contextMenuAction)
 
@@ -188,9 +231,9 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
             message = alg.checkBeforeOpeningParametersDialog()
             if message:
                 dlg = MessageDialog()
-                dlg.setTitle(self.tr('Missing dependency'))
+                dlg.setTitle(self.tr('Error executing algorithm'))
                 dlg.setMessage(
-                    self.tr('<h3>Missing dependency. This algorithm cannot '
+                    self.tr('<h3>This algorithm cannot '
                             'be run :-( </h3>\n%s') % message)
                 dlg.exec_()
                 return
@@ -219,13 +262,7 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
             action.execute()
 
     def fillTree(self):
-        settings = QSettings()
-        useCategories = settings.value(self.USE_CATEGORIES, type=bool)
-        if useCategories:
-            self.fillTreeUsingCategories()
-        else:
-            self.fillTreeUsingProviders()
-        self.algorithmTree.sortItems(0, Qt.AscendingOrder)
+        self.fillTreeUsingProviders()
         self.addRecentAlgorithms(False)
 
     def addRecentAlgorithms(self, updating):
@@ -255,96 +292,31 @@ class ProcessingToolbox(QDockWidget, Ui_ProcessingToolbox):
 
             self.algorithmTree.setWordWrap(True)
 
-    def fillTreeUsingCategories(self):
-        providersToExclude = ['model', 'script']
-        self.algorithmTree.clear()
-        text = unicode(self.searchBox.text())
-        groups = {}
-        for providerName in Processing.algs.keys():
-            provider = Processing.algs[providerName]
-            name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
-            if not ProcessingConfig.getSetting(name):
-                continue
-            if providerName in providersToExclude or \
-                    len(ModelerUtils.providers[providerName].actions) != 0:
-                continue
-            algs = provider.values()
-
-            # add algorithms
-
-            for alg in algs:
-                if not alg.showInToolbox:
-                    continue
-                (altgroup, altsubgroup, altname) = \
-                    AlgorithmDecorator.getGroupsAndName(alg)
-                if altgroup is None:
-                    continue
-                if text == '' or text.lower() in altname.lower():
-                    if altgroup not in groups:
-                        groups[altgroup] = {}
-                    group = groups[altgroup]
-                    if altsubgroup not in group:
-                        groups[altgroup][altsubgroup] = []
-                    subgroup = groups[altgroup][altsubgroup]
-                    subgroup.append(alg)
-
-        if len(groups) > 0:
-            mainItem = QTreeWidgetItem()
-            mainItem.setText(0, 'Geoalgorithms')
-            mainItem.setIcon(0, GeoAlgorithm.getDefaultIcon())
-            mainItem.setToolTip(0, mainItem.text(0))
-            for (groupname, group) in groups.items():
-                groupItem = QTreeWidgetItem()
-                groupItem.setText(0, groupname)
-                groupItem.setIcon(0, GeoAlgorithm.getDefaultIcon())
-                groupItem.setToolTip(0, groupItem.text(0))
-                mainItem.addChild(groupItem)
-                for (subgroupname, subgroup) in group.items():
-                    subgroupItem = QTreeWidgetItem()
-                    subgroupItem.setText(0, subgroupname)
-                    subgroupItem.setIcon(0, GeoAlgorithm.getDefaultIcon())
-                    subgroupItem.setToolTip(0, subgroupItem.text(0))
-                    groupItem.addChild(subgroupItem)
-                    for alg in subgroup:
-                        algItem = TreeAlgorithmItem(alg)
-                        subgroupItem.addChild(algItem)
-
-            self.algorithmTree.addTopLevelItem(mainItem)
-
-        for providerName in Processing.algs.keys():
-            if providerName not in providersToExclude:
-                continue
-            name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
-            if not ProcessingConfig.getSetting(name):
-                continue
-            providerItem = TreeProviderItem(providerName)
-            self.algorithmTree.addTopLevelItem(providerItem)
-
     def fillTreeUsingProviders(self):
         self.algorithmTree.clear()
+        self.disabledProviderItems = {}
+        disabled = []
         for providerName in Processing.algs.keys():
             name = 'ACTIVATE_' + providerName.upper().replace(' ', '_')
-            if not ProcessingConfig.getSetting(name):
-                continue
-            providerItem = TreeProviderItem(providerName)
-            self.algorithmTree.addTopLevelItem(providerItem)
-            providerItem.setHidden(providerItem.childCount() == 0)
-
+            if ProcessingConfig.getSetting(name):
+                providerItem = TreeProviderItem(providerName, self.algorithmTree, self)
+                providerItem.setHidden(providerItem.childCount() == 0)
+            else:
+                disabled.append(providerName)
+        self.algorithmTree.sortItems(0, Qt.AscendingOrder)
+        for providerName in sorted(disabled):
+            providerItem = TreeProviderItem(providerName, self.algorithmTree, self)
+            providerItem.setHidden(True)
+            self.disabledProviderItems[providerName] = providerItem
 
 
 class TreeAlgorithmItem(QTreeWidgetItem):
 
     def __init__(self, alg):
-        settings = QSettings()
-        useCategories = settings.value(ProcessingToolbox.USE_CATEGORIES,
-                                       type=bool)
         QTreeWidgetItem.__init__(self)
         self.alg = alg
         icon = alg.getIcon()
-        name = alg.name
-        if useCategories:
-            icon = GeoAlgorithm.getDefaultIcon()
-            (group, subgroup, name) = AlgorithmDecorator.getGroupsAndName(alg)
+        name = AlgorithmClassification.getDisplayName(alg)
         self.setIcon(0, icon)
         self.setToolTip(0, name)
         self.setText(0, name)
@@ -358,10 +330,13 @@ class TreeActionItem(QTreeWidgetItem):
         self.setText(0, action.name)
         self.setIcon(0, action.getIcon())
 
+
 class TreeProviderItem(QTreeWidgetItem):
 
-    def __init__(self, providerName):
-        QTreeWidgetItem.__init__(self)
+    def __init__(self, providerName, tree, toolbox):
+        QTreeWidgetItem.__init__(self, tree)
+        self.tree = tree
+        self.toolbox = toolbox
         self.providerName = providerName
         self.provider = Processing.getProviderFromName(providerName)
         self.setIcon(0, self.provider.getIcon())
@@ -377,6 +352,9 @@ class TreeProviderItem(QTreeWidgetItem):
         provider = Processing.algs[self.providerName]
         algs = provider.values()
 
+        name = 'ACTIVATE_' + self.providerName.upper().replace(' ', '_')
+        active = ProcessingConfig.getSetting(name)
+
         # Add algorithms
         for alg in algs:
             if not alg.showInToolbox:
@@ -385,10 +363,15 @@ class TreeProviderItem(QTreeWidgetItem):
                 groupItem = groups[alg.group]
             else:
                 groupItem = QTreeWidgetItem()
-                groupItem.setText(0, alg.group)
-                groupItem.setToolTip(0, alg.group)
+                name = alg.i18n_group or alg.group
+                if not active:
+                    groupItem.setForeground(0, Qt.darkGray)
+                groupItem.setText(0, name)
+                groupItem.setToolTip(0, name)
                 groups[alg.group] = groupItem
             algItem = TreeAlgorithmItem(alg)
+            if not active:
+                algItem.setForeground(0, Qt.darkGray)
             groupItem.addChild(algItem)
             count += 1
 
@@ -403,8 +386,19 @@ class TreeProviderItem(QTreeWidgetItem):
             algItem = TreeActionItem(action)
             groupItem.addChild(algItem)
 
-        self.setText(0, self.provider.getDescription()
-                  + QCoreApplication.translate( "TreeProviderItem", " [{0} geoalgorithms]" ).format( count ) )
+        text = self.provider.getDescription()
+
+        if not active:
+            def activateProvider():
+                self.toolbox.activateProvider(self.providerName)
+            label = QLabel(text + "&nbsp;&nbsp;&nbsp;&nbsp;<a href='%s'>Activate</a>")
+            label.setStyleSheet("QLabel {background-color: white; color: grey;}")
+            label.linkActivated.connect(activateProvider)
+            self.tree.setItemWidget(self, 0, label)
+
+        else:
+            text += QCoreApplication.translate("TreeProviderItem", " [{0} geoalgorithms]").format(count)
+        self.setText(0, text)
         self.setToolTip(0, self.text(0))
         for groupItem in groups.values():
             self.addChild(groupItem)
