@@ -29,7 +29,7 @@ __revision__ = '$Format:%H$'
 import os
 import re
 from qgis.core import QGis, QgsProject, QgsVectorFileWriter, QgsMapLayer, QgsRasterLayer, \
-                        QgsVectorLayer, QgsMapLayerRegistry, QgsCoordinateReferenceSystem
+    QgsVectorLayer, QgsMapLayerRegistry, QgsCoordinateReferenceSystem
 from qgis.gui import QgsSublayersDialog
 from PyQt4.QtCore import QSettings
 from qgis.utils import iface
@@ -308,7 +308,8 @@ def exportVectorLayer(layer, supported=None):
             unicode(layer.source()).decode('ascii')
         except UnicodeEncodeError:
             isASCII = False
-        if not os.path.splitext(layer.source())[1].lower() in supported or not isASCII:
+        ext = os.path.splitext(layer.source())[-1].lower()[1:]
+        if ext not in supported or not isASCII:
             writer = QgsVectorFileWriter(
                 output, systemEncoding,
                 layer.pendingFields(), provider.geometryType(),
