@@ -50,7 +50,7 @@ def algoptions(name):
                 s += param.name + '(' + param.description + ')\n'
                 i = 0
                 for option in param.options:
-                    s += '\t' + str(i) + ' - ' + str(option) + '\n'
+                    s += '\t' + unicode(i) + ' - ' + unicode(option) + '\n'
                     i += 1
         print s
     else:
@@ -61,17 +61,17 @@ def alghelp(name):
     alg = Processing.getAlgorithm(name)
     if alg is not None:
         alg = alg.getCopy()
-        print str(alg)
+        print unicode(alg)
         algoptions(name)
     else:
         print 'Algorithm not found'
 
 
-def runalg(algOrName, *args):
-    alg = Processing.runAlgorithm(algOrName, None, *args)
+def runalg(algOrName, *args, **kwargs):
+    alg = Processing.runAlgorithm(algOrName, None, *args, **kwargs)
     if alg is not None:
         return alg.getOutputValuesAsDictionary()
 
 
-def runandload(name, *args):
-    return Processing.runAlgorithm(name, handleAlgorithmResults, *args)
+def runandload(name, *args, **kwargs):
+    return Processing.runAlgorithm(name, handleAlgorithmResults, *args, **kwargs)
