@@ -52,6 +52,7 @@ my $help;
 my $domajor;
 my $dominor;
 my $dopoint;
+my $doltr = 0;
 
 my $result = GetOptions(
 		"major" => \$domajor,
@@ -59,6 +60,7 @@ my $result = GetOptions(
 		"point" => \$dopoint,
 		"releasename=s" => \$newreleasename,
 		"help" => \$help,
+		"ltr" => \$doltr,
 		"dryrun" => \$dryrun,
 	);
 
@@ -70,6 +72,7 @@ $i++ if defined $dominor;
 $i++ if defined $dopoint;
 pod2usage("Exactly one of -major, -minor or -point expected") if $i!=1;
 pod2usage("Release name for major and minor releases expected") if !$dopoint && !defined $newreleasename;
+pod2usage("Long term releases only for major and minor releases") if $doltr && $dopoint;
 pod2usage("No CMakeLists.txt in current directory") unless -r "CMakeLists.txt";
 
 my $major;
@@ -136,6 +139,7 @@ run( "git pull --rebase", "git pull rebase failed" );
 my $release = "$newmajor.$newminor";
 my $version = "$release.$newpatch";
 my $relbranch = "release-${newmajor}_${newminor}";
+my $ltrtag = $doltr ? "ltr-${newmajor}_${newminor}" : "";
 my $reltag = "final-${newmajor}_${newminor}_${newpatch}";
 
 unless( $dopoint ) {
@@ -149,13 +153,15 @@ print "Updating changelog...\n";
 run( "scripts/create_changelog.sh", "create_changelog.sh failed" );
 
 unless( $dopoint ) {
-	run( "git commit -a -m \"changelog update for $release\"", "could not commit changelog update" );
+	run( "scripts/update-news.pl $newmajor $newminor '$release'", "could not update news" ) if $major>2 || ($major==2 && $minor>14);
+
+	run( "git commit -a -m \"changelog and news update for $release\"", "could not commit changelog and news update" );
 
 	print "Creating and checking out branch...\n";
 	run( "git checkout -b $relbranch", "git checkout release branch failed" );
 }
 
-updateCMakeLists($newmajor,$newminor,$newpatch,$releasename);
+updateCMakeLists($newmajor,$newminor,$newpatch,$newreleasename);
 
 print "Updating version...\n";
 run( "dch -r ''", "dch failed" );
@@ -172,15 +178,16 @@ unless( $dopoint ) {
 		print "WARNING: NO images/splash/splash-release.xcf.bz2\n";
 	}
 
-	run( "git commit -a -m 'Release of $release ($releasename)'", "release commit failed" );
-	run( "git tag $reltag -m 'Version $release'", "tag failed" );
+	run( "git commit -a -m 'Release of $release ($newreleasename)'", "release commit failed" );
+	run( "git tag $reltag -m 'Version $release'", "release tag failed" );
+	run( "git tag $ltrtag -m 'Long term release $release'", "ltr tag failed" ) if $doltr;
 } else {
 	run( "git commit -a -m 'Release of $version'", "release commit failed" );
 	run( "git tag $reltag -m 'Version $version'", "tag failed" );
 }
 
 print "Producing archive...\n";
-run( "git archive --format tar --prefix=qgis-$version/ $reltag | bzip2 -c >qgis-$version.tar.bz2", "git archive failed" );
+run( "git archive --format tar --prefix=qgis-$version/ $reltag $ltrtag | bzip2 -c >qgis-$version.tar.bz2", "git archive failed" );
 run( "md5sum qgis-$version.tar.bz2 >qgis-$version.tar.bz2.md5", "md5sum failed" );
 
 unless( $dopoint ) {
@@ -195,11 +202,11 @@ unless( $dopoint ) {
 	run( "git commit -a -m 'Bump version to $newmajor.$newminor'", "bump version failed" );
 }
 
-my $topush = ($dopoint ? "" : "master ") . "$relbranch $reltag";
+my $topush = ($dopoint ? "" : "master ") . "$relbranch";
 
 print "Push dry-run...\n";
-run( "git push -n origin $topush", "push dry run failed" );
-print "Now manually push and upload the tarballs :\n\tgit push origin $topush\n\trsync qgis-$version.tar.bz2* qgis.org:/var/www/downloads/\n\n";
+run( "git push -n --follow-tags origin $topush", "push dry run failed" );
+print "Now manually push and upload the tarballs :\n\tgit push --follow-tags origin $topush\n\trsync qgis-$version.tar.bz2* qgis.org:/var/www/downloads/\n\n";
 
 
 =head1 NAME
@@ -208,12 +215,17 @@ release.pl - create a new release
 
 =head1 SYNOPSIS
 
-release.pl {{-major|-minor} -releasename=releasename|-point}
+release.pl {{-major|-minor} [-ltr] -releasename=releasename|-point}
 
   Options:
     -major              do a new major release
     -minor              do a new minor release
     -point              do a new point release
     -releasename=name   new release name for master/minor release
+    -ltr                new release is a long term release
     -dryrun             just echo but don't run any commands
+
+  Major and minor releases also require a new splash screen
+  images/splash/splash-M.N.png and bitmap for the NSIS
+  installer ms-windows/Installer-Files/WelcomeFinishPage-M.N.bmp.
 =cut
