@@ -30,13 +30,13 @@ struct ANALYSIS_EXPORT vertexData
   double z;
 };
 
-/**Interface class for interpolations. Interpolators take
+/** Interface class for interpolations. Interpolators take
 the vertices of a vector layer as base data. The z-Value
 can be an attribute or the z-coordinates in case of 25D types*/
 class ANALYSIS_EXPORT QgsInterpolator
 {
   public:
-    /**Describes the type of input data*/
+    /** Describes the type of input data*/
     enum InputType
     {
       POINTS,
@@ -44,7 +44,7 @@ class ANALYSIS_EXPORT QgsInterpolator
       BREAK_LINES
     };
 
-    /**A layer together with the information about interpolation attribute / z-coordinate interpolation and the type (point, structure line, breakline)*/
+    /** A layer together with the information about interpolation attribute / z-coordinate interpolation and the type (point, structure line, breakline)*/
     struct LayerData
     {
       QgsVectorLayer* vectorLayer;
@@ -57,25 +57,25 @@ class ANALYSIS_EXPORT QgsInterpolator
 
     virtual ~QgsInterpolator();
 
-    /**Calculates interpolation value for map coordinates x, y
+    /** Calculates interpolation value for map coordinates x, y
        @param x x-coordinate (in map units)
        @param y y-coordinate (in map units)
        @param result out: interpolation result
        @return 0 in case of success*/
     virtual int interpolatePoint( double x, double y, double& result ) = 0;
 
-    // @note not available in python bindings
+    //! @note not available in Python bindings
     const QList<LayerData>& layerData() const { return mLayerData; }
 
   protected:
-    /**Caches the vertex and value data from the provider. All the vertex data
+    /** Caches the vertex and value data from the provider. All the vertex data
      will be held in virtual memory
     @return 0 in case of success*/
     int cacheBaseData();
 
     QVector<vertexData> mCachedBaseData;
 
-    /**Flag that tells if the cache already has been filled*/
+    /** Flag that tells if the cache already has been filled*/
     bool mDataIsCached;
 
     //Information about the input vector layers and the attributes (or z-values) that are used for interpolation
@@ -83,12 +83,12 @@ class ANALYSIS_EXPORT QgsInterpolator
 
   private:
     QgsInterpolator(); //forbidden
-    /**Helper method that adds the vertices of a geometry to the mCachedBaseData
+    /** Helper method that adds the vertices of a geometry to the mCachedBaseData
        @param geom the geometry
        @param zCoord true if the z-coordinate of the geometry is to be interpolated
        @param attributeValue the attribute value for interpolation (if not interpolated from z-coordinate)
      @return 0 in case of success*/
-    int addVerticesToCache( QgsGeometry* geom, bool zCoord, double attributeValue );
+    int addVerticesToCache( const QgsGeometry* geom, bool zCoord, double attributeValue );
 };
 
 #endif
