@@ -51,6 +51,7 @@ class QUndoView;
 class QComboBox;
 class QLabel;
 class QTreeView;
+class QPrinter;
 
 /** \ingroup MapComposer
  * \brief A gui for composing a printable map.
@@ -98,9 +99,12 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     const QString& title() const {return mTitle;}
     void setTitle( const QString& title );
 
-    //! Load template into current or blank composer
-    //! @param newComposer whether to create a new composer first
-    void loadTemplate( const bool newComposer );
+    /** Loads the contents of a template document into the composer's composition.
+     * @param templateDoc template document to load
+     * @param clearExisting set to true to remove all existing composition settings and items before loading template
+     * @returns true if template load was successful
+     */
+    bool loadFromTemplate( const QDomDocument& templateDoc, bool clearExisting );
 
   protected:
     //! Move event
@@ -318,6 +322,9 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //!Show/hide bounding boxes
     void on_mActionShowBoxes_triggered( bool checked );
 
+    //!Show/hide pages
+    void on_mActionShowPage_triggered( bool checked );
+
     //!Show/hide rulers
     void toggleRulers( bool checked );
 
@@ -342,6 +349,9 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //!Last atlas feature
     void on_mActionAtlasLast_triggered();
 
+    //!Jump to a specific atlas page
+    void atlasPageComboEditingFinished();
+
     //! Print the atlas
     void on_mActionPrintAtlas_triggered();
 
@@ -366,40 +376,40 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Save window state
     void saveWindowState();
 
-    /**Add a composer arrow to the item/widget map and creates a configuration widget for it*/
+    /** Add a composer arrow to the item/widget map and creates a configuration widget for it*/
     void addComposerArrow( QgsComposerArrow* arrow );
 
-    /**Add a composer map to the item/widget map and creates a configuration widget for it*/
+    /** Add a composer map to the item/widget map and creates a configuration widget for it*/
     void addComposerMap( QgsComposerMap* map );
 
-    /**Adds a composer label to the item/widget map and creates a configuration widget for it*/
+    /** Adds a composer label to the item/widget map and creates a configuration widget for it*/
     void addComposerLabel( QgsComposerLabel* label );
 
-    /**Adds a composer scale bar to the item/widget map and creates a configuration widget for it*/
+    /** Adds a composer scale bar to the item/widget map and creates a configuration widget for it*/
     void addComposerScaleBar( QgsComposerScaleBar* scalebar );
 
-    /**Adds a composer legend to the item/widget map and creates a configuration widget for it*/
+    /** Adds a composer legend to the item/widget map and creates a configuration widget for it*/
     void addComposerLegend( QgsComposerLegend* legend );
 
-    /**Adds a composer picture to the item/widget map and creates a configuration widget*/
+    /** Adds a composer picture to the item/widget map and creates a configuration widget*/
     void addComposerPicture( QgsComposerPicture* picture );
 
-    /**Adds a composer shape to the item/widget map and creates a configuration widget*/
+    /** Adds a composer shape to the item/widget map and creates a configuration widget*/
     void addComposerShape( QgsComposerShape* shape );
 
-    /**Adds a composer table to the item/widget map and creates a configuration widget*/
+    /** Adds a composer table to the item/widget map and creates a configuration widget*/
     void addComposerTable( QgsComposerAttributeTable* table );
 
-    /**Adds a composer table v2 to the item/widget map and creates a configuration widget*/
+    /** Adds a composer table v2 to the item/widget map and creates a configuration widget*/
     void addComposerTableV2( QgsComposerAttributeTableV2* table, QgsComposerFrame* frame );
 
-    /**Adds composer html and creates a configuration widget*/
+    /** Adds composer html and creates a configuration widget*/
     void addComposerHtmlFrame( QgsComposerHtml* html, QgsComposerFrame* frame );
 
-    /**Removes item from the item/widget map and deletes the configuration widget. Does not delete the item itself*/
+    /** Removes item from the item/widget map and deletes the configuration widget. Does not delete the item itself*/
     void deleteItem( QgsComposerItem* item );
 
-    /**Shows the configuration widget for a composer item*/
+    /** Shows the configuration widget for a composer item*/
     void showItemOptions( QgsComposerItem* i );
 
     //XML, usually connected with QgsProject::readProject and QgsProject::writeProject
@@ -430,26 +440,26 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     void statusZoomCombo_zoomEntered();
 
     //! Updates status bar composition message
-    void updateStatusCompositionMsg( QString message );
+    void updateStatusCompositionMsg( const QString& message );
 
     //! Updates status bar atlas message
-    void updateStatusAtlasMsg( QString message );
+    void updateStatusAtlasMsg( const QString& message );
 
   private:
 
-    /**Establishes the signal slot connections from the QgsComposerView to the composer*/
+    /** Establishes the signal slot connections from the QgsComposerView to the composer*/
     void connectViewSlots();
 
-    /**Establishes the signal slot connections from the QgsComposition to the composer*/
+    /** Establishes the signal slot connections from the QgsComposition to the composer*/
     void connectCompositionSlots();
 
-    /**Establishes other signal slot connections for the composer*/
+    /** Establishes other signal slot connections for the composer*/
     void connectOtherSlots();
 
-    /**Creates the composition widget*/
+    /** Creates the composition widget*/
     void createCompositionWidget();
 
-    /**Sets up the compositions undo/redo connections*/
+    /** Sets up the compositions undo/redo connections*/
     void setupUndoView();
 
     //! True if a composer map contains a WMS layer
@@ -481,7 +491,7 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     void createComposerView();
 
     //! Write a world file
-    void writeWorldFile( QString fileName, double a, double b, double c, double d, double e, double f ) const;
+    void writeWorldFile( const QString& fileName, double a, double b, double c, double d, double e, double f ) const;
 
     //! Updates the grid/guide action status based on compositions grid/guide settings
     void restoreGridSettings();
@@ -501,27 +511,24 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Updates the "set as atlas feature" map layer action, removing it if atlas is disabled
     void updateAtlasMapLayerAction( bool atlasEnabled );
 
-    //! Set default settings for printer page settings based on composition paper size
-    void setPrinterPageDefaults();
-
     //! Load predefined scales from the project's properties
     void loadAtlasPredefinedScalesFromProject();
 
     QPrinter* printer();
 
-    /**Composer title*/
+    /** Composer title*/
     QString mTitle;
 
-    /**Labels in status bar which shows current mouse position*/
+    /** Labels in status bar which shows current mouse position*/
     QLabel* mStatusCursorXLabel;
     QLabel* mStatusCursorYLabel;
     QLabel* mStatusCursorPageLabel;
-    /**Combobox in status bar which shows/adjusts current zoom level*/
+    /** Combobox in status bar which shows/adjusts current zoom level*/
     QComboBox* mStatusZoomCombo;
     QList<double> mStatusZoomLevelsList;
-    /**Label in status bar which shows messages from the composition*/
+    /** Label in status bar which shows messages from the composition*/
     QLabel* mStatusCompositionLabel;
-    /**Label in status bar which shows atlas details*/
+    /** Label in status bar which shows atlas details*/
     QLabel* mStatusAtlasLabel;
 
     //! Pointer to composer view
@@ -559,6 +566,7 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
 
     //! Page & Printer Setup
     QPrinter* mPrinter;
+    bool mSetPageOrientation;
 
     QUndoView* mUndoView;
 
@@ -569,6 +577,8 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     QAction *mActionPreviewProtanope;
     QAction *mActionPreviewDeuteranope;
 
+    QComboBox* mAtlasPageComboBox;
+
     //! We load composer map content from project xml only on demand. Therefore we need to store the real preview mode type
     QMap< QgsComposerMap*, int > mMapsToRestore;
 
@@ -635,7 +645,9 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     void updateAtlasMapLayerAction( QgsVectorLayer* coverageLayer );
 
     //! Sets the printer page orientation when the page orientation changes
-    void setPrinterPageOrientation( QString orientation );
+    void pageOrientationChanged( const QString& orientation );
+
+    void setPrinterPageOrientation();
 
     void disablePreviewMode();
     void activateGrayscalePreview();
@@ -648,6 +660,11 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
 
     void dockVisibilityChanged( bool visible );
 
+    /** Repopulates the atlas page combo box with valid items.
+     */
+    void updateAtlasPageComboBox( int pageCount );
+
+    void atlasFeatureChanged( QgsFeature* feature );
 };
 
 #endif
