@@ -23,7 +23,7 @@
 #include <QFontDialog>
 #include <QWidget>
 
-QgsComposerScaleBarWidget::QgsComposerScaleBarWidget( QgsComposerScaleBar* scaleBar ): QgsComposerItemBaseWidget( 0, scaleBar ), mComposerScaleBar( scaleBar )
+QgsComposerScaleBarWidget::QgsComposerScaleBarWidget( QgsComposerScaleBar* scaleBar ): QgsComposerItemBaseWidget( nullptr, scaleBar ), mComposerScaleBar( scaleBar )
 {
   setupUi( this );
   connectUpdateSignal();
@@ -32,6 +32,10 @@ QgsComposerScaleBarWidget::QgsComposerScaleBarWidget( QgsComposerScaleBar* scale
   QgsComposerItemWidget* itemPropertiesWidget = new QgsComposerItemWidget( this, scaleBar );
   mainLayout->addWidget( itemPropertiesWidget );
 
+  mSegmentSizeRadioGroup.addButton( mFixedSizeRadio );
+  mSegmentSizeRadioGroup.addButton( mFitWidthRadio );
+  connect( &mSegmentSizeRadioGroup, SIGNAL( buttonClicked( QAbstractButton* ) ), this, SLOT( segmentSizeRadioChanged( QAbstractButton* ) ) );
+
   blockMemberSignals( true );
 
   //style combo box
@@ -69,10 +73,10 @@ QgsComposerScaleBarWidget::QgsComposerScaleBarWidget( QgsComposerScaleBar* scale
   mFontColorButton->setAllowAlpha( true );
   mFontColorButton->setContext( "composer" );
 
-  mStrokeColorButton->setColorDialogTitle( tr( "Select stroke color" ) );
+  mStrokeColorButton->setColorDialogTitle( tr( "Select line color" ) );
   mStrokeColorButton->setAllowAlpha( true );
   mStrokeColorButton->setContext( "composer" );
-  mStrokeColorButton->setNoColorString( tr( "Transparent stroke" ) );
+  mStrokeColorButton->setNoColorString( tr( "Transparent line" ) );
   mStrokeColorButton->setShowNoColor( true );
 
   blockMemberSignals( false );
@@ -107,7 +111,7 @@ void QgsComposerScaleBarWidget::refreshMapComboBox()
 
     if ( saveCurrentComboText.isEmpty() && mComposerScaleBar->composerMap() )
     {
-      //combo box was not initialised before
+      //combo box was not initialized before
       mMapComboBox->setCurrentIndex( mMapComboBox->findText( tr( "Map %1" ).arg( mComposerScaleBar->composerMap()->id() ) ) );
     }
   }
@@ -145,7 +149,7 @@ void QgsComposerScaleBarWidget::on_mMapComboBox_activated( const QString& text )
   //extract id
   int id;
   bool conversionOk;
-  QStringList textSplit = text.split( " " );
+  QStringList textSplit = text.split( ' ' );
   if ( textSplit.size() < 1 )
   {
     return;
@@ -221,6 +225,23 @@ void QgsComposerScaleBarWidget::setGuiElements()
   //units
   mUnitsComboBox->setCurrentIndex( mUnitsComboBox->findData(( int )mComposerScaleBar->units() ) );
 
+  if ( mComposerScaleBar->segmentSizeMode() == QgsComposerScaleBar::SegmentSizeFixed )
+  {
+    mFixedSizeRadio->setChecked( true );
+    mSegmentSizeSpinBox->setEnabled( true );
+    mMinWidthSpinBox->setEnabled( false );
+    mMaxWidthSpinBox->setEnabled( false );
+  }
+  else /*if(mComposerScaleBar->segmentSizeMode() == QgsComposerScaleBar::SegmentSizeFitWidth)*/
+  {
+    mFitWidthRadio->setChecked( true );
+    mSegmentSizeSpinBox->setEnabled( false );
+    mMinWidthSpinBox->setEnabled( true );
+    mMaxWidthSpinBox->setEnabled( true );
+  }
+  mMinWidthSpinBox->setValue( mComposerScaleBar->minBarWidth() );
+  mMaxWidthSpinBox->setValue( mComposerScaleBar->maxBarWidth() );
+
   blockMemberSignals( false );
 }
 
@@ -378,7 +399,7 @@ void QgsComposerScaleBarWidget::on_mStrokeColorButton_colorChanged( const QColor
     return;
   }
 
-  mComposerScaleBar->beginCommand( tr( "Scalebar stroke color changed" ) );
+  mComposerScaleBar->beginCommand( tr( "Scalebar line color changed" ) );
   disconnectUpdateSignal();
   QPen newPen = mComposerScaleBar->pen();
   newPen.setColor( newColor );
@@ -626,6 +647,7 @@ void QgsComposerScaleBarWidget::blockMemberSignals( bool block )
   mFillColorButton->blockSignals( block );
   mFillColor2Button->blockSignals( block );
   mStrokeColorButton->blockSignals( block );
+  mSegmentSizeRadioGroup.blockSignals( block );
 }
 
 void QgsComposerScaleBarWidget::connectUpdateSignal()
@@ -669,3 +691,61 @@ void QgsComposerScaleBarWidget::on_mLineCapStyleCombo_currentIndexChanged( int i
   mComposerScaleBar->setLineCapStyle( mLineCapStyleCombo->penCapStyle() );
   mComposerScaleBar->endCommand();
 }
+
+void QgsComposerScaleBarWidget::segmentSizeRadioChanged( QAbstractButton* radio )
+{
+  bool fixedSizeMode = radio == mFixedSizeRadio;
+  mMinWidthSpinBox->setEnabled( !fixedSizeMode );
+  mMaxWidthSpinBox->setEnabled( !fixedSizeMode );
+  mSegmentSizeSpinBox->setEnabled( fixedSizeMode );
+
+  if ( !mComposerScaleBar )
+  {
+    return;
+  }
+
+  mComposerScaleBar->beginCommand( tr( "Scalebar segment size mode" ), QgsComposerMergeCommand::ScaleBarSegmentSize );
+  disconnectUpdateSignal();
+  if ( mFixedSizeRadio->isChecked() )
+  {
+    mComposerScaleBar->setSegmentSizeMode( QgsComposerScaleBar::SegmentSizeFixed );
+    mComposerScaleBar->setNumUnitsPerSegment( mSegmentSizeSpinBox->value() );
+  }
+  else /*if(mFitWidthRadio->isChecked())*/
+  {
+    mComposerScaleBar->setSegmentSizeMode( QgsComposerScaleBar::SegmentSizeFitWidth );
+  }
+  mComposerScaleBar->update();
+  connectUpdateSignal();
+  mComposerScaleBar->endCommand();
+}
+
+void QgsComposerScaleBarWidget::on_mMinWidthSpinBox_valueChanged( int )
+{
+  if ( !mComposerScaleBar )
+  {
+    return;
+  }
+
+  mComposerScaleBar->beginCommand( tr( "Scalebar segment size mode" ), QgsComposerMergeCommand::ScaleBarSegmentSize );
+  disconnectUpdateSignal();
+  mComposerScaleBar->setMinBarWidth( mMinWidthSpinBox->value() );
+  mComposerScaleBar->update();
+  connectUpdateSignal();
+  mComposerScaleBar->endCommand();
+}
+
+void QgsComposerScaleBarWidget::on_mMaxWidthSpinBox_valueChanged( int )
+{
+  if ( !mComposerScaleBar )
+  {
+    return;
+  }
+
+  mComposerScaleBar->beginCommand( tr( "Scalebar segment size mode" ), QgsComposerMergeCommand::ScaleBarSegmentSize );
+  disconnectUpdateSignal();
+  mComposerScaleBar->setMaxBarWidth( mMaxWidthSpinBox->value() );
+  mComposerScaleBar->update();
+  connectUpdateSignal();
+  mComposerScaleBar->endCommand();
+}
