@@ -53,7 +53,7 @@ void QgsAppPluginManagerInterface::clearPythonPluginMetadata()
 
 
 //! add a single plugin to the metadata registry
-void QgsAppPluginManagerInterface::addPluginMetadata( QMap<QString, QString> metadata )
+void QgsAppPluginManagerInterface::addPluginMetadata( const QMap<QString, QString>& metadata )
 {
   if ( metadata.isEmpty() || !metadata.contains( "id" ) )
   {
@@ -72,7 +72,7 @@ void QgsAppPluginManagerInterface::reloadModel()
 
 
 //! return given plugin metadata
-const QMap<QString, QString> * QgsAppPluginManagerInterface::pluginMetadata( QString key ) const
+const QMap<QString, QString> * QgsAppPluginManagerInterface::pluginMetadata( const QString& key ) const
 {
   return mPluginManager->pluginMetadata( key );
 }
@@ -86,7 +86,7 @@ void QgsAppPluginManagerInterface::clearRepositoryList()
 
 
 //! add repository to the repository listWidget
-void QgsAppPluginManagerInterface::addToRepositoryList( QMap<QString, QString> repository )
+void QgsAppPluginManagerInterface::addToRepositoryList( const QMap<QString, QString>& repository )
 {
   mPluginManager->addToRepositoryList( repository );
 }
@@ -96,4 +96,4 @@ void QgsAppPluginManagerInterface::addToRepositoryList( QMap<QString, QString> r
 void QgsAppPluginManagerInterface::pushMessage( const QString &text, QgsMessageBar::MessageLevel level, int duration )
 {
   mPluginManager->pushMessage( text, level, duration );
-}
\ No newline at end of file
+}
