@@ -24,7 +24,7 @@
 #include <QColorDialog>
 
 
-QgsAnnotationWidget::QgsAnnotationWidget( QgsAnnotationItem* item, QWidget * parent, Qt::WindowFlags f ): QWidget( parent, f ), mItem( item ), mMarkerSymbol( 0 )
+QgsAnnotationWidget::QgsAnnotationWidget( QgsAnnotationItem* item, QWidget * parent, Qt::WindowFlags f ): QWidget( parent, f ), mItem( item ), mMarkerSymbol( nullptr )
 {
   setupUi( this );
 
@@ -54,10 +54,12 @@ QgsAnnotationWidget::QgsAnnotationWidget( QgsAnnotationItem* item, QWidget * par
     mBackgroundColorButton->setNoColorString( tr( "Transparent" ) );
     mBackgroundColorButton->setShowNoColor( true );
 
+    connect( mBackgroundColorButton, SIGNAL( colorChanged( QColor ) ), this, SIGNAL( backgroundColorChanged( QColor ) ) );
+
     const QgsMarkerSymbolV2* symbol = mItem->markerSymbol();
     if ( symbol )
     {
-      mMarkerSymbol = dynamic_cast<QgsMarkerSymbolV2*>( symbol->clone() );
+      mMarkerSymbol = symbol->clone();
       updateCenterIcon();
     }
 
@@ -79,7 +81,7 @@ void QgsAnnotationWidget::apply()
     mItem->setFrameColor( mFrameColorButton->color() );
     mItem->setFrameBackgroundColor( mBackgroundColorButton->color() );
     mItem->setMarkerSymbol( mMarkerSymbol );
-    mMarkerSymbol = 0; //item takes ownership
+    mMarkerSymbol = nullptr; //item takes ownership
     mItem->update();
   }
 }
@@ -98,8 +100,8 @@ void QgsAnnotationWidget::on_mMapMarkerButton_clicked()
   {
     return;
   }
-  QgsMarkerSymbolV2* markerSymbol = dynamic_cast<QgsMarkerSymbolV2*>( mMarkerSymbol->clone() );
-  QgsSymbolV2SelectorDialog dlg( markerSymbol, QgsStyleV2::defaultStyle(), 0, this );
+  QgsMarkerSymbolV2* markerSymbol = mMarkerSymbol->clone();
+  QgsSymbolV2SelectorDialog dlg( markerSymbol, QgsStyleV2::defaultStyle(), nullptr, this );
   if ( dlg.exec() == QDialog::Rejected )
   {
     delete markerSymbol;
@@ -112,16 +114,6 @@ void QgsAnnotationWidget::on_mMapMarkerButton_clicked()
   }
 }
 
-void QgsAnnotationWidget::on_mFrameColorButton_colorChanged( const QColor &color )
-{
-  if ( !mItem )
-  {
-    return;
-  }
-
-  mItem->setFrameColor( color );
-}
-
 void QgsAnnotationWidget::updateCenterIcon()
 {
   if ( !mMarkerSymbol )
@@ -132,13 +124,3 @@ void QgsAnnotationWidget::updateCenterIcon()
   mMapMarkerButton->setIcon( icon );
 }
 
-void QgsAnnotationWidget::on_mBackgroundColorButton_colorChanged( const QColor &color )
-{
-  if ( !mItem )
-  {
-    return;
-  }
-
-  mItem->setFrameBackgroundColor( color );
-}
-
