@@ -576,7 +576,7 @@ void QgsCustomization::createTreeItemToolbars()
 void QgsCustomization::createTreeItemDocks()
 {
   QStringList data;
-  data << "Panels";
+  data << "Docks";
 
   QTreeWidgetItem *topItem = new QTreeWidgetItem( data );
 
