@@ -56,11 +56,13 @@ const double QgsDecorationNorthArrow::TOL = 1e-8;
  */
 QgsDecorationNorthArrow::QgsDecorationNorthArrow( QObject* parent )
     : QgsDecorationItem( parent )
+    , mRotationInt( 0 )
+    , mAutomatic( true )
+    , mMarginHorizontal( 0 )
+    , mMarginVertical( 0 )
 {
-  mRotationInt = 0;
-  mAutomatic = true;
-  mPlacementLabels << tr( "Bottom Left" ) << tr( "Top Left" )
-  << tr( "Top Right" ) << tr( "Bottom Right" );
+  mPlacement = BottomLeft;
+  mMarginUnit = QgsSymbolV2::MM;
 
   setName( "North Arrow" );
   projectRead();
@@ -74,27 +76,25 @@ void QgsDecorationNorthArrow::projectRead()
 {
   QgsDecorationItem::projectRead();
   mRotationInt = QgsProject::instance()->readNumEntry( mNameConfig, "/Rotation", 0 );
-  mPlacementIndex = QgsProject::instance()->readNumEntry( mNameConfig, "/Placement", 0 );
   mAutomatic = QgsProject::instance()->readBoolEntry( mNameConfig, "/Automatic", true );
+  mMarginHorizontal = QgsProject::instance()->readNumEntry( mNameConfig, "/MarginH", 0 );
+  mMarginVertical = QgsProject::instance()->readNumEntry( mNameConfig, "/MarginV", 0 );
 }
 
 void QgsDecorationNorthArrow::saveToProject()
 {
   QgsDecorationItem::saveToProject();
   QgsProject::instance()->writeEntry( mNameConfig, "/Rotation", mRotationInt );
-  QgsProject::instance()->writeEntry( mNameConfig, "/Placement", mPlacementIndex );
   QgsProject::instance()->writeEntry( mNameConfig, "/Automatic", mAutomatic );
+  QgsProject::instance()->writeEntry( mNameConfig, "/MarginH", mMarginHorizontal );
+  QgsProject::instance()->writeEntry( mNameConfig, "/MarginV", mMarginVertical );
 }
 
 // Slot called when the buffer menu item is activated
 void QgsDecorationNorthArrow::run()
 {
   QgsDecorationNorthArrowDialog dlg( *this, QgisApp::instance() );
-
-  if ( dlg.exec() )
-  {
-    update();
-  }
+  dlg.exec();
 }
 
 void QgsDecorationNorthArrow::render( QPainter * theQPainter )
@@ -140,28 +140,55 @@ void QgsDecorationNorthArrow::render( QPainter * theQPainter )
 
       //QgsDebugMsg("Rendering north arrow at " + mPlacementLabels.at(mPlacementIndex));
 
-      //Determine placement of label from form combo box
-      switch ( mPlacementIndex )
+      // Set  margin according to selected units
+      int myXOffset = 0;
+      int myYOffset = 0;
+      switch ( mMarginUnit )
       {
-        case 0: // Bottom Left
-          theQPainter->translate( 0, myHeight - myQPixmap.height() );
+        case QgsSymbolV2::MM:
+        {
+          int myPixelsInchX = theQPainter->device()->logicalDpiX();
+          int myPixelsInchY = theQPainter->device()->logicalDpiY();
+          myXOffset = myPixelsInchX * INCHES_TO_MM * mMarginHorizontal;
+          myYOffset = myPixelsInchY * INCHES_TO_MM * mMarginVertical;
           break;
-        case 1: // Top Left
-          //no need to translate for TL corner because we're already at the origin
-          theQPainter->translate( 0, 0 );
+        }
+
+        case QgsSymbolV2::Pixel:
+          myXOffset = mMarginHorizontal - 5; // Minus 5 to shift tight into corner
+          myYOffset = mMarginVertical - 5;
           break;
-        case 2: // Top Right
-          theQPainter->translate( myWidth - myQPixmap.width(), 0 );
+
+        case QgsSymbolV2::Percentage:
+          myXOffset = (( myWidth - myQPixmap.width() ) / 100. ) * mMarginHorizontal;
+          myYOffset = (( myHeight - myQPixmap.height() ) / 100. ) * mMarginVertical;
           break;
-        case 3: // Bottom Right
-          theQPainter->translate( myWidth - myQPixmap.width(),
-                                  myHeight - myQPixmap.height() );
+
+        default:  // Use default of top left
+          break;
+      }
+      //Determine placement of label from form combo box
+      switch ( mPlacement )
+      {
+        case BottomLeft:
+          theQPainter->translate( myXOffset, myHeight - myYOffset - myQPixmap.height() );
+          break;
+        case TopLeft:
+          theQPainter->translate( myXOffset, myYOffset );
+          break;
+        case TopRight:
+          theQPainter->translate( myWidth - myXOffset - myQPixmap.width(), myYOffset );
+          break;
+        case BottomRight:
+          theQPainter->translate( myWidth - myXOffset - myQPixmap.width(),
+                                  myHeight - myYOffset - myQPixmap.height() );
           break;
         default:
         {
           //QgsDebugMsg("Unable to determine where to put north arrow so defaulting to top left");
         }
       }
+
       //rotate the canvas by the north arrow rotation amount
       theQPainter->rotate( mRotationInt );
       //Now we can actually do the drawing, and draw a smooth north arrow even when rotated
