@@ -4,7 +4,7 @@
   -------------------
          begin                : August 2012
          copyright            : (C) Matthias Kuhn
-         email                : matthias dot kuhn at gmx dot ch
+         email                : matthias at opengis dot ch
 
  ***************************************************************************
  *                                                                         *
@@ -29,23 +29,25 @@ class APP_EXPORT QgsDiagramProperties : public QWidget, private Ui::QgsDiagramPr
 
   public:
     QgsDiagramProperties( QgsVectorLayer* layer, QWidget* parent );
-    /**Adds an attribute from the list of available attributes to the assigned attributes with a random color.*/
+
+    ~QgsDiagramProperties();
+
+    /** Adds an attribute from the list of available attributes to the assigned attributes with a random color.*/
     void addAttribute( QTreeWidgetItem * item );
 
   public slots:
     void apply();
     void on_mDiagramTypeComboBox_currentIndexChanged( int index );
-    void on_mTransparencySlider_valueChanged( int value );
     void on_mAddCategoryPushButton_clicked();
     void on_mAttributesTreeWidget_itemDoubleClicked( QTreeWidgetItem * item, int column );
     void on_mFindMaximumValueButton_clicked();
-    void on_mDisplayDiagramsGroupBox_toggled( bool checked );
     void on_mRemoveCategoryPushButton_clicked();
     void on_mDiagramFontButton_clicked();
     void on_mDiagramAttributesTreeWidget_itemDoubleClicked( QTreeWidgetItem * item, int column );
     void on_mEngineSettingsButton_clicked();
-    void showSizeAttributeExpressionDialog();
     void showAddAttributeExpressionDialog();
+    void on_mDiagramStackedWidget_currentChanged( int index );
+    void on_mPlacementComboBox_currentIndexChanged( int index );
 
   protected:
     QFont mDiagramFont;
@@ -53,7 +55,9 @@ class APP_EXPORT QgsDiagramProperties : public QWidget, private Ui::QgsDiagramPr
     QgsVectorLayer* mLayer;
 
   private:
-    int mAvailableAttributes;
+
+    QString guessLegendText( const QString &expression );
+
 };
 
 #endif // QGSDIAGRAMPROPERTIES_H
