@@ -22,10 +22,13 @@
 #include "qgsmaplayerregistry.h"
 #include "qgsmaprenderer.h"
 #include "qgsvectorlayer.h"
+#include "qgisapp.h"
+#include "qgsmapcanvas.h"
 
 #include <QColorDialog>
 #include <QFontDatabase>
 #include <QSettings>
+#include <QDialogButtonBox>
 
 
 QgsLabelPropertyDialog::QgsLabelPropertyDialog( const QString& layerId, int featureId, const QFont& labelFont, const QString& labelText, QWidget * parent, Qt::WindowFlags f ):
@@ -34,6 +37,7 @@ QgsLabelPropertyDialog::QgsLabelPropertyDialog( const QString& layerId, int feat
   setupUi( this );
   fillHaliComboBox();
   fillValiComboBox();
+
   init( layerId, featureId, labelText );
 
   QSettings settings;
@@ -44,6 +48,16 @@ QgsLabelPropertyDialog::~QgsLabelPropertyDialog()
 {
   QSettings settings;
   settings.setValue( QString( "/Windows/ChangeLabelProps/geometry" ), saveGeometry() );
+
+  qDeleteAll( mDataDefinedProperties );
+}
+
+void QgsLabelPropertyDialog::on_buttonBox_clicked( QAbstractButton *button )
+{
+  if ( buttonBox->buttonRole( button ) == QDialogButtonBox::ApplyRole )
+  {
+    emit applied();
+  }
 }
 
 void QgsLabelPropertyDialog::init( const QString& layerId, int featureId, const QString& labelText )
@@ -59,7 +73,7 @@ void QgsLabelPropertyDialog::init( const QString& layerId, int featureId, const
   {
     return;
   }
-  const QgsAttributes& attributeValues = mCurLabelFeat.attributes();
+  QgsAttributes attributeValues = mCurLabelFeat.attributes();
 
   //get layerproperties. Problem: only for pallabeling...
 
@@ -82,8 +96,8 @@ void QgsLabelPropertyDialog::init( const QString& layerId, int featureId, const
       mCurLabelField = vlayer->fieldNameIndex( labelFieldName );
       if ( mCurLabelField >= 0 )
       {
-        mLabelTextLineEdit->setText( attributeValues[mCurLabelField].toString() );
-        const QgsFields& layerFields = vlayer->pendingFields();
+        mLabelTextLineEdit->setText( attributeValues.at( mCurLabelField ).toString() );
+        const QgsFields& layerFields = vlayer->fields();
         switch ( layerFields[mCurLabelField].type() )
         {
           case QVariant::Double:
@@ -111,181 +125,37 @@ void QgsLabelPropertyDialog::init( const QString& layerId, int featureId, const
   updateFont( mLabelFont, false );
 
   //set all the gui elements to the default layer-level values
+  mLabelDistanceSpinBox->clear();
+  mLabelDistanceSpinBox->setSpecialValueText( tr( "Layer default (%1)" ).arg( QString::number( layerSettings.dist, 'f', mLabelDistanceSpinBox->decimals() ) ) );
+  mBufferSizeSpinBox->clear();
+  mBufferSizeSpinBox->setSpecialValueText( tr( "Layer default (%1)" ).arg( QString::number( layerSettings.bufferSize, 'f', mBufferSizeSpinBox->decimals() ) ) );
+  mRotationSpinBox->clear();
+  mXCoordSpinBox->clear();
+  mYCoordSpinBox->clear();
+
+  mShowLabelChkbx->setChecked( true );
   mFontColorButton->setColor( layerSettings.textColor );
   mBufferColorButton->setColor( layerSettings.bufferColor );
-  mLabelDistanceSpinBox->setValue( layerSettings.dist );
-  mBufferSizeSpinBox->setValue( layerSettings.bufferSize );
   mMinScaleSpinBox->setValue( layerSettings.scaleMin );
   mMaxScaleSpinBox->setValue( layerSettings.scaleMax );
-  mHaliComboBox->setCurrentIndex( mHaliComboBox->findText( "Left" ) );
-  mValiComboBox->setCurrentIndex( mValiComboBox->findText( "Bottom" ) );
+  mHaliComboBox->setCurrentIndex( mHaliComboBox->findData( "Left" ) );
+  mValiComboBox->setCurrentIndex( mValiComboBox->findData( "Bottom" ) );
   mFontColorButton->setColorDialogTitle( tr( "Font color" ) );
   mBufferColorButton->setColorDialogTitle( tr( "Buffer color" ) );
 
   disableGuiElements();
 
-  mDataDefinedProperties = layerSettings.dataDefinedProperties;
-  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator propIt = mDataDefinedProperties.constBegin();
-
-  for ( ; propIt != mDataDefinedProperties.constEnd(); ++propIt )
+  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator it = layerSettings.dataDefinedProperties.constBegin();
+  for ( ; it != layerSettings.dataDefinedProperties.constEnd(); ++it )
   {
-    QgsDataDefined* dd = propIt.value();
-    if ( !dd )
-    {
-      continue;
-    }
-    QString ddField = dd->field();
-    if ( !dd->isActive() || dd->useExpression() || ddField.isEmpty() )
-    {
-      continue; // can only modify attributes with an active data definition of a mapped field
-    }
-
-    int ddIndx = vlayer->fieldNameIndex( ddField );
-    if ( ddIndx == -1 )
-    {
-      continue;
-    }
-
-    QgsDebugMsg( QString( "ddField: %1" ).arg( ddField ) );
-
-    bool ok = false;
-    switch ( propIt.key() )
-    {
-      case QgsPalLayerSettings::Show:
-      { // new scope to assign variables
-        mShowLabelChkbx->setEnabled( true );
-        int showLabel = mCurLabelFeat.attribute( ddIndx ).toInt( &ok );
-        mShowLabelChkbx->setChecked( !ok || showLabel != 0 );
-        break;
-      }
-      case QgsPalLayerSettings::AlwaysShow:
-        mAlwaysShowChkbx->setEnabled( true );
-        mAlwaysShowChkbx->setChecked( mCurLabelFeat.attribute( ddIndx ).toBool() );
-        break;
-      case QgsPalLayerSettings::MinScale:
-      {
-        mMinScaleSpinBox->setEnabled( true );
-        int minScale = mCurLabelFeat.attribute( ddIndx ).toInt( &ok );
-        if ( ok )
-        {
-          mMinScaleSpinBox->setValue( minScale );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::MaxScale:
-      {
-        mMaxScaleSpinBox->setEnabled( true );
-        int maxScale = mCurLabelFeat.attribute( ddIndx ).toInt( &ok );
-        if ( ok )
-        {
-          mMaxScaleSpinBox->setValue( maxScale );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::BufferSize:
-      {
-        mBufferSizeSpinBox->setEnabled( true );
-        double bufferSize = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mBufferSizeSpinBox->setValue( bufferSize );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::PositionX:
-      {
-        mXCoordSpinBox->setEnabled( true );
-        double posX = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mXCoordSpinBox->setValue( posX );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::PositionY:
-      {
-        mYCoordSpinBox->setEnabled( true );
-        double posY = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mYCoordSpinBox->setValue( posY );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::LabelDistance:
-      {
-        mLabelDistanceSpinBox->setEnabled( true );
-        double labelDist = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mLabelDistanceSpinBox->setValue( labelDist );
-        }
-        break;
-      }
-      case QgsPalLayerSettings::Hali:
-        mHaliComboBox->setEnabled( true );
-        mHaliComboBox->setCurrentIndex( mHaliComboBox->findText( mCurLabelFeat.attribute( ddIndx ).toString(), Qt::MatchFixedString ) );
-        break;
-      case QgsPalLayerSettings::Vali:
-        mValiComboBox->setEnabled( true );
-        mValiComboBox->setCurrentIndex( mValiComboBox->findText( mCurLabelFeat.attribute( ddIndx ).toString(), Qt::MatchFixedString ) );
-        break;
-      case QgsPalLayerSettings::BufferColor:
-        mBufferColorButton->setEnabled( true );
-        mBufferColorButton->setColor( QColor( mCurLabelFeat.attribute( ddIndx ).toString() ) );
-        break;
-      case QgsPalLayerSettings::Color:
-        mFontColorButton->setEnabled( true );
-        mFontColorButton->setColor( QColor( mCurLabelFeat.attribute( ddIndx ).toString() ) );
-        break;
-      case QgsPalLayerSettings::Rotation:
-      {
-        mRotationSpinBox->setEnabled( true );
-        double rot = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mRotationSpinBox->setValue( rot );
-        }
-        break;
-      }
-
-      //font related properties
-      case QgsPalLayerSettings::Family:
-        mFontFamilyCmbBx->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::FontStyle:
-        mFontStyleCmbBx->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::Underline:
-        mFontUnderlineBtn->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::Strikeout:
-        mFontStrikethroughBtn->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::Bold:
-        mFontBoldBtn->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::Italic:
-        mFontItalicBtn->setEnabled( true );
-        break;
-      case QgsPalLayerSettings::Size:
-      {
-        mFontSizeSpinBox->setEnabled( true );
-        double size = mCurLabelFeat.attribute( ddIndx ).toDouble( &ok );
-        if ( ok )
-        {
-          mFontSizeSpinBox->setValue( size );
-        }
-        else
-        {
-          mFontSizeSpinBox->setValue( 0 );
-        }
-        break;
-      }
-      default:
-        break;
-    }
+    mDataDefinedProperties.insert( it.key(), it.value() ? new QgsDataDefined( *it.value() ) : nullptr );
   }
+
+  //set widget values from data defined results
+  setDataDefinedValues( layerSettings, vlayer );
+  //enable widgets connected to data defined fields
+  enableDataDefinedWidgets( vlayer );
+
   blockElementSignals( false );
 }
 
@@ -337,6 +207,244 @@ void QgsLabelPropertyDialog::blockElementSignals( bool block )
   mRotationSpinBox->blockSignals( block );
 }
 
+void QgsLabelPropertyDialog::setDataDefinedValues( const QgsPalLayerSettings &layerSettings, QgsVectorLayer* vlayer )
+{
+  //loop through data defined properties and set all the GUI widget values. We can do this
+  //even if the data defined property is set to an expression, as it's useful to show
+  //users what the evaluated property is...
+
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::atlasScope( nullptr )
+  << QgsExpressionContextUtils::mapSettingsScope( QgisApp::instance()->mapCanvas()->mapSettings() )
+  << QgsExpressionContextUtils::layerScope( vlayer );
+  context.setFeature( mCurLabelFeat );
+
+  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator propIt = mDataDefinedProperties.constBegin();
+  for ( ; propIt != mDataDefinedProperties.constEnd(); ++propIt )
+  {
+    QgsDataDefined* dd = propIt.value();
+    if ( !dd || !dd->isActive() )
+    {
+      continue;
+    }
+
+    if ( !dd->expressionIsPrepared() )
+    {
+      dd->prepareExpression( context );
+    }
+
+    //TODO - pass expression context
+    QVariant result = layerSettings.dataDefinedValue( propIt.key(), mCurLabelFeat, vlayer->fields(), &context );
+    if ( !result.isValid() || result.isNull() )
+    {
+      //could not evaluate data defined value
+      continue;
+    }
+
+    bool ok = false;
+    switch ( propIt.key() )
+    {
+      case QgsPalLayerSettings::Show:
+      {
+        int showLabel = result.toInt( &ok );
+        mShowLabelChkbx->setChecked( !ok || showLabel != 0 );
+        break;
+      }
+      case QgsPalLayerSettings::AlwaysShow:
+        mAlwaysShowChkbx->setChecked( result.toBool() );
+        break;
+      case QgsPalLayerSettings::MinScale:
+      {
+        int minScale = result.toInt( &ok );
+        if ( ok )
+        {
+          mMinScaleSpinBox->setValue( minScale );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::MaxScale:
+      {
+        int maxScale = result.toInt( &ok );
+        if ( ok )
+        {
+          mMaxScaleSpinBox->setValue( maxScale );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::BufferSize:
+      {
+        double bufferSize = result.toDouble( &ok );
+        if ( ok )
+        {
+          mBufferSizeSpinBox->setValue( bufferSize );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::PositionX:
+      {
+        double posX = result.toDouble( &ok );
+        if ( ok )
+        {
+          mXCoordSpinBox->setValue( posX );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::PositionY:
+      {
+        double posY = result.toDouble( &ok );
+        if ( ok )
+        {
+          mYCoordSpinBox->setValue( posY );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::LabelDistance:
+      {
+        double labelDist = result.toDouble( &ok );
+        if ( ok )
+        {
+          mLabelDistanceSpinBox->setValue( labelDist );
+        }
+        break;
+      }
+      case QgsPalLayerSettings::Hali:
+        mHaliComboBox->setCurrentIndex( mHaliComboBox->findData( result.toString() ) );
+        break;
+      case QgsPalLayerSettings::Vali:
+        mValiComboBox->setCurrentIndex( mValiComboBox->findData( result.toString() ) );
+        break;
+      case QgsPalLayerSettings::BufferColor:
+        mBufferColorButton->setColor( QColor( result.toString() ) );
+        break;
+      case QgsPalLayerSettings::Color:
+        mFontColorButton->setColor( QColor( result.toString() ) );
+        break;
+      case QgsPalLayerSettings::Rotation:
+      {
+        double rot = result.toDouble( &ok );
+        if ( ok )
+        {
+          mRotationSpinBox->setValue( rot );
+        }
+        break;
+      }
+
+      case QgsPalLayerSettings::Size:
+      {
+        double size = result.toDouble( &ok );
+        if ( ok )
+        {
+          mFontSizeSpinBox->setValue( size );
+        }
+        else
+        {
+          mFontSizeSpinBox->setValue( 0 );
+        }
+        break;
+      }
+      default:
+        break;
+    }
+  }
+}
+
+void QgsLabelPropertyDialog::enableDataDefinedWidgets( QgsVectorLayer* vlayer )
+{
+  //loop through data defined properties, this time setting whether or not the widgets are enabled
+  //this can only be done for properties which are assigned to fields
+  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator propIt = mDataDefinedProperties.constBegin();
+  for ( ; propIt != mDataDefinedProperties.constEnd(); ++propIt )
+  {
+    QgsDataDefined* dd = propIt.value();
+    if ( !dd )
+    {
+      continue;
+    }
+    QString ddField = dd->field();
+    if ( !dd->isActive() || dd->useExpression() || ddField.isEmpty() )
+    {
+      continue; // can only modify attributes with an active data definition of a mapped field
+    }
+
+    int ddIndx = vlayer->fieldNameIndex( ddField );
+    if ( ddIndx == -1 )
+    {
+      continue;
+    }
+
+    QgsDebugMsg( QString( "ddField: %1" ).arg( ddField ) );
+
+    switch ( propIt.key() )
+    {
+      case QgsPalLayerSettings::Show:
+        mShowLabelChkbx->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::AlwaysShow:
+        mAlwaysShowChkbx->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::MinScale:
+        mMinScaleSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::MaxScale:
+        mMaxScaleSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::BufferSize:
+        mBufferSizeSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::PositionX:
+        mXCoordSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::PositionY:
+        mYCoordSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::LabelDistance:
+        mLabelDistanceSpinBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Hali:
+        mHaliComboBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Vali:
+        mValiComboBox->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::BufferColor:
+        mBufferColorButton->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Color:
+        mFontColorButton->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Rotation:
+        mRotationSpinBox->setEnabled( true );
+        break;
+        //font related properties
+      case QgsPalLayerSettings::Family:
+        mFontFamilyCmbBx->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::FontStyle:
+        mFontStyleCmbBx->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Underline:
+        mFontUnderlineBtn->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Strikeout:
+        mFontStrikethroughBtn->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Bold:
+        mFontBoldBtn->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Italic:
+        mFontItalicBtn->setEnabled( true );
+        break;
+      case QgsPalLayerSettings::Size:
+        mFontSizeSpinBox->setEnabled( true );
+        break;
+      default:
+        break;
+    }
+  }
+}
+
 void QgsLabelPropertyDialog::updateFont( const QFont& font, bool block )
 {
   // update background reference font
@@ -361,7 +469,7 @@ void QgsLabelPropertyDialog::updateFont( const QFont& font, bool block )
 void QgsLabelPropertyDialog::populateFontStyleComboBox()
 {
   mFontStyleCmbBx->clear();
-  foreach ( const QString &style, mFontDB.styles( mLabelFont.family() ) )
+  Q_FOREACH ( const QString &style, mFontDB.styles( mLabelFont.family() ) )
   {
     mFontStyleCmbBx->addItem( style );
   }
@@ -378,17 +486,18 @@ void QgsLabelPropertyDialog::populateFontStyleComboBox()
 
 void QgsLabelPropertyDialog::fillHaliComboBox()
 {
-  mHaliComboBox->addItem( "Left" );
-  mHaliComboBox->addItem( "Center" );
-  mHaliComboBox->addItem( "Right" );
+  mHaliComboBox->addItem( tr( "Left" ), "Left" );
+  mHaliComboBox->addItem( tr( "Center" ), "Center" );
+  mHaliComboBox->addItem( tr( "Right" ), "Right" );
 }
 
 void QgsLabelPropertyDialog::fillValiComboBox()
 {
-  mValiComboBox->addItem( "Bottom" );
-  mValiComboBox->addItem( "Base" );
-  mValiComboBox->addItem( "Half" );
-  mValiComboBox->addItem( "Top" );
+  mValiComboBox->addItem( tr( "Bottom" ), "Bottom" );
+  mValiComboBox->addItem( tr( "Base" ), "Base" );
+  mValiComboBox->addItem( tr( "Half" ), "Half" );
+  mValiComboBox->addItem( tr( "Cap" ), "Cap" );
+  mValiComboBox->addItem( tr( "Top" ), "Top" );
 }
 
 void QgsLabelPropertyDialog::on_mShowLabelChkbx_toggled( bool chkd )
@@ -413,17 +522,35 @@ void QgsLabelPropertyDialog::on_mMaxScaleSpinBox_valueChanged( int i )
 
 void QgsLabelPropertyDialog::on_mLabelDistanceSpinBox_valueChanged( double d )
 {
-  insertChangedValue( QgsPalLayerSettings::LabelDistance, d );
+  QVariant distance( d );
+  if ( d < 0 )
+  {
+    //null value so that distance is reset to default
+    distance.clear();
+  }
+  insertChangedValue( QgsPalLayerSettings::LabelDistance, distance );
 }
 
 void QgsLabelPropertyDialog::on_mXCoordSpinBox_valueChanged( double d )
 {
-  insertChangedValue( QgsPalLayerSettings::PositionX, d );
+  QVariant x( d );
+  if ( d < mXCoordSpinBox->minimum() + mXCoordSpinBox->singleStep() )
+  {
+    //null value
+    x.clear();
+  }
+  insertChangedValue( QgsPalLayerSettings::PositionX, x );
 }
 
 void QgsLabelPropertyDialog::on_mYCoordSpinBox_valueChanged( double d )
 {
-  insertChangedValue( QgsPalLayerSettings::PositionY, d );
+  QVariant y( d );
+  if ( d < mYCoordSpinBox->minimum() + mYCoordSpinBox->singleStep() )
+  {
+    //null value
+    y.clear();
+  }
+  insertChangedValue( QgsPalLayerSettings::PositionY, y );
 }
 
 void QgsLabelPropertyDialog::on_mFontFamilyCmbBx_currentFontChanged( const QFont& f )
@@ -481,12 +608,24 @@ void QgsLabelPropertyDialog::on_mFontSizeSpinBox_valueChanged( double d )
 
 void QgsLabelPropertyDialog::on_mBufferSizeSpinBox_valueChanged( double d )
 {
-  insertChangedValue( QgsPalLayerSettings::BufferSize, d );
+  QVariant size( d );
+  if ( d < 0 )
+  {
+    //null value so that size is reset to default
+    size.clear();
+  }
+  insertChangedValue( QgsPalLayerSettings::BufferSize, size );
 }
 
 void QgsLabelPropertyDialog::on_mRotationSpinBox_valueChanged( double d )
 {
-  insertChangedValue( QgsPalLayerSettings::Rotation, d );
+  QVariant rotation( d );
+  if ( d < 0 )
+  {
+    //null value so that size is reset to default
+    rotation.clear();
+  }
+  insertChangedValue( QgsPalLayerSettings::Rotation, rotation );
 }
 
 void QgsLabelPropertyDialog::on_mFontColorButton_colorChanged( const QColor &color )
@@ -499,14 +638,14 @@ void QgsLabelPropertyDialog::on_mBufferColorButton_colorChanged( const QColor &c
   insertChangedValue( QgsPalLayerSettings::BufferColor, color.name() );
 }
 
-void QgsLabelPropertyDialog::on_mHaliComboBox_currentIndexChanged( const QString& text )
+void QgsLabelPropertyDialog::on_mHaliComboBox_currentIndexChanged( const int index )
 {
-  insertChangedValue( QgsPalLayerSettings::Hali, text );
+  insertChangedValue( QgsPalLayerSettings::Hali, mHaliComboBox->itemData( index ) );
 }
 
-void QgsLabelPropertyDialog::on_mValiComboBox_currentIndexChanged( const QString& text )
+void QgsLabelPropertyDialog::on_mValiComboBox_currentIndexChanged( const int index )
 {
-  insertChangedValue( QgsPalLayerSettings::Vali, text );
+  insertChangedValue( QgsPalLayerSettings::Vali, mValiComboBox->itemData( index ) );
 }
 
 void QgsLabelPropertyDialog::on_mLabelTextLineEdit_textChanged( const QString& text )
@@ -517,14 +656,14 @@ void QgsLabelPropertyDialog::on_mLabelTextLineEdit_textChanged( const QString& t
   }
 }
 
-void QgsLabelPropertyDialog::insertChangedValue( QgsPalLayerSettings::DataDefinedProperties p, QVariant value )
+void QgsLabelPropertyDialog::insertChangedValue( QgsPalLayerSettings::DataDefinedProperties p, const QVariant& value )
 {
-  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator ddIt = mDataDefinedProperties.find( p );
+  QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* >::const_iterator ddIt = mDataDefinedProperties.constFind( p );
   if ( ddIt != mDataDefinedProperties.constEnd() )
   {
     QgsDataDefined* dd = ddIt.value();
 
-    if ( dd->isActive() && !dd->useExpression() && !dd->field().isEmpty() )
+    if ( dd && dd->isActive() && !dd->useExpression() && !dd->field().isEmpty() )
     {
       mChangedProperties.insert( mCurLabelFeat.fieldNameIndex( dd->field() ), value );
     }
