@@ -24,18 +24,26 @@
 #include <QDialog>
 
 
-/**A dialog to enter data defined label attributes*/
+/** A dialog to enter data defined label attributes*/
 class APP_EXPORT QgsLabelPropertyDialog: public QDialog, private Ui::QgsLabelPropertyDialogBase
 {
     Q_OBJECT
   public:
-    QgsLabelPropertyDialog( const QString& layerId, int featureId, const QFont& labelFont, const QString& labelText, QWidget * parent = 0, Qt::WindowFlags f = 0 );
+    QgsLabelPropertyDialog( const QString& layerId, int featureId, const QFont& labelFont, const QString& labelText, QWidget * parent = nullptr, Qt::WindowFlags f = nullptr );
     ~QgsLabelPropertyDialog();
 
-    /**Returns properties changed by the user*/
+    /** Returns properties changed by the user*/
     const QgsAttributeMap& changedProperties() const { return mChangedProperties; }
 
+  signals:
+
+    /** Emitted when dialog settings are applied
+     * @note added in QGIS 2.9
+     */
+    void applied();
+
   private slots:
+    void on_buttonBox_clicked( QAbstractButton * button );
     void on_mShowLabelChkbx_toggled( bool chkd );
     void on_mAlwaysShowChkbx_toggled( bool chkd );
     void on_mMinScaleSpinBox_valueChanged( int i );
@@ -54,17 +62,20 @@ class APP_EXPORT QgsLabelPropertyDialog: public QDialog, private Ui::QgsLabelPro
     void on_mRotationSpinBox_valueChanged( double d );
     void on_mFontColorButton_colorChanged( const QColor &color );
     void on_mBufferColorButton_colorChanged( const QColor &color );
-    void on_mHaliComboBox_currentIndexChanged( const QString& text );
-    void on_mValiComboBox_currentIndexChanged( const QString& text );
+    void on_mHaliComboBox_currentIndexChanged( const int index );
+    void on_mValiComboBox_currentIndexChanged( const int index );
     void on_mLabelTextLineEdit_textChanged( const QString& text );
 
   private:
-    /**Sets activation / values to the gui elements depending on the label settings and feature values*/
+    /** Sets activation / values to the gui elements depending on the label settings and feature values*/
     void init( const QString& layerId, int featureId, const QString& labelText );
     void disableGuiElements();
-    /**Block / unblock all input element signals*/
+    /** Block / unblock all input element signals*/
     void blockElementSignals( bool block );
 
+    void setDataDefinedValues( const QgsPalLayerSettings &layerSettings, QgsVectorLayer* vlayer );
+    void enableDataDefinedWidgets( QgsVectorLayer* vlayer );
+
     /** Updates font when family or style is updated */
     void updateFont( const QFont& font, bool block = true );
 
@@ -74,8 +85,8 @@ class APP_EXPORT QgsLabelPropertyDialog: public QDialog, private Ui::QgsLabelPro
     void fillHaliComboBox();
     void fillValiComboBox();
 
-    /**Insert changed value into mChangedProperties*/
-    void insertChangedValue( QgsPalLayerSettings::DataDefinedProperties p, QVariant value );
+    /** Insert changed value into mChangedProperties*/
+    void insertChangedValue( QgsPalLayerSettings::DataDefinedProperties p, const QVariant& value );
 
     QgsAttributeMap mChangedProperties;
     QMap< QgsPalLayerSettings::DataDefinedProperties, QgsDataDefined* > mDataDefinedProperties;
@@ -83,7 +94,7 @@ class APP_EXPORT QgsLabelPropertyDialog: public QDialog, private Ui::QgsLabelPro
 
     QFontDatabase mFontDB;
 
-    /**Label field for the current layer (or -1 if none)*/
+    /** Label field for the current layer (or -1 if none)*/
     int mCurLabelField;
 
     /** Current feature */
