@@ -24,79 +24,81 @@
 #include "qgspoint.h"
 class QgsRubberBand;
 
-/**Base class for map tools that modify label properties*/
+/** Base class for map tools that modify label properties*/
 class APP_EXPORT QgsMapToolLabel: public QgsMapTool
 {
+    Q_OBJECT
+
   public:
     QgsMapToolLabel( QgsMapCanvas* canvas );
     ~QgsMapToolLabel();
 
-    /**Returns true if label move can be applied to a layer
+    /** Returns true if label move can be applied to a layer
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if labels of layer can be moved*/
     bool labelMoveable( QgsMapLayer* ml, int& xCol, int& yCol ) const;
-    /**Returns true if diagram move can be applied to a layer
+    /** Returns true if diagram move can be applied to a layer
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if labels of layer can be moved*/
     bool diagramMoveable( QgsMapLayer* ml, int& xCol, int& yCol ) const;
-    /**Returns true if layer has attribute fields set up
+    /** Returns true if layer has attribute fields set up
         @param xCol out: index of the attribute for data defined x coordinate
         @param yCol out: index of the attribute for data defined y coordinate
         @return true if layer fields set up and exist*/
     bool layerCanPin( QgsMapLayer* ml, int& xCol, int& yCol ) const;
-    /**Returns true if layer has attribute field set up
+    /** Returns true if layer has attribute field set up
       @param showCol out: attribute column for data defined label showing*/
     bool layerCanShowHide( QgsMapLayer* layer, int& showCol ) const;
-    /**Checks if labels in a layer can be rotated
+    /** Checks if labels in a layer can be rotated
       @param rotationCol out: attribute column for data defined label rotation*/
     bool layerIsRotatable( QgsMapLayer *layer, int& rotationCol ) const;
 
   protected:
     QgsRubberBand* mLabelRubberBand;
     QgsRubberBand* mFeatureRubberBand;
-    /**Shows label fixpoint (left/bottom by default)*/
+    /** Shows label fixpoint (left/bottom by default)*/
     QgsRubberBand* mFixPointRubberBand;
 
-    /**Currently dragged label position*/
+    /** Currently dragged label position*/
     QgsLabelPosition mCurrentLabelPos;
 
-    /**Returns label position for mouse click location
+    /** Returns label position for mouse click location
       @param e mouse event
       @param p out: label position
       @return true in case of success, false if no label at this location*/
     bool labelAtPosition( QMouseEvent* e, QgsLabelPosition& p );
 
-    /**Finds out rotation point of current label position
+    /** Finds out rotation point of current label position
       @param ignoreUpsideDown treat label as right-side-up
       @return true in case of success*/
     bool rotationPoint( QgsPoint& pos, bool ignoreUpsideDown = false, bool rotatingUnpinned = false );
 
-    /**Creates label / feature / fixpoint rubber bands for the current label position*/
+    /** Creates label / feature / fixpoint rubber bands for the current label position*/
     void createRubberBands();
 
-    /**Removes label / feature / fixpoint rubber bands*/
+    /** Removes label / feature / fixpoint rubber bands*/
     void deleteRubberBands();
 
-    /**Returns vector layer for current label position*/
+    /** Returns vector layer for current label position*/
     QgsVectorLayer* currentLayer();
 
-    /**Returns layer settings of current label position*/
+    /** Returns layer settings of current label position*/
     QgsPalLayerSettings& currentLabelSettings( bool* ok );
 //    const QgsPalLayerSettings& currentLabelSettings( bool* ok ) const;
 
-    /**Returns current label's text
+    /** Returns current label's text
       @param trunc number of chars to truncate to, with ... added */
     QString currentLabelText( int trunc = 0 );
 
     void currentAlignment( QString& hali, QString& vali );
 
-    /**Gets vector feature for current label pos
+    /** Gets vector feature for current label pos
       @return true in case of success*/
     bool currentFeature( QgsFeature& f, bool fetchGeom = false );
 
-    /**Returns the font for the current feature (considering default font and data defined properties)*/
+    /** Returns the font for the current feature (considering default font and data defined properties)*/
     QFont labelFontCurrentFeature();
 
 //    /**Returns a data defined attribute column
@@ -107,16 +109,16 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
 //      @return invalid QVariant if one does not exist or an expression is used instead */
 //    QVariant dataDefinedColumn( QgsPalLayerSettings::DataDefinedProperties p, QgsPalLayerSettings& labelSettings );
 
-    /**Returns a data defined attribute column index
+    /** Returns a data defined attribute column index
       @return -1 if column does not exist or an expression is used instead */
 //    int dataDefinedColumnIndex( QgsPalLayerSettings::DataDefinedProperties p, const QgsPalLayerSettings& labelSettings, const QgsVectorLayer* vlayer ) const;
 
     int dataDefinedColumnIndex( QgsPalLayerSettings::DataDefinedProperties p, QgsVectorLayer* vlayer ) const;
 
-    /**Returns whether to preserve predefined rotation data during label pin/unpin operations*/
+    /** Returns whether to preserve predefined rotation data during label pin/unpin operations*/
     bool preserveRotation();
 
-    /**Get data defined position of a feature
+    /** Get data defined position of a feature
       @param vlayer vector layer
       @param featureId feature identification integer
       @param x out: data defined x-coordinate
@@ -126,9 +128,9 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
       @param xCol out: index of the x position column
       @param yCol out: index of the y position column
       @return false if layer does not have data defined label position enabled*/
-    bool dataDefinedPosition( QgsVectorLayer* vlayer, const QgsFeatureId &featureId, double& x, bool& xSuccess, double& y, bool& ySuccess, int& xCol, int& yCol ) const;
+    bool dataDefinedPosition( QgsVectorLayer* vlayer, QgsFeatureId featureId, double& x, bool& xSuccess, double& y, bool& ySuccess, int& xCol, int& yCol ) const;
 
-    /**Returns data defined rotation of a feature.
+    /** Returns data defined rotation of a feature.
       @param vlayer vector layer
       @param featureId feature identification integer
       @param rotation out: rotation value
@@ -136,9 +138,9 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
       @param ignoreXY ignore that x and y are required to be data-defined
       @return true if data defined rotation is enabled on the layer
       */
-    bool dataDefinedRotation( QgsVectorLayer* vlayer, const QgsFeatureId &featureId, double& rotation, bool& rotationSuccess, bool ignoreXY = false ) const;
+    bool dataDefinedRotation( QgsVectorLayer* vlayer, QgsFeatureId featureId, double& rotation, bool& rotationSuccess, bool ignoreXY = false ) const;
 
-    /**Returns data defined show/hide of a feature.
+    /** Returns data defined show/hide of a feature.
       @param vlayer vector layer
       @param featureId feature identification integer
       @param show out: show/hide value
@@ -146,7 +148,7 @@ class APP_EXPORT QgsMapToolLabel: public QgsMapTool
       @param showCol out: index of the show label column
       @return true if data defined show/hide is enabled on the layer
       */
-    bool dataDefinedShowHide( QgsVectorLayer* vlayer, const QgsFeatureId &featureId, int& show, bool& showSuccess, int& showCol ) const;
+    bool dataDefinedShowHide( QgsVectorLayer* vlayer, QgsFeatureId featureId, int& show, bool& showSuccess, int& showCol ) const;
 
   private:
     QgsPalLayerSettings mInvalidLabelSettings;
