@@ -25,7 +25,7 @@
 class QgsRubberBand;
 class QgsLabelPosition;
 
-/**A map tool for pinning (writing to attribute table) and unpinning labelpositions and rotation*/
+/** A map tool for pinning (writing to attribute table) and unpinning labelpositions and rotation*/
 class APP_EXPORT QgsMapToolPinLabels: public QgsMapToolLabel
 {
     Q_OBJECT
@@ -35,13 +35,13 @@ class APP_EXPORT QgsMapToolPinLabels: public QgsMapToolLabel
     ~QgsMapToolPinLabels();
 
     //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse press event
-    virtual void canvasPressEvent( QMouseEvent * e ) override;
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse release event
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
     bool isShowingPinned() const { return mShowPinned; }
     void setShowingPinned( bool showing ) { mShowPinned = showing; }
