@@ -19,7 +19,7 @@
 #include "qgsmaptoolcapture.h"
 #include "qgsmapmouseevent.h"
 
-/**A map tool that draws a line and splits the features cut by the line*/
+/** A map tool that draws a line and splits the features cut by the line*/
 class APP_EXPORT QgsMapToolReshape: public QgsMapToolCapture
 {
     Q_OBJECT
@@ -27,7 +27,7 @@ class APP_EXPORT QgsMapToolReshape: public QgsMapToolCapture
   public:
     QgsMapToolReshape( QgsMapCanvas* canvas );
     virtual ~QgsMapToolReshape();
-    void canvasMapReleaseEvent( QgsMapMouseEvent * e ) override;
+    void cadCanvasReleaseEvent( QgsMapMouseEvent * e ) override;
 };
 
 #endif
