@@ -37,13 +37,12 @@ QgsMapToolSelect::QgsMapToolSelect( QgsMapCanvas* canvas )
   mBorderColour = QColor( 254, 58, 29, 100 );
 }
 
-void QgsMapToolSelect::canvasReleaseEvent( QMouseEvent * e )
+void QgsMapToolSelect::canvasReleaseEvent( QgsMapMouseEvent* e )
 {
   QgsVectorLayer* vlayer = QgsMapToolSelectUtils::getCurrentVectorLayer( mCanvas );
-  if ( vlayer == NULL )
-  {
+  if ( !vlayer )
     return;
-  }
+
   QgsRubberBand rubberBand( mCanvas, QGis::Polygon );
   rubberBand.setFillColor( mFillColor );
   rubberBand.setBorderColor( mBorderColour );
@@ -51,7 +50,7 @@ void QgsMapToolSelect::canvasReleaseEvent( QMouseEvent * e )
   QgsMapToolSelectUtils::expandSelectRectangle( selectRect, vlayer, e->pos() );
   QgsMapToolSelectUtils::setRubberBand( mCanvas, selectRect, &rubberBand );
   QgsGeometry* selectGeom = rubberBand.asGeometry();
-  bool doDifference = e->modifiers() & Qt::ControlModifier ? true : false;
+  bool doDifference = e->modifiers() & Qt::ControlModifier;
   QgsMapToolSelectUtils::setSelectFeatures( mCanvas, selectGeom, false, doDifference, true );
   delete selectGeom;
   rubberBand.reset( QGis::Polygon );
