@@ -22,7 +22,7 @@
 #include "qgsfeature.h"
 
 
-/**A map tool for showing or hidding a feature's label*/
+/** A map tool for showing or hidding a feature's label*/
 class APP_EXPORT QgsMapToolShowHideLabels : public QgsMapToolLabel
 {
     Q_OBJECT
@@ -32,13 +32,13 @@ class APP_EXPORT QgsMapToolShowHideLabels : public QgsMapToolLabel
     ~QgsMapToolShowHideLabels();
 
     //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse press event
-    virtual void canvasPressEvent( QMouseEvent * e ) override;
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse release event
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
   protected:
 
@@ -66,7 +66,7 @@ class APP_EXPORT QgsMapToolShowHideLabels : public QgsMapToolLabel
 
     //! Show or hide chosen label by setting data defined Show Label to 0
     bool showHideLabel( QgsVectorLayer* vlayer,
-                        const QgsFeatureId &fid,
+                        QgsFeatureId fid,
                         bool hide );
 };
 
