@@ -25,17 +25,35 @@
 #include "qgsvectorlayer.h"
 #include "qgsvectordataprovider.h"
 #include "qgsattributeeditor.h"
+#include "qgsstatisticalsummary.h"
 
 #include <limits>
 #include <QComboBox>
 #include <QSettings>
 
+QList< QgsStatisticalSummary::Statistic > QgsMergeAttributesDialog::mDisplayStats =
+  QList< QgsStatisticalSummary::Statistic > () << QgsStatisticalSummary::Count
+  << QgsStatisticalSummary::Sum
+  << QgsStatisticalSummary::Mean
+  << QgsStatisticalSummary::Median
+  << QgsStatisticalSummary::StDev
+  << QgsStatisticalSummary::StDevSample
+  << QgsStatisticalSummary::Min
+  << QgsStatisticalSummary::Max
+  << QgsStatisticalSummary::Range
+  << QgsStatisticalSummary::Minority
+  << QgsStatisticalSummary::Majority
+  << QgsStatisticalSummary::Variety
+  << QgsStatisticalSummary::FirstQuartile
+  << QgsStatisticalSummary::ThirdQuartile
+  << QgsStatisticalSummary::InterQuartileRange;
+
 QgsMergeAttributesDialog::QgsMergeAttributesDialog( const QgsFeatureList &features, QgsVectorLayer *vl, QgsMapCanvas *canvas, QWidget *parent, Qt::WindowFlags f )
     : QDialog( parent, f )
     , mFeatureList( features )
     , mVectorLayer( vl )
     , mMapCanvas( canvas )
-    , mSelectionRubberBand( 0 )
+    , mSelectionRubberBand( nullptr )
 {
   setupUi( this );
   createTableWidgetContents();
@@ -53,13 +71,16 @@ QgsMergeAttributesDialog::QgsMergeAttributesDialog( const QgsFeatureList &featur
 
   QSettings settings;
   restoreGeometry( settings.value( "/Windows/MergeAttributes/geometry" ).toByteArray() );
+
+  connect( mSkipAllButton, SIGNAL( clicked() ), this, SLOT( setAllToSkip() ) );
+  connect( mTableWidget, SIGNAL( cellChanged( int, int ) ), this, SLOT( tableWidgetCellChanged( int, int ) ) );
 }
 
 QgsMergeAttributesDialog::QgsMergeAttributesDialog()
     : QDialog()
-    , mVectorLayer( NULL )
-    , mMapCanvas( NULL )
-    , mSelectionRubberBand( NULL )
+    , mVectorLayer( nullptr )
+    , mMapCanvas( nullptr )
+    , mSelectionRubberBand( nullptr )
 {
   setupUi( this );
 
@@ -87,27 +108,31 @@ void QgsMergeAttributesDialog::createTableWidgetContents()
   mTableWidget->setRowCount( mFeatureList.size() + 2 );
 
   //create combo boxes and insert attribute names
-  const QgsFields& fields = mVectorLayer->pendingFields();
-  QSet<int> pkAttrList = mVectorLayer->pendingPkAttributesList().toSet();
+  mFields = mVectorLayer->fields();
+  QSet<int> pkAttrList = mVectorLayer->pkAttributeList().toSet();
 
   int col = 0;
-  for ( int idx = 0; idx < fields.count(); ++idx )
+  mHiddenAttributes.clear();
+  for ( int idx = 0; idx < mFields.count(); ++idx )
   {
-    if ( mVectorLayer->editorWidgetV2( idx ) == "Hidden" ||
-         mVectorLayer->editorWidgetV2( idx ) == "Immutable" )
+    if ( mVectorLayer->editFormConfig()->widgetType( idx ) == "Hidden" ||
+         mVectorLayer->editFormConfig()->widgetType( idx ) == "Immutable" )
+    {
+      mHiddenAttributes.insert( idx );
       continue;
+    }
 
     mTableWidget->setColumnCount( col + 1 );
 
-    QComboBox *cb = createMergeComboBox( fields[idx].type() );
+    QComboBox *cb = createMergeComboBox( mFields.at( idx ) .type() );
     if ( pkAttrList.contains( idx ) )
     {
       cb->setCurrentIndex( cb->findData( "skip" ) );
     }
     mTableWidget->setCellWidget( 0, col, cb );
 
-    QTableWidgetItem *item = new QTableWidgetItem( fields[idx].name() );
-    item->setData( Qt::UserRole, idx );
+    QTableWidgetItem *item = new QTableWidgetItem( mFields.at( idx ).name() );
+    item->setData( FieldIndex, idx );
     mTableWidget->setHorizontalHeaderItem( col++, item );
   }
 
@@ -119,16 +144,17 @@ void QgsMergeAttributesDialog::createTableWidgetContents()
   {
     verticalHeaderLabels << FID_TO_STRING( mFeatureList[i].id() );
 
-    const QgsAttributes &attrs = mFeatureList[i].attributes();
+    QgsAttributes attrs = mFeatureList.at( i ).attributes();
 
     for ( int j = 0; j < mTableWidget->columnCount(); j++ )
     {
-      int idx = mTableWidget->horizontalHeaderItem( j )->data( Qt::UserRole ).toInt();
+      int idx = mTableWidget->horizontalHeaderItem( j )->data( FieldIndex ).toInt();
 
-      QTableWidgetItem* attributeValItem = new QTableWidgetItem( attrs[idx].toString() );
+      QTableWidgetItem* attributeValItem = new QTableWidgetItem( attrs.at( idx ).toString() );
       attributeValItem->setFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable );
       mTableWidget->setItem( i + 1, j, attributeValItem );
-      mTableWidget->setCellWidget( i + 1, j, QgsAttributeEditor::createAttributeEditor( mTableWidget, NULL, mVectorLayer, idx, attrs[idx] ) );
+      QWidget* attributeWidget = QgsAttributeEditor::createAttributeEditor( mTableWidget, nullptr, mVectorLayer, idx, attrs.at( idx ) );
+      mTableWidget->setCellWidget( i + 1, j, attributeWidget );
     }
   }
 
@@ -150,23 +176,32 @@ QComboBox *QgsMergeAttributesDialog::createMergeComboBox( QVariant::Type columnT
   QgsFeatureList::const_iterator f_it = mFeatureList.constBegin();
   for ( ; f_it != mFeatureList.constEnd(); ++f_it )
   {
-    newComboBox->addItem( tr( "Feature %1" ).arg( f_it->id() ), QString::number( f_it->id() ) );
+    newComboBox->addItem( tr( "Feature %1" ).arg( f_it->id() ), QString( "f%1" ).arg( FID_TO_STRING( f_it->id() ) ) );
   }
 
-  if ( columnType == QVariant::Double || columnType == QVariant::Int )
+  switch ( columnType )
   {
-    newComboBox->addItem( tr( "Minimum" ), "minimum" );
-    newComboBox->addItem( tr( "Maximum" ), "maximum" );
-    newComboBox->addItem( tr( "Median" ), "median" );
-    newComboBox->addItem( tr( "Sum" ), "sum" );
-    newComboBox->addItem( tr( "Mean" ), "mean" );
-  }
-  else if ( columnType == QVariant::String )
-  {
-    newComboBox->addItem( tr( "Concatenation" ), "concat" );
+    case QVariant::Double:
+    case QVariant::Int:
+    case QVariant::LongLong:
+    {
+      Q_FOREACH ( QgsStatisticalSummary::Statistic stat, mDisplayStats )
+      {
+        newComboBox->addItem( QgsStatisticalSummary::displayName( stat ) , stat );
+      }
+      break;
+    }
+    case QVariant::String:
+      newComboBox->addItem( tr( "Concatenation" ), "concat" );
+      break;
+
+      //TODO - add date/time/datetime handling
+    default:
+      break;
   }
 
   newComboBox->addItem( tr( "Skip attribute" ), "skip" );
+  newComboBox->addItem( tr( "Manual value" ), "manual" );
 
   QObject::connect( newComboBox, SIGNAL( currentIndexChanged( const QString& ) ),
                     this, SLOT( comboValueChanged( const QString& ) ) );
@@ -204,10 +239,10 @@ void QgsMergeAttributesDialog::selectedRowChanged()
 {
   //find out selected row
   QList<QTableWidgetItem *> selectionList = mTableWidget->selectedItems();
-  if ( selectionList.size() < 1 )
+  if ( selectionList.isEmpty() )
   {
     delete mSelectionRubberBand;
-    mSelectionRubberBand = 0;
+    mSelectionRubberBand = nullptr;
     return;
   }
 
@@ -226,12 +261,12 @@ void QgsMergeAttributesDialog::selectedRowChanged()
   }
 
   bool conversionSuccess = false;
-  int featureIdToSelect = idItem->text().toInt( &conversionSuccess );
+  QgsFeatureId featureIdToSelect = idItem->text().toLongLong( &conversionSuccess );
   if ( !conversionSuccess )
   {
     //the merge result row was selected
     delete mSelectionRubberBand;
-    mSelectionRubberBand = 0;
+    mSelectionRubberBand = nullptr;
     return;
   }
   createRubberBandForFeature( featureIdToSelect );
@@ -248,27 +283,7 @@ void QgsMergeAttributesDialog::refreshMergedValue( int col )
   //evaluate behaviour (feature value or min / max / mean )
   QString mergeBehaviourString = comboBox->itemData( comboBox->currentIndex() ).toString();
   QVariant mergeResult; // result to show in the merge result field
-  if ( mergeBehaviourString == "minimum" )
-  {
-    mergeResult = minimumAttribute( col );
-  }
-  else if ( mergeBehaviourString == "maximum" )
-  {
-    mergeResult = maximumAttribute( col );
-  }
-  else if ( mergeBehaviourString == "mean" )
-  {
-    mergeResult = meanAttribute( col );
-  }
-  else if ( mergeBehaviourString == "median" )
-  {
-    mergeResult = medianAttribute( col );
-  }
-  else if ( mergeBehaviourString == "sum" )
-  {
-    mergeResult = sumAttribute( col );
-  }
-  else if ( mergeBehaviourString == "concat" )
+  if ( mergeBehaviourString == "concat" )
   {
     mergeResult = concatenationAttribute( col );
   }
@@ -276,179 +291,84 @@ void QgsMergeAttributesDialog::refreshMergedValue( int col )
   {
     mergeResult = tr( "Skipped" );
   }
-  else //an existing feature value
+  else if ( mergeBehaviourString == "manual" )
   {
-    int featureId = mergeBehaviourString.toInt();
+    return; //nothing to do
+  }
+  else if ( mergeBehaviourString.startsWith( 'f' ) )
+  {
+    //an existing feature value
+    QgsFeatureId featureId = STRING_TO_FID( mergeBehaviourString.mid( 1 ) );
     mergeResult = featureAttribute( featureId, col );
   }
+  else
+  {
+    //numerical statistic
+    QgsStatisticalSummary::Statistic stat = ( QgsStatisticalSummary::Statistic )( comboBox->itemData( comboBox->currentIndex() ).toInt() );
+    mergeResult = calcStatistic( col, stat );
+  }
 
   //insert string into table widget
   QTableWidgetItem* newTotalItem = new QTableWidgetItem();
   newTotalItem->setData( Qt::DisplayRole, mergeResult );
   newTotalItem->setFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable );
+
+  //block signals to prevent table widget switching combo box to "manual" entry
+  mTableWidget->blockSignals( true );
   mTableWidget->setItem( mTableWidget->rowCount() - 1, col, newTotalItem );
+  mTableWidget->blockSignals( false );
 }
 
-QVariant QgsMergeAttributesDialog::featureAttribute( int featureId, int col )
+QVariant QgsMergeAttributesDialog::featureAttribute( QgsFeatureId featureId, int col )
 {
-  int idx = mTableWidget->horizontalHeaderItem( col )->data( Qt::UserRole ).toInt();
+  int fieldIdx = mTableWidget->horizontalHeaderItem( col )->data( FieldIndex ).toInt();
 
   int i;
-  for ( i = 0; i < mFeatureList.size() && mFeatureList[i].id() != featureId; i++ )
+  for ( i = 0; i < mFeatureList.size() && mFeatureList.at( i ).id() != featureId; i++ )
     ;
 
   QVariant value;
   if ( i < mFeatureList.size() &&
-       QgsAttributeEditor::retrieveValue( mTableWidget->cellWidget( i + 1, col ), mVectorLayer, idx, value ) )
+       QgsAttributeEditor::retrieveValue( mTableWidget->cellWidget( i + 1, col ), mVectorLayer, fieldIdx, value ) )
   {
     return value;
   }
   else
   {
-    return QVariant( mVectorLayer->pendingFields()[col].type() );
+    return QVariant( mVectorLayer->fields().at( fieldIdx ).type() );
   }
 }
 
-QVariant QgsMergeAttributesDialog::minimumAttribute( int col )
+
+QVariant QgsMergeAttributesDialog::calcStatistic( int col, QgsStatisticalSummary::Statistic stat )
 {
-  double minimumValue = std::numeric_limits<double>::max();
-  double currentValue;
+  QgsStatisticalSummary summary( stat );
+
   bool conversion = false;
-  int numberOfConsideredFeatures = 0;
-
-  for ( int i = 0; i < mFeatureList.size(); ++i )
-  {
-    currentValue = mTableWidget->item( i + 1, col )->text().toDouble( &conversion );
-    if ( conversion )
-    {
-      if ( currentValue < minimumValue )
-      {
-        minimumValue = currentValue;
-        ++numberOfConsideredFeatures;
-      }
-    }
-  }
-
-  if ( numberOfConsideredFeatures < 1 )
-  {
-    return QVariant( mVectorLayer->pendingFields()[col].type() );
-  }
-
-  return QVariant( minimumValue );
-}
-
-QVariant QgsMergeAttributesDialog::maximumAttribute( int col )
-{
-  double maximumValue = -std::numeric_limits<double>::max();
-  double currentValue;
-  bool conversion = false;
-  int numberOfConsideredFeatures = 0;
-
-  for ( int i = 0; i < mFeatureList.size(); ++i )
-  {
-    currentValue = mTableWidget->item( i + 1, col )->text().toDouble( &conversion );
-    if ( conversion )
-    {
-      if ( currentValue > maximumValue )
-      {
-        maximumValue = currentValue;
-        ++numberOfConsideredFeatures;
-      }
-    }
-  }
-
-  if ( numberOfConsideredFeatures < 1 )
-  {
-    return QVariant( mVectorLayer->pendingFields()[col].type() );
-  }
-
-  return QVariant( maximumValue );
-}
-
-QVariant QgsMergeAttributesDialog::meanAttribute( int col )
-{
-  int numberOfConsideredFeatures = 0;
-  double currentValue;
-  double sum = 0;
-  bool conversion = false;
-
-  for ( int i = 0; i < mFeatureList.size(); ++i )
-  {
-    currentValue = mTableWidget->item( i + 1, col )->text().toDouble( &conversion );
-    if ( conversion )
-    {
-      sum += currentValue;
-      ++numberOfConsideredFeatures;
-    }
-  }
-
-  if ( numberOfConsideredFeatures < 1 )
-  {
-    return QVariant( mVectorLayer->pendingFields()[col].type() );
-  }
-
-  double mean = sum / numberOfConsideredFeatures;
-  return QVariant( mean );
-}
-
-QVariant QgsMergeAttributesDialog::medianAttribute( int col )
-{
-  //bring all values into a list and sort
-  QList<double> valueList;
-  double currentValue;
-  bool conversionSuccess;
-
-  for ( int i = 0; i < mFeatureList.size(); ++i )
-  {
-    currentValue = mTableWidget->item( i + 1, col )->text().toDouble( &conversionSuccess );
-    if ( !conversionSuccess )
-    {
-      continue;
-    }
-    valueList.push_back( currentValue );
-  }
-  qSort( valueList );
-
-  double medianValue;
-  int size = valueList.size();
-
-
-  if ( size < 1 )
-  {
-    return QVariant( mVectorLayer->pendingFields()[col].type() );
-  }
-
-  bool even = ( size % 2 ) < 1;
-  if ( even )
-  {
-    medianValue = ( valueList[size / 2 - 1] + valueList[size / 2] ) / 2;
-  }
-  else //odd
-  {
-    medianValue = valueList[( size + 1 ) / 2 - 1];
-  }
-  return QVariant( medianValue );
-}
-
-QVariant QgsMergeAttributesDialog::sumAttribute( int col )
-{
-  double sum = 0.0;
-  bool conversion = false;
-
+  QList<double> values;
   for ( int i = 0; i < mFeatureList.size(); ++i )
   {
     double currentValue = mTableWidget->item( i + 1, col )->text().toDouble( &conversion );
     if ( conversion )
     {
-      sum += currentValue;
+      values << currentValue;
     }
   }
-  return QVariant( sum );
+
+  if ( values.isEmpty() )
+  {
+    return QVariant( mVectorLayer->fields()[col].type() );
+  }
+
+  summary.calculate( values );
+
+  return QVariant::fromValue( summary.statistic( stat ) );
 }
 
 QVariant QgsMergeAttributesDialog::concatenationAttribute( int col )
 {
   QStringList concatString;
+  concatString.reserve( mFeatureList.size() );
   for ( int i = 0; i < mFeatureList.size(); ++i )
   {
     concatString << mTableWidget->item( i + 1, col )->text();
@@ -466,7 +386,7 @@ void QgsMergeAttributesDialog::on_mFromSelectedPushButton_clicked()
 
   //find out feature id of selected row
   QList<QTableWidgetItem *> selectionList = mTableWidget->selectedItems();
-  if ( selectionList.size() < 1 )
+  if ( selectionList.isEmpty() )
   {
     return;
   }
@@ -481,13 +401,13 @@ void QgsMergeAttributesDialog::on_mFromSelectedPushButton_clicked()
   }
 
   bool conversionSuccess;
-  int featureId = selectedHeaderItem->text().toInt( &conversionSuccess );
+  QgsFeatureId featureId = selectedHeaderItem->text().toLongLong( &conversionSuccess );
   if ( !conversionSuccess )
   {
     return;
   }
 
-  QSet<int> pkAttributes = mVectorLayer->pendingPkAttributesList().toSet();
+  QSet<int> pkAttributes = mVectorLayer->pkAttributeList().toSet();
   for ( int i = 0; i < mTableWidget->columnCount(); ++i )
   {
     if ( pkAttributes.contains( i ) )
@@ -497,7 +417,7 @@ void QgsMergeAttributesDialog::on_mFromSelectedPushButton_clicked()
     QComboBox* currentComboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, i ) );
     if ( currentComboBox )
     {
-      currentComboBox->setCurrentIndex( currentComboBox->findData( QString::number( featureId ) ) );
+      currentComboBox->setCurrentIndex( currentComboBox->findData( QString( "f%1" ).arg( FID_TO_STRING( featureId ) ) ) );
     }
   }
 }
@@ -511,7 +431,7 @@ void QgsMergeAttributesDialog::on_mRemoveFeatureFromSelectionButton_clicked()
 
   //find out feature id of selected row
   QList<QTableWidgetItem *> selectionList = mTableWidget->selectedItems();
-  if ( selectionList.size() < 1 )
+  if ( selectionList.isEmpty() )
   {
     return;
   }
@@ -526,7 +446,7 @@ void QgsMergeAttributesDialog::on_mRemoveFeatureFromSelectionButton_clicked()
   }
 
   bool conversionSuccess;
-  int featureId = selectedHeaderItem->text().toInt( &conversionSuccess );
+  QgsFeatureId featureId = selectedHeaderItem->text().toLongLong( &conversionSuccess );
   if ( !conversionSuccess )
   {
     selectedRowChanged();
@@ -550,7 +470,7 @@ void QgsMergeAttributesDialog::on_mRemoveFeatureFromSelectionButton_clicked()
       continue;
 
     currentComboBox->blockSignals( true );
-    currentComboBox->removeItem( currentComboBox->findData( QString::number( featureId ) ) );
+    currentComboBox->removeItem( currentComboBox->findData( QString( "f%1" ).arg( FID_TO_STRING( featureId ) ) ) );
     currentComboBox->blockSignals( false );
   }
 
@@ -567,15 +487,32 @@ void QgsMergeAttributesDialog::on_mRemoveFeatureFromSelectionButton_clicked()
   }
 }
 
-void QgsMergeAttributesDialog::createRubberBandForFeature( int featureId )
+void QgsMergeAttributesDialog::tableWidgetCellChanged( int row, int column )
+{
+  if ( row < mTableWidget->rowCount() - 1 )
+  {
+    //only looking for edits in the final row
+    return;
+  }
+
+  QComboBox* currentComboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, column ) );
+  if ( currentComboBox )
+  {
+    currentComboBox->blockSignals( true );
+    currentComboBox->setCurrentIndex( currentComboBox->findData( "manual" ) );
+    currentComboBox->blockSignals( false );
+  }
+}
+
+void QgsMergeAttributesDialog::createRubberBandForFeature( QgsFeatureId featureId )
 {
   //create rubber band to highlight the feature
   delete mSelectionRubberBand;
-  mSelectionRubberBand = new QgsRubberBand( mMapCanvas, mVectorLayer->geometryType() == QGis::Polygon );
+  mSelectionRubberBand = new QgsRubberBand( mMapCanvas, mVectorLayer->geometryType() );
   mSelectionRubberBand->setColor( QColor( 255, 0, 0, 65 ) );
   QgsFeature featureToSelect;
   mVectorLayer->getFeatures( QgsFeatureRequest().setFilterFid( featureId ).setSubsetOfAttributes( QgsAttributeList() ) ).nextFeature( featureToSelect );
-  mSelectionRubberBand->setToGeometry( featureToSelect.geometry(), mVectorLayer );
+  mSelectionRubberBand->setToGeometry( featureToSelect.constGeometry(), mVectorLayer );
 }
 
 QgsAttributes QgsMergeAttributesDialog::mergedAttributes() const
@@ -585,30 +522,40 @@ QgsAttributes QgsMergeAttributesDialog::mergedAttributes() const
     return QgsAttributes();
   }
 
-  QgsAttributes results( mTableWidget->columnCount() );
-  for ( int i = 0; i < mTableWidget->columnCount(); i++ )
+  int widgetIndex = 0;
+  QgsAttributes results( mFields.count() );
+  for ( int fieldIdx = 0; fieldIdx < mFields.count(); ++fieldIdx )
   {
-    int idx = mTableWidget->horizontalHeaderItem( i )->data( Qt::UserRole ).toInt();
+    if ( mHiddenAttributes.contains( fieldIdx ) )
+    {
+      //hidden attribute, set to default value
+      if ( mVectorLayer->dataProvider() )
+        results[fieldIdx] = mVectorLayer->dataProvider()->defaultValue( fieldIdx );
+      else
+        results[fieldIdx] = QVariant();
+      continue;
+    }
 
-    QComboBox *comboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, i ) );
+    QComboBox *comboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, widgetIndex ) );
     if ( !comboBox )
       continue;
 
-    QTableWidgetItem *currentItem = mTableWidget->item( mFeatureList.size() + 1, i );
+    QTableWidgetItem *currentItem = mTableWidget->item( mFeatureList.size() + 1, widgetIndex );
     if ( !currentItem )
       continue;
 
-    if ( idx >= results.count() )
-      results.resize( idx + 1 ); // make sure the results vector is long enough (maybe not necessary)
+    if ( fieldIdx >= results.count() )
+      results.resize( fieldIdx + 1 ); // make sure the results vector is long enough (maybe not necessary)
 
     if ( comboBox->itemData( comboBox->currentIndex() ).toString() != "skip" )
     {
-      results[idx] = currentItem->data( Qt::DisplayRole );
+      results[fieldIdx] = currentItem->data( Qt::DisplayRole );
     }
     else if ( mVectorLayer->dataProvider() )
     {
-      results[idx] = mVectorLayer->dataProvider()->defaultValue( idx );
+      results[fieldIdx] = mVectorLayer->dataProvider()->defaultValue( fieldIdx );
     }
+    widgetIndex++;
   }
 
   return results;
@@ -617,9 +564,16 @@ QgsAttributes QgsMergeAttributesDialog::mergedAttributes() const
 QSet<int> QgsMergeAttributesDialog::skippedAttributeIndexes() const
 {
   QSet<int> skipped;
-  for ( int i = 0; i < mTableWidget->columnCount(); ++i )
+  int widgetIndex = 0;
+  for ( int i = 0; i < mFields.count(); ++i )
   {
-    QComboBox *comboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, i ) );
+    if ( mHiddenAttributes.contains( i ) )
+    {
+      skipped << i;
+      continue;
+    }
+
+    QComboBox *comboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, widgetIndex ) );
     if ( !comboBox )
     {
       //something went wrong, better skip this attribute
@@ -631,7 +585,20 @@ QSet<int> QgsMergeAttributesDialog::skippedAttributeIndexes() const
     {
       skipped << i;
     }
+    widgetIndex++;
   }
 
   return skipped;
 }
+
+void QgsMergeAttributesDialog::setAllToSkip()
+{
+  for ( int i = 0; i < mTableWidget->columnCount(); ++i )
+  {
+    QComboBox* currentComboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, i ) );
+    if ( currentComboBox )
+    {
+      currentComboBox->setCurrentIndex( currentComboBox->findData( "skip" ) );
+    }
+  }
+}
