@@ -41,10 +41,10 @@ typedef QString category_t();
 typedef int type_t();
 
 
-QgsPluginRegistry *QgsPluginRegistry::_instance = 0;
+QgsPluginRegistry *QgsPluginRegistry::_instance = nullptr;
 QgsPluginRegistry *QgsPluginRegistry::instance()
 {
-  if ( _instance == 0 )
+  if ( !_instance )
   {
     _instance = new QgsPluginRegistry();
   }
@@ -52,7 +52,7 @@ QgsPluginRegistry *QgsPluginRegistry::instance()
 }
 
 QgsPluginRegistry::QgsPluginRegistry()
-    : mPythonUtils( NULL ), mQgisInterface( NULL )
+    : mPythonUtils( nullptr ), mQgisInterface( nullptr )
 {
 // constructor does nothing
 }
@@ -67,7 +67,7 @@ void QgsPluginRegistry::setPythonUtils( QgsPythonUtils* pythonUtils )
   mPythonUtils = pythonUtils;
 }
 
-bool QgsPluginRegistry::isLoaded( QString key ) const
+bool QgsPluginRegistry::isLoaded( const QString& key ) const
 {
   QMap<QString, QgsPluginMetadata>::const_iterator it = mPlugins.find( key );
   if ( it != mPlugins.end() ) // found a c++ plugin?
@@ -81,10 +81,10 @@ bool QgsPluginRegistry::isLoaded( QString key ) const
   return false;
 }
 
-QString QgsPluginRegistry::library( QString key )
+QString QgsPluginRegistry::library( const QString& key )
 {
-  QMap<QString, QgsPluginMetadata>::iterator it = mPlugins.find( key );
-  if ( it != mPlugins.end() )
+  QMap<QString, QgsPluginMetadata>::const_iterator it = mPlugins.constFind( key );
+  if ( it != mPlugins.constEnd() )
     return it->library();
 
   if ( mPythonUtils && mPythonUtils->isEnabled() )
@@ -96,18 +96,18 @@ QString QgsPluginRegistry::library( QString key )
   return QString();
 }
 
-QgisPlugin *QgsPluginRegistry::plugin( QString key )
+QgisPlugin *QgsPluginRegistry::plugin( const QString& key )
 {
   QMap<QString, QgsPluginMetadata>::iterator it = mPlugins.find( key );
   if ( it == mPlugins.end() )
-    return NULL;
+    return nullptr;
 
   // note: not used by python plugins
 
   return it->plugin();
 }
 
-bool QgsPluginRegistry::isPythonPlugin( QString key ) const
+bool QgsPluginRegistry::isPythonPlugin( const QString& key ) const
 {
   if ( mPythonUtils && mPythonUtils->isEnabled() )
   {
@@ -117,7 +117,7 @@ bool QgsPluginRegistry::isPythonPlugin( QString key ) const
   return false;
 }
 
-void QgsPluginRegistry::addPlugin( QString key, QgsPluginMetadata metadata )
+void QgsPluginRegistry::addPlugin( const QString& key, const QgsPluginMetadata& metadata )
 {
   mPlugins.insert( key, metadata );
 }
@@ -125,28 +125,29 @@ void QgsPluginRegistry::addPlugin( QString key, QgsPluginMetadata metadata )
 void QgsPluginRegistry::dump()
 {
   QgsDebugMsg( "PLUGINS IN REGISTRY: key -> (name, library)" );
-  for ( QMap<QString, QgsPluginMetadata>::iterator it = mPlugins.begin();
-        it != mPlugins.end();
+  for ( QMap<QString, QgsPluginMetadata>::const_iterator it = mPlugins.constBegin();
+        it != mPlugins.constEnd();
         ++it )
   {
     QgsDebugMsg( QString( "PLUGIN: %1 -> (%2, %3)" )
-                 .arg( it.key() )
-                 .arg( it->name() )
-                 .arg( it->library() ) );
+                 .arg( it.key(),
+                       it->name(),
+                       it->library() ) );
   }
 
   if ( mPythonUtils && mPythonUtils->isEnabled() )
   {
     QgsDebugMsg( "PYTHON PLUGINS IN REGISTRY:" );
-    foreach ( QString pluginName, mPythonUtils->listActivePlugins() )
+    Q_FOREACH ( const QString& pluginName, mPythonUtils->listActivePlugins() )
     {
+      Q_UNUSED( pluginName );
       QgsDebugMsg( pluginName );
     }
   }
 }
 
 
-void QgsPluginRegistry::removePlugin( QString key )
+void QgsPluginRegistry::removePlugin( const QString& key )
 {
   QgsDebugMsg( "removing plugin: " + key );
   QMap<QString, QgsPluginMetadata>::iterator it = mPlugins.find( key );
@@ -176,7 +177,7 @@ void QgsPluginRegistry::unloadAll()
 
   if ( mPythonUtils && mPythonUtils->isEnabled() )
   {
-    foreach ( QString pluginName, mPythonUtils->listActivePlugins() )
+    Q_FOREACH ( const QString& pluginName, mPythonUtils->listActivePlugins() )
     {
       mPythonUtils->unloadPlugin( pluginName );
     }
@@ -184,7 +185,7 @@ void QgsPluginRegistry::unloadAll()
 }
 
 
-bool QgsPluginRegistry::checkQgisVersion( QString minVersion, QString maxVersion ) const
+bool QgsPluginRegistry::checkQgisVersion( const QString& minVersion, const QString& maxVersion ) const
 {
   // Parse qgisMinVersion. Must be in form x.y.z or just x.y
   QStringList minVersionParts = minVersion.split( '.' );
@@ -237,7 +238,7 @@ bool QgsPluginRegistry::checkQgisVersion( QString minVersion, QString maxVersion
   // our qgis version - cut release name after version number
   QString qgisVersion = QString( QGis::QGIS_VERSION ).section( '-', 0, 0 );
 
-  QStringList qgisVersionParts = qgisVersion.split( "." );
+  QStringList qgisVersionParts = qgisVersion.split( '.' );
 
   int qgisMajor = qgisVersionParts.at( 0 ).toInt();
   int qgisMinor = qgisVersionParts.at( 1 ).toInt();
@@ -259,7 +260,7 @@ bool QgsPluginRegistry::checkQgisVersion( QString minVersion, QString maxVersion
 }
 
 
-void QgsPluginRegistry::loadPythonPlugin( QString packageName )
+void QgsPluginRegistry::loadPythonPlugin( const QString& packageName )
 {
   if ( !mPythonUtils || !mPythonUtils->isEnabled() )
   {
@@ -290,12 +291,14 @@ void QgsPluginRegistry::loadPythonPlugin( QString packageName )
 
     // add to settings
     settings.setValue( "/PythonPlugins/" + packageName, true );
-    QgsMessageLog::logMessage( QObject::tr( "Loaded %1 (package: %2)" ).arg( pluginName ).arg( packageName ), QObject::tr( "Plugins" ), QgsMessageLog::INFO );
+    QgsMessageLog::logMessage( QObject::tr( "Loaded %1 (package: %2)" ).arg( pluginName, packageName ), QObject::tr( "Plugins" ), QgsMessageLog::INFO );
+
+    settings.remove( "/PythonPlugins/watchDog/" + packageName );
   }
 }
 
 
-void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
+void QgsPluginRegistry::loadCppPlugin( const QString& theFullPathName )
 {
   QSettings settings;
 
@@ -317,7 +320,7 @@ void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
   bool loaded = myLib.load();
   if ( !loaded )
   {
-    QgsMessageLog::logMessage( QObject::tr( "Failed to load %1 (Reason: %2)" ).arg( myLib.fileName() ).arg( myLib.errorString() ), QObject::tr( "Plugins" ) );
+    QgsMessageLog::logMessage( QObject::tr( "Failed to load %1 (Reason: %2)" ).arg( myLib.fileName(), myLib.errorString() ), QObject::tr( "Plugins" ) );
     return;
   }
 
@@ -343,7 +346,7 @@ void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
           addPlugin( baseName, QgsPluginMetadata( myLib.fileName(), pName(), pl ) );
           //add it to the qsettings file [ts]
           settings.setValue( "/Plugins/" + baseName, true );
-          QgsMessageLog::logMessage( QObject::tr( "Loaded %1 (Path: %2)" ).arg( pName() ).arg( myLib.fileName() ), QObject::tr( "Plugins" ), QgsMessageLog::INFO );
+          QgsMessageLog::logMessage( QObject::tr( "Loaded %1 (Path: %2)" ).arg( pName(), myLib.fileName() ), QObject::tr( "Plugins" ), QgsMessageLog::INFO );
 
           QObject *o = dynamic_cast<QObject *>( pl );
           if ( o )
@@ -351,7 +354,7 @@ void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
             QgsDebugMsg( QString( "plugin object name: %1" ).arg( o->objectName() ) );
             if ( o->objectName().isEmpty() )
             {
-#ifndef WIN32
+#ifndef Q_OS_WIN
               baseName = baseName.mid( 3 );
 #endif
               QgsDebugMsg( QString( "object name to %1" ).arg( baseName ) );
@@ -369,6 +372,8 @@ void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
               QgsDebugMsg( QString( "plugin parent already set" ) );
             }
           }
+
+          settings.remove( "/Plugins/watchDog/" + baseName );
         }
         else
         {
@@ -396,7 +401,7 @@ void QgsPluginRegistry::loadCppPlugin( QString theFullPathName )
 }
 
 
-void QgsPluginRegistry::unloadPythonPlugin( QString packageName )
+void QgsPluginRegistry::unloadPythonPlugin( const QString& packageName )
 {
   if ( !mPythonUtils || !mPythonUtils->isEnabled() )
   {
@@ -416,7 +421,7 @@ void QgsPluginRegistry::unloadPythonPlugin( QString packageName )
 }
 
 
-void QgsPluginRegistry::unloadCppPlugin( QString theFullPathName )
+void QgsPluginRegistry::unloadCppPlugin( const QString& theFullPathName )
 {
   QSettings settings;
   QString baseName = QFileInfo( theFullPathName ).baseName();
@@ -436,7 +441,7 @@ void QgsPluginRegistry::unloadCppPlugin( QString theFullPathName )
 
 
 //overloaded version of the next method that will load from multiple directories not just one
-void QgsPluginRegistry::restoreSessionPlugins( QStringList thePluginDirList )
+void QgsPluginRegistry::restoreSessionPlugins( const QStringList& thePluginDirList )
 {
   QStringListIterator myIterator( thePluginDirList );
   while ( myIterator.hasNext() )
@@ -445,11 +450,11 @@ void QgsPluginRegistry::restoreSessionPlugins( QStringList thePluginDirList )
   }
 }
 
-void QgsPluginRegistry::restoreSessionPlugins( QString thePluginDirString )
+void QgsPluginRegistry::restoreSessionPlugins( const QString& thePluginDirString )
 {
   QSettings mySettings;
 
-#if defined(WIN32) || defined(__CYGWIN__)
+#if defined(Q_OS_WIN) || defined(__CYGWIN__)
   QString pluginExt = "*.dll";
 #elif ANDROID
   QString pluginExt = "*plugin.so";
@@ -462,15 +467,22 @@ void QgsPluginRegistry::restoreSessionPlugins( QString thePluginDirString )
 
   for ( uint i = 0; i < myPluginDir.count(); i++ )
   {
-    QString myFullPath = thePluginDirString + "/" + myPluginDir[i];
+    QString myFullPath = thePluginDirString + '/' + myPluginDir[i];
     if ( checkCppPlugin( myFullPath ) )
     {
       // check if the plugin was active on last session
 
       QString baseName = QFileInfo( myFullPath ).baseName();
+      if ( mySettings.value( QString( "/Plugins/watchDog/%1" ).arg( baseName ) ).isValid() )
+      {
+        mQgisInterface->messageBar()->pushWarning( QObject::tr( "Plugin %1" ).arg( baseName ), QObject::tr( "The plugin will be disabled because it crashed QGIS during last startup. Please report an issue and re-enable the plugin when the problem has been solved." ) );
+        mySettings.setValue( "/Plugins/" + baseName, false );
+      }
       if ( mySettings.value( "/Plugins/" + baseName ).toBool() )
       {
+        mySettings.setValue( QString( "/Plugins/watchDog/%1" ).arg( baseName ), true );
         loadCppPlugin( myFullPath );
+        mySettings.remove( QString( "/Plugins/watchDog/%1" ).arg( baseName ) );
       }
     }
   }
@@ -489,25 +501,23 @@ void QgsPluginRegistry::restoreSessionPlugins( QString thePluginDirString )
     corePlugins << "MetaSearch";
 
     // make the required core plugins enabled by default:
-    for ( int i = 0; i < corePlugins.size(); i++ )
+    Q_FOREACH ( const QString& corePlugin, corePlugins )
     {
-      if ( !mySettings.contains( "/PythonPlugins/" + corePlugins[i] ) )
+      if ( !mySettings.contains( "/PythonPlugins/" + corePlugin ) )
       {
-        mySettings.setValue( "/PythonPlugins/" + corePlugins[i], true );
+        mySettings.setValue( "/PythonPlugins/" + corePlugin, true );
       }
     }
 
-    for ( int i = 0; i < pluginList.size(); i++ )
+    Q_FOREACH ( const QString& packageName, pluginList )
     {
-      QString packageName = pluginList[i];
-
       // TODO: apply better solution for #5879
       // start - temporary fix for issue #5879
       if ( QgsApplication::isRunningFromBuildDir() )
       {
         if ( corePlugins.contains( packageName ) )
         {
-          QgsApplication::setPkgDataPath( QString( "" ) );
+          QgsApplication::setPkgDataPath( QString() );
         }
         else
         {
@@ -516,14 +526,22 @@ void QgsPluginRegistry::restoreSessionPlugins( QString thePluginDirString )
       }
       // end - temporary fix for issue #5879, more below
 
-      if ( checkPythonPlugin( packageName ) )
-      {
-        // check if the plugin was active on last session
 
-        if ( mySettings.value( "/PythonPlugins/" + packageName ).toBool() )
+      if ( mySettings.value( "/PythonPlugins/watchDog/" + packageName ).isValid() )
+      {
+        mQgisInterface->messageBar()->pushWarning( QObject::tr( "Plugin %1" ).arg( packageName ), QObject::tr( "The plugin will be disabled because it crashed QGIS during last startup. Please report an issue and re-enable the plugin when the problem has been solved." ) );
+        mySettings.setValue( "/PythonPlugins/" + packageName, false );
+      }
+      // check if the plugin was active on last session
+      if ( mySettings.value( "/PythonPlugins/" + packageName ).toBool() )
+      {
+        mySettings.setValue( "/PythonPlugins/watchDog/" + packageName, true );
+        if ( checkPythonPlugin( packageName ) )
         {
           loadPythonPlugin( packageName );
         }
+        mySettings.remove( "/PythonPlugins/watchDog/" + packageName );
+
       }
     }
     // start - temporary fix for issue #5879, more above
@@ -538,13 +556,13 @@ void QgsPluginRegistry::restoreSessionPlugins( QString thePluginDirString )
 }
 
 
-bool QgsPluginRegistry::checkCppPlugin( QString pluginFullPath )
+bool QgsPluginRegistry::checkCppPlugin( const QString& pluginFullPath )
 {
   QLibrary myLib( pluginFullPath );
   bool loaded = myLib.load();
   if ( ! loaded )
   {
-    QgsMessageLog::logMessage( QObject::tr( "Failed to load %1 (Reason: %2)" ).arg( myLib.fileName() ).arg( myLib.errorString() ), QObject::tr( "Plugins" ) );
+    QgsMessageLog::logMessage( QObject::tr( "Failed to load %1 (Reason: %2)" ).arg( myLib.fileName(), myLib.errorString() ), QObject::tr( "Plugins" ) );
     return false;
   }
 
@@ -561,7 +579,7 @@ bool QgsPluginRegistry::checkCppPlugin( QString pluginFullPath )
 }
 
 
-bool QgsPluginRegistry::checkPythonPlugin( QString packageName )
+bool QgsPluginRegistry::checkPythonPlugin( const QString& packageName )
 {
   QString pluginName, description, /*category,*/ version;
 
@@ -583,7 +601,7 @@ bool QgsPluginRegistry::checkPythonPlugin( QString packageName )
   return true;
 }
 
-bool QgsPluginRegistry::isPythonPluginCompatible( QString packageName ) const
+bool QgsPluginRegistry::isPythonPluginCompatible( const QString& packageName ) const
 {
   QString minVersion = mPythonUtils->getPluginMetadata( packageName, "qgisMinimumVersion" );
   // try to read qgisMaximumVersion. Note checkQgisVersion can cope with "__error__" value.
