@@ -17,7 +17,7 @@
 
 #include <QSettings>
 
-QgsShortcutsManager* QgsShortcutsManager::mInstance = NULL;
+QgsShortcutsManager* QgsShortcutsManager::mInstance = nullptr;
 
 QgsShortcutsManager::QgsShortcutsManager( QObject *parent ) : QObject( parent )
 {
@@ -25,7 +25,7 @@ QgsShortcutsManager::QgsShortcutsManager( QObject *parent ) : QObject( parent )
 
 QgsShortcutsManager::~QgsShortcutsManager()
 {
-  mInstance = 0;
+  mInstance = nullptr;
 }
 
 QgsShortcutsManager* QgsShortcutsManager::instance( QObject *parent )
@@ -35,7 +35,7 @@ QgsShortcutsManager* QgsShortcutsManager::instance( QObject *parent )
   return mInstance;
 }
 
-bool QgsShortcutsManager::registerAction( QAction* action, QString defaultShortcut )
+bool QgsShortcutsManager::registerAction( QAction* action, const QString& defaultShortcut )
 {
   mActions.insert( action, defaultShortcut );
   connect( action, SIGNAL( destroyed() ), this, SLOT( actionDestroyed() ) );
@@ -47,8 +47,7 @@ bool QgsShortcutsManager::registerAction( QAction* action, QString defaultShortc
   QSettings settings;
   QString shortcut = settings.value( "/shortcuts/" + actionText, defaultShortcut ).toString();
 
-  if ( !shortcut.isEmpty() )
-    action->setShortcut( shortcut );
+  action->setShortcut( shortcut );
 
   return true;
 }
@@ -72,7 +71,7 @@ QString QgsShortcutsManager::actionDefaultShortcut( QAction* action )
   return mActions.value( action );
 }
 
-bool QgsShortcutsManager::setActionShortcut( QAction* action, QString shortcut )
+bool QgsShortcutsManager::setActionShortcut( QAction* action, const QString& shortcut )
 {
   action->setShortcut( shortcut );
 
@@ -85,34 +84,34 @@ bool QgsShortcutsManager::setActionShortcut( QAction* action, QString shortcut )
   return true;
 }
 
-QAction* QgsShortcutsManager::actionForShortcut( QKeySequence s )
+QAction* QgsShortcutsManager::actionForShortcut( const QKeySequence& s )
 {
   if ( s.isEmpty() )
-    return NULL;
+    return nullptr;
 
-  for ( ActionsHash::iterator it = mActions.begin(); it != mActions.end(); ++it )
+  for ( ActionsHash::const_iterator it = mActions.constBegin(); it != mActions.constEnd(); ++it )
   {
     if ( it.key()->shortcut() == s )
       return it.key();
   }
 
-  return NULL;
+  return nullptr;
 }
 
-QAction* QgsShortcutsManager::actionByName( QString name )
+QAction* QgsShortcutsManager::actionByName( const QString& name )
 {
-  for ( ActionsHash::iterator it = mActions.begin(); it != mActions.end(); ++it )
+  for ( ActionsHash::const_iterator it = mActions.constBegin(); it != mActions.constEnd(); ++it )
   {
     if ( it.key()->text() == name )
       return it.key();
   }
 
-  return NULL;
+  return nullptr;
 }
 
 void QgsShortcutsManager::registerAllChildrenActions( QObject* object )
 {
-  foreach ( QObject* child, object->children() )
+  Q_FOREACH ( QObject* child, object->children() )
   {
     if ( child->inherits( "QAction" ) )
     {
