@@ -30,13 +30,13 @@ class APP_EXPORT QgsShortcutsManager : public QObject
   public:
 
     //! return instance of the manager
-    static QgsShortcutsManager* instance( QObject *parent = NULL );
+    static QgsShortcutsManager* instance( QObject *parent = nullptr );
 
     //! register all actions which are children of the passed object
     void registerAllChildrenActions( QObject* object );
 
     //! add action to the manager so the shortcut can be changed in GUI
-    bool registerAction( QAction* action, QString defaultShortcut = QString() );
+    bool registerAction( QAction* action, const QString& defaultShortcut = QString() );
 
     //! remove action from the manager
     bool unregisterAction( QAction* action );
@@ -48,13 +48,13 @@ class APP_EXPORT QgsShortcutsManager : public QObject
     QString actionDefaultShortcut( QAction* action );
 
     //! modify action's shortcut
-    bool setActionShortcut( QAction* action, QString shortcut );
+    bool setActionShortcut( QAction* action, const QString& shortcut );
 
     //! return action which is associated for the shortcut, NULL if no action is associated
-    QAction* actionForShortcut( QKeySequence s );
+    QAction* actionForShortcut( const QKeySequence& s );
 
     // return action by it's name. NULL if nothing found
-    QAction* actionByName( QString name );
+    QAction* actionByName( const QString& name );
 
     ~QgsShortcutsManager();
 
