@@ -19,13 +19,12 @@
 #define QGSSNAPPINGDIALOG_H
 
 #include "qgsmaplayer.h"
+#include <QDockWidget>
 #include "ui_qgssnappingdialogbase.h"
 
-class QDockWidget;
-
 class QgsMapCanvas;
 
-/**A dialog to enter advanced editing properties, e.g. topological editing, snapping settings
+/** A dialog to enter advanced editing properties, e.g. topological editing, snapping settings
 for the individual layers*/
 class APP_EXPORT QgsSnappingDialog: public QDialog, private Ui::QgsSnappingDialogBase
 {
@@ -48,10 +47,10 @@ class APP_EXPORT QgsSnappingDialog: public QDialog, private Ui::QgsSnappingDialo
     //! add layer to tree
     void addLayer( QgsMapLayer* theMapLayer );
 
-    void addLayers( QList<QgsMapLayer * > layers );
+    void addLayers( const QList<QgsMapLayer*>& layers );
 
     //! layers removed
-    void layersWillBeRemoved( QStringList );
+    void layersWillBeRemoved( const QStringList& );
 
     void on_cbxEnableTopologicalEditingCheckBox_stateChanged( int );
 
@@ -64,9 +63,9 @@ class APP_EXPORT QgsSnappingDialog: public QDialog, private Ui::QgsSnappingDialo
     void emitProjectSnapSettingsChanged();
 
   protected:
-    /**Constructor
-    @param canvas pointer to the map canvas (for detecting which vector layers are loaded
-    */
+    /** Constructor
+     * @param canvas pointer to the map canvas (for detecting which vector layers are loaded
+     */
     //QgsSnappingDialog( QgsMapCanvas* canvas );
 
     /**
@@ -82,24 +81,36 @@ class APP_EXPORT QgsSnappingDialog: public QDialog, private Ui::QgsSnappingDialo
     void reload();
 
   private:
-    /**Default constructor forbidden*/
+    /** Default constructor forbidden*/
     QgsSnappingDialog();
 
-    /**Stores ids of layers where intersections of new polygons is considered. Is passed to / read from QgsAvoidIntersectionsDialog*/
+    /** Stores ids of layers where intersections of new polygons is considered. Is passed to / read from QgsAvoidIntersectionsDialog*/
     QSet<QString> mAvoidIntersectionsSettings;
 
-    /**Used to query the loaded layers*/
+    /** Used to query the loaded layers*/
     QgsMapCanvas* mMapCanvas;
 
     QDockWidget *mDock;
 
-    /**Set checkbox value based on project setting*/
+    /** Set checkbox value based on project setting*/
     void setTopologicalEditingState();
 
-    /**Set checkbox value based on project setting*/
+    /** Set checkbox value based on project setting*/
     void setIntersectionSnappingState();
 
     void setSnappingMode();
 };
 
+
+class QgsSnappingDock : public QDockWidget
+{
+    Q_OBJECT
+
+  public:
+    QgsSnappingDock( const QString & title, QWidget * parent = nullptr, Qt::WindowFlags flags = nullptr );
+
+    virtual void closeEvent( QCloseEvent *e ) override;
+
+};
+
 #endif
