@@ -21,13 +21,14 @@
 #include <QColorDialog>
 #include <QGraphicsScene>
 
-QgsTextAnnotationDialog::QgsTextAnnotationDialog( QgsTextAnnotationItem* item, QWidget * parent, Qt::WindowFlags f ): QDialog( parent, f ), mItem( item ), mTextDocument( 0 )
+QgsTextAnnotationDialog::QgsTextAnnotationDialog( QgsTextAnnotationItem* item, QWidget * parent, Qt::WindowFlags f ): QDialog( parent, f ), mItem( item ), mTextDocument( nullptr )
 {
   setupUi( this );
   mEmbeddedWidget = new QgsAnnotationWidget( mItem );
-  mEmbeddedWidget->show();
   mStackedWidget->addWidget( mEmbeddedWidget );
   mStackedWidget->setCurrentWidget( mEmbeddedWidget );
+  connect( mEmbeddedWidget, SIGNAL( backgroundColorChanged( QColor ) ), this, SLOT( backgroundColorChanged( QColor ) ) );
+  mTextEdit->setAttribute( Qt::WA_TranslucentBackground );
   if ( mItem )
   {
     mTextDocument = mItem->document();
@@ -57,6 +58,18 @@ QgsTextAnnotationDialog::~QgsTextAnnotationDialog()
   delete mTextDocument;
 }
 
+void QgsTextAnnotationDialog::showEvent( QShowEvent* )
+{
+  backgroundColorChanged( mItem ? mItem->frameBackgroundColor() : Qt::white );
+}
+
+void QgsTextAnnotationDialog::backgroundColorChanged( const QColor& color )
+{
+  QPalette p = mTextEdit->viewport()->palette();
+  p.setColor( QPalette::Base, color );
+  mTextEdit->viewport()->setPalette( p );
+}
+
 void QgsTextAnnotationDialog::applyTextToItem()
 {
   if ( mItem && mTextDocument )
@@ -139,6 +152,6 @@ void QgsTextAnnotationDialog::deleteItem()
     scene->removeItem( mItem );
   }
   delete mItem;
-  mItem = 0;
+  mItem = nullptr;
 }
 
