@@ -30,6 +30,7 @@
 #include "qgsmapcanvas.h"
 #include "qgscontexthelp.h"
 #include "qgsexpressionbuilderdialog.h"
+#include "qgsmaplayerstylemanager.h"
 
 class QgsMapLayer;
 
@@ -37,9 +38,10 @@ class QgsAttributeActionDialog;
 class QgsApplyDialog;
 class QgsLabelDialog;
 class QgsVectorLayer;
-class QgsLabelingGui;
+class QgsLabelingWidget;
 class QgsDiagramProperties;
 class QgsFieldsProperties;
+class QgsRendererV2PropertiesDialog;
 
 class APP_EXPORT QgsVectorLayerProperties : public QgsOptionsDialogBase, private Ui::QgsVectorLayerPropertiesBase
 {
@@ -53,21 +55,21 @@ class APP_EXPORT QgsVectorLayerProperties : public QgsOptionsDialogBase, private
       DB,
     };
 
-    QgsVectorLayerProperties( QgsVectorLayer *lyr = 0, QWidget *parent = 0, Qt::WindowFlags fl = QgisGui::ModalDialogFlags );
+    QgsVectorLayerProperties( QgsVectorLayer *lyr = nullptr, QWidget *parent = nullptr, Qt::WindowFlags fl = QgisGui::ModalDialogFlags );
     ~QgsVectorLayerProperties();
-    /**Returns the display name entered in the dialog*/
+    /** Returns the display name entered in the dialog*/
     QString displayName();
     void setRendererDirty( bool ) {}
 
-    /**Sets the attribute that is used in the Identify Results dialog box*/
-    void setDisplayField( QString name );
+    /** Sets the attribute that is used in the Identify Results dialog box*/
+    void setDisplayField( const QString& name );
 
-    /**Adds an attribute to the table (but does not commit it yet)
+    /** Adds an attribute to the table (but does not commit it yet)
     @param field the field to add
     @return false in case of a name conflict, true in case of success */
     bool addAttribute( const QgsField &field );
 
-    /**Deletes an attribute (but does not commit it)
+    /** Deletes an attribute (but does not commit it)
       @param name attribute name
       @return false in case of a non-existing attribute.*/
     bool deleteAttribute( int attr );
@@ -103,7 +105,7 @@ class APP_EXPORT QgsVectorLayerProperties : public QgsOptionsDialogBase, private
 
     void on_pbnQueryBuilder_clicked();
     void on_pbnIndex_clicked();
-    void on_mCrsSelector_crsChanged( QgsCoordinateReferenceSystem crs );
+    void on_mCrsSelector_crsChanged( const QgsCoordinateReferenceSystem& crs );
     void loadDefaultStyle_clicked();
     void saveDefaultStyle_clicked();
     void loadStyle_clicked();
@@ -115,35 +117,45 @@ class APP_EXPORT QgsVectorLayerProperties : public QgsOptionsDialogBase, private
     void enableLabelOptions( bool theFlag );
 
     void on_mButtonAddJoin_clicked();
+    void on_mButtonEditJoin_clicked();
     void on_mButtonRemoveJoin_clicked();
 
     void on_mSimplifyDrawingGroupBox_toggled( bool checked );
 
   signals:
 
-    /** emitted when changes to layer were saved to update legend */
-    void refreshLegend( QString layerID, bool expandItem );
-    void refreshLegend( QString layerID );
+    /** Emitted when changes to layer were saved to update legend */
+    void refreshLegend( const QString& layerID, bool expandItem );
+    void refreshLegend( const QString& layerID );
 
     void toggleEditing( QgsMapLayer * );
 
   private slots:
-    /** toggle editing of layer */
+    /** Toggle editing of layer */
     void toggleEditing();
 
-    /** save the style based on selected format from the menu */
+    /** Save the style based on selected format from the menu */
     void saveStyleAsMenuTriggered( QAction * );
 
-    /** called when is possible to choice if load the style from filesystem or from db */
+    /** Called when is possible to choice if load the style from filesystem or from db */
     void loadStyleMenuTriggered( QAction * );
 
     void aboutToShowStyleMenu();
 
+    /** Updates the variable editor to reflect layer changes
+     */
+    void updateVariableEditor();
+
+    /**
+     * @brief updates the FieldsPropertiesDialog when syncing the layer properties
+     */
+    void updateFieldsPropertiesDialog();
+
   protected:
 
     void saveStyleAs( StyleType styleType );
 
-    /** when provider supports, it will list all the styles relative the layer in a dialog */
+    /** When provider supports, it will list all the styles relative the layer in a dialog */
     void showListOfStylesFromDatabase();
 
     void updateSymbologyPage();
@@ -152,35 +164,41 @@ class APP_EXPORT QgsVectorLayerProperties : public QgsOptionsDialogBase, private
 
     bool mMetadataFilled;
 
+    QString mOriginalSubsetSQL;
+
     QMenu *mSaveAsMenu;
     QMenu *mLoadStyleMenu;
 
     QAction* mActionLoadStyle;
     QAction* mActionSaveStyleAs;
 
-    /**Renderer dialog which is shown*/
-    QDialog* mRendererDialog;
-    /**Labeling dialog. If apply is pressed, options are applied to vector's QgsLabel */
-    QgsLabelingGui* labelingDialog;
-    /**Label dialog. If apply is pressed, options are applied to vector's QgsLabel */
+    /** Renderer dialog which is shown*/
+    QgsRendererV2PropertiesDialog* mRendererDialog;
+    /** Labeling dialog. If apply is pressed, options are applied to vector's QgsLabel */
+    QgsLabelingWidget* labelingDialog;
+    /** Label dialog. If apply is pressed, options are applied to vector's QgsLabel */
     QgsLabelDialog* labelDialog;
-    /**Actions dialog. If apply is pressed, the actions are stored for later use */
+    /** Actions dialog. If apply is pressed, the actions are stored for later use */
     QgsAttributeActionDialog* actionDialog;
-    /**Diagram dialog. If apply is pressed, options are applied to vector's diagrams*/
+    /** Diagram dialog. If apply is pressed, options are applied to vector's diagrams*/
     QgsDiagramProperties* diagramPropertiesDialog;
-    /**Fields dialog. If apply is pressed, options are applied to vector's diagrams*/
+    /** Fields dialog. If apply is pressed, options are applied to vector's diagrams*/
     QgsFieldsProperties* mFieldsPropertiesDialog;
 
     //! List of joins of a layer at the time of creation of the dialog. Used to return joins to previous state if dialog is cancelled
     QList< QgsVectorJoinInfo > mOldJoins;
 
+    /** Previous layer style. Used to reset style to previous state if new style
+     * was loaded but dialog is cancelled */
+    QgsMapLayerStyle mOldStyle;
+
     void initDiagramTab();
 
-    /**Buffer pixmap which takes the picture of renderers before they are assigned to the vector layer*/
+    /** Buffer pixmap which takes the picture of renderers before they are assigned to the vector layer*/
     //QPixmap bufferPixmap;
 
-    /**Adds a new join to mJoinTreeWidget*/
-    void addJoinToTreeWidget( const QgsVectorJoinInfo& join );
+    /** Adds a new join to mJoinTreeWidget*/
+    void addJoinToTreeWidget( const QgsVectorJoinInfo& join , const int insertIndex = -1 );
 };
 
 inline QString QgsVectorLayerProperties::displayName()
