@@ -138,7 +138,7 @@ class ASResource
 		static const string AS_ASM;
 		static const string AS_FOREACH, AS_LOCK, AS_UNSAFE, AS_FIXED, AS_Q_FOREACH;
 		static const string AS_GET, AS_SET, AS_ADD, AS_REMOVE;
-		static const string AS_CONST_CAST, AS_DYNAMIC_CAST, AS_REINTERPRET_CAST, AS_STATIC_CAST;
+		static const string AS_CONST_CAST, AS_DYNAMIC_CAST, AS_REINTERPRET_CAST, AS_STATIC_CAST, AS_QOBJECT_CAST;
 };
 
 class ASBeautifier : protected ASResource
