@@ -31,8 +31,8 @@ QgsAddRemoveMultiFrameCommand::QgsAddRemoveMultiFrameCommand( State s, QgsCompos
 }
 
 QgsAddRemoveMultiFrameCommand::QgsAddRemoveMultiFrameCommand()
-    : mMultiFrame( 0 )
-    , mComposition( 0 )
+    : mMultiFrame( nullptr )
+    , mComposition( nullptr )
     , mState( Added )
     , mFirstRun( true )
 {
