@@ -24,29 +24,30 @@
 #include "qgsnetworkcontentfetcher.h"
 #include "qgsvectorlayer.h"
 #include "qgsproject.h"
+#include "qgsdistancearea.h"
+
+#include "qgswebpage.h"
+#include "qgswebframe.h"
 
 #include <QCoreApplication>
 #include <QPainter>
-#include <QWebFrame>
-#include <QWebPage>
 #include <QImage>
 #include <QNetworkReply>
 
 QgsComposerHtml::QgsComposerHtml( QgsComposition* c, bool createUndoCommands )
     : QgsComposerMultiFrame( c, createUndoCommands )
     , mContentMode( QgsComposerHtml::Url )
-    , mWebPage( 0 )
+    , mWebPage( nullptr )
     , mLoaded( false )
     , mHtmlUnitsToMM( 1.0 )
-    , mRenderedPage( 0 )
+    , mRenderedPage( nullptr )
     , mEvaluateExpressions( true )
     , mUseSmartBreaks( true )
     , mMaxBreakDistance( 10 )
-    , mExpressionFeature( 0 )
-    , mExpressionLayer( 0 )
-    , mDistanceArea( 0 )
+    , mExpressionLayer( nullptr )
+    , mDistanceArea( nullptr )
     , mEnableUserStylesheet( false )
-    , mFetcher( 0 )
+    , mFetcher( nullptr )
 {
   mDistanceArea = new QgsDistanceArea();
   mHtmlUnitsToMM = htmlUnitsToMM();
@@ -73,7 +74,7 @@ QgsComposerHtml::QgsComposerHtml( QgsComposition* c, bool createUndoCommands )
   {
     //a html item added while atlas preview is enabled needs to have the expression context set,
     //otherwise fields in the html aren't correctly evaluated until atlas preview feature changes (#9457)
-    setExpressionContext( mComposition->atlasComposition().currentFeature(), mComposition->atlasComposition().coverageLayer() );
+    setExpressionContext( mComposition->atlasComposition().feature(), mComposition->atlasComposition().coverageLayer() );
   }
 
   //connect to atlas feature changes
@@ -85,27 +86,6 @@ QgsComposerHtml::QgsComposerHtml( QgsComposition* c, bool createUndoCommands )
 
 }
 
-QgsComposerHtml::QgsComposerHtml()
-    : QgsComposerMultiFrame( 0, false )
-    , mContentMode( QgsComposerHtml::Url )
-    , mWebPage( 0 )
-    , mLoaded( false )
-    , mHtmlUnitsToMM( 1.0 )
-    , mRenderedPage( 0 )
-    , mEvaluateExpressions( true )
-    , mUseSmartBreaks( true )
-    , mMaxBreakDistance( 10 )
-    , mExpressionFeature( 0 )
-    , mExpressionLayer( 0 )
-    , mDistanceArea( 0 )
-    , mEnableUserStylesheet( false )
-    , mFetcher( 0 )
-{
-  mDistanceArea = new QgsDistanceArea();
-  mFetcher = new QgsNetworkContentFetcher();
-  connect( mFetcher, SIGNAL( finished() ), this, SLOT( frameLoaded() ) );
-}
-
 QgsComposerHtml::~QgsComposerHtml()
 {
   delete mDistanceArea;
@@ -126,7 +106,7 @@ void QgsComposerHtml::setUrl( const QUrl& url )
   emit changed();
 }
 
-void QgsComposerHtml::setHtml( const QString html )
+void QgsComposerHtml::setHtml( const QString& html )
 {
   mHtml = html;
   //TODO - this signal should be emitted, but without changing the signal which sets the html
@@ -142,13 +122,21 @@ void QgsComposerHtml::setEvaluateExpressions( bool evaluateExpressions )
   emit changed();
 }
 
-void QgsComposerHtml::loadHtml( const bool useCache )
+void QgsComposerHtml::loadHtml( const bool useCache, const QgsExpressionContext *context )
 {
   if ( !mWebPage )
   {
     return;
   }
 
+  const QgsExpressionContext* evalContext = context;
+  QScopedPointer< QgsExpressionContext > scopedContext;
+  if ( !evalContext )
+  {
+    scopedContext.reset( createExpressionContext() );
+    evalContext = scopedContext.data();
+  }
+
   QString loadedHtml;
   switch ( mContentMode )
   {
@@ -159,9 +147,9 @@ void QgsComposerHtml::loadHtml( const bool useCache )
 
       //data defined url set?
       QVariant exprVal;
-      if ( dataDefinedEvaluate( QgsComposerObject::SourceUrl, exprVal ) )
+      if ( dataDefinedEvaluate( QgsComposerObject::SourceUrl, exprVal, *evalContext ) )
       {
-        currentUrl = exprVal.toString().trimmed();;
+        currentUrl = exprVal.toString().trimmed();
         QgsDebugMsg( QString( "exprVal Source Url:%1" ).arg( currentUrl ) );
       }
       if ( currentUrl.isEmpty() )
@@ -188,7 +176,7 @@ void QgsComposerHtml::loadHtml( const bool useCache )
   //evaluate expressions
   if ( mEvaluateExpressions )
   {
-    loadedHtml = QgsExpression::replaceExpressionText( loadedHtml, mExpressionFeature, mExpressionLayer, 0, mDistanceArea );
+    loadedHtml = QgsExpression::replaceExpressionText( loadedHtml, evalContext, nullptr, mDistanceArea );
   }
 
   mLoaded = false;
@@ -235,7 +223,7 @@ double QgsComposerHtml::maxFrameWidth() const
   QList<QgsComposerFrame*>::const_iterator frameIt = mFrameItems.constBegin();
   for ( ; frameIt != mFrameItems.constEnd(); ++frameIt )
   {
-    maxWidth = qMax( maxWidth, ( double )(( *frameIt )->boundingRect().width() ) );
+    maxWidth = qMax( maxWidth, static_cast< double >(( *frameIt )->boundingRect().width() ) );
   }
 
   return maxWidth;
@@ -282,7 +270,7 @@ void QgsComposerHtml::renderCachedImage()
   painter.end();
 }
 
-QString QgsComposerHtml::fetchHtml( QUrl url )
+QString QgsComposerHtml::fetchHtml( const QUrl& url )
 {
   //pause until HTML fetch
   mLoaded = false;
@@ -345,7 +333,7 @@ void QgsComposerHtml::addFrame( QgsComposerFrame* frame, bool recalcFrameSizes )
   }
 }
 
-bool candidateSort( const QPair<int, int> &c1, const QPair<int, int> &c2 )
+bool candidateSort( QPair<int, int> c1, QPair<int, int> c2 )
 {
   if ( c1.second < c2.second )
     return true;
@@ -452,7 +440,7 @@ void QgsComposerHtml::setMaxBreakDistance( double maxBreakDistance )
   emit changed();
 }
 
-void QgsComposerHtml::setUserStylesheet( const QString stylesheet )
+void QgsComposerHtml::setUserStylesheet( const QString& stylesheet )
 {
   mUserStylesheet = stylesheet;
   //TODO - this signal should be emitted, but without changing the signal which sets the css
@@ -473,13 +461,13 @@ void QgsComposerHtml::setUserStylesheetEnabled( const bool stylesheetEnabled )
 
 QString QgsComposerHtml::displayName() const
 {
-  return tr( "<html frame>" );
+  return tr( "<HTML frame>" );
 }
 
 bool QgsComposerHtml::writeXML( QDomElement& elem, QDomDocument & doc, bool ignoreFrames ) const
 {
   QDomElement htmlElem = doc.createElement( "ComposerHtml" );
-  htmlElem.setAttribute( "contentMode", QString::number(( int ) mContentMode ) );
+  htmlElem.setAttribute( "contentMode", QString::number( static_cast< int >( mContentMode ) ) );
   htmlElem.setAttribute( "url", mUrl.toString() );
   htmlElem.setAttribute( "html", mHtml );
   htmlElem.setAttribute( "evaluateExpressions", mEvaluateExpressions ? "true" : "false" );
@@ -507,7 +495,7 @@ bool QgsComposerHtml::readXML( const QDomElement& itemElem, const QDomDocument&
   }
 
   bool contentModeOK;
-  mContentMode = ( QgsComposerHtml::ContentMode )itemElem.attribute( "contentMode" ).toInt( &contentModeOK );
+  mContentMode = static_cast< QgsComposerHtml::ContentMode >( itemElem.attribute( "contentMode" ).toInt( &contentModeOK ) );
   if ( !contentModeOK )
   {
     mContentMode = QgsComposerHtml::Url;
@@ -532,7 +520,7 @@ bool QgsComposerHtml::readXML( const QDomElement& itemElem, const QDomDocument&
   return true;
 }
 
-void QgsComposerHtml::setExpressionContext( QgsFeature* feature, QgsVectorLayer* layer )
+void QgsComposerHtml::setExpressionContext( const QgsFeature &feature, QgsVectorLayer* layer )
 {
   mExpressionFeature = feature;
   mExpressionLayer = layer;
@@ -556,8 +544,8 @@ void QgsComposerHtml::setExpressionContext( QgsFeature* feature, QgsVectorLayer*
 
 void QgsComposerHtml::refreshExpressionContext()
 {
-  QgsVectorLayer * vl = 0;
-  QgsFeature* feature = 0;
+  QgsVectorLayer * vl = nullptr;
+  QgsFeature feature;
 
   if ( mComposition->atlasComposition().enabled() )
   {
@@ -565,19 +553,27 @@ void QgsComposerHtml::refreshExpressionContext()
   }
   if ( mComposition->atlasMode() != QgsComposition::AtlasOff )
   {
-    feature = mComposition->atlasComposition().currentFeature();
+    feature = mComposition->atlasComposition().feature();
   }
 
   setExpressionContext( feature, vl );
   loadHtml( true );
 }
 
-void QgsComposerHtml::refreshDataDefinedProperty( const QgsComposerObject::DataDefinedProperty property )
+void QgsComposerHtml::refreshDataDefinedProperty( const QgsComposerObject::DataDefinedProperty property, const QgsExpressionContext* context )
 {
+  const QgsExpressionContext* evalContext = context;
+  QScopedPointer< QgsExpressionContext > scopedContext;
+  if ( !evalContext )
+  {
+    scopedContext.reset( createExpressionContext() );
+    evalContext = scopedContext.data();
+  }
+
   //updates data defined properties and redraws item to match
   if ( property == QgsComposerObject::SourceUrl || property == QgsComposerObject::AllProperties )
   {
-    loadHtml( true );
+    loadHtml( true, evalContext );
   }
-  QgsComposerObject::refreshDataDefinedProperty( property );
+  QgsComposerObject::refreshDataDefinedProperty( property, context );
 }
