@@ -29,7 +29,7 @@ QgsComposerMultiFrame::QgsComposerMultiFrame( QgsComposition* c, bool createUndo
 }
 
 QgsComposerMultiFrame::QgsComposerMultiFrame()
-    : QgsComposerObject( 0 )
+    : QgsComposerObject( nullptr )
     , mResizeMode( UseExistingFrames )
     , mCreateUndoCommands( false )
     , mIsRecalculatingSize( false )
@@ -84,7 +84,7 @@ void QgsComposerMultiFrame::recalculateFrameSizes()
 
   double currentY = 0;
   double currentHeight = 0;
-  QgsComposerFrame* currentItem = 0;
+  QgsComposerFrame* currentItem = nullptr;
 
   for ( int i = 0; i < mFrameItems.size(); ++i )
   {
@@ -189,11 +189,10 @@ void QgsComposerMultiFrame::recalculateFrameRects()
     return;
   }
 
-  QList<QgsComposerFrame*>::iterator frameIt = mFrameItems.begin();
-  for ( ; frameIt != mFrameItems.end(); ++frameIt )
+  Q_FOREACH ( QgsComposerFrame* frame, mFrameItems )
   {
-    ( *frameIt )->setSceneRect( QRectF(( *frameIt )->scenePos().x(), ( *frameIt )->scenePos().y(),
-                                       ( *frameIt )->rect().width(), ( *frameIt )->rect().height() ) );
+    frame->setSceneRect( QRectF( frame->scenePos().x(), frame->scenePos().y(),
+                                 frame->rect().width(), frame->rect().height() ) );
   }
 }
 
@@ -201,7 +200,7 @@ QgsComposerFrame* QgsComposerMultiFrame::createNewFrame( QgsComposerFrame* curre
 {
   if ( !currentFrame )
   {
-    return 0;
+    return nullptr;
   }
 
   QgsComposerFrame* newFrame = new QgsComposerFrame( mComposition, this, pos.x(),
@@ -242,7 +241,7 @@ void QgsComposerMultiFrame::handleFrameRemoval( QgsComposerItem* item )
   }
 
   mFrameItems.removeAt( index );
-  if ( mFrameItems.size() > 0 )
+  if ( !mFrameItems.isEmpty() )
   {
     if ( resizeMode() != QgsComposerMultiFrame::RepeatOnEveryPage && !mIsRecalculatingSize )
     {
@@ -270,7 +269,7 @@ void QgsComposerMultiFrame::handlePageChange()
   //remove items beginning on non-existing pages
   for ( int i = mFrameItems.size() - 1; i >= 0; --i )
   {
-    QgsComposerFrame* frame = mFrameItems[i];
+    QgsComposerFrame* frame = mFrameItems.at( i );
     int page = frame->pos().y() / ( mComposition->paperHeight() + mComposition->spaceBetweenPages() );
     if ( page > ( mComposition->numPages() - 1 ) )
     {
@@ -303,7 +302,7 @@ void QgsComposerMultiFrame::removeFrame( int i, const bool removeEmptyPages )
     return;
   }
 
-  QgsComposerFrame* frameItem = mFrameItems[i];
+  QgsComposerFrame* frameItem = mFrameItems.at( i );
   if ( mComposition )
   {
     mIsRecalculatingSize = true;
@@ -322,10 +321,9 @@ void QgsComposerMultiFrame::removeFrame( int i, const bool removeEmptyPages )
 
 void QgsComposerMultiFrame::update()
 {
-  QList<QgsComposerFrame*>::iterator frameIt = mFrameItems.begin();
-  for ( ; frameIt != mFrameItems.end(); ++frameIt )
+  Q_FOREACH ( QgsComposerFrame* frame, mFrameItems )
   {
-    ( *frameIt )->update();
+    frame->update();
   }
 }
 
@@ -334,11 +332,10 @@ void QgsComposerMultiFrame::deleteFrames()
   ResizeMode bkResizeMode = mResizeMode;
   mResizeMode = UseExistingFrames;
   QObject::disconnect( mComposition, SIGNAL( itemRemoved( QgsComposerItem* ) ), this, SLOT( handleFrameRemoval( QgsComposerItem* ) ) );
-  QList<QgsComposerFrame*>::iterator frameIt = mFrameItems.begin();
-  for ( ; frameIt != mFrameItems.end(); ++frameIt )
+  Q_FOREACH ( QgsComposerFrame* frame, mFrameItems )
   {
-    mComposition->removeComposerItem( *frameIt, false );
-    delete *frameIt;
+    mComposition->removeComposerItem( frame, false );
+    delete frame;
   }
   QObject::connect( mComposition, SIGNAL( itemRemoved( QgsComposerItem* ) ), this, SLOT( handleFrameRemoval( QgsComposerItem* ) ) );
   mFrameItems.clear();
@@ -349,7 +346,7 @@ QgsComposerFrame* QgsComposerMultiFrame::frame( int i ) const
 {
   if ( i >= mFrameItems.size() )
   {
-    return 0;
+    return nullptr;
   }
   return mFrameItems.at( i );
 }
@@ -378,7 +375,7 @@ bool QgsComposerMultiFrame::_readXML( const QDomElement& itemElem, const QDomDoc
 {
   QgsComposerObject::readXML( itemElem, doc );
 
-  mResizeMode = ( ResizeMode )itemElem.attribute( "resizeMode", "0" ).toInt();
+  mResizeMode = static_cast< ResizeMode >( itemElem.attribute( "resizeMode", "0" ).toInt() );
   if ( !ignoreFrames )
   {
     QDomNodeList frameList = itemElem.elementsByTagName( "ComposerFrame" );
