@@ -33,7 +33,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
     Q_OBJECT
   public:
 
-    /*! Controls how pictures are scaled within the item's frame
+    /** Controls how pictures are scaled within the item's frame
      */
     enum ResizeMode
     {
@@ -44,7 +44,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
       FrameToImageSize /*!< sets size of frame to match original size of image without scaling */
     };
 
-    /*! Format of source image
+    /** Format of source image
      */
     enum Mode
     {
@@ -53,83 +53,90 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
       Unknown
     };
 
+    //! Method for syncing rotation to a map's North direction
+    enum NorthMode
+    {
+      GridNorth = 0, /*!< Align to grid north */
+      TrueNorth, /*!< Align to true north */
+    };
+
     QgsComposerPicture( QgsComposition *composition );
     ~QgsComposerPicture();
 
-    /** return correct graphics item type. */
+    /** Return correct graphics item type. */
     virtual int type() const override { return ComposerPicture; }
 
-    /**Reimplementation of QCanvasItem::paint*/
+    /** Reimplementation of QCanvasItem::paint*/
     void paint( QPainter* painter, const QStyleOptionGraphicsItem* itemStyle, QWidget* pWidget ) override;
 
-    /**Sets the source file of the image (may be svg or a raster format). Data defined
+    /** Sets the source file of the image (may be svg or a raster format). Data defined
      * picture source may override this value.
      * @param path full path to the source image
      * @see usePictureExpression
      * @see pictureFile
      * @deprecated use setPicturePath instead
-    */
+     */
     Q_DECL_DEPRECATED void setPictureFile( const QString& path );
 
-    /**Returns the path of the source image file. Data defined picture source may override
+    /** Returns the path of the source image file. Data defined picture source may override
      * this value.
      * @returns path to the source image
      * @see usePictureExpression
      * @see setPictureFile
      * @deprecated use picturePath instead
-    */
+     */
     Q_DECL_DEPRECATED QString pictureFile() const;
 
-    /**Sets the source path of the image (may be svg or a raster format). Data defined
+    /** Sets the source path of the image (may be svg or a raster format). Data defined
      * picture source may override this value. The path can either be a local path
      * or a remote (http) path.
      * @param path path for the source image
      * @see usePictureExpression
      * @see picturePath
      * @note added in QGIS 2.5
-    */
+     */
     void setPicturePath( const QString& path );
 
-    /**Returns the path of the source image. Data defined picture source may override
+    /** Returns the path of the source image. Data defined picture source may override
      * this value. The path can either be a local path or a remote (http) path.
      * @returns path for the source image
      * @see usePictureExpression
      * @see setPicturePath
      * @note added in QGIS 2.5
-    */
+     */
     QString picturePath() const;
 
-    /**Sets this items bound in scene coordinates such that 1 item size units
+    /** Sets this items bound in scene coordinates such that 1 item size units
      * corresponds to 1 scene size unit and resizes the svg symbol / image
-    */
+     */
     void setSceneRect( const QRectF& rectangle ) override;
 
-    /**Stores state in Dom element
+    /** Stores state in Dom element
      * @param elem is Dom element corresponding to 'Composer' tag
      * @param doc is Dom document
      */
     bool writeXML( QDomElement& elem, QDomDocument & doc ) const override;
 
-    /**Sets state from Dom document
+    /** Sets state from Dom document
      * @param itemElem is Dom node corresponding to item tag
      * @param doc is Dom document
      */
     bool readXML( const QDomElement& itemElem, const QDomDocument& doc ) override;
 
-    /**Returns the rotation used for drawing the picture within the composer item
+    /** Returns the rotation used for drawing the picture within the composer item
      * @deprecated Use pictureRotation() instead
      */
     Q_DECL_DEPRECATED double rotation() const { return mPictureRotation; }
 
-    /**Returns the rotation used for drawing the picture within the item's frame
+    /** Returns the rotation used for drawing the picture within the item's frame
      * @returns picture rotation in degrees
      * @note added in 2.2
      * @see setPictureRotation
      * @see rotationMap
-    */
+     */
     double pictureRotation() const { return mPictureRotation; }
 
-    /**Sets the map object for rotation (by id). A value of -1 disables the map
+    /** Sets the map object for rotation (by id). A value of -1 disables the map
      * rotation.  If this is set then the picture will be rotated by the same
      * amount as the specified map object. This is useful especially for
      * syncing north arrows with a map item.
@@ -139,7 +146,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     void setRotationMap( int composerMapId );
 
-    /**Returns the id of the rotation map.  A value of -1 means map rotation is
+    /** Returns the id of the rotation map.  A value of -1 means map rotation is
      * disabled.  If this is set then the picture is rotated by the same amount
      * as the specified map object.
      * @returns id of map object
@@ -148,14 +155,46 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     int rotationMap() const;
 
-    /**True if the picture rotation is matched to a map item.
+    /** True if the picture rotation is matched to a map item.
      * @returns true if rotation map is in use
      * @see rotationMap
      * @see setRotationMap
      */
     bool useRotationMap() const { return mRotationMap; }
 
-    /**Returns the resize mode used for drawing the picture within the composer
+    /**
+     * Returns the mode used to align the picture to a map's North.
+     * @see setNorthMode()
+     * @see northOffset()
+     * @note added in QGIS 2.18
+     */
+    NorthMode northMode() const { return mNorthMode; }
+
+    /**
+     * Sets the mode used to align the picture to a map's North.
+     * @see northMode()
+     * @see setNorthOffset()
+     * @note added in QGIS 2.18
+     */
+    void setNorthMode( NorthMode mode );
+
+    /**
+     * Returns the offset added to the picture's rotation from a map's North.
+     * @see setNorthOffset()
+     * @see northMode()
+     * @note added in QGIS 2.18
+     */
+    double northOffset() const { return mNorthOffset; }
+
+    /**
+     * Sets the offset added to the picture's rotation from a map's North.
+     * @see northOffset()
+     * @see setNorthMode()
+     * @note added in QGIS 2.18
+     */
+    void setNorthOffset( double offset );
+
+    /** Returns the resize mode used for drawing the picture within the composer
      * item's frame.
      * @returns resize mode of picture
      * @note added in 2.3
@@ -163,7 +202,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     ResizeMode resizeMode() const { return mResizeMode; }
 
-    /**Sets the picture's anchor point, which controls how it is placed
+    /** Sets the picture's anchor point, which controls how it is placed
      * within the picture item's frame.
      * @param anchor anchor point for picture
      * @note added in 2.3
@@ -171,7 +210,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     void setPictureAnchor( QgsComposerItem::ItemPositionMode anchor );
 
-    /**Returns the picture's current anchor, which controls how it is placed
+    /** Returns the picture's current anchor, which controls how it is placed
      * within the picture item's frame.
      * @returns anchor point for picture
      * @note added in 2.3
@@ -179,7 +218,58 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     ItemPositionMode pictureAnchor() const { return mPictureAnchor; }
 
-    /**Returns whether the picture item is using an expression for the image source.
+    /** Returns the fill color used for parameterized SVG files.
+     * @see setSvgFillColor()
+     * @see svgBorderColor()
+     * @note added in QGIS 2.14.1
+     */
+    QColor svgFillColor() const { return mSvgFillColor; }
+
+    /** Sets the fill color used for parameterized SVG files.
+     * @param color fill color.
+     * @note this setting only has an effect on parameterized SVG files, and is ignored for
+     * non-parameterized SVG files.
+     * @see svgFillColor()
+     * @see setSvgBorderColor()
+     * @note added in QGIS 2.14.1
+     */
+    void setSvgFillColor( const QColor& color );
+
+    /** Returns the border color used for parameterized SVG files.
+     * @see setSvgBorderColor()
+     * @see svgFillColor()
+     * @note added in QGIS 2.14.1
+     */
+    QColor svgBorderColor() const { return mSvgBorderColor; }
+
+    /** Sets the border color used for parameterized SVG files.
+     * @param color border color.
+     * @note this setting only has an effect on parameterized SVG files, and is ignored for
+     * non-parameterized SVG files.
+     * @see svgBorderlColor()
+     * @see setSvgFillColor()
+     * @note added in QGIS 2.14.1
+     */
+    void setSvgBorderColor( const QColor& color );
+
+    /** Returns the border width (in mm) used for parameterized SVG files.
+     * @see setSvgBorderWidth()
+     * @see svgBorderColor()
+     * @note added in QGIS 2.14.1
+     */
+    double svgBorderWidth() const { return mSvgBorderWidth; }
+
+    /** Sets the border width used for parameterized SVG files.
+     * @param width border width in mm
+     * @note this setting only has an effect on parameterized SVG files, and is ignored for
+     * non-parameterized SVG files.
+     * @see svgBorderWidth()
+     * @see setSvgBorderColor()
+     * @note added in QGIS 2.14.1
+     */
+    void setSvgBorderWidth( double width );
+
+    /** Returns whether the picture item is using an expression for the image source.
      * @returns true if the picture is using an expression for the source, false if
      * it is using a single static file path for the source.
      * @note added in 2.3
@@ -190,7 +280,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     Q_DECL_DEPRECATED bool usePictureExpression() const;
 
-    /**Returns the expression the item is using for the picture source. This is only
+    /** Returns the expression the item is using for the picture source. This is only
      * used if usePictureExpression() is true.
      * @returns expression for the picture item's image path
      * @note added in 2.3
@@ -200,38 +290,38 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     Q_DECL_DEPRECATED QString pictureExpression() const;
 
-    /**Calculates width and hight of the picture (in mm) such that it fits into the item frame with the given rotation
+    /** Calculates width and hight of the picture (in mm) such that it fits into the item frame with the given rotation
      * @deprecated Use bool QgsComposerItem::imageSizeConsideringRotation( double& width, double& height, double rotation )
      * instead
      */
     Q_DECL_DEPRECATED bool imageSizeConsideringRotation( double& width, double& height ) const;
 
-    /**Calculates corner point after rotation and scaling
+    /** Calculates corner point after rotation and scaling
      * @deprecated Use QgsComposerItem::cornerPointOnRotatedAndScaledRect( double& x, double& y, double width, double height, double rotation )
      * instead
      */
     Q_DECL_DEPRECATED bool cornerPointOnRotatedAndScaledRect( double& x, double& y, double width, double height ) const;
 
-    /**Calculates width / height of the bounding box of a rotated rectangle
+    /** Calculates width / height of the bounding box of a rotated rectangle
      * @deprecated Use QgsComposerItem::sizeChangedByRotation( double& width, double& height, double rotation )
      * instead
      */
     Q_DECL_DEPRECATED void sizeChangedByRotation( double& width, double& height );
 
-    /**Returns the current picture mode (image format).
+    /** Returns the current picture mode (image format).
      * @returns picture mode
      * @note added in 2.3
      */
     Mode mode() const { return mMode; }
 
   public slots:
-    /**Sets the picture rotation within the item bounds. This does not affect
+    /** Sets the picture rotation within the item bounds. This does not affect
      * the item rectangle, only the way the picture is drawn within the item.
      * @deprecated Use setPictureRotation( double rotation ) instead
      */
     virtual void setRotation( double r ) override;
 
-    /**Sets the picture rotation within the item bounds. This does not affect
+    /** Sets the picture rotation within the item bounds. This does not affect
      * the item's frame, only the way the picture is drawn within the item.
      * @param r rotation in degrees clockwise
      * @see pictureRotation
@@ -239,14 +329,14 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     virtual void setPictureRotation( double r );
 
-    /**Sets the resize mode used for drawing the picture within the item bounds.
+    /** Sets the resize mode used for drawing the picture within the item bounds.
      * @param mode ResizeMode to use for image file
      * @note added in 2.3
      * @see resizeMode
      */
     virtual void setResizeMode( ResizeMode mode );
 
-    /**Sets whether the picture should use an expression based image source path
+    /** Sets whether the picture should use an expression based image source path
      * @param useExpression set to true to use an expression based image source,
      * set to false to use a single image source path
      * @note added in 2.3
@@ -257,7 +347,7 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      */
     virtual void setUsePictureExpression( bool useExpression );
 
-    /**Sets an expression to use for the picture source. This expression is only
+    /** Sets an expression to use for the picture source. This expression is only
      * used if usePictureExpression() is true.
      * @param expression to use for picture path
      * @note added in 2.3
@@ -265,42 +355,43 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
      * @see pictureExpression
      * @deprecated use QgsComposerObject::dataDefinedProperty( QgsComposerObject::PictureSource ) instead
      */
-    virtual void setPictureExpression( QString expression );
+    virtual void setPictureExpression( const QString& expression );
 
-    /**Recalculates the source image (if using an expression for picture's source)
+    /** Recalculates the source image (if using an expression for picture's source)
      * and reloads and redraws the picture.
+     * @param context expression context for evaluating data defined picture sources
      * @note added in 2.3
      */
-    void refreshPicture();
+    void refreshPicture( const QgsExpressionContext* context = nullptr );
 
-    /**Prepares the picture's source expression after it is altered or the compositions
+    /** Prepares the picture's source expression after it is altered or the compositions
      * atlas coverage layer changes.
      * @note added in 2.3
      * @deprecated no longer required
      */
-    Q_DECL_DEPRECATED void updatePictureExpression() {};
+    Q_DECL_DEPRECATED void updatePictureExpression() {}
 
-    /**Forces a recalculation of the picture's frame size
+    /** Forces a recalculation of the picture's frame size
      * @note added in 2.3
      */
     void recalculateSize();
 
-    virtual void refreshDataDefinedProperty( const QgsComposerObject::DataDefinedProperty property = QgsComposerObject::AllProperties ) override;
+    virtual void refreshDataDefinedProperty( const QgsComposerObject::DataDefinedProperty property = QgsComposerObject::AllProperties, const QgsExpressionContext *context = nullptr ) override;
 
   signals:
-    /**Is emitted on picture rotation change*/
+    /** Is emitted on picture rotation change*/
     void pictureRotationChanged( double newRotation );
 
   private:
 
     //default constructor is forbidden
     QgsComposerPicture();
-    /**Calculates bounding rect for svg file (mSourcefile) such that aspect ratio is correct*/
+    /** Calculates bounding rect for svg file (mSourcefile) such that aspect ratio is correct*/
     QRectF boundedSVGRect( double deviceWidth, double deviceHeight );
-    /**Calculates bounding rect for image such that aspect ratio is correct*/
+    /** Calculates bounding rect for image such that aspect ratio is correct*/
     QRectF boundedImageRect( double deviceWidth, double deviceHeight );
 
-    /**Returns size of current raster or svg picture */
+    /** Returns size of current raster or svg picture */
     QSizeF pictureSize();
 
     QImage mImage;
@@ -310,43 +401,56 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
 
     QSize mDefaultSvgSize;
 
-    /**Image rotation*/
+    /** Image rotation*/
     double mPictureRotation;
-    /**Map that sets the rotation (or 0 if this picture uses map independent rotation)*/
+    /** Map that sets the rotation (or 0 if this picture uses map independent rotation)*/
     const QgsComposerMap* mRotationMap;
-    /**Width of the picture (in mm)*/
+
+    //! Mode used to align to North
+    NorthMode mNorthMode;
+    //! Offset for north arrow
+    double mNorthOffset;
+
+    /** Width of the picture (in mm)*/
     double mPictureWidth;
-    /**Height of the picture (in mm)*/
+    /** Height of the picture (in mm)*/
     double mPictureHeight;
 
     ResizeMode mResizeMode;
     QgsComposerItem::ItemPositionMode mPictureAnchor;
 
+    QColor mSvgFillColor;
+    QColor mSvgBorderColor;
+    double mSvgBorderWidth;
+
     bool mHasExpressionError;
     bool mLoaded;
+    bool mLoadingSvg;
 
-    /**loads an image file into the picture item and redraws the item*/
+    /** Loads an image file into the picture item and redraws the item*/
     void loadPicture( const QString &path );
 
-    /**sets up the picture item and connects to relevant signals*/
+    /** Sets up the picture item and connects to relevant signals*/
     void init();
 
-    /**Returns part of a raster image which will be shown, given current picture
+    /** Returns part of a raster image which will be shown, given current picture
      * anchor settings
-    */
+     */
     QRect clippedImageRect( double &boundRectWidthMM, double &boundRectHeightMM, QSize imageRectPixels );
 
-    /**Loads a remote picture for the item
-    */
+    /** Loads a remote picture for the item
+     */
     void loadRemotePicture( const QString &url );
 
-    /**Loads a local picture for the item
-    */
+    /** Loads a local picture for the item
+     */
     void loadLocalPicture( const QString &path );
 
   private slots:
 
     void remotePictureLoaded();
+
+    void updateMapRotation();
 };
 
 #endif
