@@ -24,7 +24,7 @@
 
 class QgsFillSymbolV2;
 
-/**A composer items that draws common shapes (ellipse, triangle, rectangle)*/
+/** A composer items that draws common shapes (ellipse, triangle, rectangle)*/
 class CORE_EXPORT QgsComposerShape: public QgsComposerItem
 {
     Q_OBJECT
@@ -41,19 +41,19 @@ class CORE_EXPORT QgsComposerShape: public QgsComposerItem
     QgsComposerShape( qreal x, qreal y, qreal width, qreal height, QgsComposition* composition );
     ~QgsComposerShape();
 
-    /** return correct graphics item type. */
+    /** Return correct graphics item type. */
     virtual int type() const override { return ComposerShape; }
 
     /** \brief Reimplementation of QCanvasItem::paint - draw on canvas */
     void paint( QPainter* painter, const QStyleOptionGraphicsItem* itemStyle, QWidget* pWidget ) override;
 
-    /** stores state in Dom element
+    /** Stores state in Dom element
      * @param elem is Dom element corresponding to 'Composer' tag
      * @param doc write template file
      */
     bool writeXML( QDomElement& elem, QDomDocument & doc ) const override;
 
-    /** sets state from Dom document
+    /** Sets state from Dom document
      * @param itemElem is Dom node corresponding to item tag
      * @param doc is Dom document
      */
@@ -63,33 +63,33 @@ class CORE_EXPORT QgsComposerShape: public QgsComposerItem
     QgsComposerShape::Shape shapeType() const { return mShape; }
     void setShapeType( QgsComposerShape::Shape s );
 
-    /**Sets radius for rounded rectangle corners. Added in v2.1 */
+    /** Sets radius for rounded rectangle corners. Added in v2.1 */
     void setCornerRadius( double radius );
-    /**Returns the radius for rounded rectangle corners*/
-    double cornerRadius() const { return mCornerRadius; };
+    /** Returns the radius for rounded rectangle corners*/
+    double cornerRadius() const { return mCornerRadius; }
 
-    /**Sets the QgsFillSymbolV2 used to draw the shape. Must also call setUseSymbolV2( true ) to
+    /** Sets the QgsFillSymbolV2 used to draw the shape. Must also call setUseSymbolV2( true ) to
      * enable drawing with a symbol.
      * Note: added in version 2.1*/
     void setShapeStyleSymbol( QgsFillSymbolV2* symbol );
-    /**Returns the QgsFillSymbolV2 used to draw the shape.
+    /** Returns the QgsFillSymbolV2 used to draw the shape.
      * Note: added in version 2.1*/
     QgsFillSymbolV2* shapeStyleSymbol() { return mShapeStyleSymbol; }
 
-    /**Controls whether the shape should be drawn using a QgsFillSymbolV2.
+    /** Controls whether the shape should be drawn using a QgsFillSymbolV2.
      * Note: Added in v2.1 */
     void setUseSymbolV2( bool useSymbolV2 );
 
-    /**Depending on the symbol style, the bounding rectangle can be larger than the shape
+    /** Depending on the symbol style, the bounding rectangle can be larger than the shape
     @note this function was added in version 2.3*/
     QRectF boundingRect() const override;
 
-    /**Sets new scene rectangle bounds and recalculates hight and extent. Reimplemented from
+    /** Sets new scene rectangle bounds and recalculates hight and extent. Reimplemented from
      * QgsComposerItem as it needs to call updateBoundingRect after the shape's size changes
-    */
+     */
     void setSceneRect( const QRectF& rectangle ) override;
 
-    //Overriden to return shape type
+    //Overridden to return shape type
     virtual QString displayName() const override;
 
   protected:
@@ -97,18 +97,18 @@ class CORE_EXPORT QgsComposerShape: public QgsComposerItem
     virtual void drawFrame( QPainter* p ) override;
     /* reimplement drawBackground, since it's not a rect, but a custom shape */
     virtual void drawBackground( QPainter* p ) override;
-    /**reimplement estimatedFrameBleed, since frames on shapes are drawn using symbology
+    /** Reimplement estimatedFrameBleed, since frames on shapes are drawn using symbology
      * rather than the item's pen */
     virtual double estimatedFrameBleed() const override;
 
   public slots:
-    /**Should be called after the shape's symbol is changed. Redraws the shape and recalculates
+    /** Should be called after the shape's symbol is changed. Redraws the shape and recalculates
      * its selection bounds.
      * Note: added in version 2.1*/
     void refreshSymbol();
 
   private:
-    /**Ellipse, rectangle or triangle*/
+    /** Ellipse, rectangle or triangle*/
     Shape mShape;
 
     double mCornerRadius;
@@ -117,7 +117,7 @@ class CORE_EXPORT QgsComposerShape: public QgsComposerItem
 
     QgsFillSymbolV2* mShapeStyleSymbol;
     double mMaxSymbolBleed;
-    /**Current bounding rectangle of shape*/
+    /** Current bounding rectangle of shape*/
     QRectF mCurrentRectangle;
 
     /* draws the custom shape */
@@ -129,10 +129,10 @@ class CORE_EXPORT QgsComposerShape: public QgsComposerItem
     /* creates the default shape symbol */
     void createDefaultShapeStyleSymbol();
 
-    /**Returns a point on the line from startPoint to directionPoint that is a certain distance away from the starting point*/
+    /** Returns a point on the line from startPoint to directionPoint that is a certain distance away from the starting point*/
     QPointF pointOnLineWithDistance( const QPointF& startPoint, const QPointF& directionPoint, double distance ) const;
 
-    /**Updates the bounding rect of this item*/
+    /** Updates the bounding rect of this item*/
     void updateBoundingRect();
 };
 
