@@ -18,6 +18,7 @@
 #include "qgslegendmodel.h"
 #include "qgscomposerlegenditem.h"
 #include "qgsfield.h"
+#include "qgslayertree.h"
 #include "qgsmaplayer.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsrasterlayer.h"
@@ -43,19 +44,44 @@ QgsLegendModel::QgsLegendModel(): QStandardItemModel(), mAutoUpdate( true )
   }
 
   QWidgetList topLevelWidgets = QApplication::topLevelWidgets();
-  mHasTopLevelWindow = ( topLevelWidgets.size() > 0 );
+  mHasTopLevelWindow = ( !topLevelWidgets.isEmpty() );
 }
 
 QgsLegendModel::~QgsLegendModel()
 {
 }
 
+void QgsLegendModel::setLayerSetAndGroups( QgsLayerTreeGroup* rootGroup )
+{
+  clear();
+  addGroupFromLayerTree( rootGroup, invisibleRootItem() );
+}
+
+void QgsLegendModel::addGroupFromLayerTree( QgsLayerTreeGroup* parentGroup, QStandardItem* parentItem )
+{
+  Q_FOREACH ( QgsLayerTreeNode* node, parentGroup->children() )
+  {
+    if ( QgsLayerTree::isGroup( node ) )
+    {
+      QgsLayerTreeGroup* nodeGroup = QgsLayerTree::toGroup( node );
+      QStandardItem* groupItem = addGroup( nodeGroup->name(), -1, parentItem );
+      addGroupFromLayerTree( nodeGroup, groupItem );
+    }
+    else if ( QgsLayerTree::isLayer( node ) )
+    {
+      QgsLayerTreeLayer* nodeLayer = QgsLayerTree::toLayer( node );
+      if ( nodeLayer->layer() )
+        addLayer( nodeLayer->layer(), -1, QString(), parentItem );
+    }
+  }
+}
+
 void QgsLegendModel::setLayerSetAndGroups( const QStringList& layerIds, const QList< GroupLayerInfo >& groupInfo )
 {
   setLayerSet( layerIds );
 
-  QStandardItem* currentItem = 0;
-  QStandardItem* currentGroupItem = 0;
+  QStandardItem* currentItem = nullptr;
+  QStandardItem* currentGroupItem = nullptr;
   int i = 0;
 
   QList< GroupLayerInfo >::const_iterator infoIt = groupInfo.constBegin();
@@ -98,7 +124,7 @@ void QgsLegendModel::setLayerSetAndGroups( const QStringList& layerIds, const QL
   }
 }
 
-void QgsLegendModel::setLayerSet( const QStringList& layerIds, double scaleDenominator, QString rule )
+void QgsLegendModel::setLayerSet( const QStringList& layerIds, double scaleDenominator, const QString& rule )
 {
   mLayerIds = layerIds;
 
@@ -106,7 +132,7 @@ void QgsLegendModel::setLayerSet( const QStringList& layerIds, double scaleDenom
   clear();
 
   QStringList::const_iterator idIter = mLayerIds.constBegin();
-  QgsMapLayer* currentLayer = 0;
+  QgsMapLayer* currentLayer = nullptr;
 
   for ( ; idIter != mLayerIds.constEnd(); ++idIter )
   {
@@ -115,27 +141,30 @@ void QgsLegendModel::setLayerSet( const QStringList& layerIds, double scaleDenom
   }
 }
 
-QStandardItem* QgsLegendModel::addGroup( QString text, int position )
+QStandardItem* QgsLegendModel::addGroup( QString text, int position, QStandardItem* parentItem )
 {
   if ( text.isNull() )
     text = tr( "Group" );
 
+  if ( !parentItem )
+    parentItem = invisibleRootItem();
+
   QgsComposerGroupItem* groupItem = new QgsComposerGroupItem( text );
   groupItem->setUserText( text );
 
   if ( position == -1 )
   {
-    position = invisibleRootItem()->rowCount();
+    position = parentItem->rowCount();
   }
   QList<QStandardItem *> itemsList;
   itemsList << groupItem << new QgsComposerStyleItem( groupItem );
-  invisibleRootItem()->insertRow( position, itemsList );
+  parentItem->insertRow( position, itemsList );
 
   emit layersChanged();
   return groupItem;
 }
 
-int QgsLegendModel::addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLayer* vlayer, double scaleDenominator, QString rule )
+int QgsLegendModel::addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLayer* vlayer, double scaleDenominator, const QString& rule )
 {
   QgsComposerLayerItem* lItem = dynamic_cast<QgsComposerLayerItem*>( layerItem );
 
@@ -163,7 +192,7 @@ int QgsLegendModel::addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLa
   int row = 0;
   for ( ; symbolIt != lst.constEnd(); ++symbolIt )
   {
-    if ( scaleDenominator == -1 && rule.isEmpty() )
+    if ( qgsDoubleNear( scaleDenominator, -1 ) && rule.isEmpty() )
     {
       QgsComposerSymbolV2Item* currentSymbolItem = new QgsComposerSymbolV2Item( "" );
 
@@ -204,7 +233,7 @@ int QgsLegendModel::addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLa
   }
 
   // Don't remove row on getLegendGraphic (read only with filter)
-  if ( scaleDenominator == -1 && rule.isEmpty() )
+  if ( qgsDoubleNear( scaleDenominator, -1 ) && rule.isEmpty() )
   {
     // Delete following old items (if current number of items decreased)
     for ( int i = layerItem->rowCount() - 1; i >= row; --i )
@@ -524,13 +553,16 @@ void QgsLegendModel::removeLayer( const QString& layerId )
   }
 }
 
-void QgsLegendModel::addLayer( QgsMapLayer* theMapLayer, double scaleDenominator, QString rule )
+void QgsLegendModel::addLayer( QgsMapLayer* theMapLayer, double scaleDenominator, const QString& rule, QStandardItem* parentItem )
 {
   if ( !theMapLayer )
   {
     return;
   }
 
+  if ( !parentItem )
+    parentItem = invisibleRootItem();
+
   QgsComposerLayerItem* layerItem = new QgsComposerLayerItem( theMapLayer->name() );
   if ( theMapLayer->title() != "" )
   {
@@ -543,7 +575,7 @@ void QgsLegendModel::addLayer( QgsMapLayer* theMapLayer, double scaleDenominator
 
   QList<QStandardItem *> itemsList;
   itemsList << layerItem << new QgsComposerStyleItem( layerItem );
-  invisibleRootItem()->appendRow( itemsList );
+  parentItem->appendRow( itemsList );
 
   switch ( theMapLayer->type() )
   {
@@ -597,8 +629,8 @@ bool QgsLegendModel::writeXML( QDomElement& composerLegendElem, QDomDocument& do
   QDomElement legendModelElem = doc.createElement( "Model" );
   legendModelElem.setAttribute( "autoUpdate", mAutoUpdate );
   int nTopLevelItems = invisibleRootItem()->rowCount();
-  QStandardItem* currentItem = 0;
-  QgsComposerLegendItem* currentLegendItem = 0;
+  QStandardItem* currentItem = nullptr;
+  QgsComposerLegendItem* currentLegendItem = nullptr;
 
   for ( int i = 0; i < nTopLevelItems; ++i )
   {
@@ -630,7 +662,6 @@ bool QgsLegendModel::readXML( const QDomElement& legendModelElem, const QDomDocu
 
   QDomNodeList topLevelItemList = legendModelElem.childNodes();
   QDomElement currentElem;
-  QgsComposerLegendItem* currentItem = 0;
 
   int nTopLevelItems =  topLevelItemList.size();
   for ( int i = 0; i < nTopLevelItems; ++i )
@@ -642,6 +673,7 @@ bool QgsLegendModel::readXML( const QDomElement& legendModelElem, const QDomDocu
     }
 
     //toplevel items can be groups or layers
+    QgsComposerLegendItem* currentItem = nullptr;
     if ( currentElem.tagName() == "LayerItem" )
     {
       currentItem = new QgsComposerLayerItem();
@@ -650,6 +682,10 @@ bool QgsLegendModel::readXML( const QDomElement& legendModelElem, const QDomDocu
     {
       currentItem = new QgsComposerGroupItem();
     }
+
+    if ( !currentItem )
+      continue;
+
     currentItem->readXML( currentElem, mHasTopLevelWindow );
 
     QList<QStandardItem *> itemsList;
@@ -694,7 +730,7 @@ Qt::ItemFlags QgsLegendModel::flags( const QModelIndex &index ) const
   if ( index.column() == 1 && item )
   {
     // Style
-    QStandardItem* firstColumnItem = 0;
+    QStandardItem* firstColumnItem = nullptr;
     if ( item->parent() )
     {
       firstColumnItem = item->parent()->child( index.row(), 0 );
@@ -787,7 +823,7 @@ bool QgsLegendModel::dropMimeData( const QMimeData *data, Qt::DropAction action,
     return false;
   }
 
-  QStandardItem* dropIntoItem = 0;
+  QStandardItem* dropIntoItem = nullptr;
   if ( parent.isValid() )
   {
     dropIntoItem = itemFromIndex( parent );
@@ -812,7 +848,7 @@ bool QgsLegendModel::dropMimeData( const QMimeData *data, Qt::DropAction action,
   int nChildNodes = nodeList.size();
   QDomElement currentElem;
   QString currentTagName;
-  QgsComposerLegendItem* currentItem = 0;
+  QgsComposerLegendItem* currentItem = nullptr;
 
   for ( int i = 0; i < nChildNodes; ++i )
   {
