@@ -61,21 +61,21 @@ class CORE_EXPORT QgsLegendModel : public QStandardItemModel
      * @deprecated in 2.6
      */
     Q_DECL_DEPRECATED void setLayerSetAndGroups( const QStringList& layerIds, const QList< GroupLayerInfo >& groupInfo );
-    void setLayerSet( const QStringList& layerIds, double scaleDenominator = -1, QString rule = "" );
-    /**Adds a group
+    void setLayerSet( const QStringList& layerIds, double scaleDenominator = -1, const QString& rule = "" );
+    /** Adds a group
       @param text name of group (defaults to translation of "Group")
       @param position insertion position (toplevel position (or -1 if it should be placed at the end of the legend).
       @param parentItem parent item
       @returns a pointer to the added group
       */
-    QStandardItem *addGroup( QString text = QString::null, int position = -1, QStandardItem* parentItem = 0 );
+    QStandardItem *addGroup( QString text = QString::null, int position = -1, QStandardItem* parentItem = nullptr );
 
-    /**Tries to automatically update a model entry (e.g. a whole layer or only a single item)*/
+    /** Tries to automatically update a model entry (e.g. a whole layer or only a single item)*/
     void updateItem( QStandardItem* item );
-    /**Updates the whole symbology of a layer*/
+    /** Updates the whole symbology of a layer*/
     void updateLayer( QStandardItem* layerItem );
-    /**Tries to update a single classification item*/
-    void updateVectorV2ClassificationItem( QStandardItem* classificationItem, QgsSymbolV2* symbol, QString itemText )
+    /** Tries to update a single classification item*/
+    void updateVectorV2ClassificationItem( QStandardItem* classificationItem, QgsSymbolV2* symbol, const QString& itemText )
     { Q_UNUSED( classificationItem ); Q_UNUSED( symbol ); Q_UNUSED( itemText ); }
     void updateRasterClassificationItem( QStandardItem* classificationItem )
     { Q_UNUSED( classificationItem ); }
@@ -90,14 +90,14 @@ class CORE_EXPORT QgsLegendModel : public QStandardItemModel
     Qt::DropActions supportedDropActions() const override;
     Qt::ItemFlags flags( const QModelIndex &index ) const override;
 
-    /**Implemented to support drag operations*/
+    /** Implemented to support drag operations*/
     virtual bool removeRows( int row, int count, const QModelIndex & parent = QModelIndex() ) override;
 
-    /**For the drag operation*/
+    /** For the drag operation*/
     QMimeData* mimeData( const QModelIndexList &indexes ) const override;
     QStringList mimeTypes() const override;
 
-    /**Implements the drop operation*/
+    /** Implements the drop operation*/
     bool dropMimeData( const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent ) override;
 
     void setAutoUpdate( bool autoUpdate );
@@ -105,7 +105,7 @@ class CORE_EXPORT QgsLegendModel : public QStandardItemModel
 
   public slots:
     void removeLayer( const QString& layerId );
-    void addLayer( QgsMapLayer* theMapLayer, double scaleDenominator = -1, QString rule = "", QStandardItem* parentItem = 0 );
+    void addLayer( QgsMapLayer* theMapLayer, double scaleDenominator = -1, const QString& rule = "", QStandardItem* parentItem = nullptr );
 
   private slots:
     void updateLayer();
@@ -114,10 +114,10 @@ class CORE_EXPORT QgsLegendModel : public QStandardItemModel
     void layersChanged();
 
   private:
-    /**Adds classification items of vector layers using new symbology*/
-    int addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLayer* vlayer, double scaleDenominator = -1, QString rule = "" );
+    /** Adds classification items of vector layers using new symbology*/
+    int addVectorLayerItemsV2( QStandardItem* layerItem, QgsVectorLayer* vlayer, double scaleDenominator = -1, const QString& rule = "" );
 
-    /**Adds item of raster layer
+    /** Adds item of raster layer
      @return 0 in case of success*/
     int addRasterLayerItems( QStandardItem* layerItem, QgsMapLayer* rlayer );
 
@@ -129,11 +129,11 @@ class CORE_EXPORT QgsLegendModel : public QStandardItemModel
 
   protected:
     QStringList mLayerIds;
-    /**True if this application has toplevel windows (normally true). If this is false, this means that the application
+    /** True if this application has toplevel windows (normally true). If this is false, this means that the application
        might not have a running x-server on unix systems and so QPixmap and QIcon cannot be used*/
     bool mHasTopLevelWindow;
 
-    /**True if the legend is auto updated when layers are added or removed from the map canvas */
+    /** True if the legend is auto updated when layers are added or removed from the map canvas */
     bool mAutoUpdate;
 };
 
