@@ -120,7 +120,7 @@ void QgsDxfPaintEngine::drawPath( const QPainterPath& path )
   endCurve();
   endPolygon();
 
-  if ( mPolygon.size() > 0 && mBrush.style() != Qt::NoBrush )
+  if ( !mPolygon.isEmpty() && mBrush.style() != Qt::NoBrush )
     mDxf->writePolygon( mPolygon, mLayer, "SOLID", mBrush.color() );
 
   mPolygon.clear();
@@ -142,7 +142,7 @@ void QgsDxfPaintEngine::lineTo( double dx, double dy )
 void QgsDxfPaintEngine::curveTo( double dx, double dy )
 {
   endCurve();
-  if ( mCurrentPolygon.size() > 0 )
+  if ( !mCurrentPolygon.isEmpty() )
     mCurrentCurve.append( mCurrentPolygon.last() );
 
   mCurrentCurve.append( QPointF( dx, dy ) );
@@ -201,7 +201,7 @@ void QgsDxfPaintEngine::drawLines( const QLineF* lines, int lineCount )
   }
 }
 
-QgsPoint QgsDxfPaintEngine::toDxfCoordinates( const QPointF& pt ) const
+QgsPoint QgsDxfPaintEngine::toDxfCoordinates( QPointF pt ) const
 {
   if ( !mPaintDevice || !mDxf )
     return QgsPoint( pt.x(), pt.y() );
@@ -265,7 +265,7 @@ double QgsDxfPaintEngine::power( double a, int b )
     return 1;
 
   double tmp = a;
-  for ( int i = 2; i <= qAbs(( double )b ); i++ )
+  for ( int i = 2; i <= qAbs( static_cast< double >( b ) ); i++ )
     a *= tmp;
 
   if ( b > 0 )
