@@ -19,12 +19,16 @@
 #define QGSDXFPAINTENGINE_H
 
 #include <QPaintEngine>
-#include <qgsgeometry.h>
+#include "qgsgeometryfactory.h"
 
 class QgsDxfExport;
 class QgsDxfPaintDevice;
 class QgsPoint;
 
+/** \class QgsDxfPaintEngine
+ * \note not available in Python bindings
+*/
+
 class CORE_EXPORT QgsDxfPaintEngine: public QPaintEngine
 {
   public:
@@ -45,7 +49,7 @@ class CORE_EXPORT QgsDxfPaintEngine: public QPaintEngine
     void setLayer( const QString& layer ) { mLayer = layer; }
     QString layer() const { return mLayer; }
 
-    void setShift( const QPointF& shift ) { mShift = shift; }
+    void setShift( QPointF shift ) { mShift = shift; }
 
   private:
     const QgsDxfPaintDevice* mPaintDevice;
@@ -61,7 +65,7 @@ class CORE_EXPORT QgsDxfPaintEngine: public QPaintEngine
     QPolygonF mCurrentPolygon;
     QList<QPointF> mCurrentCurve;
 
-    QgsPoint toDxfCoordinates( const QPointF& pt ) const;
+    QgsPoint toDxfCoordinates( QPointF pt ) const;
     double currentWidth() const;
 
     void moveTo( double dx, double dy );
