@@ -365,7 +365,7 @@ double QgsCurvePolygonV2::area() const
 
   double totalArea = 0.0;
 
-  if ( mExteriorRing->isClosed() )
+  if ( mExteriorRing->isRing() )
   {
     double area = 0.0;
     mExteriorRing->sumUpArea( area );
@@ -376,7 +376,7 @@ double QgsCurvePolygonV2::area() const
   for ( ; ringIt != mInteriorRings.constEnd(); ++ringIt )
   {
     double area = 0.0;
-    if (( *ringIt )->isClosed() )
+    if (( *ringIt )->isRing() )
     {
       ( *ringIt )->sumUpArea( area );
       totalArea -= qAbs( area );
@@ -607,11 +607,32 @@ QgsCoordinateSequenceV2 QgsCurvePolygonV2::coordinateSequence() const
   return mCoordinateSequence;
 }
 
+int QgsCurvePolygonV2::nCoordinates() const
+{
+  if ( !mCoordinateSequence.isEmpty() )
+    return QgsAbstractGeometryV2::nCoordinates();
+
+  int count = 0;
+
+  if ( mExteriorRing )
+  {
+    count += mExteriorRing->nCoordinates();
+  }
+
+  QList<QgsCurveV2*>::const_iterator it = mInteriorRings.constBegin();
+  for ( ; it != mInteriorRings.constEnd(); ++it )
+  {
+    count += ( *it )->nCoordinates();
+  }
+
+  return count;
+}
+
 double QgsCurvePolygonV2::closestSegment( const QgsPointV2& pt, QgsPointV2& segmentPt, QgsVertexId& vertexAfter, bool* leftOf, double epsilon ) const
 {
   if ( !mExteriorRing )
   {
-    return 0.0;
+    return -1;
   }
   QList<QgsCurveV2*> segmentList;
   segmentList.append( mExteriorRing );
