@@ -17,6 +17,7 @@
 
 #include "qgslinestringv2.h"
 #include "qgsapplication.h"
+#include "qgscompoundcurvev2.h"
 #include "qgscoordinatetransform.h"
 #include "qgsgeometryutils.h"
 #include "qgsmaptopixel.h"
@@ -583,6 +584,13 @@ QPolygonF QgsLineStringV2::asQPolygonF() const
   return points;
 }
 
+QgsAbstractGeometryV2* QgsLineStringV2::toCurveType() const
+{
+  QgsCompoundCurveV2* compoundCurve = new QgsCompoundCurveV2();
+  compoundCurve->addCurve( clone() );
+  return compoundCurve;
+}
+
 /***************************************************************************
  * This class is considered CRITICAL and any change MUST be accompanied with
  * full unit tests.
@@ -736,16 +744,10 @@ double QgsLineStringV2::closestSegment( const QgsPointV2& pt, QgsPointV2& segmen
   double segmentPtX, segmentPtY;
 
   int size = mX.size();
-  if ( size == 0 )
+  if ( size == 0 || size == 1 )
   {
     vertexAfter = QgsVertexId( 0, 0, 0 );
-    return sqrDist;
-  }
-  else if ( size == 1 )
-  {
-    segmentPt = pointN( 0 );
-    vertexAfter = QgsVertexId( 0, 0, 1 );
-    return QgsGeometryUtils::sqrDistance2D( pt, segmentPt );
+    return -1;
   }
   for ( int i = 1; i < size; ++i )
   {
@@ -835,9 +837,6 @@ QgsPointV2 QgsLineStringV2::centroid() const
 void QgsLineStringV2::sumUpArea( double& sum ) const
 {
   int maxIndex = numPoints() - 1;
-  if ( maxIndex == 1 )
-    return; //no area, just a single line
-
   for ( int i = 0; i < maxIndex; ++i )
   {
     sum += 0.5 * ( mX.at( i ) * mY.at( i + 1 ) - mY.at( i ) * mX.at( i + 1 ) );
