@@ -27,8 +27,8 @@ QgsQtLocationConnection::QgsQtLocationConnection(): QgsGPSConnection( new QLocal
   //needed to fix https://sourceforge.net/p/necessitas/tickets/146/
   qRegisterMetaType< QList<QGeoSatelliteInfo> >( "QList<QGeoSatelliteInfo>" );
 
-  startGPS();
   startSatelliteMonitor();
+  startGPS();
 
   //HACK to signal the gpsinformationwidget that we have a QtLocationConnection
   QTimer::singleShot( 500, this, SLOT( broadcastConnectionAvailable() ) );
@@ -65,7 +65,6 @@ void QgsQtLocationConnection::parseData()
   {
     mStatus = GPSDataReceived;
     //const QGeoPositionInfo &info = locationDataSource->lastKnownPosition();
-    qDebug() << mInfo;
     if ( mInfo.isValid() )
     {
       // mInfo.HorizontalAccuracy;
@@ -110,7 +109,11 @@ void QgsQtLocationConnection::satellitesInViewUpdated(
     QgsSatelliteInfo satelliteInfo;
     satelliteInfo.azimuth = currentSatellite.attribute( QGeoSatelliteInfo::Azimuth );
     satelliteInfo.elevation = currentSatellite.attribute( QGeoSatelliteInfo::Elevation );
+#if defined(HAVE_QT_MOBILITY_LOCATION )
     satelliteInfo.id = currentSatellite.prnNumber();
+#else // QtPositioning
+    satelliteInfo.id = currentSatellite.satelliteIdentifier();
+#endif
     satelliteInfo.signal = currentSatellite.signalStrength();
     mLastGPSInformation.satellitesInView.append( satelliteInfo );
   }
@@ -130,13 +133,21 @@ void QgsQtLocationConnection::satellitesInUseUpdated(
   {
     QGeoSatelliteInfo currentSatellite = satellites.at( i );
     //add pnr to mLastGPSInformation.satPrn
+#if defined(HAVE_QT_MOBILITY_LOCATION )
     mLastGPSInformation.satPrn.append( currentSatellite.prnNumber() );
+#else // QtPositioning
+    mLastGPSInformation.satPrn.append( currentSatellite.satelliteIdentifier() );
+#endif
 
     //set QgsSatelliteInfo.inuse to true for the satellites in use
     for ( int i = 0; i < mLastGPSInformation.satellitesInView.size(); ++i )
     {
       QgsSatelliteInfo satInView = mLastGPSInformation.satellitesInView.at( i );
+#if defined(HAVE_QT_MOBILITY_LOCATION )
       if ( satInView.id == currentSatellite.prnNumber() )
+#else // QtPositioning
+      if ( satInView.id == currentSatellite.satelliteIdentifier() )
+#endif
       {
         satInView.inUse = true;
         break;
@@ -184,6 +195,7 @@ void QgsQtLocationConnection::startGPS()
 void QgsQtLocationConnection::startSatelliteMonitor()
 {
   QgsDebugMsg( "Starting GPS QtLocation satellite monitor" );
+
   if ( !satelliteInfoSource )
   {
     satelliteInfoSource = QGeoSatelliteInfoSource::createDefaultSource( this );
