@@ -50,7 +50,7 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
     QgsLayerTreeLayer* addLayer( QgsMapLayer* layer );
 
     //! Insert existing nodes at specified position. The nodes must not have a parent yet. The nodes will be owned by this group.
-    void insertChildNodes( int index, QList<QgsLayerTreeNode*> nodes );
+    void insertChildNodes( int index, const QList<QgsLayerTreeNode*>& nodes );
     //! Insert existing node at specified position. The node must not have a parent yet. The node will be owned by this group.
     void insertChildNode( int index, QgsLayerTreeNode* node );
     //! Append an existing node. The node must not have a parent yet. The node will be owned by this group.
@@ -87,25 +87,45 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
     virtual QString dump() const override;
 
     //! Return a clone of the group. The children are cloned too.
-    virtual QgsLayerTreeNode* clone() const override;
+    virtual QgsLayerTreeGroup* clone() const override;
 
     //! Return the check state of the group node
     Qt::CheckState isVisible() const { return mChecked; }
     //! Set check state of the group node - will also update children
     void setVisible( Qt::CheckState state );
 
+    //! Return whether the group is mutually exclusive (only one child can be checked at a time)
+    //! @note added in 2.12
+    bool isMutuallyExclusive() const;
+    //! Set whether the group is mutually exclusive (only one child can be checked at a time).
+    //! The initial child index determines which child should be initially checked. The default value
+    //! of -1 will determine automatically (either first one currently checked or none)
+    //! @note added in 2.12
+    void setIsMutuallyExclusive( bool enabled, int initialChildIndex = -1 );
+
   protected slots:
     void layerDestroyed();
     void nodeVisibilityChanged( QgsLayerTreeNode* node );
 
   protected:
+    //! Set check state of this group from its children
     void updateVisibilityFromChildren();
+    //! Set check state of children (when this group's check state changes) - if not mutually exclusive
+    void updateChildVisibility();
+    //! Set check state of children - if mutually exclusive
+    void updateChildVisibilityMutuallyExclusive();
 
   protected:
     QString mName;
     Qt::CheckState mChecked;
 
     bool mChangingChildVisibility;
+
+    //! Whether the group is mutually exclusive (i.e. only one child can be checked at a time)
+    bool mMutuallyExclusive;
+    //! Keeps track which child has been most recently selected
+    //! (so if the whole group is unchecked and checked again, we know which child to check)
+    int mMutuallyExclusiveChildIndex;
 };
 
 
