@@ -16,6 +16,7 @@
 #include "qgslayertreemodel.h"
 
 #include "qgslayertree.h"
+#include "qgslayertreeutils.h"
 #include "qgslayertreemodellegendnode.h"
 
 #include <QMimeData>
@@ -24,6 +25,7 @@
 #include "qgsdataitem.h"
 #include "qgsmaphittest.h"
 #include "qgsmaplayerlegend.h"
+#include "qgsmaplayerstylemanager.h"
 #include "qgspluginlayer.h"
 #include "qgsrasterlayer.h"
 #include "qgsrendererv2.h"
@@ -34,7 +36,7 @@
 QgsLayerTreeModel::QgsLayerTreeModel( QgsLayerTreeGroup* rootNode, QObject *parent )
     : QAbstractItemModel( parent )
     , mRootNode( rootNode )
-    , mFlags( ShowLegend | AllowLegendChangeState )
+    , mFlags( ShowLegend | AllowLegendChangeState | DeferredLegendInvalidation )
     , mAutoCollapseLegendNodesCount( -1 )
     , mLegendFilterByScale( 0 )
     , mLegendMapViewMupp( 0 )
@@ -44,6 +46,9 @@ QgsLayerTreeModel::QgsLayerTreeModel( QgsLayerTreeGroup* rootNode, QObject *pare
   connectToRootNode();
 
   mFontLayer.setBold( true );
+
+  connect( &mDeferLegendInvalidationTimer, SIGNAL( timeout() ), this, SLOT( invalidateLegendMapBasedData() ) );
+  mDeferLegendInvalidationTimer.setSingleShot( true );
 }
 
 QgsLayerTreeModel::~QgsLayerTreeModel()
@@ -167,8 +172,8 @@ QVariant QgsLayerTreeModel::data( const QModelIndex &index, int role ) const
       if ( nodeLayer->customProperty( "showFeatureCount", 0 ).toInt() && role == Qt::DisplayRole )
       {
         QgsVectorLayer* vlayer = qobject_cast<QgsVectorLayer*>( nodeLayer->layer() );
-        if ( vlayer && vlayer->pendingFeatureCount() >= 0 )
-          name += QString( " [%1]" ).arg( vlayer->pendingFeatureCount() );
+        if ( vlayer && vlayer->featureCount() >= 0 )
+          name += QString( " [%1]" ).arg( vlayer->featureCount() );
       }
       return name;
     }
@@ -192,7 +197,7 @@ QVariant QgsLayerTreeModel::data( const QModelIndex &index, int role ) const
         if ( testFlag( ShowRasterPreviewIcon ) )
         {
           QgsRasterLayer* rlayer = qobject_cast<QgsRasterLayer *>( layer );
-          return QIcon( rlayer->previewAsPixmap( QSize( 32, 32 ) ) );
+          return QIcon( QPixmap::fromImage( rlayer->previewAsImage( QSize( 32, 32 ) ) ) );
         }
         else
         {
@@ -283,7 +288,7 @@ Qt::ItemFlags QgsLayerTreeModel::flags( const QModelIndex& index ) const
 {
   if ( !index.isValid() )
   {
-    Qt::ItemFlags rootFlags = 0;
+    Qt::ItemFlags rootFlags = nullptr;
     if ( testFlag( AllowNodeReorder ) )
       rootFlags |= Qt::ItemIsDropEnabled;
     return rootFlags;
@@ -341,14 +346,14 @@ bool QgsLayerTreeModel::setData( const QModelIndex& index, const QVariant& value
     if ( QgsLayerTree::isLayer( node ) )
     {
       QgsLayerTreeLayer* layer = QgsLayerTree::toLayer( node );
-      layer->setVisible(( Qt::CheckState )value.toInt() );
+      layer->setVisible( static_cast< Qt::CheckState >( value.toInt() ) );
       return true;
     }
 
     if ( QgsLayerTree::isGroup( node ) )
     {
       QgsLayerTreeGroup* group = QgsLayerTree::toGroup( node );
-      group->setVisible(( Qt::CheckState )value.toInt() );
+      group->setVisible( static_cast< Qt::CheckState >( value.toInt() ) );
       return true;
     }
 
@@ -399,9 +404,9 @@ static bool _isChildOfNode( QgsLayerTreeNode* child, QgsLayerTreeNode* node )
   return _isChildOfNode( child->parent(), node );
 }
 
-static bool _isChildOfNodes( QgsLayerTreeNode* child, QList<QgsLayerTreeNode*> nodes )
+static bool _isChildOfNodes( QgsLayerTreeNode* child, const QList<QgsLayerTreeNode*>& nodes )
 {
-  foreach ( QgsLayerTreeNode* n, nodes )
+  Q_FOREACH ( QgsLayerTreeNode* n, nodes )
   {
     if ( _isChildOfNode( child, n ) )
       return true;
@@ -414,7 +419,7 @@ static bool _isChildOfNodes( QgsLayerTreeNode* child, QList<QgsLayerTreeNode*> n
 QList<QgsLayerTreeNode*> QgsLayerTreeModel::indexes2nodes( const QModelIndexList& list, bool skipInternal ) const
 {
   QList<QgsLayerTreeNode*> nodes;
-  foreach ( QModelIndex index, list )
+  Q_FOREACH ( const QModelIndex& index, list )
   {
     QgsLayerTreeNode* node = index2node( index );
     if ( !node )
@@ -428,7 +433,7 @@ QList<QgsLayerTreeNode*> QgsLayerTreeModel::indexes2nodes( const QModelIndexList
 
   // remove any children of nodes if both parent node and children are selected
   QList<QgsLayerTreeNode*> nodesFinal;
-  foreach ( QgsLayerTreeNode* node, nodes )
+  Q_FOREACH ( QgsLayerTreeNode* node, nodes )
   {
     if ( !_isChildOfNodes( node, nodes ) )
       nodesFinal << node;
@@ -439,13 +444,13 @@ QList<QgsLayerTreeNode*> QgsLayerTreeModel::indexes2nodes( const QModelIndexList
 
 bool QgsLayerTreeModel::isIndexSymbologyNode( const QModelIndex& index ) const
 {
-  return index2legendNode( index ) != 0;
+  return nullptr != index2legendNode( index );
 }
 
 QgsLayerTreeLayer* QgsLayerTreeModel::layerNodeForSymbologyNode( const QModelIndex& index ) const
 {
   QgsLayerTreeModelLegendNode* symNode = index2legendNode( index );
-  return symNode ? symNode->layerNode() : 0;
+  return symNode ? symNode->layerNode() : nullptr;
 }
 
 QgsLayerTreeGroup*QgsLayerTreeModel::rootGroup() const
@@ -549,25 +554,53 @@ void QgsLayerTreeModel::setLegendFilterByScale( double scaleDenominator )
 
   // this could be later done in more efficient way
   // by just updating active legend nodes, without refreshing original legend nodes
-  foreach ( QgsLayerTreeLayer* nodeLayer, mRootNode->findLayers() )
+  Q_FOREACH ( QgsLayerTreeLayer* nodeLayer, mRootNode->findLayers() )
     refreshLayerLegend( nodeLayer );
 }
 
 void QgsLayerTreeModel::setLegendFilterByMap( const QgsMapSettings* settings )
+{
+  setLegendFilter( settings, /* useExtent = */ true );
+}
+
+void QgsLayerTreeModel::setLegendFilter( const QgsMapSettings* settings, bool useExtent, const QgsGeometry& polygon, bool useExpressions )
 {
   if ( settings && settings->hasValidSettings() )
   {
-    mLegendFilterByMapSettings.reset( new QgsMapSettings( *settings ) );
-    mLegendFilterByMapHitTest.reset( new QgsMapHitTest( *mLegendFilterByMapSettings ) );
-    mLegendFilterByMapHitTest->run();
+    mLegendFilterMapSettings.reset( new QgsMapSettings( *settings ) );
+    mLegendFilterMapSettings->setLayerStyleOverrides( mLayerStyleOverrides );
+    QgsMapHitTest::LayerFilterExpression exprs;
+    // collect expression filters
+    if ( useExpressions )
+    {
+      Q_FOREACH ( QgsLayerTreeLayer* nodeLayer, mRootNode->findLayers() )
+      {
+        bool enabled;
+        QString expr = QgsLayerTreeUtils::legendFilterByExpression( *nodeLayer, &enabled );
+        if ( enabled && !expr.isEmpty() )
+        {
+          exprs[ nodeLayer->layerId()] = expr;
+        }
+      }
+    }
+    bool polygonValid = !polygon.isEmpty() && polygon.type() == QGis::Polygon;
+    if ( useExpressions && !useExtent && !polygonValid ) // only expressions
+    {
+      mLegendFilterHitTest.reset( new QgsMapHitTest( *mLegendFilterMapSettings, exprs ) );
+    }
+    else
+    {
+      mLegendFilterHitTest.reset( new QgsMapHitTest( *mLegendFilterMapSettings, polygon, exprs ) );
+    }
+    mLegendFilterHitTest->run();
   }
   else
   {
-    if ( !mLegendFilterByMapSettings )
+    if ( !mLegendFilterMapSettings )
       return; // no change
 
-    mLegendFilterByMapSettings.reset();
-    mLegendFilterByMapHitTest.reset();
+    mLegendFilterMapSettings.reset();
+    mLegendFilterHitTest.reset();
   }
 
   // temporarily disable autocollapse so that legend nodes stay visible
@@ -576,7 +609,7 @@ void QgsLayerTreeModel::setLegendFilterByMap( const QgsMapSettings* settings )
 
   // this could be later done in more efficient way
   // by just updating active legend nodes, without refreshing original legend nodes
-  foreach ( QgsLayerTreeLayer* nodeLayer, mRootNode->findLayers() )
+  Q_FOREACH ( QgsLayerTreeLayer* nodeLayer, mRootNode->findLayers() )
     refreshLayerLegend( nodeLayer );
 
   setAutoCollapseLegendNodes( bkAutoCollapse );
@@ -584,7 +617,7 @@ void QgsLayerTreeModel::setLegendFilterByMap( const QgsMapSettings* settings )
 
 void QgsLayerTreeModel::setLegendMapViewData( double mapUnitsPerPixel, int dpi, double scale )
 {
-  if ( mLegendMapViewDpi == dpi && mLegendMapViewMupp == mapUnitsPerPixel && mLegendMapViewScale == scale )
+  if ( mLegendMapViewDpi == dpi && qgsDoubleNear( mLegendMapViewMupp, mapUnitsPerPixel ) && qgsDoubleNear( mLegendMapViewScale, scale ) )
     return;
 
   mLegendMapViewMupp = mapUnitsPerPixel;
@@ -602,6 +635,16 @@ void QgsLayerTreeModel::legendMapViewData( double* mapUnitsPerPixel, int* dpi, d
   if ( scale ) *scale = mLegendMapViewScale;
 }
 
+QMap<QString, QString> QgsLayerTreeModel::layerStyleOverrides() const
+{
+  return mLayerStyleOverrides;
+}
+
+void QgsLayerTreeModel::setLayerStyleOverrides( const QMap<QString, QString>& overrides )
+{
+  mLayerStyleOverrides = overrides;
+}
+
 void QgsLayerTreeModel::nodeWillAddChildren( QgsLayerTreeNode* node, int indexFrom, int indexTo )
 {
   Q_ASSERT( node );
@@ -629,7 +672,7 @@ void QgsLayerTreeModel::nodeAddedChildren( QgsLayerTreeNode* node, int indexFrom
 
   endInsertRows();
 
-  foreach ( QgsLayerTreeLayer* newLayerNode, _layerNodesInSubtree( node, indexFrom, indexTo ) )
+  Q_FOREACH ( QgsLayerTreeLayer* newLayerNode, _layerNodesInSubtree( node, indexFrom, indexTo ) )
     connectToLayer( newLayerNode );
 }
 
@@ -640,7 +683,7 @@ void QgsLayerTreeModel::nodeWillRemoveChildren( QgsLayerTreeNode* node, int inde
   beginRemoveRows( node2index( node ), indexFrom, indexTo );
 
   // disconnect from layers and remove their legend
-  foreach ( QgsLayerTreeLayer* nodeLayer, _layerNodesInSubtree( node, indexFrom, indexTo ) )
+  Q_FOREACH ( QgsLayerTreeLayer* nodeLayer, _layerNodesInSubtree( node, indexFrom, indexTo ) )
     disconnectFromLayer( nodeLayer );
 }
 
@@ -671,7 +714,7 @@ void QgsLayerTreeModel::nodeLayerLoaded()
   if ( !nodeLayer )
     return;
 
-  // deffered connection to the layer
+  // deferred connection to the layer
   connectToLayer( nodeLayer );
 }
 
@@ -777,7 +820,7 @@ void QgsLayerTreeModel::connectToLayer( QgsLayerTreeLayer* nodeLayer )
 static int _numLayerCount( QgsLayerTreeGroup* group, const QString& layerId )
 {
   int count = 0;
-  foreach ( QgsLayerTreeNode* child, group->children() )
+  Q_FOREACH ( QgsLayerTreeNode* child, group->children() )
   {
     if ( QgsLayerTree::isLayer( child ) )
     {
@@ -794,7 +837,7 @@ static int _numLayerCount( QgsLayerTreeGroup* group, const QString& layerId )
 
 void QgsLayerTreeModel::disconnectFromLayer( QgsLayerTreeLayer* nodeLayer )
 {
-  disconnect( nodeLayer, 0, this, 0 ); // disconnect from delayed load of layer
+  disconnect( nodeLayer, nullptr, this, nullptr ); // disconnect from delayed load of layer
 
   if ( !nodeLayer->layer() )
     return; // we were never connected
@@ -807,13 +850,13 @@ void QgsLayerTreeModel::disconnectFromLayer( QgsLayerTreeLayer* nodeLayer )
   if ( _numLayerCount( mRootNode, nodeLayer->layerId() ) == 1 )
   {
     // last instance of the layer in the tree: disconnect from all signals from layer!
-    disconnect( nodeLayer->layer(), 0, this, 0 );
+    disconnect( nodeLayer->layer(), nullptr, this, nullptr );
   }
 }
 
 void QgsLayerTreeModel::connectToLayers( QgsLayerTreeGroup* parentGroup )
 {
-  foreach ( QgsLayerTreeNode* node, parentGroup->children() )
+  Q_FOREACH ( QgsLayerTreeNode* node, parentGroup->children() )
   {
     if ( QgsLayerTree::isGroup( node ) )
       connectToLayers( QgsLayerTree::toGroup( node ) );
@@ -824,7 +867,7 @@ void QgsLayerTreeModel::connectToLayers( QgsLayerTreeGroup* parentGroup )
 
 void QgsLayerTreeModel::disconnectFromLayers( QgsLayerTreeGroup* parentGroup )
 {
-  foreach ( QgsLayerTreeNode* node, parentGroup->children() )
+  Q_FOREACH ( QgsLayerTreeNode* node, parentGroup->children() )
   {
     if ( QgsLayerTree::isGroup( node ) )
       disconnectFromLayers( QgsLayerTree::toGroup( node ) );
@@ -850,7 +893,7 @@ void QgsLayerTreeModel::connectToRootNode()
 
 void QgsLayerTreeModel::disconnectFromRootNode()
 {
-  disconnect( mRootNode, 0, this, 0 );
+  disconnect( mRootNode, nullptr, this, nullptr );
 
   disconnectFromLayers( mRootNode );
 }
@@ -872,7 +915,7 @@ void QgsLayerTreeModel::recursivelyEmitDataChanged( const QModelIndex& idx )
 
 Qt::DropActions QgsLayerTreeModel::supportedDropActions() const
 {
-  return Qt::MoveAction;
+  return Qt::CopyAction | Qt::MoveAction;
 }
 
 QStringList QgsLayerTreeModel::mimeTypes() const
@@ -885,16 +928,20 @@ QStringList QgsLayerTreeModel::mimeTypes() const
 
 QMimeData* QgsLayerTreeModel::mimeData( const QModelIndexList& indexes ) const
 {
-  QList<QgsLayerTreeNode*> nodesFinal = indexes2nodes( indexes, true );
+  // Sort the indexes. Depending on how the user selected the items, the indexes may be unsorted.
+  QModelIndexList sortedIndexes = indexes;
+  qSort( sortedIndexes.begin(), sortedIndexes.end(), qLess<QModelIndex>() );
 
-  if ( nodesFinal.count() == 0 )
-    return 0;
+  QList<QgsLayerTreeNode*> nodesFinal = indexes2nodes( sortedIndexes, true );
+
+  if ( nodesFinal.isEmpty() )
+    return nullptr;
 
   QMimeData *mimeData = new QMimeData();
 
   QDomDocument doc;
   QDomElement rootElem = doc.createElement( "layer_tree_model_data" );
-  foreach ( QgsLayerTreeNode* node, nodesFinal )
+  Q_FOREACH ( QgsLayerTreeNode* node, nodesFinal )
     node->writeXML( rootElem );
   doc.appendChild( rootElem );
   QString txt = doc.toString();
@@ -940,7 +987,7 @@ bool QgsLayerTreeModel::dropMimeData( const QMimeData* data, Qt::DropAction acti
     elem = elem.nextSiblingElement();
   }
 
-  if ( nodes.count() == 0 )
+  if ( nodes.isEmpty() )
     return false;
 
   if ( parent.isValid() && row == -1 )
@@ -962,7 +1009,7 @@ bool QgsLayerTreeModel::removeRows( int row, int count, const QModelIndex& paren
   return false;
 }
 
-void QgsLayerTreeModel::setFlags( QgsLayerTreeModel::Flags f )
+void QgsLayerTreeModel::setFlags( const QgsLayerTreeModel::Flags& f )
 {
   mFlags = f;
 }
@@ -1001,22 +1048,23 @@ QList<QgsLayerTreeModelLegendNode*> QgsLayerTreeModel::filterLegendNodes( const
 
   if ( mLegendFilterByScale > 0 )
   {
-    foreach ( QgsLayerTreeModelLegendNode* node, nodes )
+    Q_FOREACH ( QgsLayerTreeModelLegendNode* node, nodes )
     {
       if ( node->isScaleOK( mLegendFilterByScale ) )
         filtered << node;
     }
   }
-  else if ( mLegendFilterByMapSettings )
+  else if ( mLegendFilterMapSettings )
   {
-    foreach ( QgsLayerTreeModelLegendNode* node, nodes )
+    Q_FOREACH ( QgsLayerTreeModelLegendNode* node, nodes )
     {
-      QgsSymbolV2* ruleKey = ( QgsSymbolV2* ) node->data( QgsSymbolV2LegendNode::SymbolV2LegacyRuleKeyRole ).value<void*>();
-      if ( ruleKey )
+      QgsSymbolV2* symbolKey = reinterpret_cast< QgsSymbolV2* >( node->data( QgsSymbolV2LegendNode::SymbolV2LegacyRuleKeyRole ).value<void*>() );
+      if ( symbolKey )
       {
+        QString ruleKey = node->data( QgsSymbolV2LegendNode::RuleKeyRole ).toString();
         if ( QgsVectorLayer* vl = qobject_cast<QgsVectorLayer*>( node->layerNode()->layer() ) )
         {
-          if ( mLegendFilterByMapHitTest->symbolsForLayer( vl ).contains( ruleKey ) )
+          if ( mLegendFilterHitTest->legendKeyVisible( ruleKey, vl ) )
             filtered << node;
         }
       }
@@ -1039,7 +1087,7 @@ QList<QgsLayerTreeModelLegendNode*> QgsLayerTreeModel::filterLegendNodes( const
 
 void QgsLayerTreeModel::legendCleanup()
 {
-  foreach ( const LayerLegendData& data, mLegend )
+  Q_FOREACH ( const LayerLegendData& data, mLegend )
   {
     qDeleteAll( data.originalNodes );
     delete data.tree;
@@ -1064,10 +1112,15 @@ void QgsLayerTreeModel::addLegendToLayer( QgsLayerTreeLayer* nodeL )
   if ( !nodeL->layer() )
     return;
 
-  QgsMapLayerLegend* layerLegend = nodeL->layer()->legend();
+  QgsMapLayer* ml = nodeL->layer();
+  QgsMapLayerLegend* layerLegend = ml->legend();
   if ( !layerLegend )
     return;
 
+  bool hasStyleOverride = mLayerStyleOverrides.contains( ml->id() );
+  if ( hasStyleOverride )
+    ml->styleManager()->setOverrideStyle( mLayerStyleOverrides.value( ml->id() ) );
+
   QList<QgsLayerTreeModelLegendNode*> lstNew = layerLegend->createLayerTreeModelLegendNodes( nodeL );
 
   // apply filtering defined in layer node's custom properties (reordering, filtering, custom labels)
@@ -1077,7 +1130,7 @@ void QgsLayerTreeModel::addLegendToLayer( QgsLayerTreeLayer* nodeL )
 
   bool isEmbedded = filteredLstNew.count() == 1 && filteredLstNew[0]->isEmbeddedInParent();
 
-  foreach ( QgsLayerTreeModelLegendNode* n, lstNew )
+  Q_FOREACH ( QgsLayerTreeModelLegendNode* n, lstNew )
   {
     n->setParent( this );
     connect( n, SIGNAL( dataChanged() ), this, SLOT( legendNodeDataChanged() ) );
@@ -1086,19 +1139,26 @@ void QgsLayerTreeModel::addLegendToLayer( QgsLayerTreeLayer* nodeL )
   LayerLegendData data;
   data.originalNodes = lstNew;
   data.activeNodes = filteredLstNew;
-  data.tree = 0;
+  data.tree = nullptr;
 
   // maybe the legend nodes form a tree - try to create a tree structure from the list
   if ( testFlag( ShowLegendAsTree ) )
     tryBuildLegendTree( data );
 
-  int count = data.tree ? data.tree->children[0].count() : filteredLstNew.count();
+  int count = data.tree ? data.tree->children[nullptr].count() : filteredLstNew.count();
 
   if ( ! isEmbedded ) beginInsertRows( node2index( nodeL ), 0, count - 1 );
 
   mLegend[nodeL] = data;
 
   if ( ! isEmbedded ) endInsertRows();
+
+  if ( hasStyleOverride )
+    ml->styleManager()->restoreOverrideStyle();
+
+  // invalidate map based data even if the data is not map-based to make sure
+  // the symbol sizes are computed at least once
+  legendInvalidateMapBasedData();
 }
 
 
@@ -1106,7 +1166,7 @@ void QgsLayerTreeModel::tryBuildLegendTree( LayerLegendData& data )
 {
   // first check whether there are any legend nodes that are not top-level
   bool hasParentKeys = false;
-  foreach ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
+  Q_FOREACH ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
   {
     if ( !n->data( QgsLayerTreeModelLegendNode::ParentRuleKeyRole ).toString().isEmpty() )
     {
@@ -1119,8 +1179,8 @@ void QgsLayerTreeModel::tryBuildLegendTree( LayerLegendData& data )
 
   // make mapping from rules to nodes and do some sanity checks
   QHash<QString, QgsLayerTreeModelLegendNode*> rule2node;
-  rule2node[QString()] = 0;
-  foreach ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
+  rule2node[QString()] = nullptr;
+  Q_FOREACH ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
   {
     QString ruleKey = n->data( QgsLayerTreeModelLegendNode::RuleKeyRole ).toString();
     if ( ruleKey.isEmpty() ) // in tree all nodes must have key
@@ -1132,10 +1192,10 @@ void QgsLayerTreeModel::tryBuildLegendTree( LayerLegendData& data )
 
   // create the tree structure
   data.tree = new LayerLegendTree;
-  foreach ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
+  Q_FOREACH ( QgsLayerTreeModelLegendNode* n, data.activeNodes )
   {
     QString parentRuleKey = n->data( QgsLayerTreeModelLegendNode::ParentRuleKeyRole ).toString();
-    QgsLayerTreeModelLegendNode* parent = rule2node.value( parentRuleKey, 0 );
+    QgsLayerTreeModelLegendNode* parent = rule2node.value( parentRuleKey, nullptr );
     data.tree->parents[n] = parent;
     data.tree->children[parent] << n;
   }
@@ -1162,7 +1222,7 @@ QModelIndex QgsLayerTreeModel::legendNode2index( QgsLayerTreeModelLegendNode* le
     else
     {
       QModelIndex parentIndex = node2index( legendNode->layerNode() );
-      int row = data.tree->children[0].indexOf( legendNode );
+      int row = data.tree->children[nullptr].indexOf( legendNode );
       return index( row, 0, parentIndex );
     }
   }
@@ -1196,7 +1256,7 @@ int QgsLayerTreeModel::legendRootRowCount( QgsLayerTreeLayer* nL ) const
 
   const LayerLegendData& data = mLegend[nL];
   if ( data.tree )
-    return data.tree->children[0].count();
+    return data.tree->children[nullptr].count();
 
   return data.activeNodes.count();
 }
@@ -1207,7 +1267,7 @@ QModelIndex QgsLayerTreeModel::legendRootIndex( int row, int column, QgsLayerTre
   Q_ASSERT( mLegend.contains( nL ) );
   const LayerLegendData& data = mLegend[nL];
   if ( data.tree )
-    return createIndex( row, column, static_cast<QObject*>( data.tree->children[0].at( row ) ) );
+    return createIndex( row, column, static_cast<QObject*>( data.tree->children[nullptr].at( row ) ) );
 
   return createIndex( row, column, static_cast<QObject*>( data.activeNodes.at( row ) ) );
 }
@@ -1278,13 +1338,80 @@ QList<QgsLayerTreeModelLegendNode*> QgsLayerTreeModel::layerLegendNodes( QgsLaye
   return mLegend.value( nodeLayer ).activeNodes;
 }
 
+QList<QgsLayerTreeModelLegendNode*> QgsLayerTreeModel::layerOriginalLegendNodes( QgsLayerTreeLayer* nodeLayer )
+{
+  return mLegend.value( nodeLayer ).originalNodes;
+}
+
+QgsLayerTreeModelLegendNode* QgsLayerTreeModel::findLegendNode( const QString& layerId, const QString& ruleKey ) const
+{
+  QMap<QgsLayerTreeLayer*, LayerLegendData>::const_iterator it = mLegend.constBegin();
+  for ( ; it != mLegend.constEnd(); ++it )
+  {
+    QgsLayerTreeLayer* layer = it.key();
+    if ( layer->layerId() == layerId )
+    {
+      Q_FOREACH ( QgsLayerTreeModelLegendNode* legendNode, mLegend.value( layer ).activeNodes )
+      {
+        if ( legendNode->data( QgsLayerTreeModelLegendNode::RuleKeyRole ).toString() == ruleKey )
+        {
+          //found it!
+          return legendNode;
+        }
+      }
+    }
+  }
+
+  return nullptr;
+}
+
 void QgsLayerTreeModel::legendInvalidateMapBasedData()
 {
-  foreach ( const LayerLegendData& data, mLegend )
+  if ( !testFlag( DeferredLegendInvalidation ) )
+    invalidateLegendMapBasedData();
+  else
+    mDeferLegendInvalidationTimer.start( 1000 );
+}
+
+void QgsLayerTreeModel::invalidateLegendMapBasedData()
+{
+  QgsDebugCall;
+
+  // we have varying icon sizes, and we want icon to be centered and
+  // text to be left aligned, so we have to compute the max width of icons
+  //
+  // we do that for nodes who share a common parent
+  //
+  // we do that here because for symbols with size defined in map units
+  // the symbol sizes changes depends on the zoom level
+
+  Q_FOREACH ( const LayerLegendData& data, mLegend )
   {
-    foreach ( QgsLayerTreeModelLegendNode* legendNode, data.originalNodes )
+    QList<QgsSymbolV2LegendNode*> symbolNodes;
+    QMap<QString, int> widthMax;
+    Q_FOREACH ( QgsLayerTreeModelLegendNode* legendNode, data.originalNodes )
+    {
+      QgsSymbolV2LegendNode* n = dynamic_cast<QgsSymbolV2LegendNode*>( legendNode );
+      if ( n )
+      {
+        const QSize sz( n->minimumIconSize() );
+        const QString parentKey( n->data( QgsLayerTreeModelLegendNode::ParentRuleKeyRole ).toString() );
+        widthMax[parentKey] = qMax( sz.width(), widthMax.contains( parentKey ) ? widthMax[parentKey] : 0 );
+        n->setIconSize( sz );
+        symbolNodes.append( n );
+      }
+    }
+    Q_FOREACH ( QgsSymbolV2LegendNode* n, symbolNodes )
+    {
+      const QString parentKey( n->data( QgsLayerTreeModelLegendNode::ParentRuleKeyRole ).toString() );
+      Q_ASSERT( widthMax[parentKey] > 0 );
+      const int twiceMarginWidth = 2; // a one pixel margin avoids hugly rendering of icon
+      n->setIconSize( QSize( widthMax[parentKey] + twiceMarginWidth, n->iconSize().rheight() + twiceMarginWidth ) );
+    }
+    Q_FOREACH ( QgsLayerTreeModelLegendNode* legendNode, data.originalNodes )
       legendNode->invalidateMapBasedData();
   }
+
 }
 
 // Legend nodes routines - end
