@@ -14,31 +14,15 @@
  *    - 2008 Portability issues fixed by Maxence Laurent
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef _MSC_VER
-#ifndef _CRT_SECURE_NO_DEPRECATE
-#define _CRT_SECURE_NO_DEPRECATE
-#endif
-#endif
-
 #ifndef RTREE_H
 #define RTREE_H
 
 #include <cstdio>
 #include <cmath>
 #include <cassert>
-#include <cstdlib>
+#include <QtGlobal>
 
 #define ASSERT assert // RTree uses ASSERT( condition )
-#ifndef Min
-#define Min(a,b) (a<b?a:b)
-#endif //Min
-#ifndef Max
-#define Max(a,b) (a>b?a:b)
-#endif //Max
 
 //
 // RTree.h
@@ -379,7 +363,7 @@ namespace pal
 
       RTFileStream()
       {
-        m_file = NULL;
+        m_file = nullptr;
       }
 
       ~RTFileStream()
@@ -389,6 +373,9 @@ namespace pal
 
       bool OpenRead( const char* a_fileName )
       {
+        if ( m_file )
+          fclose( m_file );
+
         m_file = fopen( a_fileName, "rb" );
         if ( !m_file )
         {
@@ -399,6 +386,9 @@ namespace pal
 
       bool OpenWrite( const char* a_fileName )
       {
+        if ( m_file )
+          fclose( m_file );
+
         m_file = fopen( a_fileName, "wb" );
         if ( !m_file )
         {
@@ -412,7 +402,7 @@ namespace pal
         if ( m_file )
         {
           fclose( m_file );
-          m_file = NULL;
+          m_file = nullptr;
         }
       }
 
@@ -420,28 +410,28 @@ namespace pal
       size_t Write( const TYPE& a_value )
       {
         ASSERT( m_file );
-        return fwrite(( void* ) &a_value, sizeof( a_value ), 1, m_file );
+        return fwrite( static_cast< void* >( &a_value ), sizeof( a_value ), 1, m_file );
       }
 
       template< typename TYPE >
       size_t WriteArray( const TYPE* a_array, int a_count )
       {
         ASSERT( m_file );
-        return fwrite(( void* ) a_array, sizeof( TYPE ) * a_count, 1, m_file );
+        return fwrite( static_cast< void* >( a_array ), sizeof( TYPE ) * a_count, 1, m_file );
       }
 
       template< typename TYPE >
       size_t Read( TYPE& a_value )
       {
         ASSERT( m_file );
-        return fread(( void* ) &a_value, sizeof( a_value ), 1, m_file );
+        return fread( static_cast< void* >( &a_value ), sizeof( a_value ), 1, m_file );
       }
 
       template< typename TYPE >
       size_t ReadArray( TYPE* a_array, int a_count )
       {
         ASSERT( m_file );
-        return fread(( void* ) a_array, sizeof( TYPE ) * a_count, 1, m_file );
+        return fread( static_cast< void* >( a_array ), sizeof( TYPE ) * a_count, 1, m_file );
       }
   };
 
@@ -471,7 +461,7 @@ namespace pal
 
     m_root = AllocNode();
     m_root->m_level = 0;
-    m_unitSphereVolume = ( ELEMTYPEREAL ) UNIT_SPHERE_VOLUMES[NUMDIMS];
+    m_unitSphereVolume = static_cast< ELEMTYPEREAL >( UNIT_SPHERE_VOLUMES[NUMDIMS] );
   }
 
 
@@ -869,8 +859,8 @@ namespace pal
   {
     for ( int index = 0; index < NUMDIMS; ++index )
     {
-      a_rect->m_min[index] = ( ELEMTYPE ) 0;
-      a_rect->m_max[index] = ( ELEMTYPE ) 0;
+      a_rect->m_min[index] = static_cast< ELEMTYPE >( 0 );
+      a_rect->m_max[index] = static_cast< ELEMTYPE >( 0 );
     }
   }
 
@@ -913,7 +903,7 @@ namespace pal
     else if ( a_node->m_level == a_level ) // Have reached level for insertion. Add rect, split if necessary
     {
       branch.m_rect = *a_rect;
-      branch.m_child = ( Node* ) a_id;
+      branch.m_child = reinterpret_cast< Node* >( a_id );
       // Child field of leaves contains id of data record
       return AddBranch( &branch, a_node, a_newNode );
     }
@@ -955,10 +945,10 @@ namespace pal
       newRoot->m_level = ( *a_root )->m_level + 1;
       branch.m_rect = NodeCover( *a_root );
       branch.m_child = *a_root;
-      AddBranch( &branch, newRoot, NULL );
+      AddBranch( &branch, newRoot, nullptr );
       branch.m_rect = NodeCover( newNode );
       branch.m_child = newNode;
-      AddBranch( &branch, newRoot, NULL );
+      AddBranch( &branch, newRoot, nullptr );
       *a_root = newRoot;
       return true;
     }
@@ -973,7 +963,7 @@ namespace pal
   {
     ASSERT( a_node );
 
-    int firstTime = true;
+    bool firstTime = true;
     Rect rect;
     InitRect( &rect );
 
@@ -1048,7 +1038,7 @@ namespace pal
 
     bool firstTime = true;
     ELEMTYPEREAL increase;
-    ELEMTYPEREAL bestIncr = ( ELEMTYPEREAL ) - 1;
+    ELEMTYPEREAL bestIncr = static_cast< ELEMTYPEREAL >( -1 );
     ELEMTYPEREAL area;
     ELEMTYPEREAL bestArea =  0;
     int best = 0;
@@ -1067,7 +1057,7 @@ namespace pal
         bestIncr = increase;
         firstTime = false;
       }
-      else if (( increase == bestIncr ) && ( area < bestArea ) )
+      else if ( qgsDoubleNear( increase, bestIncr ) && ( area < bestArea ) )
       {
         best = index;
         bestArea = area;
@@ -1088,8 +1078,8 @@ namespace pal
 
     for ( int index = 0; index < NUMDIMS; ++index )
     {
-      newRect.m_min[index] = Min( a_rectA->m_min[index], a_rectB->m_min[index] );
-      newRect.m_max[index] = Max( a_rectA->m_max[index], a_rectB->m_max[index] );
+      newRect.m_min[index] = qMin( a_rectA->m_min[index], a_rectB->m_min[index] );
+      newRect.m_max[index] = qMax( a_rectA->m_max[index], a_rectB->m_max[index] );
     }
 
     return newRect;
@@ -1134,14 +1124,14 @@ namespace pal
   {
     ASSERT( a_rect );
 
-    ELEMTYPEREAL volume = ( ELEMTYPEREAL ) 1;
+    ELEMTYPEREAL volume = static_cast< ELEMTYPEREAL >( 1 );
 
     for ( int index = 0; index < NUMDIMS; ++index )
     {
       volume *= a_rect->m_max[index] - a_rect->m_min[index];
     }
 
-    ASSERT( volume >= ( ELEMTYPEREAL ) 0 );
+    ASSERT( volume >= static_cast< ELEMTYPEREAL >( 0 ) );
 
     return volume;
   }
@@ -1153,16 +1143,16 @@ namespace pal
   {
     ASSERT( a_rect );
 
-    ELEMTYPEREAL sumOfSquares = ( ELEMTYPEREAL ) 0;
+    ELEMTYPEREAL sumOfSquares = static_cast< ELEMTYPEREAL >( 0 );
     ELEMTYPEREAL radius;
 
     for ( int index = 0; index < NUMDIMS; ++index )
     {
-      ELEMTYPEREAL halfExtent = (( ELEMTYPEREAL ) a_rect->m_max[index] - ( ELEMTYPEREAL ) a_rect->m_min[index] ) * 0.5f;
+      ELEMTYPEREAL halfExtent = ( static_cast< ELEMTYPEREAL >( a_rect->m_max[index] ) - static_cast< ELEMTYPEREAL >( a_rect->m_min[index] ) ) * 0.5f;
       sumOfSquares += halfExtent * halfExtent;
     }
 
-    radius = ( ELEMTYPEREAL ) sqrt( sumOfSquares );
+    radius = static_cast< ELEMTYPEREAL >( sqrt( sumOfSquares ) );
 
     // Pow maybe slow, so test for common dims like 2,3 and just use x*x, x*x*x.
     if ( NUMDIMS == 3 )
@@ -1175,7 +1165,7 @@ namespace pal
     }
     else
     {
-      return ( ELEMTYPEREAL )( pow( radius, NUMDIMS ) * m_unitSphereVolume );
+      return static_cast< ELEMTYPEREAL >( pow( radius, NUMDIMS ) * m_unitSphereVolume );
     }
   }
 
@@ -1248,7 +1238,7 @@ namespace pal
            && ( a_parVars->m_count[0] < ( a_parVars->m_total - a_parVars->m_minFill ) )
            && ( a_parVars->m_count[1] < ( a_parVars->m_total - a_parVars->m_minFill ) ) )
     {
-      biggestDiff = ( ELEMTYPEREAL ) - 1;
+      biggestDiff = static_cast< ELEMTYPEREAL >( -1 );
       for ( int index = 0; index < a_parVars->m_total; ++index )
       {
         if ( !a_parVars->m_taken[index] )
@@ -1275,7 +1265,7 @@ namespace pal
             chosen = index;
             betterGroup = group;
           }
-          else if (( diff == biggestDiff ) && ( a_parVars->m_count[group] < a_parVars->m_count[betterGroup] ) )
+          else if ( qgsDoubleNear( diff, biggestDiff ) && ( a_parVars->m_count[group] < a_parVars->m_count[betterGroup] ) )
           {
             chosen = index;
             betterGroup = group;
@@ -1325,11 +1315,11 @@ namespace pal
 
       if ( a_parVars->m_partition[index] == 0 )
       {
-        AddBranch( &a_parVars->m_branchBuf[index], a_nodeA, NULL );
+        AddBranch( &a_parVars->m_branchBuf[index], a_nodeA, nullptr );
       }
       else if ( a_parVars->m_partition[index] == 1 )
       {
-        AddBranch( &a_parVars->m_branchBuf[index], a_nodeB, NULL );
+        AddBranch( &a_parVars->m_branchBuf[index], a_nodeB, nullptr );
       }
     }
   }
@@ -1342,7 +1332,7 @@ namespace pal
     ASSERT( a_parVars );
 
     a_parVars->m_count[0] = a_parVars->m_count[1] = 0;
-    a_parVars->m_area[0] = a_parVars->m_area[1] = ( ELEMTYPEREAL ) 0;
+    a_parVars->m_area[0] = a_parVars->m_area[1] = static_cast< ELEMTYPEREAL >( 0 );
     a_parVars->m_total = a_maxRects;
     a_parVars->m_minFill = a_minFill;
     for ( int index = 0; index < a_maxRects; ++index )
@@ -1419,7 +1409,7 @@ namespace pal
     ASSERT( *a_root );
 
     Node* tempNode;
-    ListNode* reInsertList = NULL;
+    ListNode* reInsertList = nullptr;
 
     if ( !RemoveRectRec( a_rect, a_id, *a_root, &reInsertList ) )
     {
@@ -1501,7 +1491,7 @@ namespace pal
     {
       for ( int index = 0; index < a_node->m_count; ++index )
       {
-        if ( a_node->m_branch[index].m_child == ( Node* ) a_id )
+        if ( a_node->m_branch[index].m_child == reinterpret_cast< Node* >( a_id ) )
         {
           DisconnectBranch( a_node, index ); // Must return after this call as count has changed
           return false;
