@@ -22,6 +22,7 @@
 #include "qgis.h"
 #include "qgspoint.h"
 #include "qgsrectangle.h"
+#include "qgswkbptr.h"
 
 #include <QVector>
 #include <QPolygonF>
@@ -65,26 +66,28 @@ class CORE_EXPORT QgsClipper
     static const double MIN_Y;
 
 
-    // A handy way to refer to the four boundaries
+    //! A handy way to refer to the four boundaries
     enum Boundary {XMax, XMin, YMax, YMin};
 
-    // Trims the given feature to a rectangular box. Returns the trimmed
-    // feature in x and y. The shapeOpen parameter determines whether
-    // the function treats the points as a closed shape (polygon), or as
-    // an open shape (linestring).
-    //
-    // @note not available in python bindings on android
+    /**
+     * Trims the given feature to a rectangular box. Returns the trimmed
+     * feature in x and y. The shapeOpen parameter determines whether
+     * the function treats the points as a closed shape (polygon), or as
+     * an open shape (linestring).
+     *
+     * @note not available in python bindings on android
+     */
     static void trimFeature( QVector<double>& x,
                              QVector<double>& y,
                              bool shapeOpen );
 
     static void trimPolygon( QPolygonF& pts, const QgsRectangle& clipRect );
 
-    /**Reads a polyline from WKB and clips it to clipExtent
+    /** Reads a polyline from WKB and clips it to clipExtent
       @param wkb pointer to the start of the line wkb
       @param clipExtent clipping bounds
       @param line out: clipped line coordinates*/
-    static const unsigned char* clippedLineWKB( const unsigned char* wkb, const QgsRectangle& clipExtent, QPolygonF& line );
+    static QgsConstWkbPtr clippedLineWKB( QgsConstWkbPtr wkb, const QgsRectangle& clipExtent, QPolygonF& line );
 
   private:
 
@@ -105,7 +108,7 @@ class CORE_EXPORT QgsClipper
     // Determines if a point is inside or outside the given boundary
     static bool inside( const double x, const double y, Boundary b );
 
-    static bool inside( const QPointF& pt, Boundary b, double val );
+    static bool inside( QPointF pt, Boundary b, double val );
 
     // Calculates the intersection point between a line defined by a
     // (x1, y1), and (x2, y2) and the given boundary
@@ -113,14 +116,14 @@ class CORE_EXPORT QgsClipper
                                const double x2, const double y2,
                                Boundary b );
 
-    static QPointF intersectRect( const QPointF& pt1,
-                                  const QPointF& pt2,
+    static QPointF intersectRect( QPointF pt1,
+                                  QPointF pt2,
                                   Boundary b, const QgsRectangle& rect );
 
     //Implementation of 'Fast clipping' algorithm (Sobkow et al. 1987, Computers & Graphics Vol.11, 4, p.459-467)
     static bool clipLineSegment( double xLeft, double xRight, double yBottom, double yTop, double& x0, double& y0, double& x1, double& y1 );
 
-    /**Connects two lines split by the clip (by inserting points on the clip border)
+    /** Connects two lines split by the clip (by inserting points on the clip border)
       @param x0 x-coordinate of the first line end
       @param y0 y-coordinate of the first line end
       @param x1 x-coordinate of the second line start
@@ -132,14 +135,14 @@ class CORE_EXPORT QgsClipper
                                        const QgsRectangle& clipRect, QPolygonF& pts );
 
     //low level clip methods for fast clip algorithm
-    static void clipStartTop( double& x0, double& y0, const double& x1, const double& y1, double yMax );
-    static void clipStartBottom( double& x0, double& y0, const double& x1, const double& y1, double yMin );
-    static void clipStartRight( double& x0, double& y0, const double& x1, const double& y1, double xMax );
-    static void clipStartLeft( double& x0, double& y0, const double& x1, const double& y1, double xMin );
-    static void clipEndTop( const double& x0, const double& y0, double& x1, double& y1, double yMax );
-    static void clipEndBottom( const double& x0, const double& y0, double& x1, double& y1, double yMin );
-    static void clipEndRight( const double& x0, const double& y0, double& x1, double& y1, double xMax );
-    static void clipEndLeft( const double& x0, const double& y0, double& x1, double& y1, double xMin );
+    static void clipStartTop( double& x0, double& y0, double x1, double y1, double yMax );
+    static void clipStartBottom( double& x0, double& y0, double x1, double y1, double yMin );
+    static void clipStartRight( double& x0, double& y0, double x1, double y1, double xMax );
+    static void clipStartLeft( double& x0, double& y0, double& x1, double& y1, double xMin );
+    static void clipEndTop( double x0, double y0, double& x1, double& y1, double yMax );
+    static void clipEndBottom( double x0, double y0, double& x1, double& y1, double yMin );
+    static void clipEndRight( double x0, double y0, double& x1, double& y1, double xMax );
+    static void clipEndLeft( double x0, double y0, double& x1, double& y1, double xMin );
 };
 
 // The inline functions
@@ -320,7 +323,7 @@ inline bool QgsClipper::inside( const double x, const double y, Boundary b )
   return false;
 }
 
-inline bool QgsClipper::inside( const QPointF& pt, Boundary b, double val )
+inline bool QgsClipper::inside( QPointF pt, Boundary b, double val )
 {
   switch ( b )
   {
@@ -388,8 +391,8 @@ inline QgsPoint QgsClipper::intersect( const double x1, const double y1,
   return p;
 }
 
-inline QPointF QgsClipper::intersectRect( const QPointF& pt1,
-    const QPointF& pt2,
+inline QPointF QgsClipper::intersectRect( QPointF pt1,
+    QPointF pt2,
     Boundary b, const QgsRectangle& rect )
 {
   // This function assumes that the two given points (x1, y1), and
@@ -428,49 +431,49 @@ inline QPointF QgsClipper::intersectRect( const QPointF& pt1,
   return QPointF( x1 + r*( x2 - x1 ), y1 + r*( y2 - y1 ) );
 }
 
-inline void QgsClipper::clipStartTop( double& x0, double& y0, const double& x1, const double& y1, double yMax )
+inline void QgsClipper::clipStartTop( double& x0, double& y0, double x1, double y1, double yMax )
 {
   x0 += ( x1 - x0 )  * ( yMax - y0 ) / ( y1 - y0 );
   y0 = yMax;
 }
 
-inline void QgsClipper::clipStartBottom( double& x0, double& y0, const double& x1, const double& y1, double yMin )
+inline void QgsClipper::clipStartBottom( double& x0, double& y0, double x1, double y1, double yMin )
 {
   x0 += ( x1 - x0 ) * ( yMin - y0 ) / ( y1 - y0 );
   y0 = yMin;
 }
 
-inline void QgsClipper::clipStartRight( double& x0, double& y0, const double& x1, const double& y1, double xMax )
+inline void QgsClipper::clipStartRight( double& x0, double& y0, double x1, double y1, double xMax )
 {
   y0 += ( y1 - y0 ) * ( xMax - x0 ) / ( x1 - x0 );
   x0 = xMax;
 }
 
-inline void QgsClipper::clipStartLeft( double& x0, double& y0, const double& x1, const double& y1, double xMin )
+inline void QgsClipper::clipStartLeft( double& x0, double& y0, double& x1, double& y1, double xMin )
 {
   y0 += ( y1 - y0 ) * ( xMin - x0 ) / ( x1 - x0 );
   x0 = xMin;
 }
 
-inline void QgsClipper::clipEndTop( const double& x0, const double& y0, double& x1, double& y1, double yMax )
+inline void QgsClipper::clipEndTop( double x0, double y0, double& x1, double& y1, double yMax )
 {
   x1 += ( x1 - x0 ) * ( yMax - y1 ) / ( y1 - y0 );
   y1 = yMax;
 }
 
-inline void QgsClipper::clipEndBottom( const double& x0, const double& y0, double& x1, double& y1, double yMin )
+inline void QgsClipper::clipEndBottom( double x0, double y0, double& x1, double& y1, double yMin )
 {
   x1 += ( x1 - x0 ) * ( yMin - y1 ) / ( y1 - y0 );
   y1 = yMin;
 }
 
-inline void QgsClipper::clipEndRight( const double& x0, const double& y0, double& x1, double& y1, double xMax )
+inline void QgsClipper::clipEndRight( double x0, double y0, double& x1, double& y1, double xMax )
 {
   y1 += ( y1 - y0 ) * ( xMax - x1 ) / ( x1 - x0 );
   x1 = xMax;
 }
 
-inline void QgsClipper::clipEndLeft( const double& x0, const double& y0, double& x1, double& y1, double xMin )
+inline void QgsClipper::clipEndLeft( double x0, double y0, double& x1, double& y1, double xMin )
 {
   y1 += ( y1 - y0 ) * ( xMin - x1 ) / ( x1 - x0 );
   x1 = xMin;
