@@ -45,7 +45,7 @@
 #include <cpl_conv.h>
 #include <cpl_csv.h>
 
-CUSTOM_CRS_VALIDATION QgsCoordinateReferenceSystem::mCustomSrsValidation = NULL;
+CUSTOM_CRS_VALIDATION QgsCoordinateReferenceSystem::mCustomSrsValidation = nullptr;
 
 //--------------------------
 
@@ -58,10 +58,10 @@ QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem()
     , mValidationHint( "" )
     , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( NULL );
+  mCRS = OSRNewSpatialReference( nullptr );
 }
 
-QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( QString theDefinition )
+QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QString& theDefinition )
     : mSrsId( 0 )
     , mGeoFlag( false )
     , mMapUnits( QGis::UnknownUnit )
@@ -70,7 +70,7 @@ QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( QString theDefinitio
     , mValidationHint( "" )
     , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( NULL );
+  mCRS = OSRNewSpatialReference( nullptr );
   createFromString( theDefinition );
 }
 
@@ -84,7 +84,7 @@ QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const long theId, Cr
     , mValidationHint( "" )
     , mAxisInverted( false )
 {
-  mCRS = OSRNewSpatialReference( NULL );
+  mCRS = OSRNewSpatialReference( nullptr );
   createFromId( theId, theType );
 }
 
@@ -144,8 +144,8 @@ bool QgsCoordinateReferenceSystem::createFromString( const QString &theDefinitio
         if ( srsid() == 0 )
         {
           QString myName = QString( " * %1 (%2)" )
-                           .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ) )
-                           .arg( toProj4() );
+                           .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ),
+                                 toProj4() );
           saveAsUserCRS( myName );
         }
       }
@@ -161,8 +161,8 @@ bool QgsCoordinateReferenceSystem::createFromString( const QString &theDefinitio
 bool QgsCoordinateReferenceSystem::createFromUserInput( const QString &theDefinition )
 {
   QString theWkt;
-  char *wkt = NULL;
-  OGRSpatialReferenceH crs = OSRNewSpatialReference( NULL );
+  char *wkt = nullptr;
+  OGRSpatialReferenceH crs = OSRNewSpatialReference( nullptr );
 
   // make sure towgs84 parameter is loaded if using an ESRI definition and gdal >= 1.9
 #if GDAL_VERSION_NUM >= 1900
@@ -198,7 +198,7 @@ void QgsCoordinateReferenceSystem::setupESRIWktFix()
     CPLSetConfigOption( "GDAL_FIX_ESRI_WKT", configNew );
     if ( strcmp( configNew, CPLGetConfigOption( "GDAL_FIX_ESRI_WKT", "" ) ) != 0 )
       QgsLogger::warning( QString( "GDAL_FIX_ESRI_WKT could not be set to %1 : %2" )
-                          .arg( configNew ).arg( CPLGetConfigOption( "GDAL_FIX_ESRI_WKT", "" ) ) );
+                          .arg( configNew, CPLGetConfigOption( "GDAL_FIX_ESRI_WKT", "" ) ) );
     QgsDebugMsg( QString( "set GDAL_FIX_ESRI_WKT : %1" ).arg( configNew ) );
   }
   else
@@ -213,7 +213,7 @@ bool QgsCoordinateReferenceSystem::createFromOgcWmsCrs( QString theCrs )
   QRegExp re( "urn:ogc:def:crs:([^:]+).+([^:]+)", Qt::CaseInsensitive );
   if ( re.exactMatch( theCrs ) )
   {
-    theCrs = re.cap( 1 ) + ":" + re.cap( 2 );
+    theCrs = re.cap( 1 ) + ':' + re.cap( 2 );
   }
   else
   {
@@ -257,7 +257,7 @@ bool QgsCoordinateReferenceSystem::createFromOgcWmsCrs( QString theCrs )
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QgsCoordinateReferenceSystem &srs )
 {
-  mCRS = OSRNewSpatialReference( NULL );
+  mCRS = OSRNewSpatialReference( nullptr );
   *this = srs;
 }
 
@@ -318,7 +318,7 @@ bool QgsCoordinateReferenceSystem::createFromSrsId( long id )
                      "srs_id", QString::number( id ) );
 }
 
-bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString expression, QString value )
+bool QgsCoordinateReferenceSystem::loadFromDb( const QString& db, const QString& expression, const QString& value )
 {
   QgsDebugMsgLevel( "load CRS from " + db + " where " + expression + " is " + value, 3 );
   mIsValidFlag = false;
@@ -357,23 +357,23 @@ bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString expression, Q
 
   QString mySql = "select srs_id,description,projection_acronym,"
                   "ellipsoid_acronym,parameters,srid,auth_name||':'||auth_id,is_geo "
-                  "from tbl_srs where " + expression + "=" + quotedValue( value ) + " order by deprecated";
+                  "from tbl_srs where " + expression + '=' + quotedValue( value ) + " order by deprecated";
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(),
                               mySql.toUtf8().length(),
                               &myPreparedStatement, &myTail );
   // XXX Need to free memory from the error msg if one is set
   if ( myResult == SQLITE_OK && sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
   {
-    mSrsId = QString::fromUtf8(( char * )sqlite3_column_text(
-                                 myPreparedStatement, 0 ) ).toLong();
-    mDescription = QString::fromUtf8(( char * )sqlite3_column_text(
-                                       myPreparedStatement, 1 ) );
-    mProjectionAcronym = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 2 ) );
-    mEllipsoidAcronym = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 3 ) );
-    mProj4 = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 4 ) );
-    mSRID = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 5 ) ).toLong();
-    mAuthId = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 6 ) );
-    mGeoFlag = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 7 ) ).toInt() != 0;
+    mSrsId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
+                                  myPreparedStatement, 0 ) ) ).toLong();
+    mDescription = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text(
+                                        myPreparedStatement, 1 ) ) );
+    mProjectionAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 2 ) ) );
+    mEllipsoidAcronym = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 3 ) ) );
+    mProj4 = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 4 ) ) );
+    mSRID = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 5 ) ) ).toLong() ;
+    mAuthId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 6 ) ) );
+    mGeoFlag = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 7 ) ) ).toInt() != 0;
     mAxisInverted = -1;
 
     if ( mSrsId >= USER_CRS_START_ID && mAuthId.isEmpty() )
@@ -383,7 +383,7 @@ bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString expression, Q
     else if ( mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
     {
       OSRDestroySpatialReference( mCRS );
-      mCRS = OSRNewSpatialReference( NULL );
+      mCRS = OSRNewSpatialReference( nullptr );
       mIsValidFlag = OSRSetFromUserInput( mCRS, mAuthId.toLower().toAscii() ) == OGRERR_NONE;
       setMapUnits();
     }
@@ -412,7 +412,7 @@ bool QgsCoordinateReferenceSystem::axisInverted() const
     // If axis orientation is unknown, try again with OSRImportFromEPSGA for EPSG crs
     if ( orientation == OAO_Other && mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
     {
-      OGRSpatialReferenceH crs = OSRNewSpatialReference( NULL );
+      OGRSpatialReferenceH crs = OSRNewSpatialReference( nullptr );
 
       if ( OSRImportFromEPSGA( crs, mAuthId.mid( 5 ).toInt() ) == OGRERR_NONE )
       {
@@ -436,14 +436,14 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
 
   if ( theWkt.isEmpty() )
   {
-    QgsDebugMsg( "theWkt is uninitialised, operation failed" );
+    QgsDebugMsg( "theWkt is uninitialized, operation failed" );
     return mIsValidFlag;
   }
   QgsDebugMsg( "wkt: " + theWkt );
   QByteArray ba = theWkt.toLatin1();
   const char *pWkt = ba.data();
 
-  OGRErr myInputResult = OSRImportFromWkt( mCRS, ( char ** ) & pWkt );
+  OGRErr myInputResult = OSRImportFromWkt( mCRS, const_cast< char ** >( & pWkt ) );
 
   if ( myInputResult != OGRERR_NONE )
   {
@@ -458,8 +458,8 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
   if ( OSRAutoIdentifyEPSG( mCRS ) == OGRERR_NONE )
   {
     QString authid = QString( "%1:%2" )
-                     .arg( OSRGetAuthorityName( mCRS, NULL ) )
-                     .arg( OSRGetAuthorityCode( mCRS, NULL ) );
+                     .arg( OSRGetAuthorityName( mCRS, nullptr ),
+                           OSRGetAuthorityCode( mCRS, nullptr ) );
     QgsDebugMsg( "authid recognized as " + authid );
     return createFromOgcWmsCrs( authid );
   }
@@ -469,7 +469,7 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
   //myOgrSpatialRef.morphFromESRI();
 
   // create the proj4 structs needed for transforming
-  char *proj4src = NULL;
+  char *proj4src = nullptr;
   OSRExportToProj4( mCRS, &proj4src );
 
   //now that we have the proj4string, delegate to createFromProj4 so
@@ -493,8 +493,8 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &theWkt )
   if ( mSrsId == 0 )
   {
     QString myName = QString( " * %1 (%2)" )
-                     .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ) )
-                     .arg( toProj4() );
+                     .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ),
+                           toProj4() );
     saveAsUserCRS( myName );
   }
 
@@ -622,7 +622,7 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4Strin
     // also with parameters containing spaces (e.g. +nadgrids)
     // make sure result is trimmed (#5598)
     QStringList myParams;
-    foreach ( QString param, myProj4String.split( QRegExp( "\\s+(?=\\+)" ), QString::SkipEmptyParts ) )
+    Q_FOREACH ( const QString& param, myProj4String.split( QRegExp( "\\s+(?=\\+)" ), QString::SkipEmptyParts ) )
     {
       QString arg = QString( "' '||parameters||' ' LIKE %1" ).arg( quotedValue( QString( "% %1 %" ).arg( param.trimmed() ) ) );
       if ( param.startsWith( "+datum=" ) )
@@ -652,7 +652,7 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4Strin
     {
       // Bugfix 8487 : test param lists are equal, except for +datum
       QStringList foundParams;
-      foreach ( QString param, myRecord["parameters"].split( QRegExp( "\\s+(?=\\+)" ), QString::SkipEmptyParts ) )
+      Q_FOREACH ( const QString& param, myRecord["parameters"].split( QRegExp( "\\s+(?=\\+)" ), QString::SkipEmptyParts ) )
       {
         if ( !param.startsWith( "+datum=" ) )
           foundParams << param.trimmed();
@@ -706,7 +706,7 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString &theProj4Strin
 }
 
 //private method meant for internal use by this class only
-QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord( QString theSql )
+QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord( const QString& theSql )
 {
   QString myDatabaseFileName;
   QgsCoordinateReferenceSystem::RecordMap myMap;
@@ -743,8 +743,8 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
     //loop through each column in the record adding its expression name and value to the map
     for ( int myColNo = 0; myColNo < myColumnCount; myColNo++ )
     {
-      myFieldName = QString::fromUtf8(( char * )sqlite3_column_name( myPreparedStatement, myColNo ) );
-      myFieldValue = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, myColNo ) );
+      myFieldName = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_name( myPreparedStatement, myColNo ) ) );
+      myFieldValue = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, myColNo ) ) );
       myMap[myFieldName] = myFieldValue;
     }
     if ( sqlite3_step( myPreparedStatement ) != SQLITE_DONE )
@@ -788,8 +788,8 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
       //loop through each column in the record adding its field name and value to the map
       for ( int myColNo = 0; myColNo < myColumnCount; myColNo++ )
       {
-        myFieldName = QString::fromUtf8(( char * )sqlite3_column_name( myPreparedStatement, myColNo ) );
-        myFieldValue = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, myColNo ) );
+        myFieldName = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_name( myPreparedStatement, myColNo ) ) );
+        myFieldValue = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, myColNo ) ) );
         myMap[myFieldName] = myFieldValue;
       }
 
@@ -879,8 +879,7 @@ QString QgsCoordinateReferenceSystem::toProj4() const
 
   if ( mProj4.isEmpty() )
   {
-    QString toProj4;
-    char *proj4src = NULL;
+    char *proj4src = nullptr;
     OSRExportToProj4( mCRS, &proj4src );
     mProj4 = proj4src;
     CPLFree( proj4src );
@@ -907,7 +906,7 @@ void QgsCoordinateReferenceSystem::setInternalId( long theSrsId )
 {
   mSrsId = theSrsId;
 }
-void QgsCoordinateReferenceSystem::setAuthId( QString authId )
+void QgsCoordinateReferenceSystem::setAuthId( const QString& authId )
 {
   mAuthId = authId;
 }
@@ -915,19 +914,33 @@ void QgsCoordinateReferenceSystem::setSrid( long theSrid )
 {
   mSRID = theSrid;
 }
-void QgsCoordinateReferenceSystem::setDescription( QString theDescription )
+void QgsCoordinateReferenceSystem::setDescription( const QString& theDescription )
 {
   mDescription = theDescription;
 }
-void QgsCoordinateReferenceSystem::setProj4String( QString theProj4String )
+void QgsCoordinateReferenceSystem::setProj4String( const QString& theProj4String )
 {
   mProj4 = theProj4String;
 
   QgsLocaleNumC l;
 
   OSRDestroySpatialReference( mCRS );
-  mCRS = OSRNewSpatialReference( NULL );
+  mCRS = OSRNewSpatialReference( nullptr );
   mIsValidFlag = OSRImportFromProj4( mCRS, theProj4String.trimmed().toLatin1().constData() ) == OGRERR_NONE;
+  // OSRImportFromProj4() may accept strings that are not valid proj.4 strings,
+  // e.g if they lack a +ellps parameter, it will automatically add +ellps=WGS84, but as
+  // we use the original mProj4 with QgsCoordinateTransform, it will fail to initialize
+  // so better detect it now.
+  projPJ theProj = pj_init_plus( theProj4String.trimmed().toLatin1().constData() );
+  if ( !theProj )
+  {
+    QgsDebugMsg( "proj.4 string rejected by pj_init_plus()" );
+    mIsValidFlag = false;
+  }
+  else
+  {
+    pj_free( theProj );
+  }
   mWkt.clear();
   setMapUnits();
 
@@ -943,11 +956,11 @@ void QgsCoordinateReferenceSystem::setEpsg( long theEpsg )
 {
   mAuthId = QString( "EPSG:%1" ).arg( theEpsg );
 }
-void  QgsCoordinateReferenceSystem::setProjectionAcronym( QString theProjectionAcronym )
+void  QgsCoordinateReferenceSystem::setProjectionAcronym( const QString& theProjectionAcronym )
 {
   mProjectionAcronym = theProjectionAcronym;
 }
-void  QgsCoordinateReferenceSystem::setEllipsoidAcronym( QString theEllipsoidAcronym )
+void  QgsCoordinateReferenceSystem::setEllipsoidAcronym( const QString& theEllipsoidAcronym )
 {
   mEllipsoidAcronym = theEllipsoidAcronym;
 }
@@ -1039,8 +1052,8 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
   // needed to populate the list
   QString mySql = QString( "select srs_id,parameters from tbl_srs where "
                            "projection_acronym=%1 and ellipsoid_acronym=%2 order by deprecated" )
-                  .arg( quotedValue( mProjectionAcronym ) )
-                  .arg( quotedValue( mEllipsoidAcronym ) );
+                  .arg( quotedValue( mProjectionAcronym ),
+                        quotedValue( mEllipsoidAcronym ) );
   // Get the full path name to the sqlite3 spatial reference database.
   QString myDatabaseFileName = QgsApplication::srsDbFilePath();
 
@@ -1058,8 +1071,8 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
 
     while ( sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
     {
-      QString mySrsId = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 0 ) );
-      QString myProj4String = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 1 ) );
+      QString mySrsId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 0 ) ) );
+      QString myProj4String = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 1 ) ) );
       if ( toProj4() == myProj4String.trimmed() )
       {
         QgsDebugMsg( "-------> MATCH FOUND in srs.db srsid: " + mySrsId );
@@ -1097,8 +1110,8 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
 
     while ( sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
     {
-      QString mySrsId = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 0 ) );
-      QString myProj4String = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 1 ) );
+      QString mySrsId = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 0 ) ) );
+      QString myProj4String = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 1 ) ) );
       if ( toProj4() == myProj4String.trimmed() )
       {
         QgsDebugMsg( "-------> MATCH FOUND in user qgis.db srsid: " + mySrsId );
@@ -1146,9 +1159,10 @@ QString QgsCoordinateReferenceSystem::toWkt() const
   return mWkt;
 }
 
-bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
+bool QgsCoordinateReferenceSystem::readXML( const QDomNode & theNode )
 {
   QgsDebugMsg( "Reading Spatial Ref Sys from xml ------------------------!" );
+  bool result = true;
   QDomNode srsNode  = theNode.namedItem( "spatialrefsys" );
 
   if ( ! srsNode.isNull() )
@@ -1250,8 +1264,8 @@ bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
       if ( mSrsId == 0 )
       {
         QString myName = QString( " * %1 (%2)" )
-                         .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ) )
-                         .arg( toProj4() );
+                         .arg( QObject::tr( "Generated CRS", "A CRS automatically generated from layer info get this prefix for description" ),
+                               toProj4() );
         saveAsUserCRS( myName );
       }
 
@@ -1261,8 +1275,9 @@ bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
   {
     // Return default CRS if none was found in the XML.
     createFromId( GEOCRS_ID, InternalCrsId );
+    result = false;
   }
-  return true;
+  return result;
 }
 
 bool QgsCoordinateReferenceSystem::writeXML( QDomNode & theNode, QDomDocument & theDoc ) const
@@ -1346,7 +1361,7 @@ QString QgsCoordinateReferenceSystem::proj4FromSrsId( const int theSrsId )
     if ( !myFileInfo.exists() ) //its unlikely that this condition will ever be reached
     {
       QgsDebugMsg( "users qgis.db not found" );
-      return NULL;
+      return nullptr;
     }
   }
   else //must be  a system projection then
@@ -1373,7 +1388,7 @@ QString QgsCoordinateReferenceSystem::proj4FromSrsId( const int theSrsId )
   {
     if ( sqlite3_step( ppStmt ) == SQLITE_ROW )
     {
-      myProjString = QString::fromUtf8(( char* )sqlite3_column_text( ppStmt, 0 ) );
+      myProjString = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( ppStmt, 0 ) ) );
     }
   }
   // close the statement
@@ -1385,11 +1400,11 @@ QString QgsCoordinateReferenceSystem::proj4FromSrsId( const int theSrsId )
   return myProjString;
 }
 
-int QgsCoordinateReferenceSystem::openDb( QString path, sqlite3 **db, bool readonly )
+int QgsCoordinateReferenceSystem::openDb( const QString& path, sqlite3 **db, bool readonly )
 {
   QgsDebugMsgLevel( "path = " + path, 3 );
   int myResult = readonly
-                 ? sqlite3_open_v2( path.toUtf8().data(), db, SQLITE_OPEN_READONLY, NULL )
+                 ? sqlite3_open_v2( path.toUtf8().data(), db, SQLITE_OPEN_READONLY, nullptr )
                  : sqlite3_open( path.toUtf8().data(), db );
 
   if ( myResult != SQLITE_OK )
@@ -1438,7 +1453,7 @@ void QgsCoordinateReferenceSystem::debugPrint()
   }
 }
 
-void QgsCoordinateReferenceSystem::setValidationHint( QString html )
+void QgsCoordinateReferenceSystem::setValidationHint( const QString& html )
 {
   mValidationHint = html;
 }
@@ -1451,7 +1466,7 @@ QString QgsCoordinateReferenceSystem::validationHint()
 /// Copied from QgsCustomProjectionDialog ///
 /// Please refactor into SQL handler !!!  ///
 
-bool QgsCoordinateReferenceSystem::saveAsUserCRS( QString name )
+bool QgsCoordinateReferenceSystem::saveAsUserCRS( const QString& name )
 {
   if ( ! mIsValidFlag )
   {
@@ -1475,19 +1490,19 @@ bool QgsCoordinateReferenceSystem::saveAsUserCRS( QString name )
   {
     mySql = "insert into tbl_srs (srs_id,description,projection_acronym,ellipsoid_acronym,parameters,is_geo) values ("
             + QString::number( USER_CRS_START_ID )
-            + "," + quotedValue( name )
-            + "," + quotedValue( projectionAcronym() )
-            + "," + quotedValue( ellipsoidAcronym() )
-            + "," + quotedValue( toProj4() )
+            + ',' + quotedValue( name )
+            + ',' + quotedValue( projectionAcronym() )
+            + ',' + quotedValue( ellipsoidAcronym() )
+            + ',' + quotedValue( toProj4() )
             + ",0)"; // <-- is_geo shamelessly hard coded for now
   }
   else
   {
     mySql = "insert into tbl_srs (description,projection_acronym,ellipsoid_acronym,parameters,is_geo) values ("
             + quotedValue( name )
-            + "," + quotedValue( projectionAcronym() )
-            + "," + quotedValue( ellipsoidAcronym() )
-            + "," + quotedValue( toProj4() )
+            + ',' + quotedValue( projectionAcronym() )
+            + ',' + quotedValue( ellipsoidAcronym() )
+            + ',' + quotedValue( toProj4() )
             + ",0)"; // <-- is_geo shamelessly hard coded for now
   }
   sqlite3      *myDatabase;
@@ -1499,8 +1514,8 @@ bool QgsCoordinateReferenceSystem::saveAsUserCRS( QString name )
   if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( QString( "Can't open or create database %1: %2" )
-                 .arg( QgsApplication::qgisUserDbFilePath() )
-                 .arg( sqlite3_errmsg( myDatabase ) ) );
+                 .arg( QgsApplication::qgisUserDbFilePath(),
+                       sqlite3_errmsg( myDatabase ) ) );
     return false;
   }
   QgsDebugMsg( QString( "Update or insert sql \n%1" ).arg( mySql ) );
@@ -1540,7 +1555,7 @@ long QgsCoordinateReferenceSystem::getRecordCount()
   int           myResult;
   long          myRecordCount = 0;
   //check the db is available
-  myResult = sqlite3_open_v2( QgsApplication::qgisUserDbFilePath().toUtf8().data(), &myDatabase, SQLITE_OPEN_READONLY, NULL );
+  myResult = sqlite3_open_v2( QgsApplication::qgisUserDbFilePath().toUtf8().data(), &myDatabase, SQLITE_OPEN_READONLY, nullptr );
   if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( QString( "Can't open database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
@@ -1554,7 +1569,7 @@ long QgsCoordinateReferenceSystem::getRecordCount()
   {
     if ( sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
     {
-      QString myRecordCountString = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 0 ) );
+      QString myRecordCountString = QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( myPreparedStatement, 0 ) ) );
       myRecordCount = myRecordCountString.toLong();
     }
   }
@@ -1566,8 +1581,8 @@ long QgsCoordinateReferenceSystem::getRecordCount()
 
 QString QgsCoordinateReferenceSystem::quotedValue( QString value )
 {
-  value.replace( "'", "''" );
-  return value.prepend( "'" ).append( "'" );
+  value.replace( '\'', "''" );
+  return value.prepend( '\'' ).append( '\'' );
 }
 
 // adapted from gdal/ogr/ogr_srs_dict.cpp
@@ -1601,7 +1616,7 @@ bool QgsCoordinateReferenceSystem::loadWkts( QHash<int, QString> &wkts, const ch
     }
     else
     {
-      int pos = line.indexOf( "," );
+      int pos = line.indexOf( ',' );
       if ( pos < 0 )
         return false;
 
@@ -1621,9 +1636,9 @@ bool QgsCoordinateReferenceSystem::loadWkts( QHash<int, QString> &wkts, const ch
 
 bool QgsCoordinateReferenceSystem::loadIDs( QHash<int, QString> &wkts )
 {
-  OGRSpatialReferenceH crs = OSRNewSpatialReference( NULL );
+  OGRSpatialReferenceH crs = OSRNewSpatialReference( nullptr );
 
-  foreach ( QString csv, QStringList() << "gcs.csv" << "pcs.csv" << "vertcs.csv" << "compdcs.csv" << "geoccs.csv" )
+  Q_FOREACH ( const QString& csv, QStringList() << "gcs.csv" << "pcs.csv" << "vertcs.csv" << "compdcs.csv" << "geoccs.csv" )
   {
     QString filename = CPLFindFile( "gdal", csv.toUtf8() );
 
@@ -1642,7 +1657,7 @@ bool QgsCoordinateReferenceSystem::loadIDs( QHash<int, QString> &wkts )
       if ( line.isNull() )
         break;
 
-      int pos = line.indexOf( "," );
+      int pos = line.indexOf( ',' );
       if ( pos < 0 )
         continue;
 
@@ -1656,7 +1671,11 @@ bool QgsCoordinateReferenceSystem::loadIDs( QHash<int, QString> &wkts )
            epsg == 2303 || epsg == 2304 || epsg == 2305 || epsg == 2306 || epsg == 2307 || epsg == 2963 || epsg == 2985 || epsg == 2986 || epsg == 3052 ||
            epsg == 3053 || epsg == 3139 || epsg == 3144 || epsg == 3145 || epsg == 3173 || epsg == 3295 || epsg == 3993 || epsg == 4087 || epsg == 4088 ||
            epsg == 5017 || epsg == 5221 || epsg == 5224 || epsg == 5225 || epsg == 5514 || epsg == 5515 || epsg == 5516 || epsg == 5819 || epsg == 5820 ||
-           epsg == 5821 || epsg == 32600 || epsg == 32663 || epsg == 32700 )
+           epsg == 5821 || epsg == 6200 || epsg == 6201 || epsg == 6202 || epsg == 6244 || epsg == 6245 || epsg == 6246 || epsg == 6247 || epsg == 6248 ||
+           epsg == 6249 || epsg == 6250 || epsg == 6251 || epsg == 6252 || epsg == 6253 || epsg == 6254 || epsg == 6255 || epsg == 6256 || epsg == 6257 ||
+           epsg == 6258 || epsg == 6259 || epsg == 6260 || epsg == 6261 || epsg == 6262 || epsg == 6263 || epsg == 6264 || epsg == 6265 || epsg == 6266 ||
+           epsg == 6267 || epsg == 6268 || epsg == 6269 || epsg == 6270 || epsg == 6271 || epsg == 6272 || epsg == 6273 || epsg == 6274 || epsg == 6275 ||
+           epsg == 32600 || epsg == 32663 || epsg == 32700 )
         continue;
 
       if ( OSRImportFromEPSG( crs, epsg ) != OGRERR_NONE )
@@ -1665,7 +1684,7 @@ bool QgsCoordinateReferenceSystem::loadIDs( QHash<int, QString> &wkts )
         continue;
       }
 
-      char *wkt = 0;
+      char *wkt = nullptr;
       if ( OSRExportToWkt( crs, &wkt ) != OGRERR_NONE )
       {
         qWarning( "EPSG %d: not exported to WKT", epsg );
@@ -1704,22 +1723,22 @@ int QgsCoordinateReferenceSystem::syncDb()
     return -1;
   }
 
-  if ( sqlite3_exec( database, "BEGIN TRANSACTION", 0, 0, 0 ) != SQLITE_OK )
+  if ( sqlite3_exec( database, "BEGIN TRANSACTION", nullptr, nullptr, nullptr ) != SQLITE_OK )
   {
     qCritical( "Could not begin transaction: %s [%s]\n", QgsApplication::srsDbFilePath().toLocal8Bit().constData(), sqlite3_errmsg( database ) );
     return -1;
   }
 
   // fix up database, if not done already //
-  if ( sqlite3_exec( database, "alter table tbl_srs add noupdate boolean", 0, 0, 0 ) == SQLITE_OK )
-    ( void )sqlite3_exec( database, "update tbl_srs set noupdate=(auth_name='EPSG' and auth_id in (5513,5514,5221,2065,102067,4156,4818))", 0, 0, 0 );
+  if ( sqlite3_exec( database, "alter table tbl_srs add noupdate boolean", nullptr, nullptr, nullptr ) == SQLITE_OK )
+    ( void )sqlite3_exec( database, "update tbl_srs set noupdate=(auth_name='EPSG' and auth_id in (5513,5514,5221,2065,102067,4156,4818))", nullptr, 0, 0 );
 
-  ( void )sqlite3_exec( database, "UPDATE tbl_srs SET srid=141001 WHERE srid=41001 AND auth_name='OSGEO' AND auth_id='41001'", 0, 0, 0 );
+  ( void )sqlite3_exec( database, "UPDATE tbl_srs SET srid=141001 WHERE srid=41001 AND auth_name='OSGEO' AND auth_id='41001'", nullptr, 0, 0 );
 
-  OGRSpatialReferenceH crs = OSRNewSpatialReference( NULL );
+  OGRSpatialReferenceH crs = OSRNewSpatialReference( nullptr );
   const char *tail;
   sqlite3_stmt *select;
-  char *errMsg = NULL;
+  char *errMsg = nullptr;
 
   QString proj4;
   QString sql;
@@ -1758,9 +1777,9 @@ int QgsCoordinateReferenceSystem::syncDb()
     QString srsProj4;
     if ( sqlite3_step( select ) == SQLITE_ROW )
     {
-      srsProj4 = ( const char * ) sqlite3_column_text( select, 0 );
+      srsProj4 = reinterpret_cast< const char * >( sqlite3_column_text( select, 0 ) );
 
-      if ( QString::fromUtf8(( char * )sqlite3_column_text( select, 1 ) ).toInt() != 0 )
+      if ( QString::fromUtf8( reinterpret_cast< const char * >( sqlite3_column_text( select, 1 ) ) ).toInt() != 0 )
         continue;
     }
 
@@ -1770,10 +1789,10 @@ int QgsCoordinateReferenceSystem::syncDb()
     {
       if ( proj4 != srsProj4 )
       {
-        errMsg = NULL;
+        errMsg = nullptr;
         sql = QString( "UPDATE tbl_srs SET parameters=%1 WHERE auth_name='EPSG' AND auth_id=%2" ).arg( quotedValue( proj4 ) ).arg( it.key() );
 
-        if ( sqlite3_exec( database, sql.toUtf8(), 0, 0, &errMsg ) != SQLITE_OK )
+        if ( sqlite3_exec( database, sql.toUtf8(), nullptr, nullptr, &errMsg ) != SQLITE_OK )
         {
           qCritical( "Could not execute: %s [%s/%s]\n",
                      sql.toLocal8Bit().constData(),
@@ -1784,7 +1803,7 @@ int QgsCoordinateReferenceSystem::syncDb()
         else
         {
           updated++;
-          QgsDebugMsgLevel( QString( "SQL: %1\n OLD:%2\n NEW:%3" ).arg( sql ).arg( srsProj4 ).arg( proj4 ), 3 );
+          QgsDebugMsgLevel( QString( "SQL: %1\n OLD:%2\n NEW:%3" ).arg( sql, srsProj4, proj4 ), 3 );
         }
       }
     }
@@ -1809,15 +1828,15 @@ int QgsCoordinateReferenceSystem::syncDb()
         name = QObject::tr( "Imported from GDAL" );
 
       sql = QString( "INSERT INTO tbl_srs(description,projection_acronym,ellipsoid_acronym,parameters,srid,auth_name,auth_id,is_geo,deprecated) VALUES (%1,%2,%3,%4,%5,'EPSG',%5,%6,0)" )
-            .arg( quotedValue( name ) )
-            .arg( quotedValue( projRegExp.cap( 1 ) ) )
-            .arg( quotedValue( ellps ) )
-            .arg( quotedValue( proj4 ) )
+            .arg( quotedValue( name ),
+                  quotedValue( projRegExp.cap( 1 ) ),
+                  quotedValue( ellps ),
+                  quotedValue( proj4 ) )
             .arg( it.key() )
             .arg( OSRIsGeographic( crs ) );
 
-      errMsg = NULL;
-      if ( sqlite3_exec( database, sql.toUtf8(), 0, 0, &errMsg ) == SQLITE_OK )
+      errMsg = nullptr;
+      if ( sqlite3_exec( database, sql.toUtf8(), nullptr, nullptr, &errMsg ) == SQLITE_OK )
       {
         inserted++;
       }
@@ -1837,14 +1856,15 @@ int QgsCoordinateReferenceSystem::syncDb()
 
   sql = "DELETE FROM tbl_srs WHERE auth_name='EPSG' AND NOT auth_id IN (";
   QString delim;
-  foreach ( int i, wkts.keys() )
+  QHash<int, QString>::const_iterator it = wkts.constBegin();
+  for ( ; it != wkts.constEnd(); ++it )
   {
-    sql += delim + QString::number( i );
-    delim = ",";
+    sql += delim + QString::number( it.key() );
+    delim = ',';
   }
   sql += ") AND NOT noupdate";
 
-  if ( sqlite3_exec( database, sql.toUtf8(), 0, 0, 0 ) == SQLITE_OK )
+  if ( sqlite3_exec( database, sql.toUtf8(), nullptr, nullptr, nullptr ) == SQLITE_OK )
   {
     deleted = sqlite3_changes( database );
   }
@@ -1862,15 +1882,15 @@ int QgsCoordinateReferenceSystem::syncDb()
   {
     while ( sqlite3_step( select ) == SQLITE_ROW )
     {
-      const char *auth_name = ( const char * ) sqlite3_column_text( select, 0 );
-      const char *auth_id   = ( const char * ) sqlite3_column_text( select, 1 );
-      const char *params    = ( const char * ) sqlite3_column_text( select, 2 );
+      const char *auth_name = reinterpret_cast< const char * >( sqlite3_column_text( select, 0 ) );
+      const char *auth_id   = reinterpret_cast< const char * >( sqlite3_column_text( select, 1 ) );
+      const char *params    = reinterpret_cast< const char * >( sqlite3_column_text( select, 2 ) );
 
-      QString input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toLower() ).arg( auth_id );
+      QString input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toLower(), auth_id );
       projPJ pj = pj_init_plus( input.toAscii() );
       if ( !pj )
       {
-        input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toUpper() ).arg( auth_id );
+        input = QString( "+init=%1:%2" ).arg( QString( auth_name ).toUpper(), auth_id );
         pj = pj_init_plus( input.toAscii() );
       }
 
@@ -1892,14 +1912,14 @@ int QgsCoordinateReferenceSystem::syncDb()
           if ( proj4 != params )
           {
             sql = QString( "UPDATE tbl_srs SET parameters=%1 WHERE auth_name=%2 AND auth_id=%3" )
-                  .arg( quotedValue( proj4 ) )
-                  .arg( quotedValue( auth_name ) )
-                  .arg( quotedValue( auth_id ) );
+                  .arg( quotedValue( proj4 ),
+                        quotedValue( auth_name ),
+                        quotedValue( auth_id ) );
 
-            if ( sqlite3_exec( database, sql.toUtf8(), 0, 0, &errMsg ) == SQLITE_OK )
+            if ( sqlite3_exec( database, sql.toUtf8(), nullptr, nullptr, &errMsg ) == SQLITE_OK )
             {
               updated++;
-              QgsDebugMsgLevel( QString( "SQL: %1\n OLD:%2\n NEW:%3" ).arg( sql ).arg( params ).arg( proj4 ), 3 );
+              QgsDebugMsgLevel( QString( "SQL: %1\n OLD:%2\n NEW:%3" ).arg( sql, params, proj4 ), 3 );
             }
             else
             {
@@ -1935,7 +1955,7 @@ int QgsCoordinateReferenceSystem::syncDb()
 
   OSRDestroySpatialReference( crs );
 
-  if ( sqlite3_exec( database, "COMMIT", 0, 0, 0 ) != SQLITE_OK )
+  if ( sqlite3_exec( database, "COMMIT", nullptr, nullptr, nullptr ) != SQLITE_OK )
   {
     qCritical( "Could not commit transaction: %s [%s]\n", QgsApplication::srsDbFilePath().toLocal8Bit().constData(), sqlite3_errmsg( database ) );
     return -1;
@@ -2027,8 +2047,8 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
 
     if ( i > 0 )
     {
-      insert += ",";
-      values += ",";
+      insert += ',';
+      values += ',';
 
       if ( last )
       {
@@ -2036,7 +2056,7 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
       }
       else
       {
-        update += ",";
+        update += ',';
       }
     }
 
@@ -2046,7 +2066,7 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
     values += QString( "%%1" ).arg( i + 1 );
   }
 
-  insert = "INSERT INTO tbl_datum_transform(" + insert + ") VALUES (" + values + ")";
+  insert = "INSERT INTO tbl_datum_transform(" + insert + ") VALUES (" + values + ')';
 
   QgsDebugMsgLevel( QString( "insert:%1" ).arg( insert ), 4 );
   QgsDebugMsgLevel( QString( "update:%1" ).arg( update ), 4 );
@@ -2066,7 +2086,7 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
     return false;
   }
 
-  if ( sqlite3_exec( db, "BEGIN TRANSACTION", 0, 0, 0 ) != SQLITE_OK )
+  if ( sqlite3_exec( db, "BEGIN TRANSACTION", nullptr, nullptr, nullptr ) != SQLITE_OK )
   {
     qCritical( "Could not begin transaction: %s [%s]\n", QgsApplication::srsDbFilePath().toLocal8Bit().constData(), sqlite3_errmsg( db ) );
     sqlite3_close( db );
@@ -2098,25 +2118,25 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
     }
 
     //switch sign of rotation parameters. See http://trac.osgeo.org/proj/wiki/GenParms#towgs84-DatumtransformationtoWGS84
-    if ( v.at( idxmcode ).compare( QString( "'9607'" ) ) == 0 )
+    if ( v.at( idxmcode ).compare( QLatin1String( "'9607'" ) ) == 0 )
     {
       v[ idxmcode ] = "'9606'";
-      v[ idxrx ] = "'" + qgsDoubleToString( -( v[ idxrx ].remove( "'" ).toDouble() ) ) + "'";
-      v[ idxry ] = "'" + qgsDoubleToString( -( v[ idxry ].remove( "'" ).toDouble() ) ) + "'";
-      v[ idxrz ] = "'" + qgsDoubleToString( -( v[ idxrz ].remove( "'" ).toDouble() ) ) + "'";
+      v[ idxrx ] = '\'' + qgsDoubleToString( -( v[ idxrx ].remove( '\'' ).toDouble() ) ) + '\'';
+      v[ idxry ] = '\'' + qgsDoubleToString( -( v[ idxry ].remove( '\'' ).toDouble() ) ) + '\'';
+      v[ idxrz ] = '\'' + qgsDoubleToString( -( v[ idxrz ].remove( '\'' ).toDouble() ) ) + '\'';
     }
 
     //entry already in db?
     sqlite3_stmt *stmt;
     QString cOpCode;
     QString sql = QString( "SELECT coord_op_code FROM tbl_datum_transform WHERE coord_op_code=%1" ).arg( v[ idxid ] );
-    int prepareRes = sqlite3_prepare( db, sql.toAscii(), sql.size(), &stmt, NULL );
+    int prepareRes = sqlite3_prepare( db, sql.toAscii(), sql.size(), &stmt, nullptr );
     if ( prepareRes != SQLITE_OK )
       continue;
 
     if ( sqlite3_step( stmt ) == SQLITE_ROW )
     {
-      cOpCode = ( const char * ) sqlite3_column_text( stmt, 0 );
+      cOpCode = reinterpret_cast< const char * >( sqlite3_column_text( stmt, 0 ) );
     }
     sqlite3_finalize( stmt );
 
@@ -2126,14 +2146,14 @@ bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString& dbPath )
       sql = sql.arg( v[i] );
     }
 
-    if ( sqlite3_exec( db, sql.toUtf8(), 0, 0, 0 ) != SQLITE_OK )
+    if ( sqlite3_exec( db, sql.toUtf8(), nullptr, nullptr, nullptr ) != SQLITE_OK )
     {
       qCritical( "SQL: %s", sql.toUtf8().constData() );
       qCritical( "Error: %s", sqlite3_errmsg( db ) );
     }
   }
 
-  if ( sqlite3_exec( db, "COMMIT", 0, 0, 0 ) != SQLITE_OK )
+  if ( sqlite3_exec( db, "COMMIT", nullptr, nullptr, nullptr ) != SQLITE_OK )
   {
     qCritical( "Could not commit transaction: %s [%s]\n", QgsApplication::srsDbFilePath().toLocal8Bit().constData(), sqlite3_errmsg( db ) );
     return false;
@@ -2151,7 +2171,7 @@ QString QgsCoordinateReferenceSystem::geographicCRSAuthId() const
   }
   else if ( mCRS )
   {
-    return OSRGetAuthorityName( mCRS, "GEOGCS" ) + QString( ":" ) + OSRGetAuthorityCode( mCRS, "GEOGCS" );
+    return OSRGetAuthorityName( mCRS, "GEOGCS" ) + QLatin1String( ":" ) + OSRGetAuthorityCode( mCRS, "GEOGCS" );
   }
   else
   {
