@@ -19,37 +19,43 @@
 #define QGSCRSCACHE_H
 
 #include "qgscoordinatereferencesystem.h"
-#include "qgssingleton.h"
 #include <QHash>
 
 class QgsCoordinateTransform;
 
-/**Cache coordinate transform by authid of source/dest transformation to avoid the
-overhead of initialisation for each redraw*/
-class CORE_EXPORT QgsCoordinateTransformCache : public QgsSingleton<QgsCoordinateTransformCache>
+/** Cache coordinate transform by authid of source/dest transformation to avoid the
+overhead of initialization for each redraw*/
+class CORE_EXPORT QgsCoordinateTransformCache
 {
   public:
+    static QgsCoordinateTransformCache* instance();
+
     ~QgsCoordinateTransformCache();
-    /**Returns coordinate transformation. Cache keeps ownership
+    /** Returns coordinate transformation. Cache keeps ownership
         @param srcAuthId auth id string of source crs
         @param destAuthId auth id string of dest crs
         @param srcDatumTransform id of source's datum transform
         @param destDatumTransform id of destinations's datum transform
      */
     const QgsCoordinateTransform* transform( const QString& srcAuthId, const QString& destAuthId, int srcDatumTransform = -1, int destDatumTransform = -1 );
-    /**Removes transformations where a changed crs is involved from the cache*/
+    /** Removes transformations where a changed crs is involved from the cache*/
     void invalidateCrs( const QString& crsAuthId );
 
   private:
+    static QgsCoordinateTransformCache* mInstance;
     QMultiHash< QPair< QString, QString >, QgsCoordinateTransform* > mTransforms; //same auth_id pairs might have different datum transformations
+
+    QgsCoordinateTransformCache();
+    QgsCoordinateTransformCache( const QgsCoordinateTransformCache& rh );
+    QgsCoordinateTransformCache& operator=( const QgsCoordinateTransformCache& rh );
 };
 
 class CORE_EXPORT QgsCRSCache
 {
   public:
     static QgsCRSCache* instance();
-    ~QgsCRSCache();
-    /**Returns the CRS for authid, e.g. 'EPSG:4326' (or an invalid CRS in case of error)*/
+
+    /** Returns the CRS for authid, e.g. 'EPSG:4326' (or an invalid CRS in case of error)*/
     const QgsCoordinateReferenceSystem& crsByAuthId( const QString& authid );
     const QgsCoordinateReferenceSystem& crsByEpsgId( long epsg );
 
@@ -60,7 +66,7 @@ class CORE_EXPORT QgsCRSCache
 
   private:
     QHash< QString, QgsCoordinateReferenceSystem > mCRS;
-    /**CRS that is not initialised (returned in case of error)*/
+    /** CRS that is not initialized (returned in case of error)*/
     QgsCoordinateReferenceSystem mInvalidCRS;
 };
 
