@@ -28,6 +28,7 @@ class QgsGeometry;
 class QgsRectangle;
 class QgsFeature;
 class QgsFields;
+class QgsFeaturePrivate;
 
 // feature id class (currently 64 bit)
 #if 0
@@ -64,6 +65,11 @@ class QgsFeatureId
     friend uint qHash( const QgsFeatureId &id );
 };
 
+/** Writes the feature id to stream out. QGIS version compatibility is not guaranteed. */
+CORE_EXPORT QDataStream& operator<<( QDataStream& out, const QgsFeatureId& featureId );
+/** Reads a feature id from stream in into feature id. QGIS version compatibility is not guaranteed. */
+CORE_EXPORT QDataStream& operator>>( QDataStream& in, QgsFeatureId& featureId );
+
 inline uint qHash( const QgsFeatureId &id )
 {
   return qHash( id.mId );
@@ -97,6 +103,12 @@ typedef int QgsFeatureId;
 // key = field index, value = field value
 typedef QMap<int, QVariant> QgsAttributeMap;
 
+/***************************************************************************
+ * This class is considered CRITICAL and any change MUST be accompanied with
+ * full unit tests in testqgsfeature.cpp.
+ * See details in QEP #17
+ ****************************************************************************/
+
 /**
  * A vector of attributes. Mostly equal to QVector<QVariant>.
  */
@@ -106,13 +118,27 @@ class CORE_EXPORT QgsAttributes : public QVector<QVariant>
     QgsAttributes()
         : QVector<QVariant>()
     {}
+    /**
+     * Create a new vector of attributes with the given size
+     *
+     * @param size Number of attributes
+     */
     QgsAttributes( int size )
         : QVector<QVariant>( size )
     {}
+    /**
+     * Constructs a vector with an initial size of size elements. Each element is initialized with value.
+     * @param size Number of elements
+     * @param v    Initial value
+     */
     QgsAttributes( int size, const QVariant& v )
         : QVector<QVariant>( size, v )
     {}
 
+    /**
+     * Copies another vector of attributes
+     * @param v Attributes to copy
+     */
     QgsAttributes( const QVector<QVariant>& v )
         : QVector<QVariant>( v )
     {}
@@ -146,211 +172,258 @@ class QgsField;
 
 #include "qgsfield.h"
 
+/***************************************************************************
+ * This class is considered CRITICAL and any change MUST be accompanied with
+ * full unit tests in testqgsfeature.cpp.
+ * See details in QEP #17
+ ****************************************************************************/
 
 /** \ingroup core
  * The feature class encapsulates a single feature including its id,
  * geometry and a list of field/values attributes.
- *
+ * \note QgsFeature objects are implicitly shared.
  * @author Gary E.Sherman
  */
 class CORE_EXPORT QgsFeature
 {
   public:
-    //! Constructor
+
+    /** Constructor for QgsFeature
+     * @param id feature id
+     */
     QgsFeature( QgsFeatureId id = QgsFeatureId() );
 
+    /** Constructor for QgsFeature
+     * @param fields feature's fields
+     * @param id feature id
+     */
     QgsFeature( const QgsFields& fields, QgsFeatureId id = QgsFeatureId() );
 
-    /** copy ctor needed due to internal pointer */
+    /** Copy constructor
+     */
     QgsFeature( const QgsFeature & rhs );
 
-    /** assignment operator needed due to internal pointer */
+    /** Assignment operator
+     */
     QgsFeature & operator=( QgsFeature const & rhs );
 
     //! Destructor
-    ~QgsFeature();
+    virtual ~QgsFeature();
 
-    /**
-     * Get the feature id for this feature
-     * @return Feature id
+    /** Get the feature ID for this feature.
+     * @returns feature ID
+     * @see setFeatureId
      */
     QgsFeatureId id() const;
 
-    /**
-     * Set the feature id for this feature
-     * @param id Feature id
+    /** Sets the feature ID for this feature.
+     * @param id feature id
+     * @see id
      */
     void setFeatureId( QgsFeatureId id );
 
-    const QgsAttributes& attributes() const { return mAttributes; }
-    QgsAttributes& attributes() { return mAttributes; }
-    void setAttributes( const QgsAttributes& attrs ) { mAttributes = attrs; }
+    /** Returns the feature's attributes.
+     * @link attributes @endlink method.
+     * @returns list of feature's attributes
+     * @see setAttributes
+     * @note added in QGIS 2.9
+     */
+    QgsAttributes attributes() const;
 
-    /**
-     * Set an attribute by id
-     *
-     * @param field  The index of the field to set
-     * @param attr   The value of the attribute
-     *
-     * @return false, if the field id does not exist
-     *
+    /** Sets the feature's attributes.
+     * @param attrs attribute list
+     * @see setAttribute
+     * @see attributes
+     */
+    void setAttributes( const QgsAttributes& attrs );
+
+    /** Set an attribute's value by field index.
+     * @param field the index of the field to set
+     * @param attr the value of the attribute
+     * @return false, if the field index does not exist
      * @note For Python: raises a KeyError exception instead of returning false
+     * @see setAttributes
      */
     bool setAttribute( int field, const QVariant& attr );
 
-    /**
-     * Initialize this feature with the given number of fields. Discard any previously set attribute data.
+    /** Initialize this feature with the given number of fields. Discard any previously set attribute data.
      * @param fieldCount Number of fields to initialize
      */
     void initAttributes( int fieldCount );
 
-    /**
-     * Deletes an attribute and its value
-     *
-     * @param field The index of the field
-     *
+    /** Deletes an attribute and its value.
+     * @param field the index of the field
+     * @see setAttribute
      * @note For Python: raises a KeyError exception if the field is not found
      */
     void deleteAttribute( int field );
 
-    /**
-     * Return the validity of this feature. This is normally set by
+    /** Returns the validity of this feature. This is normally set by
      * the provider to indicate some problem that makes the feature
      * invalid or to indicate a null feature.
+     * @see setValid
      */
     bool isValid() const;
 
-    /**
-     * Set the validity of the feature.
+    /** Sets the validity of the feature.
+     * @param validity set to true if feature is valid
+     * @see isValid
      */
     void setValid( bool validity );
 
-    /**
-     * Get the geometry object associated with this feature
+    /** Get the geometry object associated with this feature. If the geometry
+     * is not going to be modified than calling the const @link constGeometry @endlink
+     * method is preferable as it avoids a potentially expensive detach operation.
+     *
+     * It is possible to modify the geometry in place but this will
+     * be removed in 3.0 and therefore @link setGeometry @endlink should be called explicitly.
+     *
+     * @note will be modified to return by value in QGIS 3.0: `QgsGeometry geometry() const;`
+     *
+     * @returns pointer to feature's geometry
+     * @see constGeometry
+     * @see geometryAndOwnership
+     * @see setGeometry
      */
-    QgsGeometry *geometry() const;
+    QgsGeometry* geometry();
 
-    /**
-     * Get the geometry object associated with this feature
-     * The caller assumes responsibility for the QgsGeometry*'s destruction.
+    /** Gets a const pointer to the geometry object associated with this feature. If the geometry
+     * is not going to be modified than this method is preferable to the non-const
+     * @link geometry @endlink method.
+     * @note this is a temporary method for 2.x release cycle. Will be removed in QGIS 3.0.
+     * @returns const pointer to feature's geometry
+     * @see geometry
+     * @see geometryAndOwnership
+     * @see setGeometry
+     * @note added in QGIS 2.9
+     * @note will be removed in QGIS 3.0
      */
-    QgsGeometry *geometryAndOwnership();
+    const QgsGeometry* constGeometry() const;
 
-    /** Set this feature's geometry from another QgsGeometry object (deep copy)
+    /** Get the geometry object associated with this feature, and transfer ownership of the
+     * geometry to the caller. The caller assumes responsibility for the QgsGeometry*'s destruction.
+     * @returns pointer to feature's geometry
+     * @see geometry
+     * @see setGeometry
+     * @deprecated use constGeometry() instead
+     */
+    Q_DECL_DEPRECATED QgsGeometry *geometryAndOwnership();
+
+    /** Set this feature's geometry from another QgsGeometry object. This method performs a deep copy
+     * of the geometry.
+     * @param geom new feature geometry
+     * @see geometry
+     * @see constGeometry
+     * @see geometryAndOwnership
+     * @see setGeometryAndOwnership
      */
     void setGeometry( const QgsGeometry& geom );
 
-    /** Set this feature's geometry (takes geometry ownership)
+    /** Set this feature's geometry from a QgsGeometry pointer. Ownership of the geometry is transferred
+     * to the feature.
+     * @param geom new feature geometry
      * @note not available in python bindings
+     * @see geometry
+     * @see constGeometry
+     * @see geometryAndOwnership
+     * @see setGeometryAndOwnership
      */
     void setGeometry( QgsGeometry* geom );
 
-    /**
-     * Set this feature's geometry from WKB
-     *
-     * This feature assumes responsibility for destroying geom.
+    /** Set this feature's geometry from WKB. This feature assumes responsibility for destroying the
+     * created geometry.
+     * @param geom geometry as WKB
+     * @param length size of WKB
+     * @see setGeometry
+     * @see geometry
+     * @see constGeometry
+     * @see geometryAndOwnership
+     * @deprecated will be removed in QGIS 3.0
      */
-    void setGeometryAndOwnership( unsigned char * geom, size_t length );
+    Q_DECL_DEPRECATED void setGeometryAndOwnership( unsigned char * geom, int length );
 
-    /** Assign a field map with the feature to allow attribute access by attribute name
-     *
-     *  @param fields         The attribute fields which this feature holds. When used from python, make sure
-     *                        a copy of the fields is held by python, as ownership stays there.
-     *                        I.e. Do not call feature.setFields( myDataProvider.fields() ) but instead call
-     *                        myFields = myDataProvider.fields()
-     *                        feature.setFields( myFields )
+    /** Assign a field map with the feature to allow attribute access by attribute name.
+     *  @param fields The attribute fields which this feature holds
      *  @param initAttributes If true, attributes are initialized. Clears any data previously assigned.
      *                        C++: Defaults to false
      *                        Python: Defaults to true
-     *
-     * TODO: QGIS3 - take reference, not pointer
+     * @deprecated use setFields( const QgsFields& fields, bool initAttributes = false ) instead
+     * @note not available in Python bindings
      */
-    void setFields( const QgsFields* fields, bool initAttributes = false );
+    Q_DECL_DEPRECATED void setFields( const QgsFields* fields, bool initAttributes = false );
 
-    /** Get associated field map.
-     *
-     * TODO: QGIS 3 - return reference or value, not pointer
+    /** Assign a field map with the feature to allow attribute access by attribute name.
+     *  @param fields The attribute fields which this feature holds
+     *  @param initAttributes If true, attributes are initialized. Clears any data previously assigned.
+     *                        C++: Defaults to false
+     *                        Python: Defaults to true
+     * @note added in QGIS 2.9
+     * @see fields
      */
-    const QgsFields* fields() const { return &mFields; }
+    void setFields( const QgsFields& fields, bool initAttributes = false );
+
+    /** Returns the field map associated with the feature.
+     * @see setFields
+     * TODO: QGIS 3 - return value, not pointer
+     */
+    const QgsFields* fields() const;
 
     /** Insert a value into attribute. Returns false if attribute name could not be converted to index.
-     *  Field map must be associated to make this work.
-     *
+     *  Field map must be associated using @link setFields @endlink before this method can be used.
      *  @param name The name of the field to set
      *  @param value The value to set
-     *
      *  @return false if attribute name could not be converted to index (C++ only)
-     *
      *  @note For Python: raises a KeyError exception instead of returning false
+     *  @see setFields
      */
-    bool setAttribute( const QString& name, QVariant value );
+    bool setAttribute( const QString& name, const QVariant& value );
 
-    /** Remove an attribute value.
-     *  Returns false if attribute name could not be converted to index.
-     *  Field map must be associated to make this work.
-     *
+    /** Removes an attribute value by field name. Field map must be associated using @link setFields @endlink
+     *  before this method can be used.
      *  @param name The name of the field to delete
-     *
      *  @return false if attribute name could not be converted to index (C++ only)
-     *
      *  @note For Python: raises a KeyError exception instead of returning false
+     *  @see setFields
      */
     bool deleteAttribute( const QString& name );
 
-    /** Lookup attribute value from attribute name.
-     *  Returns invalid variant if attribute name could not be converted to index (C++ only)
-     *  Field map must be associated to make this work.
-     *
+    /** Lookup attribute value from attribute name. Field map must be associated using @link setFields @endlink
+     *  before this method can be used.
      *  @param name The name of the attribute to get
-     *
      *  @return The value of the attribute (C++: Invalid variant if no such name exists )
-     *
      *  @note For Python: raises a KeyError exception if field is not found
+     *  @see setFields
      */
     QVariant attribute( const QString& name ) const;
 
-    /** Lookup attribute value from its index. Returns invalid variant if the index does not exist.
-     *
+    /** Lookup attribute value from its index. Field map must be associated using @link setFields @endlink
+     *  before this method can be used.
      *  @param fieldIdx The index of the attribute to get
-     *
      *  @return The value of the attribute (C++: Invalid variant if no such index exists )
-     *
      *  @note For Python: raises a KeyError exception if field is not found
+     *  @see setFields
      */
     QVariant attribute( int fieldIdx ) const;
 
-    /** Utility method to get attribute index from name. Returns -1 if field does not exist or field map is not associated.
-     *  Field map must be associated to make this work.
+    /** Utility method to get attribute index from name. Field map must be associated using @link setFields @endlink
+     *  before this method can be used.
+     *  @param fieldName name of field to get attribute index of
+     *  @returns -1 if field does not exist or field map is not associated.
+     *  @see setFields
      */
     int fieldNameIndex( const QString& fieldName ) const;
 
   private:
 
-    //! feature id
-    QgsFeatureId mFid;
-
-    /** attributes accessed by field index */
-    QgsAttributes mAttributes;
-
-    /** pointer to geometry in binary WKB format
-
-       This is usually set by a call to OGRGeometry::exportToWkb()
-     */
-    QgsGeometry *mGeometry;
-
-    /** Indicator if the mGeometry is owned by this QgsFeature.
-        If so, this QgsFeature takes responsibility for the mGeometry's destruction.
-     */
-    bool mOwnsGeometry;
-
-    //! Flag to indicate if this feature is valid
-    bool mValid;
-
-    //! Optional field map for name-based attribute lookups
-    QgsFields mFields;
+    QExplicitlySharedDataPointer<QgsFeaturePrivate> d;
 
 }; // class QgsFeature
 
+/** Writes the feature to stream out. QGIS version compatibility is not guaranteed. */
+CORE_EXPORT QDataStream& operator<<( QDataStream& out, const QgsFeature& feature );
+/** Reads a feature from stream in into feature. QGIS version compatibility is not guaranteed. */
+CORE_EXPORT QDataStream& operator>>( QDataStream& in, QgsFeature& feature );
+
 // key = feature id, value = changed attributes
 typedef QMap<QgsFeatureId, QgsAttributeMap> QgsChangedAttributesMap;
 
