@@ -33,8 +33,12 @@ QgsLabelSearchTree::~QgsLabelSearchTree()
 
 void QgsLabelSearchTree::label( const QgsPoint& p, QList<QgsLabelPosition*>& posList ) const
 {
-  double c_min[2]; c_min[0] = p.x() - 0.1; c_min[1] = p.y() - 0.1;
-  double c_max[2]; c_max[0] = p.x() + 0.1; c_max[1] = p.y() + 0.1;
+  double c_min[2];
+  c_min[0] = p.x() - 0.1;
+  c_min[1] = p.y() - 0.1;
+  double c_max[2];
+  c_max[0] = p.x() + 0.1;
+  c_max[1] = p.y() + 0.1;
 
   QList<QgsLabelPosition*> searchResults;
   mSpatialIndex.Search( c_min, c_max, searchCallback, &searchResults );
@@ -53,8 +57,12 @@ void QgsLabelSearchTree::label( const QgsPoint& p, QList<QgsLabelPosition*>& pos
 
 void QgsLabelSearchTree::labelsInRect( const QgsRectangle& r, QList<QgsLabelPosition*>& posList ) const
 {
-  double c_min[2]; c_min[0] = r.xMinimum(); c_min[1] = r.yMinimum();
-  double c_max[2]; c_max[0] = r.xMaximum(); c_max[1] = r.yMaximum();
+  double c_min[2];
+  c_min[0] = r.xMinimum();
+  c_min[1] = r.yMinimum();
+  double c_max[2];
+  c_max[0] = r.xMaximum();
+  c_max[1] = r.yMaximum();
 
   QList<QgsLabelPosition*> searchResults;
   mSpatialIndex.Search( c_min, c_max, searchCallback, &searchResults );
@@ -67,7 +75,7 @@ void QgsLabelSearchTree::labelsInRect( const QgsRectangle& r, QList<QgsLabelPosi
   }
 }
 
-bool QgsLabelSearchTree::insertLabel( LabelPosition* labelPos, int featureId, const QString& layerName, const QString& labeltext, const QFont& labelfont, bool diagram, bool pinned )
+bool QgsLabelSearchTree::insertLabel( pal::LabelPosition* labelPos, int featureId, const QString& layerName, const QString& labeltext, const QFont& labelfont, bool diagram, bool pinned )
 {
   if ( !labelPos )
   {
@@ -79,6 +87,7 @@ bool QgsLabelSearchTree::insertLabel( LabelPosition* labelPos, int featureId, co
   labelPos->getBoundingBox( c_min, c_max );
 
   QVector<QgsPoint> cornerPoints;
+  cornerPoints.reserve( 4 );
   for ( int i = 0; i < 4; ++i )
   {
     cornerPoints.push_back( QgsPoint( labelPos->getX( i ), labelPos->getY( i ) ) );
@@ -86,17 +95,15 @@ bool QgsLabelSearchTree::insertLabel( LabelPosition* labelPos, int featureId, co
   QgsLabelPosition* newEntry = new QgsLabelPosition( featureId, labelPos->getAlpha(), cornerPoints, QgsRectangle( c_min[0], c_min[1], c_max[0], c_max[1] ),
       labelPos->getWidth(), labelPos->getHeight(), layerName, labeltext, labelfont, labelPos->getUpsideDown(), diagram, pinned );
   mSpatialIndex.Insert( c_min, c_max, newEntry );
+  mOwnedPositions << newEntry;
   return true;
 }
 
 void QgsLabelSearchTree::clear()
 {
-  RTree<QgsLabelPosition*, double, 2, double>::Iterator indexIt;
-  mSpatialIndex.GetFirst( indexIt );
-  while ( !mSpatialIndex.IsNull( indexIt ) )
-  {
-    delete mSpatialIndex.GetAt( indexIt );
-    mSpatialIndex.GetNext( indexIt );
-  }
   mSpatialIndex.RemoveAll();
+
+  //PAL rtree iterator is buggy and doesn't iterate over all items, so we can't iterate through the tree to delete positions
+  qDeleteAll( mOwnedPositions );
+  mOwnedPositions.clear();
 }
