@@ -23,30 +23,31 @@
 #include <QSet>
 #include <QObject>
 #include <QStringList>
-
-#include "qgssingleton.h"
 class QString;
 class QgsMapLayer;
 
 /** \ingroup core
-* This class tracks map layers that are currently loaded and provides
-* a means to fetch a pointer to a map layer and delete it.
-*/
-class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsMapLayerRegistry>
+ * This class tracks map layers that are currently loaded and provides
+ * a means to fetch a pointer to a map layer and delete it.
+ */
+class CORE_EXPORT QgsMapLayerRegistry : public QObject
 {
     Q_OBJECT
 
   public:
+    //! Returns the instance pointer, creating the object on the first call
+    static QgsMapLayerRegistry * instance();
+
     //! Return the number of registered layers.
     int count();
 
     ~QgsMapLayerRegistry();
 
     //! Retrieve a pointer to a loaded layer by id
-    QgsMapLayer *mapLayer( QString theLayerId );
+    QgsMapLayer *mapLayer( const QString& theLayerId );
 
     //! Retrieve a pointer to a loaded layer by name
-    QList<QgsMapLayer *> mapLayersByName( QString layerName );
+    QList<QgsMapLayer *> mapLayersByName( const QString& layerName );
 
     //! Retrieve the mapLayers collection (mainly intended for use by projection)
     const QMap<QString, QgsMapLayer*> & mapLayers();
@@ -71,9 +72,9 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *         it will not be part of the returned QList.
      *
      * @note As a side-effect QgsProject is made dirty.
-     * @note takeOwner not available in python binding - always takes ownership
+     * @note takeOwnership not available in python binding - always takes ownership
      */
-    QList<QgsMapLayer *> addMapLayers( QList<QgsMapLayer *> theMapLayers,
+    QList<QgsMapLayer *> addMapLayers( const QList<QgsMapLayer*>& theMapLayers,
                                        bool addToLegend = true,
                                        bool takeOwnership = true );
 
@@ -94,7 +95,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *                      If you specify false here, you have take care of deleting
      *                      the layer yourself. Not available in python.
      *
-     * @return NULL if unable to add layer, otherwise pointer to newly added layer
+     * @return nullptr if unable to add layer, otherwise pointer to newly added layer
      *
      * @see addMapLayers
      *
@@ -117,7 +118,22 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *
      * @note As a side-effect QgsProject is made dirty.
      */
-    void removeMapLayers( QStringList theLayerIds );
+    void removeMapLayers( const QStringList& theLayerIds );
+
+    /**
+     * @brief
+     * Remove a set of layers from the registry
+     *
+     * Any canvases using the affected layers will need to remove them
+     *
+     * The layers being removed are deleted as well as the registry
+     * table entries.
+     *
+     * @param layers  The layers to remove. Null pointers are ignored
+     *
+     * @note As a side-effect QgsProject is made dirty.
+     */
+    void removeMapLayers( const QList<QgsMapLayer*>& layers );
 
     /**
      * @brief
@@ -134,6 +150,21 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      */
     void removeMapLayer( const QString& theLayerId );
 
+    /**
+     * @brief
+     * Remove a layer from qgis
+     *
+     * Any canvases using the affected layers will need to remove them
+     *
+     * The layer being removed is deleted as well as the registry
+     * table entry.
+     *
+     * @param layer   The layer to remove. Nothing happens if the pointer is null
+     *
+     * @note As a side-effect QgsProject is made dirty.
+     */
+    void removeMapLayer( QgsMapLayer* layer );
+
     /**
      * Remove all registered layers
      *
@@ -149,7 +180,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      * see ticket #1974 for more details.
      */
     //! @deprecated since 2.4 - does nothing
-    Q_DECL_DEPRECATED void clearAllLayerCaches();
+    Q_DECL_DEPRECATED void clearAllLayerCaches() {}
 
     /**
      * Reload all provider data caches (currently used for WFS and WMS providers)
@@ -162,23 +193,39 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *
      * @param theLayerIds  A list of ids of the layers which are removed.
      */
-    void layersWillBeRemoved( QStringList theLayerIds );
+    void layersWillBeRemoved( const QStringList& theLayerIds );
 
     /**
-     * Emitted when a layer is removed from the registry
+     * Emitted when one or more layers are removed from the registry
+     *
+     * @param layers  A list of layers which are removed.
+     */
+    void layersWillBeRemoved( const QList<QgsMapLayer*>& layers );
+
+    /**
+     * Emitted when an owned layer is removed from the registry
      *
      * @param theLayerId  The id of the layer being removed
      *
      * @note Consider using {@link layersWillBeRemoved()} instead
      */
-    void layerWillBeRemoved( QString theLayerId );
+    void layerWillBeRemoved( const QString& theLayerId );
+
+    /**
+     * Emitted when an owned layer is removed from the registry
+     *
+     * @param layer  The layer being removed
+     *
+     * @note Consider using {@link layersWillBeRemoved()} instead
+     */
+    void layerWillBeRemoved( QgsMapLayer* layer );
 
     /**
      * Emitted after one or more layers were removed from the registry
      *
      * @param theLayerIds  A list of ids of the layers which were removed.
      */
-    void layersRemoved( QStringList theLayerIds );
+    void layersRemoved( const QStringList& theLayerIds );
 
     /**
      * Emitted after a layer was removed from the registry
@@ -187,7 +234,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *
      * @note Consider using {@link layersRemoved()} instead
      */
-    void layerRemoved( QString theLayerId );
+    void layerRemoved( const QString& theLayerId );
 
 
     /**
@@ -207,7 +254,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *
      * @see legendLayersAdded()
      */
-    void layersAdded( QList<QgsMapLayer *> theMapLayers );
+    void layersAdded( const QList<QgsMapLayer *>& theMapLayers );
 
     /**
      * Emitted when a layer is added to the registry.
@@ -226,24 +273,24 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject, public QgsSingleton<QgsM
      *
      * @param theMapLayers  The {@link QgsMapLayer}s which are added to the legend.
      */
-    void legendLayersAdded( QList<QgsMapLayer*> theMapLayers );
+    void legendLayersAdded( const QList<QgsMapLayer*>& theMapLayers );
 
   protected:
 #if 0
-    /** debugging member
-        invoked when a connect() is made to this object
-    */
+    /** Debugging member
+     *  invoked when a connect() is made to this object
+     */
     void connectNotify( const char * signal ) override;
 #endif
 
+  private slots:
+    void onMapLayerDeleted( QObject* obj );
+
   private:
     //! private singleton constructor
-    QgsMapLayerRegistry( QObject * parent = 0 );
+    QgsMapLayerRegistry( QObject * parent = nullptr );
 
     QMap<QString, QgsMapLayer*> mMapLayers;
-    QSet<QgsMapLayer*> mOwnedLayers;
-
-    friend class QgsSingleton<QgsMapLayerRegistry>; // Let QgsSingleton access private constructor
 }; // class QgsMapLayerRegistry
 
 #endif //QgsMapLayerRegistry_H
