@@ -15,16 +15,21 @@
 
 #include "qgsmaprenderercustompainterjob.h"
 
+#include "qgslabelingenginev2.h"
 #include "qgslogger.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsmaplayerrenderer.h"
 #include "qgspallabeling.h"
+#include "qgsvectorlayer.h"
+#include "qgsrendererv2.h"
 
+#define LABELING_V2
 
 QgsMapRendererCustomPainterJob::QgsMapRendererCustomPainterJob( const QgsMapSettings& settings, QPainter* painter )
     : QgsMapRendererJob( settings )
     , mPainter( painter )
-    , mLabelingEngine( 0 )
+    , mLabelingEngine( nullptr )
+    , mLabelingEngineV2( nullptr )
     , mActive( false )
     , mRenderSynchronously( false )
 {
@@ -38,7 +43,10 @@ QgsMapRendererCustomPainterJob::~QgsMapRendererCustomPainterJob()
   //cancel();
 
   delete mLabelingEngine;
-  mLabelingEngine = 0;
+  mLabelingEngine = nullptr;
+
+  delete mLabelingEngineV2;
+  mLabelingEngineV2 = nullptr;
 }
 
 void QgsMapRendererCustomPainterJob::start()
@@ -69,16 +77,25 @@ void QgsMapRendererCustomPainterJob::start()
   Q_ASSERT_X( thePaintDevice->logicalDpiX() == mSettings.outputDpi(), "Job::startRender()", errMsg.toAscii().data() );
 
   delete mLabelingEngine;
-  mLabelingEngine = 0;
+  mLabelingEngine = nullptr;
+
+  delete mLabelingEngineV2;
+  mLabelingEngineV2 = nullptr;
 
   if ( mSettings.testFlag( QgsMapSettings::DrawLabeling ) )
   {
+#ifdef LABELING_V2
+    mLabelingEngineV2 = new QgsLabelingEngineV2();
+    mLabelingEngineV2->readSettingsFromProject();
+    mLabelingEngineV2->setMapSettings( mSettings );
+#else
     mLabelingEngine = new QgsPalLabeling;
     mLabelingEngine->loadEngineSettings();
     mLabelingEngine->init( mSettings );
+#endif
   }
 
-  mLayerJobs = prepareJobs( mPainter, mLabelingEngine );
+  mLayerJobs = prepareJobs( mPainter, mLabelingEngine, mLabelingEngineV2 );
 
   QgsDebugMsg( "Rendering prepared in (seconds): " + QString( "%1" ).arg( prepareTime.elapsed() / 1000.0 ) );
 
@@ -151,11 +168,16 @@ bool QgsMapRendererCustomPainterJob::isActive() const
 
 QgsLabelingResults* QgsMapRendererCustomPainterJob::takeLabelingResults()
 {
-  return mLabelingEngine ? mLabelingEngine->takeResults() : 0;
+  if ( mLabelingEngine )
+    return mLabelingEngine->takeResults();
+  else if ( mLabelingEngineV2 )
+    return mLabelingEngineV2->takeResults();
+  else
+    return nullptr;
 }
 
 
-void QgsMapRendererCustomPainterJob::waitForFinishedWithEventLoop( QEventLoop::ProcessEventsFlags flags )
+void QgsMapRendererCustomPainterJob::waitForFinishedWithEventLoop( const QEventLoop::ProcessEventsFlags& flags )
 {
   QEventLoop loop;
   connect( &mFutureWatcher, SIGNAL( finished() ), &loop, SLOT( quit() ) );
@@ -178,6 +200,8 @@ void QgsMapRendererCustomPainterJob::futureFinished()
   mRenderingTime = mRenderingStart.elapsed();
   QgsDebugMsg( "QPAINTER futureFinished" );
 
+  logRenderingTime( mLayerJobs );
+
   // final cleanup
   cleanupJobs( mLayerJobs );
 
@@ -193,10 +217,12 @@ void QgsMapRendererCustomPainterJob::staticRender( QgsMapRendererCustomPainterJo
   }
   catch ( QgsException & e )
   {
+    Q_UNUSED( e );
     QgsDebugMsg( "Caught unhandled QgsException: " + e.what() );
   }
   catch ( std::exception & e )
   {
+    Q_UNUSED( e );
     QgsDebugMsg( "Caught unhandled std::exception: " + QString::fromAscii( e.what() ) );
   }
   catch ( ... )
@@ -226,8 +252,15 @@ void QgsMapRendererCustomPainterJob::doRender()
     }
 
     if ( !job.cached )
+    {
+      QTime layerTime;
+      layerTime.start();
+
       job.renderer->render();
 
+      job.renderingTime = layerTime.elapsed();
+    }
+
     if ( job.img )
     {
       // If we flattened this layer for alternate blend modes, composite it now
@@ -239,13 +272,13 @@ void QgsMapRendererCustomPainterJob::doRender()
   QgsDebugMsg( "Done rendering map layers" );
 
   if ( mSettings.testFlag( QgsMapSettings::DrawLabeling ) && !mLabelingRenderContext.renderingStopped() )
-    drawLabeling( mSettings, mLabelingRenderContext, mLabelingEngine, mPainter );
+    drawLabeling( mSettings, mLabelingRenderContext, mLabelingEngine, mLabelingEngineV2, mPainter );
 
   QgsDebugMsg( "Rendering completed in (seconds): " + QString( "%1" ).arg( renderTime.elapsed() / 1000.0 ) );
 }
 
 
-void QgsMapRendererJob::drawLabeling( const QgsMapSettings& settings, QgsRenderContext& renderContext, QgsPalLabeling* labelingEngine, QPainter* painter )
+void QgsMapRendererJob::drawLabeling( const QgsMapSettings& settings, QgsRenderContext& renderContext, QgsPalLabeling* labelingEngine, QgsLabelingEngineV2* labelingEngine2, QPainter* painter )
 {
   QgsDebugMsg( "Draw labeling start" );
 
@@ -265,6 +298,15 @@ void QgsMapRendererJob::drawLabeling( const QgsMapSettings& settings, QgsRenderC
 
   drawNewLabeling( settings, renderContext, labelingEngine );
 
+  if ( labelingEngine2 )
+  {
+    // set correct extent
+    renderContext.setExtent( settings.visibleExtent() );
+    renderContext.setCoordinateTransform( nullptr );
+
+    labelingEngine2->run( renderContext );
+  }
+
   QgsDebugMsg( QString( "Draw labeling took (seconds): %1" ).arg( t.elapsed() / 1000. ) );
 }
 
@@ -293,14 +335,14 @@ void QgsMapRendererJob::drawOldLabeling( const QgsMapSettings& settings, QgsRend
     if ( ml->hasScaleBasedVisibility() && ( settings.scale() < ml->minimumScale() || settings.scale() > ml->maximumScale() ) )
       continue;
 
-    const QgsCoordinateTransform* ct = 0;
+    const QgsCoordinateTransform* ct = nullptr;
     QgsRectangle r1 = settings.visibleExtent(), r2;
 
     if ( settings.hasCrsTransformEnabled() )
     {
       ct = settings.layerTransform( ml );
       if ( ct )
-        reprojectToLayerExtent( ct, ml->crs().geographicFlag(), r1, r2 );
+        reprojectToLayerExtent( ml, ct, r1, r2 );
     }
 
     renderContext.setCoordinateTransform( ct );
@@ -317,7 +359,7 @@ void QgsMapRendererJob::drawNewLabeling( const QgsMapSettings& settings, QgsRend
   {
     // set correct extent
     renderContext.setExtent( settings.visibleExtent() );
-    renderContext.setCoordinateTransform( NULL );
+    renderContext.setCoordinateTransform( nullptr );
 
     labelingEngine->drawLabeling( renderContext );
     labelingEngine->exit();
@@ -326,10 +368,11 @@ void QgsMapRendererJob::drawNewLabeling( const QgsMapSettings& settings, QgsRend
 
 void QgsMapRendererJob::updateLayerGeometryCaches()
 {
-  foreach ( QString id, mGeometryCaches.keys() )
+  QMap<QString, QgsGeometryCache>::const_iterator it = mGeometryCaches.constBegin();
+  for ( ; it != mGeometryCaches.constEnd(); ++it )
   {
-    const QgsGeometryCache& cache = mGeometryCaches[id];
-    if ( QgsVectorLayer* vl = qobject_cast<QgsVectorLayer*>( QgsMapLayerRegistry::instance()->mapLayer( id ) ) )
+    const QgsGeometryCache& cache = it.value();
+    if ( QgsVectorLayer* vl = qobject_cast<QgsVectorLayer*>( QgsMapLayerRegistry::instance()->mapLayer( it.key() ) ) )
       * vl->cache() = cache;
   }
   mGeometryCaches.clear();
@@ -338,13 +381,22 @@ void QgsMapRendererJob::updateLayerGeometryCaches()
 
 bool QgsMapRendererJob::needTemporaryImage( QgsMapLayer* ml )
 {
-  if ( mSettings.testFlag( QgsMapSettings::UseAdvancedEffects ) && ml->type() == QgsMapLayer::VectorLayer )
+  if ( ml->type() == QgsMapLayer::VectorLayer )
   {
     QgsVectorLayer* vl = qobject_cast<QgsVectorLayer *>( ml );
-    if ((( vl->blendMode() != QPainter::CompositionMode_SourceOver )
-         || ( vl->featureBlendMode() != QPainter::CompositionMode_SourceOver )
-         || ( vl->layerTransparency() != 0 ) ) )
+    if ( vl->rendererV2() && vl->rendererV2()->forceRasterRender() )
+    {
+      //raster rendering is forced for this layer
       return true;
+    }
+    if ( mSettings.testFlag( QgsMapSettings::UseAdvancedEffects ) &&
+         (( vl->blendMode() != QPainter::CompositionMode_SourceOver )
+          || ( vl->featureBlendMode() != QPainter::CompositionMode_SourceOver )
+          || ( vl->layerTransparency() != 0 ) ) )
+    {
+      //layer properties require rasterisation
+      return true;
+    }
   }
 
   return false;
