@@ -26,6 +26,7 @@
 #include "qgsmaptopixel.h"
 #include "qgsrectangle.h"
 #include "qgsscalecalculator.h"
+#include "qgsexpressioncontext.h"
 
 class QPainter;
 
@@ -68,7 +69,7 @@ class CORE_EXPORT QgsMapSettings
     //! Return the size of the resulting map image
     QSize outputSize() const;
     //! Set the size of the resulting map image
-    void setOutputSize( const QSize& size );
+    void setOutputSize( QSize size );
 
     //! Return the rotation of the resulting map image
     //! Units are clockwise degrees
@@ -124,22 +125,24 @@ class CORE_EXPORT QgsMapSettings
     //! Get color that is used for drawing of selected vector features
     QColor selectionColor() const { return mSelectionColor; }
 
-    //! Enumeration of flags that adjust the way how map is rendered
+    //! Enumeration of flags that adjust the way the map is rendered
     enum Flag
     {
-      Antialiasing       = 0x01,  //!< Enable anti-aliasin for map rendering
-      DrawEditingInfo    = 0x02,  //!< Enable drawing of vertex markers for layers in editing mode
-      ForceVectorOutput  = 0x04,  //!< Vector graphics should not be cached and drawn as raster images
-      UseAdvancedEffects = 0x08,  //!< Enable layer transparency and blending effects
-      DrawLabeling       = 0x10,  //!< Enable drawing of labels on top of the map
-      UseRenderingOptimization = 0x20, //!< Enable vector simplification and other rendering optimizations
-      DrawSelection      = 0x40,  //!< Whether vector selections should be shown in the rendered map
+      Antialiasing             = 0x01,  //!< Enable anti-aliasing for map rendering
+      DrawEditingInfo          = 0x02,  //!< Enable drawing of vertex markers for layers in editing mode
+      ForceVectorOutput        = 0x04,  //!< Vector graphics should not be cached and drawn as raster images
+      UseAdvancedEffects       = 0x08,  //!< Enable layer transparency and blending effects
+      DrawLabeling             = 0x10,  //!< Enable drawing of labels on top of the map
+      UseRenderingOptimization = 0x20,  //!< Enable vector simplification and other rendering optimizations
+      DrawSelection            = 0x40,  //!< Whether vector selections should be shown in the rendered map
+      DrawSymbolBounds         = 0x80,  //!< Draw bounds of symbols (for debugging/testing)
+      RenderMapTile            = 0x100  //!< Draw map such that there are no problems between adjacent tiles
       // TODO: ignore scale-based visibility (overview)
     };
     Q_DECLARE_FLAGS( Flags, Flag )
 
     //! Set combination of flags that will be used for rendering
-    void setFlags( Flags flags );
+    void setFlags( const QgsMapSettings::Flags& flags );
     //! Enable or disable a particular flag (other flags are not affected)
     void setFlag( Flag flag, bool on = true );
     //! Return combination of flags used for rendering
@@ -164,14 +167,35 @@ class CORE_EXPORT QgsMapSettings
     //! Return the calculated scale of the map
     double scale() const;
 
+    /** Sets the expression context. This context is used for all expression evaluation
+     * associated with this map settings.
+     * @see expressionContext()
+     * @note added in QGIS 2.12
+     */
+    void setExpressionContext( const QgsExpressionContext& context ) { mExpressionContext = context; }
+
+    /** Gets the expression context. This context should be used for all expression evaluation
+     * associated with this map settings.
+     * @see setExpressionContext()
+     * @note added in QGIS 2.12
+     */
+    const QgsExpressionContext& expressionContext() const { return mExpressionContext; }
 
     // -- utility functions --
 
+    //! @note not available in python bindings
     const QgsDatumTransformStore& datumTransformStore() const { return mDatumTransformStore; }
     QgsDatumTransformStore& datumTransformStore() { return mDatumTransformStore; }
 
     const QgsMapToPixel& mapToPixel() const { return mMapToPixel; }
 
+    /** Computes an *estimated* conversion factor between layer and map units: layerUnits * layerToMapUnits = mapUnits
+     * @param theLayer The layer
+     * @param referenceExtent A reference extent based on which to perform the computation. If not specified, the layer extent is used
+     * @note added in QGIS 2.12
+     */
+    double layerToMapUnits( QgsMapLayer* theLayer, const QgsRectangle& referenceExtent = QgsRectangle() ) const;
+
     /**
      * @brief transform bounding box from layer's CRS to output CRS
      * @see layerToMapCoordinates( QgsMapLayer* theLayer, QgsRectangle rect ) if you want to transform a rectangle
@@ -240,6 +264,7 @@ class CORE_EXPORT QgsMapSettings
 
     QStringList mLayers;
     QMap<QString, QString> mLayerStyleOverrides;
+    QgsExpressionContext mExpressionContext;
 
     bool mProjectionsEnabled;
     QgsCoordinateReferenceSystem mDestCRS;
