@@ -43,8 +43,8 @@ QgsOWSConnection::QgsOWSConnection( const QString & theService, const QString &
 
   QSettings settings;
 
-  QString key = "/Qgis/connections-" + mService.toLower() + "/" + mConnName;
-  QString credentialsKey = "/Qgis/" + mService + "/" + mConnName;
+  QString key = "/Qgis/connections-" + mService.toLower() + '/' + mConnName;
+  QString credentialsKey = "/Qgis/" + mService + '/' + mConnName;
 
   QStringList connStringParts;
 
@@ -61,6 +61,13 @@ QgsOWSConnection::QgsOWSConnection( const QString & theService, const QString &
     mUri.setParam( "password", password );
   }
 
+  QString authcfg = settings.value( credentialsKey + "/authcfg" ).toString();
+  if ( !authcfg.isEmpty() )
+  {
+    mUri.setParam( "authcfg", authcfg );
+  }
+  mConnectionInfo.append( ",authcfg=" + authcfg );
+
   bool ignoreGetMap = settings.value( key + "/ignoreGetMapURI", false ).toBool();
   bool ignoreGetFeatureInfo = settings.value( key + "/ignoreGetFeatureInfoURI", false ).toBool();
   bool ignoreAxisOrientation = settings.value( key + "/ignoreAxisOrientation", false ).toBool();
@@ -122,6 +129,6 @@ void QgsOWSConnection::setSelectedConnection( const QString & theService, const
 void QgsOWSConnection::deleteConnection( const QString & theService, const QString & name )
 {
   QSettings settings;
-  settings.remove( "/Qgis/connections-" + theService.toLower() + "/" + name );
-  settings.remove( "/Qgis/" + theService + "/" + name );
+  settings.remove( "/Qgis/connections-" + theService.toLower() + '/' + name );
+  settings.remove( "/Qgis/" + theService + '/' + name );
 }
