@@ -32,9 +32,9 @@ class CORE_EXPORT QgsPluginLayer : public QgsMapLayer
     Q_OBJECT
 
   public:
-    QgsPluginLayer( QString layerType, QString layerName = QString() );
+    QgsPluginLayer( const QString& layerType, const QString& layerName = QString() );
 
-    /** return plugin layer type (the same as used in QgsPluginLayerRegistry) */
+    /** Return plugin layer type (the same as used in QgsPluginLayerRegistry) */
     QString pluginLayerType();
 
     void setExtent( const QgsRectangle &extent ) override;
@@ -42,7 +42,7 @@ class CORE_EXPORT QgsPluginLayer : public QgsMapLayer
     //! return a list of symbology items for the legend
     //! (defult implementation returns nothing)
     //! @note Added in v2.1
-    virtual QgsLegendSymbologyList legendSymbologyItems( const QSize& iconSize );
+    virtual QgsLegendSymbologyList legendSymbologyItems( QSize iconSize );
 
     /** Return new instance of QgsMapLayerRenderer that will be used for rendering of given context
      *
