@@ -27,10 +27,12 @@
 #include <QList>
 #include <QObject>
 #include <QPair>
+#include <QFileInfo>
 
 //for the snap settings
 #include "qgssnapper.h"
 #include "qgstolerance.h"
+#include "qgsunittypes.h"
 
 //#include <QDomDocument>
 
@@ -45,6 +47,7 @@ class QgsMapLayer;
 class QgsProjectBadLayerHandler;
 class QgsRelationManager;
 class QgsVectorLayer;
+class QgsVisibilityPresetCollection;
 
 /** \ingroup core
  * Reads and writes project states.
@@ -71,32 +74,33 @@ class CORE_EXPORT QgsProject : public QObject
   public:
 
     /**
-       @todo XXX Should have semantics for saving project if dirty as last gasp?
-    */
+     * @todo XXX Should have semantics for saving project if dirty as last gasp?
+     */
     ~QgsProject();
 
     /// access to canonical QgsProject instance
     static QgsProject * instance();
 
     /**
-       Every project has an associated title string
-
-       ### QGIS 3: remove in favor of setTitle(...)
+     * Every project has an associated title string
+     *
+     * @deprecated Use setTitle instead.
      */
     //@{
-    void title( const QString & title );
+    Q_DECL_DEPRECATED inline void title( const QString & title ) { setTitle( title ); }
 
     /** Set project title
-     *  @note added in 2.4 */
+     *  @note added in 2.4
+     */
     void setTitle( const QString& title );
 
-    /** returns title */
-    const QString & title() const;
+    /** Returns title */
+    QString title() const;
     //@}
 
     /**
-       the dirty flag is true if the project has been modified since the last
-       write()
+     * the dirty flag is true if the project has been modified since the last
+     * write()
      */
     //@{
     bool isDirty() const;
@@ -111,36 +115,41 @@ class CORE_EXPORT QgsProject : public QObject
 
 
     /**
-       Every project has an associated file that contains its XML
+     * Every project has an associated file that contains its XML
      */
     //@{
     void setFileName( const QString & name );
 
-    /** returns file name */
+    /** Returns file name */
     QString fileName() const;
     //@}
 
+    /** Returns QFileInfo object for the project's associated file.
+     * @note added in QGIS 2.9
+     */
+    QFileInfo fileInfo() const;
+
     /** Clear the project
      * @note added in 2.4
      */
     void clear();
 
 
-    /** read project file
-
-       @note Any current plug-in state is erased
-
-       @note dirty set to false after successful invocation
-
-       @note file name argument implicitly sets file
-
-       @note
-
-       - Gets the extents
-       - Creates maplayers
-       - Registers maplayers
-
-       @note it's presumed that the caller has already reset the map canvas, map registry, and legend
+    /** Read project file
+     *
+     * @note Any current plug-in state is erased
+     *
+     * @note dirty set to false after successful invocation
+     *
+     * @note file name argument implicitly sets file
+     *
+     * @note
+     *
+     * - Gets the extents
+     * - Creates maplayers
+     * - Registers maplayers
+     *
+     * @note it's presumed that the caller has already reset the map canvas, map registry, and legend
      */
     //@{
     bool read( const QFileInfo & file );
@@ -148,29 +157,29 @@ class CORE_EXPORT QgsProject : public QObject
     //@}
 
 
-    /** read the layer described in the associated Dom node
-
-        @param layerNode   represents a QgsProject Dom node that maps to a specific layer.
-
-        QgsProject raises an exception when one of the QgsProject::read()
-        implementations fails.  Since the read()s are invoked from qgisapp,
-        then qgisapp handles the exception.  It prompts the user for the new
-        location of the data, if any.  If there is a new location, the Dom
-        node associated with the layer has its datasource tag corrected.
-        Then that node is passed to this member function to be re-opened.
-
+    /** Read the layer described in the associated Dom node
+     *
+     * @param layerNode   represents a QgsProject Dom node that maps to a specific layer.
+     *
+     * QgsProject raises an exception when one of the QgsProject::read()
+     * implementations fails.  Since the read()s are invoked from qgisapp,
+     * then qgisapp handles the exception.  It prompts the user for the new
+     * location of the data, if any.  If there is a new location, the Dom
+     * node associated with the layer has its datasource tag corrected.
+     * Then that node is passed to this member function to be re-opened.
+     *
      */
     bool read( QDomNode & layerNode );
 
 
-    /** write project file
-
-       XXX How to best get read access to Qgis state?  Actually we can finagle
-       that by searching for qgisapp in object hiearchy.
-
-       @note file name argument implicitly sets file
-
-       @note dirty set to false after successful invocation
+    /** Write project file
+     *
+     * XXX How to best get read access to Qgis state?  Actually we can finagle
+     * that by searching for qgisapp in object hiearchy.
+     *
+     * @note file name argument implicitly sets file
+     *
+     * @note dirty set to false after successful invocation
      */
     //@{
     bool write( const QFileInfo & file );
@@ -178,20 +187,20 @@ class CORE_EXPORT QgsProject : public QObject
     //@}
 
     /**
-       removes all project properties
-
-       ### QGIS 3: remove in favor of clear()
-    */
+     * removes all project properties
+     *
+     * ### QGIS 3: remove in favor of clear()
+     */
     void clearProperties();
 
 
     /* key value mutators
-
-      keys would be the familiar QSettings-like '/' delimited entries, implying
-      a hierarchy of keys and corresponding values
-
-      @note The key string <em>must</em> include '/'s.  E.g., "/foo" not "foo".
-    */
+     *
+     * keys would be the familiar QSettings-like '/' delimited entries, implying
+     * a hierarchy of keys and corresponding values
+     *
+     * @note The key string must be valid xml tag names in order to be saved to the file.
+     */
     //@{
     //! @note available in python bindings as writeEntryBool
     bool writeEntry( const QString & scope, const QString & key, bool value );
@@ -202,66 +211,66 @@ class CORE_EXPORT QgsProject : public QObject
     bool writeEntry( const QString & scope, const QString & key, const QStringList & value );
     //@}
 
-    /** key value accessors
-
-        keys would be the familiar QSettings-like '/' delimited entries,
-        implying a hierarchy of keys and corresponding values
-
-
-        @note The key string <em>must</em> include '/'s.  E.g., "/foo" not "foo".
-    */
+    /** Key value accessors
+     *
+     * keys would be the familiar QSettings-like '/' delimited entries,
+     * implying a hierarchy of keys and corresponding values
+     *
+     */
     //@{
-    QStringList readListEntry( const QString & scope, const QString & key, QStringList def = QStringList(), bool *ok = 0 ) const;
+    QStringList readListEntry( const QString & scope, const QString & key, const QStringList& def = QStringList(), bool *ok = nullptr ) const;
 
-    QString readEntry( const QString & scope, const QString & key, const QString & def = QString::null, bool * ok = 0 ) const;
-    int readNumEntry( const QString & scope, const QString & key, int def = 0, bool * ok = 0 ) const;
-    double readDoubleEntry( const QString & scope, const QString & key, double def = 0, bool * ok = 0 ) const;
-    bool readBoolEntry( const QString & scope, const QString & key, bool def = false, bool * ok = 0 ) const;
+    QString readEntry( const QString & scope, const QString & key, const QString & def = QString::null, bool * ok = nullptr ) const;
+    int readNumEntry( const QString & scope, const QString & key, int def = 0, bool * ok = nullptr ) const;
+    double readDoubleEntry( const QString & scope, const QString & key, double def = 0, bool * ok = nullptr ) const;
+    bool readBoolEntry( const QString & scope, const QString & key, bool def = false, bool * ok = nullptr ) const;
     //@}
 
 
-    /** remove the given key */
+    /** Remove the given key */
     bool removeEntry( const QString & scope, const QString & key );
 
 
-    /** return keys with values -- do not return keys that contain other keys
-
-      @note equivalent to QSettings entryList()
-    */
+    /** Return keys with values -- do not return keys that contain other keys
+     *
+     * @note equivalent to QSettings entryList()
+     */
     QStringList entryList( const QString & scope, const QString & key ) const;
 
-    /** return keys with keys -- do not return keys that contain only values
-
-      @note equivalent to QSettings subkeyList()
-    */
+    /** Return keys with keys -- do not return keys that contain only values
+     *
+     * @note equivalent to QSettings subkeyList()
+     */
     QStringList subkeyList( const QString & scope, const QString & key ) const;
 
 
-    /** dump out current project properties to stderr
-
-      @todo XXX Now slightly broken since re-factoring.  Won't print out top-level key
-                and redundantly prints sub-keys.
-    */
+    /** Dump out current project properties to stderr
+     *
+     * @todo XXX Now slightly broken since re-factoring.  Won't print out top-level key
+     *           and redundantly prints sub-keys.
+     */
     void dumpProperties() const;
 
-    /** prepare a filename to save it to the project file */
-    QString writePath( QString filename, QString relativeBasePath = QString::null ) const;
+    /** Prepare a filename to save it to the project file */
+    QString writePath( const QString& filename, const QString& relativeBasePath = QString::null ) const;
 
-    /** turn filename read from the project file to an absolute path */
+    /** Turn filename read from the project file to an absolute path */
     QString readPath( QString filename ) const;
 
     /** Return error message from previous read/write */
     QString error() const;
 
     /** Change handler for missing layers.
-      Deletes old handler and takes ownership of the new one. */
+     * Deletes old handler and takes ownership of the new one.
+     */
     void setBadLayerHandler( QgsProjectBadLayerHandler* handler );
 
     /** Returns project file path if layer is embedded from other project file. Returns empty string if layer is not embedded*/
     QString layerIsEmbedded( const QString& id ) const;
 
     /** Creates a maplayer instance defined in an arbitrary project file. Caller takes ownership
-      @return the layer or 0 in case of error
+     * @return the layer or 0 in case of error
+     * @note not available in Python bindings
      */
     bool createEmbeddedLayer( const QString& layerId, const QString& projectFilePath, QList<QDomNode>& brokenNodes,
                               QList< QPair< QgsVectorLayer*, QDomElement > >& vectorLayerList, bool saveFlag = true );
@@ -285,6 +294,18 @@ class CORE_EXPORT QgsProject : public QObject
     /** Convenience function to query topological editing status */
     bool topologicalEditing() const;
 
+    /** Convenience function to query default distance measurement units for project.
+     * @note added in QGIS 2.14
+     * @see areaUnits()
+     */
+    QGis::UnitType distanceUnits() const;
+
+    /** Convenience function to query default area measurement units for project.
+     * @note added in QGIS 2.14
+     * @see distanceUnits()
+     */
+    QgsUnitTypes::AreaUnit areaUnits() const;
+
     /** Return project's home path
       @return home path of project (or QString::null if not set) */
     QString homePath() const;
@@ -301,15 +322,24 @@ class CORE_EXPORT QgsProject : public QObject
      */
     QgsLayerTreeRegistryBridge* layerTreeRegistryBridge() const { return mLayerTreeRegistryBridge; }
 
+    /** Returns pointer to the project's visibility preset collection.
+     * @note added in QGIS 2.12
+     */
+    QgsVisibilityPresetCollection* visibilityPresetCollection();
+
   protected:
 
-    /** Set error message from read/write operation */
-    void setError( QString errorMessage );
+    /** Set error message from read/write operation
+     * @note not available in Python bindings
+     */
+    void setError( const QString& errorMessage );
 
-    /** Clear error message */
+    /** Clear error message
+     * @note not available in Python bindings
+     */
     void clearError();
 
-    //Creates layer and adds it to maplayer registry
+    //! Creates layer and adds it to maplayer registry
     //! @note not available in python bindings
     bool addLayer( const QDomElement& layerElem, QList<QDomNode>& brokenNodes, QList< QPair< QgsVectorLayer*, QDomElement > >& vectorLayerList );
 
@@ -327,7 +357,7 @@ class CORE_EXPORT QgsProject : public QObject
     void writeProject( QDomDocument & );
 
     /**
-     * Emitted, after the basic initialisation of a layer from the project
+     * Emitted, after the basic initialization of a layer from the project
      * file is done. You can use this signal to read additional information
      * from the project file.
      *
@@ -350,17 +380,31 @@ class CORE_EXPORT QgsProject : public QObject
     void projectSaved();
 
     //! emitted when an old project file is read.
-    void oldProjectVersionWarning( QString );
+    void oldProjectVersionWarning( const QString& );
 
     //! emitted when a layer from a projects was read
     // @param i current layer
     // @param n number of layers
     void layerLoaded( int i, int n );
 
-    void loadingLayer( QString );
+    void loadingLayer( const QString& );
 
     void snapSettingsChanged();
 
+    /** Emitted whenever the expression variables stored in the project have been changed.
+     * @note added in QGIS 3.0
+     */
+    void variablesChanged();
+
+  public slots:
+
+    /** Causes the project to emit the variablesChanged() signal. This should
+     * be called whenever expression variables related to the project are changed.
+     * @see variablesChanged()
+     * @note added in QGIS 3.0
+     */
+    void emitVariablesChanged();
+
   private:
 
     QgsProject(); // private 'cause it's a singleton
@@ -380,9 +424,10 @@ class CORE_EXPORT QgsProject : public QObject
 
     QgsProjectBadLayerHandler* mBadLayerHandler;
 
-    /**Embeded layers which are defined in other projects. Key: layer id,
-    value: pair< project file path, save layer yes / no (e.g. if the layer is part of an embedded group, loading/saving is done by the legend)
-       If the project file path is empty, QgsProject is going to ignore the layer for saving (e.g. because it is part and managed by an embedded group)*/
+    /** Embeded layers which are defined in other projects. Key: layer id,
+     * value: pair< project file path, save layer yes / no (e.g. if the layer is part of an embedded group, loading/saving is done by the legend)
+     *  If the project file path is empty, QgsProject is going to ignore the layer for saving (e.g. because it is part and managed by an embedded group)
+     */
     QHash< QString, QPair< QString, bool> > mEmbeddedLayers;
 
     void snapSettings( QStringList& layerIdList, QStringList& enabledList, QStringList& snapTypeList, QStringList& snapUnitList, QStringList& toleranceUnitList,
@@ -394,6 +439,8 @@ class CORE_EXPORT QgsProject : public QObject
 
     QgsLayerTreeRegistryBridge* mLayerTreeRegistryBridge;
 
+    QScopedPointer<QgsVisibilityPresetCollection> mVisibilityPresetCollection;
+
 }; // QgsProject
 
 
@@ -401,7 +448,7 @@ class CORE_EXPORT QgsProject : public QObject
 class CORE_EXPORT QgsProjectBadLayerHandler
 {
   public:
-    virtual void handleBadLayers( QList<QDomNode> layers, QDomDocument projectDom ) = 0;
+    virtual void handleBadLayers( const QList<QDomNode>& layers, const QDomDocument& projectDom ) = 0;
     virtual ~QgsProjectBadLayerHandler() {}
 };
 
@@ -410,7 +457,7 @@ class CORE_EXPORT QgsProjectBadLayerHandler
 class CORE_EXPORT QgsProjectBadLayerDefaultHandler : public QgsProjectBadLayerHandler
 {
   public:
-    virtual void handleBadLayers( QList<QDomNode> layers, QDomDocument projectDom ) override;
+    virtual void handleBadLayers( const QList<QDomNode>& layers, const QDomDocument& projectDom ) override;
 
 };
 
