@@ -21,6 +21,18 @@
 #include <QDomDocument>
 #include <QStringList>
 
+QgsProperty::QgsProperty()
+{
+}
+
+QgsProperty::~QgsProperty()
+{
+}
+
+QgsPropertyValue::~QgsPropertyValue()
+{
+}
+
 void QgsPropertyValue::dump( int tabs ) const
 {
   QString tabString;
