@@ -47,63 +47,58 @@ class QDomDocument;
 class CORE_EXPORT QgsProperty
 {
   public:
+    QgsProperty();
+    virtual ~QgsProperty();
 
-    QgsProperty()
-    {}
-
-    virtual ~ QgsProperty()
-    {}
-
-    /** dumps out the keys and values
-
-    @param tabs is number of tabs to print; used for pretty-printing
-    hierarchy
-    */
+    /** Dumps out the keys and values
+     *
+     * @param tabs is number of tabs to print; used for pretty-printing hierarchy
+     */
     virtual void dump( int tabs = 0 ) const = 0;
 
-    /** returns true if is a QgsPropertyKey */
+    /** Returns true if is a QgsPropertyKey */
     virtual bool isKey() const = 0;
 
-    /** returns true if is a QgsPropertyValue */
+    /** Returns true if is a QgsPropertyValue */
     virtual bool isValue() const = 0;
 
-    /** returns true if a leaf node
-
-    A leaf node is a key node that has either no value or a single value.
-    A non-leaf node would be a key node with key sub-nodes.
-
-    This is used for entryList() and subkeyList() implementation.
-    */
+    /** Returns true if a leaf node
+     *
+     * A leaf node is a key node that has either no value or a single value.
+     * A non-leaf node would be a key node with key sub-nodes.
+     *
+     * This is used for entryList() and subkeyList() implementation.
+     */
     virtual bool isLeaf() const = 0;
 
     /**
-       restores property hierarchy to given Dom node
-
-       Used for restoring properties from project file
-    */
+     * restores property hierarchy to given Dom node
+     *
+     *  Used for restoring properties from project file
+     */
     virtual bool readXML( QDomNode & keyNode ) = 0;
 
     /**
-       adds property hierarchy to given Dom element
-
-       Used for saving properties to project file.
-
-       @param nodeName the tag name associated with this element
-       @param element the parent (or encompassing) property element
-       @param document the overall project file Dom document
-    */
+     * adds property hierarchy to given Dom element
+     *
+     * Used for saving properties to project file.
+     *
+     * @param nodeName the tag name associated with this element
+     * @param element the parent (or encompassing) property element
+     * @param document the overall project file Dom document
+     */
     virtual bool writeXML( const QString & nodeName,
                            QDomElement   & element,
                            QDomDocument  & document ) = 0;
 
-    /** return the node's value
-
-       For QgsPropertyValue nodes, this is straightforward -- just return the
-       embedded QVariant, _value.  For QgsPropertyKey, this means returning
-       the QgsPropertyValue _value that is keyed by its name, if it exists;
-       i.e., QgsPropertyKey "foo" will return the property value mapped to its
-       name, "foo", in its QHash of QProperties.
-
+    /** Return the node's value
+     *
+     * For QgsPropertyValue nodes, this is straightforward -- just return the
+     * embedded QVariant, _value.  For QgsPropertyKey, this means returning
+     * the QgsPropertyValue _value that is keyed by its name, if it exists;
+     * i.e., QgsPropertyKey "foo" will return the property value mapped to its
+     * name, "foo", in its QHash of QProperties.
+     *
      */
     virtual QVariant value() const = 0;
 
@@ -125,21 +120,21 @@ class CORE_EXPORT QgsPropertyValue : public QgsProperty
         : value_( value )
     {}
 
-    virtual ~QgsPropertyValue() {}
+    virtual ~QgsPropertyValue();
 
-    /** returns true if is a QgsPropertyKey */
+    /** Returns true if is a QgsPropertyKey */
     virtual bool isKey() const override { return false; }
 
-    /** returns true if is a QgsPropertyValue */
+    /** Returns true if is a QgsPropertyValue */
     virtual bool isValue() const override { return true; }
 
     QVariant value() const override { return value_; }
 
-    /** returns true if is a leaf node
-
-    @note I suppose, in a way, value nodes can also be qualified as leaf
-    nodes even though we're only counting key nodes.
-    */
+    /** Returns true if is a leaf node
+     *
+     * @note I suppose, in a way, value nodes can also be qualified as leaf
+     * nodes even though we're only counting key nodes.
+     */
     bool isLeaf() const override { return true; }
 
     void dump( int tabs = 0 ) const override;
@@ -152,18 +147,17 @@ class CORE_EXPORT QgsPropertyValue : public QgsProperty
 
     int count() const { return 0; }
 
-    /** return keys that do not contain other keys
-
-    Since QgsPropertyValue isn't a key, don't do anything.
-    */
+    /** Return keys that do not contain other keys
+     * Since QgsPropertyValue isn't a key, don't do anything.
+     */
     void entryList( QStringList & keyName, QStringList & entries ) const
     { Q_UNUSED( keyName ); Q_UNUSED( entries ); /* NOP */ }
 
   private:
 
     /** We use QVariant as it's very handy to keep multiple types and provides
-        type conversions
-    */
+     * type conversions
+     */
     QVariant value_;
 
 }; // class QgsPropertyValue
@@ -191,24 +185,25 @@ class CORE_EXPORT QgsPropertyKey : public QgsProperty
 {
   public:
     QgsPropertyKey( const QString &name = "" );
-    virtual ~ QgsPropertyKey();
+    virtual ~QgsPropertyKey();
 
     /// every key has a name
     // @{
-    const QString &name() const { return mName; }
+    // @note not available in python bindings
+    QString name() const { return mName; }
 
     QString &name() { return mName; }
     // @}
 
 
-    /** if this key has a value, it will be stored by its name in its
+    /** If this key has a value, it will be stored by its name in its
      * properties
      */
     QVariant value() const override;
 
 
     /// add the given property key
-    QgsPropertyKey * addKey( const QString & keyName )
+    QgsPropertyKey *addKey( const QString & keyName )
     {
       delete mProperties.take( keyName );
       mProperties.insert( keyName, new QgsPropertyKey( keyName ) );
@@ -223,11 +218,11 @@ class CORE_EXPORT QgsPropertyKey : public QgsProperty
       delete mProperties.take( keyName );
     }
 
-    /** set the value associated with this key
-        @param name is the key name
-        @param value is the value to set
-    @return pointer to property value
-    */
+    /** Set the value associated with this key
+     * @param name is the key name
+     * @param value is the value to set
+     * @return pointer to property value
+     */
     QgsPropertyValue * setValue( const QString & name, const QVariant & value )
     {
       delete mProperties.take( name );
@@ -236,18 +231,17 @@ class CORE_EXPORT QgsPropertyKey : public QgsProperty
       return dynamic_cast<QgsPropertyValue*>( mProperties.value( name ) );
     }
 
-    /** set the value associated with this key
-
-    @note that the single value node associated with each key is always
-    stored keyed by the current key name
-    */
+    /** Set the value associated with this key
+     *
+     * @note that the single value node associated with each key is always
+     * stored keyed by the current key name
+     */
     QgsPropertyValue * setValue( const QVariant & value )
     {
       return setValue( name(), value );
     }
 
 
-
     void dump( int tabs = 0 ) const override;
 
     bool readXML( QDomNode & keyNode ) override;
@@ -260,10 +254,10 @@ class CORE_EXPORT QgsPropertyKey : public QgsProperty
     /// Does this property not have any subkeys or values?
     /* virtual */ bool isEmpty() const { return mProperties.isEmpty(); }
 
-    /** returns true if is a QgsPropertyKey */
+    /** Returns true if is a QgsPropertyKey */
     virtual bool isKey() const override { return true; }
 
-    /** returns true if is a QgsPropertyValue */
+    /** Returns true if is a QgsPropertyValue */
     virtual bool isValue() const override { return false; }
 
     /// return keys that do not contain other keys
@@ -272,11 +266,10 @@ class CORE_EXPORT QgsPropertyKey : public QgsProperty
     /// return keys that contain other keys
     void subkeyList( QStringList & entries ) const;
 
-    /** returns true if a leaf node
-
-    A leaf node is a key node that has either no value or a single value.  A
-    non-leaf node would be a key node with key sub-nodes.
-    */
+    /** Returns true if a leaf node
+     * A leaf node is a key node that has either no value or a single value.
+     * A non-leaf node would be a key node with key sub-nodes.
+     */
     bool isLeaf() const override;
 
     /// reset the QgsProperty key to prestine state
