@@ -35,7 +35,7 @@ QgsRectangle::QgsRectangle( double newxmin, double newymin, double newxmax, doub
   normalize();
 }
 
-QgsRectangle::QgsRectangle( QgsPoint const & p1, QgsPoint const & p2 )
+QgsRectangle::QgsRectangle( const QgsPoint& p1, const QgsPoint& p2 )
 {
   set( p1, p2 );
 }
@@ -126,6 +126,26 @@ void QgsRectangle::scale( double scaleFactor, double centerX, double centerY )
   ymax = centerY + newHeight / 2.0;
 }
 
+void QgsRectangle::grow( double delta )
+{
+  xmin -= delta;
+  xmax += delta;
+  ymin -= delta;
+  ymax += delta;
+}
+
+void QgsRectangle::include( const QgsPoint &p )
+{
+  if ( p.x() < xMinimum() )
+    setXMinimum( p.x() );
+  else if ( p.x() > xMaximum() )
+    setXMaximum( p.x() );
+  if ( p.y() < yMinimum() )
+    setYMinimum( p.y() );
+  if ( p.y() > yMaximum() )
+    setYMaximum( p.y() );
+}
+
 QgsRectangle QgsRectangle::buffer( double width )
 {
   return QgsRectangle( xmin - width, ymin - width, xmax + width, ymax + width );
@@ -201,16 +221,16 @@ bool QgsRectangle::isEmpty() const
 bool QgsRectangle::isNull() const
 {
   // rectangle created QgsRectangle() or with rect.setMinimal() ?
-  return ( xmin == 0 && xmax == 0 && ymin == 0 && ymax == 0 ) ||
-         ( xmin == std::numeric_limits<double>::max() && ymin == std::numeric_limits<double>::max() &&
-           xmax == -std::numeric_limits<double>::max() && ymax == -std::numeric_limits<double>::max() );
+  return ( qgsDoubleNear( xmin, 0.0 ) && qgsDoubleNear( xmax, 0.0 ) && qgsDoubleNear( ymin, 0.0 ) && qgsDoubleNear( ymax, 0.0 ) ) ||
+         ( qgsDoubleNear( xmin, std::numeric_limits<double>::max() ) && qgsDoubleNear( ymin, std::numeric_limits<double>::max() ) &&
+           qgsDoubleNear( xmax, -std::numeric_limits<double>::max() ) && qgsDoubleNear( ymax, -std::numeric_limits<double>::max() ) );
 }
 
 QString QgsRectangle::asWktCoordinates() const
 {
   QString rep =
-    qgsDoubleToString( xmin ) + " " + qgsDoubleToString( ymin ) + ", " +
-    qgsDoubleToString( xmax ) + " " + qgsDoubleToString( ymax );
+    qgsDoubleToString( xmin ) + ' ' + qgsDoubleToString( ymin ) + ", " +
+    qgsDoubleToString( xmax ) + ' ' + qgsDoubleToString( ymax );
 
   return rep;
 }
@@ -219,11 +239,11 @@ QString QgsRectangle::asWktPolygon() const
 {
   QString rep =
     QString( "POLYGON((" ) +
-    qgsDoubleToString( xmin ) + " " + qgsDoubleToString( ymin ) + ", " +
-    qgsDoubleToString( xmax ) + " " + qgsDoubleToString( ymin ) + ", " +
-    qgsDoubleToString( xmax ) + " " + qgsDoubleToString( ymax ) + ", " +
-    qgsDoubleToString( xmin ) + " " + qgsDoubleToString( ymax ) + ", " +
-    qgsDoubleToString( xmin ) + " " + qgsDoubleToString( ymin ) +
+    qgsDoubleToString( xmin ) + ' ' + qgsDoubleToString( ymin ) + ", " +
+    qgsDoubleToString( xmax ) + ' ' + qgsDoubleToString( ymin ) + ", " +
+    qgsDoubleToString( xmax ) + ' ' + qgsDoubleToString( ymax ) + ", " +
+    qgsDoubleToString( xmin ) + ' ' + qgsDoubleToString( ymax ) + ", " +
+    qgsDoubleToString( xmin ) + ' ' + qgsDoubleToString( ymin ) +
     QString( "))" );
 
   return rep;
@@ -232,7 +252,7 @@ QString QgsRectangle::asWktPolygon() const
 //! returns a QRectF with same coordinates.
 QRectF QgsRectangle::toRectF() const
 {
-  return QRectF(( qreal )xmin, ( qreal )ymin, ( qreal )xmax - xmin, ( qreal )ymax - ymin );
+  return QRectF( static_cast< qreal >( xmin ), static_cast< qreal >( ymin ), static_cast< qreal >( xmax - xmin ), static_cast< qreal >( ymax - ymin ) );
 }
 
 // Return a string representation of the rectangle with automatic or high precision
@@ -288,11 +308,11 @@ QString QgsRectangle::asPolygon() const
   // NOTE: a polygon isn't a polygon unless its closed. In the case of
   //       a rectangle, that means 5 points (last == first)
   foo
-  << xmin << " " << ymin << ", "
-  << xmin << " " << ymax << ", "
-  << xmax << " " << ymax << ", "
-  << xmax << " " << ymin << ", "
-  << xmin << " " << ymin;
+  << xmin << ' ' << ymin << ", "
+  << xmin << ' ' << ymax << ", "
+  << xmax << ' ' << ymax << ", "
+  << xmax << ' ' << ymin << ", "
+  << xmin << ' ' << ymin;
 
   return rep;
 
@@ -301,10 +321,10 @@ QString QgsRectangle::asPolygon() const
 
 bool QgsRectangle::operator==( const QgsRectangle & r1 ) const
 {
-  return r1.xMaximum() == xMaximum() &&
-         r1.xMinimum() == xMinimum() &&
-         r1.yMaximum() == yMaximum() &&
-         r1.yMinimum() == yMinimum();
+  return qgsDoubleNear( r1.xMaximum(), xMaximum() ) &&
+         qgsDoubleNear( r1.xMinimum(), xMinimum() ) &&
+         qgsDoubleNear( r1.yMaximum(), yMaximum() ) &&
+         qgsDoubleNear( r1.yMinimum(), yMinimum() );
 }
 
 
@@ -356,6 +376,27 @@ bool QgsRectangle::isFinite() const
 void QgsRectangle::invert()
 {
   double tmp;
-  tmp = xmin; xmin = ymin; ymin = tmp;
-  tmp = xmax; xmax = ymax; ymax = tmp;
+  tmp = xmin;
+  xmin = ymin;
+  ymin = tmp;
+  tmp = xmax;
+  xmax = ymax;
+  ymax = tmp;
+}
+
+QDataStream& operator<<( QDataStream& out, const QgsRectangle& rectangle )
+{
+  out << rectangle.xMinimum() << rectangle.yMinimum() << rectangle.xMaximum() << rectangle.yMaximum();
+  return out;
+}
+
+QDataStream& operator>>( QDataStream& in, QgsRectangle& rectangle )
+{
+  double xmin, ymin, xmax, ymax;
+  in >> xmin >> ymin >> xmax >> ymax;
+  rectangle.setXMinimum( xmin );
+  rectangle.setYMinimum( ymin );
+  rectangle.setXMaximum( xmax );
+  rectangle.setYMaximum( ymax );
+  return in;
 }
