@@ -25,7 +25,7 @@
 #include <qgsmaprenderer.h>
 #include <qgslogger.h>
 #include <qgsmapsettings.h>
-#include "qgsdartmeasurement.h"
+#include <qgsdartmeasurement.h>
 
 class QImage;
 
@@ -41,11 +41,12 @@ class CORE_EXPORT QgsRenderChecker
     QgsRenderChecker();
 
     //! Destructor
-    ~QgsRenderChecker() {};
+    ~QgsRenderChecker() {}
 
     QString controlImagePath() const;
 
-    QString report() { return mReport; };
+    QString report() { return mReport; }
+
     float matchPercent()
     {
       return static_cast<float>( mMismatchCount ) /
@@ -55,25 +56,25 @@ class CORE_EXPORT QgsRenderChecker
     unsigned int matchTarget() { return mMatchTarget; }
     //only records time for actual render part
     int elapsedTime() { return mElapsedTime; }
-    void setElapsedTimeTarget( int theTarget ) { mElapsedTimeTarget = theTarget; };
+    void setElapsedTimeTarget( int theTarget ) { mElapsedTimeTarget = theTarget; }
 
     /** Base directory name for the control image (with control image path
-      * suffixed) the path to the image will be constructed like this:
-      * controlImagePath + '/' + mControlName + '/' + mControlName + '.png'
-      */
+     * suffixed) the path to the image will be constructed like this:
+     * controlImagePath + '/' + mControlName + '/' + mControlName + '.png'
+     */
     void setControlName( const QString &theName );
 
     /** Prefix where the control images are kept.
      * This will be appended to controlImagePath
-      */
-    void setControlPathPrefix( const QString &theName ) { mControlPathPrefix = theName + QDir::separator(); }
+     */
+    void setControlPathPrefix( const QString &theName ) { mControlPathPrefix = theName + '/'; }
 
-    void setControlPathSuffix( const QString& theName ) { mControlPathSuffix = theName + QDir::separator(); }
+    void setControlPathSuffix( const QString& theName );
 
     /** Get an md5 hash that uniquely identifies an image */
-    QString imageToHash( QString theImageFile );
+    QString imageToHash( const QString& theImageFile );
 
-    void setRenderedImage( QString theImageFileName ) { mRenderedImageFile = theImageFileName; }
+    void setRenderedImage( const QString& theImageFileName ) { mRenderedImageFile = theImageFileName; }
 
     /**
      * The path of the rendered image can be retrieved through that method.
@@ -81,10 +82,10 @@ class CORE_EXPORT QgsRenderChecker
      *
      * @return The path to the rendered image
      */
-    const QString& renderedImage() { return mRenderedImageFile; }
+    QString renderedImage() { return mRenderedImageFile; }
 
     //! @deprecated since 2.4 - use setMapSettings()
-    Q_DECL_DEPRECATED void setMapRenderer( QgsMapRenderer *  thepMapRenderer );
+    Q_DECL_DEPRECATED void setMapRenderer( QgsMapRenderer *thepMapRenderer );
 
     //! @note added in 2.4
     void setMapSettings( const QgsMapSettings& mapSettings );
@@ -96,6 +97,14 @@ class CORE_EXPORT QgsRenderChecker
      * @note added in 2.1
      */
     void setColorTolerance( unsigned int theColorTolerance ) { mColorTolerance = theColorTolerance; }
+
+    /** Sets the largest allowable difference in size between the rendered and the expected image.
+     * @param xTolerance x tolerance in pixels
+     * @param yTolerance y tolerance in pixels
+     * @note added in QGIS 2.12
+     */
+    void setSizeTolerance( int xTolerance, int yTolerance ) { mMaxSizeDifferenceX = xTolerance; mMaxSizeDifferenceY = yTolerance; }
+
     /**
      * Test using renderer to generate the image to be compared.
      * @param theTestName - to be used as the basis for writing a file to
@@ -106,7 +115,7 @@ class CORE_EXPORT QgsRenderChecker
      * for that by providing a tolerance.
      * @note make sure to call setExpectedImage and setMapRenderer first
      */
-    bool runTest( QString theTestName, unsigned int theMismatchCount = 0 );
+    bool runTest( const QString& theTestName, unsigned int theMismatchCount = 0 );
 
     /**
      * Test using two arbitary images (map renderer will not be used)
@@ -119,17 +128,17 @@ class CORE_EXPORT QgsRenderChecker
      * @param theRenderedImageFile to optionally override the output filename
      * @note: make sure to call setExpectedImage and setRenderedImage first.
      */
-    bool compareImages( QString theTestName, unsigned int theMismatchCount = 0, QString theRenderedImageFile = "" );
+    bool compareImages( const QString& theTestName, unsigned int theMismatchCount = 0, const QString& theRenderedImageFile = "" );
     /** Get a list of all the anomalies. An anomaly is a rendered difference
-      * file where there is some red pixel content (indicating a render check
-      * mismatch), but where the output was still acceptible. If the render
-      * diff matches one of these anomalies we will still consider it to be
-      * acceptible.
-      * @return a bool indicating if the diff matched one of the anomaly files
-    */
-    bool isKnownAnomaly( QString theDiffImageFile );
+     * file where there is some red pixel content (indicating a render check
+     * mismatch), but where the output was still acceptible. If the render
+     * diff matches one of these anomalies we will still consider it to be
+     * acceptible.
+     * @return a bool indicating if the diff matched one of the anomaly files
+     */
+    bool isKnownAnomaly( const QString& theDiffImageFile );
 
-    /**Draws a checkboard pattern for image backgrounds, so that transparency is visible
+    /** Draws a checkboard pattern for image backgrounds, so that transparency is visible
      * without requiring a transparent background for the image
      */
     static void drawBackground( QImage* image );
@@ -139,7 +148,7 @@ class CORE_EXPORT QgsRenderChecker
      *
      * @return Path to the expected image file
      */
-    const QString& expectedImageFile() const { return mExpectedImageFile; }
+    QString expectedImageFile() const { return mExpectedImageFile; }
 
     /**
      * Call this to enable internal buffering of dash messages. You may later call
@@ -157,7 +166,7 @@ class CORE_EXPORT QgsRenderChecker
      * @return buffered dash messages
      * @note not available in python bindings
      */
-    const QVector<QgsDartMeasurement>& dartMeasurements() const { return mDashMessages; }
+    QVector<QgsDartMeasurement> dartMeasurements() const { return mDashMessages; }
 
   protected:
     QString mReport;
@@ -173,6 +182,8 @@ class CORE_EXPORT QgsRenderChecker
     QString mControlName;
     unsigned int mMismatchCount;
     unsigned int mColorTolerance;
+    int mMaxSizeDifferenceX;
+    int mMaxSizeDifferenceY;
     int mElapsedTimeTarget;
     QgsMapSettings mMapSettings;
     QString mControlPathPrefix;
@@ -189,16 +200,16 @@ class CORE_EXPORT QgsRenderChecker
  * @return bool indicating if the WKT are sufficiently equal
  */
 
-inline bool compareWkt( QString a, QString b, double tolerance = 0.000001 )
+inline bool compareWkt( const QString& a, const QString& b, double tolerance = 0.000001 )
 {
-  QgsDebugMsg( QString( "a:%1 b:%2 tol:%3" ).arg( a ).arg( b ).arg( tolerance ) );
+  QgsDebugMsg( QString( "a:%1 b:%2 tol:%3" ).arg( a, b ).arg( tolerance ) );
   QRegExp re( "-?\\d+(?:\\.\\d+)?(?:[eE]\\d+)?" );
 
   QString a0( a ), b0( b );
   a0.replace( re, "#" );
   b0.replace( re, "#" );
 
-  QgsDebugMsg( QString( "a0:%1 b0:%2" ).arg( a0 ).arg( b0 ) );
+  QgsDebugMsg( QString( "a0:%1 b0:%2" ).arg( a0, b0 ) );
 
   if ( a0 != b0 )
     return false;
