@@ -84,7 +84,7 @@ void QgsVectorLayerEditBuffer::updateFeatureGeometry( QgsFeature &f )
 
 void QgsVectorLayerEditBuffer::updateChangedAttributes( QgsFeature &f )
 {
-  QgsAttributes& attrs = f.attributes();
+  QgsAttributes attrs = f.attributes();
 
   // remove all attributes that will disappear - from higher indices to lower
   for ( int idx = mDeletedAttributeIds.count() - 1; idx >= 0; --idx )
@@ -102,6 +102,8 @@ void QgsVectorLayerEditBuffer::updateChangedAttributes( QgsFeature &f )
     for ( QgsAttributeMap::const_iterator it = map.begin(); it != map.end(); ++it )
       attrs[it.key()] = it.value();
   }
+
+  f.setAttributes( attrs );
 }
 
 
@@ -159,6 +161,17 @@ bool QgsVectorLayerEditBuffer::deleteFeature( QgsFeatureId fid )
   return true;
 }
 
+bool QgsVectorLayerEditBuffer::deleteFeatures( const QgsFeatureIds& fids )
+{
+  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures ) )
+    return false;
+
+  Q_FOREACH ( QgsFeatureId fid, fids )
+    deleteFeature( fid );
+
+  return true;
+}
+
 
 bool QgsVectorLayerEditBuffer::changeGeometry( QgsFeatureId fid, QgsGeometry* geom )
 {
@@ -194,9 +207,9 @@ bool QgsVectorLayerEditBuffer::changeAttributeValue( QgsFeatureId fid, int field
     return false;
   }
 
-  if ( field < 0 || field >= L->pendingFields().count() ||
-       L->pendingFields().fieldOrigin( field ) == QgsFields::OriginJoin ||
-       L->pendingFields().fieldOrigin( field ) == QgsFields::OriginExpression )
+  if ( field < 0 || field >= L->fields().count() ||
+       L->fields().fieldOrigin( field ) == QgsFields::OriginJoin ||
+       L->fields().fieldOrigin( field ) == QgsFields::OriginExpression )
     return false;
 
   L->undoStack()->push( new QgsVectorLayerUndoCommandChangeAttribute( this, fid, field, newValue, oldValue ) );
@@ -212,7 +225,7 @@ bool QgsVectorLayerEditBuffer::addAttribute( const QgsField &field )
   if ( field.name().isEmpty() )
     return false;
 
-  const QgsFields& updatedFields = L->pendingFields();
+  const QgsFields& updatedFields = L->fields();
   for ( int idx = 0; idx < updatedFields.count(); ++idx )
   {
     if ( updatedFields[idx].name() == field.name() )
@@ -232,12 +245,12 @@ bool QgsVectorLayerEditBuffer::deleteAttribute( int index )
   if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes ) )
     return false;
 
-  if ( index < 0 || index >= L->pendingFields().count() )
+  if ( index < 0 || index >= L->fields().count() )
     return false;
 
   // find out source of the field
-  QgsFields::FieldOrigin origin = L->pendingFields().fieldOrigin( index );
-  int originIndex = L->pendingFields().fieldOriginIndex( index );
+  QgsFields::FieldOrigin origin = L->fields().fieldOrigin( index );
+  int originIndex = L->fields().fieldOriginIndex( index );
 
   if ( origin == QgsFields::OriginProvider && mDeletedAttributeIds.contains( originIndex ) )
     return false;
@@ -258,7 +271,31 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
   int cap = provider->capabilities();
   bool success = true;
 
-  QgsFields oldFields = L->pendingFields();
+  // geometry updates   attribute updates
+  // yes                no                    => changeGeometryValues
+  // no                 yes                   => changeAttributeValues
+  // yes                yes                   => changeFeatures
+
+  //
+  // update geometries
+  //
+  if ( !mChangedGeometries.isEmpty() && (( cap & QgsVectorDataProvider::ChangeFeatures ) == 0 || mChangedAttributeValues.isEmpty() ) )
+  {
+    if ( provider->changeGeometryValues( mChangedGeometries ) )
+    {
+      commitErrors << tr( "SUCCESS: %n geometries were changed.", "changed geometries count", mChangedGeometries.size() );
+
+      emit committedGeometriesChanges( L->id(), mChangedGeometries );
+      mChangedGeometries.clear();
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %n geometries not changed.", "not changed geometries count", mChangedGeometries.size() );
+      success = false;
+    }
+  }
+
+  QgsFields oldFields = L->fields();
 
   //
   // delete attributes
@@ -280,9 +317,9 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
       commitErrors << tr( "ERROR: %n attribute(s) not deleted.", "not deleted attributes count", mDeletedAttributeIds.size() );
 #if 0
       QString list = "ERROR: Pending attribute deletes:";
-      foreach ( int idx, mDeletedAttributeIds )
+      Q_FOREACH ( int idx, mDeletedAttributeIds )
       {
-        list.append( " " + L->pendingFields()[idx].name() );
+        list.append( ' ' + L->pendingFields().at( idx ).name() );
       }
       commitErrors << list;
 #endif
@@ -309,9 +346,9 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
       commitErrors << tr( "ERROR: %n new attribute(s) not added", "not added attributes count", mAddedAttributes.size() );
 #if 0
       QString list = "ERROR: Pending adds:";
-      foreach ( QgsField f, mAddedAttributes )
+      Q_FOREACH ( QgsField f, mAddedAttributes )
       {
-        list.append( " " + f.name() );
+        list.append( ' ' + f.name() );
       }
       commitErrors << list;
 #endif
@@ -326,7 +363,7 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
   if ( attributesChanged )
   {
     L->updateFields();
-    QgsFields newFields = L->pendingFields();
+    QgsFields newFields = L->fields();
 
     if ( oldFields.count() != newFields.count() )
     {
@@ -336,8 +373,8 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
 
     for ( int i = 0; i < qMin( oldFields.count(), newFields.count() ); ++i )
     {
-      const QgsField& oldField = oldFields[i];
-      const QgsField& newField = newFields[i];
+      const QgsField& oldField = oldFields.at( i );
+      const QgsField& newField = newFields.at( i );
       if ( attributeChangesOk && oldField != newField )
       {
         commitErrors
@@ -345,17 +382,17 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
         << tr( "Provider: %1" ).arg( L->providerType() )
         << tr( "Storage: %1" ).arg( L->storageType() )
         << QString( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
-        .arg( tr( "expected field" ) )
-        .arg( oldField.name() )
-        .arg( QVariant::typeToName( oldField.type() ) )
-        .arg( oldField.typeName() )
+        .arg( tr( "expected field" ),
+              oldField.name(),
+              QVariant::typeToName( oldField.type() ),
+              oldField.typeName() )
         .arg( oldField.length() )
         .arg( oldField.precision() )
         << QString( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
-        .arg( tr( "retrieved field" ) )
-        .arg( newField.name() )
-        .arg( QVariant::typeToName( newField.type() ) )
-        .arg( newField.typeName() )
+        .arg( tr( "retrieved field" ),
+              newField.name(),
+              QVariant::typeToName( newField.type() ),
+              newField.typeName() )
         .arg( newField.length() )
         .arg( newField.precision() );
         attributeChangesOk = false;   // don't try attribute updates - they'll fail.
@@ -365,38 +402,58 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
 
   if ( attributeChangesOk )
   {
-    //
-    // change attributes
-    //
-    if ( !mChangedAttributeValues.isEmpty() )
+    if ( cap & QgsVectorDataProvider::ChangeFeatures && !mChangedGeometries.isEmpty() && !mChangedAttributeValues.isEmpty() )
     {
-      if (( cap & QgsVectorDataProvider::ChangeAttributeValues ) && provider->changeAttributeValues( mChangedAttributeValues ) )
+      Q_ASSERT(( cap & ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) ) == ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) );
+
+      if ( provider->changeFeatures( mChangedAttributeValues, mChangedGeometries ) )
       {
-        commitErrors << tr( "SUCCESS: %n attribute value(s) changed.", "changed attribute values count", mChangedAttributeValues.size() );
-
+        commitErrors << tr( "SUCCESS: %1 attribute value(s) and %2 geometries changed." ).arg( mChangedAttributeValues.size(), mChangedGeometries.size() );
         emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
-
         mChangedAttributeValues.clear();
+
+        emit committedGeometriesChanges( L->id(), mChangedGeometries );
+        mChangedGeometries.clear();
       }
       else
       {
-        commitErrors << tr( "ERROR: %n attribute value change(s) not applied.", "not changed attribute values count", mChangedAttributeValues.size() );
-#if 0
-        QString list = "ERROR: pending changes:";
-        foreach ( QgsFeatureId id, mChangedAttributeValues.keys() )
-        {
-          list.append( "\n  " + FID_TO_STRING( id ) + "[" );
-          foreach ( int idx, mChangedAttributeValues[ id ].keys() )
-          {
-            list.append( QString( " %1:%2" ).arg( L->pendingFields()[idx].name() ).arg( mChangedAttributeValues[id][idx].toString() ) );
-          }
-          list.append( " ]" );
-        }
-        commitErrors << list;
-#endif
         success = false;
       }
     }
+    else
+    {
+      //
+      // change attributes
+      //
+      if ( !mChangedAttributeValues.isEmpty() && (( cap & QgsVectorDataProvider::ChangeFeatures ) == 0 || mChangedGeometries.isEmpty() ) )
+      {
+        if (( cap & QgsVectorDataProvider::ChangeAttributeValues ) && provider->changeAttributeValues( mChangedAttributeValues ) )
+        {
+          commitErrors << tr( "SUCCESS: %n attribute value(s) changed.", "changed attribute values count", mChangedAttributeValues.size() );
+
+          emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
+          mChangedAttributeValues.clear();
+        }
+        else
+        {
+          commitErrors << tr( "ERROR: %n attribute value change(s) not applied.", "not changed attribute values count", mChangedAttributeValues.size() );
+#if 0
+          QString list = "ERROR: pending changes:";
+          Q_FOREACH ( QgsFeatureId id, mChangedAttributeValues.keys() )
+          {
+            list.append( "\n  " + FID_TO_STRING( id ) + '[' );
+            Q_FOREACH ( int idx, mChangedAttributeValues[ id ].keys() )
+            {
+              list.append( QString( " %1:%2" ).arg( L->pendingFields().at( idx ).name() ).arg( mChangedAttributeValues[id][idx].toString() ) );
+            }
+            list.append( " ]" );
+          }
+          commitErrors << list;
+#endif
+          success = false;
+        }
+      }
+    }
 
     //
     // delete features
@@ -407,10 +464,10 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
       {
         commitErrors << tr( "SUCCESS: %n feature(s) deleted.", "deleted features count", mDeletedFeatureIds.size() );
         // TODO[MD]: we should not need this here
-        for ( QgsFeatureIds::const_iterator it = mDeletedFeatureIds.begin(); it != mDeletedFeatureIds.end(); ++it )
+        Q_FOREACH ( QgsFeatureId id, mDeletedFeatureIds )
         {
-          mChangedAttributeValues.remove( *it );
-          mChangedGeometries.remove( *it );
+          mChangedAttributeValues.remove( id );
+          mChangedGeometries.remove( id );
         }
 
         emit committedFeaturesRemoved( L->id(), mDeletedFeatureIds );
@@ -422,9 +479,9 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
         commitErrors << tr( "ERROR: %n feature(s) not deleted.", "not deleted features count", mDeletedFeatureIds.size() );
 #if 0
         QString list = "ERROR: pending deletes:";
-        foreach ( QgsFeatureId id, mDeletedFeatureIds )
+        Q_FOREACH ( QgsFeatureId id, mDeletedFeatureIds )
         {
-          list.append( " " + FID_TO_STRING( id ) );
+          list.append( ' ' + FID_TO_STRING( id ) );
         }
         commitErrors << list;
 #endif
@@ -474,12 +531,12 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
           commitErrors << tr( "ERROR: %n feature(s) not added.", "not added features count", mAddedFeatures.size() );
 #if 0
           QString list = "ERROR: pending adds:";
-          foreach ( QgsFeature f, mAddedFeatures )
+          Q_FOREACH ( QgsFeature f, mAddedFeatures )
           {
-            list.append( " " + FID_TO_STRING( f.id() ) + "[" );
+            list.append( ' ' + FID_TO_STRING( f.id() ) + '[' );
             for ( int i = 0; i < L->pendingFields().size(); i++ )
             {
-              list.append( QString( " %1:%2" ).arg( L->pendingFields()[i].name() ).arg( f.attributes()[i].toString() ) );
+              list.append( QString( " %1:%2" ).arg( L->pendingFields().at( i ).name() ).arg( f.attributes()[i].toString() ) );
             }
             list.append( " ]" );
           }
@@ -500,32 +557,12 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList& commitErrors )
     success = false;
   }
 
-  //
-  // update geometries
-  //
-  if ( success && !mChangedGeometries.isEmpty() )
-  {
-    if (( cap & QgsVectorDataProvider::ChangeGeometries ) && provider->changeGeometryValues( mChangedGeometries ) )
-    {
-      commitErrors << tr( "SUCCESS: %n geometries were changed.", "changed geometries count", mChangedGeometries.size() );
-
-      emit committedGeometriesChanges( L->id(), mChangedGeometries );
-
-      mChangedGeometries.clear();
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n geometries not changed.", "not changed geometries count", mChangedGeometries.size() );
-      success = false;
-    }
-  }
-
   if ( !success && provider->hasErrors() )
   {
     commitErrors << tr( "\n  Provider errors:" );
-    foreach ( QString e, provider->errors() )
+    Q_FOREACH ( QString e, provider->errors() )
     {
-      commitErrors << "    " + e.replace( "\n", "\n    " );
+      commitErrors << "    " + e.replace( '\n', "\n    " );
     }
     provider->clearErrors();
   }
@@ -570,26 +607,29 @@ QString QgsVectorLayerEditBuffer::dumpEditBuffer()
 void QgsVectorLayerEditBuffer::handleAttributeAdded( int index )
 {
   // go through the changed attributes map and adapt indices
-  for ( int i = 0; i < mChangedAttributeValues.size(); ++i )
+  QgsChangedAttributesMap::iterator it = mChangedAttributeValues.begin();
+  for ( ; it != mChangedAttributeValues.end(); ++it )
   {
-    updateAttributeMapIndex( mChangedAttributeValues[i], index, + 1 );
+    updateAttributeMapIndex( it.value(), index, + 1 );
   }
 
   // go through added features and adapt attributes
   QgsFeatureMap::iterator featureIt = mAddedFeatures.begin();
   for ( ; featureIt != mAddedFeatures.end(); ++featureIt )
   {
-    QgsAttributes& attrs = featureIt->attributes();
+    QgsAttributes attrs = featureIt->attributes();
     attrs.insert( index, QVariant() );
+    featureIt->setAttributes( attrs );
   }
 }
 
 void QgsVectorLayerEditBuffer::handleAttributeDeleted( int index )
 {
   // go through the changed attributes map and adapt indices
-  for ( int i = 0; i < mChangedAttributeValues.size(); ++i )
+  QgsChangedAttributesMap::iterator it = mChangedAttributeValues.begin();
+  for ( ; it != mChangedAttributeValues.end(); ++it )
   {
-    QgsAttributeMap& attrMap = mChangedAttributeValues[i];
+    QgsAttributeMap& attrMap = it.value();
     // remove the attribute
     if ( attrMap.contains( index ) )
       attrMap.remove( index );
@@ -602,8 +642,9 @@ void QgsVectorLayerEditBuffer::handleAttributeDeleted( int index )
   QgsFeatureMap::iterator featureIt = mAddedFeatures.begin();
   for ( ; featureIt != mAddedFeatures.end(); ++featureIt )
   {
-    QgsAttributes& attrs = featureIt->attributes();
+    QgsAttributes attrs = featureIt->attributes();
     attrs.remove( index );
+    featureIt->setAttributes( attrs );
   }
 }
 
