@@ -25,7 +25,7 @@
 #include <QVector>
 
 QgsPalettedRasterRenderer::QgsPalettedRasterRenderer( QgsRasterInterface* input, int bandNumber,
-    QColor* colorArray, int nColors, const QVector<QString> labels ):
+    QColor* colorArray, int nColors, const QVector<QString>& labels ):
     QgsRasterRenderer( input, "paletted" ), mBand( bandNumber ), mNColors( nColors ), mLabels( labels )
 {
   mColors = new QRgb[nColors];
@@ -36,7 +36,7 @@ QgsPalettedRasterRenderer::QgsPalettedRasterRenderer( QgsRasterInterface* input,
   delete[] colorArray;
 }
 
-QgsPalettedRasterRenderer::QgsPalettedRasterRenderer( QgsRasterInterface* input, int bandNumber, QRgb* colorArray, int nColors, const QVector<QString> labels ):
+QgsPalettedRasterRenderer::QgsPalettedRasterRenderer( QgsRasterInterface* input, int bandNumber, QRgb* colorArray, int nColors, const QVector<QString>& labels ):
     QgsRasterRenderer( input, "paletted" ), mBand( bandNumber ), mColors( colorArray ), mNColors( nColors ), mLabels( labels )
 {
 }
@@ -46,12 +46,12 @@ QgsPalettedRasterRenderer::~QgsPalettedRasterRenderer()
   delete[] mColors;
 }
 
-QgsRasterInterface * QgsPalettedRasterRenderer::clone() const
+QgsPalettedRasterRenderer* QgsPalettedRasterRenderer::clone() const
 {
-  QgsPalettedRasterRenderer * renderer = new QgsPalettedRasterRenderer( 0, mBand, rgbArray(), mNColors );
+  QgsPalettedRasterRenderer * renderer = new QgsPalettedRasterRenderer( nullptr, mBand, rgbArray(), mNColors );
   renderer->setOpacity( mOpacity );
   renderer->setAlphaBand( mAlphaBand );
-  renderer->setRasterTransparency( mRasterTransparency ? new QgsRasterTransparency( *mRasterTransparency ) : 0 );
+  renderer->setRasterTransparency( mRasterTransparency ? new QgsRasterTransparency( *mRasterTransparency ) : nullptr );
   renderer->mLabels = mLabels;
   return renderer;
 }
@@ -60,12 +60,12 @@ QgsRasterRenderer* QgsPalettedRasterRenderer::create( const QDomElement& elem, Q
 {
   if ( elem.isNull() )
   {
-    return 0;
+    return nullptr;
   }
 
   int bandNumber = elem.attribute( "band", "-1" ).toInt();
   int nColors = 0;
-  QRgb* colors = 0;
+  QRgb* colors = nullptr;
   QVector<QString> labels;
 
   QDomElement paletteElem = elem.firstChildElement( "colorPalette" );
@@ -85,7 +85,7 @@ QgsRasterRenderer* QgsPalettedRasterRenderer::create( const QDomElement& elem, Q
       value = ( int )entryElem.attribute( "value", "0" ).toDouble();
       if ( value >= nColors && value <= 10000 ) nColors = value + 1;
     }
-    QgsDebugMsg( QString( "nColors = %1" ).arg( nColors ) );
+    QgsDebugMsgLevel( QString( "nColors = %1" ).arg( nColors ), 4 );
 
     colors = new QRgb[ nColors ];
 
@@ -93,7 +93,7 @@ QgsRasterRenderer* QgsPalettedRasterRenderer::create( const QDomElement& elem, Q
     {
       entryElem = paletteEntries.at( i ).toElement();
       value = ( int )entryElem.attribute( "value", "0" ).toDouble();
-      QgsDebugMsg( entryElem.attribute( "color", "#000000" ) );
+      QgsDebugMsgLevel( entryElem.attribute( "color", "#000000" ), 4 );
       if ( value >= 0 && value < nColors )
       {
         colors[value] = QColor( entryElem.attribute( "color", "#000000" ) ).rgba();
@@ -119,7 +119,7 @@ QColor* QgsPalettedRasterRenderer::colors() const
 {
   if ( mNColors < 1 )
   {
-    return 0;
+    return nullptr;
   }
   QColor* colorArray = new QColor[ mNColors ];
   for ( int i = 0; i < mNColors; ++i )
@@ -133,7 +133,7 @@ QRgb* QgsPalettedRasterRenderer::rgbArray() const
 {
   if ( mNColors < 1 )
   {
-    return 0;
+    return nullptr;
   }
   QRgb* rgbValues = new QRgb[mNColors];
   for ( int i = 0; i < mNColors; ++i )
@@ -143,7 +143,7 @@ QRgb* QgsPalettedRasterRenderer::rgbArray() const
   return rgbValues;
 }
 
-void QgsPalettedRasterRenderer::setLabel( int idx, QString label )
+void QgsPalettedRasterRenderer::setLabel( int idx, const QString& label )
 {
   if ( idx >= mLabels.size() )
   {
@@ -173,7 +173,7 @@ QgsRasterBlock * QgsPalettedRasterRenderer::block( int bandNo, QgsRectangle  con
 
   //rendering is faster without considering user-defined transparency
   bool hasTransparency = usesTransparency();
-  QgsRasterBlock *alphaBlock = 0;
+  QgsRasterBlock *alphaBlock = nullptr;
 
   if ( mAlphaBand > 0 && mAlphaBand != mBand )
   {
