@@ -31,7 +31,7 @@ QgsRasterFileWriter::QgsRasterFileWriter( const QString& outputUrl ):
     mTiledMode( false ), mMaxTileWidth( 500 ), mMaxTileHeight( 500 ),
     mBuildPyramidsFlag( QgsRaster::PyramidsFlagNo ),
     mPyramidsFormat( QgsRaster::PyramidsGTiff ),
-    mProgressDialog( 0 ), mPipe( 0 ), mInput( 0 )
+    mProgressDialog( nullptr ), mPipe( nullptr ), mInput( nullptr )
 {
 
 }
@@ -45,14 +45,9 @@ QgsRasterFileWriter::QgsRasterFileWriter()
     , mMaxTileHeight( 500 )
     , mBuildPyramidsFlag( QgsRaster::PyramidsFlagNo )
     , mPyramidsFormat( QgsRaster::PyramidsGTiff )
-    , mProgressDialog( 0 )
-    , mPipe( 0 )
-    , mInput( 0 )
-{
-
-}
-
-QgsRasterFileWriter::~QgsRasterFileWriter()
+    , mProgressDialog( nullptr )
+    , mPipe( nullptr )
+    , mInput( nullptr )
 {
 
 }
@@ -60,7 +55,7 @@ QgsRasterFileWriter::~QgsRasterFileWriter()
 QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeRaster( const QgsRasterPipe* pipe, int nCols, int nRows, QgsRectangle outputExtent,
     const QgsCoordinateReferenceSystem& crs, QProgressDialog* progressDialog )
 {
-  QgsDebugMsg( "Entered" );
+  QgsDebugMsgLevel( "Entered", 4 );
 
   if ( !pipe )
   {
@@ -85,14 +80,17 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeRaster( const QgsRast
     mMode = Raw;
   }
 
-  QgsDebugMsg( QString( "reading from %1" ).arg( typeid( *iface ).name() ) );
+  QgsDebugMsgLevel( QString( "reading from %1" ).arg( typeid( *iface ).name() ), 4 );
 
   if ( !iface->srcInput() )
   {
     QgsDebugMsg( "iface->srcInput() == 0" );
     return SourceProviderError;
   }
-  QgsDebugMsg( QString( "srcInput = %1" ).arg( typeid( *( iface->srcInput() ) ).name() ) );
+#ifdef QGISDEBUG
+  const QgsRasterInterface &srcInput = *iface->srcInput();
+  QgsDebugMsgLevel( QString( "srcInput = %1" ).arg( typeid( srcInput ).name() ), 4 );
+#endif
 
   mProgressDialog = progressDialog;
 
@@ -116,12 +114,12 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeRaster( const QgsRast
   if ( mMode == Image )
   {
     WriterError e = writeImageRaster( &iter, nCols, nRows, outputExtent, crs, progressDialog );
-    mProgressDialog = 0;
+    mProgressDialog = nullptr;
     return e;
   }
   else
   {
-    mProgressDialog = 0;
+    mProgressDialog = nullptr;
     WriterError e = writeDataRaster( pipe, &iter, nCols, nRows, outputExtent, crs, progressDialog );
     return e;
   }
@@ -130,7 +128,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeRaster( const QgsRast
 QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const QgsRasterPipe* pipe, QgsRasterIterator* iter, int nCols, int nRows, const QgsRectangle& outputExtent,
     const QgsCoordinateReferenceSystem& crs, QProgressDialog* progressDialog )
 {
-  QgsDebugMsg( "Entered" );
+  QgsDebugMsgLevel( "Entered", 4 );
   if ( !iter )
   {
     return SourceProviderError;
@@ -186,7 +184,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
     double destNoDataValue = std::numeric_limits<double>::quiet_NaN();
     QGis::DataType destDataType = srcProvider->srcDataType( bandNo );
     // TODO: verify what happens/should happen if srcNoDataValue is disabled by setUseSrcNoDataValue
-    QgsDebugMsg( QString( "srcHasNoDataValue = %1 srcNoDataValue = %2" ).arg( srcHasNoDataValue ).arg( srcProvider->srcNoDataValue( bandNo ) ) );
+    QgsDebugMsgLevel( QString( "srcHasNoDataValue = %1 srcNoDataValue = %2" ).arg( srcHasNoDataValue ).arg( srcProvider->srcNoDataValue( bandNo ) ), 4 );
     if ( srcHasNoDataValue )
     {
 
@@ -194,7 +192,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
       destNoDataValue = srcProvider->srcNoDataValue( bandNo );
       destHasNoDataValue = true;
     }
-    else if ( nuller && nuller->noData( bandNo ).size() > 0 )
+    else if ( nuller && !nuller->noData( bandNo ).isEmpty() )
     {
       // Use one user defined no data value
       destNoDataValue = nuller->noData( bandNo ).value( 0 ).min();
@@ -217,8 +215,8 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
         QgsRasterBandStats stats = srcProvider->bandStatistics( bandNo, QgsRasterBandStats::Min | QgsRasterBandStats::Max, srcExtent, 250000 );
 
         // Test if we have free (not used) values
-        double typeMinValue = QgsContrastEnhancement::maximumValuePossible(( QGis::DataType )srcProvider->srcDataType( bandNo ) );
-        double typeMaxValue = QgsContrastEnhancement::maximumValuePossible(( QGis::DataType )srcProvider->srcDataType( bandNo ) );
+        double typeMinValue = QgsContrastEnhancement::maximumValuePossible( static_cast< QGis::DataType >( srcProvider->srcDataType( bandNo ) ) );
+        double typeMaxValue = QgsContrastEnhancement::maximumValuePossible( static_cast< QGis::DataType >( srcProvider->srcDataType( bandNo ) ) );
         if ( stats.minimumValue > typeMinValue )
         {
           destNoDataValue = typeMinValue;
@@ -241,7 +239,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
       nuller->setOutputNoDataValue( bandNo, destNoDataValue );
     }
 
-    QgsDebugMsg( QString( "bandNo = %1 destDataType = %2 destHasNoDataValue = %3 destNoDataValue = %4" ).arg( bandNo ).arg( destDataType ).arg( destHasNoDataValue ).arg( destNoDataValue ) );
+    QgsDebugMsgLevel( QString( "bandNo = %1 destDataType = %2 destHasNoDataValue = %3 destNoDataValue = %4" ).arg( bandNo ).arg( destDataType ).arg( destHasNoDataValue ).arg( destNoDataValue ), 4 );
     destDataTypeList.append( destDataType );
     destHasNoDataValueList.append( destHasNoDataValue );
     destNoDataValueList.append( destNoDataValue );
@@ -259,7 +257,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
   }
 
   //create destProvider for whole dataset here
-  QgsRasterDataProvider* destProvider = 0;
+  QgsRasterDataProvider* destProvider = nullptr;
   double pixelSize;
   double geoTransform[6];
   globalOutputParameters( outputExtent, nCols, nRows, geoTransform, pixelSize );
@@ -276,7 +274,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster( const Qgs
     {
       destProvider->remove();
       delete destProvider;
-      destProvider = 0;
+      destProvider = nullptr;
     }
     else // VRT
     {
@@ -311,19 +309,19 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
   const QgsRectangle& outputExtent,
   const QgsCoordinateReferenceSystem& crs,
   QGis::DataType destDataType,
-  QList<bool> destHasNoDataValueList,
-  QList<double> destNoDataValueList,
+  const QList<bool>& destHasNoDataValueList,
+  const QList<double>& destNoDataValueList,
   QgsRasterDataProvider* destProvider,
   QProgressDialog* progressDialog )
 {
   Q_UNUSED( pipe );
   Q_UNUSED( destHasNoDataValueList );
-  QgsDebugMsg( "Entered" );
+  QgsDebugMsgLevel( "Entered", 4 );
 
   const QgsRasterInterface* iface = iter->input();
   const QgsRasterDataProvider *srcProvider = dynamic_cast<const QgsRasterDataProvider*>( iface->srcInput() );
   int nBands = iface->bandCount();
-  QgsDebugMsg( QString( "nBands = %1" ).arg( nBands ) );
+  QgsDebugMsgLevel( QString( "nBands = %1" ).arg( nBands ), 4 );
 
   //Get output map units per pixel
   int iterLeft = 0;
@@ -332,10 +330,11 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
   int iterRows = 0;
 
   QList<QgsRasterBlock*> blockList;
+  blockList.reserve( nBands );
   for ( int i = 1; i <= nBands; ++i )
   {
     iter->startRasterRead( i, nCols, nRows, outputExtent );
-    blockList.push_back( 0 );
+    blockList.push_back( nullptr );
     if ( destProvider ) // no tiles
     {
       destProvider->setNoDataValue( i, destNoDataValueList.value( i - 1 ) );
@@ -356,7 +355,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
 
   // hmm why is there a for(;;) here ..
   // not good coding practice IMHO, it might be better to use [ for() and break ] or  [ while (test) ]
-  for ( ;; )
+  Q_FOREVER
   {
     for ( int i = 1; i <= nBands; ++i )
     {
@@ -365,7 +364,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
         // No more parts, create VRT and return
         if ( mTiledMode )
         {
-          QString vrtFilePath( mOutputUrl + "/" + vrtFileName() );
+          QString vrtFilePath( mOutputUrl + '/' + vrtFileName() );
           writeVRT( vrtFilePath );
           if ( mBuildPyramidsFlag == QgsRaster::PyramidsFlagYes )
           {
@@ -380,7 +379,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
           }
         }
 
-        QgsDebugMsg( "Done" );
+        QgsDebugMsgLevel( "Done", 4 );
         return NoError; //reached last tile, bail out
       }
       // TODO: verify if NoDataConflict happened, to do that we need the whole pipe or nuller interface
@@ -415,7 +414,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
         blockList[i-1]->convert( destDataType );
         destBlockList.push_back( blockList[i-1] );
       }
-      blockList[i-1] = 0;
+      blockList[i-1] = nullptr;
     }
 
     if ( mTiledMode ) //write to file
@@ -450,14 +449,14 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeDataRaster(
     ++fileIndex;
   }
 
-  QgsDebugMsg( "Done" );
+  QgsDebugMsgLevel( "Done", 4 );
   return NoError;
 }
 
 QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRasterIterator* iter, int nCols, int nRows, const QgsRectangle& outputExtent,
     const QgsCoordinateReferenceSystem& crs, QProgressDialog* progressDialog )
 {
-  QgsDebugMsg( "Entered" );
+  QgsDebugMsgLevel( "Entered", 4 );
   if ( !iter )
   {
     return SourceProviderError;
@@ -485,7 +484,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
   int fileIndex = 0;
 
   //create destProvider for whole dataset here
-  QgsRasterDataProvider* destProvider = 0;
+  QgsRasterDataProvider* destProvider = nullptr;
   double pixelSize;
   double geoTransform[6];
   globalOutputParameters( outputExtent, nCols, nRows, geoTransform, pixelSize );
@@ -505,7 +504,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
     progressDialog->setLabelText( QObject::tr( "Reading raster part %1 of %2" ).arg( fileIndex + 1 ).arg( nParts ) );
   }
 
-  QgsRasterBlock *inputBlock = 0;
+  QgsRasterBlock *inputBlock = nullptr;
   while ( iter->readNextRasterPart( 1, iterCols, iterRows, &inputBlock, iterLeft, iterTop ) )
   {
     if ( !inputBlock )
@@ -526,7 +525,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
     }
 
     //fill into red/green/blue/alpha channels
-    qgssize nPixels = ( qgssize )iterCols * iterRows;
+    qgssize nPixels = static_cast< qgssize >( iterCols ) * iterRows;
     // TODO: should be char not int? we are then copying 1 byte
     int red = 0;
     int green = 0;
@@ -536,20 +535,22 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
     {
       QRgb c = inputBlock->color( i );
       alpha = qAlpha( c );
-      red = qRed( c ); green = qGreen( c ); blue = qBlue( c );
+      red = qRed( c );
+      green = qGreen( c );
+      blue = qBlue( c );
 
       if ( inputDataType == QGis::ARGB32_Premultiplied )
       {
         double a = alpha / 255.;
-        QgsDebugMsgLevel( QString( "red = %1 green = %2 blue = %3 alpha = %4 p = %5 a = %6" ).arg( red ).arg( green ).arg( blue ).arg( alpha ).arg(( int )c, 0, 16 ).arg( a ), 5 );
+        QgsDebugMsgLevel( QString( "red = %1 green = %2 blue = %3 alpha = %4 p = %5 a = %6" ).arg( red ).arg( green ).arg( blue ).arg( alpha ).arg( static_cast< int >( c ), 0, 16 ).arg( a ), 5 );
         red /= a;
         green /= a;
         blue /= a;
       }
-      memcpy(( char* )redData + i, &red, 1 );
-      memcpy(( char* )greenData + i, &green, 1 );
-      memcpy(( char* )blueData + i, &blue, 1 );
-      memcpy(( char* )alphaData + i, &alpha, 1 );
+      memcpy( reinterpret_cast< char* >( redData ) + i, &red, 1 );
+      memcpy( reinterpret_cast< char* >( greenData ) + i, &green, 1 );
+      memcpy( reinterpret_cast< char* >( blueData ) + i, &blue, 1 );
+      memcpy( reinterpret_cast< char* >( alphaData ) + i, &alpha, 1 );
     }
     delete inputBlock;
 
@@ -591,7 +592,10 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
   if ( destProvider )
     delete destProvider;
 
-  qgsFree( redData ); qgsFree( greenData ); qgsFree( blueData ); qgsFree( alphaData );
+  qgsFree( redData );
+  qgsFree( greenData );
+  qgsFree( blueData );
+  qgsFree( alphaData );
 
   if ( progressDialog )
   {
@@ -600,7 +604,7 @@ QgsRasterFileWriter::WriterError QgsRasterFileWriter::writeImageRaster( QgsRaste
 
   if ( mTiledMode )
   {
-    QString vrtFilePath( mOutputUrl + "/" + vrtFileName() );
+    QString vrtFilePath( mOutputUrl + '/' + vrtFileName() );
     writeVRT( vrtFilePath );
     if ( mBuildPyramidsFlag == QgsRaster::PyramidsFlagYes )
     {
@@ -699,9 +703,9 @@ void QgsRasterFileWriter::buildPyramids( const QString& filename )
 
 void QgsRasterFileWriter::buildPyramids( const QString& filename )
 {
-  QgsDebugMsg( "filename = " + filename );
+  QgsDebugMsgLevel( "filename = " + filename, 4 );
   // open new dataProvider so we can build pyramids with it
-  QgsRasterDataProvider* destProvider = ( QgsRasterDataProvider* ) QgsProviderRegistry::instance()->provider( mOutputProviderKey, filename );
+  QgsRasterDataProvider* destProvider = dynamic_cast< QgsRasterDataProvider* >( QgsProviderRegistry::instance()->provider( mOutputProviderKey, filename ) );
   if ( !destProvider )
   {
     return;
@@ -718,7 +722,7 @@ void QgsRasterFileWriter::buildPyramids( const QString& filename )
     myPyramidList[myCounterInt].build = true;
   }
 
-  QgsDebugMsg( QString( "building pyramids : %1 pyramids, %2 resampling, %3 format, %4 options" ).arg( myPyramidList.count() ).arg( mPyramidsResampling ).arg( mPyramidsFormat ).arg( mPyramidsConfigOptions.count() ) );
+  QgsDebugMsgLevel( QString( "building pyramids : %1 pyramids, %2 resampling, %3 format, %4 options" ).arg( myPyramidList.count() ).arg( mPyramidsResampling ).arg( mPyramidsFormat ).arg( mPyramidsConfigOptions.count() ), 4 );
   // QApplication::setOverrideCursor( Qt::WaitCursor );
   QString res = destProvider->buildPyramids( myPyramidList, mPyramidsResampling,
                 mPyramidsFormat, mPyramidsConfigOptions );
@@ -754,8 +758,8 @@ void QgsRasterFileWriter::buildPyramids( const QString& filename )
       title = QObject::tr( "Building pyramids failed." );
       message = QObject::tr( "Building pyramid overviews is not supported on this type of raster." );
     }
-    QMessageBox::warning( 0, title, message );
-    QgsDebugMsg( res + " - " + message );
+    QMessageBox::warning( nullptr, title, message );
+    QgsDebugMsgLevel( res + " - " + message, 4 );
   }
   delete destProvider;
 }
@@ -780,7 +784,7 @@ int QgsRasterFileWriter::pyramidsProgress( double dfComplete, const char *pszMes
 }
 #endif
 
-void QgsRasterFileWriter::createVRT( int xSize, int ySize, const QgsCoordinateReferenceSystem& crs, double* geoTransform, QGis::DataType type, QList<bool> destHasNoDataValueList, QList<double> destNoDataValueList )
+void QgsRasterFileWriter::createVRT( int xSize, int ySize, const QgsCoordinateReferenceSystem& crs, double* geoTransform, QGis::DataType type, const QList<bool>& destHasNoDataValueList, const QList<double>& destNoDataValueList )
 {
   mVRTDocument.clear();
   QDomElement VRTDatasetElem = mVRTDocument.createElement( "VRTDataset" );
@@ -883,7 +887,7 @@ QgsRasterDataProvider* QgsRasterFileWriter::createPartProvider( const QgsRectang
   double mapBottom = mapTop - iterRows * mup;
   QgsRectangle mapRect( mapLeft, mapBottom, mapRight, mapTop );
 
-  QString outputFile = outputUrl + "/" + partFileName( fileIndex );
+  QString outputFile = outputUrl + '/' + partFileName( fileIndex );
 
   //geotransform
   double geoTransform[6];
@@ -904,12 +908,12 @@ QgsRasterDataProvider* QgsRasterFileWriter::createPartProvider( const QgsRectang
 
 QgsRasterDataProvider* QgsRasterFileWriter::initOutput( int nCols, int nRows, const QgsCoordinateReferenceSystem& crs,
     double* geoTransform, int nBands, QGis::DataType type,
-    QList<bool> destHasNoDataValueList, QList<double> destNoDataValueList )
+    const QList<bool>& destHasNoDataValueList, const QList<double>& destNoDataValueList )
 {
   if ( mTiledMode )
   {
     createVRT( nCols, nRows, crs, geoTransform, type, destHasNoDataValueList, destNoDataValueList );
-    return 0;
+    return nullptr;
   }
   else
   {
@@ -939,7 +943,7 @@ void QgsRasterFileWriter::globalOutputParameters( const QgsRectangle& extent, in
   //calculate nRows automatically for providers without exact resolution
   if ( nRows < 0 )
   {
-    nRows = ( double )nCols / extent.width() * extent.height() + 0.5;
+    nRows = static_cast< double >( nCols ) / extent.width() * extent.height() + 0.5;
   }
   geoTransform[0] = extent.xMinimum();
   geoTransform[1] = pixelSize;
