@@ -38,36 +38,42 @@ class QgsPoint;
 class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
 {
   public:
+    /** Precision defines if each pixel is reprojected or approximate reprojection based
+     *  on an approximation matrix of reprojected points is used.
+     */
+    enum Precision
+    {
+      Approximate = 0, //!< Approximate (default), fast but possibly inaccurate
+      Exact = 1,   //!< Exact, precise but slow
+    };
+
     /** \brief QgsRasterProjector implements approximate projection support for
      * it calculates grid of points in source CRS for target CRS + extent
      * which are used to calculate affine transformation matrices.
      */
 
-    QgsRasterProjector(
-      QgsCoordinateReferenceSystem theSrcCRS,
-      QgsCoordinateReferenceSystem theDestCRS,
-      int theSrcDatumTransform,
-      int theDestDatumTransform,
-      QgsRectangle theDestExtent,
-      int theDestRows, int theDestCols,
-      double theMaxSrcXRes, double theMaxSrcYRes,
-      QgsRectangle theExtent
-    );
+    QgsRasterProjector( const QgsCoordinateReferenceSystem& theSrcCRS,
+                        const QgsCoordinateReferenceSystem& theDestCRS,
+                        int theSrcDatumTransform,
+                        int theDestDatumTransform,
+                        const QgsRectangle& theDestExtent,
+                        int theDestRows, int theDestCols,
+                        double theMaxSrcXRes, double theMaxSrcYRes,
+                        const QgsRectangle& theExtent
+                      );
 
-    QgsRasterProjector(
-      QgsCoordinateReferenceSystem theSrcCRS,
-      QgsCoordinateReferenceSystem theDestCRS,
-      QgsRectangle theDestExtent,
-      int theDestRows, int theDestCols,
-      double theMaxSrcXRes, double theMaxSrcYRes,
-      QgsRectangle theExtent
-    );
-    QgsRasterProjector(
-      QgsCoordinateReferenceSystem theSrcCRS,
-      QgsCoordinateReferenceSystem theDestCRS,
-      double theMaxSrcXRes, double theMaxSrcYRes,
-      QgsRectangle theExtent
-    );
+    QgsRasterProjector( const QgsCoordinateReferenceSystem& theSrcCRS,
+                        const QgsCoordinateReferenceSystem& theDestCRS,
+                        const QgsRectangle& theDestExtent,
+                        int theDestRows, int theDestCols,
+                        double theMaxSrcXRes, double theMaxSrcYRes,
+                        const QgsRectangle& theExtent
+                      );
+    QgsRasterProjector( const QgsCoordinateReferenceSystem& theSrcCRS,
+                        const QgsCoordinateReferenceSystem& theDestCRS,
+                        double theMaxSrcXRes, double theMaxSrcYRes,
+                        const QgsRectangle& theExtent
+                      );
     QgsRasterProjector();
     /** \brief Copy constructor */
     // To avoid synthesized which fails on copy of QgsCoordinateTransform
@@ -79,7 +85,7 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
 
     QgsRasterProjector & operator=( const QgsRasterProjector &projector );
 
-    QgsRasterInterface *clone() const override;
+    QgsRasterProjector *clone() const override;
 
     int bandCount() const override;
 
@@ -90,24 +96,39 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
                  int srcDatumTransform = -1, int destDatumTransform = -1 );
 
     /** \brief Get source CRS */
-    QgsCoordinateReferenceSystem srcCrs() const  { return mSrcCRS; }
+    QgsCoordinateReferenceSystem srcCrs() const { return mSrcCRS; }
 
     /** \brief Get destination CRS */
-    QgsCoordinateReferenceSystem destCrs() const  { return mDestCRS; }
+    QgsCoordinateReferenceSystem destCrs() const { return mDestCRS; }
 
     /** \brief set maximum source resolution */
     void setMaxSrcRes( double theMaxSrcXRes, double theMaxSrcYRes )
     {
-      mMaxSrcXRes = theMaxSrcXRes; mMaxSrcYRes = theMaxSrcYRes;
+      mMaxSrcXRes = theMaxSrcXRes;
+      mMaxSrcYRes = theMaxSrcYRes;
     }
 
+    Precision precision() const { return mPrecision; }
+    void setPrecision( Precision precision ) { mPrecision = precision; }
+    // Translated precision mode, for use in ComboBox etc.
+    static QString precisionLabel( Precision precision );
+
     QgsRasterBlock *block( int bandNo, const QgsRectangle & extent, int width, int height ) override;
 
+    /** Calculate destination extent and size from source extent and size */
+    bool destExtentSize( const QgsRectangle& theSrcExtent, int theSrcXSize, int theSrcYSize,
+                         QgsRectangle& theDestExtent, int& theDestXSize, int& theDestYSize );
+
+    /** Calculate destination extent and size from source extent and size */
+    static bool extentSize( const QgsCoordinateTransform* ct,
+                            const QgsRectangle& theSrcExtent, int theSrcXSize, int theSrcYSize,
+                            QgsRectangle& theDestExtent, int& theDestXSize, int& theDestYSize );
+
   private:
-    /** get source extent */
+    /** Get source extent */
     QgsRectangle srcExtent() { return mSrcExtent; }
 
-    /** get/set source width/height */
+    /** Get/set source width/height */
     int srcRows() { return mSrcRows; }
     int srcCols() { return mSrcCols; }
     void setSrcRows( int theRows ) { mSrcRows = theRows; mSrcXRes = mSrcExtent.height() / mSrcRows; }
@@ -147,7 +168,7 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
     /** \brief insert columns to matrix */
     void insertCols( const QgsCoordinateTransform* ct );
 
-    /* calculate single control point in current matrix */
+    /** Calculate single control point in current matrix */
     void calcCP( int theRow, int theCol, const QgsCoordinateTransform* ct );
 
     /** \brief calculate matrix row */
@@ -176,7 +197,7 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
     /** Calc / switch helper */
     void nextHelper();
 
-    /** get mCPMatrix as string */
+    /** Get mCPMatrix as string */
     QString cpToString();
 
     /** Source CRS */
@@ -224,10 +245,10 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
     /** Source y resolution */
     double mSrcYRes;
 
-    /** number of destination rows per matrix row */
+    /** Number of destination rows per matrix row */
     double mDestRowsPerMatrixRow;
 
-    /** number of destination cols per matrix col */
+    /** Number of destination cols per matrix col */
     double mDestColsPerMatrixCol;
 
     /** Grid of source control points */
@@ -260,7 +281,11 @@ class CORE_EXPORT QgsRasterProjector : public QgsRasterInterface
     double mMaxSrcXRes;
     double mMaxSrcYRes;
 
-    /** Use approximation */
+    /** Requested precision */
+    Precision mPrecision;
+
+    /** Use approximation (requested precision is Approximate and it is possible to calculate
+     *  an approximation matrix with a sufficient precision) */
     bool mApproximate;
 };
 
