@@ -25,18 +25,20 @@
 class QgsVectorColorRampV2;
 class QgsVectorLayer;
 
-/* \brief categorized renderer */
+/** \brief categorized renderer */
 class CORE_EXPORT QgsRendererCategoryV2
 {
   public:
     QgsRendererCategoryV2();
 
     //! takes ownership of symbol
-    QgsRendererCategoryV2( QVariant value, QgsSymbolV2* symbol, QString label, bool render = true );
+    QgsRendererCategoryV2( const QVariant& value, QgsSymbolV2* symbol, const QString& label, bool render = true );
 
     //! copy constructor
     QgsRendererCategoryV2( const QgsRendererCategoryV2& cat );
 
+    ~QgsRendererCategoryV2() {}
+
     QgsRendererCategoryV2& operator=( QgsRendererCategoryV2 cat );
 
     QVariant value() const;
@@ -54,7 +56,12 @@ class CORE_EXPORT QgsRendererCategoryV2
     // debugging
     QString dump() const;
 
-    void toSld( QDomDocument& doc, QDomElement &element, QgsStringMap props ) const;
+    /** Creates a DOM element representing the category in SLD format.
+     * @param doc DOM document
+     * @param element destination DOM element
+     * @param props graduated renderer properties
+     */
+    void toSld( QDomDocument& doc, QDomElement &element, const QgsStringMap& props ) const;
 
   protected:
     QVariant mValue;
@@ -67,17 +74,20 @@ class CORE_EXPORT QgsRendererCategoryV2
 
 typedef QList<QgsRendererCategoryV2> QgsCategoryList;
 
+Q_NOWARN_DEPRECATED_PUSH
 class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
 {
   public:
 
-    QgsCategorizedSymbolRendererV2( QString attrName = QString(), QgsCategoryList categories = QgsCategoryList() );
+    QgsCategorizedSymbolRendererV2( const QString& attrName = QString(), const QgsCategoryList& categories = QgsCategoryList() );
 
     virtual ~QgsCategorizedSymbolRendererV2();
 
-    virtual QgsSymbolV2* symbolForFeature( QgsFeature& feature ) override;
+    //! @note available in python as symbolForFeature2
+    virtual QgsSymbolV2* symbolForFeature( QgsFeature& feature, QgsRenderContext& context ) override;
 
-    virtual QgsSymbolV2* originalSymbolForFeature( QgsFeature& feature ) override;
+    //! @note available in python as originalSymbolForFeature2
+    virtual QgsSymbolV2* originalSymbolForFeature( QgsFeature& feature, QgsRenderContext& context ) override;
 
     virtual void startRender( QgsRenderContext& context, const QgsFields& fields ) override;
 
@@ -87,28 +97,40 @@ class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
 
     virtual QString dump() const override;
 
-    virtual QgsFeatureRendererV2* clone() const override;
+    virtual QgsCategorizedSymbolRendererV2* clone() const override;
 
+    //! Writes the SLD element following the SLD v1.1 specs
     virtual void toSld( QDomDocument& doc, QDomElement &element ) const override;
+    //! Writes the SLD element following the SLD v1.1 specs
+    virtual void toSld( QDomDocument& doc, QDomElement &element, const QgsStringMap& props ) const override;
 
     //! returns bitwise OR-ed capabilities of the renderer
     virtual int capabilities() override { return SymbolLevels | RotationField | Filter; }
 
-    virtual QgsSymbolV2List symbols() override;
+    virtual QString filter( const QgsFields& fields = QgsFields() ) override;
+
+    //! @note available in python as symbols2
+    virtual QgsSymbolV2List symbols( QgsRenderContext& context ) override;
+
+    /** Update all the symbols but leave categories and colors. This method also sets the source
+     * symbol for the renderer.
+     * @param sym source symbol to use for categories. Ownership is not transferred.
+     * @see setSourceSymbol()
+     */
     void updateSymbols( QgsSymbolV2 * sym );
 
     const QgsCategoryList& categories() const { return mCategories; }
 
     //! return index of category with specified value (-1 if not found)
-    int categoryIndexForValue( QVariant val );
+    int categoryIndexForValue( const QVariant& val );
 
     //! return index of category with specified label (-1 if not found or not unique)
     //! @note added in 2.5
-    int categoryIndexForLabel( QString val );
+    int categoryIndexForLabel( const QString& val );
 
     bool updateCategoryValue( int catIndex, const QVariant &value );
     bool updateCategorySymbol( int catIndex, QgsSymbolV2* symbol );
-    bool updateCategoryLabel( int catIndex, QString label );
+    bool updateCategoryLabel( int catIndex, const QString& label );
 
     //! @note added in 2.5
     bool updateCategoryRenderState( int catIndex, bool render );
@@ -124,7 +146,7 @@ class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
     void sortByLabel( Qt::SortOrder order = Qt::AscendingOrder );
 
     QString classAttribute() const { return mAttrName; }
-    void setClassAttribute( QString attr ) { mAttrName = attr; }
+    void setClassAttribute( const QString& attr ) { mAttrName = attr; }
 
     //! create renderer from XML element
     static QgsFeatureRendererV2* create( QDomElement& element );
@@ -137,26 +159,56 @@ class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
 
     //! return a list of item text / symbol
     //! @note not available in python bindings
-    virtual QgsLegendSymbolList legendSymbolItems( double scaleDenominator = -1, QString rule = QString() ) override;
+    virtual QgsLegendSymbolList legendSymbolItems( double scaleDenominator = -1, const QString& rule = QString() ) override;
 
+    //! @note added in 2.10
+    QgsLegendSymbolListV2 legendSymbolItemsV2() const override;
+
+    virtual QSet< QString > legendKeysForFeature( QgsFeature& feature, QgsRenderContext& context ) override;
+
+    /** Returns the renderer's source symbol, which is the base symbol used for the each categories' symbol before applying
+     * the categories' color.
+     * @see setSourceSymbol()
+     * @see sourceColorRamp()
+     */
     QgsSymbolV2* sourceSymbol();
+
+    /** Sets the source symbol for the renderer, which is the base symbol used for the each categories' symbol before applying
+     * the categories' color.
+     * @param sym source symbol, ownership is transferred to the renderer
+     * @see sourceSymbol()
+     * @see setSourceColorRamp()
+     */
     void setSourceSymbol( QgsSymbolV2* sym );
 
+    /** Returns the source color ramp, from which each categories' color is derived.
+     * @see setSourceColorRamp()
+     * @see sourceSymbol()
+     */
     QgsVectorColorRampV2* sourceColorRamp();
+
+    /** Sets the source color ramp.
+      * @param ramp color ramp. Ownership is transferred to the renderer
+      * @see sourceColorRamp()
+      * @see setSourceSymbol()
+      */
     void setSourceColorRamp( QgsVectorColorRampV2* ramp );
 
     //! @note added in 2.1
     bool invertedColorRamp() { return mInvertedColorRamp; }
     void setInvertedColorRamp( bool inverted ) { mInvertedColorRamp = inverted; }
 
-    // Update the color ramp used and all symbols colors.
-    //! @note added in 2.5
+    /** Update the color ramp used and all symbols colors.
+      * @param ramp color ramp. Ownership is transferred to the renderer
+      * @param inverted set to true to invert ramp colors
+      * @note added in 2.5
+      */
     void updateColorRamp( QgsVectorColorRampV2* ramp, bool inverted = false );
 
-    void setRotationField( QString fieldOrExpression ) override;
-    QString rotationField() const override;
+    Q_DECL_DEPRECATED void setRotationField( const QString& fieldOrExpression ) override;
+    Q_DECL_DEPRECATED QString rotationField() const override;
 
-    void setSizeScaleField( QString fieldOrExpression );
+    void setSizeScaleField( const QString& fieldOrExpression );
     QString sizeScaleField() const;
 
     void setScaleMethod( QgsSymbolV2::ScaleMethod scaleMethod );
@@ -168,11 +220,13 @@ class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
 
     //! item in symbology was checked
     // @note added in 2.5
-    virtual bool legendSymbolItemChecked( QString key ) override;
+    virtual bool legendSymbolItemChecked( const QString& key ) override;
+
+    virtual void setLegendSymbolItem( const QString& key, QgsSymbolV2* symbol ) override;
 
     //! item in symbology was checked
     // @note added in 2.5
-    virtual void checkLegendSymbolItem( QString key, bool state = true ) override;
+    virtual void checkLegendSymbolItem( const QString& key, bool state = true ) override;
 
     //! If supported by the renderer, return classification attribute for the use in legend
     //! @note added in 2.6
@@ -206,10 +260,17 @@ class CORE_EXPORT QgsCategorizedSymbolRendererV2 : public QgsFeatureRendererV2
 
     void rebuildHash();
 
-    QgsSymbolV2* symbolForValue( QVariant value );
+    QgsSymbolV2* skipRender();
+
+    QgsSymbolV2* symbolForValue( const QVariant& value );
+
+  private:
+
+    /** Returns calculated classification value for a feature */
+    QVariant valueForFeature( QgsFeature& feature, QgsRenderContext &context ) const;
 
-    static QgsMarkerSymbolV2 sSkipRender;
 };
+Q_NOWARN_DEPRECATED_POP
 
 
 #endif // QGSCATEGORIZEDSYMBOLRENDERERV2_H
