@@ -33,15 +33,15 @@ class QgsCptCitySelectionItem;
 class CORE_EXPORT QgsCptCityArchive
 {
   public:
-    QgsCptCityArchive( QString archiveName = DEFAULT_CPTCITY_ARCHIVE,
-                       QString baseDir = QString() );
+    QgsCptCityArchive( const QString& archiveName = DEFAULT_CPTCITY_ARCHIVE,
+                       const QString& baseDir = QString() );
     ~QgsCptCityArchive();
 
     // basic dir info
     QString baseDir() const;
     static QString baseDir( QString archiveName );
     static QString defaultBaseDir();
-    void setBaseDir( QString dirName ) { mBaseDir = dirName; }
+    void setBaseDir( const QString& dirName ) { mBaseDir = dirName; }
 
     // collection + selection info
     QString copyingFileName( const QString& dirName ) const;
@@ -49,14 +49,14 @@ class CORE_EXPORT QgsCptCityArchive
     static QString findFileName( const QString & target, const QString & startDir, const QString & baseDir );
     static QMap< QString, QString > copyingInfo( const QString& fileName );
     static QMap< QString, QString > description( const QString& fileName );
-    //! note not available in python bindings
+    //! @note not available in python bindings
     static QMap< double, QPair<QColor, QColor> > gradientColorMap( const QString& fileName );
 
     // archive management
     bool isEmpty();
     QString archiveName() const { return mArchiveName; }
     static void initArchives( bool loadAll = false );
-    static void initArchive( QString archiveName, QString archiveBaseDir );
+    static void initArchive( const QString& archiveName, const QString& archiveBaseDir );
     static void initDefaultArchive();
     static void clearArchives();
     static QgsCptCityArchive* defaultArchive();
@@ -78,9 +78,13 @@ class CORE_EXPORT QgsCptCityArchive
     // mapping of copyinginfo, key is fileName
     static QMap< QString, QMap< QString, QString > > mCopyingInfoMap;
 
+  private:
+
+    QgsCptCityArchive( const QgsCptCityArchive& rh );
+    QgsCptCityArchive& operator=( const QgsCptCityArchive& rh );
 };
 
-/** base class for all items in the model */
+/** Base class for all items in the model */
 class CORE_EXPORT QgsCptCityDataItem : public QObject
 {
     Q_OBJECT
@@ -95,7 +99,7 @@ class CORE_EXPORT QgsCptCityDataItem : public QObject
     };
 
     QgsCptCityDataItem( QgsCptCityDataItem::Type type, QgsCptCityDataItem* parent,
-                        QString name, QString path );
+                        const QString& name, const QString& path );
     virtual ~QgsCptCityDataItem();
 
     bool hasChildren();
@@ -128,7 +132,7 @@ class CORE_EXPORT QgsCptCityDataItem : public QObject
 
     virtual bool equal( const QgsCptCityDataItem *other );
 
-    virtual QWidget *paramWidget() { return 0; }
+    virtual QWidget *paramWidget() { return nullptr; }
 
     // list of actions provided by this item - usually used for popup menu on right-click
     virtual QList<QAction*> actions() { return QList<QAction*>(); }
@@ -151,15 +155,15 @@ class CORE_EXPORT QgsCptCityDataItem : public QObject
     void setParent( QgsCptCityDataItem* parent ) { mParent = parent; }
     QVector<QgsCptCityDataItem*> children() const { return mChildren; }
     virtual QIcon icon() { return mIcon; }
-    virtual QIcon icon( const QSize& size ) { Q_UNUSED( size ) ; return icon(); }
+    virtual QIcon icon( QSize size ) { Q_UNUSED( size ) ; return icon(); }
     QString name() const { return mName; }
     QString path() const { return mPath; }
     QString info() const { return mInfo; }
     QString shortInfo() const { return mShortInfo; }
 
-    void setIcon( QIcon icon ) { mIcon = icon; }
+    void setIcon( const QIcon& icon ) { mIcon = icon; }
 
-    void setToolTip( QString msg ) { mToolTip = msg; }
+    void setToolTip( const QString& msg ) { mToolTip = msg; }
     QString toolTip() const { return mToolTip; }
 
     bool isValid() { return mValid; }
@@ -197,12 +201,12 @@ class CORE_EXPORT QgsCptCityColorRampItem : public QgsCptCityDataItem
     Q_OBJECT
   public:
     QgsCptCityColorRampItem( QgsCptCityDataItem* parent,
-                             QString name, QString path,
-                             QString variantName = QString(),
+                             const QString& name, const QString& path,
+                             const QString& variantName = QString(),
                              bool initialize = false );
     QgsCptCityColorRampItem( QgsCptCityDataItem* parent,
-                             QString name, QString path,
-                             QStringList variantList,
+                             const QString& name, const QString& path,
+                             const QStringList& variantList,
                              bool initialize = false );
     ~QgsCptCityColorRampItem() {}
 
@@ -214,7 +218,7 @@ class CORE_EXPORT QgsCptCityColorRampItem : public QgsCptCityDataItem
     // --- New virtual methods for layer item derived classes ---
     const QgsCptCityColorRampV2& ramp() const { return mRamp; }
     QIcon icon() override;
-    QIcon icon( const QSize& size ) override;
+    QIcon icon( QSize size ) override;
     void init();
 
   protected:
@@ -231,7 +235,7 @@ class CORE_EXPORT QgsCptCityCollectionItem : public QgsCptCityDataItem
     Q_OBJECT
   public:
     QgsCptCityCollectionItem( QgsCptCityDataItem* parent,
-                              QString name, QString path );
+                              const QString& name, const QString& path );
     ~QgsCptCityCollectionItem();
 
     void setPopulated() { mPopulated = true; }
@@ -248,7 +252,7 @@ class CORE_EXPORT QgsCptCityDirectoryItem : public QgsCptCityCollectionItem
     Q_OBJECT
   public:
     QgsCptCityDirectoryItem( QgsCptCityDataItem* parent,
-                             QString name, QString path );
+                             const QString& name, const QString& path );
     ~QgsCptCityDirectoryItem();
 
     QVector<QgsCptCityDataItem*> createChildren() override;
@@ -256,7 +260,7 @@ class CORE_EXPORT QgsCptCityDirectoryItem : public QgsCptCityCollectionItem
     virtual bool equal( const QgsCptCityDataItem *other ) override;
 
     static QgsCptCityDataItem* dataItem( QgsCptCityDataItem* parent,
-                                         QString name, QString path );
+                                         const QString& name, const QString& path );
 
   protected:
     QMap< QString, QStringList > rampsMap();
@@ -269,7 +273,7 @@ class CORE_EXPORT QgsCptCitySelectionItem : public QgsCptCityCollectionItem
 {
     Q_OBJECT
   public:
-    QgsCptCitySelectionItem( QgsCptCityDataItem* parent, QString name, QString path );
+    QgsCptCitySelectionItem( QgsCptCityDataItem* parent, const QString& name, const QString& path );
     ~QgsCptCitySelectionItem();
 
     QVector<QgsCptCityDataItem*> createChildren() override;
@@ -288,8 +292,8 @@ class CORE_EXPORT QgsCptCityAllRampsItem : public QgsCptCityCollectionItem
 {
     Q_OBJECT
   public:
-    QgsCptCityAllRampsItem( QgsCptCityDataItem* parent, QString name,
-                            QVector<QgsCptCityDataItem*> items );
+    QgsCptCityAllRampsItem( QgsCptCityDataItem* parent, const QString& name,
+                            const QVector<QgsCptCityDataItem*>& items );
     ~QgsCptCityAllRampsItem();
 
     QVector<QgsCptCityDataItem*> createChildren() override;
@@ -312,7 +316,7 @@ class CORE_EXPORT QgsCptCityBrowserModel : public QAbstractItemModel
       List = 2 // not used anymore
     };
 
-    QgsCptCityBrowserModel( QObject* parent = 0,
+    QgsCptCityBrowserModel( QObject* parent = nullptr,
                             QgsCptCityArchive* archive = QgsCptCityArchive::defaultArchive(),
                             ViewType Type = Authors );
     ~QgsCptCityBrowserModel();
@@ -343,7 +347,7 @@ class CORE_EXPORT QgsCptCityBrowserModel : public QAbstractItemModel
     /** Returns the index of the item in the model specified by the given row, column and parent index. */
     virtual QModelIndex index( int row, int column, const QModelIndex & parent = QModelIndex() ) const override;
 
-    QModelIndex findItem( QgsCptCityDataItem *item, QgsCptCityDataItem *parent = 0 ) const;
+    QModelIndex findItem( QgsCptCityDataItem *item, QgsCptCityDataItem *parent = nullptr ) const;
 
     /** Returns the parent of the model item with the given index.
      * If the item has no parent, an invalid QModelIndex is returned.
@@ -367,13 +371,13 @@ class CORE_EXPORT QgsCptCityBrowserModel : public QAbstractItemModel
     void reload();
 
     // Refresh item specified by path
-    void refresh( QString path );
+    void refresh( const QString& path );
 
     // Refresh item childs
     void refresh( const QModelIndex &index = QModelIndex() );
 
     //! return index of a path
-    QModelIndex findPath( QString path );
+    QModelIndex findPath( const QString& path );
 
     void connectItem( QgsCptCityDataItem *item );
 
