@@ -42,13 +42,15 @@ class CORE_EXPORT QgsInvertedPolygonRenderer : public QgsFeatureRendererV2
   public:
 
     /** Constructor
-     * @param embeddedRenderer optional embeddedRenderer. If null, a default one will be assigned
+     * @param embeddedRenderer optional embeddedRenderer. If null, a default one will be assigned.
+     * Ownership will be transferred.
      */
-    QgsInvertedPolygonRenderer( const QgsFeatureRendererV2* embeddedRenderer = 0 );
+    QgsInvertedPolygonRenderer( QgsFeatureRendererV2* embeddedRenderer = nullptr );
+
     virtual ~QgsInvertedPolygonRenderer();
 
     /** Used to clone this feature renderer.*/
-    virtual QgsFeatureRendererV2* clone() const override;
+    virtual QgsInvertedPolygonRenderer* clone() const override;
 
     virtual void startRender( QgsRenderContext& context, const QgsFields& fields ) override;
 
@@ -76,24 +78,36 @@ class CORE_EXPORT QgsInvertedPolygonRenderer : public QgsFeatureRendererV2
     virtual QList<QString> usedAttributes() override;
     /** Proxy that will call this method on the embedded renderer. */
     virtual int capabilities() override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual QgsSymbolV2List symbols() override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual QgsSymbolV2* symbolForFeature( QgsFeature& feature ) override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual QgsSymbolV2* originalSymbolForFeature( QgsFeature& feat ) override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual QgsSymbolV2List symbolsForFeature( QgsFeature& feat ) override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual QgsSymbolV2List originalSymbolsForFeature( QgsFeature& feat ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as symbol2
+     */
+    virtual QgsSymbolV2List symbols( QgsRenderContext& context ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as symbolForFeature2
+     */
+    virtual QgsSymbolV2* symbolForFeature( QgsFeature& feature, QgsRenderContext& context ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as originalSymbolForFeature2
+     */
+    virtual QgsSymbolV2* originalSymbolForFeature( QgsFeature& feat, QgsRenderContext& context ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as symbolsForFeature
+     */
+    virtual QgsSymbolV2List symbolsForFeature( QgsFeature& feat, QgsRenderContext& context ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as originalSymbolsForFeature2
+     */
+    virtual QgsSymbolV2List originalSymbolsForFeature( QgsFeature& feat, QgsRenderContext& context ) override;
     /** Proxy that will call this method on the embedded renderer. */
     virtual QgsLegendSymbologyList legendSymbologyItems( QSize iconSize ) override;
     /** Proxy that will call this method on the embedded renderer.
-      @note not available in python bindings
+     * @note not available in python bindings
      */
-    virtual QgsLegendSymbolList legendSymbolItems( double scaleDenominator = -1, QString rule = "" ) override;
-    /** Proxy that will call this method on the embedded renderer. */
-    virtual bool willRenderFeature( QgsFeature& feat ) override;
+    virtual QgsLegendSymbolList legendSymbolItems( double scaleDenominator = -1, const QString& rule = "" ) override;
+    /** Proxy that will call this method on the embedded renderer.
+     * @note available in python bindings as willRenderFeature2
+     */
+    virtual bool willRenderFeature( QgsFeature& feat, QgsRenderContext& context ) override;
 
     /** Creates a renderer out of an XML, for loading*/
     static QgsFeatureRendererV2* create( QDomElement& element );
@@ -104,28 +118,29 @@ class CORE_EXPORT QgsInvertedPolygonRenderer : public QgsFeatureRendererV2
      */
     virtual QDomElement save( QDomDocument& doc ) override;
 
-    /** sets the embedded renderer
-     * @param subRenderer the embedded renderer (will be cloned)
-     */
-    void setEmbeddedRenderer( const QgsFeatureRendererV2* subRenderer );
-    /** @returns the current embedded renderer
-     */
-    const QgsFeatureRendererV2* embeddedRenderer() const;
+    void setEmbeddedRenderer( QgsFeatureRendererV2* subRenderer ) override;
+    const QgsFeatureRendererV2* embeddedRenderer() const override;
+
+    virtual void setLegendSymbolItem( const QString& key, QgsSymbolV2* symbol ) override;
+
+    virtual bool legendSymbolItemsCheckable() const override;
+    virtual bool legendSymbolItemChecked( const QString& key ) override;
+    virtual void checkLegendSymbolItem( const QString& key, bool state = true ) override;
 
     /** @returns true if the geometries are to be preprocessed (merged with an union) before rendering.*/
     bool preprocessingEnabled() const { return mPreprocessingEnabled; }
     /**
-        @param enabled enables or disables the preprocessing.
-        When enabled, geometries will be merged with an union before being rendered.
-        It allows fixing some rendering artefacts (when rendering overlapping polygons for instance).
-        This will involve some CPU-demanding computations and is thus disabled by default.
-    */
+     * @param enabled enables or disables the preprocessing.
+     * When enabled, geometries will be merged with an union before being rendered.
+     * It allows fixing some rendering artifacts (when rendering overlapping polygons for instance).
+     * This will involve some CPU-demanding computations and is thus disabled by default.
+     */
     void setPreprocessingEnabled( bool enabled ) { mPreprocessingEnabled = enabled; }
 
-    /** creates a QgsInvertedPolygonRenderer by a conversion from an existing renderer.
-        @note added in 2.5
-        @returns a new renderer if the conversion was possible, otherwise 0.
-        */
+    /** Creates a QgsInvertedPolygonRenderer by a conversion from an existing renderer.
+     * @note added in 2.5
+     * @returns a new renderer if the conversion was possible, otherwise 0.
+     */
     static QgsInvertedPolygonRenderer* convertFromRenderer( const QgsFeatureRendererV2* renderer );
 
   private:
@@ -144,24 +159,24 @@ class CORE_EXPORT QgsInvertedPolygonRenderer : public QgsFeatureRendererV2
       QgsFeature feature;             //< one feature (for attriute-based rendering)
     };
     typedef QVector<CombinedFeature> FeatureCategoryVector;
-    /** where features are stored, based on the index of their symbol category @see mSymbolCategories */
+    /** Where features are stored, based on the index of their symbol category @see mSymbolCategories */
     FeatureCategoryVector mFeaturesCategories;
 
-    /** maps a category to an index */
+    /** Maps a category to an index */
     QMap<QByteArray, int> mSymbolCategories;
 
-    /** the polygon used as exterior ring that covers the current extent */
+    /** The polygon used as exterior ring that covers the current extent */
     QgsPolygon mExtentPolygon;
 
-    /** the context used for rendering */
+    /** The context used for rendering */
     QgsRenderContext mContext;
 
-    /** fields of each feature*/
+    /** Fields of each feature*/
     QgsFields mFields;
 
     /** Class used to represent features that must be rendered
-        with decorations (selection, vertex markers)
-    */
+     *  with decorations (selection, vertex markers)
+     */
     struct FeatureDecoration
     {
       QgsFeature feature;
@@ -173,7 +188,7 @@ class CORE_EXPORT QgsInvertedPolygonRenderer : public QgsFeatureRendererV2
     };
     QList<FeatureDecoration> mFeatureDecorations;
 
-    /** whether to preprocess (merge) geometries before rendering*/
+    /** Whether to preprocess (merge) geometries before rendering*/
     bool mPreprocessingEnabled;
 };
 
