@@ -88,7 +88,12 @@ QgsPointDisplacementRenderer* QgsPointDisplacementRenderer::clone() const
 
 void QgsPointDisplacementRenderer::toSld( QDomDocument& doc, QDomElement &element ) const
 {
-  mRenderer->toSld( doc, element );
+  toSld( doc, element, QgsStringMap() );
+}
+
+void QgsPointDisplacementRenderer::toSld( QDomDocument& doc, QDomElement &element, const QgsStringMap& props ) const
+{
+  mRenderer->toSld( doc, element, props );
 }
 
 
@@ -324,6 +329,13 @@ QgsSymbolV2List QgsPointDisplacementRenderer::originalSymbolsForFeature( QgsFeat
   return mRenderer->originalSymbolsForFeature( feat, context );
 }
 
+QSet< QString > QgsPointDisplacementRenderer::legendKeysForFeature( QgsFeature& feat, QgsRenderContext& context )
+{
+  if ( !mRenderer )
+    return QSet< QString >() << QString();
+  return mRenderer->legendKeysForFeature( feat, context );
+}
+
 bool QgsPointDisplacementRenderer::willRenderFeature( QgsFeature& feat, QgsRenderContext& context )
 {
   if ( !mRenderer )
