@@ -20,6 +20,7 @@
 #include "qgsattributeaction.h"
 #include "qgseditorwidgetregistry.h"
 #include "qgsexpression.h"
+#include "qgsconditionalstyle.h"
 #include "qgsfield.h"
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
@@ -27,6 +28,8 @@
 #include "qgsmaplayerregistry.h"
 #include "qgsrendererv2.h"
 #include "qgsvectorlayer.h"
+#include "qgsvectordataprovider.h"
+#include "qgssymbollayerv2utils.h"
 
 #include <QVariant>
 
@@ -40,6 +43,10 @@ QgsAttributeTableModel::QgsAttributeTableModel( QgsVectorLayerCache *layerCache,
 {
   QgsDebugMsg( "entered." );
 
+  mExpressionContext << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( layerCache->layer() );
+
   if ( layerCache->layer()->geometryType() == QGis::NoGeometry )
   {
     mFeatureRequest.setFlags( QgsFeatureRequest::NoGeometry );
@@ -53,7 +60,7 @@ QgsAttributeTableModel::QgsAttributeTableModel( QgsVectorLayerCache *layerCache,
   loadAttributes();
 
   connect( mLayerCache, SIGNAL( attributeValueChanged( QgsFeatureId, int, const QVariant& ) ), this, SLOT( attributeValueChanged( QgsFeatureId, int, const QVariant& ) ) );
-  connect( layer(), SIGNAL( featureDeleted( QgsFeatureId ) ), this, SLOT( featureDeleted( QgsFeatureId ) ) );
+  connect( layer(), SIGNAL( featuresDeleted( QgsFeatureIds ) ), this, SLOT( featuresDeleted( QgsFeatureIds ) ) );
   connect( layer(), SIGNAL( attributeDeleted( int ) ), this, SLOT( attributeDeleted( int ) ) );
   connect( layer(), SIGNAL( updatedFields() ), this, SLOT( updatedFields() ) );
   connect( layer(), SIGNAL( editCommandEnded() ), this, SLOT( editCommandEnded() ) );
@@ -73,30 +80,78 @@ bool QgsAttributeTableModel::loadFeatureAtId( QgsFeatureId fid ) const
   return mLayerCache->featureAtId( fid, mFeat );
 }
 
-void QgsAttributeTableModel::featureDeleted( QgsFeatureId fid )
+void QgsAttributeTableModel::featuresDeleted( const QgsFeatureIds& fids )
 {
-  QgsDebugMsgLevel( QString( "(%2) fid: %1" ).arg( fid ).arg( mFeatureRequest.filterType() ), 4 );
-  mFieldCache.remove( fid );
+  QList<int> rows;
 
-  int row = idToRow( fid );
-
-  if ( row != -1 )
+  Q_FOREACH ( QgsFeatureId fid, fids )
   {
-    beginRemoveRows( QModelIndex(), row, row );
-    removeRow( row );
-    endRemoveRows();
+    QgsDebugMsgLevel( QString( "(%2) fid: %1, size: %3" ).arg( fid ).arg( mFeatureRequest.filterType() ).arg( mIdRowMap.size() ), 4 );
+
+    int row = idToRow( fid );
+    if ( row != -1 )
+      rows << row;
   }
+
+  qSort( rows );
+
+  int lastRow = -1;
+  int beginRow = -1;
+  int currentRowCount = 0;
+  int removedRows = 0;
+  bool reset = false;
+
+  Q_FOREACH ( int row, rows )
+  {
+#if 0
+    qDebug() << "Row: " << row << ", begin " << beginRow << ", last " << lastRow << ", current " << currentRowCount << ", removed " << removedRows;
+#endif
+    if ( lastRow == -1 )
+    {
+      beginRow = row;
+    }
+
+    if ( row != lastRow + 1 && lastRow != -1 )
+    {
+      if ( rows.count() > 100 && currentRowCount < 10 )
+      {
+        reset = true;
+        break;
+      }
+      removeRows( beginRow - removedRows, currentRowCount );
+
+      beginRow = row;
+      removedRows += currentRowCount;
+      currentRowCount = 0;
+    }
+
+    currentRowCount++;
+
+    lastRow = row;
+  }
+
+  if ( !reset )
+    removeRows( beginRow - removedRows, currentRowCount );
+  else
+    resetModel();
 }
 
 bool QgsAttributeTableModel::removeRows( int row, int count, const QModelIndex &parent )
 {
-  Q_UNUSED( parent );
-  QgsDebugMsgLevel( QString( "remove %2 rows at %1 (rows %3, ids %4)" ).arg( row ).arg( count ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 3 );
+  if ( row < 0 || count < 1 )
+    return false;
+
+  beginRemoveRows( parent, row, row + count - 1 );
+#ifdef QGISDEBUG
+  if ( 3 <= QgsLogger::debugLevel() )
+    QgsDebugMsgLevel( QString( "remove %2 rows at %1 (rows %3, ids %4)" ).arg( row ).arg( count ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 3 );
+#endif
 
   // clean old references
   for ( int i = row; i < row + count; i++ )
   {
-    mIdRowMap.remove( mRowIdMap[ i ] );
+    mFieldCache.remove( mRowIdMap[i] );
+    mIdRowMap.remove( mRowIdMap[i] );
     mRowIdMap.remove( i );
   }
 
@@ -105,26 +160,29 @@ bool QgsAttributeTableModel::removeRows( int row, int count, const QModelIndex &
   for ( int i = row + count; i < n; i++ )
   {
     QgsFeatureId id = mRowIdMap[i];
-    mIdRowMap[ id ] -= count;
-    mRowIdMap[ i-count ] = id;
+    mIdRowMap[id] -= count;
+    mRowIdMap[i-count] = id;
     mRowIdMap.remove( i );
   }
 
 #ifdef QGISDEBUG
-  QgsDebugMsgLevel( QString( "after removal rows %1, ids %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 4 );
-  QgsDebugMsgLevel( "id->row", 4 );
-  for ( QHash<QgsFeatureId, int>::iterator it = mIdRowMap.begin(); it != mIdRowMap.end(); ++it )
-    QgsDebugMsgLevel( QString( "%1->%2" ).arg( FID_TO_STRING( it.key() ) ).arg( *it ), 4 );
+  if ( 4 <= QgsLogger::debugLevel() )
+  {
+    QgsDebugMsgLevel( QString( "after removal rows %1, ids %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 4 );
+    QgsDebugMsgLevel( "id->row", 4 );
+    for ( QHash<QgsFeatureId, int>::const_iterator it = mIdRowMap.constBegin(); it != mIdRowMap.constEnd(); ++it )
+      QgsDebugMsgLevel( QString( "%1->%2" ).arg( FID_TO_STRING( it.key() ) ).arg( *it ), 4 );
 
-  QHash<QgsFeatureId, int>::iterator idit;
-
-  QgsDebugMsgLevel( "row->id", 4 );
-  for ( QHash<int, QgsFeatureId>::iterator it = mRowIdMap.begin(); it != mRowIdMap.end(); ++it )
-    QgsDebugMsgLevel( QString( "%1->%2" ).arg( it.key() ).arg( FID_TO_STRING( *it ) ), 4 );
+    QgsDebugMsgLevel( "row->id", 4 );
+    for ( QHash<int, QgsFeatureId>::const_iterator it = mRowIdMap.constBegin(); it != mRowIdMap.constEnd(); ++it )
+      QgsDebugMsgLevel( QString( "%1->%2" ).arg( it.key() ).arg( FID_TO_STRING( *it ) ), 4 );
+  }
 #endif
 
   Q_ASSERT( mRowIdMap.size() == mIdRowMap.size() );
 
+  endRemoveRows();
+
   return true;
 }
 
@@ -138,7 +196,7 @@ void QgsAttributeTableModel::featureAdded( QgsFeatureId fid )
 
   if ( featOk && mFeatureRequest.acceptFeature( mFeat ) )
   {
-    mFieldCache[ fid ] = mFeat.attribute( mCachedField );
+    mFieldCache[fid] = mFeat.attribute( mCachedField );
 
     int n = mRowIdMap.size();
     beginInsertRows( QModelIndex(), n, n );
@@ -179,9 +237,7 @@ void QgsAttributeTableModel::layerDeleted()
 {
   QgsDebugMsg( "entered." );
 
-  beginRemoveRows( QModelIndex(), 0, rowCount() - 1 );
   removeRows( 0, rowCount() );
-  endRemoveRows();
 
   mAttributeWidgetCaches.clear();
   mAttributes.clear();
@@ -194,7 +250,7 @@ void QgsAttributeTableModel::attributeValueChanged( QgsFeatureId fid, int idx, c
   QgsDebugMsgLevel( QString( "(%4) fid: %1, idx: %2, value: %3" ).arg( fid ).arg( idx ).arg( value.toString() ).arg( mFeatureRequest.filterType() ), 3 );
 
   if ( idx == mCachedField )
-    mFieldCache[ fid ] = value;
+    mFieldCache[fid] = value;
 
   // No filter request: skip all possibly heavy checks
   if ( mFeatureRequest.filterType() == QgsFeatureRequest::FilterNone )
@@ -223,7 +279,7 @@ void QgsAttributeTableModel::attributeValueChanged( QgsFeatureId fid, int idx, c
         if ( mIdRowMap.contains( fid ) )
         {
           // Feature changed such, that it is no longer shown
-          featureDeleted( fid );
+          featuresDeleted( QgsFeatureIds() << fid );
         }
         // else: we don't care
       }
@@ -241,7 +297,7 @@ void QgsAttributeTableModel::loadAttributes()
   bool ins = false, rm = false;
 
   QgsAttributeList attributes;
-  const QgsFields& fields = layer()->pendingFields();
+  const QgsFields& fields = layer()->fields();
 
   mWidgetFactories.clear();
   mAttributeWidgetCaches.clear();
@@ -249,12 +305,12 @@ void QgsAttributeTableModel::loadAttributes()
 
   for ( int idx = 0; idx < fields.count(); ++idx )
   {
-    const QString widgetType = layer()->editorWidgetV2( idx );
+    const QString widgetType = layer()->editFormConfig()->widgetType( idx );
     QgsEditorWidgetFactory* widgetFactory = QgsEditorWidgetRegistry::instance()->factory( widgetType );
     if ( widgetFactory && widgetType != "Hidden" )
     {
       mWidgetFactories.append( widgetFactory );
-      mWidgetConfigs.append( layer()->editorWidgetV2Config( idx ) );
+      mWidgetConfigs.append( layer()->editFormConfig()->widgetConfig( idx ) );
       mAttributeWidgetCaches.append( widgetFactory->createCache( layer(), idx, mWidgetConfigs.last() ) );
 
       attributes << idx;
@@ -289,11 +345,17 @@ void QgsAttributeTableModel::loadLayer()
 {
   QgsDebugMsg( "entered." );
 
+  // make sure attributes are properly updated before caching the data
+  // (emit of progress() signal may enter event loop and thus attribute
+  // table view may be updated with inconsistent model which may assume
+  // wrong number of attributes)
+  loadAttributes();
+
+  beginResetModel();
+
   if ( rowCount() != 0 )
   {
-    beginRemoveRows( QModelIndex(), 0, rowCount() - 1 );
     removeRows( 0, rowCount() );
-    endRemoveRows();
   }
 
   QgsFeatureIterator features = mLayerCache->getFeatures( mFeatureRequest );
@@ -323,7 +385,27 @@ void QgsAttributeTableModel::loadLayer()
 
   emit finished();
 
-  mFieldCount = mAttributes.size();
+  connect( mLayerCache, SIGNAL( invalidated() ), this, SLOT( loadLayer() ), Qt::UniqueConnection );
+
+  endResetModel();
+}
+
+void QgsAttributeTableModel::fieldConditionalStyleChanged( const QString &fieldName )
+{
+  if ( fieldName.isNull() )
+  {
+    mRowStylesMap.clear();
+    emit dataChanged( index( 0, 0 ), index( rowCount() - 1, columnCount() - 1 ) );
+    return;
+  }
+
+  int fieldIndex = mLayerCache->layer()->fieldNameIndex( fieldName );
+  if ( fieldIndex == -1 )
+    return;
+
+  //whole column has changed
+  int col = fieldCol( fieldIndex );
+  emit dataChanged( index( 0, col ), index( rowCount() - 1, col ) );
 }
 
 void QgsAttributeTableModel::swapRows( QgsFeatureId a, QgsFeatureId b )
@@ -370,7 +452,9 @@ QModelIndexList QgsAttributeTableModel::idToIndexList( QgsFeatureId id ) const
   QModelIndexList indexes;
 
   int row = idToRow( id );
-  for ( int column = 0; column < columnCount(); ++column )
+  int columns = columnCount();
+  indexes.reserve( columns );
+  for ( int column = 0; column < columns; ++column )
   {
     indexes.append( index( row, column ) );
   }
@@ -392,7 +476,7 @@ QgsFeatureId QgsAttributeTableModel::rowToId( const int row ) const
 
 int QgsAttributeTableModel::fieldIdx( int col ) const
 {
-  return mAttributes[ col ];
+  return mAttributes[col];
 }
 
 int QgsAttributeTableModel::fieldCol( int idx ) const
@@ -428,7 +512,7 @@ QVariant QgsAttributeTableModel::headerData( int section, Qt::Orientation orient
       QString attributeName = layer()->attributeAlias( mAttributes[section] );
       if ( attributeName.isEmpty() )
       {
-        QgsField field = layer()->pendingFields()[ mAttributes[section] ];
+        QgsField field = layer()->fields().at( mAttributes[section] );
         attributeName = field.name();
       }
       return QVariant( attributeName );
@@ -453,6 +537,10 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role ) cons
          && role != SortRole
          && role != FeatureIdRole
          && role != FieldIndexRole
+         && role != Qt::BackgroundColorRole
+         && role != Qt::TextColorRole
+         && role != Qt::DecorationRole
+         && role != Qt::FontRole
        )
      )
     return QVariant();
@@ -465,19 +553,20 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role ) cons
   if ( index.column() >= mFieldCount )
     return role == Qt::DisplayRole ? rowId : QVariant();
 
-  int fieldId = mAttributes[ index.column()];
+  int fieldId = mAttributes.at( index.column() );
 
   if ( role == FieldIndexRole )
     return fieldId;
 
-  const QgsField& field = layer()->pendingFields()[ fieldId ];
+  QgsField field = layer()->fields().at( fieldId );
 
   QVariant::Type fldType = field.type();
-  bool fldNumeric = ( fldType == QVariant::Int || fldType == QVariant::Double || fldType == QVariant::LongLong );
+  bool fldRightAlign = ( fldType == QVariant::Int || fldType == QVariant::Double || fldType == QVariant::LongLong
+                         || fldType == QVariant::DateTime || fldType == QVariant::Date || fldType == QVariant::Time );
 
   if ( role == Qt::TextAlignmentRole )
   {
-    if ( fldNumeric )
+    if ( fldRightAlign )
       return QVariant( Qt::AlignRight );
     else
       return QVariant( Qt::AlignLeft );
@@ -488,7 +577,7 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role ) cons
   // if we don't have the row in current cache, load it from layer first
   if ( mCachedField == fieldId )
   {
-    val = mFieldCache[ rowId ];
+    val = mFieldCache[rowId];
   }
   else
   {
@@ -504,12 +593,61 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role ) cons
     val = mFeat.attribute( fieldId );
   }
 
-  if ( role == Qt::DisplayRole )
+  if ( role == SortRole )
   {
-    return mWidgetFactories[ index.column()]->representValue( layer(), fieldId, mWidgetConfigs[ index.column()], mAttributeWidgetCaches[ index.column()], val );
+    return val;
   }
 
-  return val;
+  switch ( role )
+  {
+    case Qt::DisplayRole:
+      return mWidgetFactories.at( index.column() )->representValue( layer(), fieldId, mWidgetConfigs.at( index.column() ),
+             mAttributeWidgetCaches.at( index.column() ), val );
+
+    case Qt::EditRole:
+      return val;
+
+    case Qt::BackgroundColorRole:
+    case Qt::TextColorRole:
+    case Qt::DecorationRole:
+    case Qt::FontRole:
+    {
+      mExpressionContext.setFeature( mFeat );
+      QList<QgsConditionalStyle> styles;
+      if ( mRowStylesMap.contains( index.row() ) )
+      {
+        styles = mRowStylesMap[index.row()];
+      }
+      else
+      {
+        styles = QgsConditionalStyle::matchingConditionalStyles( layer()->conditionalStyles()->rowStyles(), QVariant(),  mExpressionContext );
+        mRowStylesMap.insert( index.row(), styles );
+
+      }
+
+      QgsConditionalStyle rowstyle = QgsConditionalStyle::compressStyles( styles );
+      styles = layer()->conditionalStyles()->fieldStyles( field.name() );
+      styles = QgsConditionalStyle::matchingConditionalStyles( styles , val,  mExpressionContext );
+      styles.insert( 0, rowstyle );
+      QgsConditionalStyle style = QgsConditionalStyle::compressStyles( styles );
+
+      if ( style.isValid() )
+      {
+        if ( role == Qt::BackgroundColorRole && style.validBackgroundColor() )
+          return style.backgroundColor();
+        if ( role == Qt::TextColorRole && style.validTextColor() )
+          return style.textColor();
+        if ( role == Qt::DecorationRole )
+          return style.icon();
+        if ( role == Qt::FontRole )
+          return style.font();
+      }
+
+      return QVariant();
+    }
+  }
+
+  return QVariant();
 }
 
 bool QgsAttributeTableModel::setData( const QModelIndex &index, const QVariant &value, int role )
@@ -560,7 +698,9 @@ Qt::ItemFlags QgsAttributeTableModel::flags( const QModelIndex &index ) const
   Qt::ItemFlags flags = QAbstractItemModel::flags( index );
 
   if ( layer()->isEditable() &&
-       layer()->fieldEditable( mAttributes[ index.column()] ) )
+       !layer()->editFormConfig()->readOnly( mAttributes[index.column()] ) &&
+       (( layer()->dataProvider() && layer()->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues ) ||
+        FID_IS_NEW( rowToId( index.row() ) ) ) )
     flags |= Qt::ItemIsEditable;
 
   return flags;
@@ -572,11 +712,6 @@ void QgsAttributeTableModel::reload( const QModelIndex &index1, const QModelInde
   emit dataChanged( index1, index2 );
 }
 
-void QgsAttributeTableModel::resetModel()
-{
-  beginResetModel();
-  endResetModel();
-}
 
 void QgsAttributeTableModel::executeAction( int action, const QModelIndex &idx ) const
 {
@@ -615,10 +750,10 @@ void QgsAttributeTableModel::prefetchColumnData( int column )
   {
     if ( column >= mAttributes.count() )
       return;
-    int fieldId = mAttributes[ column ];
-    const QgsFields& fields = layer()->pendingFields();
+    int fieldId = mAttributes.at( column );
+    const QgsFields& fields = layer()->fields();
     QStringList fldNames;
-    fldNames << fields[ fieldId ].name();
+    fldNames << fields[fieldId].name();
 
     QgsFeatureRequest r( mFeatureRequest );
     QgsFeatureIterator it = mLayerCache->getFeatures( r.setFlags( QgsFeatureRequest::NoGeometry ).setSubsetOfAttributes( fldNames, fields ) );
