@@ -27,8 +27,8 @@
 
 QgsDateTimeEditWrapper::QgsDateTimeEditWrapper( QgsVectorLayer* vl, int fieldIdx, QWidget* editor, QWidget* parent )
     : QgsEditorWidgetWrapper( vl, fieldIdx, editor, parent )
-    , mQDateTimeEdit( NULL )
-    , mQgsDateTimeEdit( NULL )
+    , mQDateTimeEdit( nullptr )
+    , mQgsDateTimeEdit( nullptr )
 {
 }
 
@@ -90,17 +90,34 @@ void QgsDateTimeEditWrapper::initWidget( QWidget *editor )
   }
 }
 
+bool QgsDateTimeEditWrapper::valid() const
+{
+  return mQgsDateTimeEdit || mQDateTimeEdit;
+}
+
 void QgsDateTimeEditWrapper::dateTimeChanged( const QDateTime& dateTime )
 {
   const QString fieldFormat = config( "field_format", QGSDATETIMEEDIT_DATEFORMAT ).toString();
   emit valueChanged( dateTime.toString( fieldFormat ) );
 }
 
-QVariant QgsDateTimeEditWrapper::value()
+QVariant QgsDateTimeEditWrapper::value() const
 {
   if ( !mQDateTimeEdit )
     return QVariant( field().type() );
 
+  if ( field().type() == QVariant::DateTime )
+  {
+    if ( mQgsDateTimeEdit )
+    {
+      return mQgsDateTimeEdit->dateTime();
+    }
+    else
+    {
+      return mQDateTimeEdit->dateTime();
+    }
+  }
+
   const QString fieldFormat = config( "field_format", QGSDATETIMEEDIT_DATEFORMAT ).toString();
 
   if ( mQgsDateTimeEdit )
@@ -119,7 +136,7 @@ void QgsDateTimeEditWrapper::setValue( const QVariant &value )
     return;
 
   const QString fieldFormat = config( "field_format", QGSDATETIMEEDIT_DATEFORMAT ).toString();
-  const QDateTime date = QDateTime::fromString( value.toString(), fieldFormat );
+  const QDateTime date = field().type() == QVariant::DateTime ? value.toDateTime() : QDateTime::fromString( value.toString(), fieldFormat );
 
   if ( mQgsDateTimeEdit )
   {
