@@ -32,14 +32,14 @@
  * <li><b>allow_null</b> <i>If True, will allow NULL values. Requires QgsDateTimeEdit as widget (Be aware if you work with .ui files).</i></li>
  * <li><b>field_format</b> <i>The format used to save the date/time.</i></li>
  * </ul>
- *
+ * \note not available in Python bindings
  */
 
 class GUI_EXPORT QgsDateTimeEditWrapper : public QgsEditorWidgetWrapper
 {
     Q_OBJECT
   public:
-    explicit QgsDateTimeEditWrapper( QgsVectorLayer* vl, int fieldIdx, QWidget* editor, QWidget* parent = 0 );
+    explicit QgsDateTimeEditWrapper( QgsVectorLayer* vl, int fieldIdx, QWidget* editor, QWidget* parent = nullptr );
 
   private slots:
     void dateTimeChanged( const QDateTime &dateTime );
@@ -51,9 +51,10 @@ class GUI_EXPORT QgsDateTimeEditWrapper : public QgsEditorWidgetWrapper
 
     // QgsEditorWidgetWrapper interface
   public:
-    QVariant value() override;
+    QVariant value() const override;
     QWidget *createWidget( QWidget *parent ) override;
     void initWidget( QWidget *editor ) override;
+    bool valid() const override;
 
   public slots:
     void setValue( const QVariant &value ) override;
