@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 14.5.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -17,13 +17,14 @@
 
 #include "qgsrelationeditorwidget.h"
 #include "qgsattributeeditorcontext.h"
+#include "qgsproject.h"
 
 #include <QWidget>
 
 QgsRelationWidgetWrapper::QgsRelationWidgetWrapper( QgsVectorLayer* vl, const QgsRelation& relation, QWidget* editor, QWidget* parent )
     : QgsWidgetWrapper( vl, editor, parent )
     , mRelation( relation )
-    , mWidget( NULL )
+    , mWidget( nullptr )
 {
 }
 
@@ -35,7 +36,7 @@ QWidget* QgsRelationWidgetWrapper::createWidget( QWidget* parent )
 void QgsRelationWidgetWrapper::setFeature( const QgsFeature& feature )
 {
   if ( mWidget && mRelation.isValid() )
-    mWidget->setRelationFeature( mRelation, feature );
+    mWidget->setFeature( feature );
 }
 
 void QgsRelationWidgetWrapper::initWidget( QWidget* editor )
@@ -71,5 +72,14 @@ void QgsRelationWidgetWrapper::initWidget( QWidget* editor )
   }
   while ( ctx );
 
+  QgsRelation nmrel = QgsProject::instance()->relationManager()->relation( config( "nm-rel" ).toString() );
+
+  w->setRelations( mRelation, nmrel );
+
   mWidget = w;
 }
+
+bool QgsRelationWidgetWrapper::valid() const
+{
+  return mWidget;
+}
