@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 5.1.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -111,9 +111,9 @@ void QgsValueMapConfigDlg::removeSelectedButtonPushed()
       }
     }
   }
-  for ( i = 0; i < rowsToRemove.values().size(); i++ )
+  for ( i = 0; i < rowsToRemove.size(); i++ )
   {
-    tableWidget->removeRow( rowsToRemove.values()[i] - removed );
+    tableWidget->removeRow( rowsToRemove.values().at( i ) - removed );
     removed++;
   }
 }
@@ -161,7 +161,7 @@ void QgsValueMapConfigDlg::loadFromLayerButtonPushed()
 
 void QgsValueMapConfigDlg::loadFromCSVButtonPushed()
 {
-  QString fileName = QFileDialog::getOpenFileName( 0, tr( "Select a file" ) );
+  QString fileName = QFileDialog::getOpenFileName( nullptr, tr( "Select a file" ), QDir::homePath() );
   if ( fileName.isNull() )
     return;
 
@@ -169,9 +169,9 @@ void QgsValueMapConfigDlg::loadFromCSVButtonPushed()
 
   if ( !f.open( QIODevice::ReadOnly ) )
   {
-    QMessageBox::information( NULL,
+    QMessageBox::information( nullptr,
                               tr( "Error" ),
-                              tr( "Could not open file %1\nError was:%2" ).arg( fileName ).arg( f.errorString() ),
+                              tr( "Could not open file %1\nError was:%2" ).arg( fileName, f.errorString() ),
                               QMessageBox::Cancel );
     return;
   }
@@ -205,14 +205,14 @@ void QgsValueMapConfigDlg::loadFromCSVButtonPushed()
     else
       continue;
 
-    if (( key.startsWith( "\"" ) && key.endsWith( "\"" ) ) ||
-        ( key.startsWith( "'" ) && key.endsWith( "'" ) ) )
+    if (( key.startsWith( '\"' ) && key.endsWith( '\"' ) ) ||
+        ( key.startsWith( '\'' ) && key.endsWith( '\'' ) ) )
     {
       key = key.mid( 1, key.length() - 2 );
     }
 
-    if (( val.startsWith( "\"" ) && val.endsWith( "\"" ) ) ||
-        ( val.startsWith( "'" ) && val.endsWith( "'" ) ) )
+    if (( val.startsWith( '\"' ) && val.endsWith( '\"' ) ) ||
+        ( val.startsWith( '\'' ) && val.endsWith( '\'' ) ) )
     {
       val = val.mid( 1, val.length() - 2 );
     }
