@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 5.1.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -16,6 +16,8 @@
 #include "qgsvaluemapwidgetfactory.h"
 
 #include "qgsvaluemapwidgetwrapper.h"
+#include "qgsvaluemapsearchwidgetwrapper.h"
+#include "qgsdefaultsearchwidgetwrapper.h"
 #include "qgsvaluemapconfigdlg.h"
 
 QgsValueMapWidgetFactory::QgsValueMapWidgetFactory( const QString& name )
@@ -29,6 +31,12 @@ QgsEditorWidgetWrapper* QgsValueMapWidgetFactory::create( QgsVectorLayer* vl, in
   return new QgsValueMapWidgetWrapper( vl, fieldIdx, editor, parent );
 }
 
+
+QgsSearchWidgetWrapper* QgsValueMapWidgetFactory::createSearchWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
+{
+  return new QgsValueMapSearchWidgetWrapper( vl, fieldIdx, parent );
+}
+
 QgsEditorConfigWidget* QgsValueMapWidgetFactory::configWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
 {
   return new QgsValueMapConfigDlg( vl, fieldIdx, parent );
@@ -43,7 +51,7 @@ QgsEditorWidgetConfig QgsValueMapWidgetFactory::readConfig( const QDomElement& c
 
   QDomNodeList nodes = configElement.elementsByTagName( "value" );
 
-  for ( unsigned int i = 0; i < nodes.length(); ++i )
+  for ( int i = 0; i < nodes.size(); ++i )
   {
     QDomElement elem = nodes.at( i ).toElement();
     cfg.insert( elem.attribute( "key" ), elem.attribute( "value" ) );
@@ -80,3 +88,10 @@ QString QgsValueMapWidgetFactory::representValue( QgsVectorLayer* vl, int fieldI
 
   return config.key( value, QVariant( QString( "(%1)" ).arg( value.toString() ) ).toString() );
 }
+
+QMap<const char*, int> QgsValueMapWidgetFactory::supportedWidgetTypes()
+{
+  QMap<const char*, int> map = QMap<const char*, int>();
+  map.insert( QComboBox::staticMetaObject.className(), 10 );
+  return map;
+}
