@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 5.1.2014
     Copyright            : (C) 2014 Matthias Kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,7 +19,7 @@
 #include "qgslogger.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsvaluerelationconfigdlg.h"
-#include "qgsvaluerelationwidgetfactory.h"
+#include "qgsvaluerelationsearchwidgetwrapper.h"
 #include "qgsvectorlayer.h"
 
 #include <QSettings>
@@ -34,6 +34,11 @@ QgsEditorWidgetWrapper* QgsValueRelationWidgetFactory::create( QgsVectorLayer* v
   return new QgsValueRelationWidgetWrapper( vl, fieldIdx, editor, parent );
 }
 
+QgsSearchWidgetWrapper *QgsValueRelationWidgetFactory::createSearchWidget( QgsVectorLayer *vl, int fieldIdx, QWidget *parent ) const
+{
+  return new QgsValueRelationSearchWidgetWrapper( vl, fieldIdx, parent );
+}
+
 QgsEditorConfigWidget* QgsValueRelationWidgetFactory::configWidget( QgsVectorLayer* vl, int fieldIdx, QWidget* parent ) const
 {
   return new QgsValueRelationConfigDlg( vl, fieldIdx, parent );
@@ -53,6 +58,7 @@ QgsEditorWidgetConfig QgsValueRelationWidgetFactory::readConfig( const QDomEleme
   cfg.insert( "OrderByValue", configElement.attribute( "OrderByValue" ) );
   cfg.insert( "AllowMulti", configElement.attribute( "AllowMulti" ) );
   cfg.insert( "AllowNull", configElement.attribute( "AllowNull" ) );
+  cfg.insert( "UseCompleter", configElement.attribute( "UseCompleter" ) );
 
   return cfg;
 }
@@ -70,6 +76,7 @@ void QgsValueRelationWidgetFactory::writeConfig( const QgsEditorWidgetConfig& co
   configElement.setAttribute( "OrderByValue", config.value( "OrderByValue" ).toBool() );
   configElement.setAttribute( "AllowMulti", config.value( "AllowMulti" ).toBool() );
   configElement.setAttribute( "AllowNull", config.value( "AllowNull" ).toBool() );
+  configElement.setAttribute( "UseCompleter", config.value( "UseCompleter" ).toBool() );
 }
 
 QString QgsValueRelationWidgetFactory::representValue( QgsVectorLayer* vl, int fieldIdx, const QgsEditorWidgetConfig& config, const QVariant& cache, const QVariant& value ) const
@@ -90,7 +97,7 @@ QString QgsValueRelationWidgetFactory::representValue( QgsVectorLayer* vl, int f
 
   if ( config.value( "AllowMulti" ).toBool() )
   {
-    QStringList keyList = value.toString().remove( QChar( '{' ) ).remove( QChar( '}' ) ).split( "," );
+    QStringList keyList = value.toString().remove( QChar( '{' ) ).remove( QChar( '}' ) ).split( ',' );
     QStringList valueList;
 
     Q_FOREACH ( const QgsValueRelationWidgetWrapper::ValueRelationItem& item, vrCache )
