@@ -29,7 +29,7 @@ QgsLayerTreeMapCanvasBridge::QgsLayerTreeMapCanvasBridge( QgsLayerTreeGroup *roo
     , mHasCustomLayerOrder( false )
     , mAutoSetupOnFirstLayer( true )
     , mAutoEnableCrsTransform( true )
-    , mLastLayerCount( root->findLayers().count() )
+    , mLastLayerCount( !root->findLayers().isEmpty() )
 {
   connect( root, SIGNAL( addedChildren( QgsLayerTreeNode*, int, int ) ), this, SLOT( nodeAddedChildren( QgsLayerTreeNode*, int, int ) ) );
   connect( root, SIGNAL( customPropertyChanged( QgsLayerTreeNode*, QString ) ), this, SLOT( nodeCustomPropertyChanged( QgsLayerTreeNode*, QString ) ) );
@@ -60,7 +60,7 @@ void QgsLayerTreeMapCanvasBridge::defaultLayerOrder( QgsLayerTreeNode* node, QSt
     order << nodeLayer->layerId();
   }
 
-  foreach ( QgsLayerTreeNode* child, node->children() )
+  Q_FOREACH ( QgsLayerTreeNode* child, node->children() )
     defaultLayerOrder( child, order );
 }
 
@@ -119,7 +119,7 @@ void QgsLayerTreeMapCanvasBridge::setCanvasLayers()
 
   if ( mHasCustomLayerOrder )
   {
-    foreach ( QString layerId, mCustomLayerOrder )
+    Q_FOREACH ( const QString& layerId, mCustomLayerOrder )
     {
       QgsLayerTreeLayer* nodeLayer = mRoot->findLayer( layerId );
       if ( nodeLayer )
@@ -138,14 +138,13 @@ void QgsLayerTreeMapCanvasBridge::setCanvasLayers()
     // also setup destination CRS and map units if the OTF projections are not yet enabled
     if ( !mCanvas->mapSettings().hasCrsTransformEnabled() )
     {
-      foreach ( QgsLayerTreeLayer* layerNode, layerNodes )
+      Q_FOREACH ( QgsLayerTreeLayer* layerNode, layerNodes )
       {
-        if ( layerNode->layer() &&
-             (
-               qobject_cast<QgsVectorLayer *>( layerNode->layer() ) == 0 ||
-               qobject_cast<QgsVectorLayer *>( layerNode->layer() )->geometryType() != QGis::NoGeometry
-             )
-           )
+        if ( !layerNode->layer() )
+          continue;
+
+        QgsVectorLayer *vl = qobject_cast<QgsVectorLayer *>( layerNode->layer() );
+        if ( !vl || vl->geometryType() != QGis::NoGeometry )
         {
           mCanvas->setDestinationCrs( layerNode->layer()->crs() );
           mCanvas->setMapUnits( layerNode->layer()->crs().mapUnits() );
@@ -166,7 +165,7 @@ void QgsLayerTreeMapCanvasBridge::setCanvasLayers()
   if ( !mFirstCRS.isValid() )
   {
     // find out what is the first used CRS in case we may need to turn on OTF projections later
-    foreach ( QgsLayerTreeLayer* layerNode, layerNodes )
+    Q_FOREACH ( QgsLayerTreeLayer* layerNode, layerNodes )
     {
       if ( layerNode->layer() && layerNode->layer()->crs().isValid() )
       {
@@ -179,7 +178,7 @@ void QgsLayerTreeMapCanvasBridge::setCanvasLayers()
   if ( mAutoEnableCrsTransform && mFirstCRS.isValid() && !mCanvas->mapSettings().hasCrsTransformEnabled() )
   {
     // check whether all layers still have the same CRS
-    foreach ( QgsLayerTreeLayer* layerNode, layerNodes )
+    Q_FOREACH ( QgsLayerTreeLayer* layerNode, layerNodes )
     {
       if ( layerNode->layer() && layerNode->layer()->crs().isValid() && layerNode->layer()->crs() != mFirstCRS )
       {
@@ -225,7 +224,7 @@ void QgsLayerTreeMapCanvasBridge::readProject( const QDomDocument& doc )
       itemElem = itemElem.nextSiblingElement( "item" );
     }
 
-    setHasCustomLayerOrder( customOrderElem.attribute( "enabled", 0 ).toInt() );
+    setHasCustomLayerOrder( customOrderElem.attribute( "enabled", nullptr ).toInt() );
     setCustomLayerOrder( order );
   }
 }
@@ -236,7 +235,7 @@ void QgsLayerTreeMapCanvasBridge::writeProject( QDomDocument& doc )
   QDomElement customOrderElem = doc.createElement( "custom-order" );
   customOrderElem.setAttribute( "enabled", mHasCustomLayerOrder ? 1 : 0 );
 
-  foreach ( QString layerId, mCustomLayerOrder )
+  Q_FOREACH ( const QString& layerId, mCustomLayerOrder )
   {
     QDomElement itemElem = doc.createElement( "item" );
     itemElem.appendChild( doc.createTextNode( layerId ) );
@@ -255,7 +254,7 @@ void QgsLayerTreeMapCanvasBridge::setCanvasLayers( QgsLayerTreeNode *node, QList
     layers << QgsMapCanvasLayer( nodeLayer->layer(), nodeLayer->isVisible() == Qt::Checked, nodeLayer->customProperty( "overview", 0 ).toInt() );
   }
 
-  foreach ( QgsLayerTreeNode* child, node->children() )
+  Q_FOREACH ( QgsLayerTreeNode* child, node->children() )
     setCanvasLayers( child, layers );
 }
 
@@ -284,12 +283,12 @@ void QgsLayerTreeMapCanvasBridge::nodeAddedChildren( QgsLayerTreeNode* node, int
     }
     else if ( QgsLayerTree::isGroup( child ) )
     {
-      foreach ( QgsLayerTreeLayer* nodeL, QgsLayerTree::toGroup( child )->findLayers() )
+      Q_FOREACH ( QgsLayerTreeLayer* nodeL, QgsLayerTree::toGroup( child )->findLayers() )
         layerIds << nodeL->layerId();
     }
   }
 
-  foreach ( QString layerId, layerIds )
+  Q_FOREACH ( const QString& layerId, layerIds )
   {
     if ( !mCustomLayerOrder.contains( layerId ) )
       mCustomLayerOrder.append( layerId );
@@ -324,7 +323,7 @@ void QgsLayerTreeMapCanvasBridge::nodeVisibilityChanged()
   deferredSetCanvasLayers();
 }
 
-void QgsLayerTreeMapCanvasBridge::nodeCustomPropertyChanged( QgsLayerTreeNode* node, QString key )
+void QgsLayerTreeMapCanvasBridge::nodeCustomPropertyChanged( QgsLayerTreeNode* node, const QString& key )
 {
   Q_UNUSED( node );
   if ( key == "overview" )
