@@ -33,16 +33,16 @@ namespace QgisGui
 
     QSettings settings;
     QString lastUsedFilter = settings.value( "/UI/" + filterName, "" ).toString();
-    QString lastUsedDir = settings.value( "/UI/" + filterName + "Dir", "." ).toString();
+    QString lastUsedDir = settings.value( "/UI/" + filterName + "Dir", QDir::homePath() ).toString();
 
     QgsDebugMsg( "Opening file dialog with filters: " + filters );
     if ( !cancelAll )
     {
-      selectedFiles = QFileDialog::getOpenFileNames( 0, title, lastUsedDir, filters, &lastUsedFilter );
+      selectedFiles = QFileDialog::getOpenFileNames( nullptr, title, lastUsedDir, filters, &lastUsedFilter );
     }
     else //we have to use non-native dialog to add cancel all button
     {
-      QgsEncodingFileDialog* openFileDialog = new QgsEncodingFileDialog( 0, title, lastUsedDir, filters, QString( "" ) );
+      QgsEncodingFileDialog* openFileDialog = new QgsEncodingFileDialog( nullptr, title, lastUsedDir, filters, QString() );
 
       // allow for selection of more than one file
       openFileDialog->setFileMode( QFileDialog::ExistingFiles );
@@ -83,11 +83,11 @@ namespace QgisGui
     return false;
   }
 
-  QPair<QString, QString> GUI_EXPORT getSaveAsImageName( QWidget *theParent, QString theMessage, QString defaultFilename )
+  QPair<QString, QString> GUI_EXPORT getSaveAsImageName( QWidget *theParent, const QString& theMessage, const QString& defaultFilename )
   {
     // get a list of supported output image types
     QMap<QString, QString> filterMap;
-    foreach ( QByteArray format, QImageWriter::supportedImageFormats() )
+    Q_FOREACH ( const QByteArray& format, QImageWriter::supportedImageFormats() )
     {
       //svg doesnt work so skip it
       if ( format ==  "svg" )
@@ -105,7 +105,7 @@ namespace QgisGui
 #endif
 
     QSettings settings;  // where we keep last used filter in persistent state
-    QString lastUsedDir = settings.value( "/UI/lastSaveAsImageDir", "." ).toString();
+    QString lastUsedDir = settings.value( "/UI/lastSaveAsImageDir", QDir::homePath() ).toString();
 
     // Prefer "png" format unless the user previously chose a different format
     QString pngExtension = "png";
@@ -168,9 +168,9 @@ namespace QgisGui
 #endif
 
     // Add the file type suffix to the fileName if required
-    if ( !ext.isNull() && !outputFileName.toLower().endsWith( "." + ext.toLower() ) )
+    if ( !ext.isNull() && !outputFileName.endsWith( '.' + ext.toLower(), Qt::CaseInsensitive ) )
     {
-      outputFileName += "." + ext;
+      outputFileName += '.' + ext;
     }
 
     return qMakePair<QString, QString>( outputFileName, ext );
@@ -178,7 +178,7 @@ namespace QgisGui
 
   QString createFileFilter_( QString const &longName, QString const &glob )
   {
-    return QString( "%1 (%2 %3)" ).arg( longName ).arg( glob.toLower() ).arg( glob.toUpper() );
+    return QString( "%1 (%2 %3)" ).arg( longName, glob.toLower(), glob.toUpper() );
   }
 
   QString createFileFilter_( QString const &format )
@@ -197,7 +197,7 @@ namespace QgisGui
     // Native Mac dialog works only for Qt Carbon
     return QFontDialog::getFont( &ok, initial, 0, title, QFontDialog::DontUseNativeDialog );
 #else
-    return QFontDialog::getFont( &ok, initial, 0, title );
+    return QFontDialog::getFont( &ok, initial, nullptr, title );
 #endif
   }
 
