@@ -46,7 +46,7 @@ class GUI_EXPORT QgsColorButton: public QPushButton
      * @param cdo Options for the color chooser dialog
      * @note changed in 1.9
      */
-    QgsColorButton( QWidget *parent = 0, QString cdt = "", QColorDialog::ColorDialogOptions cdo = 0 );
+    QgsColorButton( QWidget *parent = nullptr, const QString& cdt = "", const QColorDialog::ColorDialogOptions& cdo = nullptr );
     ~QgsColorButton();
 
     /**
@@ -67,7 +67,7 @@ class GUI_EXPORT QgsColorButton: public QPushButton
      *
      * @param cdo Options for the color chooser dialog
      */
-    void setColorDialogOptions( QColorDialog::ColorDialogOptions cdo );
+    void setColorDialogOptions( const QColorDialog::ColorDialogOptions& cdo );
 
     /**
      * Returns the options for the color chooser dialog.
@@ -81,7 +81,7 @@ class GUI_EXPORT QgsColorButton: public QPushButton
      *
      * @param cdt Title for the color chooser dialog
      */
-    void setColorDialogTitle( QString cdt );
+    void setColorDialogTitle( const QString& cdt );
 
     /**
      * Returns the title, which the color chooser dialog shows.
