@@ -28,6 +28,7 @@
 #include <QSettings>
 #include <QDrag>
 #include <cmath>
+#include "qgslogger.h"
 
 
 //
@@ -221,6 +222,25 @@ void QgsColorWidget::dropEvent( QDropEvent *e )
   //could not get color from mime data
 }
 
+void QgsColorWidget::mouseMoveEvent( QMouseEvent *e )
+{
+  emit hovered();
+  e->accept();
+  //don't pass to QWidget::mouseMoveEvent, causes issues with widget used in QWidgetAction
+}
+
+void QgsColorWidget::mousePressEvent( QMouseEvent *e )
+{
+  e->accept();
+  //don't pass to QWidget::mousePressEvent, causes issues with widget used in QWidgetAction
+}
+
+void QgsColorWidget::mouseReleaseEvent( QMouseEvent *e )
+{
+  e->accept();
+  //don't pass to QWidget::mouseReleaseEvent, causes issues with widget used in QWidgetAction
+}
+
 QColor QgsColorWidget::color() const
 {
   return mCurrentColor;
@@ -352,9 +372,9 @@ QgsColorWheel::QgsColorWheel( QWidget *parent )
     , mMargin( 4 )
     , mWheelThickness( 18 )
     , mClickedPart( QgsColorWheel::None )
-    , mWheelImage( 0 )
-    , mTriangleImage( 0 )
-    , mWidgetImage( 0 )
+    , mWheelImage( nullptr )
+    , mTriangleImage( nullptr )
+    , mWidgetImage( nullptr )
     , mWheelDirty( true )
     , mTriangleDirty( true )
 {
@@ -378,6 +398,11 @@ QgsColorWheel::~QgsColorWheel()
   delete mWidgetImage;
 }
 
+QSize QgsColorWheel::sizeHint() const
+{
+  return QSize( 200, 200 );
+}
+
 void QgsColorWheel::paintEvent( QPaintEvent *event )
 {
   Q_UNUSED( event );
@@ -585,6 +610,7 @@ void QgsColorWheel::setColorFromPos( const QPointF pos )
 void QgsColorWheel::mouseMoveEvent( QMouseEvent *event )
 {
   setColorFromPos( event->posF() );
+  QgsColorWidget::mouseMoveEvent( event );
 }
 
 void QgsColorWheel::mousePressEvent( QMouseEvent *event )
@@ -724,7 +750,7 @@ void QgsColorWheel::createTriangle()
 QgsColorBox::QgsColorBox( QWidget *parent, const ColorComponent component )
     : QgsColorWidget( parent, component )
     , mMargin( 2 )
-    , mBoxImage( 0 )
+    , mBoxImage( nullptr )
     , mDirty( true )
 {
   setFocusPolicy( Qt::StrongFocus );
@@ -828,6 +854,7 @@ void QgsColorBox::resizeEvent( QResizeEvent *event )
 void QgsColorBox::mouseMoveEvent( QMouseEvent *event )
 {
   setColorFromPoint( event->pos() );
+  QgsColorWidget::mouseMoveEvent( event );
 }
 
 void QgsColorBox::mousePressEvent( QMouseEvent *event )
@@ -920,7 +947,7 @@ int QgsColorBox::xComponentValue() const
   return componentValue( xComponent() );
 }
 
-void QgsColorBox::setColorFromPoint( const QPoint &point )
+void QgsColorBox::setColorFromPoint( QPoint point )
 {
   int valX = valueRangeX() * ( point.x() - mMargin ) / ( width() - 2 * mMargin - 1 );
   valX = qMin( qMax( valX, 0 ), valueRangeX() );
@@ -1148,6 +1175,7 @@ void QgsColorRampWidget::setMarkerSize( const int markerSize )
 void QgsColorRampWidget::mouseMoveEvent( QMouseEvent *event )
 {
   setColorFromPoint( event->posF() );
+  QgsColorWidget::mouseMoveEvent( event );
 }
 
 void QgsColorRampWidget::mousePressEvent( QMouseEvent *event )
@@ -1203,7 +1231,7 @@ void QgsColorRampWidget::keyPressEvent( QKeyEvent *event )
   }
 }
 
-void QgsColorRampWidget::setColorFromPoint( const QPointF &point )
+void QgsColorRampWidget::setColorFromPoint( QPointF point )
 {
   int oldValue = componentValue();
   int val;
@@ -1232,14 +1260,14 @@ void QgsColorRampWidget::setColorFromPoint( const QPointF &point )
 
 QgsColorSliderWidget::QgsColorSliderWidget( QWidget *parent, const ColorComponent component )
     : QgsColorWidget( parent, component )
-    , mRampWidget( 0 )
-    , mSpinBox( 0 )
+    , mRampWidget( nullptr )
+    , mSpinBox( nullptr )
 {
   QHBoxLayout* hLayout = new QHBoxLayout();
   hLayout->setMargin( 0 );
   hLayout->setSpacing( 5 );
 
-  mRampWidget = new QgsColorRampWidget( 0, component );
+  mRampWidget = new QgsColorRampWidget( nullptr, component );
   mRampWidget->setColor( mCurrentColor );
   hLayout->addWidget( mRampWidget, 1 );
 
@@ -1365,15 +1393,15 @@ int QgsColorSliderWidget::convertDisplayToReal( const int displayValue ) const
 
 QgsColorTextWidget::QgsColorTextWidget( QWidget *parent )
     : QgsColorWidget( parent )
-    , mLineEdit( 0 )
-    , mMenuButton( 0 )
+    , mLineEdit( nullptr )
+    , mMenuButton( nullptr )
     , mFormat( QgsColorTextWidget::HexRgb )
 {
   QHBoxLayout* hLayout = new QHBoxLayout();
   hLayout->setMargin( 0 );
   hLayout->setSpacing( 0 );
 
-  mLineEdit = new QLineEdit( 0 );
+  mLineEdit = new QLineEdit( nullptr );
   hLayout->addWidget( mLineEdit );
 
   mMenuButton = new QToolButton( mLineEdit );
@@ -1468,13 +1496,13 @@ void QgsColorTextWidget::showMenu()
 {
   QMenu colorContextMenu;
 
-  QAction* hexRgbAction = new QAction( tr( "#RRGGBB" ), 0 );
+  QAction* hexRgbAction = new QAction( tr( "#RRGGBB" ), nullptr );
   colorContextMenu.addAction( hexRgbAction );
-  QAction* hexRgbaAction = new QAction( tr( "#RRGGBBAA" ), 0 );
+  QAction* hexRgbaAction = new QAction( tr( "#RRGGBBAA" ), nullptr );
   colorContextMenu.addAction( hexRgbaAction );
-  QAction* rgbAction = new QAction( tr( "rgb( r, g, b )" ), 0 );
+  QAction* rgbAction = new QAction( tr( "rgb( r, g, b )" ), nullptr );
   colorContextMenu.addAction( rgbAction );
-  QAction* rgbaAction = new QAction( tr( "rgba( r, g, b, a )" ), 0 );
+  QAction* rgbaAction = new QAction( tr( "rgba( r, g, b, a )" ), nullptr );
   colorContextMenu.addAction( rgbaAction );
 
   QAction* selectedAction = colorContextMenu.exec( QCursor::pos() );
@@ -1519,7 +1547,7 @@ QgsColorPreviewWidget::~QgsColorPreviewWidget()
 
 }
 
-void QgsColorPreviewWidget::drawColor( const QColor &color, const QRect &rect, QPainter& painter )
+void QgsColorPreviewWidget::drawColor( const QColor &color, QRect rect, QPainter& painter )
 {
   painter.setPen( Qt::NoPen );
   //if color has an alpha, start with a checkboard pattern
@@ -1531,7 +1559,7 @@ void QgsColorPreviewWidget::drawColor( const QColor &color, const QRect &rect, Q
 
     //draw half of widget showing solid color, the other half showing color with alpha
 
-    //ensure at least a 1px overlap to avoid artefacts
+    //ensure at least a 1px overlap to avoid artifacts
     QBrush colorBrush = QBrush( color );
     painter.setBrush( colorBrush );
     painter.drawRect( floor( rect.width() / 2.0 ) + rect.left(), rect.top(), rect.width() - floor( rect.width() / 2.0 ), rect.height() );
@@ -1571,6 +1599,11 @@ void QgsColorPreviewWidget::paintEvent( QPaintEvent *event )
   painter.end();
 }
 
+QSize QgsColorPreviewWidget::sizeHint() const
+{
+  return QSize( 200, 150 );
+}
+
 void QgsColorPreviewWidget::setColor2( const QColor &color )
 {
   if ( color == mColor2 )
@@ -1587,7 +1620,7 @@ void QgsColorPreviewWidget::mousePressEvent( QMouseEvent *e )
   {
     mDragStartPosition = e->pos();
   }
-  QWidget::mousePressEvent( e );
+  QgsColorWidget::mousePressEvent( e );
 }
 
 void QgsColorPreviewWidget::mouseReleaseEvent( QMouseEvent *e )
@@ -1595,7 +1628,7 @@ void QgsColorPreviewWidget::mouseReleaseEvent( QMouseEvent *e )
   if (( e->pos() - mDragStartPosition ).manhattanLength() >= QApplication::startDragDistance() )
   {
     //mouse moved, so a drag. nothing to do here
-    QWidget::mouseReleaseEvent( e );
+    QgsColorWidget::mouseReleaseEvent( e );
     return;
   }
 
@@ -1621,14 +1654,14 @@ void QgsColorPreviewWidget::mouseMoveEvent( QMouseEvent *e )
   if ( !( e->buttons() & Qt::LeftButton ) )
   {
     //left button not depressed, so not a drag
-    QWidget::mouseMoveEvent( e );
+    QgsColorWidget::mouseMoveEvent( e );
     return;
   }
 
   if (( e->pos() - mDragStartPosition ).manhattanLength() < QApplication::startDragDistance() )
   {
     //mouse not moved, so not a drag
-    QWidget::mouseMoveEvent( e );
+    QgsColorWidget::mouseMoveEvent( e );
     return;
   }
 
@@ -1651,3 +1684,53 @@ void QgsColorPreviewWidget::mouseMoveEvent( QMouseEvent *e )
   drag->setPixmap( createDragIcon( dragColor ) );
   drag->exec( Qt::CopyAction );
 }
+
+
+//
+// QgsColorWidgetAction
+//
+
+QgsColorWidgetAction::QgsColorWidgetAction( QgsColorWidget* colorWidget, QMenu* menu, QWidget* parent )
+    : QWidgetAction( parent )
+    , mMenu( menu )
+    , mColorWidget( colorWidget )
+    , mSuppressRecurse( false )
+    , mDismissOnColorSelection( true )
+{
+  setDefaultWidget( mColorWidget );
+  connect( mColorWidget, SIGNAL( colorChanged( QColor ) ), this, SLOT( setColor( QColor ) ) );
+
+  connect( this, SIGNAL( hovered() ), this, SLOT( onHover() ) );
+  connect( mColorWidget, SIGNAL( hovered() ), this, SLOT( onHover() ) );
+}
+
+QgsColorWidgetAction::~QgsColorWidgetAction()
+{
+
+}
+
+void QgsColorWidgetAction::onHover()
+{
+  //see https://bugreports.qt-project.org/browse/QTBUG-10427?focusedCommentId=185610&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-185610
+  if ( mSuppressRecurse )
+  {
+    return;
+  }
+
+  if ( mMenu )
+  {
+    mSuppressRecurse = true;
+    mMenu->setActiveAction( this );
+    mSuppressRecurse = false;
+  }
+}
+
+void QgsColorWidgetAction::setColor( const QColor& color )
+{
+  emit colorChanged( color );
+  QAction::trigger();
+  if ( mMenu && mDismissOnColorSelection )
+  {
+    mMenu->hide();
+  }
+}
