@@ -6,7 +6,7 @@
 class QgsComposition;
 class QGraphicsLineItem;
 
-/**A class to show paper scale and the current cursor position*/
+/** A class to show paper scale and the current cursor position*/
 class GUI_EXPORT QgsComposerRuler: public QWidget
 {
     Q_OBJECT
@@ -24,7 +24,7 @@ class GUI_EXPORT QgsComposerRuler: public QWidget
     QSize minimumSizeHint() const override;
 
     void setSceneTransform( const QTransform& transform );
-    void updateMarker( const QPointF& pos ) { mMarkerPos = pos; repaint(); }
+    void updateMarker( QPointF pos ) { mMarkerPos = pos; repaint(); }
 
     void setComposition( QgsComposition* c ) { mComposition = c; }
     QgsComposition* composition() { return mComposition; }
@@ -58,7 +58,7 @@ class GUI_EXPORT QgsComposerRuler: public QWidget
     int mTextBaseline;
     int mMinSpacingVerticalLabels;
 
-    void setSnapLinePosition( const QPointF& pos );
+    void setSnapLinePosition( QPointF pos );
 
     //calculate optimum labeled units for ruler so that labels are a good distance apart
     int optimumScale( double minPixelDiff, int &magnitude, int &multiple );
@@ -79,7 +79,7 @@ class GUI_EXPORT QgsComposerRuler: public QWidget
     void drawMarkerPos( QPainter *painter );
 
   signals:
-    /**Is emitted when mouse cursor coordinates change*/
+    /** Is emitted when mouse cursor coordinates change*/
     void cursorPosChanged( QPointF );
 
 };
