@@ -50,7 +50,7 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
 
   public:
 
-    /**Current tool*/
+    /** Current tool*/
     enum Tool
     {
       Select = 0,      // Select/Move item
@@ -91,70 +91,77 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
       ActiveUntilMouseRelease
     };
 
-    QgsComposerView( QWidget* parent = 0, const char* name = 0, Qt::WindowFlags f = 0 );
+    QgsComposerView( QWidget* parent = nullptr, const char* name = nullptr, const Qt::WindowFlags& f = nullptr );
 
-    /**Add an item group containing the selected items*/
+    /** Add an item group containing the selected items*/
     void groupItems();
 
-    /**Ungroups the selected items*/
+    /** Ungroups the selected items*/
     void ungroupItems();
 
-    /**Cuts or copies the selected items*/
+    /** Cuts or copies the selected items*/
     void copyItems( ClipboardMode mode );
 
-    /**Pastes items from clipboard*/
+    /** Pastes items from clipboard*/
     void pasteItems( PasteMode mode );
 
-    /**Deletes selected items*/
+    /** Deletes selected items*/
     void deleteSelectedItems();
 
-    /**Selects all items*/
+    /** Selects all items*/
     void selectAll();
 
-    /**Deselects all items*/
+    /** Deselects all items*/
     void selectNone();
 
-    /**Inverts current selection*/
+    /** Inverts current selection*/
     void selectInvert();
 
     QgsComposerView::Tool currentTool() const {return mCurrentTool;}
     void setCurrentTool( QgsComposerView::Tool t );
 
-    /**Sets the composition for the view. If the composition is being set manually and not by a QgsComposer, then this must
+    /** Sets the composition for the view. If the composition is being set manually and not by a QgsComposer, then this must
      * be set BEFORE adding any items to the composition.
-    */
+     */
     void setComposition( QgsComposition* c );
 
-    /**Returns the composition or 0 in case of error*/
+    /** Returns the composition or 0 in case of error*/
     QgsComposition* composition();
 
-    /**Returns the composer main window*/
+    /** Returns the composer main window*/
     QMainWindow* composerWindow();
 
     void setPaintingEnabled( bool enabled ) { mPaintingEnabled = enabled; }
     bool paintingEnabled() const { return mPaintingEnabled; }
 
-    /**Update rulers with current scene rect*/
+    /** Update rulers with current scene rect*/
     void updateRulers();
 
     void setHorizontalRuler( QgsComposerRuler* r ) { mHorizontalRuler = r; }
     void setVerticalRuler( QgsComposerRuler* r ) { mVerticalRuler = r; }
 
-    /**Set zoom level, where a zoom level of 1.0 corresponds to 100%*/
+    /** Set zoom level, where a zoom level of 1.0 corresponds to 100%*/
     void setZoomLevel( double zoomLevel );
 
-    /**Sets whether a preview effect should be used to alter the view's appearance
+    /** Scales the view in a safe way, by limiting the acceptable range
+     * of the scale applied.
+     * @param scale factor to scale view by
+     * @note added in QGIS 2.16
+     */
+    void scaleSafe( double scale );
+
+    /** Sets whether a preview effect should be used to alter the view's appearance
      * @param enabled Set to true to enable the preview effect on the view
      * @note added in 2.3
      * @see setPreviewMode
-    */
+     */
     void setPreviewModeEnabled( bool enabled );
-    /**Sets the preview mode which should be used to modify the view's appearance. Preview modes are only used
+    /** Sets the preview mode which should be used to modify the view's appearance. Preview modes are only used
      * if setPreviewMode is set to true.
      * @param mode PreviewMode to be used to draw the view
      * @note added in 2.3
      * @see setPreviewModeEnabled
-    */
+     */
     void setPreviewMode( QgsPreviewEffect::PreviewMode mode );
 
   protected:
@@ -177,27 +184,27 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
     void scrollContentsBy( int dx, int dy ) override;
 
   private:
-    /**Current composer tool*/
+    /** Current composer tool*/
     QgsComposerView::Tool mCurrentTool;
-    /**Previous composer tool*/
+    /** Previous composer tool*/
     QgsComposerView::Tool mPreviousTool;
 
-    /**Rubber band item*/
+    /** Rubber band item*/
     QGraphicsRectItem* mRubberBandItem;
-    /**Rubber band item for arrows*/
+    /** Rubber band item for arrows*/
     QGraphicsLineItem* mRubberBandLineItem;
-    /**Item to move content*/
+    /** Item to move content*/
     QgsComposerItem* mMoveContentItem;
-    /**Start position of content move*/
+    /** Start position of content move*/
     QPointF mMoveContentStartPos;
-    /**Start of rubber band creation*/
+    /** Start of rubber band creation*/
     QPointF mRubberBandStartPos;
 
-    /**True if user is currently selecting by marquee*/
+    /** True if user is currently selecting by marquee*/
     bool mMarqueeSelect;
-    /**True if user is currently zooming by marquee*/
+    /** True if user is currently zooming by marquee*/
     bool mMarqueeZoom;
-    /**True if user is currently temporarily activating the zoom tool by holding control+space*/
+    /** True if user is currently temporarily activating the zoom tool by holding control+space*/
     QgsComposerView::ToolStatus mTemporaryZoomStatus;
 
     bool mPaintingEnabled;
@@ -208,14 +215,14 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
     /** Draw a shape on the canvas */
     void addShape( Tool currentTool );
 
-    /**True if user is currently panning by clicking and dragging with the pan tool*/
+    /** True if user is currently panning by clicking and dragging with the pan tool*/
     bool mToolPanning;
-    /**True if user is currently panning by holding the middle mouse button*/
+    /** True if user is currently panning by holding the middle mouse button*/
     bool mMousePanning;
-    /**True if user is currently panning by holding the space key*/
+    /** True if user is currently panning by holding the space key*/
     bool mKeyPanning;
 
-    /**True if user is currently dragging with the move item content tool*/
+    /** True if user is currently dragging with the move item content tool*/
     bool mMovingItemContent;
 
     QPoint mMouseLastXY;
@@ -224,48 +231,48 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
 
     QgsPreviewEffect* mPreviewEffect;
 
-    /**Returns the default mouse cursor for a tool*/
+    /** Returns the default mouse cursor for a tool*/
     QCursor defaultCursorForTool( Tool currentTool );
 
-    /**Zoom composition from a mouse wheel event*/
+    /** Zoom composition from a mouse wheel event*/
     void wheelZoom( QWheelEvent * event );
-    /**Redraws the rectangular rubber band*/
+    /** Redraws the rectangular rubber band*/
     void updateRubberBandRect( QPointF & pos, const bool constrainSquare = false, const bool fromCenter = false );
-    /**Redraws the linear rubber band*/
-    void updateRubberBandLine( const QPointF & pos, const bool constrainAngles = false );
-    /**Removes the rubber band and cleans up*/
+    /** Redraws the linear rubber band*/
+    void updateRubberBandLine( QPointF pos, const bool constrainAngles = false );
+    /** Removes the rubber band and cleans up*/
     void removeRubberBand();
 
-    /**Starts a marquee selection*/
+    /** Starts a marquee selection*/
     void startMarqueeSelect( QPointF & scenePoint );
-    /**Finalises a marquee selection*/
+    /** Finalises a marquee selection*/
     void endMarqueeSelect( QMouseEvent* e );
-    /**Starts a zoom in marquee*/
+    /** Starts a zoom in marquee*/
     void startMarqueeZoom( QPointF & scenePoint );
-    /**Finalises a marquee zoom*/
+    /** Finalises a marquee zoom*/
     void endMarqueeZoom( QMouseEvent* e );
 
     //void connectAddRemoveCommandSignals( QgsAddRemoveItemCommand* c );
 
   signals:
-    /**Is emitted when selected item changed. If 0, no item is selected*/
+    /** Is emitted when selected item changed. If 0, no item is selected*/
     void selectedItemChanged( QgsComposerItem* selected );
-    /**Is emitted when a composer item has been removed from the scene*/
+    /** Is emitted when a composer item has been removed from the scene*/
     void itemRemoved( QgsComposerItem* );
-    /**Current action (e.g. adding composer map) has been finished. The purpose of this signal is that
+    /** Current action (e.g. adding composer map) has been finished. The purpose of this signal is that
      QgsComposer may set the selection tool again*/
     void actionFinished();
-    /**Is emitted when mouse cursor coordinates change*/
+    /** Is emitted when mouse cursor coordinates change*/
     void cursorPosChanged( QPointF );
-    /**Is emitted when the view zoom changes*/
+    /** Is emitted when the view zoom changes*/
     void zoomLevelChanged();
 
-    /**Emitted before composerview is shown*/
+    /** Emitted before composerview is shown*/
     void composerViewShow( QgsComposerView* );
-    /**Emitted before composerview is hidden*/
+    /** Emitted before composerview is hidden*/
     void composerViewHide( QgsComposerView* );
 
-    /**Emitted when the composition is set for the view*/
+    /** Emitted when the composition is set for the view*/
     void compositionSet( QgsComposition* );
 };
 
