@@ -26,6 +26,7 @@
 #include <QMenu>
 #include <QMouseEvent>
 #include <QPointer>
+#include <QGroupBox>
 
 
 QIcon QgsDataDefinedButton::mIconDataDefine;
@@ -38,9 +39,11 @@ QIcon QgsDataDefinedButton::mIconDataDefineExpressionError;
 QgsDataDefinedButton::QgsDataDefinedButton( QWidget* parent,
     const QgsVectorLayer* vl,
     const QgsDataDefined* datadefined,
-    DataTypes datatypes,
-    QString description )
+    const DataTypes& datatypes,
+    const QString& description )
     : QToolButton( parent )
+    , mExpressionContextCallback( nullptr )
+    , mExpressionContextCallbackContext( nullptr )
 {
   // set up static icons
   if ( mIconDataDefine.isNull() )
@@ -67,11 +70,14 @@ QgsDataDefinedButton::QgsDataDefinedButton( QWidget* parent,
   setMenu( mDefineMenu );
 
   mFieldsMenu = new QMenu( this );
-
   mActionDataTypes = new QAction( this );
   // list fields and types in submenu, since there may be many
   mActionDataTypes->setMenu( mFieldsMenu );
 
+  mActionVariables = new QAction( tr( "Variable" ), this );
+  mVariablesMenu = new QMenu( this );
+  mActionVariables->setMenu( mVariablesMenu );
+
   mActionActive = new QAction( this );
   QFont f = mActionActive->font();
   f.setBold( true );
@@ -80,10 +86,15 @@ QgsDataDefinedButton::QgsDataDefinedButton( QWidget* parent,
   mActionDescription = new QAction( tr( "Description..." ), this );
 
   mActionExpDialog = new QAction( tr( "Edit..." ), this );
-  mActionExpression = 0;
+  mActionExpression = nullptr;
   mActionPasteExpr = new QAction( tr( "Paste" ), this );
   mActionCopyExpr = new QAction( tr( "Copy" ), this );
   mActionClearExpr = new QAction( tr( "Clear" ), this );
+  mActionAssistant = new QAction( tr( "Assistant..." ), this );
+  QFont assistantFont = mActionAssistant->font();
+  assistantFont.setBold( true );
+  mActionAssistant->setFont( assistantFont );
+  mDefineMenu->addAction( mActionAssistant );
 
   // set up sibling widget connections
   connect( this, SIGNAL( dataDefinedActivated( bool ) ), this, SLOT( disableEnabledWidgets( bool ) ) );
@@ -100,8 +111,8 @@ QgsDataDefinedButton::~QgsDataDefinedButton()
 
 void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
                                  const QgsDataDefined* datadefined,
-                                 DataTypes datatypes,
-                                 QString description )
+                                 const DataTypes& datatypes,
+                                 const QString& description )
 {
   mVectorLayer = vl;
   // construct default property if none or incorrect passed in
@@ -109,8 +120,8 @@ void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
   {
     mProperty.insert( "active", "0" );
     mProperty.insert( "useexpr", "0" );
-    mProperty.insert( "expression", "" );
-    mProperty.insert( "field", "" );
+    mProperty.insert( "expression", QString() );
+    mProperty.insert( "field", QString() );
   }
   else
   {
@@ -125,12 +136,12 @@ void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
   mFieldTypeList.clear();
 
   mInputDescription = description;
-  mFullDescription = QString( "" );
-  mUsageInfo = QString( "" );
-  mCurrentDefinition = QString( "" );
+  mFullDescription.clear();
+  mUsageInfo.clear();
+  mCurrentDefinition.clear();
 
   // set up data types string
-  mDataTypesString = QString( "" );
+  mDataTypesString.clear();
 
   QStringList ts;
   if ( mDataTypes.testFlag( String ) )
@@ -155,7 +166,7 @@ void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
   if ( mVectorLayer )
   {
     // store just a list of fields of unknown type or those that match the expected type
-    const QgsFields& fields = mVectorLayer->pendingFields();
+    const QgsFields& fields = mVectorLayer->fields();
     for ( int i = 0; i < fields.count(); ++i )
     {
       const QgsField& f = fields.at( i );
@@ -192,6 +203,24 @@ void QgsDataDefinedButton::init( const QgsVectorLayer* vl,
   updateGui();
 }
 
+void QgsDataDefinedButton::updateDataDefined( QgsDataDefined *dd ) const
+{
+  if ( !dd )
+    return;
+
+  dd->setActive( isActive() );
+  dd->setExpressionString( getExpression() );
+  dd->setField( getField() );
+  dd->setUseExpression( useExpression() );
+}
+
+QgsDataDefined QgsDataDefinedButton::currentDataDefined() const
+{
+  QgsDataDefined dd;
+  updateDataDefined( &dd );
+  return dd;
+}
+
 void QgsDataDefinedButton::mouseReleaseEvent( QMouseEvent *event )
 {
   // Ctrl-click to toggle activated state
@@ -237,7 +266,7 @@ void QgsDataDefinedButton::aboutToShowMenu()
 
   if ( addActiveAction )
   {
-    ddTitleAct->setText( ddTitle + " (" + ( useExpression() ? tr( "expression" ) : tr( "field" ) ) + ")" );
+    ddTitleAct->setText( ddTitle + " (" + ( useExpression() ? tr( "expression" ) : tr( "field" ) ) + ')' );
     mDefineMenu->addAction( mActionActive );
     mActionActive->setText( isActive() ? tr( "Deactivate" ) : tr( "Activate" ) );
     mActionActive->setData( QVariant( isActive() ? false : true ) );
@@ -250,6 +279,7 @@ void QgsDataDefinedButton::aboutToShowMenu()
 
   mDefineMenu->addSeparator();
 
+  bool fieldActive = false;
   if ( !mDataTypesString.isEmpty() )
   {
     QAction* fieldTitleAct = mDefineMenu->addAction( tr( "Attribute field" ) );
@@ -260,18 +290,19 @@ void QgsDataDefinedButton::aboutToShowMenu()
 
     mFieldsMenu->clear();
 
-    if ( mFieldNameList.size() > 0 )
+    if ( !mFieldNameList.isEmpty() )
     {
 
       for ( int j = 0; j < mFieldNameList.count(); ++j )
       {
         QString fldname = mFieldNameList.at( j );
-        QAction* act = mFieldsMenu->addAction( fldname + "    (" + mFieldTypeList.at( j ) + ")" );
+        QAction* act = mFieldsMenu->addAction( fldname + "    (" + mFieldTypeList.at( j ) + ')' );
         act->setData( QVariant( fldname ) );
         if ( getField() == fldname )
         {
           act->setCheckable( true );
           act->setChecked( !useExpression() );
+          fieldActive = !useExpression();
         }
       }
     }
@@ -284,10 +315,48 @@ void QgsDataDefinedButton::aboutToShowMenu()
     mDefineMenu->addSeparator();
   }
 
+  mFieldsMenu->menuAction()->setCheckable( true );
+  mFieldsMenu->menuAction()->setChecked( fieldActive );
+
   QAction* exprTitleAct = mDefineMenu->addAction( tr( "Expression" ) );
   exprTitleAct->setFont( titlefont );
   exprTitleAct->setEnabled( false );
 
+  mVariablesMenu->clear();
+  bool variableActive = false;
+  if ( mExpressionContextCallback )
+  {
+    QgsExpressionContext context = mExpressionContextCallback( mExpressionContextCallbackContext );
+    QStringList variables = context.variableNames();
+    Q_FOREACH ( const QString& variable, variables )
+    {
+      if ( context.isReadOnly( variable ) ) //only want to show user-set variables
+        continue;
+      if ( variable.startsWith( '_' ) ) //no hidden variables
+        continue;
+
+      QAction* act = mVariablesMenu->addAction( variable );
+      act->setData( QVariant( variable ) );
+
+      if ( useExpression() && hasExp && getExpression() == '@' + variable )
+      {
+        act->setCheckable( true );
+        act->setChecked( true );
+        variableActive = true;
+      }
+    }
+  }
+
+  if ( mVariablesMenu->actions().isEmpty() )
+  {
+    QAction* act = mVariablesMenu->addAction( tr( "No variables set" ) );
+    act->setEnabled( false );
+  }
+
+  mDefineMenu->addAction( mActionVariables );
+  mVariablesMenu->menuAction()->setCheckable( true );
+  mVariablesMenu->menuAction()->setChecked( variableActive );
+
   if ( hasExp )
   {
     QString expString = getExpression();
@@ -309,7 +378,7 @@ void QgsDataDefinedButton::aboutToShowMenu()
       mActionExpression->setText( expString );
     }
     mDefineMenu->addAction( mActionExpression );
-    mActionExpression->setChecked( useExpression() );
+    mActionExpression->setChecked( useExpression() && !variableActive );
 
     mDefineMenu->addAction( mActionExpDialog );
     mDefineMenu->addAction( mActionCopyExpr );
@@ -322,6 +391,11 @@ void QgsDataDefinedButton::aboutToShowMenu()
     mDefineMenu->addAction( mActionPasteExpr );
   }
 
+  if ( mAssistant.data() )
+  {
+    mDefineMenu->addSeparator();
+    mDefineMenu->addAction( mActionAssistant );
+  }
 }
 
 void QgsDataDefinedButton::menuActionTriggered( QAction* action )
@@ -368,9 +442,13 @@ void QgsDataDefinedButton::menuActionTriggered( QAction* action )
       setUseExpression( false );
       setActive( false );
     }
-    setExpression( QString( "" ) );
+    setExpression( QString() );
     updateGui();
   }
+  else if ( action == mActionAssistant )
+  {
+    showAssistant();
+  }
   else if ( mFieldsMenu->actions().contains( action ) )  // a field name clicked
   {
     if ( action->isEnabled() )
@@ -384,6 +462,16 @@ void QgsDataDefinedButton::menuActionTriggered( QAction* action )
       updateGui();
     }
   }
+  else if ( mVariablesMenu->actions().contains( action ) )  // a variable name clicked
+  {
+    if ( getExpression() != action->text().prepend( "@" ) )
+    {
+      setExpression( action->data().toString().prepend( "@" ) );
+    }
+    setUseExpression( true );
+    setActive( true );
+    updateGui();
+  }
 }
 
 void QgsDataDefinedButton::showDescriptionDialog()
@@ -394,9 +482,30 @@ void QgsDataDefinedButton::showDescriptionDialog()
   mv->exec();
 }
 
+void QgsDataDefinedButton::showAssistant()
+{
+  if ( !mAssistant.data() )
+    return;
+
+  if ( mAssistant->exec() == QDialog::Accepted )
+  {
+    QgsDataDefined dd = mAssistant->dataDefined();
+    setUseExpression( dd.useExpression() );
+    setActive( dd.isActive() );
+    if ( dd.isActive() && dd.useExpression() )
+      setExpression( dd.expressionString() );
+    else if ( dd.isActive() )
+      setField( dd.field() );
+    updateGui();
+  }
+  activateWindow(); // reset focus to parent window
+}
+
 void QgsDataDefinedButton::showExpressionDialog()
 {
-  QgsExpressionBuilderDialog d( const_cast<QgsVectorLayer*>( mVectorLayer ), getExpression() );
+  QgsExpressionContext context = mExpressionContextCallback ? mExpressionContextCallback( mExpressionContextCallbackContext ) : QgsExpressionContext();
+
+  QgsExpressionBuilderDialog d( const_cast<QgsVectorLayer*>( mVectorLayer ), getExpression(), this, "generic", context );
   if ( d.exec() == QDialog::Accepted )
   {
     QString newExp = d.expressionText();
@@ -499,7 +608,7 @@ void QgsDataDefinedButton::updateGui()
     deftip.append( "..." );
   }
 
-  mFullDescription += tr( "<b>Current definition %1:</b><br>%2" ).arg( deftype ).arg( deftip );
+  mFullDescription += tr( "<b>Current definition %1:</b><br>%2" ).arg( deftype, deftip );
 
   setToolTip( mFullDescription );
 
@@ -514,7 +623,7 @@ void QgsDataDefinedButton::setActive( bool active )
   }
 }
 
-void QgsDataDefinedButton::registerEnabledWidgets( QList<QWidget*> wdgts )
+void QgsDataDefinedButton::registerEnabledWidgets( const QList<QWidget*>& wdgts )
 {
   for ( int i = 0; i < wdgts.size(); ++i )
   {
@@ -534,6 +643,7 @@ void QgsDataDefinedButton::registerEnabledWidget( QWidget* wdgt )
 QList<QWidget*> QgsDataDefinedButton::registeredEnabledWidgets()
 {
   QList<QWidget*> wdgtList;
+  wdgtList.reserve( mEnabledWidgets.size() );
   for ( int i = 0; i < mEnabledWidgets.size(); ++i )
   {
     wdgtList << mEnabledWidgets.at( i );
@@ -549,7 +659,7 @@ void QgsDataDefinedButton::disableEnabledWidgets( bool disable )
   }
 }
 
-void QgsDataDefinedButton::registerCheckedWidgets( QList<QWidget*> wdgts )
+void QgsDataDefinedButton::registerCheckedWidgets( const QList<QWidget*>& wdgts )
 {
   for ( int i = 0; i < wdgts.size(); ++i )
   {
@@ -569,6 +679,7 @@ void QgsDataDefinedButton::registerCheckedWidget( QWidget* wdgt )
 QList<QWidget*> QgsDataDefinedButton::registeredCheckedWidgets()
 {
   QList<QWidget*> wdgtList;
+  wdgtList.reserve( mCheckedWidgets.size() );
   for ( int i = 0; i < mCheckedWidgets.size(); ++i )
   {
     wdgtList << mCheckedWidgets.at( i );
@@ -576,6 +687,24 @@ QList<QWidget*> QgsDataDefinedButton::registeredCheckedWidgets()
   return wdgtList;
 }
 
+void QgsDataDefinedButton::registerGetExpressionContextCallback( QgsDataDefinedButton::ExpressionContextCallback fnGetExpressionContext, const void *context )
+{
+  mExpressionContextCallback = fnGetExpressionContext;
+  mExpressionContextCallbackContext = context;
+}
+
+void QgsDataDefinedButton::setAssistant( const QString& title, QgsDataDefinedAssistant *assistant )
+{
+  mActionAssistant->setText( title.isEmpty() ? tr( "Assistant..." ) : title );
+  mAssistant.reset( assistant );
+  mAssistant.data()->setParent( this, Qt::Dialog );
+}
+
+QgsDataDefinedAssistant *QgsDataDefinedButton::assistant()
+{
+  return mAssistant.data();
+}
+
 void QgsDataDefinedButton::checkCheckedWidgets( bool check )
 {
   // don't uncheck, only set to checked
@@ -605,6 +734,11 @@ QString QgsDataDefinedButton::trString()
   return tr( "string " );
 }
 
+QString QgsDataDefinedButton::charDesc()
+{
+  return tr( "single character" );
+}
+
 QString QgsDataDefinedButton::boolDesc()
 {
   return tr( "bool [<b>1</b>=True|<b>0</b>=False]" );
@@ -697,16 +831,16 @@ QString QgsDataDefinedButton::penJoinStyleDesc()
 
 QString QgsDataDefinedButton::blendModesDesc()
 {
-  return trString() + QString( "[<b>Normal</b>|<b>Lighten</b>|<b>Screen</b>|<b>Dodge</b>|<br>"
-                               "<b>Addition</b>|<b>Darken</b>|<b>Multiply</b>|<b>Burn</b>|<b>Overlay</b>|<br>"
-                               "<b>SoftLight</b>|<b>HardLight</b>|<b>Difference</b>|<b>Subtract</b>]" );
+  return trString() + QLatin1String( "[<b>Normal</b>|<b>Lighten</b>|<b>Screen</b>|<b>Dodge</b>|<br>"
+                                     "<b>Addition</b>|<b>Darken</b>|<b>Multiply</b>|<b>Burn</b>|<b>Overlay</b>|<br>"
+                                     "<b>SoftLight</b>|<b>HardLight</b>|<b>Difference</b>|<b>Subtract</b>]" );
 }
 
 QString QgsDataDefinedButton::svgPathDesc()
 {
-  return trString() + QString( "[<b>filepath</b>] as<br>"
-                               "<b>''</b>=empty|absolute|search-paths-relative|<br>"
-                               "project-relative|URL" );
+  return trString() + QLatin1String( "[<b>filepath</b>] as<br>"
+                                     "<b>''</b>=empty|absolute|search-paths-relative|<br>"
+                                     "project-relative|URL" );
 }
 
 QString QgsDataDefinedButton::filePathDesc()
@@ -716,63 +850,63 @@ QString QgsDataDefinedButton::filePathDesc()
 
 QString QgsDataDefinedButton::paperSizeDesc()
 {
-  return trString() + QString( "[<b>A5</b>|<b>A4</b>|<b>A3</b>|<b>A2</b>|<b>A1</b>|<b>A0</b>"
-                               "<b>B5</b>|<b>B4</b>|<b>B3</b>|<b>B2</b>|<b>B1</b>|<b>B0</b>"
-                               "<b>Legal</b>|<b>Ansi A</b>|<b>Ansi B</b>|<b>Ansi C</b>|<b>Ansi D</b>|<b>Ansi E</b>"
-                               "<b>Arch A</b>|<b>Arch B</b>|<b>Arch C</b>|<b>Arch D</b>|<b>Arch E</b>|<b>Arch E1</b>]"
-                             );
+  return trString() + QLatin1String( "[<b>A5</b>|<b>A4</b>|<b>A3</b>|<b>A2</b>|<b>A1</b>|<b>A0</b>"
+                                     "<b>B5</b>|<b>B4</b>|<b>B3</b>|<b>B2</b>|<b>B1</b>|<b>B0</b>"
+                                     "<b>Legal</b>|<b>Ansi A</b>|<b>Ansi B</b>|<b>Ansi C</b>|<b>Ansi D</b>|<b>Ansi E</b>"
+                                     "<b>Arch A</b>|<b>Arch B</b>|<b>Arch C</b>|<b>Arch D</b>|<b>Arch E</b>|<b>Arch E1</b>]"
+                                   );
 }
 
 QString QgsDataDefinedButton::paperOrientationDesc()
 {
-  return trString() + QString( "[<b>portrait</b>|<b>landscape</b>]" );
+  return trString() + QLatin1String( "[<b>portrait</b>|<b>landscape</b>]" );
 }
 
 QString QgsDataDefinedButton::horizontalAnchorDesc()
 {
-  return trString() + QString( "[<b>left</b>|<b>center</b>|<b>right</b>]" );
+  return trString() + QLatin1String( "[<b>left</b>|<b>center</b>|<b>right</b>]" );
 }
 
 QString QgsDataDefinedButton::verticalAnchorDesc()
 {
-  return trString() + QString( "[<b>top</b>|<b>center</b>|<b>bottom</b>]" );
+  return trString() + QLatin1String( "[<b>top</b>|<b>center</b>|<b>bottom</b>]" );
 }
 
 QString QgsDataDefinedButton::gradientTypeDesc()
 {
-  return trString() + QString( "[<b>linear</b>|<b>radial</b>|<b>conical</b>]" );
+  return trString() + QLatin1String( "[<b>linear</b>|<b>radial</b>|<b>conical</b>]" );
 }
 
 QString QgsDataDefinedButton::gradientCoordModeDesc()
 {
-  return trString() + QString( "[<b>feature</b>|<b>viewport</b>]" );
+  return trString() + QLatin1String( "[<b>feature</b>|<b>viewport</b>]" );
 }
 
 QString QgsDataDefinedButton::gradientSpreadDesc()
 {
-  return trString() + QString( "[<b>pad</b>|<b>repeat</b>|<b>reflect</b>]" );
+  return trString() + QLatin1String( "[<b>pad</b>|<b>repeat</b>|<b>reflect</b>]" );
 }
 
 QString QgsDataDefinedButton::lineStyleDesc()
 {
-  return trString() + QString( "[<b>no</b>|<b>solid</b>|<b>dash</b>|<b>dot</b>|<b>dash dot</b>|<b>dash dot dot</b>]" );
+  return trString() + QLatin1String( "[<b>no</b>|<b>solid</b>|<b>dash</b>|<b>dot</b>|<b>dash dot</b>|<b>dash dot dot</b>]" );
 }
 
 QString QgsDataDefinedButton::capStyleDesc()
 {
-  return trString() + QString( "[<b>square</b>|<b>flat</b>|<b>round</b>]" );
+  return trString() + QLatin1String( "[<b>square</b>|<b>flat</b>|<b>round</b>]" );
 }
 
 QString QgsDataDefinedButton::fillStyleDesc()
 {
-  return trString() + QString( "[<b>solid</b>|<b>horizontal</b>|<b>vertical</b>|<b>cross</b>|<b>b_diagonal</b>|<b>f_diagonal"
-                               "</b>|<b>diagonal_x</b>|<b>dense1</b>|<b>dense2</b>|<b>dense3</b>|<b>dense4</b>|<b>dense5"
-                               "</b>|<b>dense6</b>|<b>dense7</b>|<b>no]" );
+  return trString() + QLatin1String( "[<b>solid</b>|<b>horizontal</b>|<b>vertical</b>|<b>cross</b>|<b>b_diagonal</b>|<b>f_diagonal"
+                                     "</b>|<b>diagonal_x</b>|<b>dense1</b>|<b>dense2</b>|<b>dense3</b>|<b>dense4</b>|<b>dense5"
+                                     "</b>|<b>dense6</b>|<b>dense7</b>|<b>no]" );
 }
 
 QString QgsDataDefinedButton::markerStyleDesc()
 {
-  return trString() + QString( "[<b>circle</b>|<b>rectangle</b>|<b>cross</b>|<b>triangle</b>]" );
+  return trString() + QLatin1String( "[<b>circle</b>|<b>rectangle</b>|<b>cross</b>|<b>triangle</b>]" );
 }
 
 QString QgsDataDefinedButton::customDashDesc()
