@@ -16,7 +16,7 @@
 #include "qgsexpressionbuilderdialog.h"
 #include <QSettings>
 
-QgsExpressionBuilderDialog::QgsExpressionBuilderDialog( QgsVectorLayer* layer, QString startText, QWidget* parent, QString key )
+QgsExpressionBuilderDialog::QgsExpressionBuilderDialog( QgsVectorLayer* layer, const QString& startText, QWidget* parent, const QString& key, const QgsExpressionContext &context )
     : QDialog( parent ), mRecentKey( key )
 {
   setupUi( this );
@@ -24,6 +24,7 @@ QgsExpressionBuilderDialog::QgsExpressionBuilderDialog( QgsVectorLayer* layer, Q
   QPushButton* okButton = buttonBox->button( QDialogButtonBox::Ok );
   connect( builder, SIGNAL( expressionParsed( bool ) ), okButton, SLOT( setEnabled( bool ) ) );
 
+  builder->setExpressionContext( context );
   builder->setLayer( layer );
   builder->setExpressionText( startText );
   builder->loadFieldNames();
@@ -48,6 +49,16 @@ QString QgsExpressionBuilderDialog::expressionText()
   return builder->expressionText();
 }
 
+QgsExpressionContext QgsExpressionBuilderDialog::expressionContext() const
+{
+  return builder->expressionContext();
+}
+
+void QgsExpressionBuilderDialog::setExpressionContext( const QgsExpressionContext &context )
+{
+  builder->setExpressionContext( context );
+}
+
 void QgsExpressionBuilderDialog::done( int r )
 {
   QDialog::done( r );
