@@ -19,19 +19,26 @@
 #include "qgsmessageviewer.h"
 #include "qgsapplication.h"
 #include "qgspythonrunner.h"
+#include "qgsgeometry.h"
+#include "qgsfeature.h"
 
 #include <QSettings>
 #include <QMenu>
 #include <QFile>
 #include <QTextStream>
-#include <QSettings>
 #include <QDir>
+#include <QInputDialog>
 #include <QComboBox>
+#include <QGraphicsOpacityEffect>
+#include <QPropertyAnimation>
+
+
 
 QgsExpressionBuilderWidget::QgsExpressionBuilderWidget( QWidget *parent )
     : QWidget( parent )
-    , mLayer( NULL )
-    , highlighter( NULL )
+    , mAutoSave( true )
+    , mLayer( nullptr )
+    , highlighter( nullptr )
     , mExpressionValid( false )
 {
   setupUi( this );
@@ -42,8 +49,11 @@ QgsExpressionBuilderWidget::QgsExpressionBuilderWidget( QWidget *parent )
 
   mModel = new QStandardItemModel();
   mProxyModel = new QgsExpressionItemSearchProxy();
+  mProxyModel->setDynamicSortFilter( true );
   mProxyModel->setSourceModel( mModel );
   expressionTree->setModel( mProxyModel );
+  expressionTree->setSortingEnabled( true );
+  expressionTree->sortByColumn( 0, Qt::AscendingOrder );
 
   expressionTree->setContextMenuPolicy( Qt::CustomContextMenu );
   connect( this, SIGNAL( expressionParsed( bool ) ), this, SLOT( setExpressionState( bool ) ) );
@@ -54,15 +64,22 @@ QgsExpressionBuilderWidget::QgsExpressionBuilderWidget( QWidget *parent )
   connect( btnLoadAll, SIGNAL( pressed() ), this, SLOT( loadAllValues() ) );
   connect( btnLoadSample, SIGNAL( pressed() ), this, SLOT( loadSampleValues() ) );
 
-  foreach ( QPushButton* button, mOperatorsGroupBox->findChildren<QPushButton *>() )
+  Q_FOREACH ( QPushButton* button, mOperatorsGroupBox->findChildren<QPushButton *>() )
   {
     connect( button, SIGNAL( pressed() ), this, SLOT( operatorButtonClicked() ) );
   }
 
   txtSearchEdit->setPlaceholderText( tr( "Search" ) );
 
+  mValuesModel = new QStringListModel();
+  mProxyValues = new QSortFilterProxyModel();
+  mProxyValues->setSourceModel( mValuesModel );
+  mValuesListView->setModel( mProxyValues );
+  txtSearchEditValues->setPlaceholderText( tr( "Search" ) );
+
   QSettings settings;
   splitter->restoreState( settings.value( "/windows/QgsExpressionBuilderWidget/splitter" ).toByteArray() );
+  editorSplit->restoreState( settings.value( "/windows/QgsExpressionBuilderWidget/editorsplitter" ).toByteArray() );
   functionsplit->restoreState( settings.value( "/windows/QgsExpressionBuilderWidget/functionsplitter" ).toByteArray() );
 
   txtExpressionString->setFoldingVisible( false );
@@ -72,20 +89,19 @@ QgsExpressionBuilderWidget::QgsExpressionBuilderWidget( QWidget *parent )
   if ( QgsPythonRunner::isValid() )
   {
     QgsPythonRunner::eval( "qgis.user.expressionspath", mFunctionsPath );
-    newFunctionFile();
-    // The scratch file gets written each time the widget opens.
-    saveFunctionFile( "scratch" );
     updateFunctionFileList( mFunctionsPath );
   }
   else
   {
-    tab_2->setEnabled( false );
+    tab_2->hide();
   }
 
-  // select the first item in the function list 
+  // select the first item in the function list
   // in order to avoid a blank help widget
   QModelIndex firstItem = mProxyModel->index( 0, 0, QModelIndex() );
   expressionTree->setCurrentIndex( firstItem );
+
+  lblAutoSave->setText( "" );
 }
 
 
@@ -93,43 +109,60 @@ QgsExpressionBuilderWidget::~QgsExpressionBuilderWidget()
 {
   QSettings settings;
   settings.setValue( "/windows/QgsExpressionBuilderWidget/splitter", splitter->saveState() );
+  settings.setValue( "/windows/QgsExpressionBuilderWidget/editorsplitter", editorSplit->saveState() );
   settings.setValue( "/windows/QgsExpressionBuilderWidget/functionsplitter", functionsplit->saveState() );
+
+  delete mModel;
+  delete mProxyModel;
+  delete mValuesModel;
+  delete mProxyValues;
 }
 
 void QgsExpressionBuilderWidget::setLayer( QgsVectorLayer *layer )
 {
   mLayer = layer;
+
+  //TODO - remove existing layer scope from context
+
+  if ( mLayer )
+    mExpressionContext << QgsExpressionContextUtils::layerScope( mLayer );
 }
 
 void QgsExpressionBuilderWidget::currentChanged( const QModelIndex &index, const QModelIndex & )
 {
+  txtSearchEditValues->setText( QString( "" ) );
+
   // Get the item
   QModelIndex idx = mProxyModel->mapToSource( index );
   QgsExpressionItem* item = dynamic_cast<QgsExpressionItem*>( mModel->itemFromIndex( idx ) );
   if ( !item )
     return;
 
-  if ( item->getItemType() != QgsExpressionItem::Field )
+  if ( item->getItemType() == QgsExpressionItem::Field && mFieldValues.contains( item->text() ) )
   {
-    mValueListWidget->clear();
+    const QStringList& values = mFieldValues[item->text()];
+    mValuesModel->setStringList( values );
   }
 
   mLoadGroupBox->setVisible( item->getItemType() == QgsExpressionItem::Field && mLayer );
-  mValueGroupBox->setVisible( item->getItemType() == QgsExpressionItem::Field && mLayer );
+  mValueGroupBox->setVisible(( item->getItemType() == QgsExpressionItem::Field && mLayer ) || mValuesListView->model()->rowCount() > 0 );
 
   // Show the help for the current item.
   QString help = loadFunctionHelp( item );
   txtHelpText->setText( help );
-  txtHelpText->setToolTip( txtHelpText->toPlainText() );
 }
 
 void QgsExpressionBuilderWidget::on_btnRun_pressed()
 {
-  saveFunctionFile( cmbFileNames->currentText() );
+  if ( !cmbFileNames->currentItem() )
+    return;
+
+  QString file = cmbFileNames->currentItem()->text();
+  saveFunctionFile( file );
   runPythonCode( txtPython->text() );
 }
 
-void QgsExpressionBuilderWidget::runPythonCode( QString code )
+void QgsExpressionBuilderWidget::runPythonCode( const QString& code )
 {
   if ( QgsPythonRunner::isValid() )
   {
@@ -156,7 +189,7 @@ void QgsExpressionBuilderWidget::saveFunctionFile( QString fileName )
 
   fileName = mFunctionsPath + QDir::separator() + fileName;
   QFile myFile( fileName );
-  if ( myFile.open( QIODevice::WriteOnly ) )
+  if ( myFile.open( QIODevice::WriteOnly | QFile::Truncate ) )
   {
     QTextStream myFileStream( &myFile );
     myFileStream << txtPython->text() << endl;
@@ -164,44 +197,58 @@ void QgsExpressionBuilderWidget::saveFunctionFile( QString fileName )
   }
 }
 
-void QgsExpressionBuilderWidget::updateFunctionFileList( QString path )
+void QgsExpressionBuilderWidget::updateFunctionFileList( const QString& path )
 {
   mFunctionsPath = path;
   QDir dir( path );
   dir.setNameFilters( QStringList() << "*.py" );
   QStringList files = dir.entryList( QDir::Files );
   cmbFileNames->clear();
-  foreach ( QString name, files )
+  Q_FOREACH ( const QString& name, files )
   {
     QFileInfo info( mFunctionsPath + QDir::separator() + name );
     if ( info.baseName() == "__init__" ) continue;
-    cmbFileNames->addItem( info.baseName() );
+    QListWidgetItem* item = new QListWidgetItem( QgsApplication::getThemeIcon( "console/iconTabEditorConsole.png" ), info.baseName() );
+    cmbFileNames->addItem( item );
   }
+  if ( !cmbFileNames->currentItem() )
+    cmbFileNames->setCurrentRow( 0 );
 }
 
-void QgsExpressionBuilderWidget::newFunctionFile( QString fileName )
+void QgsExpressionBuilderWidget::newFunctionFile( const QString& fileName )
 {
+  QList<QListWidgetItem*> items = cmbFileNames->findItems( fileName, Qt::MatchExactly );
+  if ( !items.isEmpty() )
+    return;
+
   QString templatetxt;
   QgsPythonRunner::eval( "qgis.user.expressions.template", templatetxt );
   txtPython->setText( templatetxt );
-  int index = cmbFileNames->findText( fileName );
-  if ( index == -1 )
-    cmbFileNames->setEditText( fileName );
-  else
-    cmbFileNames->setCurrentIndex( index );
+  cmbFileNames->insertItem( 0, fileName );
+  cmbFileNames->setCurrentRow( 0 );
+  saveFunctionFile( fileName );
 }
 
 void QgsExpressionBuilderWidget::on_btnNewFile_pressed()
 {
-  newFunctionFile();
+  bool ok;
+  QString text = QInputDialog::getText( this, tr( "Enter new file name" ),
+                                        tr( "File name:" ), QLineEdit::Normal,
+                                        "", &ok );
+  if ( ok && !text.isEmpty() )
+  {
+    newFunctionFile( text );
+  }
 }
 
-void QgsExpressionBuilderWidget::on_cmbFileNames_currentIndexChanged( int index )
+void QgsExpressionBuilderWidget::on_cmbFileNames_currentItemChanged( QListWidgetItem* item, QListWidgetItem* lastitem )
 {
-  if ( index == -1 )
-    return;
-
-  QString path = mFunctionsPath + QDir::separator() + cmbFileNames->currentText();
+  if ( lastitem )
+  {
+    QString filename = lastitem->text();
+    saveFunctionFile( filename );
+  }
+  QString path = mFunctionsPath + QDir::separator() + item->text();
   loadCodeFromFile( path );
 }
 
@@ -213,28 +260,16 @@ void QgsExpressionBuilderWidget::loadCodeFromFile( QString path )
   txtPython->loadScript( path );
 }
 
-void QgsExpressionBuilderWidget::loadFunctionCode( QString code )
+void QgsExpressionBuilderWidget::loadFunctionCode( const QString& code )
 {
   txtPython->setText( code );
 }
 
-void QgsExpressionBuilderWidget::on_btnSaveFile_pressed()
-{
-  QString name = cmbFileNames->currentText();
-  saveFunctionFile( name );
-  int index = cmbFileNames->findText( name );
-  if ( index == -1 )
-  {
-    cmbFileNames->addItem( name );
-    cmbFileNames->setCurrentIndex( cmbFileNames->count() - 1 );
-  }
-}
-
 void QgsExpressionBuilderWidget::on_expressionTree_doubleClicked( const QModelIndex &index )
 {
   QModelIndex idx = mProxyModel->mapToSource( index );
   QgsExpressionItem* item = dynamic_cast<QgsExpressionItem*>( mModel->itemFromIndex( idx ) );
-  if ( item == 0 )
+  if ( !item )
     return;
 
   // Don't handle the double click it we are on a header node.
@@ -253,7 +288,7 @@ void QgsExpressionBuilderWidget::loadFieldNames()
   if ( !mLayer )
     return;
 
-  loadFieldNames( mLayer->pendingFields() );
+  loadFieldNames( mLayer->fields() );
 }
 
 void QgsExpressionBuilderWidget::loadFieldNames( const QgsFields& fields )
@@ -262,17 +297,29 @@ void QgsExpressionBuilderWidget::loadFieldNames( const QgsFields& fields )
     return;
 
   QStringList fieldNames;
-  //foreach ( const QgsField& field, fields )
+  //Q_FOREACH ( const QgsField& field, fields )
+  fieldNames.reserve( fields.count() );
   for ( int i = 0; i < fields.count(); ++i )
   {
     QString fieldName = fields[i].name();
     fieldNames << fieldName;
-    registerItem( "Fields and Values", fieldName, " \"" + fieldName + "\" ", "", QgsExpressionItem::Field );
+    registerItem( "Fields and Values", fieldName, " \"" + fieldName + "\" ", "", QgsExpressionItem::Field, false, i );
   }
 //  highlighter->addFields( fieldNames );
 }
 
-void QgsExpressionBuilderWidget::fillFieldValues( int fieldIndex, int countLimit )
+void QgsExpressionBuilderWidget::loadFieldsAndValues( const QMap<QString, QStringList> &fieldValues )
+{
+  QgsFields fields;
+  Q_FOREACH ( const QString& fieldName, fieldValues.keys() )
+  {
+    fields.append( QgsField( fieldName ) );
+  }
+  loadFieldNames( fields );
+  mFieldValues = fieldValues;
+}
+
+void QgsExpressionBuilderWidget::fillFieldValues( const QString& fieldName, int countLimit )
 {
   // TODO We should really return a error the user of the widget that
   // the there is no layer set.
@@ -280,38 +327,40 @@ void QgsExpressionBuilderWidget::fillFieldValues( int fieldIndex, int countLimit
     return;
 
   // TODO We should thread this so that we don't hold the user up if the layer is massive.
-  mValueListWidget->clear();
+
+  int fieldIndex = mLayer->fieldNameIndex( fieldName );
 
   if ( fieldIndex < 0 )
     return;
 
-  mValueListWidget->setUpdatesEnabled( false );
-  mValueListWidget->blockSignals( true );
-
   QList<QVariant> values;
+  QStringList strValues;
   mLayer->uniqueValues( fieldIndex, values, countLimit );
-  foreach ( QVariant value, values )
+  Q_FOREACH ( const QVariant& value, values )
   {
+    QString strValue;
     if ( value.isNull() )
-      mValueListWidget->addItem( "NULL" );
+      strValue = "NULL";
     else if ( value.type() == QVariant::Int || value.type() == QVariant::Double || value.type() == QVariant::LongLong )
-      mValueListWidget->addItem( value.toString() );
+      strValue = value.toString();
     else
-      mValueListWidget->addItem( "'" + value.toString().replace( "'", "''" ) + "'" );
+      strValue = '\'' + value.toString().replace( '\'', "''" ) + '\'';
+    strValues.append( strValue );
   }
-
-  mValueListWidget->setUpdatesEnabled( true );
-  mValueListWidget->blockSignals( false );
+  mValuesModel->setStringList( strValues );
+  mFieldValues[fieldName] = strValues;
 }
 
-void QgsExpressionBuilderWidget::registerItem( QString group,
-    QString label,
-    QString expressionText,
-    QString helpText,
-    QgsExpressionItem::ItemType type )
+void QgsExpressionBuilderWidget::registerItem( const QString& group,
+    const QString& label,
+    const QString& expressionText,
+    const QString& helpText,
+    QgsExpressionItem::ItemType type, bool highlightedItem, int sortOrder )
 {
   QgsExpressionItem* item = new QgsExpressionItem( label, expressionText, helpText, type );
   item->setData( label, Qt::UserRole );
+  item->setData( sortOrder, QgsExpressionItem::CustomSortRole );
+
   // Look up the group and insert the new function.
   if ( mExpressionGroups.contains( group ) )
   {
@@ -323,10 +372,26 @@ void QgsExpressionBuilderWidget::registerItem( QString group,
     // If the group doesn't exist yet we make it first.
     QgsExpressionItem *newgroupNode = new QgsExpressionItem( QgsExpression::group( group ), "", QgsExpressionItem::Header );
     newgroupNode->setData( group, Qt::UserRole );
+    //Recent group should always be last group
+    newgroupNode->setData( group.startsWith( "Recent (" ) ? 2 : 1, QgsExpressionItem::CustomSortRole );
     newgroupNode->appendRow( item );
+    newgroupNode->setBackground( QBrush( QColor( "#eee" ) ) );
     mModel->appendRow( newgroupNode );
     mExpressionGroups.insert( group, newgroupNode );
   }
+
+  if ( highlightedItem )
+  {
+    //insert a copy as a top level item
+    QgsExpressionItem* topLevelItem = new QgsExpressionItem( label, expressionText, helpText, type );
+    topLevelItem->setData( label, Qt::UserRole );
+    item->setData( 0, QgsExpressionItem::CustomSortRole );
+    QFont font = topLevelItem->font();
+    font.setBold( true );
+    topLevelItem->setFont( font );
+    mModel->appendRow( topLevelItem );
+  }
+
 }
 
 bool QgsExpressionBuilderWidget::isExpressionValid()
@@ -334,7 +399,7 @@ bool QgsExpressionBuilderWidget::isExpressionValid()
   return mExpressionValid;
 }
 
-void QgsExpressionBuilderWidget::saveToRecent( QString key )
+void QgsExpressionBuilderWidget::saveToRecent( const QString& key )
 {
   QSettings settings;
   QString location = QString( "/expressions/recent/%1" ).arg( key );
@@ -352,7 +417,7 @@ void QgsExpressionBuilderWidget::saveToRecent( QString key )
   this->loadRecent( key );
 }
 
-void QgsExpressionBuilderWidget::loadRecent( QString key )
+void QgsExpressionBuilderWidget::loadRecent( const QString& key )
 {
   mRecentKey = key;
   QString name = tr( "Recent (%1)" ).arg( key );
@@ -365,9 +430,11 @@ void QgsExpressionBuilderWidget::loadRecent( QString key )
   QSettings settings;
   QString location = QString( "/expressions/recent/%1" ).arg( key );
   QStringList expressions = settings.value( location ).toStringList();
-  foreach ( QString expression, expressions )
+  int i = 0;
+  Q_FOREACH ( const QString& expression, expressions )
   {
-    this->registerItem( name, expression, expression, expression );
+    this->registerItem( name, expression, expression, expression, QgsExpressionItem::ExpressionNode, false, i );
+    i++;
   }
 }
 
@@ -376,24 +443,19 @@ void QgsExpressionBuilderWidget::updateFunctionTree()
   mModel->clear();
   mExpressionGroups.clear();
   // TODO Can we move this stuff to QgsExpression, like the functions?
-  registerItem( "Operators", "+", " + ", tr( "Addition operator" ) );
-  registerItem( "Operators", "-", " - ", tr( "Subtraction operator" ) );
-  registerItem( "Operators", "*", " * ", tr( "Multiplication operator" ) );
-  registerItem( "Operators", "/", " / ", tr( "Division operator" ) );
-  registerItem( "Operators", "%", " % ", tr( "Modulo operator" ) );
-  registerItem( "Operators", "^", " ^ ", tr( "Power operator" ) );
-  registerItem( "Operators", "=", " = ", tr( "Equal operator" ) );
-  registerItem( "Operators", ">", " > ", tr( "Greater as operator" ) );
-  registerItem( "Operators", "<", " < ", tr( "Less than operator" ) );
-  registerItem( "Operators", "<>", " <> ", tr( "Unequal operator" ) );
-  registerItem( "Operators", "<=", " <= ", tr( "Less or equal operator" ) );
-  registerItem( "Operators", ">=", " >= ", tr( "Greater or equal operator" ) );
-  registerItem( "Operators", "||", " || ",
-                QString( "<b>|| %1</b><br><i>%2</i><br><i>%3:</i>%4" )
-                .arg( tr( "(String Concatenation)" ) )
-                .arg( tr( "Joins two values together into a string" ) )
-                .arg( tr( "Usage" ) )
-                .arg( tr( "'Dia' || Diameter" ) ) );
+  registerItem( "Operators", "+", " + " );
+  registerItem( "Operators", "-", " - " );
+  registerItem( "Operators", "*", " * " );
+  registerItem( "Operators", "/", " / " );
+  registerItem( "Operators", "%", " % " );
+  registerItem( "Operators", "^", " ^ " );
+  registerItem( "Operators", "=", " = " );
+  registerItem( "Operators", ">", " > " );
+  registerItem( "Operators", "<", " < " );
+  registerItem( "Operators", "<>", " <> " );
+  registerItem( "Operators", "<=", " <= " );
+  registerItem( "Operators", ">=", " >= " );
+  registerItem( "Operators", "||", " || " );
   registerItem( "Operators", "IN", " IN " );
   registerItem( "Operators", "LIKE", " LIKE " );
   registerItem( "Operators", "ILIKE", " ILIKE " );
@@ -403,9 +465,7 @@ void QgsExpressionBuilderWidget::updateFunctionTree()
   registerItem( "Operators", "NOT", " NOT " );
 
   QString casestring = "CASE WHEN condition THEN result END";
-  QString caseelsestring = "CASE WHEN condition THEN result ELSE result END";
   registerItem( "Conditionals", "CASE", casestring );
-  registerItem( "Conditionals", "CASE ELSE", caseelsestring );
 
   registerItem( "Fields and Values", "NULL", "NULL" );
 
@@ -415,19 +475,31 @@ void QgsExpressionBuilderWidget::updateFunctionTree()
   {
     QgsExpression::Function* func = QgsExpression::Functions()[i];
     QString name = func->name();
-    if ( name.startsWith( "_" ) ) // do not display private functions
+    if ( name.startsWith( '_' ) ) // do not display private functions
       continue;
+    if ( func->group() == "deprecated" ) // don't show deprecated functions
+      continue;
+    if ( func->isContextual() )
+    {
+      //don't show contextual functions by default - it's up the the QgsExpressionContext
+      //object to provide them if supported
+      continue;
+    }
     if ( func->params() != 0 )
-      name += "(";
-    registerItem( func->group(), func->name(), " " + name + " ", func->helptext() );
+      name += '(';
+    else if ( !name.startsWith( '$' ) )
+      name += "()";
+    registerItem( func->group(), func->name(), ' ' + name + ' ', func->helptext() );
   }
 
   QList<QgsExpression::Function*> specials = QgsExpression::specialColumns();
   for ( int i = 0; i < specials.size(); ++i )
   {
     QString name = specials[i]->name();
-    registerItem( specials[i]->group(), name, " " + name + " " );
+    registerItem( specials[i]->group(), name, ' ' + name + ' ' );
   }
+
+  loadExpressionContext();
 }
 
 void QgsExpressionBuilderWidget::setGeomCalculator( const QgsDistanceArea & da )
@@ -445,6 +517,13 @@ void QgsExpressionBuilderWidget::setExpressionText( const QString& expression )
   txtExpressionString->setText( expression );
 }
 
+void QgsExpressionBuilderWidget::setExpressionContext( const QgsExpressionContext &context )
+{
+  mExpressionContext = context;
+
+  loadExpressionContext();
+}
+
 void QgsExpressionBuilderWidget::on_txtExpressionString_textChanged()
 {
   QString text = expressionText();
@@ -475,9 +554,10 @@ void QgsExpressionBuilderWidget::on_txtExpressionString_textChanged()
 
     if ( mFeature.isValid() )
     {
-      QVariant value = exp.evaluate( &mFeature, mLayer->pendingFields() );
+      mExpressionContext.setFeature( mFeature );
+      QVariant value = exp.evaluate( &mExpressionContext );
       if ( !exp.hasEvalError() )
-        lblPreview->setText( formatPreviewString( value.toString() ) );
+        lblPreview->setText( QgsExpression::formatPreviewString( value ) );
     }
     else
     {
@@ -489,18 +569,18 @@ void QgsExpressionBuilderWidget::on_txtExpressionString_textChanged()
   else
   {
     // No layer defined
-    QVariant value = exp.evaluate();
+    QVariant value = exp.evaluate( &mExpressionContext );
     if ( !exp.hasEvalError() )
     {
-      lblPreview->setText( formatPreviewString( value.toString() ) );
+      lblPreview->setText( QgsExpression::formatPreviewString( value ) );
     }
   }
 
   if ( exp.hasParserError() || exp.hasEvalError() )
   {
-    QString tooltip = QString( "<b>%1:</b><br>%2" ).arg( tr( "Parser Error" ) ).arg( exp.parserErrorString() );
+    QString tooltip = QString( "<b>%1:</b><br>%2" ).arg( tr( "Parser Error" ), exp.parserErrorString() );
     if ( exp.hasEvalError() )
-      tooltip += QString( "<br><br><b>%1:</b><br>%2" ).arg( tr( "Eval Error" ) ).arg( exp.evalErrorString() );
+      tooltip += QString( "<br><br><b>%1:</b><br>%2" ).arg( tr( "Eval Error" ), exp.evalErrorString() );
 
     lblPreview->setText( tr( "Expression is invalid <a href=""more"">(more info)</a>" ) );
     lblPreview->setStyleSheet( "color: rgba(255, 6, 10,  255);" );
@@ -518,28 +598,63 @@ void QgsExpressionBuilderWidget::on_txtExpressionString_textChanged()
   }
 }
 
-QString QgsExpressionBuilderWidget::formatPreviewString( const QString& previewString ) const
+void QgsExpressionBuilderWidget::loadExpressionContext()
 {
-  if ( previewString.length() > 63 )
+  QStringList variableNames = mExpressionContext.filteredVariableNames();
+  Q_FOREACH ( const QString& variable, variableNames )
   {
-    return QString( tr( "%1..." ) ).arg( previewString.left( 60 ) );
+    registerItem( "Variables", variable, " @" + variable + ' ',
+                  QgsExpression::variableHelpText( variable, true, mExpressionContext.variable( variable ) ),
+                  QgsExpressionItem::ExpressionNode,
+                  mExpressionContext.isHighlightedVariable( variable ) );
   }
-  else
+
+  // Load the functions from the expression context
+  QStringList contextFunctions = mExpressionContext.functionNames();
+  Q_FOREACH ( const QString& functionName, contextFunctions )
   {
-    return previewString;
+    QgsExpression::Function* func = mExpressionContext.function( functionName );
+    QString name = func->name();
+    if ( name.startsWith( '_' ) ) // do not display private functions
+      continue;
+    if ( func->params() != 0 )
+      name += '(';
+    registerItem( func->group(), func->name(), ' ' + name + ' ', func->helptext() );
   }
 }
 
+void QgsExpressionBuilderWidget::showEvent( QShowEvent *e )
+{
+  QWidget::showEvent( e );
+  txtExpressionString->setFocus();
+}
+
 void QgsExpressionBuilderWidget::on_txtSearchEdit_textChanged()
 {
   mProxyModel->setFilterWildcard( txtSearchEdit->text() );
   if ( txtSearchEdit->text().isEmpty() )
+  {
     expressionTree->collapseAll();
+  }
   else
+  {
     expressionTree->expandAll();
+    QModelIndex index = mProxyModel->index( 0, 0 );
+    if ( mProxyModel->hasChildren( index ) )
+    {
+      QModelIndex child = mProxyModel->index( 0, 0, index );
+      expressionTree->selectionModel()->setCurrentIndex( child, QItemSelectionModel::ClearAndSelect );
+    }
+  }
 }
 
-void QgsExpressionBuilderWidget::on_lblPreview_linkActivated( QString link )
+void QgsExpressionBuilderWidget::on_txtSearchEditValues_textChanged()
+{
+  mProxyValues->setFilterCaseSensitivity( Qt::CaseInsensitive );
+  mProxyValues->setFilterWildcard( txtSearchEditValues->text() );
+}
+
+void QgsExpressionBuilderWidget::on_lblPreview_linkActivated( const QString& link )
 {
   Q_UNUSED( link );
   QgsMessageViewer * mv = new QgsMessageViewer( this );
@@ -548,10 +663,10 @@ void QgsExpressionBuilderWidget::on_lblPreview_linkActivated( QString link )
   mv->exec();
 }
 
-void QgsExpressionBuilderWidget::on_mValueListWidget_itemDoubleClicked( QListWidgetItem *item )
+void QgsExpressionBuilderWidget::on_mValuesListView_doubleClicked( const QModelIndex &index )
 {
   // Insert the item text or replace selected text
-  txtExpressionString->insertText( " " + item->text() + " " );
+  txtExpressionString->insertText( ' ' + index.data( Qt::DisplayRole ).toString() + ' ' );
   txtExpressionString->setFocus();
 }
 
@@ -560,11 +675,11 @@ void QgsExpressionBuilderWidget::operatorButtonClicked()
   QPushButton* button = dynamic_cast<QPushButton*>( sender() );
 
   // Insert the button text or replace selected text
-  txtExpressionString->insertText( " " + button->text() + " " );
+  txtExpressionString->insertText( ' ' + button->text() + ' ' );
   txtExpressionString->setFocus();
 }
 
-void QgsExpressionBuilderWidget::showContextMenu( const QPoint & pt )
+void QgsExpressionBuilderWidget::showContextMenu( QPoint pt )
 {
   QModelIndex idx = expressionTree->indexAt( pt );
   idx = mProxyModel->mapToSource( idx );
@@ -591,9 +706,7 @@ void QgsExpressionBuilderWidget::loadSampleValues()
     return;
 
   mValueGroupBox->show();
-  int fieldIndex = mLayer->fieldNameIndex( item->text() );
-
-  fillFieldValues( fieldIndex, 10 );
+  fillFieldValues( item->text(), 10 );
 }
 
 void QgsExpressionBuilderWidget::loadAllValues()
@@ -606,8 +719,39 @@ void QgsExpressionBuilderWidget::loadAllValues()
     return;
 
   mValueGroupBox->show();
-  int fieldIndex = mLayer->fieldNameIndex( item->text() );
-  fillFieldValues( fieldIndex, -1 );
+  fillFieldValues( item->text(), -1 );
+}
+
+void QgsExpressionBuilderWidget::on_txtPython_textChanged()
+{
+  lblAutoSave->setText( "Saving..." );
+  if ( mAutoSave )
+  {
+    autosave();
+  }
+}
+
+void QgsExpressionBuilderWidget::autosave()
+{
+  // Don't auto save if not on function editor that would be silly.
+  if ( tabWidget->currentIndex() != 1 )
+    return;
+
+  QListWidgetItem* item = cmbFileNames->currentItem();
+  if ( !item )
+    return;
+
+  QString file = item->text();
+  saveFunctionFile( file );
+  lblAutoSave->setText( "Saved" );
+  QGraphicsOpacityEffect *effect = new QGraphicsOpacityEffect();
+  lblAutoSave->setGraphicsEffect( effect );
+  QPropertyAnimation *anim = new QPropertyAnimation( effect, "opacity" );
+  anim->setDuration( 2000 );
+  anim->setStartValue( 1.0 );
+  anim->setEndValue( 0.0 );
+  anim->setEasingCurve( QEasingCurve::OutQuad );
+  anim->start( QAbstractAnimation::DeleteWhenStopped );
 }
 
 void QgsExpressionBuilderWidget::setExpressionState( bool state )
@@ -615,6 +759,19 @@ void QgsExpressionBuilderWidget::setExpressionState( bool state )
   mExpressionValid = state;
 }
 
+QString QgsExpressionBuilderWidget::helpStylesheet() const
+{
+  //start with default QGIS report style
+  QString style = QgsApplication::reportStyleSheet();
+
+  //add some tweaks
+  style += " .functionname {color: #0a6099; font-weight: bold;} "
+           " .argument {font-family: monospace; color: #bf0c0c; font-style: italic; } "
+           " td.argument { padding-right: 10px; }";
+
+  return style;
+}
+
 QString QgsExpressionBuilderWidget::loadFunctionHelp( QgsExpressionItem* expressionItem )
 {
   if ( !expressionItem )
@@ -633,6 +790,58 @@ QString QgsExpressionBuilderWidget::loadFunctionHelp( QgsExpressionItem* express
       helpContents = QgsExpression::helptext( name );
   }
 
-  QString myStyle = QgsApplication::reportStyleSheet();
-  return "<head><style>" + myStyle + "</style></head><body>" + helpContents + "</body>";
+  return "<head><style>" + helpStylesheet() + "</style></head><body>" + helpContents + "</body>";
+}
+
+
+
+
+
+QgsExpressionItemSearchProxy::QgsExpressionItemSearchProxy()
+{
+  setFilterCaseSensitivity( Qt::CaseInsensitive );
+}
+
+bool QgsExpressionItemSearchProxy::filterAcceptsRow( int source_row, const QModelIndex& source_parent ) const
+{
+  QModelIndex index = sourceModel()->index( source_row, 0, source_parent );
+  QgsExpressionItem::ItemType itemType = QgsExpressionItem::ItemType( sourceModel()->data( index, QgsExpressionItem::ItemTypeRole ).toInt() );
+
+  int count = sourceModel()->rowCount( index );
+  bool matchchild = false;
+  for ( int i = 0; i < count; ++i )
+  {
+    if ( filterAcceptsRow( i, index ) )
+    {
+      matchchild = true;
+      break;
+    }
+  }
+
+  if ( itemType == QgsExpressionItem::Header && matchchild )
+    return true;
+
+  if ( itemType == QgsExpressionItem::Header )
+    return false;
+
+  return QSortFilterProxyModel::filterAcceptsRow( source_row, source_parent );
+}
+
+bool QgsExpressionItemSearchProxy::lessThan( const QModelIndex& left, const QModelIndex& right ) const
+{
+  int leftSort = sourceModel()->data( left, QgsExpressionItem::CustomSortRole ).toInt();
+  int rightSort = sourceModel()->data( right,  QgsExpressionItem::CustomSortRole ).toInt();
+  if ( leftSort != rightSort )
+    return leftSort < rightSort;
+
+  QString leftString = sourceModel()->data( left, Qt::DisplayRole ).toString();
+  QString rightString = sourceModel()->data( right, Qt::DisplayRole ).toString();
+
+  //ignore $ prefixes when sorting
+  if ( leftString.startsWith( '$' ) )
+    leftString = leftString.mid( 1 );
+  if ( rightString.startsWith( '$' ) )
+    rightString = rightString.mid( 1 );
+
+  return QString::localeAwareCompare( leftString, rightString ) < 0;
 }
