@@ -3,7 +3,7 @@
      --------------------------------------
     Date                 : 24.1.2013
     Copyright            : (C) 2013 by Matthias kuhn
-    Email                : matthias dot kuhn at gmx dot ch
+    Email                : matthias at opengis dot ch
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,21 +19,23 @@
 
 #include <QSettings>
 
-QgsExpressionSelectionDialog::QgsExpressionSelectionDialog( QgsVectorLayer* layer, QString startText, QWidget* parent )
+QgsExpressionSelectionDialog::QgsExpressionSelectionDialog( QgsVectorLayer* layer, const QString& startText, QWidget* parent )
     : QDialog( parent )
     , mLayer( layer )
 {
   setupUi( this );
 
+  setWindowTitle( QString( "Select by expression - %1" ).arg( layer->name() ) );
+
   mActionSelect->setIcon( QgsApplication::getThemeIcon( "/mIconExpressionSelect.svg" ) );
   mActionAddToSelection->setIcon( QgsApplication::getThemeIcon( "/mIconSelectAdd.svg" ) );
   mActionRemoveFromSelection->setIcon( QgsApplication::getThemeIcon( "/mIconSelectRemove.svg" ) );
-  mActionSelectInstersect->setIcon( QgsApplication::getThemeIcon( "/mIconSelectIntersect.svg" ) );
+  mActionSelectIntersect->setIcon( QgsApplication::getThemeIcon( "/mIconSelectIntersect.svg" ) );
 
   mButtonSelect->addAction( mActionSelect );
   mButtonSelect->addAction( mActionAddToSelection );
   mButtonSelect->addAction( mActionRemoveFromSelection );
-  mButtonSelect->addAction( mActionSelectInstersect );
+  mButtonSelect->addAction( mActionSelectIntersect );
   mButtonSelect->setDefaultAction( mActionSelect );
 
   mExpressionBuilder->setLayer( layer );
@@ -41,6 +43,12 @@ QgsExpressionSelectionDialog::QgsExpressionSelectionDialog( QgsVectorLayer* laye
   mExpressionBuilder->loadFieldNames();
   mExpressionBuilder->loadRecent( "Selection" );
 
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( mLayer );
+  mExpressionBuilder->setExpressionContext( context );
+
   QSettings settings;
   restoreGeometry( settings.value( "/Windows/ExpressionSelectionDialog/geometry" ).toByteArray() );
 }
@@ -71,19 +79,18 @@ void QgsExpressionSelectionDialog::on_mActionSelect_triggered()
   QgsFeatureIds newSelection;
   QgsExpression* expression = new QgsExpression( mExpressionBuilder->expressionText() );
 
-  const QgsFields fields = mLayer->pendingFields();
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( mLayer );
 
-  QgsFeatureIterator features = mLayer->getFeatures();
-
-  expression->prepare( fields );
+  QgsFeatureRequest request = QgsFeatureRequest().setFilterExpression( mExpressionBuilder->expressionText() ).setExpressionContext( context );
+  QgsFeatureIterator features = mLayer->getFeatures( request );
 
   QgsFeature feat;
   while ( features.nextFeature( feat ) )
   {
-    if ( expression->evaluate( &feat, fields ).toBool() )
-    {
-      newSelection << feat.id();
-    }
+    newSelection << feat.id();
   }
 
   features.close();
@@ -99,19 +106,18 @@ void QgsExpressionSelectionDialog::on_mActionAddToSelection_triggered()
   QgsFeatureIds newSelection = mLayer->selectedFeaturesIds();
   QgsExpression* expression = new QgsExpression( mExpressionBuilder->expressionText() );
 
-  const QgsFields fields = mLayer->pendingFields();
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( mLayer );
 
-  QgsFeatureIterator features = mLayer->getFeatures();
-
-  expression->prepare( fields );
+  QgsFeatureRequest request = QgsFeatureRequest().setFilterExpression( mExpressionBuilder->expressionText() ).setExpressionContext( context );
+  QgsFeatureIterator features = mLayer->getFeatures( request );
 
   QgsFeature feat;
   while ( features.nextFeature( feat ) )
   {
-    if ( expression->evaluate( &feat, fields ).toBool() )
-    {
-      newSelection << feat.id();
-    }
+    newSelection << feat.id();
   }
 
   features.close();
@@ -122,25 +128,29 @@ void QgsExpressionSelectionDialog::on_mActionAddToSelection_triggered()
   saveRecent();
 }
 
-void QgsExpressionSelectionDialog::on_mActionSelectInstersect_triggered()
+void QgsExpressionSelectionDialog::on_mActionSelectIntersect_triggered()
 {
   const QgsFeatureIds &oldSelection = mLayer->selectedFeaturesIds();
   QgsFeatureIds newSelection;
 
   QgsExpression* expression = new QgsExpression( mExpressionBuilder->expressionText() );
 
-  const QgsFields fields = mLayer->pendingFields();
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( mLayer );
 
-  expression->prepare( fields );
+  expression->prepare( &context );
 
   QgsFeature feat;
-  foreach ( const QgsFeatureId fid, oldSelection )
+  Q_FOREACH ( const QgsFeatureId fid, oldSelection )
   {
     QgsFeatureIterator features = mLayer->getFeatures( QgsFeatureRequest().setFilterFid( fid ) );
 
     if ( features.nextFeature( feat ) )
     {
-      if ( expression->evaluate( &feat, fields ).toBool() )
+      context.setFeature( feat );
+      if ( expression->evaluate( &context ).toBool() )
       {
         newSelection << feat.id();
       }
@@ -166,18 +176,22 @@ void QgsExpressionSelectionDialog::on_mActionRemoveFromSelection_triggered()
 
   QgsExpression* expression = new QgsExpression( mExpressionBuilder->expressionText() );
 
-  const QgsFields fields = mLayer->pendingFields();
+  QgsExpressionContext context;
+  context << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::layerScope( mLayer );
 
-  expression->prepare( fields );
+  expression->prepare( &context );
 
   QgsFeature feat;
-  foreach ( const QgsFeatureId fid, oldSelection )
+  Q_FOREACH ( const QgsFeatureId fid, oldSelection )
   {
     QgsFeatureIterator features = mLayer->getFeatures( QgsFeatureRequest().setFilterFid( fid ) );
 
     if ( features.nextFeature( feat ) )
     {
-      if ( expression->evaluate( &feat, fields ).toBool() )
+      context.setFeature( feat );
+      if ( expression->evaluate( &context ).toBool() )
       {
         newSelection.remove( feat.id() );
       }
