@@ -5,6 +5,7 @@
 
 QgsExtentGroupBox::QgsExtentGroupBox( QWidget* parent )
     : QgsCollapsibleGroupBox( parent )
+    , mTitleBase( tr( "Extent" ) )
     , mExtentState( OriginalExtent )
 {
   setupUi( this );
@@ -16,6 +17,7 @@ QgsExtentGroupBox::QgsExtentGroupBox( QWidget* parent )
 
   connect( mCurrentExtentButton, SIGNAL( clicked() ), this, SLOT( setOutputExtentFromCurrent() ) );
   connect( mOriginalExtentButton, SIGNAL( clicked() ), this, SLOT( setOutputExtentFromOriginal() ) );
+  connect( this, SIGNAL( clicked( bool ) ), this, SLOT( groupBoxClicked() ) );
 }
 
 
@@ -58,6 +60,9 @@ void QgsExtentGroupBox::setOutputExtent( const QgsRectangle& r, const QgsCoordin
 
   mExtentState = state;
 
+  if ( isCheckable() && !isChecked() )
+    setChecked( true );
+
   updateTitle();
 
   emit extentChanged( extent );
@@ -91,7 +96,9 @@ void QgsExtentGroupBox::updateTitle()
     default:
       break;
   }
-  msg = tr( "Extent (current: %1)" ).arg( msg );
+  if ( isCheckable() && !isChecked() )
+    msg = tr( "none" );
+  msg = tr( "%1 (current: %2)" ).arg( mTitleBase, msg );
 
   setTitle( msg );
 }
@@ -114,9 +121,34 @@ void QgsExtentGroupBox::setOutputExtentFromUser( const QgsRectangle& extent, con
   setOutputExtent( extent, crs, UserExtent );
 }
 
+void QgsExtentGroupBox::groupBoxClicked()
+{
+  if ( !isCheckable() )
+    return;
+
+  updateTitle();
+
+  // output extent just went from null to something (or vice versa)
+  emit extentChanged( outputExtent() );
+}
+
 
 QgsRectangle QgsExtentGroupBox::outputExtent() const
 {
+  if ( isCheckable() && !isChecked() )
+    return QgsRectangle();
+
   return QgsRectangle( mXMinLineEdit->text().toDouble(), mYMinLineEdit->text().toDouble(),
                        mXMaxLineEdit->text().toDouble(), mYMaxLineEdit->text().toDouble() );
 }
+
+void QgsExtentGroupBox::setTitleBase( const QString& title )
+{
+  mTitleBase = title;
+  updateTitle();
+}
+
+QString QgsExtentGroupBox::titleBase() const
+{
+  return mTitleBase;
+}
