@@ -34,18 +34,18 @@ class GUI_EXPORT QgsFieldModel : public QAbstractItemModel
   public:
     enum FieldRoles
     {
-      FieldNameRole = Qt::UserRole + 1,  /* return field name if index corresponds to a field */
-      FieldIndexRole = Qt::UserRole + 2, /* return field index if index corresponds to a field */
-      ExpressionRole = Qt::UserRole + 3, /* return field name or expression */
-      IsExpressionRole = Qt::UserRole + 4, /* return if index corresponds to an expression */
-      ExpressionValidityRole = Qt::UserRole + 5, /* return if expression is valid or not */
-      FieldTypeRole = Qt::UserRole + 6 /* return the field type (if a field, return QVariant if expression) */
+      FieldNameRole = Qt::UserRole + 1,  /*!< return field name if index corresponds to a field */
+      FieldIndexRole = Qt::UserRole + 2, /*!< return field index if index corresponds to a field */
+      ExpressionRole = Qt::UserRole + 3, /*!< return field name or expression */
+      IsExpressionRole = Qt::UserRole + 4, /*!< return if index corresponds to an expression */
+      ExpressionValidityRole = Qt::UserRole + 5, /*!< return if expression is valid or not */
+      FieldTypeRole = Qt::UserRole + 6 /*!< return the field type (if a field, return QVariant if expression) */
     };
 
     /**
      * @brief QgsFieldModel creates a model to display the fields of a given layer
      */
-    explicit QgsFieldModel( QObject *parent = 0 );
+    explicit QgsFieldModel( QObject *parent = nullptr );
 
     //! return the index corresponding to a given fieldName
     QModelIndex indexFromName( const QString &fieldName );
@@ -67,6 +67,13 @@ class GUI_EXPORT QgsFieldModel : public QAbstractItemModel
     //! returns the currently used layer
     QgsVectorLayer* layer() { return mLayer; }
 
+    // QAbstractItemModel interface
+    QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const override;
+    QModelIndex parent( const QModelIndex &child ) const override;
+    int rowCount( const QModelIndex &parent = QModelIndex() ) const override;
+    int columnCount( const QModelIndex &parent ) const override;
+    QVariant data( const QModelIndex &index, int role ) const override;
+
   public slots:
     //! set the layer of whch fields are displayed
     void setLayer( QgsVectorLayer *layer );
@@ -86,14 +93,6 @@ class GUI_EXPORT QgsFieldModel : public QAbstractItemModel
 
   private:
     void fetchFeature();
-
-    // QAbstractItemModel interface
-  public:
-    QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const override;
-    QModelIndex parent( const QModelIndex &child ) const override;
-    int rowCount( const QModelIndex &parent = QModelIndex() ) const override;
-    int columnCount( const QModelIndex &parent ) const override;
-    QVariant data( const QModelIndex &index, int role ) const override;
 };
 
 #endif // QGSFIELDMODEL_H
