@@ -34,8 +34,12 @@
 #include <QWidget>
 
 QgsFormAnnotationItem::QgsFormAnnotationItem( QgsMapCanvas* canvas, QgsVectorLayer* vlayer, bool hasFeature, int feature )
-    : QgsAnnotationItem( canvas ), mWidgetContainer( 0 ), mDesignerWidget( 0 ), mVectorLayer( vlayer ),
-    mHasAssociatedFeature( hasFeature ), mFeature( feature )
+    : QgsAnnotationItem( canvas )
+    , mWidgetContainer( nullptr )
+    , mDesignerWidget( nullptr )
+    , mVectorLayer( vlayer )
+    , mHasAssociatedFeature( hasFeature )
+    , mFeature( feature )
 {
   mWidgetContainer = new QGraphicsProxyWidget( this );
   mWidgetContainer->setData( 0, "AnnotationItem" ); //mark embedded widget as belonging to an annotation item (composer knows it needs to be printed)
@@ -58,7 +62,7 @@ QgsFormAnnotationItem::~QgsFormAnnotationItem()
 void QgsFormAnnotationItem::setDesignerForm( const QString& uiFile )
 {
   mDesignerForm = uiFile;
-  mWidgetContainer->setWidget( 0 );
+  mWidgetContainer->setWidget( nullptr );
   delete mDesignerWidget;
   mDesignerWidget = createDesignerWidget( uiFile );
   if ( mDesignerWidget )
@@ -74,13 +78,13 @@ QWidget* QgsFormAnnotationItem::createDesignerWidget( const QString& filePath )
   QFile file( filePath );
   if ( !file.open( QFile::ReadOnly ) )
   {
-    return 0;
+    return nullptr;
   }
 
   QUiLoader loader;
   QFileInfo fi( file );
   loader.setWorkingDirectory( fi.dir() );
-  QWidget* widget = loader.load( &file, 0 );
+  QWidget* widget = loader.load( &file, nullptr );
   file.close();
 
   //get feature and set attribute information
@@ -89,8 +93,8 @@ QWidget* QgsFormAnnotationItem::createDesignerWidget( const QString& filePath )
     QgsFeature f;
     if ( mVectorLayer->getFeatures( QgsFeatureRequest().setFilterFid( mFeature ).setFlags( QgsFeatureRequest::NoGeometry ) ).nextFeature( f ) )
     {
-      const QgsFields& fields = mVectorLayer->pendingFields();
-      const QgsAttributes& attrs = f.attributes();
+      const QgsFields& fields = mVectorLayer->fields();
+      QgsAttributes attrs = f.attributes();
       for ( int i = 0; i < attrs.count(); ++i )
       {
         if ( i < fields.count() )
@@ -98,7 +102,7 @@ QWidget* QgsFormAnnotationItem::createDesignerWidget( const QString& filePath )
           QWidget* attWidget = widget->findChild<QWidget*>( fields[i].name() );
           if ( attWidget )
           {
-            QgsAttributeEditor::createAttributeEditor( widget, attWidget, mVectorLayer, i, attrs[i] );
+            QgsAttributeEditor::createAttributeEditor( widget, attWidget, mVectorLayer, i, attrs.at( i ) );
           }
         }
       }
@@ -190,7 +194,7 @@ void QgsFormAnnotationItem::writeXML( QDomDocument& doc ) const
 
 void QgsFormAnnotationItem::readXML( const QDomDocument& doc, const QDomElement& itemElem )
 {
-  mVectorLayer = 0;
+  mVectorLayer = nullptr;
   if ( itemElem.hasAttribute( "vectorLayer" ) )
   {
     mVectorLayer = dynamic_cast<QgsVectorLayer*>( QgsMapLayerRegistry::instance()->mapLayer( itemElem.attribute( "vectorLayer", "" ) ) );
@@ -247,7 +251,7 @@ void QgsFormAnnotationItem::setFeatureForMapPosition()
   mFeature = currentFeatureId;
 
   //create new embedded widget
-  mWidgetContainer->setWidget( 0 );
+  mWidgetContainer->setWidget( nullptr );
   delete mDesignerWidget;
   mDesignerWidget = createDesignerWidget( mDesignerForm );
   if ( mDesignerWidget )
