@@ -45,13 +45,13 @@ class GUI_EXPORT QgsMapLayerAction : public QAction
 
     //! Creates a map layer action which can run on any layer
     //! @note using AllActions as a target probably does not make a lot of sense. This default action was settled for API compatiblity reasons.
-    QgsMapLayerAction( QString name, QObject *parent, Targets targets = AllActions, QIcon icon = QIcon() );
+    QgsMapLayerAction( const QString& name, QObject *parent, const Targets& targets = AllActions, const QIcon& icon = QIcon() );
 
     //! Creates a map layer action which can run only on a specific layer
-    QgsMapLayerAction( QString name, QObject *parent, QgsMapLayer* layer, Targets targets = AllActions, QIcon icon = QIcon() );
+    QgsMapLayerAction( const QString& name, QObject *parent, QgsMapLayer* layer, const Targets& targets = AllActions, const QIcon& icon = QIcon() );
 
     //! Creates a map layer action which can run on a specific type of layer
-    QgsMapLayerAction( QString name, QObject *parent, QgsMapLayer::LayerType layerType, Targets targets = AllActions, QIcon icon = QIcon() );
+    QgsMapLayerAction( const QString& name, QObject *parent, QgsMapLayer::LayerType layerType, const Targets& targets = AllActions, const QIcon& icon = QIcon() );
 
     ~QgsMapLayerAction();
 
@@ -59,7 +59,7 @@ class GUI_EXPORT QgsMapLayerAction : public QAction
     bool canRunUsingLayer( QgsMapLayer* layer ) const;
 
     /** Triggers the action with the specified layer and list of feature. */
-    void triggerForFeatures( QgsMapLayer* layer, const QList<QgsFeature> featureList );
+    void triggerForFeatures( QgsMapLayer* layer, const QList<QgsFeature>& featureList );
 
     /** Triggers the action with the specified layer and feature. */
     void triggerForFeature( QgsMapLayer* layer, const QgsFeature* feature );
@@ -68,13 +68,13 @@ class GUI_EXPORT QgsMapLayerAction : public QAction
     void triggerForLayer( QgsMapLayer* layer );
 
     /** Define the targets of the action */
-    void setTargets( Targets targets ) {mTargets = targets;}
+    void setTargets( const Targets& targets ) {mTargets = targets;}
     /** Return availibity of action */
     const Targets& targets() const {return mTargets;}
 
   signals:
     /** Triggered when action has been run for a specific list of features */
-    void triggeredForFeatures( QgsMapLayer* layer, const QList<QgsFeature> featureList );
+    void triggeredForFeatures( QgsMapLayer* layer, const QList<QgsFeature>& featureList );
 
     /** Triggered when action has been run for a specific feature */
     void triggeredForFeature( QgsMapLayer* layer, const QgsFeature& feature );
@@ -113,23 +113,23 @@ class GUI_EXPORT QgsMapLayerActionRegistry : public QObject
 
     ~QgsMapLayerActionRegistry();
 
-    /**Adds a map layer action to the registry*/
+    /** Adds a map layer action to the registry*/
     void addMapLayerAction( QgsMapLayerAction * action );
 
-    /**Returns the map layer actions which can run on the specified layer*/
-    QList<QgsMapLayerAction *> mapLayerActions( QgsMapLayer* layer, QgsMapLayerAction::Targets targets = QgsMapLayerAction::AllActions );
+    /** Returns the map layer actions which can run on the specified layer*/
+    QList<QgsMapLayerAction *> mapLayerActions( QgsMapLayer* layer, const QgsMapLayerAction::Targets& targets = QgsMapLayerAction::AllActions );
 
-    /**Removes a map layer action from the registry*/
+    /** Removes a map layer action from the registry*/
     bool removeMapLayerAction( QgsMapLayerAction *action );
 
-    /**Sets the default action for a layer*/
+    /** Sets the default action for a layer*/
     void setDefaultActionForLayer( QgsMapLayer* layer, QgsMapLayerAction* action );
-    /**Returns the default action for a layer*/
+    /** Returns the default action for a layer*/
     QgsMapLayerAction * defaultActionForLayer( QgsMapLayer* layer );
 
   protected:
     //! protected constructor
-    QgsMapLayerActionRegistry( QObject * parent = 0 );
+    QgsMapLayerActionRegistry( QObject * parent = nullptr );
 
     QList< QgsMapLayerAction* > mMapLayerActionList;
 
