@@ -38,11 +38,11 @@ class GUI_EXPORT QgsMapLayerModel : public QAbstractItemModel
     /**
      * @brief QgsMapLayerModel creates a model to display layers in widgets.
      */
-    explicit QgsMapLayerModel( QObject *parent = 0 );
+    explicit QgsMapLayerModel( QObject *parent = nullptr );
     /**
      * @brief QgsMapLayerModel creates a model to display a specific list of layers in a widget.
      */
-    explicit QgsMapLayerModel( QList<QgsMapLayer*> layers, QObject *parent = 0 );
+    explicit QgsMapLayerModel( const QList<QgsMapLayer*>& layers, QObject *parent = nullptr );
 
     /**
      * @brief setItemsCheckable defines if layers should be selectable in the widget
@@ -66,8 +66,8 @@ class GUI_EXPORT QgsMapLayerModel : public QAbstractItemModel
 
 
   protected slots:
-    void removeLayers( const QStringList layerIds );
-    void addLayers( QList<QgsMapLayer*> layers );
+    void removeLayers( const QStringList& layerIds );
+    void addLayers( const QList<QgsMapLayer*>& layers );
 
   protected:
     QList<QgsMapLayer*> mLayers;
