@@ -21,6 +21,7 @@
 QgsMapLayerProxyModel::QgsMapLayerProxyModel( QObject *parent )
     : QSortFilterProxyModel( parent )
     , mFilters( All )
+    , mExceptList( QList<QgsMapLayer*>() )
     , mModel( new QgsMapLayerModel( parent ) )
 {
   setSourceModel( mModel );
@@ -30,16 +31,22 @@ QgsMapLayerProxyModel::QgsMapLayerProxyModel( QObject *parent )
   sort( 0 );
 }
 
-QgsMapLayerProxyModel *QgsMapLayerProxyModel::setFilters( Filters filters )
+QgsMapLayerProxyModel *QgsMapLayerProxyModel::setFilters( const Filters& filters )
 {
   mFilters = filters;
   invalidateFilter();
   return this;
 }
 
+void QgsMapLayerProxyModel::setExceptedLayerList( const QList<QgsMapLayer*>& exceptList )
+{
+  mExceptList = exceptList;
+  invalidateFilter();
+}
+
 bool QgsMapLayerProxyModel::filterAcceptsRow( int source_row, const QModelIndex &source_parent ) const
 {
-  if ( mFilters.testFlag( All ) )
+  if ( mFilters.testFlag( All ) && mExceptList.isEmpty() )
     return true;
 
   QModelIndex index = sourceModel()->index( source_row, 0, source_parent );
@@ -47,6 +54,9 @@ bool QgsMapLayerProxyModel::filterAcceptsRow( int source_row, const QModelIndex
   if ( !layer )
     return false;
 
+  if ( mExceptList.contains( layer ) )
+    return false;
+
   // layer type
   if (( mFilters.testFlag( RasterLayer ) && layer->type() == QgsMapLayer::RasterLayer ) ||
       ( mFilters.testFlag( VectorLayer ) && layer->type() == QgsMapLayer::VectorLayer ) ||
