@@ -30,49 +30,11 @@
 #include "qgslogger.h"
 #include <limits.h>
 
-//! widget that serves as rectangle showing current extent in overview
-class QgsPanningWidget : public QWidget
-{
-    QPolygon mPoly;
-
-  public:
-    QgsPanningWidget( QWidget* parent )
-        : QWidget( parent )
-    {
-      setObjectName( "panningWidget" );
-      setMinimumSize( 5, 5 );
-      setAttribute( Qt::WA_NoSystemBackground );
-    }
-
-    void setPolygon( const QPolygon& p )
-    {
-      if ( p == mPoly ) return;
-      mPoly = p;
-      setGeometry( p.boundingRect() );
-      update();
-    }
-
-
-    void paintEvent( QPaintEvent* pe ) override
-    {
-      Q_UNUSED( pe );
-
-      QPainter p;
-      p.begin( this );
-      p.setPen( Qt::red );
-      QPolygonF t = mPoly.translated( -mPoly.boundingRect().left(), -mPoly.boundingRect().top() );
-      p.drawConvexPolygon( t );
-      p.end();
-    }
-
-};
-
-
 
 QgsMapOverviewCanvas::QgsMapOverviewCanvas( QWidget * parent, QgsMapCanvas* mapCanvas )
     : QWidget( parent )
     , mMapCanvas( mapCanvas )
-    , mJob( 0 )
+    , mJob( nullptr )
 {
   setAutoFillBackground( true );
   setObjectName( "theOverviewCanvas" );
@@ -184,7 +146,7 @@ void QgsMapOverviewCanvas::mouseMoveEvent( QMouseEvent * e )
 }
 
 
-void QgsMapOverviewCanvas::updatePanningWidget( const QPoint& pos )
+void QgsMapOverviewCanvas::updatePanningWidget( QPoint pos )
 {
 //  if (mPanningWidget->isHidden())
 //    return;
@@ -232,7 +194,7 @@ void QgsMapOverviewCanvas::mapRenderingFinished()
   mPixmap = QPixmap::fromImage( mJob->renderedImage() );
 
   delete mJob;
-  mJob = 0;
+  mJob = nullptr;
 
   // schedule repaint
   update();
@@ -258,7 +220,7 @@ void QgsMapOverviewCanvas::setLayerSet( const QStringList& layerSet )
 {
   QgsDebugMsg( "layerSet: " + layerSet.join( ", " ) );
 
-  foreach ( const QString& layerID, mSettings.layers() )
+  Q_FOREACH ( const QString& layerID, mSettings.layers() )
   {
     if ( QgsMapLayer* ml = QgsMapLayerRegistry::instance()->mapLayer( layerID ) )
       disconnect( ml, SIGNAL( repaintRequested() ), this, SLOT( layerRepaintRequested() ) );
@@ -266,7 +228,7 @@ void QgsMapOverviewCanvas::setLayerSet( const QStringList& layerSet )
 
   mSettings.setLayers( layerSet );
 
-  foreach ( const QString& layerID, mSettings.layers() )
+  Q_FOREACH ( const QString& layerID, mSettings.layers() )
   {
     if ( QgsMapLayer* ml = QgsMapLayerRegistry::instance()->mapLayer( layerID ) )
       connect( ml, SIGNAL( repaintRequested() ), this, SLOT( layerRepaintRequested() ) );
@@ -304,3 +266,38 @@ QStringList QgsMapOverviewCanvas::layerSet() const
 {
   return mSettings.layers();
 }
+
+
+/// @cond PRIVATE
+
+QgsPanningWidget::QgsPanningWidget( QWidget* parent )
+    : QWidget( parent )
+{
+  setObjectName( "panningWidget" );
+  setMinimumSize( 5, 5 );
+  setAttribute( Qt::WA_NoSystemBackground );
+}
+
+void QgsPanningWidget::setPolygon( const QPolygon& p )
+{
+  if ( p == mPoly ) return;
+  mPoly = p;
+  setGeometry( p.boundingRect() );
+  update();
+}
+
+void QgsPanningWidget::paintEvent( QPaintEvent* pe )
+{
+  Q_UNUSED( pe );
+
+  QPainter p;
+  p.begin( this );
+  p.setPen( Qt::red );
+  QPolygonF t = mPoly.translated( -mPoly.boundingRect().left(), -mPoly.boundingRect().top() );
+  p.drawConvexPolygon( t );
+  p.end();
+}
+
+
+
+///@endcond
