@@ -96,7 +96,17 @@ QString QgsMapTip::fetchFeature( QgsMapLayer *layer, QgsPoint &mapPosition, QgsM
 
   int idx = vlayer->fieldNameIndex( vlayer->displayField() );
   if ( idx < 0 )
-    return QgsExpression::replaceExpressionText( vlayer->displayField(), &feature, vlayer );
+  {
+    QgsExpressionContext context;
+    context << QgsExpressionContextUtils::globalScope()
+    << QgsExpressionContextUtils::projectScope()
+    << QgsExpressionContextUtils::layerScope( vlayer );
+    if ( mpMapCanvas )
+      context.appendScope( QgsExpressionContextUtils::mapSettingsScope( mpMapCanvas->mapSettings() ) );
+
+    context.setFeature( feature );
+    return QgsExpression::replaceExpressionText( vlayer->displayField(), &context );
+  }
   else
     return feature.attribute( idx ).toString();
 }
