@@ -26,15 +26,20 @@ class QgsMapCanvas;
  */
 class GUI_EXPORT QgsMapToolPan : public QgsMapTool
 {
+    Q_OBJECT
+
   public:
     //! constructor
     QgsMapToolPan( QgsMapCanvas* canvas );
 
+    //! Mouse press event
+    virtual void canvasPressEvent( QgsMapMouseEvent* e ) override;
+
     //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e ) override;
+    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
 
     //! Overridden mouse release event
-    virtual void canvasReleaseEvent( QMouseEvent * e ) override;
+    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
 
     virtual bool isTransient() override { return true; }
 
