@@ -52,16 +52,19 @@ class GUI_EXPORT QgsMessageBar: public QFrame
       SUCCESS = 3
     };
 
-    QgsMessageBar( QWidget *parent = 0 );
+    QgsMessageBar( QWidget *parent = nullptr );
     ~QgsMessageBar();
 
-    /*! display a message item on the bar after hiding the currently visible one
-     *  and putting it in a stack.
-     * @param item item to display
+    /**
+     * Display a message item on the bar after hiding the currently visible one
+     * and putting it in a stack.
+     * The message bar will take ownership of the item.
+     *
+     * @param item Item to display
      */
     void pushItem( QgsMessageBarItem *item );
 
-    /*! display a widget as a message on the bar after hiding the currently visible one
+    /** Display a widget as a message on the bar after hiding the currently visible one
      *  and putting it in a stack.
      * @param widget message widget to display
      * @param level is QgsMessageBar::INFO, WARNING, CRITICAL or SUCCESS
@@ -69,7 +72,7 @@ class GUI_EXPORT QgsMessageBar: public QFrame
      */
     QgsMessageBarItem *pushWidget( QWidget *widget, MessageLevel level = INFO, int duration = 0 );
 
-    /*! remove the passed widget from the bar (if previously added),
+    /** Remove the passed widget from the bar (if previously added),
      *  then display the next one in the stack if any or hide the bar
      *  @param item item to remove
      *  @return true if the widget was removed, false otherwise
@@ -77,17 +80,19 @@ class GUI_EXPORT QgsMessageBar: public QFrame
     bool popWidget( QgsMessageBarItem *item );
 
     //! make out a widget containing a message to be displayed on the bar
-    static QgsMessageBarItem* createMessage( const QString &text, QWidget *parent = 0 );
+    static QgsMessageBarItem* createMessage( const QString &text, QWidget *parent = nullptr );
     //! make out a widget containing title and message to be displayed on the bar
-    static QgsMessageBarItem* createMessage( const QString &title, const QString &text, QWidget *parent = 0 );
+    static QgsMessageBarItem* createMessage( const QString &title, const QString &text, QWidget *parent = nullptr );
     //! make out a widget containing title and message to be displayed on the bar
-    static QgsMessageBarItem* createMessage( QWidget *widget, QWidget *parent = 0 );
+    static QgsMessageBarItem* createMessage( QWidget *widget, QWidget *parent = nullptr );
 
     //! convenience method for pushing a message to the bar
     void pushMessage( const QString &text, MessageLevel level = INFO, int duration = 0 ) { return pushMessage( QString::null, text, level, duration ); }
     //! convenience method for pushing a message with title to the bar
     void pushMessage( const QString &title, const QString &text, MessageLevel level = INFO, int duration = 0 );
 
+    QgsMessageBarItem *currentItem() { return mCurrentItem; }
+
   signals:
     //! emitted when a message widget is added to the bar
     void widgetAdded( QgsMessageBarItem *item );
@@ -96,19 +101,19 @@ class GUI_EXPORT QgsMessageBar: public QFrame
     void widgetRemoved( QgsMessageBarItem *item );
 
   public slots:
-    /*! remove the currently displayed widget from the bar and
-     *  display the next in the stack if any or hide the bar
+    /** Remove the currently displayed widget from the bar and
+     *  display the next in the stack if any or hide the bar.
      *  @return true if the widget was removed, false otherwise
      */
     bool popWidget();
 
-    /*! remove all items from the bar's widget list
+    /** Remove all items from the bar's widget list
      *  @return true if all items were removed, false otherwise
      */
     bool clearWidgets();
 
     /**
-     * Pushes a warning with default timeout to the message bar
+     * Pushes a success message with default timeout to the message bar
      * @param title title string for message
      * @param message The message to be displayed
      * @note added in 2.8
@@ -116,7 +121,7 @@ class GUI_EXPORT QgsMessageBar: public QFrame
     void pushSuccess( const QString& title, const QString& message );
 
     /**
-     * Pushes a warning with default timeout to the message bar
+     * Pushes a information message with default timeout to the message bar
      * @param title title string for message
      * @param message The message to be displayed
      * @note added in 2.8
@@ -132,7 +137,7 @@ class GUI_EXPORT QgsMessageBar: public QFrame
     void pushWarning( const QString& title, const QString& message );
 
     /**
-     * Pushes a warning with default timeout to the message bar
+     * Pushes a critical warning with default timeout to the message bar
      * @param title title string for message
      * @param message The message to be displayed
      * @note added in 2.8
