@@ -30,7 +30,7 @@
 #include <QScrollBar>
 
 
-QgsMessageLogViewer::QgsMessageLogViewer( QStatusBar *statusBar, QWidget *parent, Qt::WindowFlags fl )
+QgsMessageLogViewer::QgsMessageLogViewer( QStatusBar *statusBar, QWidget *parent, const Qt::WindowFlags& fl )
     : QDialog( parent, fl )
 {
   Q_UNUSED( statusBar )
@@ -72,11 +72,12 @@ void QgsMessageLogViewer::logMessage( QString message, QString tag, QgsMessageLo
   QString prefix = QString( "%1\t%2\t" )
                    .arg( QDateTime::currentDateTime().toString( Qt::ISODate ) )
                    .arg( level );
-  w->appendPlainText( message.prepend( prefix ).replace( "\n", "\n\t\t\t" ) );
+  w->appendPlainText( message.prepend( prefix ).replace( '\n', "\n\t\t\t" ) );
   w->verticalScrollBar()->setValue( w->verticalScrollBar()->maximum() );
 }
 
 void QgsMessageLogViewer::closeTab( int index )
 {
-  tabWidget->removeTab( index );
+  if ( tabWidget->count() > 1 )
+    tabWidget->removeTab( index );
 }
