@@ -54,12 +54,12 @@
 #include <QNetworkRequest>
 #include <QNetworkReply>
 
-QgsOWSSourceSelect::QgsOWSSourceSelect( QString service, QWidget * parent, Qt::WindowFlags fl, bool managerMode, bool embeddedMode )
+QgsOWSSourceSelect::QgsOWSSourceSelect( const QString& service, QWidget * parent, const Qt::WindowFlags& fl, bool managerMode, bool embeddedMode )
     : QDialog( parent, fl )
     , mService( service )
     , mManagerMode( managerMode )
     , mEmbeddedMode( embeddedMode )
-    , mCurrentTileset( 0 )
+    , mCurrentTileset( nullptr )
 {
   setupUi( this );
 
@@ -146,7 +146,7 @@ void QgsOWSSourceSelect::populateFormats()
 
   clearFormats();
 
-  if ( mProviderFormats.size() == 0 )
+  if ( mProviderFormats.isEmpty() )
   {
     mProviderFormats = providerFormats();
     for ( int i = 0; i < mProviderFormats.size(); i++ )
@@ -182,7 +182,7 @@ void QgsOWSSourceSelect::populateFormats()
   {
     QString format = layersFormats.value( i );
     QgsDebugMsg( "server format = " + format );
-    QString simpleFormat = format.toLower().replace( "image/", "" ).replace( QRegExp( "_.*" ), "" );
+    QString simpleFormat = format.toLower().remove( "image/" ).remove( QRegExp( "_.*" ) );
     QgsDebugMsg( "server simpleFormat = " + simpleFormat );
     QString mimeFormat = "image/" + formatsMap.value( simpleFormat );
     QgsDebugMsg( "server mimeFormat = " + mimeFormat );
@@ -199,7 +199,7 @@ void QgsOWSSourceSelect::populateFormats()
 
       if ( simpleFormat.contains( "tif" ) ) // prefer *tif*
       {
-        if ( preferred < 0 || simpleFormat.startsWith( "g" ) ) // prefer geotiff
+        if ( preferred < 0 || simpleFormat.startsWith( 'g' ) ) // prefer geotiff
         {
           preferred = i;
         }
@@ -245,7 +245,7 @@ void QgsOWSSourceSelect::populateConnectionList()
 }
 void QgsOWSSourceSelect::on_mNewButton_clicked()
 {
-  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, "/Qgis/connections-" + mService.toLower() + "/" );
+  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, "/Qgis/connections-" + mService.toLower() + '/' );
 
   if ( nc->exec() )
   {
@@ -258,7 +258,7 @@ void QgsOWSSourceSelect::on_mNewButton_clicked()
 
 void QgsOWSSourceSelect::on_mEditButton_clicked()
 {
-  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, "/Qgis/connections-" + mService.toLower() + "/", mConnectionsComboBox->currentText() );
+  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, "/Qgis/connections-" + mService.toLower() + '/', mConnectionsComboBox->currentText() );
 
   if ( nc->exec() )
   {
@@ -291,7 +291,7 @@ void QgsOWSSourceSelect::on_mSaveButton_clicked()
 
 void QgsOWSSourceSelect::on_mLoadButton_clicked()
 {
-  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), ".",
+  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), QDir::homePath(),
                      tr( "XML files (*.xml *XML)" ) );
   if ( fileName.isEmpty() )
   {
@@ -376,7 +376,7 @@ void QgsOWSSourceSelect::enableLayersForCrs( QTreeWidgetItem * )
 void QgsOWSSourceSelect::on_mChangeCRSButton_clicked()
 {
   QStringList layers;
-  foreach ( QTreeWidgetItem *item, mLayersTreeWidget->selectedItems() )
+  Q_FOREACH ( QTreeWidgetItem *item, mLayersTreeWidget->selectedItems() )
   {
     QString layer = item->data( 0, Qt::UserRole + 0 ).toString();
     if ( !layer.isEmpty() )
@@ -419,7 +419,7 @@ void QgsOWSSourceSelect::populateCRS()
   QgsDebugMsg( "Entered" );
   clearCRS();
   mSelectedLayersCRSs = selectedLayersCRSs().toSet();
-  mCRSLabel->setText( tr( "Coordinate Reference System (%n available)", "crs count", mSelectedLayersCRSs.count() ) + ":" );
+  mCRSLabel->setText( tr( "Coordinate Reference System (%n available)", "crs count", mSelectedLayersCRSs.count() ) + ':' );
 
   mChangeCRSButton->setDisabled( mSelectedLayersCRSs.isEmpty() );
 
@@ -428,8 +428,8 @@ void QgsOWSSourceSelect::populateCRS()
     // check whether current CRS is supported
     // if not, use one of the available CRS
     QString defaultCRS;
-    QSet<QString>::const_iterator it = mSelectedLayersCRSs.begin();
-    for ( ; it != mSelectedLayersCRSs.end(); ++it )
+    QSet<QString>::const_iterator it = mSelectedLayersCRSs.constBegin();
+    for ( ; it != mSelectedLayersCRSs.constEnd(); ++it )
     {
       if ( it->compare( mSelectedCRS, Qt::CaseInsensitive ) == 0 )
         break;
@@ -456,7 +456,7 @@ void QgsOWSSourceSelect::populateCRS()
 
 void QgsOWSSourceSelect::clearCRS()
 {
-  mCRSLabel->setText( tr( "Coordinate Reference System" ) + ":" );
+  mCRSLabel->setText( tr( "Coordinate Reference System" ) + ':' );
   mSelectedCRS = "";
   mSelectedCRSLabel->setText( "" );
   mChangeCRSButton->setEnabled( false );
@@ -479,7 +479,7 @@ void QgsOWSSourceSelect::on_mTilesetsTableWidget_itemClicked( QTableWidgetItem *
   }
   else
   {
-    mCurrentTileset = 0;
+    mCurrentTileset = nullptr;
   }
   mTilesetsTableWidget->blockSignals( false );
 
@@ -589,7 +589,7 @@ void QgsOWSSourceSelect::on_mAddDefaultButton_clicked()
   addDefaultServers();
 }
 
-QString QgsOWSSourceSelect::descriptionForAuthId( QString authId )
+QString QgsOWSSourceSelect::descriptionForAuthId( const QString& authId )
 {
   if ( mCrsNames.contains( authId ) )
     return mCrsNames[ authId ];
