@@ -18,6 +18,7 @@
 #include "qgsapplication.h"
 #include "qgslogger.h"
 #include "qgscoordinatereferencesystem.h"
+#include "qgsmessagelog.h"
 
 //qt includes
 #include <QFileInfo>
@@ -26,8 +27,11 @@
 #include <QMessageBox>
 #include <QSettings>
 
-QgsProjectionSelector::QgsProjectionSelector( QWidget* parent, const char *name, Qt::WindowFlags fl )
+QgsProjectionSelector::QgsProjectionSelector( QWidget* parent, const char *name, const Qt::WindowFlags& fl )
     : QWidget( parent, fl )
+    , mUserProjList( nullptr )
+    , mGeoList( nullptr )
+    , mProjList( nullptr )
     , mProjListDone( false )
     , mUserProjListDone( false )
     , mRecentProjListDone( false )
@@ -60,42 +64,7 @@ QgsProjectionSelector::QgsProjectionSelector( QWidget* parent, const char *name,
   // Hide (internal) ID column
   lstRecent->setColumnHidden( QGIS_CRS_ID_COLUMN, true );
 
-  // Read settings from persistent storage
-  QSettings settings;
-  mRecentProjections = settings.value( "/UI/recentProjections" ).toStringList();
-  /*** The reading (above) of internal id from persistent storage should be removed sometime in the future */
-  /*** This is kept now for backwards compatibility */
-
-  QStringList projectionsProj4  = settings.value( "/UI/recentProjectionsProj4" ).toStringList();
-  QStringList projectionsAuthId = settings.value( "/UI/recentProjectionsAuthId" ).toStringList();
-  if ( projectionsAuthId.size() >= mRecentProjections.size() )
-  {
-    // We had saved state with AuthId and Proj4. Use that instead
-    // to find out the crs id
-    QgsDebugMsg( "Use popular projection list from AuthId/Proj4 saved state" );
-    mRecentProjections.clear();
-    for ( int i = 0; i <  projectionsAuthId.size(); i++ )
-    {
-      // Create a crs from the EPSG
-      QgsCoordinateReferenceSystem crs;
-      crs.createFromOgcWmsCrs( projectionsAuthId.at( i ) );
-      if ( ! crs.isValid() )
-      {
-        // Couldn't create from EPSG, try the Proj4 string instead
-        if ( i >= projectionsProj4.size() || !crs.createFromProj4( projectionsProj4.at( i ) ) )
-        {
-          // No? Skip this entry
-          continue;
-        }
-        //If the CRS can be created but do not correspond to a CRS in the database, skip it (for example a deleted custom CRS)
-        if ( crs.srsid() == 0 )
-        {
-          continue;
-        }
-      }
-      mRecentProjections << QString::number( crs.srsid() );
-    }
-  }
+  mRecentProjections = QgsCoordinateReferenceSystem::recentProjections();
 }
 
 QgsProjectionSelector::~QgsProjectionSelector()
@@ -203,9 +172,9 @@ QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * c
 
   // iterate through all incoming CRSs
 
-  foreach ( QString auth_id, crsFilter->values() )
+  Q_FOREACH ( const QString& auth_id, *crsFilter )
   {
-    QStringList parts = auth_id.split( ":" );
+    QStringList parts = auth_id.split( ':' );
 
     if ( parts.size() < 2 )
       continue;
@@ -216,18 +185,18 @@ QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * c
   if ( authParts.isEmpty() )
     return sqlExpression;
 
-  if ( authParts.size() > 0 )
+  if ( !authParts.isEmpty() )
   {
     QString prefix = " AND (";
-    foreach ( QString auth_name, authParts.keys() )
+    Q_FOREACH ( const QString& auth_name, authParts.keys() )
     {
       sqlExpression += QString( "%1(upper(auth_name)='%2' AND upper(auth_id) IN ('%3'))" )
-                       .arg( prefix )
-                       .arg( auth_name )
-                       .arg( authParts[auth_name].join( "','" ) );
+                       .arg( prefix,
+                             auth_name,
+                             authParts[auth_name].join( "','" ) );
       prefix = " OR ";
     }
-    sqlExpression += ")";
+    sqlExpression += ')';
   }
 
   QgsDebugMsg( "exiting with '" + sqlExpression + "'." );
@@ -235,7 +204,7 @@ QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * c
   return sqlExpression;
 }
 
-void QgsProjectionSelector::setSelectedCrsName( QString theCRSName )
+void QgsProjectionSelector::setSelectedCrsName( const QString& theCRSName )
 {
   applySelection( NAME_COLUMN, theCRSName );
 }
@@ -245,7 +214,7 @@ void QgsProjectionSelector::setSelectedCrsId( long theCRSID )
   applySelection( QGIS_CRS_ID_COLUMN, QString::number( theCRSID ) );
 }
 
-void QgsProjectionSelector::setSelectedAuthId( QString id )
+void QgsProjectionSelector::setSelectedAuthId( const QString& id )
 {
   applySelection( AUTHID_COLUMN, id );
 }
@@ -274,7 +243,7 @@ void QgsProjectionSelector::applySelection( int column, QString value )
     return;
 
   QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( value, Qt::MatchExactly | Qt::MatchRecursive, column );
-  if ( nodes.count() > 0 )
+  if ( !nodes.isEmpty() )
   {
     QgsDebugMsg( QString( "found %1,%2" ).arg( column ).arg( value ) );
     lstCoordinateSystems->setCurrentItem( nodes.first() );
@@ -282,7 +251,7 @@ void QgsProjectionSelector::applySelection( int column, QString value )
   else
   {
     QgsDebugMsg( QString( "nothing found for %1,%2" ).arg( column ).arg( value ) );
-    // unselect the selected item to avoid confusing the user
+    // deselect the selected item to avoid confusing the user
     lstCoordinateSystems->clearSelection();
     lstRecent->clearSelection();
     teProjection->setText( "" );
@@ -296,7 +265,7 @@ void QgsProjectionSelector::insertRecent( long theCrsId )
     return;
 
   QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( QString::number( theCrsId ), Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
-  if ( nodes.count() == 0 )
+  if ( nodes.isEmpty() )
     return;
 
   lstRecent->insertTopLevelItem( 0, new QTreeWidgetItem( lstRecent, QStringList()
@@ -340,7 +309,7 @@ QString QgsProjectionSelector::selectedProj4String()
   {
     databaseFileName = QgsApplication::qgisUserDbFilePath();
     if ( !QFileInfo( databaseFileName ).exists() ) //its unlikely that this condition will ever be reached
-      return QString( "" );
+      return QString();
   }
   else //must be a system projection then
   {
@@ -350,7 +319,7 @@ QString QgsProjectionSelector::selectedProj4String()
   QgsDebugMsg( "db = " + databaseFileName );
 
   sqlite3 *database;
-  int rc = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+  int rc = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
   if ( rc )
   {
     showDBMissingWarning( databaseFileName );
@@ -382,7 +351,7 @@ QString QgsProjectionSelector::selectedProj4String()
   return projString;
 }
 
-QString QgsProjectionSelector::getSelectedExpression( QString expression )
+QString QgsProjectionSelector::getSelectedExpression( const QString& expression )
 {
   // Only return the attribute if there is a node in the tree
   // selected that has an srs_id.  This prevents error if the user
@@ -393,7 +362,7 @@ QString QgsProjectionSelector::getSelectedExpression( QString expression )
   // not a top-level projection node
   QTreeWidgetItem *lvi = lstCoordinateSystems->currentItem();
   if ( !lvi || lvi->text( QGIS_CRS_ID_COLUMN ).isEmpty() )
-    return 0;
+    return nullptr;
 
   //
   // Determine if this is a user projection or a system on
@@ -405,7 +374,7 @@ QString QgsProjectionSelector::getSelectedExpression( QString expression )
     databaseFileName = QgsApplication::qgisUserDbFilePath();
     if ( !QFileInfo( databaseFileName ).exists() )
     {
-      return 0;
+      return nullptr;
     }
   }
   else
@@ -419,19 +388,19 @@ QString QgsProjectionSelector::getSelectedExpression( QString expression )
   // assuming that it will never be used anywhere else. Given the low overhead,
   // opening it each time seems to be a reasonable approach at this time.
   sqlite3 *database;
-  int rc = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+  int rc = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
   if ( rc )
   {
     showDBMissingWarning( databaseFileName );
-    return 0;
+    return nullptr;
   }
 
   // prepare the sql statement
   const char *tail;
   sqlite3_stmt *stmt;
   QString sql = QString( "select %1 from tbl_srs where srs_id=%2" )
-                .arg( expression )
-                .arg( lvi->text( QGIS_CRS_ID_COLUMN ) );
+                .arg( expression,
+                      lvi->text( QGIS_CRS_ID_COLUMN ) );
 
   QgsDebugMsg( QString( "Finding selected attribute using : %1" ).arg( sql ) );
   rc = sqlite3_prepare( database, sql.toUtf8(), sql.toUtf8().length(), &stmt, &tail );
@@ -468,7 +437,6 @@ QString QgsProjectionSelector::selectedAuthId()
     return getSelectedExpression( "upper(auth_name||':'||auth_id)" );
 }
 
-
 long QgsProjectionSelector::selectedCrsId()
 {
   QTreeWidgetItem* item = lstCoordinateSystems->currentItem();
@@ -480,7 +448,7 @@ long QgsProjectionSelector::selectedCrsId()
 }
 
 
-void QgsProjectionSelector::setOgcWmsCrsFilter( QSet<QString> crsFilter )
+void QgsProjectionSelector::setOgcWmsCrsFilter( const QSet<QString>& crsFilter )
 {
   mCrsFilter = crsFilter;
   mProjListDone = false;
@@ -515,7 +483,7 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> *crsFilter )
   // if it doesnt exist we copy it in from the global resources dir
 
   //return straight away if the user has not created any custom projections
-  if ( !QFileInfo( databaseFileName ).exists( ) )
+  if ( !QFileInfo( databaseFileName ).exists() )
   {
     QgsDebugMsg( "Users qgis.db not found...skipping" );
     mUserProjListDone = true;
@@ -526,7 +494,7 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> *crsFilter )
   const char   *tail;
   sqlite3_stmt *stmt;
   //check the db is available
-  int result = sqlite3_open_v2( databaseFileName.toUtf8().constData(), &database, SQLITE_OPEN_READONLY, NULL );
+  int result = sqlite3_open_v2( databaseFileName.toUtf8().constData(), &database, SQLITE_OPEN_READONLY, nullptr );
   if ( result )
   {
     // XXX This will likely never happen since on open, sqlite creates the
@@ -606,7 +574,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> *crsFilter )
 
   // open the database containing the spatial reference data
   sqlite3 *database;
-  int rc = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+  int rc = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
   if ( rc )
   {
     // XXX This will likely never happen since on open, sqlite creates the
@@ -614,22 +582,14 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> *crsFilter )
     showDBMissingWarning( mSrsDatabaseFileName );
     return;
   }
-  // prepare the sql statement
+
   const char *tail;
   sqlite3_stmt *stmt;
-  // get total count of records in the projection table
-  QString sql = "select count(*) from tbl_srs";
-
-  rc = sqlite3_prepare( database, sql.toUtf8(), sql.toUtf8().length(), &stmt, &tail );
-  Q_ASSERT( rc == SQLITE_OK );
-  sqlite3_step( stmt );
-  sqlite3_finalize( stmt );
-
   // Set up the query to retrieve the projection information needed to populate the list
   //note I am giving the full field names for clarity here and in case someone
   //changes the underlying view TS
-  sql = QString( "select description, srs_id, upper(auth_name||':'||auth_id), is_geo, name, parameters, deprecated from vw_srs where %1 order by name,description" )
-        .arg( sqlFilter );
+  QString sql = QString( "select description, srs_id, upper(auth_name||':'||auth_id), is_geo, name, parameters, deprecated from vw_srs where %1 order by name,description" )
+                .arg( sqlFilter );
 
   rc = sqlite3_prepare( database, sql.toUtf8(), sql.toUtf8().length(), &stmt, &tail );
   // XXX Need to free memory from the error msg if one is set
@@ -639,7 +599,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> *crsFilter )
     // Cache some stuff to speed up creating of the list of projected
     // spatial reference systems
     QString previousSrsType( "" );
-    QTreeWidgetItem* previousSrsTypeNode = 0;
+    QTreeWidgetItem* previousSrsTypeNode = nullptr;
 
     while ( sqlite3_step( stmt ) == SQLITE_ROW )
     {
@@ -671,7 +631,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> *crsFilter )
         else
         { // Different from last one, need to search
           QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( srsType, Qt::MatchExactly | Qt::MatchRecursive, NAME_COLUMN );
-          if ( nodes.count() == 0 )
+          if ( nodes.isEmpty() )
           {
             // the node doesn't exist -- create it
             // Make in an italic font to distinguish them from real projections
@@ -737,7 +697,7 @@ void QgsProjectionSelector::on_lstCoordinateSystems_currentItemChanged( QTreeWid
     teSelected->setText( selectedName() );
 
     QList<QTreeWidgetItem*> nodes = lstRecent->findItems( current->text( QGIS_CRS_ID_COLUMN ), Qt::MatchExactly, QGIS_CRS_ID_COLUMN );
-    if ( nodes.count() > 0 )
+    if ( !nodes.isEmpty() )
     {
       QgsDebugMsg( QString( "found srs %1 in recent" ).arg( current->text( QGIS_CRS_ID_COLUMN ) ) );
       lstRecent->setCurrentItem( nodes.first() );
@@ -759,6 +719,24 @@ void QgsProjectionSelector::on_lstCoordinateSystems_currentItemChanged( QTreeWid
   }
 }
 
+void QgsProjectionSelector::on_lstCoordinateSystems_itemDoubleClicked( QTreeWidgetItem *current, int column )
+{
+  Q_UNUSED( column );
+
+  QgsDebugMsg( "Entered." );
+
+  if ( !current )
+  {
+    QgsDebugMsg( "no current item" );
+    return;
+  }
+
+  // If the item has children, it's not an end node in the tree, and
+  // hence is just a grouping thingy, not an actual CRS.
+  if ( current->childCount() == 0 )
+    emit projectionDoubleClicked();
+}
+
 void QgsProjectionSelector::on_lstRecent_currentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem * )
 {
   QgsDebugMsg( "Entered." );
@@ -772,10 +750,27 @@ void QgsProjectionSelector::on_lstRecent_currentItemChanged( QTreeWidgetItem *cu
   lstRecent->scrollToItem( current );
 
   QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( current->text( QGIS_CRS_ID_COLUMN ), Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
-  if ( nodes.count() > 0 )
+  if ( !nodes.isEmpty() )
     lstCoordinateSystems->setCurrentItem( nodes.first() );
 }
 
+void QgsProjectionSelector::on_lstRecent_itemDoubleClicked( QTreeWidgetItem *current, int column )
+{
+  Q_UNUSED( column );
+
+  QgsDebugMsg( "Entered." );
+
+  if ( !current )
+  {
+    QgsDebugMsg( "no current item" );
+    return;
+  }
+
+  QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( current->text( QGIS_CRS_ID_COLUMN ), Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
+  if ( !nodes.isEmpty() )
+    emit projectionDoubleClicked();
+}
+
 void QgsProjectionSelector::hideDeprecated( QTreeWidgetItem *item )
 {
   if ( item->data( 0, Qt::UserRole ).toBool() )
@@ -876,7 +871,7 @@ void QgsProjectionSelector::pushProjectionToFront()
 }
 
 
-long QgsProjectionSelector::getLargestCRSIDMatch( QString theSql )
+long QgsProjectionSelector::getLargestCRSIDMatch( const QString& theSql )
 {
   long srsId = 0;
 
@@ -895,7 +890,7 @@ long QgsProjectionSelector::getLargestCRSIDMatch( QString theSql )
   QString databaseFileName = QgsApplication::qgisUserDbFilePath();
   if ( QFileInfo( databaseFileName ).exists() ) //only bother trying to open if the file exists
   {
-    result = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+    result = sqlite3_open_v2( databaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
     if ( result )
     {
       // XXX This will likely never happen since on open, sqlite creates the
@@ -921,7 +916,7 @@ long QgsProjectionSelector::getLargestCRSIDMatch( QString theSql )
   else
   {
     //only bother looking in srs.db if it wasnt found above
-    result = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+    result = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
     if ( result )
     {
       QgsDebugMsg( QString( "Can't open * user * database: %1" ).arg( sqlite3_errmsg( database ) ) );
@@ -951,7 +946,7 @@ QStringList QgsProjectionSelector::authorities()
   const char   *tail;
   sqlite3_stmt *stmt;
 
-  int result = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, NULL );
+  int result = sqlite3_open_v2( mSrsDatabaseFileName.toUtf8().data(), &database, SQLITE_OPEN_READONLY, nullptr );
   if ( result )
   {
     QgsDebugMsg( QString( "Can't open * user * database: %1" ).arg( sqlite3_errmsg( database ) ) );
@@ -979,7 +974,7 @@ QStringList QgsProjectionSelector::authorities()
   return authorities;
 }
 
-/*!linfinity qtcreator qgis
+/** Linfinity qtcreator qgis
 * \brief Make the string safe for use in SQL statements.
 *  This involves escaping single quotes, double quotes, backslashes,
 *  and optionally, percentage symbols.  Percentage symbols are used
@@ -988,17 +983,17 @@ QStringList QgsProjectionSelector::authorities()
 * \arg const QString in The input string to make safe.
 * \return The string made safe for SQL statements.
 */
-const QString QgsProjectionSelector::sqlSafeString( const QString theSQL )
+const QString QgsProjectionSelector::sqlSafeString( const QString& theSQL )
 {
   QString retval = theSQL;
-  retval.replace( "\\", "\\\\" );
+  retval.replace( '\\', "\\\\" );
   retval.replace( '\"', "\\\"" );
-  retval.replace( "\'", "\\'" );
-  retval.replace( "%", "\\%" );
+  retval.replace( '\'', "\\'" );
+  retval.replace( '%', "\\%" );
   return retval;
 }
 
-void QgsProjectionSelector::showDBMissingWarning( const QString theFileName )
+void QgsProjectionSelector::showDBMissingWarning( const QString& theFileName )
 {
 
   QMessageBox::critical( this, tr( "Resource Location Error" ),
