@@ -24,10 +24,15 @@ QgsMapUnitScaleDialog::QgsMapUnitScaleDialog( QWidget* parent )
   setupUi( this );
   mComboBoxMinScale->setScale( 0.0000001 );
   mComboBoxMaxScale->setScale( 1 );
+  mSpinBoxMinSize->setShowClearButton( false );
+  mSpinBoxMaxSize->setShowClearButton( false );
   connect( mCheckBoxMinScale, SIGNAL( toggled( bool ) ), this, SLOT( configureMinComboBox() ) );
   connect( mCheckBoxMaxScale, SIGNAL( toggled( bool ) ), this, SLOT( configureMaxComboBox() ) );
   connect( mComboBoxMinScale, SIGNAL( scaleChanged() ), this, SLOT( configureMaxComboBox() ) );
   connect( mComboBoxMaxScale, SIGNAL( scaleChanged() ), this, SLOT( configureMinComboBox() ) );
+
+  connect( mCheckBoxMinSize, SIGNAL( toggled( bool ) ), mSpinBoxMinSize, SLOT( setEnabled( bool ) ) );
+  connect( mCheckBoxMaxSize, SIGNAL( toggled( bool ) ), mSpinBoxMaxSize, SLOT( setEnabled( bool ) ) );
 }
 
 void QgsMapUnitScaleDialog::setMapUnitScale( const QgsMapUnitScale &scale )
@@ -38,6 +43,22 @@ void QgsMapUnitScaleDialog::setMapUnitScale( const QgsMapUnitScale &scale )
   mComboBoxMaxScale->setScale( scale.maxScale > 0.0 ? scale.maxScale : 1.0 );
   mCheckBoxMaxScale->setChecked( scale.maxScale > 0.0 );
   mComboBoxMaxScale->setEnabled( scale.maxScale > 0.0 );
+
+  mCheckBoxMinSize->setChecked( scale.minSizeMMEnabled );
+  mSpinBoxMinSize->setEnabled( scale.minSizeMMEnabled );
+  mSpinBoxMinSize->setValue( scale.minSizeMM );
+
+  mCheckBoxMaxSize->setChecked( scale.maxSizeMMEnabled );
+  mSpinBoxMaxSize->setEnabled( scale.maxSizeMMEnabled );
+  mSpinBoxMaxSize->setValue( scale.maxSizeMM );
+}
+
+void QgsMapUnitScaleDialog::setMapCanvas( QgsMapCanvas *canvas )
+{
+  mComboBoxMinScale->setMapCanvas( canvas );
+  mComboBoxMinScale->setShowCurrentScaleButton( true );
+  mComboBoxMaxScale->setMapCanvas( canvas );
+  mComboBoxMaxScale->setShowCurrentScaleButton( true );
 }
 
 void QgsMapUnitScaleDialog::configureMinComboBox()
@@ -63,10 +84,13 @@ QgsMapUnitScale QgsMapUnitScaleDialog::getMapUnitScale() const
   QgsMapUnitScale scale;
   scale.minScale = mCheckBoxMinScale->isChecked() ? mComboBoxMinScale->scale() : 0;
   scale.maxScale = mCheckBoxMaxScale->isChecked() ? mComboBoxMaxScale->scale() : 0;
+  scale.minSizeMMEnabled = mCheckBoxMinSize->isChecked();
+  scale.minSizeMM = mSpinBoxMinSize->value();
+  scale.maxSizeMMEnabled = mCheckBoxMaxSize->isChecked();
+  scale.maxSizeMM = mSpinBoxMaxSize->value();
   return scale;
 }
 
-
 QgsUnitSelectionWidget::QgsUnitSelectionWidget( QWidget *parent )
     : QWidget( parent )
 {
@@ -93,6 +117,48 @@ void QgsUnitSelectionWidget::setUnits( const QStringList &units, int mapUnitIdx
   blockSignals( false );
 }
 
+void QgsUnitSelectionWidget::setUnits( const QgsSymbolV2::OutputUnitList &units )
+{
+  blockSignals( true );
+  mUnitCombo->clear();
+
+  //instead of iterating over the units list, we specifically check for presence of unit types
+  //to ensure that the widget always keeps the same order for units, regardless of the
+  //order specified in the units list
+  mMapUnitIdx = -1;
+  if ( units.contains( QgsSymbolV2::MM ) )
+  {
+    mUnitCombo->addItem( tr( "Millimeter" ), QgsSymbolV2::MM );
+  }
+  if ( units.contains( QgsSymbolV2::Pixel ) )
+  {
+    mUnitCombo->addItem( tr( "Pixels" ), QgsSymbolV2::Pixel );
+  }
+  if ( units.contains( QgsSymbolV2::MapUnit ) )
+  {
+    mUnitCombo->addItem( tr( "Map unit" ), QgsSymbolV2::MapUnit );
+  }
+  if ( units.contains( QgsSymbolV2::Percentage ) )
+  {
+    mUnitCombo->addItem( tr( "Percentage" ), QgsSymbolV2::Percentage );
+  }
+  blockSignals( false );
+}
+
+QgsSymbolV2::OutputUnit QgsUnitSelectionWidget::unit() const
+{
+  if ( mUnitCombo->count() == 0 )
+    return QgsSymbolV2::Mixed;
+
+  QVariant currentData = mUnitCombo->itemData( mUnitCombo->currentIndex() );
+  if ( currentData.isValid() )
+  {
+    return ( QgsSymbolV2::OutputUnit ) currentData.toInt();
+  }
+  //unknown
+  return QgsSymbolV2::Mixed;
+}
+
 void QgsUnitSelectionWidget::setUnit( int unitIndex )
 {
   blockSignals( true );
@@ -100,6 +166,17 @@ void QgsUnitSelectionWidget::setUnit( int unitIndex )
   blockSignals( false );
 }
 
+void QgsUnitSelectionWidget::setUnit( QgsSymbolV2::OutputUnit unit )
+{
+  int idx = mUnitCombo->findData( QVariant(( int ) unit ) );
+  mUnitCombo->setCurrentIndex( idx == -1 ? 0 : idx );
+}
+
+void QgsUnitSelectionWidget::setMapCanvas( QgsMapCanvas *canvas )
+{
+  mUnitScaleDialog->setMapCanvas( canvas );
+}
+
 void QgsUnitSelectionWidget::showDialog()
 {
   QgsMapUnitScale scale = mUnitScaleDialog->getMapUnitScale();
@@ -110,7 +187,7 @@ void QgsUnitSelectionWidget::showDialog()
   else
   {
     QgsMapUnitScale newScale = mUnitScaleDialog->getMapUnitScale();
-    if ( scale.minScale != newScale.minScale || scale.maxScale != newScale.maxScale )
+    if ( scale != newScale )
     {
       emit changed();
     }
@@ -119,8 +196,13 @@ void QgsUnitSelectionWidget::showDialog()
 
 void QgsUnitSelectionWidget::toggleUnitRangeButton()
 {
-  mMapScaleButton->setVisible( mMapUnitIdx != -1 && mUnitCombo->currentIndex() == mMapUnitIdx );
+  if ( unit() != QgsSymbolV2::Mixed )
+  {
+    mMapScaleButton->setVisible( unit() == QgsSymbolV2::MapUnit );
+  }
+  else
+  {
+    mMapScaleButton->setVisible( mMapUnitIdx != -1 && mUnitCombo->currentIndex() == mMapUnitIdx );
+  }
 }
 
-
-
