@@ -21,7 +21,7 @@
 
 QgsSingleBandGrayRendererWidget::QgsSingleBandGrayRendererWidget( QgsRasterLayer* layer, const QgsRectangle &extent )
     : QgsRasterRendererWidget( layer, extent )
-    , mMinMaxWidget( NULL )
+    , mMinMaxWidget( nullptr )
 {
   setupUi( this );
 
@@ -75,12 +75,12 @@ QgsRasterRenderer* QgsSingleBandGrayRendererWidget::renderer()
 {
   if ( !mRasterLayer )
   {
-    return 0;
+    return nullptr;
   }
   QgsRasterDataProvider* provider = mRasterLayer->dataProvider();
   if ( !provider )
   {
-    return 0;
+    return nullptr;
   }
   int band = mGrayBandComboBox->itemData( mGrayBandComboBox->currentIndex() ).toInt();
 
