@@ -25,11 +25,13 @@ class QgsRendererCategoryV2;
 
 #include "ui_qgscategorizedsymbolrendererv2widget.h"
 
+///@cond PRIVATE
+
 class GUI_EXPORT QgsCategorizedSymbolRendererV2Model : public QAbstractItemModel
 {
     Q_OBJECT
   public:
-    QgsCategorizedSymbolRendererV2Model( QObject * parent = 0 );
+    QgsCategorizedSymbolRendererV2Model( QObject * parent = nullptr );
     Qt::ItemFlags flags( const QModelIndex & index ) const override;
     Qt::DropActions supportedDropActions() const override;
     QVariant data( const QModelIndex &index, int role ) const override;
@@ -63,12 +65,16 @@ class GUI_EXPORT QgsCategorizedSymbolRendererV2Model : public QAbstractItemModel
 // View style which shows drop indicator line between items
 class QgsCategorizedSymbolRendererV2ViewStyle: public QProxyStyle
 {
-  public:
-    QgsCategorizedSymbolRendererV2ViewStyle( QStyle* style = 0 );
+    Q_OBJECT
 
-    void drawPrimitive( PrimitiveElement element, const QStyleOption * option, QPainter * painter, const QWidget * widget = 0 ) const override;
+  public:
+    explicit QgsCategorizedSymbolRendererV2ViewStyle( QStyle* style = nullptr );
+
+    void drawPrimitive( PrimitiveElement element, const QStyleOption * option, QPainter * painter, const QWidget * widget = nullptr ) const override;
 };
 
+///@endcond
+
 class GUI_EXPORT QgsCategorizedSymbolRendererV2Widget : public QgsRendererV2Widget, private Ui::QgsCategorizedSymbolRendererV2Widget
 {
     Q_OBJECT
@@ -80,9 +86,19 @@ class GUI_EXPORT QgsCategorizedSymbolRendererV2Widget : public QgsRendererV2Widg
 
     virtual QgsFeatureRendererV2* renderer() override;
 
+    /** Replaces category symbols with the symbols from a style that have a matching
+     * name.
+     * @param style style containing symbols to match with
+     * @return number of symbols matched
+     * @see matchToSymbolsFromLibrary
+     * @see matchToSymbolsFromXml
+     * @note added in QGIS 2.9
+     */
+    int matchToSymbols( QgsStyleV2* style );
+
   public slots:
     void changeCategorizedSymbol();
-    void categoryColumnChanged( QString field );
+    void categoryColumnChanged( const QString& field );
     void categoriesDoubleClicked( const QModelIndex & idx );
     void addCategory();
     void addCategories();
@@ -90,14 +106,29 @@ class GUI_EXPORT QgsCategorizedSymbolRendererV2Widget : public QgsRendererV2Widg
     void deleteCategories();
     void deleteAllCategories();
 
-    void rotationFieldChanged( QString fldName );
-    void sizeScaleFieldChanged( QString fldName );
+    void sizeScaleFieldChanged( const QString& fldName );
     void scaleMethodChanged( QgsSymbolV2::ScaleMethod scaleMethod );
 
     void showSymbolLevels();
 
     void rowsMoved();
 
+    /** Replaces category symbols with the symbols from the users' symbol library that have a
+     * matching name.
+     * @see matchToSymbolsFromXml
+     * @see matchToSymbols
+     * @note added in QGIS 2.9
+     */
+    void matchToSymbolsFromLibrary();
+
+    /** Prompts for selection of an xml file, then replaces category symbols with the symbols
+     * from the XML file with a matching name.
+     * @see matchToSymbolsFromLibrary
+     * @see matchToSymbols
+     * @note added in QGIS 2.9
+     */
+    void matchToSymbolsFromXml();
+
   protected:
 
     void updateUiFromRenderer();
@@ -130,8 +161,6 @@ class GUI_EXPORT QgsCategorizedSymbolRendererV2Widget : public QgsRendererV2Widg
 
     QgsSymbolV2* mCategorizedSymbol;
 
-    QgsRendererV2DataDefinedMenus* mDataDefinedMenus;
-
     QgsCategorizedSymbolRendererV2Model* mModel;
 
   private:
