@@ -16,18 +16,21 @@
 #include "qgssymbolv2.h"
 #include "qgsvectorlayer.h"
 #include "qgscolordialog.h"
+#include "qgssymbollevelsv2dialog.h"
+#include "qgsexpressionbuilderdialog.h"
+#include "qgsmapcanvas.h"
+
 #include <QMessageBox>
 #include <QInputDialog>
 #include <QMenu>
 
-#include "qgssymbollevelsv2dialog.h"
-#include "qgsexpressionbuilderdialog.h"
-
-
 QgsRendererV2Widget::QgsRendererV2Widget( QgsVectorLayer* layer, QgsStyleV2* style )
-    : QWidget(), mLayer( layer ), mStyle( style )
+    : QWidget()
+    , mLayer( layer )
+    , mStyle( style )
+    , mMapCanvas( nullptr )
 {
-  contextMenu = new QMenu( "Renderer Options " );
+  contextMenu = new QMenu( tr( "Renderer Options" ), this );
 
   mCopyAction = contextMenu->addAction( tr( "Copy" ), this, SLOT( copy() ) );
   mCopyAction->setShortcut( QKeySequence( QKeySequence::Copy ) );
@@ -46,10 +49,11 @@ QgsRendererV2Widget::QgsRendererV2Widget( QgsVectorLayer* layer, QgsStyleV2* sty
   else if ( mLayer && mLayer->geometryType() == QGis::Point )
   {
     contextMenu->addAction( tr( "Change size" ), this, SLOT( changeSymbolSize() ) );
+    contextMenu->addAction( tr( "Change angle" ), this, SLOT( changeSymbolAngle() ) );
   }
 }
 
-void QgsRendererV2Widget::contextMenuViewCategories( const QPoint & )
+void QgsRendererV2Widget::contextMenuViewCategories( QPoint )
 {
   contextMenu->exec( QCursor::pos() );
 }
@@ -57,18 +61,30 @@ void QgsRendererV2Widget::contextMenuViewCategories( const QPoint & )
 void QgsRendererV2Widget::changeSymbolColor()
 {
   QList<QgsSymbolV2*> symbolList = selectedSymbols();
-  if ( symbolList.size() < 1 )
+  if ( symbolList.isEmpty() )
   {
     return;
   }
 
-  QColor color = QgsColorDialogV2::getColor( symbolList.at( 0 )->color(), this, "Change Symbol Color", true );
+  QgsSymbolV2* firstSymbol = nullptr;
+  Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+  {
+    if ( symbol )
+    {
+      firstSymbol = symbol;
+      break;
+    }
+  }
+  if ( !firstSymbol )
+    return;
+
+  QColor color = QgsColorDialogV2::getColor( firstSymbol->color(), this, "Change Symbol Color", true );
   if ( color.isValid() )
   {
-    QList<QgsSymbolV2*>::iterator symbolIt = symbolList.begin();
-    for ( ; symbolIt != symbolList.end(); ++symbolIt )
+    Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
     {
-      ( *symbolIt )->setColor( color );
+      if ( symbol )
+        symbol->setColor( color );
     }
     refreshSymbolView();
   }
@@ -77,20 +93,32 @@ void QgsRendererV2Widget::changeSymbolColor()
 void QgsRendererV2Widget::changeSymbolTransparency()
 {
   QList<QgsSymbolV2*> symbolList = selectedSymbols();
-  if ( symbolList.size() < 1 )
+  if ( symbolList.isEmpty() )
   {
     return;
   }
 
+  QgsSymbolV2* firstSymbol = nullptr;
+  Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+  {
+    if ( symbol )
+    {
+      firstSymbol = symbol;
+      break;
+    }
+  }
+  if ( !firstSymbol )
+    return;
+
   bool ok;
-  double oldTransparency = ( 1 - symbolList.at( 0 )->alpha() ) * 100; // convert to percents
+  double oldTransparency = ( 1 - firstSymbol->alpha() ) * 100; // convert to percents
   double transparency = QInputDialog::getDouble( this, tr( "Transparency" ), tr( "Change symbol transparency [%]" ), oldTransparency, 0.0, 100.0, 0, &ok );
   if ( ok )
   {
-    QList<QgsSymbolV2*>::iterator symbolIt = symbolList.begin();
-    for ( ; symbolIt != symbolList.end(); ++symbolIt )
+    Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
     {
-      ( *symbolIt )->setAlpha( 1 - transparency / 100 );
+      if ( symbol )
+        symbol->setAlpha( 1 - transparency / 100 );
     }
     refreshSymbolView();
   }
@@ -99,22 +127,34 @@ void QgsRendererV2Widget::changeSymbolTransparency()
 void QgsRendererV2Widget::changeSymbolUnit()
 {
   QList<QgsSymbolV2*> symbolList = selectedSymbols();
-  if ( symbolList.size() < 1 )
+  if ( symbolList.isEmpty() )
   {
     return;
   }
 
+  QgsSymbolV2* firstSymbol = nullptr;
+  Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+  {
+    if ( symbol )
+    {
+      firstSymbol = symbol;
+      break;
+    }
+  }
+  if ( !firstSymbol )
+    return;
+
   bool ok;
-  int currentUnit = ( symbolList.at( 0 )->outputUnit() == QgsSymbolV2::MM ) ? 0 : 1;
+  int currentUnit = ( firstSymbol->outputUnit() == QgsSymbolV2::MM ) ? 0 : 1;
   QString item = QInputDialog::getItem( this, tr( "Symbol unit" ), tr( "Select symbol unit" ), QStringList() << tr( "Millimeter" ) << tr( "Map unit" ), currentUnit, false, &ok );
   if ( ok )
   {
     QgsSymbolV2::OutputUnit unit = ( item.compare( tr( "Millimeter" ) ) == 0 ) ? QgsSymbolV2::MM : QgsSymbolV2::MapUnit;
 
-    QList<QgsSymbolV2*>::iterator symbolIt = symbolList.begin();
-    for ( ; symbolIt != symbolList.end(); ++symbolIt )
+    Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
     {
-      ( *symbolIt )->setOutputUnit( unit );
+      if ( symbol )
+        symbol->setOutputUnit( unit );
     }
     refreshSymbolView();
   }
@@ -123,22 +163,26 @@ void QgsRendererV2Widget::changeSymbolUnit()
 void QgsRendererV2Widget::changeSymbolWidth()
 {
   QList<QgsSymbolV2*> symbolList = selectedSymbols();
-  if ( symbolList.size() < 1 )
+  if ( symbolList.isEmpty() )
   {
     return;
   }
 
-  bool ok;
-  QgsLineSymbolV2* line = dynamic_cast<QgsLineSymbolV2*>( symbolList.at( 0 ) ) ;
-  double width = QInputDialog::getDouble( this, tr( "Width" ), tr( "Change symbol width" ), line ? line->width() : 0.0 , 0.0, 999999, 1, &ok );
-  if ( ok )
+  QgsDataDefinedWidthDialog dlg( symbolList, mLayer );
+  dlg.setMapCanvas( mMapCanvas );
+
+  if ( QDialog::Accepted == dlg.exec() )
   {
-    QList<QgsSymbolV2*>::iterator symbolIt = symbolList.begin();
-    for ( ; symbolIt != symbolList.end(); ++symbolIt )
+    if ( !dlg.mDDBtn->isActive() )
     {
-      line = dynamic_cast<QgsLineSymbolV2*>( *symbolIt );
-      if ( line )
-        line->setWidth( width );
+      Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+      {
+        if ( !symbol )
+          continue;
+
+        if ( symbol->type() == QgsSymbolV2::Line )
+          static_cast<QgsLineSymbolV2*>( symbol )->setWidth( dlg.mSpinBox->value() );
+      }
     }
     refreshSymbolView();
   }
@@ -147,23 +191,54 @@ void QgsRendererV2Widget::changeSymbolWidth()
 void QgsRendererV2Widget::changeSymbolSize()
 {
   QList<QgsSymbolV2*> symbolList = selectedSymbols();
-  if ( symbolList.size() < 1 )
+  if ( symbolList.isEmpty() )
   {
     return;
   }
 
-  bool ok;
-  QgsMarkerSymbolV2* marker = dynamic_cast<QgsMarkerSymbolV2*>( symbolList.at( 0 ) );
+  QgsDataDefinedSizeDialog dlg( symbolList, mLayer );
+  dlg.setMapCanvas( mMapCanvas );
 
-  double size = QInputDialog::getDouble( this, tr( "Size" ), tr( "Change symbol size" ), marker ? marker->size() : 0.0 , 0.0, 999999, 1, &ok );
-  if ( ok )
+  if ( QDialog::Accepted == dlg.exec() )
   {
-    QList<QgsSymbolV2*>::iterator symbolIt = symbolList.begin();
-    for ( ; symbolIt != symbolList.end(); ++symbolIt )
+    if ( !dlg.mDDBtn->isActive() )
     {
-      marker = dynamic_cast<QgsMarkerSymbolV2*>( *symbolIt );
-      if ( marker )
-        marker->setSize( size );
+      Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+      {
+        if ( !symbol )
+          continue;
+
+        if ( symbol->type() == QgsSymbolV2::Marker )
+          static_cast<QgsMarkerSymbolV2*>( symbol )->setSize( dlg.mSpinBox->value() );
+      }
+    }
+    refreshSymbolView();
+  }
+}
+
+void QgsRendererV2Widget::changeSymbolAngle()
+{
+  QList<QgsSymbolV2*> symbolList = selectedSymbols();
+  if ( symbolList.isEmpty() )
+  {
+    return;
+  }
+
+  QgsDataDefinedRotationDialog dlg( symbolList, mLayer );
+  dlg.setMapCanvas( mMapCanvas );
+
+  if ( QDialog::Accepted == dlg.exec() )
+  {
+    if ( !dlg.mDDBtn->isActive() )
+    {
+      Q_FOREACH ( QgsSymbolV2* symbol, symbolList )
+      {
+        if ( !symbol )
+          continue;
+
+        if ( symbol->type() == QgsSymbolV2::Marker )
+          static_cast<QgsMarkerSymbolV2*>( symbol )->setAngle( dlg.mSpinBox->value() );
+      }
     }
     refreshSymbolView();
   }
@@ -181,14 +256,27 @@ void QgsRendererV2Widget::showSymbolLevelsDialog( QgsFeatureRendererV2* r )
   }
 }
 
+void QgsRendererV2Widget::setMapCanvas( QgsMapCanvas *canvas )
+{
+  mMapCanvas = canvas;
+}
+
+const QgsMapCanvas* QgsRendererV2Widget::mapCanvas() const
+{
+  return mMapCanvas;
+}
+
+void QgsRendererV2Widget::applyChanges()
+{
+  apply();
+}
+
 
 ////////////
 
-//#include <QAction>
 #include "qgsfield.h"
-#include <QMenu>
 
-QgsRendererV2DataDefinedMenus::QgsRendererV2DataDefinedMenus( QMenu* menu, QgsVectorLayer* layer, QString rotationField, QString sizeScaleField, QgsSymbolV2::ScaleMethod scaleMethod )
+QgsRendererV2DataDefinedMenus::QgsRendererV2DataDefinedMenus( QMenu* menu, QgsVectorLayer* layer, const QString& rotationField, const QString& sizeScaleField, QgsSymbolV2::ScaleMethod scaleMethod )
     : QObject( menu ), mLayer( layer )
 {
   mRotationMenu = new QMenu( tr( "Rotation field" ) );
@@ -220,8 +308,11 @@ QgsRendererV2DataDefinedMenus::QgsRendererV2DataDefinedMenus( QMenu* menu, QgsVe
 
   mSizeScaleMenu->addActions( mSizeMethodActionGroup->actions() );
 
-  menu->addMenu( mRotationMenu );
-  menu->addMenu( mSizeScaleMenu );
+  //@todo cleanup the class since Rotation and SizeScale are now
+  //defined using QgsDataDefinedButton
+  //
+  //menu->addMenu( mRotationMenu );
+  //menu->addMenu( mSizeScaleMenu );
 
   connect( mSizeMethodActionGroup, SIGNAL( triggered( QAction* ) ), this, SLOT( scaleMethodSelected( QAction* ) ) );
   connect( mRotationAttributeActionGroup, SIGNAL( triggered( QAction* ) ), this, SLOT( rotationFieldSelected( QAction* ) ) );
@@ -237,7 +328,7 @@ QgsRendererV2DataDefinedMenus::~QgsRendererV2DataDefinedMenus()
   delete mSizeScaleMenu;
 }
 
-void QgsRendererV2DataDefinedMenus::populateMenu( QMenu* menu, QString fieldName, QActionGroup *actionGroup )
+void QgsRendererV2DataDefinedMenus::populateMenu( QMenu* menu, const QString& fieldName, QActionGroup *actionGroup )
 {
   QAction* aExpr = new QAction( tr( "- expression -" ), actionGroup );
   aExpr->setCheckable( true );
@@ -249,7 +340,7 @@ void QgsRendererV2DataDefinedMenus::populateMenu( QMenu* menu, QString fieldName
   menu->addSeparator();
 
   bool hasField = false;
-  const QgsFields & flds = mLayer->pendingFields();
+  const QgsFields & flds = mLayer->fields();
   for ( int idx = 0; idx < flds.count(); ++idx )
   {
     const QgsField& fld = flds[idx];
@@ -283,7 +374,7 @@ void QgsRendererV2DataDefinedMenus::populateMenu( QMenu* menu, QString fieldName
 
 void QgsRendererV2DataDefinedMenus::rotationFieldSelected( QAction* a )
 {
-  if ( a == NULL )
+  if ( !a )
     return;
 
   QString fldName = a->text();
@@ -310,7 +401,7 @@ void QgsRendererV2DataDefinedMenus::rotationFieldSelected( QAction* a )
 
 void QgsRendererV2DataDefinedMenus::sizeScaleFieldSelected( QAction* a )
 {
-  if ( a == NULL )
+  if ( !a )
     return;
 
   QString fldName = a->text();
@@ -337,7 +428,7 @@ void QgsRendererV2DataDefinedMenus::sizeScaleFieldSelected( QAction* a )
 
 void QgsRendererV2DataDefinedMenus::scaleMethodSelected( QAction* a )
 {
-  if ( a == NULL )
+  if ( !a )
     return;
 
   if ( a->text() == tr( "Scale area" ) )
@@ -352,9 +443,150 @@ void QgsRendererV2DataDefinedMenus::scaleMethodSelected( QAction* a )
 #if 0 // MK: is there any reason for this?
 void QgsRendererV2DataDefinedMenus::updateMenu( QActionGroup* actionGroup, QString fieldName )
 {
-  foreach ( QAction* a, actionGroup->actions() )
+  Q_FOREACH ( QAction* a, actionGroup->actions() )
   {
     a->setChecked( a->text() == fieldName );
   }
 }
 #endif
+
+QgsDataDefinedValueDialog::QgsDataDefinedValueDialog( const QList<QgsSymbolV2*>& symbolList, QgsVectorLayer * layer, const QString & label )
+    : mSymbolList( symbolList )
+    , mLayer( layer )
+    , mMapCanvas( nullptr )
+{
+  setupUi( this );
+  setWindowFlags( Qt::WindowStaysOnTopHint );
+  mLabel->setText( label );
+  connect( mDDBtn, SIGNAL( dataDefinedChanged( const QString& ) ), this, SLOT( dataDefinedChanged() ) );
+  connect( mDDBtn, SIGNAL( dataDefinedActivated( bool ) ), this, SLOT( dataDefinedChanged() ) );
+
+}
+
+void QgsDataDefinedValueDialog::setMapCanvas( QgsMapCanvas *canvas )
+{
+  mMapCanvas = canvas;
+  Q_FOREACH ( QgsDataDefinedButton* ddButton, findChildren<QgsDataDefinedButton*>() )
+  {
+    if ( ddButton->assistant() )
+      ddButton->assistant()->setMapCanvas( mMapCanvas );
+  }
+}
+
+const QgsMapCanvas *QgsDataDefinedValueDialog::mapCanvas() const
+{
+  return mMapCanvas;
+}
+
+static QgsExpressionContext _getExpressionContext( const void* context )
+{
+  const QgsDataDefinedValueDialog* widget = ( const QgsDataDefinedValueDialog* ) context;
+
+  QgsExpressionContext expContext;
+  expContext << QgsExpressionContextUtils::globalScope()
+  << QgsExpressionContextUtils::projectScope()
+  << QgsExpressionContextUtils::atlasScope( nullptr );
+  if ( widget->mapCanvas() )
+  {
+    expContext << QgsExpressionContextUtils::mapSettingsScope( widget->mapCanvas()->mapSettings() )
+    << new QgsExpressionContextScope( widget->mapCanvas()->expressionContextScope() );
+  }
+  else
+  {
+    expContext << QgsExpressionContextUtils::mapSettingsScope( QgsMapSettings() );
+  }
+
+  if ( widget->vectorLayer() )
+    expContext << QgsExpressionContextUtils::layerScope( widget->vectorLayer() );
+
+  return expContext;
+}
+
+void QgsDataDefinedValueDialog::init( const QString & description )
+{
+  QgsDataDefined dd = symbolDataDefined();
+  mDDBtn->init( mLayer, &dd, QgsDataDefinedButton::Double, description );
+  mDDBtn->registerGetExpressionContextCallback( &_getExpressionContext, this );
+
+  QgsSymbolV2* initialSymbol = nullptr;
+  Q_FOREACH ( QgsSymbolV2* symbol, mSymbolList )
+  {
+    if ( symbol )
+    {
+      initialSymbol = symbol;
+    }
+  }
+  mSpinBox->setValue( initialSymbol ? value( initialSymbol ) : 0 );
+  mSpinBox->setEnabled( !mDDBtn->isActive() );
+}
+
+QgsDataDefined QgsDataDefinedValueDialog::symbolDataDefined() const
+{
+  if ( mSymbolList.isEmpty() || !mSymbolList.back() )
+    return QgsDataDefined();
+
+  // check that all symbols share the same size expression
+  QgsDataDefined dd = symbolDataDefined( mSymbolList.back() );
+  Q_FOREACH ( QgsSymbolV2 * it, mSymbolList )
+  {
+    if ( !it || symbolDataDefined( it ) != dd )
+      return QgsDataDefined();
+  }
+  return dd;
+}
+
+void QgsDataDefinedValueDialog::dataDefinedChanged()
+{
+  QgsDataDefined dd = mDDBtn->currentDataDefined();
+  mSpinBox->setEnabled( !dd.isActive() );
+
+  if ( // shall we remove datadefined expressions for layers ?
+    ( symbolDataDefined().isActive() && !dd.isActive() )
+    // shall we set the "en masse" expression for properties ?
+    || dd.isActive() )
+  {
+    Q_FOREACH ( QgsSymbolV2 * it, mSymbolList )
+      setDataDefined( it, dd );
+  }
+}
+
+QgsDataDefined QgsDataDefinedSizeDialog::symbolDataDefined( const QgsSymbolV2 *symbol ) const
+{
+  const QgsMarkerSymbolV2* marker = static_cast<const QgsMarkerSymbolV2*>( symbol );
+  return marker->dataDefinedSize();
+}
+
+void QgsDataDefinedSizeDialog::setDataDefined( QgsSymbolV2* symbol, const QgsDataDefined& dd )
+{
+  static_cast<QgsMarkerSymbolV2*>( symbol )->setDataDefinedSize( dd );
+  static_cast<QgsMarkerSymbolV2*>( symbol )->setScaleMethod( QgsSymbolV2::ScaleDiameter );
+}
+
+
+QgsDataDefined QgsDataDefinedRotationDialog::symbolDataDefined( const QgsSymbolV2 *symbol ) const
+{
+  const QgsMarkerSymbolV2* marker = static_cast<const QgsMarkerSymbolV2*>( symbol );
+  return marker->dataDefinedAngle();
+}
+
+void QgsDataDefinedRotationDialog::setDataDefined( QgsSymbolV2 *symbol, const QgsDataDefined &dd )
+{
+  static_cast<QgsMarkerSymbolV2*>( symbol )->setDataDefinedAngle( dd );
+}
+
+
+QgsDataDefined QgsDataDefinedWidthDialog::symbolDataDefined( const QgsSymbolV2 *symbol ) const
+{
+  const QgsLineSymbolV2* line = static_cast<const QgsLineSymbolV2*>( symbol );
+  return line->dataDefinedWidth();
+}
+
+void QgsDataDefinedWidthDialog::setDataDefined( QgsSymbolV2 *symbol, const QgsDataDefined &dd )
+{
+  static_cast<QgsLineSymbolV2*>( symbol )->setDataDefinedWidth( dd );
+}
+
+void QgsRendererV2Widget::apply()
+{
+
+}
