@@ -58,14 +58,14 @@ eVisDatabaseConnectionGui::eVisDatabaseConnectionGui( QList<QTemporaryFile*>* th
 
   //Initialize varaibles
   mQueryDefinitionMap = new QMap<int, eVisQueryDefinition>;
-  mDatabaseConnection = 0;
+  mDatabaseConnection = nullptr;
 
   //Create a new instance of the file selector
   mDatabaseLayerFieldSelector = new eVisDatabaseLayerFieldSelectionGui( this, fl );
   connect( mDatabaseLayerFieldSelector, SIGNAL( eVisDatabaseLayerFieldsSelected( QString, QString, QString ) ), this, SLOT( drawNewVectorLayer( QString, QString, QString ) ) );
 
   //Populate gui components
-#ifdef WIN32
+#ifdef Q_OS_WIN
   cboxDatabaseType->insertItem( 0, "MSAccess" );
 #endif
   cboxDatabaseType->insertItem( 0, "MYSQL" );
@@ -105,10 +105,10 @@ eVisDatabaseConnectionGui::~eVisDatabaseConnectionGui()
 * @param xCoordinate - Name of the field containing the x coordinate
 * @param yCoordinate - Name of the field containing the y coordinate
 */
-void eVisDatabaseConnectionGui::drawNewVectorLayer( QString layerName, QString xCoordinate, QString yCoordinate )
+void eVisDatabaseConnectionGui::drawNewVectorLayer( const QString& layerName, const QString& xCoordinate, const QString& yCoordinate )
 {
   //if coorindate fields are defined, load as a delimited text layer
-  if ( !xCoordinate.isEmpty() && !yCoordinate.isEmpty() && mTempOutputFileList->size() > 0 )
+  if ( !xCoordinate.isEmpty() && !yCoordinate.isEmpty() && !mTempOutputFileList->isEmpty() )
   {
     //fileName is only available if the file is open
     //the last file in the list is always the newest
@@ -129,18 +129,18 @@ void eVisDatabaseConnectionGui::drawNewVectorLayer( QString layerName, QString x
 void eVisDatabaseConnectionGui::on_buttonBox_accepted()
 {
   //Deallocate memory, basically a predescructor
-  if ( 0 != mDatabaseConnection )
+  if ( mDatabaseConnection )
   {
     mDatabaseConnection->close();
     delete( mDatabaseConnection );
   }
 
-  if ( 0 != mDatabaseLayerFieldSelector )
+  if ( mDatabaseLayerFieldSelector )
   {
     delete( mDatabaseLayerFieldSelector );
   }
 
-  if ( 0 != mQueryDefinitionMap )
+  if ( mQueryDefinitionMap )
   {
     mQueryDefinitionMap->clear();
     delete mQueryDefinitionMap;
@@ -273,7 +273,7 @@ void eVisDatabaseConnectionGui::on_pbtnConnect_clicked()
     }
 
     //If there is aready a database connection object, reset with the current parameters
-    if ( 0 != mDatabaseConnection )
+    if ( mDatabaseConnection )
     {
       mDatabaseConnection->resetConnectionParameters( leDatabaseHost->text(), leDatabasePort->text().toInt(), leDatabaseName->text(), leDatabaseUsername->text(), leDatabasePassword->text(), myDatabaseType );
     }
@@ -285,11 +285,11 @@ void eVisDatabaseConnectionGui::on_pbtnConnect_clicked()
     //Try to connect the database connection object
     if ( mDatabaseConnection->connect() )
     {
-      teditConsole->append( tr( "Connection to [%1.%2] established" ).arg( leDatabaseHost->text() ).arg( leDatabaseName->text() ) );
+      teditConsole->append( tr( "Connection to [%1.%2] established" ).arg( leDatabaseHost->text(), leDatabaseName->text() ) );
       lblConnectionStatus->setText( tr( "connected" ) );
 
       //List the tables in the database
-      teditConsole->append( tr( "Tables" ) + ":" );
+      teditConsole->append( tr( "Tables" ) + ':' );
       QStringList myTableList = mDatabaseConnection->tables();
       for ( int myIterator = 0; myIterator < myTableList.size(); myIterator++ )
       {
@@ -299,7 +299,7 @@ void eVisDatabaseConnectionGui::on_pbtnConnect_clicked()
     else
     {
       teditConsole->append( tr( "Connection to [%1.%2] failed: %3" )
-                            .arg( leDatabaseHost->text() ).arg( leDatabaseName->text() ).arg( mDatabaseConnection->lastError() ) );
+                            .arg( leDatabaseHost->text(), leDatabaseName->text(), mDatabaseConnection->lastError() ) );
     }
   }
 }
@@ -313,7 +313,7 @@ void eVisDatabaseConnectionGui::on_pbtnLoadPredefinedQueries_clicked()
   //There probably needs to be some more error checking, but works for now.
 
   //Select the XML file to parse
-  QString myFilename = QFileDialog::getOpenFileName( this, tr( "Open File" ), ".", "XML ( *.xml )" );
+  QString myFilename = QFileDialog::getOpenFileName( this, tr( "Open File" ), QDir::homePath(), "XML ( *.xml )" );
   if ( myFilename != "" )
   {
     //Display the name of the file being parsed
@@ -453,9 +453,9 @@ void eVisDatabaseConnectionGui::on_cboxPredefinedQueryList_currentIndexChanged(
 void eVisDatabaseConnectionGui::on_pbtnOpenFile_clicked()
 {
   if ( cboxDatabaseType->currentText() == "MSAccess" )
-    leDatabaseName->setText( QFileDialog::getOpenFileName( this, tr( "Open File" ), ".", "MSAccess ( *.mdb )" ) );
+    leDatabaseName->setText( QFileDialog::getOpenFileName( this, tr( "Open File" ), QDir::homePath(), "MSAccess ( *.mdb )" ) );
   else
-    leDatabaseName->setText( QFileDialog::getOpenFileName( this, tr( "Open File" ), ".", "Sqlite ( *.db )" ) );
+    leDatabaseName->setText( QFileDialog::getOpenFileName( this, tr( "Open File" ), QDir::homePath(), "Sqlite ( *.db )" ) );
 }
 
 /**
@@ -467,11 +467,11 @@ void eVisDatabaseConnectionGui::on_pbtnRunQuery_clicked()
   if ( !teditSqlStatement->toPlainText().isEmpty() )
   {
     //Verify that we have an active database connection
-    if ( 0 != mDatabaseConnection )
+    if ( mDatabaseConnection )
     {
       //Execute query
       QSqlQuery* myResults = mDatabaseConnection->query( teditSqlStatement->toPlainText() );
-      if ( 0 == myResults )
+      if ( !myResults )
       {
         teditConsole->append( tr( "Error: Query failed: %1" ).arg( mDatabaseConnection->lastError() ) );
       }
