@@ -140,9 +140,19 @@ QgsGrassMapcalc::QgsGrassMapcalc(
   tb->addAction( mActionSaveAs );
   connect( mActionSaveAs, SIGNAL( triggered() ), this, SLOT( saveAs() ) );
 
+  // Map input
+  mMapComboBox = new QgsGrassModuleInputComboBox( QgsGrassObject::Raster, this );
+  mMapComboBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy:: Preferred );
+  // QComboBox does not emit activated() when item is selected in completer popup
+  connect( mMapComboBox, SIGNAL( activated( const QString & ) ), this, SLOT( mapChanged( const QString & ) ) );
+  connect( mMapComboBox->completer(), SIGNAL( activated( const QString & ) ), this, SLOT( mapChanged( const QString & ) ) );
+  connect( mMapComboBox, SIGNAL( editTextChanged( const QString & ) ), this, SLOT( mapChanged( const QString & ) ) );
+  bool firstSet = mMapComboBox->setFirst();
+  Q_UNUSED( firstSet );
+  mInputFrame->layout()->addWidget( mMapComboBox );
+
   /* Create functions */
   int t = QgsGrassMapcalcFunction::Operator;
-  //mFunctions.push_back(QgsGrassMapcalcFunction("-",2, "Odcitani", "in1,in2" ));
   // Arithmetical
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "+", 2, tr( "Addition" ) ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "-", 2, tr( "Subtraction" ) ) );
@@ -171,7 +181,7 @@ QgsGrassMapcalc::QgsGrassMapcalc(
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "ewres", 0, tr( "Current east-west resolution" ) ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "exp", 1, tr( "Exponential function of x" ), "exp(x)" ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "exp", 2, tr( "x to the power y" ), "exp(x,y)" ) );
-  mFunctions.push_back( QgsGrassMapcalcFunction( t, "float", 2, tr( "Convert x to single-precision floating point" ), "float(x)" ) );
+  mFunctions.push_back( QgsGrassMapcalcFunction( t, "float", 1, tr( "Convert x to single-precision floating point" ), "float(x)" ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "if", 1, tr( "Decision: 1 if x not zero, 0 otherwise" ), "if(x)" ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "if", 2, tr( "Decision: a if x not zero, 0 otherwise" ), "if(x,a)" ) );
   mFunctions.push_back( QgsGrassMapcalcFunction( t, "if", 3, tr( "Decision: a if x not zero, b otherwise" ), "if(x,a,b)", "if,then,else", false ) );
@@ -216,8 +226,7 @@ QgsGrassMapcalc::QgsGrassMapcalc(
   mOutput->QGraphicsRectItem::show();
 
   // Set default tool
-  updateMaps();
-  if ( mMaps.size() > 0 )
+  if ( mMapComboBox->count() > 0 )
   {
     setTool( AddMap );
   }
@@ -481,11 +490,13 @@ QStringList QgsGrassMapcalc::checkRegion()
   QList<QGraphicsItem *> l = mCanvasScene->items();
 
   struct Cell_head currentWindow;
-  if ( !QgsGrass::region( QgsGrass::getDefaultGisdbase(),
-                          QgsGrass::getDefaultLocation(),
-                          QgsGrass::getDefaultMapset(), &currentWindow ) )
+  try
   {
-    QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot get current region" ) );
+    QgsGrass::region( &currentWindow );
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QgsGrass::warning( e );
     return list;
   }
 
@@ -512,7 +523,7 @@ QStringList QgsGrassMapcalc::checkRegion()
     if ( mm.size() > 1 )
       mapset = mm.at( 1 );
 
-    if ( !QgsGrass::mapRegion( QgsGrass::Raster,
+    if ( !QgsGrass::mapRegion( QgsGrassObject::Raster,
                                QgsGrass::getDefaultGisdbase(),
                                QgsGrass::getDefaultLocation(), mapset, map,
                                &window ) )
@@ -536,11 +547,13 @@ bool QgsGrassMapcalc::inputRegion( struct Cell_head *window, QgsCoordinateRefere
   Q_UNUSED( all );
   QgsDebugMsg( "entered." );
 
-  if ( !QgsGrass::region( QgsGrass::getDefaultGisdbase(),
-                          QgsGrass::getDefaultLocation(),
-                          QgsGrass::getDefaultMapset(), window ) )
+  try
   {
-    QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot get current region" ) );
+    QgsGrass::region( window );
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QgsGrass::warning( e );
     return false;
   }
 
@@ -570,7 +583,7 @@ bool QgsGrassMapcalc::inputRegion( struct Cell_head *window, QgsCoordinateRefere
     if ( mm.size() > 1 )
       mapset = mm.at( 1 );
 
-    if ( !QgsGrass::mapRegion( QgsGrass::Raster,
+    if ( !QgsGrass::mapRegion( QgsGrassObject::Raster,
                                QgsGrass::getDefaultGisdbase(),
                                QgsGrass::getDefaultLocation(), mapset, map,
                                &mapWindow ) )
@@ -648,21 +661,10 @@ void QgsGrassMapcalc::setOption()
   {
     case QgsGrassMapcalcObject::Map :
     {
-      bool found = false;
-      for ( unsigned int i = 0 ; i < mMaps.size(); i++ )
+      QStringList mapMapset = mObject->value().split( "@" );
+      if ( !mMapComboBox->setCurrent( mapMapset.value( 0 ), mapMapset.value( 1 ) ) )
       {
-        if ( mMapComboBox->itemText( i ) == mObject->label()
-             && mMaps[i] == mObject->value() )
-        {
-          mMapComboBox->setCurrentIndex( i );
-          found = true;
-        }
-      }
-      if ( !found )
-      {
-        mMaps.push_back( mObject->value() );
-        mMapComboBox->addItem( mObject->label() );
-        mMapComboBox->setCurrentIndex( mMapComboBox->count() - 1 );
+        mMapComboBox->setEditText( mObject->value() );
       }
       break;
     }
@@ -720,11 +722,7 @@ void QgsGrassMapcalc::setTool( int tool )
     case AddMap:
       mObject = new QgsGrassMapcalcObject( QgsGrassMapcalcObject::Map );
       mObject->setId( nextId() );
-
-      // TODO check if there are maps
-      mObject->setValue( mMaps[mMapComboBox->currentIndex()],
-                         mMapComboBox->currentText() );
-
+      mObject->setValue( mMapComboBox->currentText() );
       mObject->setCenter( mLastPoint.x(), mLastPoint.y() );
       mCanvasScene->addItem( mObject );
       mObject->QGraphicsRectItem::show();
@@ -776,10 +774,9 @@ void QgsGrassMapcalc::setTool( int tool )
 
 void QgsGrassMapcalc::addMap()
 {
-  updateMaps();
-  if ( mMaps.size() == 0 )
+  if ( mMapComboBox->count() == 0 )
   {
-    QMessageBox::warning( 0, tr( "Warning" ), tr( "No GRASS raster maps currently in QGIS" ) );
+    QMessageBox::warning( 0, tr( "Warning" ), tr( "No GRASS raster maps available" ) );
 
     setTool( AddConstant );
     return;
@@ -847,75 +844,7 @@ void QgsGrassMapcalc::setToolActionsOff()
   mActionDeleteItem->setChecked( false );
 }
 
-void QgsGrassMapcalc::updateMaps()
-{
-  // TODO: this copy and paste from QgsGrassModuleInput, do it better
-  QgsDebugMsg( "entered." );
-  QString current = mMapComboBox->currentText();
-  mMapComboBox->clear();
-  mMaps.clear();
-
-  QgsMapCanvas *canvas = mIface->mapCanvas();
-
-  int nlayers = canvas->layerCount();
-  QgsDebugMsg( QString( "nlayers = %1" ).arg( nlayers ) );
-  for ( int i = 0; i < nlayers; i++ )
-  {
-    QgsMapLayer *layer = canvas->layer( i );
-
-    if ( layer->type() != QgsMapLayer::RasterLayer )
-      continue;
-
-    // Check if it is GRASS raster
-    QString source = QDir::cleanPath( layer->source() );
-
-    // Note: QDir::cleanPath is using '/' also on Windows
-    //QChar sep = QDir::separator();
-    QChar sep = '/';
-
-    if ( source.contains( "cellhd" ) == 0 )
-      continue;
-
-    // Most probably GRASS layer, check GISBASE and LOCATION
-    QStringList split = source.split( sep, QString::SkipEmptyParts );
-
-    if ( split.size() < 4 )
-      continue;
-
-    QString map = split.last();
-    split.pop_back(); // map
-    if ( split.last() != "cellhd" )
-      continue;
-    split.pop_back(); // cellhd
-
-    QString mapset = split.last();
-    split.pop_back(); // mapset
-
-    //QDir locDir ( sep + split.join ( QString(sep) ) );
-    //QString loc = locDir.canonicalPath();
-
-    QString loc =  source.remove( QRegExp( "/[^/]+/[^/]+/[^/]+$" ) );
-    loc = QDir( loc ).canonicalPath();
-
-    QDir curlocDir( QgsGrass::getDefaultGisdbase() + sep + QgsGrass::getDefaultLocation() );
-    QString curloc = curlocDir.canonicalPath();
-
-    if ( loc != curloc )
-      continue;
-
-#if 0
-    if ( mUpdate && mapset != QgsGrass::getDefaultMapset() )
-      continue;
-#endif
-
-    mMapComboBox->addItem( layer->name() );
-    //if ( layer->name() == current )
-    //  mMapComboBox->setItemText( mMapComboBox->currentIndex(), current );
-    mMaps.push_back( map + "@" + mapset );
-  }
-}
-
-void QgsGrassMapcalc::mapChanged()
+void QgsGrassMapcalc::mapChanged( const QString & text )
 {
   QgsDebugMsg( "entered." );
 
@@ -924,8 +853,7 @@ void QgsGrassMapcalc::mapChanged()
   if ( mObject->type() != QgsGrassMapcalcObject::Map )
     return;
 
-  mObject->setValue( mMaps[mMapComboBox->currentIndex()],
-                     mMapComboBox->currentText() );
+  mObject->setValue( text );
   mCanvasScene->update();
 }
 
@@ -1082,7 +1010,7 @@ void QgsGrassMapcalc::saveAs()
 
   // Ask for file name
   QString name;
-  while ( 1 )
+  for ( ;; )
   {
     bool ok;
     name = QInputDialog::getText( this, tr( "New mapcalc" ),
@@ -1651,10 +1579,6 @@ void QgsGrassMapcalcObject::resetSize()
   {
     for ( int i = 0; i < mFunction.inputLabels().size(); i++ )
     {
-#if 0
-      QStringList::Iterator it = mFunction.inputLabels().at( i );
-      QString l = *it;
-#endif
       QString l = mFunction.inputLabels().at( i );
       int len = metrics.width( l );
       if ( len > mInputTextWidth )
