@@ -17,6 +17,7 @@
 #include "qgsgrassnewmapset.h"
 #include "qgsgrassplugin.h"
 #include "qgsgrass.h"
+#include "qgis.h"
 
 #include "qgisinterface.h"
 #include "qgsapplication.h"
@@ -37,6 +38,9 @@
 
 extern "C"
 {
+#if defined(_MSC_VER) && defined(M_PI_4)
+#undef M_PI_4 //avoid redefinition warning
+#endif
 #include <grass/gprojects.h>
 }
 
@@ -83,63 +87,24 @@ QgsGrassNewMapset::QgsGrassNewMapset( QgisInterface *iface,
   mRegionsInited = false;
   mPlugin = plugin;
 
-  setError( mDatabaseErrorLabel, "" );
-  setError( mLocationErrorLabel, "" );
-  setError( mProjErrorLabel, "" );
-  setError( mRegionErrorLabel, "" );
-  setError( mMapsetErrorLabel, "" );
-
-  const QColor& paletteBackgroundColor = palette().color( backgroundRole() );
-  QPalette palette = mDatabaseText->palette();
-  palette.setColor( mDatabaseText->backgroundRole(), paletteBackgroundColor );
-  mDatabaseText->setPalette( palette );
-  palette = mLocationText->palette();
-  palette.setColor( mLocationText->backgroundRole(), paletteBackgroundColor );
-  mLocationText->setPalette( palette );
-  palette = mRegionText->palette();
-  palette.setColor( mRegionText->backgroundRole(), paletteBackgroundColor );
-  mRegionText->setPalette( palette );
-  palette = mMapsetText->palette();
-  palette.setColor( mMapsetText->backgroundRole(), paletteBackgroundColor );
-  mMapsetText->setPalette( palette );
+  setError( mDatabaseErrorLabel );
+  setError( mLocationErrorLabel );
+  setError( mProjErrorLabel );
+  setError( mRegionErrorLabel );
+  setError( mMapsetErrorLabel );
 
   // DATABASE
   QSettings settings;
-  QString db = settings.value( "/GRASS/lastGisdbase" ).toString();
-  if ( !db.isNull() )
+  QString gisdbase = settings.value( "/GRASS/lastGisdbase" ).toString();
+  if ( gisdbase.isEmpty() )
   {
-    mDatabaseLineEdit->setText( db );
-  }
-  else
-  {
-    mDatabaseLineEdit->setText( QDir::currentPath() );
+    gisdbase = QDir::homePath() + QDir::separator() + "grassdata";
   }
+  mDatabaseLineEdit->setText( gisdbase );
   databaseChanged();
 
-  // Create example tree structure
-  mTreeListView->clear();
-  QTreeWidgetItem *dbi = new QTreeWidgetItem( mTreeListView, QStringList() << "OurDatabase" << tr( "Database" ) );
-  dbi->setExpanded( true );
-
-  QTreeWidgetItem *l = new QTreeWidgetItem( dbi, QStringList() << "Mexico" << tr( "Location 1" ) );
-  l->setExpanded( true );
-  QTreeWidgetItem *m = new QTreeWidgetItem( l, QStringList() << "PERMANENT" << tr( "System mapset" ) );
-  m->setExpanded( true );
-  m = new QTreeWidgetItem( l, QStringList() << "Alejandra" << tr( "User's mapset" ) );
-  m->setExpanded( true );
-  m = new QTreeWidgetItem( l, QStringList() << "Juan" << tr( "User's mapset" ) );
-  m->setExpanded( true );
-
-  l = new QTreeWidgetItem( dbi, QStringList() << "New Zealand" << tr( "Location 2" ) );
-  l->setExpanded( true );
-  m = new QTreeWidgetItem( l, QStringList() << "PERMANENT" << tr( "System mapset" ) );
-  m->setExpanded( true );
-  m = new QTreeWidgetItem( l, QStringList() << "Cimrman" << tr( "User's mapset" ) );
-  m->setExpanded( true );
-
   // LOCATION
-  QRegExp rx;
-  rx.setPattern( "[A-Za-z0-9_.]+" );
+  QRegExp rx( "[A-Za-z0-9_.]+" );
   mLocationLineEdit->setValidator( new QRegExpValidator( rx, mLocationLineEdit ) );
 
   // CRS
@@ -148,24 +113,30 @@ QgsGrassNewMapset::QgsGrassNewMapset( QgisInterface *iface,
   mMapsetsListView->clear();
   mMapsetLineEdit->setValidator( new QRegExpValidator( rx, mMapsetLineEdit ) );
 
-  // FINISH
+  mMapsetsListView->header()->setResizeMode( QHeaderView::ResizeToContents );
 
-  connect( this, SIGNAL( currentIdChanged( int ) ),
-           this, SLOT( pageSelected( int ) ) );
+  // FINISH
+  mOpenNewMapsetCheckBox->setChecked( settings.value( "/GRASS/newMapsetWizard/openMapset", true ).toBool() );
+  connect( this, SIGNAL( currentIdChanged( int ) ), SLOT( pageSelected( int ) ) );
+
+  restoreGeometry( settings.value( "/Windows/QgsGrassNewMapset/geometry" ).toByteArray() );
 }
 
 QgsGrassNewMapset::~QgsGrassNewMapset()
 {
-  QgsDebugMsg( "entered." );
-
+  QgsDebugMsg( "entered" );
+  QSettings settings;
+  settings.setValue( "/Windows/QgsGrassNewMapset/geometry", saveGeometry() );
   mRunning = false;
 }
 /*************************** DATABASE *******************************/
 void QgsGrassNewMapset::browseDatabase()
 {
-  QString selectedDir = QFileDialog::getExistingDirectory( this, NULL, mDatabaseLineEdit->text() );
+  QString selectedDir = QFileDialog::getExistingDirectory( this, nullptr, mDatabaseLineEdit->text() );
   if ( selectedDir.isEmpty() )
+  {
     return;
+  }
 
   mDatabaseLineEdit->setText( selectedDir );
   databaseChanged();
@@ -173,106 +144,120 @@ void QgsGrassNewMapset::browseDatabase()
 
 void QgsGrassNewMapset::databaseChanged()
 {
-  QgsDebugMsg( "entered." );
-  // TODO: reset next tabs
-  //
+  QgsDebugMsg( "entered" );
+
   QSettings settings;
   settings.setValue( "/GRASS/lastGisdbase", mDatabaseLineEdit->text() );
 
   button( QWizard::NextButton )->setEnabled( false );
-  setError( mDatabaseErrorLabel, "" );
+  setError( mDatabaseErrorLabel );
 
-  QString database = mDatabaseLineEdit->text().trimmed();
-
-  if ( database.length() == 0 )
+  if ( gisdbase().isEmpty() )
   {
-    setError( mDatabaseErrorLabel, tr( "Enter path to GRASS database" ) );
+    //setError( mDatabaseErrorLabel, tr( "Enter path to GRASS database" ) );
+    button( QWizard::NextButton )->setEnabled( false );
     return;
   }
+  button( QWizard::NextButton )->setEnabled( true );
 
-  QFileInfo databaseInfo( mDatabaseLineEdit->text() );
-
-  if ( !databaseInfo.exists() )
+  if ( !gisdbaseExists() )
   {
-    setError( mDatabaseErrorLabel, tr( "The directory doesn't exist!" ) );
-    return;
-  }
-
-  // Check if at least one writable location exists or
-  // database is writable
-  bool locationExists = false;
-  QDir d( mDatabaseLineEdit->text() );
-  for ( unsigned int i = 0; i < d.count(); i++ )
-  {
-    if ( d[i] == "." || d[i] == ".." )
-      continue;
-
-    QString windName = mDatabaseLineEdit->text() + "/" + d[i] + "/PERMANENT/DEFAULT_WIND";
-    QString locationName = mDatabaseLineEdit->text() + "/" + d[i];
-    QFileInfo locationInfo( locationName );
-
-    if ( QFile::exists( windName ) && locationInfo.isWritable() )
-    {
-      locationExists = true;
-      break;
-    }
-  }
-
-  if ( locationExists || databaseInfo.isWritable() )
-  {
-    button( QWizard::NextButton )->setEnabled( true );
+    // Do not warn, it may be default $HOME/grassdata, if does not exist, it will be created on finish
+    //setError( mDatabaseErrorLabel, tr( "The directory doesn't exist!" ) );
+    //return;
   }
   else
   {
-    setError( mDatabaseErrorLabel, tr( "No writable locations, the database is not writable!" ) );
+    // Check if at least one writable location exists or database is writable
+    bool locationExists = false;
+    QDir dir( gisdbase() );
+    for ( unsigned int i = 0; i < dir.count(); i++ )
+    {
+      if ( dir[i] == "." || dir[i] == ".." )
+        continue;
+
+      QString windName = gisdbase() + "/" + dir[i] + "/PERMANENT/DEFAULT_WIND";
+      QString locationName = gisdbase() + "/" + dir[i];
+      QFileInfo locationInfo( locationName );
+
+      if ( QFile::exists( windName ) && locationInfo.isWritable() )
+      {
+        locationExists = true;
+        break;
+      }
+    }
+
+    QFileInfo gisdbaseInfo( gisdbase() );
+    if ( locationExists || gisdbaseInfo.isWritable() )
+    {
+      button( QWizard::NextButton )->setEnabled( true );
+    }
+    else
+    {
+      setError( mDatabaseErrorLabel, tr( "No writable locations, the database is not writable!" ) );
+    }
   }
 }
 
+QString QgsGrassNewMapset::gisdbase()
+{
+  return mDatabaseLineEdit->text().trimmed();
+}
+
+bool QgsGrassNewMapset::gisdbaseExists()
+{
+  QFileInfo databaseInfo( gisdbase() );
+  return databaseInfo.exists();
+}
+
 /*************************** LOCATION *******************************/
 void QgsGrassNewMapset::setLocationPage()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   setLocations();
 }
 
 void QgsGrassNewMapset::setLocations()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   mLocationComboBox->clear();
 
   QSettings settings;
   QString lastLocation = settings.value( "/GRASS/lastLocation" ).toString();
 
-  // Get available locations with write permissions
-  QDir d( mDatabaseLineEdit->text() );
-
-  // Add all subdirs containing PERMANENT/DEFAULT_WIND
-  int idx = 0;
-  int sel = -1;
-  for ( unsigned int i = 0; i < d.count(); i++ )
+  if ( gisdbaseExists() )
   {
-    if ( d[i] == "." || d[i] == ".." )
-      continue;
+    // Get available locations with write permissions
+    QDir gisdbaseDir( gisdbase() );
 
-    QString windName = mDatabaseLineEdit->text() + "/" + d[i] + "/PERMANENT/DEFAULT_WIND";
-    QString locationName = mDatabaseLineEdit->text() + "/" + d[i];
-    QFileInfo locationInfo( locationName );
-
-    if ( QFile::exists( windName ) && locationInfo.isWritable() )
+    // Add all subdirs containing PERMANENT/DEFAULT_WIND
+    int idx = 0;
+    int sel = -1;
+    for ( unsigned int i = 0; i < gisdbaseDir.count(); i++ )
     {
-      mLocationComboBox->insertItem( -1, QString( d[i] ) );
-      if ( QString( d[i] ) == lastLocation )
+      if ( gisdbaseDir[i] == "." || gisdbaseDir[i] == ".." )
+        continue;
+
+      QString windName = mDatabaseLineEdit->text() + "/" + gisdbaseDir[i] + "/PERMANENT/DEFAULT_WIND";
+      QString locationName = mDatabaseLineEdit->text() + "/" + gisdbaseDir[i];
+      QFileInfo locationInfo( locationName );
+
+      if ( QFile::exists( windName ) && locationInfo.isWritable() )
       {
-        sel = idx;
+        mLocationComboBox->insertItem( -1, QString( gisdbaseDir[i] ) );
+        if ( QString( gisdbaseDir[i] ) == lastLocation )
+        {
+          sel = idx;
+        }
+        idx++;
       }
-      idx++;
     }
-  }
-  if ( sel >= 0 )
-  {
-    mLocationComboBox->setCurrentIndex( sel );
+    if ( sel >= 0 )
+    {
+      mLocationComboBox->setCurrentIndex( sel );
+    }
   }
 
   if ( mLocationComboBox->count() == 0 )
@@ -314,6 +299,7 @@ int QgsGrassNewMapset::nextId() const
         id = MAPSET;
         break;
       }
+      FALLTHROUGH;
     case DATABASE:
     case CRS:
     case REGION:
@@ -329,23 +315,21 @@ int QgsGrassNewMapset::nextId() const
 
 void QgsGrassNewMapset::checkLocation()
 {
-  setError( mLocationErrorLabel, "" );
+  setError( mLocationErrorLabel );
   button( QWizard::NextButton )->setEnabled( true );
 
   if ( mCreateLocationRadioButton->isChecked() )
   {
-    // TODO?: Check spaces in the name
+    QString location = mLocationLineEdit->text();
 
-    QString location = mLocationLineEdit->text().trimmed();
-
-    if ( location.length() ==  0 )
+    if ( location.isEmpty() )
     {
       button( QWizard::NextButton )->setEnabled( false );
       setError( mLocationErrorLabel, tr( "Enter location name!" ) );
     }
     else
     {
-      if ( QFile::exists( mDatabaseLineEdit->text() + "/" + location ) )
+      if ( QFile::exists( gisdbase() + "/" + location ) )
       {
         button( QWizard::NextButton )->setEnabled( false );
         setError( mLocationErrorLabel, tr( "The location exists!" ) );
@@ -357,38 +341,38 @@ void QgsGrassNewMapset::checkLocation()
 void QgsGrassNewMapset::existingLocationChanged( const QString &text )
 {
   Q_UNUSED( text );
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 }
 
 void QgsGrassNewMapset::newLocationChanged()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   checkLocation();
 }
 
 /************************** CRS ******************************/
 void QgsGrassNewMapset::setProjectionPage()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   setGrassProjection();
 }
 
 void QgsGrassNewMapset::sridSelected( QString theSRID )
 {
   Q_UNUSED( theSRID );
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   projectionSelected();
 }
 
 void QgsGrassNewMapset::projectionSelected()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   setGrassProjection();
 }
 
 void QgsGrassNewMapset::projRadioSwitched()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   if ( mNoProjRadioButton->isChecked() )
   {
     mProjectionSelector->setEnabled( false );
@@ -403,8 +387,8 @@ void QgsGrassNewMapset::projRadioSwitched()
 
 void QgsGrassNewMapset::setGrassProjection()
 {
-  QgsDebugMsg( "entered." );
-  setError( mProjErrorLabel, "" );
+  QgsDebugMsg( "entered" );
+  setError( mProjErrorLabel );
 
   QString proj4 = mProjectionSelector->selectedProj4String();
 
@@ -425,8 +409,8 @@ void QgsGrassNewMapset::setGrassProjection()
   {
     QgsDebugMsg( QString( "proj4 = %1" ).arg( proj4.toLocal8Bit().constData() ) );
 
-    OGRSpatialReferenceH hCRS = NULL;
-    hCRS = OSRNewSpatialReference( NULL );
+    OGRSpatialReferenceH hCRS = nullptr;
+    hCRS = OSRNewSpatialReference( nullptr );
     int errcode;
 
     {
@@ -445,7 +429,7 @@ void QgsGrassNewMapset::setGrassProjection()
     }
     else
     {
-      char *wkt = NULL;
+      char *wkt = nullptr;
 
       QgsDebugMsg( QString( "OSRIsGeographic = %1" ).arg( OSRIsGeographic( hCRS ) ) );
       QgsDebugMsg( QString( "OSRIsProjected = %1" ).arg( OSRIsProjected( hCRS ) ) );
@@ -493,7 +477,7 @@ void QgsGrassNewMapset::setGrassProjection()
 /**************************** REGION ********************************/
 void QgsGrassNewMapset::setRegionPage()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   // Set defaults
   if ( !mRegionModified )
@@ -502,27 +486,27 @@ void QgsGrassNewMapset::setRegionPage()
   }
 
   // Create new projection
-  QgsCoordinateReferenceSystem newSrs;
+  QgsCoordinateReferenceSystem newCrs;
   if ( mProjRadioButton->isChecked() )
   {
     QgsDebugMsg( QString( "selectedCrsId() = %1" ).arg( mProjectionSelector->selectedCrsId() ) );
 
     if ( mProjectionSelector->selectedCrsId() > 0 )
     {
-      newSrs.createFromSrsId( mProjectionSelector->selectedCrsId() );
-      if ( ! newSrs.isValid() )
+      newCrs.createFromSrsId( mProjectionSelector->selectedCrsId() );
+      if ( ! newCrs.isValid() )
       {
-        QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot create projection." ) );
+        QgsGrass::warning( tr( "Cannot create projection." ) );
       }
     }
   }
 
   // Reproject previous region if it was modified
   // and if previous and current projection is valid
-  if ( mRegionModified && newSrs.isValid() && mSrs.isValid()
-       && newSrs.srsid() != mSrs.srsid() )
+  if ( mRegionModified && newCrs.isValid() && mCrs.isValid()
+       && newCrs.srsid() != mCrs.srsid() )
   {
-    QgsCoordinateTransform trans( mSrs, newSrs );
+    QgsCoordinateTransform trans( mCrs, newCrs );
 
     double n = mNorthLineEdit->text().toDouble();
     double s = mSouthLineEdit->text().toDouble();
@@ -553,21 +537,21 @@ void QgsGrassNewMapset::setRegionPage()
 
     if ( ok )
     {
-      mNorthLineEdit->setText( QString::number( points[1].y() ) );
-      mSouthLineEdit->setText( QString::number( points[0].y() ) );
-      mEastLineEdit->setText( QString::number( points[1].x() ) );
-      mWestLineEdit->setText( QString::number( points[0].x() ) );
+      int precision = newCrs.mapUnits() == QGis::Degrees ? 6 : 1;
+      mNorthLineEdit->setText( qgsDoubleToString( points[1].y(), precision ) );
+      mSouthLineEdit->setText( qgsDoubleToString( points[0].y(), precision ) );
+      mEastLineEdit->setText( qgsDoubleToString( points[1].x(), precision ) );
+      mWestLineEdit->setText( qgsDoubleToString( points[0].x(), precision ) );
     }
     else
     {
-      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot reproject previously set region, default region set." ) );
-
+      QgsGrass::warning( tr( "Cannot reproject previously set region, default region set." ) );
       setGrassRegionDefaults();
     }
   }
 
   // Set current region projection
-  mSrs = newSrs;
+  mCrs = newCrs;
 
   // Enable / disable region selection widgets
   if ( mNoProjRadioButton->isChecked() )
@@ -588,10 +572,7 @@ void QgsGrassNewMapset::setRegionPage()
 
     QgsRectangle ext = mIface->mapCanvas()->extent();
 
-    if ( ext.xMinimum() >= ext.xMaximum() || ext.yMinimum() >= ext.yMaximum() )
-    {
-      mCurrentRegionButton->setEnabled( false );
-    }
+    mCurrentRegionButton->setEnabled( !ext.isEmpty() );
   }
 
   checkRegion();
@@ -606,10 +587,8 @@ void QgsGrassNewMapset::setGrassRegionDefaults()
 {
   QgsDebugMsg( QString( "mCellHead.proj = %1" ).arg( mCellHead.proj ) );
 
-  int srsid = QgsProject::instance()->readNumEntry(
-                "SpatialRefSys", "/ProjectCRSID", 0 );
-
-  QgsDebugMsg( QString( "current project srsid = %1" ).arg( srsid ) );
+  QgsCoordinateReferenceSystem srs = mIface->mapCanvas()->mapSettings().destinationCrs();
+  QgsDebugMsg( "srs = " + srs.toWkt() );
 
   QgsRectangle ext = mIface->mapCanvas()->extent();
   bool extSet = false;
@@ -621,7 +600,7 @@ void QgsGrassNewMapset::setGrassRegionDefaults()
   if ( extSet &&
        ( mNoProjRadioButton->isChecked() ||
          ( mProjRadioButton->isChecked()
-           && srsid == mProjectionSelector->selectedCrsId() )
+           && srs.srsid() == mProjectionSelector->selectedCrsId() )
        )
      )
   {
@@ -656,7 +635,7 @@ void QgsGrassNewMapset::setGrassRegionDefaults()
 
 void QgsGrassNewMapset::regionChanged()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   mRegionModified = true;
   checkRegion();
@@ -665,11 +644,11 @@ void QgsGrassNewMapset::regionChanged()
 
 void QgsGrassNewMapset::checkRegion()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   bool err = false;
 
-  setError( mRegionErrorLabel, "" );
+  setError( mRegionErrorLabel );
   button( QWizard::NextButton )->setEnabled( false );
 
   if ( mNorthLineEdit->text().trimmed().length() == 0
@@ -720,14 +699,15 @@ void QgsGrassNewMapset::checkRegion()
   mCellHead.ns_res  = res;
   mCellHead.ns_res3 = res3;
   mCellHead.tb_res  = 1.;
-  mCellHead.zone = 0;
+  // Do not override zone, it was set in setGrassProjection()
+  //mCellHead.zone = 0;
 
   button( QWizard::NextButton )->setEnabled( true );
 }
 
 void QgsGrassNewMapset::loadRegions()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   QString path = QgsApplication::pkgDataPath() + "/grass/locations.gml";
   QgsDebugMsg( QString( "load:%1" ).arg( path.toLocal8Bit().constData() ) );
@@ -736,14 +716,12 @@ void QgsGrassNewMapset::loadRegions()
 
   if ( !file.exists() )
   {
-    QMessageBox::warning( 0, tr( "Warning" ),
-                          tr( "Regions file (%1) not found." ).arg( path ) );
+    QgsGrass::warning( tr( "Regions file (%1) not found." ).arg( path ) );
     return;
   }
   if ( ! file.open( QIODevice::ReadOnly ) )
   {
-    QMessageBox::warning( 0, tr( "Warning" ),
-                          tr( "Cannot open locations file (%1)" ).arg( path ) );
+    QgsGrass::warning( tr( "Cannot open locations file (%1)" ).arg( path ) );
     return;
   }
 
@@ -756,7 +734,7 @@ void QgsGrassNewMapset::loadRegions()
     QString errmsg = tr( "Cannot read locations file (%1):" ).arg( path )
                      + tr( "\n%1\nat line %2 column %3" ).arg( err ).arg( line ).arg( column );
     QgsDebugMsg( errmsg );
-    QMessageBox::warning( 0, tr( "Warning" ), errmsg );
+    QgsGrass::warning( errmsg );
     file.close();
     return;
   }
@@ -823,13 +801,14 @@ void QgsGrassNewMapset::loadRegions()
     QgsPoint urp( ur[0].toDouble(), ur[1].toDouble() );
     mRegionsPoints.push_back( urp );
   }
+  mRegionsComboBox->setCurrentIndex( -1 );
 
   file.close();
 }
 
 void QgsGrassNewMapset::setSelectedRegion()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   // mRegionsPoints are in EPSG 4326 = LL WGS84
   int index = 2 * mRegionsComboBox->currentIndex();
@@ -855,8 +834,7 @@ void QgsGrassNewMapset::setSelectedRegion()
 
     if ( !source.isValid() )
     {
-      QMessageBox::warning( 0, tr( "Warning" ),
-                            tr( "Cannot create QgsCoordinateReferenceSystem" ) );
+      QgsGrass::warning( tr( "Cannot create QgsCoordinateReferenceSystem" ) );
       return;
     }
 
@@ -865,8 +843,7 @@ void QgsGrassNewMapset::setSelectedRegion()
 
     if ( !dest.isValid() )
     {
-      QMessageBox::warning( 0, tr( "Warning" ),
-                            tr( "Cannot create QgsCoordinateReferenceSystem" ) );
+      QgsGrass::warning( tr( "Cannot create QgsCoordinateReferenceSystem" ) );
       return;
     }
 
@@ -892,8 +869,7 @@ void QgsGrassNewMapset::setSelectedRegion()
 
     if ( !ok )
     {
-      QMessageBox::warning( 0, tr( "Warning" ),
-                            tr( "Cannot reproject selected region." ) );
+      QgsGrass::warning( tr( "Cannot reproject selected region." ) );
       return;
     }
   }
@@ -945,16 +921,12 @@ void QgsGrassNewMapset::setSelectedRegion()
 
 void QgsGrassNewMapset::setCurrentRegion()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   QgsRectangle ext = mIface->mapCanvas()->extent();
 
-  int srsid = QgsProject::instance()->readNumEntry(
-                "SpatialRefSys", "/ProjectCRSID", 0 );
-
-  QgsCoordinateReferenceSystem srs( srsid, QgsCoordinateReferenceSystem::InternalCrsId );
-  QgsDebugMsg( QString( "current project srsid = %1" ).arg( srsid ) );
-  QgsDebugMsg( QString( "srs.isValid() = %1" ).arg( srs.isValid() ) );
+  QgsCoordinateReferenceSystem srs = mIface->mapCanvas()->mapSettings().destinationCrs();
+  QgsDebugMsg( "srs = " + srs.toWkt() );
 
   std::vector<QgsPoint> points;
 
@@ -963,10 +935,10 @@ void QgsGrassNewMapset::setCurrentRegion()
   points.push_back( QgsPoint( ext.xMaximum(), ext.yMaximum() ) );
 
   // TODO add a method, this code is copy-paste from setSelectedRegion
-  if ( srs.isValid() && mSrs.isValid()
-       && srs.srsid() != mSrs.srsid() )
+  if ( srs.isValid() && mCrs.isValid()
+       && srs.srsid() != mCrs.srsid() )
   {
-    QgsCoordinateTransform trans( srs, mSrs );
+    QgsCoordinateTransform trans( srs, mCrs );
 
     bool ok = true;
     for ( int i = 0; i < 2; i++ )
@@ -986,7 +958,7 @@ void QgsGrassNewMapset::setCurrentRegion()
 
     if ( !ok )
     {
-      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot reproject region" ) );
+      QgsGrass::warning( tr( "Cannot reproject region" ) );
       return;
     }
   }
@@ -1003,7 +975,7 @@ void QgsGrassNewMapset::setCurrentRegion()
 
 void QgsGrassNewMapset::clearRegion()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   QPixmap pm = mPixmap;
   mRegionMap->setPixmap( pm );
@@ -1011,7 +983,7 @@ void QgsGrassNewMapset::clearRegion()
 
 void QgsGrassNewMapset::drawRegion()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   QPixmap pm = mPixmap;
   mRegionMap->setPixmap( pm );
@@ -1041,17 +1013,17 @@ void QgsGrassNewMapset::drawRegion()
     }
   }
 
-  std::vector<QgsPoint> tpoints; // ll lr ur ul ll
-  tpoints.push_back( QgsPoint( w, s ) );
-  tpoints.push_back( QgsPoint( e, s ) );
-  tpoints.push_back( QgsPoint( e, n ) );
-  tpoints.push_back( QgsPoint( w, n ) );
-  tpoints.push_back( QgsPoint( w, s ) );
+  QList<QgsPoint> tpoints; // ll lr ur ul ll
+  tpoints << QgsPoint( w, s );
+  tpoints << QgsPoint( e, s );
+  tpoints << QgsPoint( e, n );
+  tpoints << QgsPoint( w, n );
+  tpoints << QgsPoint( w, s );
 
 
   // Because of possible shift +/- 360 in LL we have to split
   // the lines at least in 3 parts
-  std::vector<QgsPoint> points; //
+  QList<QgsPoint> points; //
   for ( int i = 0; i < 4; i++ )
   {
     for ( int j = 0; j < 3; j++ )
@@ -1061,11 +1033,11 @@ void QgsGrassNewMapset::drawRegion()
       double dx = ( tpoints[i+1].x() - x ) / 3;
       double dy = ( tpoints[i+1].y() - y ) / 3;
       QgsDebugMsg( QString( "dx = %1 x = %2" ).arg( dx ).arg( x + j*dx ) );
-      points.push_back( QgsPoint( x + j*dx, y + j*dy ) );
+      points << QgsPoint( x + j*dx, y + j*dy );
 
     }
   }
-  points.push_back( QgsPoint( points[0] ) ); // close polygon
+  points << points[0]; // close polygon
 
   // Warning: seems that crashes if source == dest
   if ( mProjectionSelector->selectedCrsId() != GEOCRS_ID )
@@ -1075,8 +1047,7 @@ void QgsGrassNewMapset::drawRegion()
 
     if ( !source.isValid() )
     {
-      QMessageBox::warning( 0, tr( "Warning" ),
-                            tr( "Cannot create QgsCoordinateReferenceSystem" ) );
+      QgsGrass::warning( tr( "Cannot create QgsCoordinateReferenceSystem" ) );
       return;
     }
 
@@ -1084,16 +1055,13 @@ void QgsGrassNewMapset::drawRegion()
 
     if ( !dest.isValid() )
     {
-      QMessageBox::warning( 0, tr( "Warning" ),
-                            tr( "Cannot create QgsCoordinateReferenceSystem" ) );
+      QgsGrass::warning( tr( "Cannot create QgsCoordinateReferenceSystem" ) );
       return;
     }
 
     QgsCoordinateTransform trans( source, dest );
 
-
-    bool ok = true;
-    for ( int i = 0; i < 13; i++ )
+    for ( int i = points.size() - 1; i >= 0; i-- )
     {
       // Warning: I found that with some projections (e.g. Abidjan 1987)
       // if N = 90 or S = -90 the coordinate projected to
@@ -1109,6 +1077,7 @@ void QgsGrassNewMapset::drawRegion()
 
       QgsDebugMsg( QString( "%1,%2" ).arg( points[i].x() ).arg( points[i].y() ) );
 
+      // exclude points if transformation failed
       try
       {
         points[i] = trans.transform( points[i] );
@@ -1118,12 +1087,11 @@ void QgsGrassNewMapset::drawRegion()
       {
         Q_UNUSED( cse );
         QgsDebugMsg( "Cannot transform point" );
-        ok = false;
-        break;
+        points.removeAt( i );
       }
     }
 
-    if ( !ok )
+    if ( points.size() < 3 )
     {
       QgsDebugMsg( "Cannot reproject region." );
       return;
@@ -1161,17 +1129,19 @@ void QgsGrassNewMapset::drawRegion()
 /**************************** MAPSET ********************************/
 void QgsGrassNewMapset::setMapsets()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
   mMapsetsListView->clear();
 
   if ( mCreateLocationRadioButton->isChecked() )
   {
-    mMapsetsFrame->hide();
+    mMapsetsLabel->hide();
+    mMapsetsListView->hide();
     return;
   }
   else
   {
-    mMapsetsFrame->show();
+    mMapsetsLabel->show();
+    mMapsetsListView->show();
   }
 
   // Get available mapsets
@@ -1197,17 +1167,16 @@ void QgsGrassNewMapset::setMapsets()
 
 void QgsGrassNewMapset::mapsetChanged()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   button( QWizard::NextButton )->setEnabled( false );
-  setError( mMapsetErrorLabel, "" );
+  setError( mMapsetErrorLabel );
 
   QString mapset = mMapsetLineEdit->text().trimmed();
 
-  // TODO?: Check spaces in the name
-  if ( mapset.length() == 0 )
+  if ( mapset.isEmpty() )
   {
-    setError( mMapsetErrorLabel, tr( "Enter mapset name." ) );
+    //setError( mMapsetErrorLabel, tr( "Enter mapset name." ) );
     return;
   }
 
@@ -1231,11 +1200,19 @@ void QgsGrassNewMapset::mapsetChanged()
 }
 
 /**************************** FINISH ********************************/
+void QgsGrassNewMapset::on_mOpenNewMapsetCheckBox_stateChanged( int state )
+{
+  Q_UNUSED( state );
+  QgsDebugMsg( "entered" );
+  QSettings settings;
+  settings.setValue( "/GRASS/newMapsetWizard/openMapset", mOpenNewMapsetCheckBox->isChecked() );
+}
+
 void QgsGrassNewMapset::setFinishPage()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
-  mDatabaseLabel->setText( tr( "Database: " ) + mDatabaseLineEdit->text() );
+  mDatabaseLabel->setText( tr( "Database" ) + " : " + mDatabaseLineEdit->text() );
 
   QString location;
   if ( mSelectLocationRadioButton->isChecked() )
@@ -1244,52 +1221,58 @@ void QgsGrassNewMapset::setFinishPage()
   }
   else
   {
-    location = mLocationLineEdit->text().trimmed();
+    location = mLocationLineEdit->text();
   }
-  mLocationLabel->setText( tr( "Location: " ) + location );
+  mLocationLabel->setText( tr( "Location" ) + " : " + location );
 
-  mMapsetLabel->setText( tr( "Mapset: " ) + mMapsetLineEdit->text() );
+  mMapsetLabel->setText( tr( "Mapset" ) + " : " + mMapsetLineEdit->text() );
 }
 
 void QgsGrassNewMapset::createMapset()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
-  // TODO: handle all possible errors better, especially
-  //       half created location/mapset
+  // TODO: handle all possible errors better, especially half created location/mapset
+
+  if ( !gisdbaseExists() )
+  {
+    QgsDebugMsg( "create gisdbase " + gisdbase() );
+    QDir gisdbaseDir( gisdbase() );
+    QString dirName = gisdbaseDir.dirName();
+    gisdbaseDir.cdUp();
+    if ( !gisdbaseDir.mkdir( dirName ) )
+    {
+      QgsGrass::warning( tr( "Cannot create new GRASS database directory" ) + gisdbase() );
+      return;
+    }
+  }
 
   QString location;
-
   if ( mCreateLocationRadioButton->isChecked() )
   {
-    location = mLocationLineEdit->text().trimmed();
+    location = mLocationLineEdit->text();
 
-    // TODO: add QgsGrass::setLocation or G_make_location with
-    //       database path
-    QgsGrass::activeMode(); // because it calls private QgsGrass::init()
-
-#if defined(WIN32)
-    G__setenv(( char * ) "GISDBASE", QgsGrass::shortPath( mDatabaseLineEdit->text() ).toUtf8().data() );
-#else
-    G__setenv(( char * ) "GISDBASE", mDatabaseLineEdit->text().toUtf8().data() );
-#endif
+    QgsGrass::setLocation( gisdbase(), location );
 
     int ret = 0;
-
-    try
+    QString error;
+    G_TRY
     {
+#if GRASS_VERSION_MAJOR < 7
       ret = G_make_location( location.toUtf8().data(), &mCellHead, mProjInfo, mProjUnits, stdout );
+#else
+      ret = G_make_location( location.toUtf8().data(), &mCellHead, mProjInfo, mProjUnits );
+#endif
     }
-    catch ( QgsGrass::Exception &e )
+    G_CATCH( QgsGrass::Exception &e )
     {
-      ret = -1;
       Q_UNUSED( e );
+      error = QString( e.what() );
     }
 
     if ( ret != 0 )
     {
-      QMessageBox::warning( this, tr( "Create location" ),
-                            tr( "Cannot create new location: %1" ).arg( QgsGrass::errorMessage() ) );
+      QgsGrass::warning( tr( "Cannot create new location: %1" ).arg( error ) );
       return;
     }
 
@@ -1306,63 +1289,40 @@ void QgsGrassNewMapset::createMapset()
   }
 
   // Create mapset
-  QString mapset = mMapsetLineEdit->text().trimmed();
+  QString mapset = mMapsetLineEdit->text();
 
   if ( mapset != "PERMANENT" )
   {
-    QString locationPath = mDatabaseLineEdit->text() + "/" + location;
-    QDir d( locationPath );
-
-    if ( !d.mkdir( mapset ) )
+    QString error;
+    QgsGrass::createMapset( gisdbase(), location, mapset, error );
+    if ( !error.isEmpty() )
     {
-      QMessageBox::warning( this, tr( "Create mapset" ),
-                            tr( "Cannot create new mapset directory" ) );
+      QgsGrass::warning( tr( "Cannot create new mapset: %1" ).arg( error ) );
       return;
     }
-
-    // Copy WIND Better way to copy file in Qt?
-    QStringList lines;
-    QFile in( locationPath + "/PERMANENT/DEFAULT_WIND" );
-    if ( !in.open( QIODevice::ReadOnly ) )
-    {
-      QMessageBox::warning( this, tr( "Create mapset" ), tr( "Cannot open DEFAULT_WIND" ) );
-      return;
-    }
-
-    QFile out( locationPath + "/" + mapset + "/WIND" );
-    if ( !out.open( QIODevice::WriteOnly ) )
-    {
-      QMessageBox::warning( this, tr( "Create mapset" ), tr( "Cannot open WIND" ) );
-      return;
-    }
-    QTextStream stream( &out );
-
-    //QTextStream stream( &file );
-    QString line;
-    char buf[100];
-    while ( in.readLine( buf, 100 ) != -1 )
-    {
-      stream << buf;
-    }
-
-    in.close();
-    out.close();
   }
 
-  QString err = QgsGrass::openMapset(
-                  mDatabaseLineEdit->text(), location, mapset );
-
-  if ( err.length() > 0 )
+  if ( mOpenNewMapsetCheckBox->isChecked() )
   {
-    QMessageBox::information( this, tr( "New mapset" ),
-                              tr( "New mapset successfully created, but cannot be opened: %1" ).arg( err ) );
+    QString error = QgsGrass::openMapset(
+                      mDatabaseLineEdit->text(), location, mapset );
+
+    if ( !error.isEmpty() )
+    {
+      QMessageBox::information( this, tr( "New mapset" ),
+                                tr( "New mapset successfully created, but cannot be opened: %1" ).arg( error ) );
+    }
+    else
+    {
+      QMessageBox::information( this, tr( "New mapset" ),
+                                tr( "New mapset successfully created and set as current working mapset." ) );
+
+      mPlugin->mapsetChanged();
+    }
   }
   else
   {
-    QMessageBox::information( this, tr( "New mapset" ),
-                              tr( "New mapset successfully created and set as current working mapset." ) );
-
-    mPlugin->mapsetChanged();
+    QMessageBox::information( this, tr( "New mapset" ), tr( "New mapset successfully created" ) );
   }
 
   deleteLater();
@@ -1370,7 +1330,7 @@ void QgsGrassNewMapset::createMapset()
 
 void QgsGrassNewMapset::accept()
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   createMapset();
 }
@@ -1378,11 +1338,14 @@ void QgsGrassNewMapset::accept()
 /********************************************************************/
 void QgsGrassNewMapset::setError( QLabel *line, const QString &err )
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
-  if ( err.length() > 0 )
+  if ( !err.isEmpty() )
   {
     line->setText( err );
+    QPalette palette = line->palette();
+    palette.setColor( QPalette::WindowText, Qt::red );
+    line->setPalette( palette );
     line->show();
   }
   else
@@ -1428,16 +1391,12 @@ void QgsGrassNewMapset::pageSelected( int index )
         connect( mProjectionSelector, SIGNAL( sridSelected( QString ) ),
                  this, SLOT( sridSelected( QString ) ) );
 
-        // Se current QGIS projection
-        int srsid = QgsProject::instance()->readNumEntry(
-                      "SpatialRefSys", "/ProjectCRSID", 0 );
+        QgsCoordinateReferenceSystem  srs = mIface->mapCanvas()->mapSettings().destinationCrs();
+        QgsDebugMsg( "srs = " + srs.toWkt() );
 
-        QgsCoordinateReferenceSystem srs( srsid, QgsCoordinateReferenceSystem::InternalCrsId );
-        QgsDebugMsg( QString( "current project srsid = %1" ).arg( srsid ) );
-        QgsDebugMsg( QString( "srs.isValid() = %1" ).arg( srs.isValid() ) );
         if ( srs.isValid() )
         {
-          mProjectionSelector->setSelectedCrsId( srsid );
+          mProjectionSelector->setSelectedCrsId( srs.srsid() );
           mProjRadioButton->setChecked( true );
           projRadioSwitched();
         }
@@ -1484,7 +1443,7 @@ bool QgsGrassNewMapset::isRunning( void )
 
 void QgsGrassNewMapset::close( void )
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   hide();
   mRunning = false;
@@ -1493,7 +1452,7 @@ void QgsGrassNewMapset::close( void )
 
 void QgsGrassNewMapset::closeEvent( QCloseEvent *e )
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsg( "entered" );
 
   e->accept();
   close();
