@@ -48,7 +48,7 @@ class RoadGraphPlugin: public QObject, public QgisPlugin
      * QGIS when it attempts to instantiate the plugin.
      * @param theQgisInterface Pointer to the QgisInterface object.
      */
-    RoadGraphPlugin( QgisInterface * theQgisInterface );
+    explicit RoadGraphPlugin( QgisInterface * theQgisInterface );
     //! Destructor
     virtual ~RoadGraphPlugin();
     /**
@@ -123,8 +123,8 @@ class RoadGraphPlugin: public QObject, public QgisPlugin
     //
     ////////////////////////////////////////////////////////////////////
     /**
-    * on show settings
-    */
+     * on show settings
+     */
     QAction * mQSettingsAction;
 
     /**
