@@ -122,8 +122,8 @@ topolTest::topolTest( QgisInterface* qgsIface )
 
 topolTest::~topolTest()
 {
-  QMap<QString, QgsSpatialIndex*>::Iterator lit = mLayerIndexes.begin();
-  for ( ; lit != mLayerIndexes.end(); ++lit )
+  QMap<QString, QgsSpatialIndex*>::const_iterator lit = mLayerIndexes.constBegin();
+  for ( ; lit != mLayerIndexes.constEnd(); ++lit )
     delete *lit;
 }
 
@@ -240,8 +240,7 @@ ErrorList topolTest::checkDanglingLines( double tolerance, QgsVectorLayer* layer
     return errorList;
   }
 
-  QList<FeatureLayer>::Iterator it;
-  QList<FeatureLayer>::ConstIterator FeatureListEnd = mFeatureList1.end();
+  QList<FeatureLayer>::iterator it;
 
   qDebug() << mFeatureList1.count();
 
@@ -250,7 +249,7 @@ ErrorList topolTest::checkDanglingLines( double tolerance, QgsVectorLayer* layer
 
   std::multimap<QgsPoint, QgsFeatureId, PointComparer> endVerticesMap;
 
-  for ( it = mFeatureList1.begin(); it != FeatureListEnd; ++it )
+  for ( it = mFeatureList1.begin(); it != mFeatureList1.end(); ++it )
   {
     if ( !( ++i % 100 ) )
       emit progress( i );
@@ -355,9 +354,8 @@ ErrorList topolTest::checkDuplicates( double tolerance, QgsVectorLayer *layer1,
 
   QgsGeometry* canvasExtentPoly = QgsGeometry::fromWkt( theQgsInterface->mapCanvas()->extent().asWktPolygon() );
 
-  QMap<QgsFeatureId, FeatureLayer>::Iterator it;
-  QMap<QgsFeatureId, FeatureLayer>::ConstIterator FeatureListEnd = mFeatureMap2.end();
-  for ( it = mFeatureMap2.begin(); it != FeatureListEnd; ++it )
+  QMap<QgsFeatureId, FeatureLayer>::const_iterator it;
+  for ( it = mFeatureMap2.constBegin(); it != mFeatureMap2.constEnd(); ++it )
   {
     if ( !( ++i % 100 ) )
       emit progress( i );
@@ -373,7 +371,7 @@ ErrorList topolTest::checkDuplicates( double tolerance, QgsVectorLayer *layer1,
     if ( testCancelled() )
       break;
 
-    QgsGeometry* g1 = it->feature.geometry();
+    const QgsGeometry* g1 = it->feature.constGeometry();
     QgsRectangle bb = g1->boundingBox();
 
     QList<QgsFeatureId> crossingIds;
@@ -391,7 +389,7 @@ ErrorList topolTest::checkDuplicates( double tolerance, QgsVectorLayer *layer1,
       if ( mFeatureMap2[*cit].feature.id() == it->feature.id() )
         continue;
 
-      QgsGeometry* g2 = mFeatureMap2[*cit].feature.geometry();
+      const QgsGeometry* g2 = mFeatureMap2[*cit].feature.constGeometry();
       if ( !g2 )
       {
         QgsMessageLog::logMessage( tr( "Invalid second geometry in duplicate geometry test." ), tr( "Topology plugin" ) );
@@ -416,21 +414,21 @@ ErrorList topolTest::checkDuplicates( double tolerance, QgsVectorLayer *layer1,
 
         QList<FeatureLayer> fls;
         fls << *it << *it;
-        QgsGeometry* conflict = new QgsGeometry( *g1 );
+        QScopedPointer<QgsGeometry> conflict( new QgsGeometry( *g1 ) );
 
         if ( isExtent )
         {
-          if ( canvasExtentPoly->disjoint( conflict ) )
+          if ( canvasExtentPoly->disjoint( conflict.data() ) )
           {
             continue;
           }
-          if ( canvasExtentPoly->crosses( conflict ) )
+          if ( canvasExtentPoly->crosses( conflict.data() ) )
           {
-            conflict = conflict->intersection( canvasExtentPoly );
+            conflict.reset( conflict->intersection( canvasExtentPoly ) );
           }
         }
 
-        TopolErrorDuplicates* err = new TopolErrorDuplicates( bb, conflict, fls );
+        TopolErrorDuplicates* err = new TopolErrorDuplicates( bb, conflict.take(), fls );
 
         errorList << err;
       }
@@ -467,9 +465,8 @@ ErrorList topolTest::checkOverlaps( double tolerance, QgsVectorLayer *layer1, Qg
     return errorList;
   }
 
-  QMap<QgsFeatureId, FeatureLayer>::Iterator it;
-  QMap<QgsFeatureId, FeatureLayer>::ConstIterator FeatureListEnd = mFeatureMap2.end();
-  for ( it = mFeatureMap2.begin(); it != FeatureListEnd; ++it )
+  QMap<QgsFeatureId, FeatureLayer>::const_iterator it;
+  for ( it = mFeatureMap2.constBegin(); it != mFeatureMap2.constEnd(); ++it )
   {
     if ( !( ++i % 100 ) )
       emit progress( i );
@@ -485,7 +482,7 @@ ErrorList topolTest::checkOverlaps( double tolerance, QgsVectorLayer *layer1, Qg
     if ( testCancelled() )
       break;
 
-    QgsGeometry* g1 = it->feature.geometry();
+    const QgsGeometry* g1 = it->feature.constGeometry();
 
     if ( !g1->isGeosValid() )
     {
@@ -512,7 +509,7 @@ ErrorList topolTest::checkOverlaps( double tolerance, QgsVectorLayer *layer1, Qg
       if ( mFeatureMap2[*cit].feature.id() == it->feature.id() )
         continue;
 
-      QgsGeometry* g2 = mFeatureMap2[*cit].feature.geometry();
+      const QgsGeometry* g2 = mFeatureMap2[*cit].feature.constGeometry();
       if ( !g2 )
       {
         QgsMessageLog::logMessage( tr( "Invalid second geometry in overlaps test." ), tr( "Topology plugin" ) );
@@ -543,21 +540,21 @@ ErrorList topolTest::checkOverlaps( double tolerance, QgsVectorLayer *layer1, Qg
       {
         QList<FeatureLayer> fls;
         fls << *it << *it;
-        QgsGeometry* conflictGeom = g1->intersection( g2 );
+        QScopedPointer< QgsGeometry > conflictGeom( g1->intersection( g2 ) );
 
         if ( isExtent )
         {
-          if ( canvasExtentPoly->disjoint( conflictGeom ) )
+          if ( canvasExtentPoly->disjoint( conflictGeom.data() ) )
           {
             continue;
           }
-          if ( canvasExtentPoly->crosses( conflictGeom ) )
+          if ( canvasExtentPoly->crosses( conflictGeom.data() ) )
           {
-            conflictGeom = conflictGeom->intersection( canvasExtentPoly );
+            conflictGeom.reset( conflictGeom->intersection( canvasExtentPoly ) );
           }
         }
 
-        TopolErrorOverlaps* err = new TopolErrorOverlaps( bb, conflictGeom, fls );
+        TopolErrorOverlaps* err = new TopolErrorOverlaps( bb, conflictGeom.take(), fls );
 
         errorList << err;
       }
@@ -588,14 +585,13 @@ ErrorList topolTest::checkGaps( double tolerance, QgsVectorLayer *layer1, QgsVec
     return errorList;
   }
 
-  QList<FeatureLayer>::Iterator it;
+  QList<FeatureLayer>::iterator it;
   QgsGeometry* g1;
 
   QList<GEOSGeometry*> geomList;
 
   qDebug() << mFeatureList1.count() << " features in list!";
-  QList<FeatureLayer>::ConstIterator FeatureListEnd = mFeatureList1.end();
-  for ( it = mFeatureList1.begin(); it != FeatureListEnd; ++it )
+  for ( it = mFeatureList1.begin(); it != mFeatureList1.end(); ++it )
   {
     qDebug() << "reading features-" << i;
 
@@ -637,6 +633,7 @@ ErrorList topolTest::checkGaps( double tolerance, QgsVectorLayer *layer1, QgsVec
         QgsGeometry* polyGeom = QgsGeometry::fromPolygon( polygon );
 
         geomList.push_back( GEOSGeom_clone_r( geosctxt, polyGeom->asGeos() ) );
+        delete polyGeom;
       }
 
     }
@@ -655,14 +652,14 @@ ErrorList topolTest::checkGaps( double tolerance, QgsVectorLayer *layer1, QgsVec
 
   qDebug() << "creating geometry collection-";
 
-  if ( geomList.size() == 0 )
+  if ( geomList.isEmpty() )
   {
     //qDebug() << "geometry list is empty!";
     delete [] geomArray;
     return errorList;
   }
 
-  GEOSGeometry* collection = 0;
+  GEOSGeometry* collection = nullptr;
   collection = GEOSGeom_createCollection_r( geosctxt, GEOS_MULTIPOLYGON, geomArray, geomList.size() );
 
 
@@ -694,6 +691,7 @@ ErrorList topolTest::checkGaps( double tolerance, QgsVectorLayer *layer1, QgsVec
   //qDebug() << "difference gometry - " << diffGeoms->exportToWkt();
 
   QList<QgsGeometry*> geomColl = diffGeoms->asGeometryCollection();
+  delete diffGeoms;
 
   QgsGeometry* canvasExtentPoly = QgsGeometry::fromWkt( theQgsInterface->mapCanvas()->extent().asWktPolygon() );
 
@@ -712,7 +710,6 @@ ErrorList topolTest::checkGaps( double tolerance, QgsVectorLayer *layer1, QgsVec
       }
     }
     QgsRectangle bBox = conflictGeom->boundingBox();
-    QgsFeature feat;
     FeatureLayer ftrLayer1;
     ftrLayer1.layer = layer1;
     QList<FeatureLayer> errorFtrLayers;
@@ -739,8 +736,7 @@ ErrorList topolTest::checkPseudos( double tolerance, QgsVectorLayer *layer1, Qgs
     return errorList;
   }
 
-  QList<FeatureLayer>::Iterator it;
-  QList<FeatureLayer>::ConstIterator FeatureListEnd = mFeatureList1.end();
+  QList<FeatureLayer>::iterator it;
 
   qDebug() << mFeatureList1.count();
 
@@ -749,7 +745,7 @@ ErrorList topolTest::checkPseudos( double tolerance, QgsVectorLayer *layer1, Qgs
 
   std::multimap<QgsPoint, QgsFeatureId, PointComparer> endVerticesMap;
 
-  for ( it = mFeatureList1.begin(); it != FeatureListEnd; ++it )
+  for ( it = mFeatureList1.begin(); it != mFeatureList1.end(); ++it )
   {
     if ( !( ++i % 100 ) )
       emit progress( i );
@@ -930,7 +926,7 @@ ErrorList topolTest::checkPointCoveredBySegment( double tolerance, QgsVectorLaye
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
 
       if ( !g2 )
       {
@@ -954,6 +950,7 @@ ErrorList topolTest::checkPointCoveredBySegment( double tolerance, QgsVectorLaye
       {
         if ( canvasExtentPoly->disjoint( conflictGeom ) )
         {
+          delete conflictGeom;
           continue;
         }
       }
@@ -982,8 +979,7 @@ ErrorList topolTest::checkSegmentLength( double tolerance, QgsVectorLayer* layer
   QgsFeature f;
 
 
-  QList<FeatureLayer>::Iterator it;
-  QList<FeatureLayer>::ConstIterator FeatureListEnd = mFeatureList1.end();
+  QList<FeatureLayer>::iterator it;
 
   QgsPolygon pol;
 
@@ -995,7 +991,7 @@ ErrorList topolTest::checkSegmentLength( double tolerance, QgsVectorLayer* layer
   TopolErrorShort* err;
   double distance;
 
-  for ( it = mFeatureList1.begin(); it != FeatureListEnd; ++it )
+  for ( it = mFeatureList1.begin(); it != mFeatureList1.end(); ++it )
   {
     if ( !( ++i % 100 ) )
     {
@@ -1139,9 +1135,8 @@ ErrorList topolTest::checkOverlapWithLayer( double tolerance, QgsVectorLayer* la
   QgsGeometry* canvasExtentPoly = QgsGeometry::fromWkt( theQgsInterface->mapCanvas()->extent().asWktPolygon() );
 
 
-  QList<FeatureLayer>::Iterator it;
-  QList<FeatureLayer>::ConstIterator FeatureListEnd = mFeatureList1.end();
-  for ( it = mFeatureList1.begin(); it != FeatureListEnd; ++it )
+  QList<FeatureLayer>::iterator it;
+  for ( it = mFeatureList1.begin(); it != mFeatureList1.end(); ++it )
   {
     if ( !( ++i % 100 ) )
       emit progress( i );
@@ -1160,7 +1155,7 @@ ErrorList topolTest::checkOverlapWithLayer( double tolerance, QgsVectorLayer* la
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
 
       // skip itself, when invoked with the same layer
       if ( skipItself && f.id() == it->feature.id() )
@@ -1178,7 +1173,7 @@ ErrorList topolTest::checkOverlapWithLayer( double tolerance, QgsVectorLayer* la
         QgsRectangle r2 = g2->boundingBox();
         r.combineExtentWith( &r2 );
 
-        QgsGeometry* conflictGeom = g1->intersection( g2 );
+        QScopedPointer<QgsGeometry> conflictGeom( g1->intersection( g2 ) );
         // could this for some reason return NULL?
         if ( !conflictGeom )
         {
@@ -1187,13 +1182,13 @@ ErrorList topolTest::checkOverlapWithLayer( double tolerance, QgsVectorLayer* la
 
         if ( isExtent )
         {
-          if ( canvasExtentPoly->disjoint( conflictGeom ) )
+          if ( canvasExtentPoly->disjoint( conflictGeom.data() ) )
           {
             continue;
           }
-          if ( canvasExtentPoly->crosses( conflictGeom ) )
+          if ( canvasExtentPoly->crosses( conflictGeom.data() ) )
           {
-            conflictGeom = conflictGeom->intersection( canvasExtentPoly );
+            conflictGeom.reset( conflictGeom->intersection( canvasExtentPoly ) );
           }
         }
 
@@ -1204,7 +1199,7 @@ ErrorList topolTest::checkOverlapWithLayer( double tolerance, QgsVectorLayer* la
         fl.feature = f;
         fl.layer = layer2;
         fls << *it << fl;
-        TopolErrorIntersection* err = new TopolErrorIntersection( r, conflictGeom, fls );
+        TopolErrorIntersection* err = new TopolErrorIntersection( r, conflictGeom.take(), fls );
 
         errorList << err;
       }
@@ -1255,13 +1250,13 @@ ErrorList topolTest::checkPointCoveredByLineEnds( double tolerance, QgsVectorLay
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
       if ( !g2 || !g2->asGeos() )
       {
         QgsMessageLog::logMessage( tr( "Second geometry missing or GEOS import failed." ), tr( "Topology plugin" ) );
         continue;
       }
-      QgsGeometry* startPoint = QgsGeometry::fromPoint( g2->asPolyline().first() );
+      QgsGeometry* startPoint = QgsGeometry::fromPoint( g2->asPolyline().at( 0 ) );
       QgsGeometry* endPoint = QgsGeometry::fromPoint( g2->asPolyline().last() );
       touched = g1->intersects( startPoint ) || g1->intersects( endPoint );
       delete startPoint;
@@ -1279,6 +1274,7 @@ ErrorList topolTest::checkPointCoveredByLineEnds( double tolerance, QgsVectorLay
       {
         if ( canvasExtentPoly->disjoint( conflictGeom ) )
         {
+          delete conflictGeom;
           continue;
         }
       }
@@ -1326,7 +1322,7 @@ ErrorList topolTest::checkyLineEndsCoveredByPoints( double tolerance, QgsVectorL
       break;
     QgsGeometry* g1 = it->feature.geometry();
 
-    QgsGeometry* startPoint = QgsGeometry::fromPoint( g1->asPolyline().first() );
+    QgsGeometry* startPoint = QgsGeometry::fromPoint( g1->asPolyline().at( 0 ) );
     QgsGeometry* endPoint = QgsGeometry::fromPoint( g1->asPolyline().last() );
 
     QgsRectangle bb = g1->boundingBox();
@@ -1342,7 +1338,7 @@ ErrorList topolTest::checkyLineEndsCoveredByPoints( double tolerance, QgsVectorL
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
       if ( !g2 || !g2->asGeos() )
       {
         QgsMessageLog::logMessage( tr( "Second geometry missing or GEOS import failed." ), tr( "Topology plugin" ) );
@@ -1372,25 +1368,24 @@ ErrorList topolTest::checkyLineEndsCoveredByPoints( double tolerance, QgsVectorL
 
     if ( !touched )
     {
-      QgsGeometry* conflictGeom = new QgsGeometry( *g1 );
+      QScopedPointer<QgsGeometry> conflictGeom( new QgsGeometry( *g1 ) );
 
       if ( isExtent )
       {
-        if ( canvasExtentPoly->disjoint( conflictGeom ) )
+        if ( canvasExtentPoly->disjoint( conflictGeom.data() ) )
         {
           continue;
         }
-        if ( canvasExtentPoly->crosses( conflictGeom ) )
+        if ( canvasExtentPoly->crosses( conflictGeom.data() ) )
         {
-          conflictGeom = conflictGeom->intersection( canvasExtentPoly );
+          conflictGeom.reset( conflictGeom->intersection( canvasExtentPoly ) );
         }
       }
       QList<FeatureLayer> fls;
       fls << *it << *it;
       //bb.scale(10);
 
-
-      TopolErrorLineEndsNotCoveredByPoints* err = new TopolErrorLineEndsNotCoveredByPoints( bb, conflictGeom, fls );
+      TopolErrorLineEndsNotCoveredByPoints* err = new TopolErrorLineEndsNotCoveredByPoints( bb, conflictGeom.take(), fls );
       errorList << err;
     }
   }
@@ -1436,7 +1431,7 @@ ErrorList topolTest::checkPointInPolygon( double tolerance, QgsVectorLayer *laye
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
       if ( !g2 || !g2->asGeos() )
       {
         QgsMessageLog::logMessage( tr( "Second geometry missing or GEOS import failed." ), tr( "Topology plugin" ) );
@@ -1456,6 +1451,7 @@ ErrorList topolTest::checkPointInPolygon( double tolerance, QgsVectorLayer *laye
       {
         if ( canvasExtentPoly->disjoint( conflictGeom ) )
         {
+          delete conflictGeom;
           continue;
         }
       }
@@ -1511,7 +1507,7 @@ ErrorList topolTest::checkPolygonContainsPoint( double tolerance, QgsVectorLayer
     for ( ; cit != crossingIdsEnd; ++cit )
     {
       QgsFeature& f = mFeatureMap2[*cit].feature;
-      QgsGeometry* g2 = f.geometry();
+      const QgsGeometry* g2 = f.constGeometry();
       if ( !g2 || !g2->asGeos() )
       {
         QgsMessageLog::logMessage( tr( "Second geometry missing or GEOS import failed." ), tr( "Topology plugin" ) );
@@ -1573,7 +1569,7 @@ ErrorList topolTest::checkMultipart( double tolerance, QgsVectorLayer *layer1, Q
   return errorList;
 }
 
-void topolTest::fillFeatureMap( QgsVectorLayer* layer, QgsRectangle extent )
+void topolTest::fillFeatureMap( QgsVectorLayer* layer, const QgsRectangle& extent )
 {
   QgsFeatureIterator fit;
   if ( extent.isEmpty() )
@@ -1592,14 +1588,14 @@ void topolTest::fillFeatureMap( QgsVectorLayer* layer, QgsRectangle extent )
 
   while ( fit.nextFeature( f ) )
   {
-    if ( f.geometry() )
+    if ( f.constGeometry() )
     {
       mFeatureMap2[f.id()] = FeatureLayer( layer, f );
     }
   }
 }
 
-void topolTest::fillFeatureList( QgsVectorLayer* layer, QgsRectangle extent )
+void topolTest::fillFeatureList( QgsVectorLayer* layer, const QgsRectangle& extent )
 {
   QgsFeatureIterator fit;
   if ( extent.isEmpty() )
@@ -1618,7 +1614,7 @@ void topolTest::fillFeatureList( QgsVectorLayer* layer, QgsRectangle extent )
 
   while ( fit.nextFeature( f ) )
   {
-    if ( f.geometry() )
+    if ( f.constGeometry() )
     {
       mFeatureList1 << FeatureLayer( layer, f );
     }
@@ -1626,7 +1622,7 @@ void topolTest::fillFeatureList( QgsVectorLayer* layer, QgsRectangle extent )
 
 }
 
-QgsSpatialIndex* topolTest::createIndex( QgsVectorLayer* layer, QgsRectangle extent )
+QgsSpatialIndex* topolTest::createIndex( QgsVectorLayer* layer, const QgsRectangle& extent )
 {
   QgsSpatialIndex* index = new QgsSpatialIndex();
 
@@ -1654,10 +1650,10 @@ QgsSpatialIndex* topolTest::createIndex( QgsVectorLayer* layer, QgsRectangle ext
     if ( testCancelled() )
     {
       delete index;
-      return 0;
+      return nullptr;
     }
 
-    if ( f.geometry() )
+    if ( f.constGeometry() )
     {
       index->insertFeature( f );
       mFeatureMap2[f.id()] = FeatureLayer( layer, f );
@@ -1667,7 +1663,7 @@ QgsSpatialIndex* topolTest::createIndex( QgsVectorLayer* layer, QgsRectangle ext
   return index;
 }
 
-ErrorList topolTest::runTest( QString testName, QgsVectorLayer* layer1, QgsVectorLayer* layer2, ValidateType type, double tolerance )
+ErrorList topolTest::runTest( const QString& testName, QgsVectorLayer* layer1, QgsVectorLayer* layer2, ValidateType type, double tolerance )
 {
   QgsDebugMsg( QString( "Running test %1" ).arg( testName ) );
   ErrorList errors;
