@@ -32,10 +32,6 @@ QgsGdalProviderBase::QgsGdalProviderBase()
   QgsGdalProviderBase::registerGdalDrivers();
 }
 
-QgsGdalProviderBase::~QgsGdalProviderBase()
-{
-}
-
 /**
  * @param theBandNumber the number of the band for which you want a color table
  * @param theList a pointer the object that will hold the color table
@@ -75,11 +71,11 @@ QList<QgsColorRampShader::ColorRampItem> QgsGdalProviderBase::colorTable( GDALDa
 
     int myEntryCount = GDALGetColorEntryCount( myGdalColorTable );
     GDALColorInterp myColorInterpretation =  GDALGetRasterColorInterpretation( myGdalBand );
-    QgsDebugMsg( "Color Interpretation: " + QString::number(( int )myColorInterpretation ) );
+    QgsDebugMsg( "Color Interpretation: " + QString::number( static_cast< int >( myColorInterpretation ) ) );
     GDALPaletteInterp myPaletteInterpretation  = GDALGetPaletteInterpretation( myGdalColorTable );
-    QgsDebugMsg( "Palette Interpretation: " + QString::number(( int )myPaletteInterpretation ) );
+    QgsDebugMsg( "Palette Interpretation: " + QString::number( static_cast< int >( myPaletteInterpretation ) ) );
 
-    const GDALColorEntry* myColorEntry = 0;
+    const GDALColorEntry* myColorEntry = nullptr;
     for ( int myIterator = 0; myIterator < myEntryCount; myIterator++ )
     {
       myColorEntry = GDALGetColorEntry( myGdalColorTable, myIterator );
@@ -99,7 +95,7 @@ QList<QgsColorRampShader::ColorRampItem> QgsGdalProviderBase::colorTable( GDALDa
         if ( myColorInterpretation == GCI_GrayIndex )
         {
           QgsColorRampShader::ColorRampItem myColorRampItem;
-          myColorRampItem.value = ( double )myIterator;
+          myColorRampItem.value = static_cast< double >( myIterator );
           myColorRampItem.label = label;
           myColorRampItem.color = QColor::fromRgb( myColorEntry->c1, myColorEntry->c1, myColorEntry->c1, myColorEntry->c4 );
           ct.append( myColorRampItem );
@@ -107,7 +103,7 @@ QList<QgsColorRampShader::ColorRampItem> QgsGdalProviderBase::colorTable( GDALDa
         else if ( myColorInterpretation == GCI_PaletteIndex )
         {
           QgsColorRampShader::ColorRampItem myColorRampItem;
-          myColorRampItem.value = ( double )myIterator;
+          myColorRampItem.value = static_cast< double >( myIterator );
           myColorRampItem.label = label;
           //Branch on palette interpretation
           if ( myPaletteInterpretation  == GPI_RGB )
@@ -146,107 +142,77 @@ QList<QgsColorRampShader::ColorRampItem> QgsGdalProviderBase::colorTable( GDALDa
   return ct;
 }
 
-QGis::DataType QgsGdalProviderBase::dataTypeFromGdal( int theGdalDataType ) const
+QGis::DataType QgsGdalProviderBase::dataTypeFromGdal( const GDALDataType theGdalDataType ) const
 {
   switch ( theGdalDataType )
   {
-    case GDT_Unknown:
-      return QGis::UnknownDataType;
-      break;
     case GDT_Byte:
       return QGis::Byte;
-      break;
     case GDT_UInt16:
       return QGis::UInt16;
-      break;
     case GDT_Int16:
       return QGis::Int16;
-      break;
     case GDT_UInt32:
       return QGis::UInt32;
-      break;
     case GDT_Int32:
       return QGis::Int32;
-      break;
     case GDT_Float32:
       return QGis::Float32;
-      break;
     case GDT_Float64:
       return QGis::Float64;
-      break;
     case GDT_CInt16:
       return QGis::CInt16;
-      break;
     case GDT_CInt32:
       return QGis::CInt32;
-      break;
     case GDT_CFloat32:
       return QGis::CFloat32;
-      break;
     case GDT_CFloat64:
       return QGis::CFloat64;
-      break;
+    case GDT_Unknown:
+    case GDT_TypeCount:
+      return QGis::UnknownDataType;
   }
   return QGis::UnknownDataType;
 }
 
-int QgsGdalProviderBase::colorInterpretationFromGdal( int gdalColorInterpretation ) const
+int QgsGdalProviderBase::colorInterpretationFromGdal( const GDALColorInterp gdalColorInterpretation ) const
 {
   switch ( gdalColorInterpretation )
   {
-    case GCI_Undefined:
-      return QgsRaster::UndefinedColorInterpretation;
-      break;
     case GCI_GrayIndex:
       return QgsRaster::GrayIndex;
-      break;
     case GCI_PaletteIndex:
       return QgsRaster::PaletteIndex;
-      break;
     case GCI_RedBand:
       return QgsRaster::RedBand;
-      break;
     case GCI_GreenBand:
       return QgsRaster::GreenBand;
-      break;
     case GCI_BlueBand:
       return QgsRaster::BlueBand;
-      break;
     case GCI_AlphaBand:
       return QgsRaster::AlphaBand;
-      break;
     case GCI_HueBand:
       return QgsRaster::HueBand;
-      break;
     case GCI_SaturationBand:
       return QgsRaster::SaturationBand;
-      break;
     case GCI_LightnessBand:
       return QgsRaster::LightnessBand;
-      break;
     case GCI_CyanBand:
       return QgsRaster::CyanBand;
-      break;
     case GCI_MagentaBand:
       return QgsRaster::MagentaBand;
-      break;
     case GCI_YellowBand:
       return QgsRaster::YellowBand;
-      break;
     case GCI_BlackBand:
       return QgsRaster::BlackBand;
-      break;
     case GCI_YCbCr_YBand:
       return QgsRaster::YCbCr_YBand;
-      break;
     case GCI_YCbCr_CbBand:
       return QgsRaster::YCbCr_CbBand;
-      break;
     case GCI_YCbCr_CrBand:
       return QgsRaster::YCbCr_CrBand;
-      break;
-    default:
-      break;
+    case GCI_Undefined:
+      return QgsRaster::UndefinedColorInterpretation;
   }
   return QgsRaster::UndefinedColorInterpretation;
 }
@@ -258,7 +224,7 @@ void QgsGdalProviderBase::registerGdalDrivers()
   QString myJoinedList = mySettings.value( "gdal/skipList", "" ).toString();
   if ( !myJoinedList.isEmpty() )
   {
-    QStringList myList = myJoinedList.split( " " );
+    QStringList myList = myJoinedList.split( ' ' );
     for ( int i = 0; i < myList.size(); ++i )
     {
       QgsApplication::skipGdalDriver( myList.at( i ) );
