@@ -31,7 +31,7 @@ QMap<QString, QgsOracleConn *> QgsOracleConn::sConnections;
 int QgsOracleConn::snConnections = 0;
 const int QgsOracleConn::sGeomTypeSelectLimit = 100;
 
-QgsOracleConn *QgsOracleConn::connectDb( QgsDataSourceURI uri )
+QgsOracleConn *QgsOracleConn::connectDb( const QgsDataSourceURI& uri )
 {
   QString conninfo = uri.connectionInfo();
 
@@ -361,7 +361,7 @@ QString QgsOracleConn::fieldExpression( const QgsField &fld )
 
 void QgsOracleConn::retrieveLayerTypes( QgsOracleLayerProperty &layerProperty, bool useEstimatedMetadata, bool onlyExistingTypes )
 {
-  QgsDebugMsg( "entering: " + layerProperty.toString() );
+  QgsDebugMsgLevel( "entering: " + layerProperty.toString(), 3 );
 
   if ( layerProperty.isView )
   {
@@ -657,6 +657,7 @@ void QgsOracleConn::deleteConnection( QString theConnName )
   settings.remove( key + "/allowGeometrylessTables" );
   settings.remove( key + "/estimatedMetadata" );
   settings.remove( key + "/onlyExistingTypes" );
+  settings.remove( key + "/includeGeoAttributes" );
   settings.remove( key + "/saveUsername" );
   settings.remove( key + "/savePassword" );
   settings.remove( key + "/save" );
@@ -677,7 +678,7 @@ void QgsOracleConn::setSelectedConnection( QString name )
 
 QgsDataSourceURI QgsOracleConn::connUri( QString theConnName )
 {
-  QgsDebugMsg( "theConnName = " + theConnName );
+  QgsDebugMsgLevel( "theConnName = " + theConnName, 3 );
 
   QSettings settings;
 
