@@ -15,18 +15,23 @@
 
 #include "qgsoraclefeatureiterator.h"
 #include "qgsoracleprovider.h"
+#include "qgsoracleconnpool.h"
+#include "qgsoracleexpressioncompiler.h"
 
 #include "qgslogger.h"
 #include "qgsmessagelog.h"
 #include "qgsgeometry.h"
 
 #include <QObject>
+#include <QSettings>
 
 QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* source, bool ownSource, const QgsFeatureRequest &request )
     : QgsAbstractFeatureIteratorFromSource<QgsOracleFeatureSource>( source, ownSource, request )
     , mRewind( false )
+    , mExpressionCompiled( false )
+    , mFetchGeometry( false )
 {
-  mConnection = QgsOracleConn::connectDb( mSource->mUri.connectionInfo() );
+  mConnection = QgsOracleConnPool::instance()->acquireConnection( mSource->mUri.connectionInfo() );
   if ( !mConnection )
   {
     close();
@@ -40,19 +45,38 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
     mAttributeList = mRequest.subsetOfAttributes();
     if ( mAttributeList.isEmpty() )
       mAttributeList = mSource->mFields.allAttributesList();
+
+    // ensure that all attributes required for expression filter are being fetched
+    if ( mRequest.filterType() == QgsFeatureRequest::FilterExpression )
+    {
+      Q_FOREACH ( const QString& field, mRequest.filterExpression()->referencedColumns() )
+      {
+        int attrIdx = mSource->mFields.fieldNameIndex( field );
+        if ( !mAttributeList.contains( attrIdx ) )
+          mAttributeList << attrIdx;
+      }
+    }
+
   }
   else
     mAttributeList = mSource->mFields.allAttributesList();
 
+  bool limitAtProvider = ( mRequest.limit() >= 0 );
   QString whereClause;
 
-  switch ( request.filterType() )
+  if ( !mSource->mGeometryColumn.isNull() )
   {
-    case QgsFeatureRequest::FilterExpression:
-      break;
+    // fetch geometry if requested
+    mFetchGeometry = ( mRequest.flags() & QgsFeatureRequest::NoGeometry ) == 0;
+    if ( mRequest.filterType() == QgsFeatureRequest::FilterExpression && mRequest.filterExpression()->needsGeometry() )
+    {
+      mFetchGeometry = true;
+    }
 
-    case QgsFeatureRequest::FilterRect:
-      if ( !mSource->mGeometryColumn.isNull() && mSource->mHasSpatialIndex )
+    if ( !mRequest.filterRect().isNull() )
+    {
+      // sdo_filter requires spatial index
+      if ( mSource->mHasSpatialIndex )
       {
         QgsRectangle rect( mRequest.filterRect() );
         QString bbox = QString( "mdsys.sdo_geometry(2003,%1,NULL,"
@@ -67,25 +91,60 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
 
         whereClause = QString( "sdo_filter(%1,%2)='TRUE'" ).arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ) ).arg( bbox );
 
-        if ( mRequest.flags() & QgsFeatureRequest::ExactIntersect && mConnection->hasSpatial() )
+        if (( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) != 0 )
         {
-          whereClause += QString( " AND sdo_relate(%1,%2,'mask=ANYINTERACT')='TRUE'" )
-                         .arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ) )
-                         .arg( bbox );
+          // sdo_relate requires Spatial
+          if ( mConnection->hasSpatial() )
+          {
+            whereClause += QString( " AND sdo_relate(%1,%2,'mask=ANYINTERACT')='TRUE'" )
+                           .arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ) )
+                           .arg( bbox );
+          }
+          else
+          {
+            // request geometry to do exact intersect in fetchFeature
+            mFetchGeometry = true;
+          }
         }
       }
-      break;
+      else
+      {
+        // request geometry to do bbox intersect in fetchFeature
+        mFetchGeometry = true;
+      }
+    }
+  }
+  else if ( !mRequest.filterRect().isNull() )
+  {
+    QgsDebugMsg( "filterRect without geometry ignored" );
+  }
 
+  switch ( mRequest.filterType() )
+  {
     case QgsFeatureRequest::FilterFid:
-      whereClause = QgsOracleUtils::whereClause( request.filterFid(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
-      break;
+    {
+      QString fidWhereClause = QgsOracleUtils::whereClause( mRequest.filterFid(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
+      whereClause = QgsOracleUtils::andWhereClauses( whereClause, fidWhereClause );
+    }
+    break;
 
     case QgsFeatureRequest::FilterFids:
-      whereClause = QgsOracleUtils::whereClause( request.filterFids(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
-      break;
+    {
+      QString fidsWhereClause = QgsOracleUtils::whereClause( mRequest.filterFids(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
+      whereClause = QgsOracleUtils::andWhereClauses( whereClause, fidsWhereClause );
+    }
+    break;
 
     case QgsFeatureRequest::FilterNone:
       break;
+
+    case QgsFeatureRequest::FilterExpression:
+      //handled below
+      break;
+
+    case QgsFeatureRequest::FilterRect:
+      // Handled in the if-statement above
+      break;
   }
 
   if ( mSource->mRequestedGeomType != QGis::WKBUnknown && mSource->mRequestedGeomType != mSource->mDetectedGeomType )
@@ -93,7 +152,7 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
     if ( !whereClause.isEmpty() )
       whereClause += " AND ";
 
-    whereClause += QgsOracleConn::databaseTypeFilter( "featureRequest", mSource->mGeometryColumn, mSource->mRequestedGeomType );
+    whereClause += QgsOracleConn::databaseTypeFilter( "FEATUREREQUEST", mSource->mGeometryColumn, mSource->mRequestedGeomType );
   }
 
   if ( !mSource->mSqlWhereClause.isEmpty() )
@@ -103,7 +162,60 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
     whereClause += "(" + mSource->mSqlWhereClause + ")";
   }
 
-  openQuery( whereClause );
+  //NOTE - must be last added!
+  mExpressionCompiled = false;
+  QString fallbackStatement;
+  bool useFallback = false;
+  if ( request.filterType() == QgsFeatureRequest::FilterExpression )
+  {
+    if ( QSettings().value( "/qgis/compileExpressions", true ).toBool() )
+    {
+      QgsOracleExpressionCompiler compiler( mSource );
+      QgsSqlExpressionCompiler::Result result = compiler.compile( mRequest.filterExpression() );
+      if ( result == QgsSqlExpressionCompiler::Complete || result == QgsSqlExpressionCompiler::Partial )
+      {
+        fallbackStatement = whereClause;
+        useFallback = true;
+        whereClause = QgsOracleUtils::andWhereClauses( whereClause, compiler.result() );
+
+        //if only partial success when compiling expression, we need to double-check results using QGIS' expressions
+        mExpressionCompiled = ( result == QgsSqlExpressionCompiler::Complete );
+        limitAtProvider = mExpressionCompiled;
+      }
+      else
+      {
+        limitAtProvider = false;
+      }
+    }
+    else
+    {
+      limitAtProvider = false;
+    }
+  }
+
+  if ( !mRequest.orderBy().isEmpty() )
+  {
+    limitAtProvider = false;
+  }
+
+  if ( mRequest.limit() >= 0 && limitAtProvider )
+  {
+    if ( !whereClause.isEmpty() )
+      whereClause += " AND ";
+
+    whereClause += QString( "rownum<=%1" ).arg( mRequest.limit() );
+    fallbackStatement += QString( "rownum<=%1" ).arg( mRequest.limit() );
+  }
+
+  bool result = openQuery( whereClause, !useFallback );
+  if ( !result && useFallback )
+  {
+    result = openQuery( fallbackStatement );
+    if ( result )
+    {
+      mExpressionCompiled = false;
+    }
+  }
 }
 
 QgsOracleFeatureIterator::~QgsOracleFeatureIterator()
@@ -111,6 +223,14 @@ QgsOracleFeatureIterator::~QgsOracleFeatureIterator()
   close();
 }
 
+bool QgsOracleFeatureIterator::nextFeatureFilterExpression( QgsFeature& f )
+{
+  if ( !mExpressionCompiled )
+    return QgsAbstractFeatureIterator::nextFeatureFilterExpression( f );
+  else
+    return fetchFeature( f );
+}
+
 bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
 {
   feature.setValid( false );
@@ -126,39 +246,71 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
     if ( mRewind )
     {
       mRewind = false;
-      if ( !mQry.first() )
-        return true;
+      if ( !QgsOracleProvider::exec( mQry, mSql ) )
+      {
+        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL:%1\nError: %2" )
+                                   .arg( mQry.lastQuery() )
+                                   .arg( mQry.lastError().text() ),
+                                   QObject::tr( "Oracle" ) );
+        return false;
+      }
     }
-    else if ( !mQry.next() )
+    if ( !mQry.next() )
     {
       return false;
     }
 
     int col = 0;
 
-    if ((( mRequest.flags() & QgsFeatureRequest::NoGeometry ) == 0 && !mSource->mGeometryColumn.isNull() ) ||
-        (( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) != 0 && ( !mConnection->hasSpatial() || !mSource->mHasSpatialIndex ) ) )
+    if ( mFetchGeometry )
     {
       QByteArray *ba = static_cast<QByteArray*>( mQry.value( col++ ).data() );
-      unsigned char *copy = new unsigned char[ba->size()];
-      memcpy( copy, ba->constData(), ba->size() );
-
-      feature.setGeometryAndOwnership( copy, ba->size() );
-
-      if ( !mConnection->hasSpatial() &&
-           mRequest.filterType() == QgsFeatureRequest::FilterRect &&
-           ( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) != 0 &&
-           ( !feature.geometry() || !feature.geometry()->intersects( mRequest.filterRect() ) ) )
+      if ( ba->size() > 0 )
       {
-        // skip feature that don't intersect with our rectangle
-        QgsDebugMsg( "no intersect" );
-        continue;
+        unsigned char *copy = new unsigned char[ba->size()];
+        memcpy( copy, ba->constData(), ba->size() );
+
+        QgsGeometry *g = new QgsGeometry();
+        g->fromWkb( copy, ba->size() );
+        feature.setGeometry( g );
+      }
+      else
+      {
+        feature.setGeometry( 0 );
       }
 
-
-      if (( mRequest.flags() & QgsFeatureRequest::NoGeometry ) != 0 )
+      if ( !mRequest.filterRect().isNull() )
       {
-        feature.setGeometryAndOwnership( 0, 0 );
+        if ( !feature.geometry() )
+        {
+          QgsDebugMsg( "no geometry to intersect" );
+          continue;
+        }
+
+        if (( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) == 0 )
+        {
+          // couldn't use sdo_filter earlier
+          if ( !mSource->mHasSpatialIndex )
+          {
+            // only intersect with bbox
+            if ( !feature.geometry()->boundingBox().intersects( mRequest.filterRect() ) )
+            {
+              // skip feature that don't intersect with our rectangle
+              QgsDebugMsg( "no bbox intersect" );
+              continue;
+            }
+          }
+        }
+        else if ( !mConnection->hasSpatial() || !mSource->mHasSpatialIndex )
+        {
+          // couldn't use sdo_relate earlier
+          if ( !feature.geometry()->intersects( mRequest.filterRect() ) )
+          {
+            // skip feature that don't intersect with our rectangle
+            QgsDebugMsg( "no exact intersect" );
+            continue;
+          }
+        }
       }
     }
 
@@ -180,7 +332,7 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
 
         if ( mSource->mPrimaryKeyType == pktFidMap )
         {
-          foreach ( int idx, mSource->mPrimaryKeyAttrs )
+          Q_FOREACH ( int idx, mSource->mPrimaryKeyAttrs )
           {
             const QgsField &fld = mSource->mFields[idx];
 
@@ -213,7 +365,7 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
     QgsDebugMsgLevel( QString( "fid=%1" ).arg( fid ), 5 );
 
     // iterate attributes
-    foreach ( int idx, mAttributeList )
+    Q_FOREACH ( int idx, mAttributeList )
     {
       if ( mSource->mPrimaryKeyAttrs.contains( idx ) )
         continue;
@@ -221,7 +373,25 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
       const QgsField &fld = mSource->mFields[idx];
 
       QVariant v = mQry.value( col );
-      if ( v.type() != fld.type() )
+      if ( fld.type() == QVariant::ByteArray && fld.typeName().endsWith( ".SDO_GEOMETRY" ) )
+      {
+        QByteArray *ba = static_cast<QByteArray*>( v.data() );
+        if ( ba->size() > 0 )
+        {
+          unsigned char *copy = new unsigned char[ba->size()];
+          memcpy( copy, ba->constData(), ba->size() );
+
+          QgsGeometry *g = new QgsGeometry();
+          g->fromWkb( copy, ba->size() );
+          v = g->exportToWkt();
+          delete g;
+        }
+        else
+        {
+          v = QVariant( QVariant::String );
+        }
+      }
+      else if ( v.type() != fld.type() )
         v = QgsVectorDataProvider::convertValue( fld.type(), v.toString() );
       feature.setAttribute( idx, v );
 
@@ -229,7 +399,7 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
     }
 
     feature.setValid( true );
-    feature.setFields( &mSource->mFields ); // allow name-based attribute lookups
+    feature.setFields( mSource->mFields ); // allow name-based attribute lookups
 
     return true;
   }
@@ -251,7 +421,7 @@ bool QgsOracleFeatureIterator::close()
     mQry.finish();
 
   if ( mConnection )
-    mConnection->disconnect();
+    QgsOracleConnPool::instance()->releaseConnection( mConnection );
   mConnection = 0;
 
   iteratorClosed();
@@ -259,13 +429,13 @@ bool QgsOracleFeatureIterator::close()
   return true;
 }
 
-bool QgsOracleFeatureIterator::openQuery( QString whereClause )
+bool QgsOracleFeatureIterator::openQuery( QString whereClause, bool showLog )
 {
   try
   {
     QString query = "SELECT ", delim = "";
 
-    if (( mRequest.flags() & QgsFeatureRequest::NoGeometry ) == 0 && !mSource->mGeometryColumn.isNull() )
+    if ( mFetchGeometry )
     {
       query += QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn );
       delim = ",";
@@ -284,7 +454,7 @@ bool QgsOracleFeatureIterator::openQuery( QString whereClause )
         break;
 
       case pktFidMap:
-        foreach ( int idx, mSource->mPrimaryKeyAttrs )
+        Q_FOREACH ( int idx, mSource->mPrimaryKeyAttrs )
         {
           query += delim + mConnection->fieldExpression( mSource->mFields[idx] );
           delim = ",";
@@ -297,7 +467,7 @@ bool QgsOracleFeatureIterator::openQuery( QString whereClause )
         break;
     }
 
-    foreach ( int idx, mAttributeList )
+    Q_FOREACH ( int idx, mAttributeList )
     {
       if ( mSource->mPrimaryKeyAttrs.contains( idx ) )
         continue;
@@ -305,18 +475,22 @@ bool QgsOracleFeatureIterator::openQuery( QString whereClause )
       query += delim + mConnection->fieldExpression( mSource->mFields[idx] );
     }
 
-    query += QString( " FROM %1 \"featureRequest\"" ).arg( mSource->mQuery );
+    query += QString( " FROM %1 \"FEATUREREQUEST\"" ).arg( mSource->mQuery );
 
     if ( !whereClause.isEmpty() )
       query += QString( " WHERE %1" ).arg( whereClause );
 
     QgsDebugMsg( QString( "Fetch features: %1" ).arg( query ) );
+    mSql = query;
     if ( !QgsOracleProvider::exec( mQry, query ) )
     {
-      QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL:%1\nError: %2" )
-                                 .arg( mQry.lastQuery() )
-                                 .arg( mQry.lastError().text() ),
-                                 QObject::tr( "Oracle" ) );
+      if ( showLog )
+      {
+        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL:%1\nError: %2" )
+                                   .arg( mQry.lastQuery() )
+                                   .arg( mQry.lastError().text() ),
+                                   QObject::tr( "Oracle" ) );
+      }
       return false;
     }
   }
