@@ -75,7 +75,7 @@ class QgsOracleProvider : public QgsVectorDataProvider
      * @param uri String containing the required parameters to connect to the database
      * and query the table.
      */
-    QgsOracleProvider( QString const &uri = "" );
+    explicit QgsOracleProvider( QString const &uri = "" );
 
     //! Destructor
     virtual ~QgsOracleProvider();
@@ -87,7 +87,7 @@ class QgsOracleProvider : public QgsVectorDataProvider
       */
     virtual QString storageType() const;
 
-    /*! Get the QgsCoordinateReferenceSystem for this layer
+    /** Get the QgsCoordinateReferenceSystem for this layer
      * @note Must be reimplemented by each provider.
      * If the provider isn't capable of returning
      * its projection an empty srs will be returned
@@ -105,9 +105,9 @@ class QgsOracleProvider : public QgsVectorDataProvider
      */
     QGis::WkbType geometryType() const;
 
-    /** return the number of layers for the current data source
+    /** Return the number of layers for the current data source
      * @note Should this be subLayerCount() instead?
-    */
+     */
     size_t layerCount() const;
 
     /**
@@ -132,15 +132,15 @@ class QgsOracleProvider : public QgsVectorDataProvider
     void setExtent( QgsRectangle& newExtent );
 
     /** Return the extent for this data layer
-    */
+     */
     virtual QgsRectangle extent();
 
     /** Update the extent
-    */
+     */
     virtual void updateExtents();
 
     /** Determine the fields making up the primary key
-    */
+     */
     bool determinePrimaryKey();
 
     /**
@@ -172,38 +172,38 @@ class QgsOracleProvider : public QgsVectorDataProvider
      *  @param values reference to the list of unique values */
     virtual void uniqueValues( int index, QList<QVariant> &uniqueValues, int limit = -1 );
 
-    /**Returns true if layer is valid
-    */
+    /** Returns true if layer is valid
+     */
     bool isValid();
 
     QgsAttributeList pkAttributeIndexes() { return mPrimaryKeyAttrs; }
 
-    /**Returns the default value for field specified by @c fieldName */
+    /** Returns the default value for field specified by @c fieldName */
     QVariant defaultValue( QString fieldName, QString tableName = QString::null, QString schemaName = QString::null );
 
-    /**Returns the default value for field specified by @c fieldId */
+    /** Returns the default value for field specified by @c fieldId */
     QVariant defaultValue( int fieldId );
 
-    /**Adds a list of features
+    /** Adds a list of features
       @return true in case of success and false in case of failure*/
     bool addFeatures( QgsFeatureList & flist );
 
-    /**Deletes a list of features
+    /** Deletes a list of features
       @param id list of feature ids
       @return true in case of success and false in case of failure*/
     bool deleteFeatures( const QgsFeatureIds & id );
 
-    /**Adds new attributes
+    /** Adds new attributes
       @param name map with attribute name as key and type as value
       @return true in case of success and false in case of failure*/
     bool addAttributes( const QList<QgsField> &attributes );
 
-    /**Deletes existing attributes
+    /** Deletes existing attributes
       @param names of the attributes to delete
       @return true in case of success and false in case of failure*/
     bool deleteAttributes( const QgsAttributeIds & name );
 
-    /**Changes attribute values of existing features
+    /** Changes attribute values of existing features
       @param attr_map a map containing the new attributes. The integer is the feature id,
       the first QString is the attribute name and the second one is the new attribute value
       @return true in case of success and false in case of failure*/
@@ -215,9 +215,9 @@ class QgsOracleProvider : public QgsVectorDataProvider
                              the second map parameter being the new geometries themselves
        @return               true in case of success and false in case of failure
      */
-    bool changeGeometryValues( QgsGeometryMap & geometry_map );
+    bool changeGeometryValues( const QgsGeometryMap & geometry_map );
 
-    /**Tries to create an spatial index file for faster access if only a subset of the features is required
+    /** Tries to create an spatial index file for faster access if only a subset of the features is required
      @return true in case of success*/
     bool createSpatialIndex();
 
@@ -227,41 +227,41 @@ class QgsOracleProvider : public QgsVectorDataProvider
     /** Accessor for sql where clause used to limit dataset */
     QString subsetString();
 
-    /** mutator for sql where clause used to limit dataset size */
-    bool setSubsetString( QString theSQL, bool updateFeatureCount = true );
+    /** Mutator for sql where clause used to limit dataset size */
+    bool setSubsetString( const QString& theSQL, bool updateFeatureCount = true );
 
     virtual bool supportsSubsetString() { return true; }
 
-    /**Returns a bitmask containing the supported capabilities*/
+    /** Returns a bitmask containing the supported capabilities*/
     int capabilities() const;
 
-    /** return a provider name
-
-    Essentially just returns the provider key.  Should be used to build file
-    dialogs so that providers can be shown with their supported types. Thus
-    if more than one provider supports a given format, the user is able to
-    select a specific provider to open that file.
-
-    @note
-
-    Instead of being pure virtual, might be better to generalize this
-    behavior and presume that none of the sub-classes are going to do
-    anything strange with regards to their name or description?
-
-    */
+    /** Return a provider name
+     *
+     * Essentially just returns the provider key.  Should be used to build file
+     * dialogs so that providers can be shown with their supported types. Thus
+     * if more than one provider supports a given format, the user is able to
+     * select a specific provider to open that file.
+     *
+     * @note
+     *
+     * Instead of being pure virtual, might be better to generalize this
+     * behavior and presume that none of the sub-classes are going to do
+     * anything strange with regards to their name or description?
+     *
+     */
     QString name() const;
 
-    /** return description
-
-    Return a terse string describing what the provider is.
-
-    @note
-
-    Instead of being pure virtual, might be better to generalize this
-    behavior and presume that none of the sub-classes are going to do
-    anything strange with regards to their name or description?
-
-    */
+    /** Return description
+     *
+     * Return a terse string describing what the provider is.
+     *
+     * @note
+     *
+     * Instead of being pure virtual, might be better to generalize this
+     * behavior and presume that none of the sub-classes are going to do
+     * anything strange with regards to their name or description?
+     *
+     */
     QString description() const;
 
     /**
@@ -283,10 +283,10 @@ class QgsOracleProvider : public QgsVectorDataProvider
     const QgsField &field( int index ) const;
 
     /** Load the field list
-    */
+     */
     bool loadFields();
 
-    /** convert a QgsField to work with Oracle */
+    /** Convert a QgsField to work with Oracle */
     static bool convertField( QgsField &field );
 
     QgsFields mAttributeFields;  //! List of fields
@@ -348,6 +348,9 @@ class QgsOracleProvider : public QgsVectorDataProvider
     /* Use estimated metadata. Uses fast table counts, geometry type and extent determination */
     bool mUseEstimatedMetadata;
 
+    /* Include additional geo attributes */
+    bool mIncludeGeoAttributes;
+
     struct OracleFieldNotFound {}; //! Exception to throw
 
     struct OracleException
@@ -397,7 +400,6 @@ class QgsOracleProvider : public QgsVectorDataProvider
     QgsFeatureId mFidCounter;                //! next feature id if map is used
     QgsOracleConn *mConnection;
 
-    bool mHasSpatial;                        //! Oracle Spatial is installed
     bool mHasSpatialIndex;                   //! Geometry column is indexed
     QString mSpatialIndexName;               //! name of spatial index of geometry column
 
@@ -423,6 +425,8 @@ class QgsOracleUtils
                                 QgsOraclePrimaryKeyType primaryKeyType,
                                 const QList<int>& primaryKeyAttrs,
                                 QSharedPointer<QgsOracleSharedData> sharedData );
+
+    static QString andWhereClauses( const QString& c1, const QString& c2 );
 };
 
 
