@@ -36,6 +36,7 @@ email                : sherman at mrcc.com
 #include <QTextStream>
 #include <QHeaderView>
 #include <QStringList>
+#include <QStyledItemDelegate>
 
 /** Used to create an editor for when the user tries to change the contents of a cell */
 QWidget *QgsPgSourceSelectDelegate::createEditor( QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index ) const
@@ -44,7 +45,7 @@ QWidget *QgsPgSourceSelectDelegate::createEditor( QWidget *parent, const QStyleO
 
   QString tableName = index.sibling( index.row(), QgsPgTableModel::dbtmTable ).data( Qt::DisplayRole ).toString();
   if ( tableName.isEmpty() )
-    return 0;
+    return nullptr;
 
   if ( index.column() == QgsPgTableModel::dbtmSql )
   {
@@ -54,15 +55,15 @@ QWidget *QgsPgSourceSelectDelegate::createEditor( QWidget *parent, const QStyleO
   if ( index.column() == QgsPgTableModel::dbtmType && index.data( Qt::UserRole + 1 ).toBool() )
   {
     QComboBox *cb = new QComboBox( parent );
-    foreach ( QGis::WkbType type,
-              QList<QGis::WkbType>()
-              << QGis::WKBPoint
-              << QGis::WKBLineString
-              << QGis::WKBPolygon
-              << QGis::WKBMultiPoint
-              << QGis::WKBMultiLineString
-              << QGis::WKBMultiPolygon
-              << QGis::WKBNoGeometry )
+    Q_FOREACH ( QGis::WkbType type,
+                QList<QGis::WkbType>()
+                << QGis::WKBPoint
+                << QGis::WKBLineString
+                << QGis::WKBPolygon
+                << QGis::WKBMultiPoint
+                << QGis::WKBMultiLineString
+                << QGis::WKBMultiPolygon
+                << QGis::WKBNoGeometry )
     {
       cb->addItem( QgsPgTableModel::iconForWkbType( type ), QgsPostgresConn::displayStringForWkbType( type ), type );
     }
@@ -73,10 +74,25 @@ QWidget *QgsPgSourceSelectDelegate::createEditor( QWidget *parent, const QStyleO
   {
     QStringList values = index.data( Qt::UserRole + 1 ).toStringList();
 
-    if ( values.size() > 0 )
+    if ( !values.isEmpty() )
     {
       QComboBox *cb = new QComboBox( parent );
-      cb->addItems( values );
+      cb->setItemDelegate( new QStyledItemDelegate( parent ) );
+
+      QStandardItemModel *model = new QStandardItemModel( values.size(), 1, cb );
+
+      int row = 0;
+      Q_FOREACH ( const QString& value, values )
+      {
+        QStandardItem *item = new QStandardItem( value );
+        item->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
+        item->setCheckable( true );
+        item->setData( Qt::Unchecked, Qt::CheckStateRole );
+        model->setItem( row++, 0, item );
+      }
+
+      cb->setModel( model );
+
       return cb;
     }
   }
@@ -88,7 +104,7 @@ QWidget *QgsPgSourceSelectDelegate::createEditor( QWidget *parent, const QStyleO
     return le;
   }
 
-  return 0;
+  return nullptr;
 }
 
 void QgsPgSourceSelectDelegate::setEditorData( QWidget *editor, const QModelIndex &index ) const
@@ -101,8 +117,25 @@ void QgsPgSourceSelectDelegate::setEditorData( QWidget *editor, const QModelInde
     if ( index.column() == QgsPgTableModel::dbtmType )
       cb->setCurrentIndex( cb->findData( index.data( Qt::UserRole + 2 ).toInt() ) );
 
-    if ( index.column() == QgsPgTableModel::dbtmPkCol && !index.data( Qt::UserRole + 2 ).toString().isEmpty() )
-      cb->setCurrentIndex( cb->findText( index.data( Qt::UserRole + 2 ).toString() ) );
+    if ( index.column() == QgsPgTableModel::dbtmPkCol && !index.data( Qt::UserRole + 2 ).toStringList().isEmpty() )
+    {
+      QStringList cols = index.data( Qt::UserRole + 2 ).toStringList();
+
+      Q_FOREACH ( const QString& col, cols )
+      {
+        QStandardItemModel *cbm = qobject_cast<QStandardItemModel*>( cb->model() );
+        for ( int idx = 0; idx < cbm->rowCount(); idx++ )
+        {
+          QStandardItem *item = cbm->item( idx, 0 );
+          if ( item->text() != col )
+            continue;
+
+          item->setData( Qt::Checked, Qt::CheckStateRole );
+          break;
+        }
+      }
+
+    }
   }
 
   QLineEdit *le = qobject_cast<QLineEdit*>( editor );
@@ -132,9 +165,17 @@ void QgsPgSourceSelectDelegate::setModelData( QWidget *editor, QAbstractItemMode
     }
     else if ( index.column() == QgsPgTableModel::dbtmPkCol )
     {
-      QString value( cb->currentText() );
-      model->setData( index, value.isEmpty() ? tr( "Select..." ) : value );
-      model->setData( index, value, Qt::UserRole + 2 );
+      QStandardItemModel *cbm = qobject_cast<QStandardItemModel*>( cb->model() );
+      QStringList cols;
+      for ( int idx = 0; idx < cbm->rowCount(); idx++ )
+      {
+        QStandardItem *item = cbm->item( idx, 0 );
+        if ( item->data( Qt::CheckStateRole ) == Qt::Checked )
+          cols << item->text();
+      }
+
+      model->setData( index, cols.isEmpty() ? tr( "Select..." ) : cols.join( ", " ) );
+      model->setData( index, cols, Qt::UserRole + 2 );
     }
   }
 
@@ -156,7 +197,7 @@ QgsPgSourceSelect::QgsPgSourceSelect( QWidget *parent, Qt::WindowFlags fl, bool
     : QDialog( parent, fl )
     , mManagerMode( managerMode )
     , mEmbeddedMode( embeddedMode )
-    , mColumnTypeThread( 0 )
+    , mColumnTypeThread( nullptr )
     , mUseEstimatedMetadata( false )
 {
   setupUi( this );
@@ -194,9 +235,10 @@ QgsPgSourceSelect::QgsPgSourceSelect( QWidget *parent, Qt::WindowFlags fl, bool
   mSearchColumnComboBox->addItem( tr( "All" ) );
   mSearchColumnComboBox->addItem( tr( "Schema" ) );
   mSearchColumnComboBox->addItem( tr( "Table" ) );
+  mSearchColumnComboBox->addItem( tr( "Comment" ) );
   mSearchColumnComboBox->addItem( tr( "Type" ) );
   mSearchColumnComboBox->addItem( tr( "Geometry column" ) );
-  mSearchColumnComboBox->addItem( tr( "Primary key column" ) );
+  mSearchColumnComboBox->addItem( tr( "Feature id" ) );
   mSearchColumnComboBox->addItem( tr( "SRID" ) );
   mSearchColumnComboBox->addItem( tr( "Sql" ) );
 
@@ -271,7 +313,7 @@ void QgsPgSourceSelect::on_btnSave_clicked()
 
 void QgsPgSourceSelect::on_btnLoad_clicked()
 {
-  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), ".",
+  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), QDir::homePath(),
                      tr( "XML files (*.xml *XML)" ) );
   if ( fileName.isEmpty() )
   {
@@ -370,6 +412,10 @@ void QgsPgSourceSelect::on_mSearchColumnComboBox_currentIndexChanged( const QStr
   {
     mProxyModel.setFilterKeyColumn( QgsPgTableModel::dbtmTable );
   }
+  else if ( text == tr( "Comment" ) )
+  {
+    mProxyModel.setFilterKeyColumn( QgsPgTableModel::dbtmComment );
+  }
   else if ( text == tr( "Type" ) )
   {
     mProxyModel.setFilterKeyColumn( QgsPgTableModel::dbtmType );
@@ -378,7 +424,7 @@ void QgsPgSourceSelect::on_mSearchColumnComboBox_currentIndexChanged( const QStr
   {
     mProxyModel.setFilterKeyColumn( QgsPgTableModel::dbtmGeomCol );
   }
-  else if ( text == tr( "Primary key column" ) )
+  else if ( text == tr( "Feature id" ) )
   {
     mProxyModel.setFilterKeyColumn( QgsPgTableModel::dbtmPkCol );
   }
@@ -443,12 +489,12 @@ void QgsPgSourceSelect::addTables()
 {
   mSelectedTables.clear();
 
-  foreach ( QModelIndex idx, mTablesTreeView->selectionModel()->selection().indexes() )
+  Q_FOREACH ( const QModelIndex& idx, mTablesTreeView->selectionModel()->selection().indexes() )
   {
     if ( idx.column() != QgsPgTableModel::dbtmTable )
       continue;
 
-    QString uri = mTableModel.layerURI( mProxyModel.mapToSource( idx ), mConnInfo, mUseEstimatedMetadata );
+    QString uri = mTableModel.layerURI( mProxyModel.mapToSource( idx ), connectionInfo( false ), mUseEstimatedMetadata );
     if ( uri.isNull() )
       continue;
 
@@ -485,9 +531,9 @@ void QgsPgSourceSelect::on_btnConnect_clicked()
   // populate the table list
   QgsDataSourceURI uri = QgsPostgresConn::connUri( cmbConnections->currentText() );
 
-  QgsDebugMsg( "Connection info: " + uri.connectionInfo() );
+  QgsDebugMsg( "Connection info: " + uri.connectionInfo( false ) );
 
-  mConnInfo = uri.connectionInfo();
+  mDataSrcUri = uri;
   mUseEstimatedMetadata = uri.useEstimatedMetadata();
 
   QApplication::setOverrideCursor( Qt::BusyCursor );
@@ -526,7 +572,7 @@ void QgsPgSourceSelect::finishList()
 void QgsPgSourceSelect::columnThreadFinished()
 {
   delete mColumnTypeThread;
-  mColumnTypeThread = 0;
+  mColumnTypeThread = nullptr;
   btnConnect->setText( tr( "Connect" ) );
 
   finishList();
@@ -537,9 +583,14 @@ QStringList QgsPgSourceSelect::selectedTables()
   return mSelectedTables;
 }
 
-QString QgsPgSourceSelect::connectionInfo()
+QString QgsPgSourceSelect::connectionInfo( bool expandAuthCfg )
 {
-  return mConnInfo;
+  return mDataSrcUri.connectionInfo( expandAuthCfg );
+}
+
+QgsDataSourceURI QgsPgSourceSelect::dataSourceUri()
+{
+  return mDataSrcUri;
 }
 
 void QgsPgSourceSelect::setSql( const QModelIndex &index )
@@ -553,7 +604,7 @@ void QgsPgSourceSelect::setSql( const QModelIndex &index )
   QModelIndex idx = mProxyModel.mapToSource( index );
   QString tableName = mTableModel.itemFromIndex( idx.sibling( idx.row(), QgsPgTableModel::dbtmTable ) )->text();
 
-  QString uri = mTableModel.layerURI( idx, mConnInfo, mUseEstimatedMetadata );
+  QString uri = mTableModel.layerURI( idx, connectionInfo( false ), mUseEstimatedMetadata );
   if ( uri.isNull() )
   {
     QgsDebugMsg( "no uri" );
@@ -582,7 +633,7 @@ QString QgsPgSourceSelect::fullDescription( const QString& schema, const QString
 {
   QString full_desc = "";
   if ( !schema.isEmpty() )
-    full_desc = QgsPostgresConn::quotedIdentifier( schema ) + ".";
+    full_desc = QgsPostgresConn::quotedIdentifier( schema ) + '.';
   full_desc += QgsPostgresConn::quotedIdentifier( table ) + " (" + column + ") " + type;
   return full_desc;
 }
