@@ -19,6 +19,7 @@
 
 #include "ui_qgsdbsourceselectbase.h"
 #include "qgisgui.h"
+#include "qgsdatasourceuri.h"
 #include "qgsdbfilterproxymodel.h"
 #include "qgspgtablemodel.h"
 #include "qgscontexthelp.h"
@@ -36,10 +37,10 @@ class QgsPgSourceSelect;
 
 class QgsPgSourceSelectDelegate : public QItemDelegate
 {
-    Q_OBJECT;
+    Q_OBJECT
 
   public:
-    QgsPgSourceSelectDelegate( QObject *parent = NULL )
+    explicit QgsPgSourceSelectDelegate( QObject *parent = nullptr )
         : QItemDelegate( parent )
     {}
 
@@ -49,7 +50,7 @@ class QgsPgSourceSelectDelegate : public QItemDelegate
 };
 
 
-/*! \class QgsPgSourceSelect
+/** \class QgsPgSourceSelect
  * \brief Dialog to create connections and add tables from PostgresQL.
  *
  * This dialog allows the user to define and save connection information
@@ -62,7 +63,7 @@ class QgsPgSourceSelect : public QDialog, private Ui::QgsDbSourceSelectBase
 
   public:
     //! Constructor
-    QgsPgSourceSelect( QWidget *parent = 0, Qt::WindowFlags fl = QgisGui::ModalDialogFlags, bool managerMode = false, bool embeddedMode = false );
+    QgsPgSourceSelect( QWidget *parent = nullptr, Qt::WindowFlags fl = QgisGui::ModalDialogFlags, bool managerMode = false, bool embeddedMode = false );
     //! Destructor
     ~QgsPgSourceSelect();
     //! Populate the connection list combo box
@@ -70,7 +71,9 @@ class QgsPgSourceSelect : public QDialog, private Ui::QgsDbSourceSelectBase
     //! String list containing the selected tables
     QStringList selectedTables();
     //! Connection info (database, host, user, password)
-    QString connectionInfo();
+    QString connectionInfo( bool expandAuthCfg = true );
+    //! Data source URI
+    QgsDataSourceURI dataSourceUri();
 
   signals:
     void addDatabaseLayers( QStringList const & layerPathList, QString const & providerKey );
@@ -84,9 +87,9 @@ class QgsPgSourceSelect : public QDialog, private Ui::QgsDbSourceSelectBase
     void addTables();
     void buildQuery();
 
-    /*! Connects to the database using the stored connection parameters.
-    * Once connected, available layers are displayed.
-    */
+    /** Connects to the database using the stored connection parameters.
+     * Once connected, available layers are displayed.
+     */
     void on_btnConnect_clicked();
     void on_cbxAllowGeometrylessTables_stateChanged( int );
     //! Opens the create connection dialog to build a new connection
@@ -139,7 +142,7 @@ class QgsPgSourceSelect : public QDialog, private Ui::QgsDbSourceSelectBase
     QStringList mColumnLabels;
     // Our thread for doing long running queries
     QgsGeomColumnTypeThread* mColumnTypeThread;
-    QString mConnInfo;
+    QgsDataSourceURI mDataSrcUri;
     QStringList mSelectedTables;
     bool mUseEstimatedMetadata;
     // Storage for the range of layer type icons
