@@ -35,13 +35,24 @@ inline void qgsConnectionPool_ConnectionDestroy( QgsPostgresConn* c )
   c->unref(); // will delete itself
 }
 
+inline void qgsConnectionPool_InvalidateConnection( QgsPostgresConn* c )
+{
+  Q_UNUSED( c );
+}
+
+inline bool qgsConnectionPool_ConnectionIsValid( QgsPostgresConn* c )
+{
+  Q_UNUSED( c );
+  return true;
+}
+
 
 class QgsPostgresConnPoolGroup : public QObject, public QgsConnectionPoolGroup<QgsPostgresConn*>
 {
     Q_OBJECT
 
   public:
-    QgsPostgresConnPoolGroup( QString name ) : QgsConnectionPoolGroup<QgsPostgresConn*>( name ) { initTimer( this ); }
+    explicit QgsPostgresConnPoolGroup( QString name ) : QgsConnectionPoolGroup<QgsPostgresConn*>( name ) { initTimer( this ); }
 
   protected slots:
     void handleConnectionExpired() { onConnectionExpired(); }
@@ -60,7 +71,7 @@ class QgsPostgresConnPool : public QgsConnectionPool<QgsPostgresConn*, QgsPostgr
     static QgsPostgresConnPool* instance();
 
   protected:
-    Q_DISABLE_COPY( QgsPostgresConnPool );
+    Q_DISABLE_COPY( QgsPostgresConnPool )
 
   private:
     QgsPostgresConnPool();
