@@ -17,54 +17,179 @@
 #include "qgsspatialiteconnection.h"
 #include "qgsspatialiteconnpool.h"
 #include "qgsspatialiteprovider.h"
+#include "qgsspatialiteexpressioncompiler.h"
 
+#include "qgsgeometry.h"
 #include "qgslogger.h"
 #include "qgsmessagelog.h"
-
-
+#include <QSettings>
 
 QgsSpatiaLiteFeatureIterator::QgsSpatiaLiteFeatureIterator( QgsSpatiaLiteFeatureSource* source, bool ownSource, const QgsFeatureRequest& request )
     : QgsAbstractFeatureIteratorFromSource<QgsSpatiaLiteFeatureSource>( source, ownSource, request )
-    , sqliteStatement( NULL )
+    , sqliteStatement( nullptr )
+    , mExpressionCompiled( false )
 {
 
   mHandle = QgsSpatiaLiteConnPool::instance()->acquireConnection( mSource->mSqlitePath );
 
   mFetchGeometry = !mSource->mGeometryColumn.isNull() && !( mRequest.flags() & QgsFeatureRequest::NoGeometry );
+  mHasPrimaryKey = !mSource->mPrimaryKey.isEmpty();
+  mRowNumber = 0;
 
+  QStringList whereClauses;
+  bool useFallbackWhereClause = false;
+  QString fallbackWhereClause;
   QString whereClause;
-  if ( request.filterType() == QgsFeatureRequest::FilterRect && !mSource->mGeometryColumn.isNull() )
+
+  //beware - limitAtProvider needs to be set to false if the request cannot be completely handled
+  //by the provider (eg utilising QGIS expression filters)
+  bool limitAtProvider = ( mRequest.limit() >= 0 );
+
+  if ( !request.filterRect().isNull() && !mSource->mGeometryColumn.isNull() )
   {
     // some kind of MBR spatial filtering is required
-    whereClause += whereClauseRect();
-  }
-
-  if ( request.filterType() == QgsFeatureRequest::FilterFid )
-  {
-    whereClause += whereClauseFid();
-  }
-
-  if ( request.filterType() == QgsFeatureRequest::FilterFids )
-  {
-    whereClause += whereClauseFids();
+    whereClause = whereClauseRect();
+    if ( ! whereClause.isEmpty() )
+    {
+      whereClauses.append( whereClause );
+    }
   }
 
   if ( !mSource->mSubsetString.isEmpty() )
   {
-    if ( !whereClause.isEmpty() )
+    whereClause = "( " + mSource->mSubsetString + ')';
+    if ( ! whereClause.isEmpty() )
     {
-      whereClause += " AND ";
+      whereClauses.append( whereClause );
     }
-    whereClause += "( " + mSource->mSubsetString + ")";
   }
 
+  if ( request.filterType() == QgsFeatureRequest::FilterFid )
+  {
+    whereClause = whereClauseFid();
+    if ( ! whereClause.isEmpty() )
+    {
+      whereClauses.append( whereClause );
+    }
+  }
+  else if ( request.filterType() == QgsFeatureRequest::FilterFids )
+  {
+    whereClause = whereClauseFids();
+    if ( ! whereClause.isEmpty() )
+    {
+      whereClauses.append( whereClause );
+    }
+  }
+  //IMPORTANT - this MUST be the last clause added!
+  else if ( request.filterType() == QgsFeatureRequest::FilterExpression )
+  {
+    // ensure that all attributes required for expression filter are being fetched
+    if ( mRequest.flags() & QgsFeatureRequest::SubsetOfAttributes && request.filterType() == QgsFeatureRequest::FilterExpression )
+    {
+      QgsAttributeList attrs = request.subsetOfAttributes();
+      Q_FOREACH ( const QString& field, request.filterExpression()->referencedColumns() )
+      {
+        int attrIdx = mSource->mFields.fieldNameIndex( field );
+        if ( !attrs.contains( attrIdx ) )
+          attrs << attrIdx;
+      }
+      mRequest.setSubsetOfAttributes( attrs );
+    }
+    if ( request.filterExpression()->needsGeometry() )
+    {
+      mFetchGeometry = true;
+    }
+
+    if ( QSettings().value( "/qgis/compileExpressions", true ).toBool() )
+    {
+      QgsSpatiaLiteExpressionCompiler compiler = QgsSpatiaLiteExpressionCompiler( source );
+
+      QgsSqlExpressionCompiler::Result result = compiler.compile( request.filterExpression() );
+
+      if ( result == QgsSqlExpressionCompiler::Complete || result == QgsSqlExpressionCompiler::Partial )
+      {
+        whereClause = compiler.result();
+        if ( !whereClause.isEmpty() )
+        {
+          useFallbackWhereClause = true;
+          fallbackWhereClause = whereClauses.join( " AND " );
+          whereClauses.append( whereClause );
+          //if only partial success when compiling expression, we need to double-check results using QGIS' expressions
+          mExpressionCompiled = ( result == QgsSqlExpressionCompiler::Complete );
+        }
+      }
+      if ( result != QgsSqlExpressionCompiler::Complete )
+      {
+        //can't apply limit at provider side as we need to check all results using QGIS expressions
+        limitAtProvider = false;
+      }
+    }
+    else
+    {
+      limitAtProvider = false;
+    }
+  }
+
+
+  whereClause = whereClauses.join( " AND " );
+
+  // Setup the order by
+  QStringList orderByParts;
+
+  mOrderByCompiled = true;
+
+  if ( QSettings().value( "/qgis/compileExpressions", true ).toBool() )
+  {
+    Q_FOREACH ( const QgsFeatureRequest::OrderByClause& clause, request.orderBy() )
+    {
+      QgsSpatiaLiteExpressionCompiler compiler = QgsSpatiaLiteExpressionCompiler( source );
+      QgsExpression expression = clause.expression();
+      if ( compiler.compile( &expression ) == QgsSqlExpressionCompiler::Complete )
+      {
+        QString part;
+        part = compiler.result();
+
+        if ( clause.nullsFirst() )
+          orderByParts << QString( "%1 IS NOT NULL" ).arg( part );
+        else
+          orderByParts << QString( "%1 IS NULL" ).arg( part );
+
+        part += clause.ascending() ? " COLLATE NOCASE ASC" : " COLLATE NOCASE DESC";
+        orderByParts << part;
+      }
+      else
+      {
+        // Bail out on first non-complete compilation.
+        // Most important clauses at the beginning of the list
+        // will still be sent and used to pre-sort so the local
+        // CPU can use its cycles for fine-tuning.
+        mOrderByCompiled = false;
+        break;
+      }
+    }
+  }
+  else
+  {
+    mOrderByCompiled = false;
+  }
+
+  if ( !mOrderByCompiled )
+    limitAtProvider = false;
+
   // preparing the SQL statement
-  if ( !prepareStatement( whereClause ) )
+  bool success = prepareStatement( whereClause, limitAtProvider ? mRequest.limit() : -1, orderByParts.join( "," ) );
+  if ( !success && useFallbackWhereClause )
+  {
+    //try with the fallback where clause, eg for cases when using compiled expression failed to prepare
+    mExpressionCompiled = false;
+    success = prepareStatement( fallbackWhereClause, -1, orderByParts.join( "," ) );
+  }
+
+  if ( !success )
   {
     // some error occurred
-    sqliteStatement = NULL;
+    sqliteStatement = nullptr;
     close();
-    return;
   }
 }
 
@@ -76,12 +201,12 @@ QgsSpatiaLiteFeatureIterator::~QgsSpatiaLiteFeatureIterator()
 
 bool QgsSpatiaLiteFeatureIterator::fetchFeature( QgsFeature& feature )
 {
+  feature.setValid( false );
+
   if ( mClosed )
     return false;
 
-  feature.setValid( false );
-
-  if ( sqliteStatement == NULL )
+  if ( !sqliteStatement )
   {
     QgsDebugMsg( "Invalid current SQLite statement" );
     close();
@@ -91,7 +216,7 @@ bool QgsSpatiaLiteFeatureIterator::fetchFeature( QgsFeature& feature )
   if ( !getFeature( sqliteStatement, feature ) )
   {
     sqlite3_finalize( sqliteStatement );
-    sqliteStatement = NULL;
+    sqliteStatement = nullptr;
     close();
     return false;
   }
@@ -100,6 +225,14 @@ bool QgsSpatiaLiteFeatureIterator::fetchFeature( QgsFeature& feature )
   return true;
 }
 
+bool QgsSpatiaLiteFeatureIterator::nextFeatureFilterExpression( QgsFeature& f )
+{
+  if ( !mExpressionCompiled )
+    return QgsAbstractFeatureIterator::nextFeatureFilterExpression( f );
+  else
+    return fetchFeature( f );
+}
+
 
 bool QgsSpatiaLiteFeatureIterator::rewind()
 {
@@ -108,6 +241,7 @@ bool QgsSpatiaLiteFeatureIterator::rewind()
 
   if ( sqlite3_reset( sqliteStatement ) == SQLITE_OK )
   {
+    mRowNumber = 0;
     return true;
   }
   else
@@ -123,14 +257,20 @@ bool QgsSpatiaLiteFeatureIterator::close()
 
   iteratorClosed();
 
+  if ( !mHandle )
+  {
+    mClosed = true;
+    return false;
+  }
+
   if ( sqliteStatement )
   {
     sqlite3_finalize( sqliteStatement );
-    sqliteStatement = NULL;
+    sqliteStatement = nullptr;
   }
 
   QgsSpatiaLiteConnPool::instance()->releaseConnection( mHandle );
-  mHandle = 0;
+  mHandle = nullptr;
 
   mClosed = true;
   return true;
@@ -139,19 +279,22 @@ bool QgsSpatiaLiteFeatureIterator::close()
 ////
 
 
-bool QgsSpatiaLiteFeatureIterator::prepareStatement( QString whereClause )
+bool QgsSpatiaLiteFeatureIterator::prepareStatement( const QString& whereClause, long limit, const QString& orderBy )
 {
+  if ( !mHandle )
+    return false;
+
   try
   {
-    QString sql = QString( "SELECT %1" ).arg( quotedPrimaryKey() );
+    QString sql = QString( "SELECT %1" ).arg( mHasPrimaryKey ? quotedPrimaryKey() : "0" );
     int colIdx = 1; // column 0 is primary key
 
     if ( mRequest.flags() & QgsFeatureRequest::SubsetOfAttributes )
     {
-      const QgsAttributeList& fetchAttributes = mRequest.subsetOfAttributes();
+      QgsAttributeList fetchAttributes = mRequest.subsetOfAttributes();
       for ( QgsAttributeList::const_iterator it = fetchAttributes.constBegin(); it != fetchAttributes.constEnd(); ++it )
       {
-        sql += "," + fieldName( mSource->mFields.field( *it ) );
+        sql += ',' + fieldName( mSource->mFields.field( *it ) );
         colIdx++;
       }
     }
@@ -160,7 +303,7 @@ bool QgsSpatiaLiteFeatureIterator::prepareStatement( QString whereClause )
       // fetch all attributes
       for ( int idx = 0; idx < mSource->mFields.count(); ++idx )
       {
-        sql += "," + fieldName( mSource->mFields[idx] );
+        sql += ',' + fieldName( mSource->mFields.at( idx ) );
         colIdx++;
       }
     }
@@ -175,10 +318,16 @@ bool QgsSpatiaLiteFeatureIterator::prepareStatement( QString whereClause )
     if ( !whereClause.isEmpty() )
       sql += QString( " WHERE %1" ).arg( whereClause );
 
-    if ( sqlite3_prepare_v2( mHandle->handle(), sql.toUtf8().constData(), -1, &sqliteStatement, NULL ) != SQLITE_OK )
+    if ( !orderBy.isEmpty() )
+      sql += QString( " ORDER BY %1" ).arg( orderBy );
+
+    if ( limit >= 0 )
+      sql += QString( " LIMIT %1" ).arg( limit );
+
+    if ( sqlite3_prepare_v2( mHandle->handle(), sql.toUtf8().constData(), -1, &sqliteStatement, nullptr ) != SQLITE_OK )
     {
       // some error occurred
-      QgsMessageLog::logMessage( QObject::tr( "SQLite error: %2\nSQL: %1" ).arg( sql ).arg( sqlite3_errmsg( mHandle->handle() ) ), QObject::tr( "SpatiaLite" ) );
+      QgsMessageLog::logMessage( QObject::tr( "SQLite error: %2\nSQL: %1" ).arg( sql, sqlite3_errmsg( mHandle->handle() ) ), QObject::tr( "SpatiaLite" ) );
       return false;
     }
   }
@@ -193,7 +342,7 @@ bool QgsSpatiaLiteFeatureIterator::prepareStatement( QString whereClause )
 
 QString QgsSpatiaLiteFeatureIterator::quotedPrimaryKey()
 {
-  return !mSource->isQuery ? "ROWID" : QgsSpatiaLiteProvider::quotedIdentifier( mSource->mPrimaryKey );
+  return mSource->mPrimaryKey.isEmpty() ? "ROWID" : QgsSpatiaLiteProvider::quotedIdentifier( mSource->mPrimaryKey );
 }
 
 QString QgsSpatiaLiteFeatureIterator::whereClauseFid()
@@ -203,12 +352,17 @@ QString QgsSpatiaLiteFeatureIterator::whereClauseFid()
 
 QString QgsSpatiaLiteFeatureIterator::whereClauseFids()
 {
-  QStringList whereClauses;
-  foreach ( const QgsFeatureId featureId, mRequest.filterFids() )
+  if ( mRequest.filterFids().isEmpty() )
+    return "";
+
+  QString expr = QString( "%1 IN (" ).arg( quotedPrimaryKey() ), delim;
+  Q_FOREACH ( const QgsFeatureId featureId, mRequest.filterFids() )
   {
-    whereClauses << QString( "%1=%2" ).arg( quotedPrimaryKey() ).arg( featureId );
+    expr += delim + QString::number( featureId );
+    delim = ',';
   }
-  return whereClauses.isEmpty() ? "" : whereClauses.join( " OR " ).prepend( "(" ).append( ")" );
+  expr += ')';
+  return expr;
 }
 
 QString QgsSpatiaLiteFeatureIterator::whereClauseRect()
@@ -219,12 +373,12 @@ QString QgsSpatiaLiteFeatureIterator::whereClauseRect()
   if ( mRequest.flags() & QgsFeatureRequest::ExactIntersect )
   {
     // we are requested to evaluate a true INTERSECT relationship
-    whereClause += QString( "Intersects(%1, BuildMbr(%2)) AND " ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ) ).arg( mbr( rect ) );
+    whereClause += QString( "Intersects(%1, BuildMbr(%2)) AND " ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ), mbr( rect ) );
   }
   if ( mSource->mVShapeBased )
   {
     // handling a VirtualShape layer
-    whereClause += QString( "MbrIntersects(%1, BuildMbr(%2))" ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ) ).arg( mbr( rect ) );
+    whereClause += QString( "MbrIntersects(%1, BuildMbr(%2))" ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ), mbr( rect ) );
   }
   else if ( rect.isFinite() )
   {
@@ -235,30 +389,30 @@ QString QgsSpatiaLiteFeatureIterator::whereClauseRect()
       mbrFilter += QString( "xmax >= %1 AND " ).arg( qgsDoubleToString( rect.xMinimum() ) );
       mbrFilter += QString( "ymin <= %1 AND " ).arg( qgsDoubleToString( rect.yMaximum() ) );
       mbrFilter += QString( "ymax >= %1" ).arg( qgsDoubleToString( rect.yMinimum() ) );
-      QString idxName = QString( "idx_%1_%2" ).arg( mSource->mIndexTable ).arg( mSource->mIndexGeometry );
+      QString idxName = QString( "idx_%1_%2" ).arg( mSource->mIndexTable, mSource->mIndexGeometry );
       whereClause += QString( "%1 IN (SELECT pkid FROM %2 WHERE %3)" )
-                     .arg( quotedPrimaryKey() )
-                     .arg( QgsSpatiaLiteProvider::quotedIdentifier( idxName ) )
-                     .arg( mbrFilter );
+                     .arg( quotedPrimaryKey(),
+                           QgsSpatiaLiteProvider::quotedIdentifier( idxName ),
+                           mbrFilter );
     }
     else if ( mSource->spatialIndexMbrCache )
     {
       // using the MbrCache spatial index
-      QString idxName = QString( "cache_%1_%2" ).arg( mSource->mIndexTable ).arg( mSource->mIndexGeometry );
+      QString idxName = QString( "cache_%1_%2" ).arg( mSource->mIndexTable, mSource->mIndexGeometry );
       whereClause += QString( "%1 IN (SELECT rowid FROM %2 WHERE mbr = FilterMbrIntersects(%3))" )
-                     .arg( quotedPrimaryKey() )
-                     .arg( QgsSpatiaLiteProvider::quotedIdentifier( idxName ) )
-                     .arg( mbr( rect ) );
+                     .arg( quotedPrimaryKey(),
+                           QgsSpatiaLiteProvider::quotedIdentifier( idxName ),
+                           mbr( rect ) );
     }
     else
     {
       // using simple MBR filtering
-      whereClause += QString( "MbrIntersects(%1, BuildMbr(%2))" ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ) ).arg( mbr( rect ) );
+      whereClause += QString( "MbrIntersects(%1, BuildMbr(%2))" ).arg( QgsSpatiaLiteProvider::quotedIdentifier( mSource->mGeometryColumn ), mbr( rect ) );
     }
   }
   else
   {
-    whereClause = "1";
+    whereClause = '1';
   }
   return whereClause;
 }
@@ -267,10 +421,10 @@ QString QgsSpatiaLiteFeatureIterator::whereClauseRect()
 QString QgsSpatiaLiteFeatureIterator::mbr( const QgsRectangle& rect )
 {
   return QString( "%1, %2, %3, %4" )
-         .arg( qgsDoubleToString( rect.xMinimum() ) )
-         .arg( qgsDoubleToString( rect.yMinimum() ) )
-         .arg( qgsDoubleToString( rect.xMaximum() ) )
-         .arg( qgsDoubleToString( rect.yMaximum() ) );
+         .arg( qgsDoubleToString( rect.xMinimum() ),
+               qgsDoubleToString( rect.yMinimum() ),
+               qgsDoubleToString( rect.xMaximum() ),
+               qgsDoubleToString( rect.yMaximum() ) );
 }
 
 
@@ -308,11 +462,11 @@ bool QgsSpatiaLiteFeatureIterator::getFeature( sqlite3_stmt *stmt, QgsFeature &f
   if ( !mFetchGeometry )
   {
     // no geometry was required
-    feature.setGeometryAndOwnership( 0, 0 );
+    feature.setGeometry( nullptr );
   }
 
   feature.initAttributes( mSource->mFields.count() );
-  feature.setFields( &mSource->mFields ); // allow name-based attribute lookups
+  feature.setFields( mSource->mFields ); // allow name-based attribute lookups
 
   int ic;
   int n_columns = sqlite3_column_count( stmt );
@@ -320,10 +474,19 @@ bool QgsSpatiaLiteFeatureIterator::getFeature( sqlite3_stmt *stmt, QgsFeature &f
   {
     if ( ic == 0 )
     {
-      // first column always contains the ROWID (or the primary key)
-      QgsFeatureId fid = sqlite3_column_int64( stmt, ic );
-      QgsDebugMsgLevel( QString( "fid=%1" ).arg( fid ), 3 );
-      feature.setFeatureId( fid );
+      if ( mHasPrimaryKey )
+      {
+        // first column always contains the ROWID (or the primary key)
+        QgsFeatureId fid = sqlite3_column_int64( stmt, ic );
+        QgsDebugMsgLevel( QString( "fid=%1" ).arg( fid ), 3 );
+        feature.setFeatureId( fid );
+      }
+      else
+      {
+        // autoincrement a row number
+        mRowNumber++;
+        feature.setFeatureId( mRowNumber );
+      }
     }
     else if ( mFetchGeometry && ic == mGeomColIdx )
     {
@@ -350,7 +513,7 @@ bool QgsSpatiaLiteFeatureIterator::getFeature( sqlite3_stmt *stmt, QgsFeature &f
   return true;
 }
 
-QVariant QgsSpatiaLiteFeatureIterator::getFeatureAttribute( sqlite3_stmt* stmt, int ic, const QVariant::Type& type )
+QVariant QgsSpatiaLiteFeatureIterator::getFeatureAttribute( sqlite3_stmt* stmt, int ic, QVariant::Type type )
 {
   if ( sqlite3_column_type( stmt, ic ) == SQLITE_INTEGER )
   {
@@ -387,24 +550,34 @@ void QgsSpatiaLiteFeatureIterator::getFeatureGeometry( sqlite3_stmt* stmt, int i
 {
   if ( sqlite3_column_type( stmt, ic ) == SQLITE_BLOB )
   {
-    unsigned char *featureGeom = NULL;
-    size_t geom_size = 0;
+    unsigned char *featureGeom = nullptr;
+    int geom_size = 0;
     const void *blob = sqlite3_column_blob( stmt, ic );
-    size_t blob_size = sqlite3_column_bytes( stmt, ic );
-    QgsSpatiaLiteProvider::convertToGeosWKB(( const unsigned char * )blob, blob_size,
-                                            &featureGeom, &geom_size );
+    int blob_size = sqlite3_column_bytes( stmt, ic );
+    QgsSpatiaLiteProvider::convertToGeosWKB(( const unsigned char * )blob, blob_size, &featureGeom, &geom_size );
     if ( featureGeom )
-      feature.setGeometryAndOwnership( featureGeom, geom_size );
+    {
+      QgsGeometry *g = new QgsGeometry();
+      g->fromWkb( featureGeom, geom_size );
+      feature.setGeometry( g );
+    }
     else
-      feature.setGeometryAndOwnership( 0, 0 );
+      feature.setGeometry( nullptr );
   }
   else
   {
     // NULL geometry
-    feature.setGeometryAndOwnership( 0, 0 );
+    feature.setGeometry( nullptr );
   }
 }
 
+bool QgsSpatiaLiteFeatureIterator::prepareOrderBy( const QList<QgsFeatureRequest::OrderByClause>& orderBys )
+{
+  Q_UNUSED( orderBys )
+  // Preparation has already been done in the constructor, so we just communicate the result
+  return mOrderByCompiled;
+}
+
 
 QgsSpatiaLiteFeatureSource::QgsSpatiaLiteFeatureSource( const QgsSpatiaLiteProvider* p )
     : mGeometryColumn( p->mGeometryColumn )
@@ -412,6 +585,7 @@ QgsSpatiaLiteFeatureSource::QgsSpatiaLiteFeatureSource( const QgsSpatiaLiteProvi
     , mFields( p->attributeFields )
     , mQuery( p->mQuery )
     , isQuery( p->isQuery )
+    , mViewBased( p->mViewBased )
     , mVShapeBased( p->mVShapeBased )
     , mIndexTable( p->mIndexTable )
     , mIndexGeometry( p->mIndexGeometry )
