@@ -89,7 +89,7 @@ void QgsWCSSourceSelect::populateLayerList()
     lItem->setData( 0, Qt::UserRole + 1, "" );
 
     // Make only leaves selectable
-    if ( coverageParents.keys( coverage->orderId ).size() > 0 )
+    if ( !coverageParents.keys( coverage->orderId ).isEmpty() )
     {
       lItem->setFlags( Qt::ItemIsEnabled );
     }
@@ -198,7 +198,7 @@ QList<QgsWCSSourceSelect::SupportedFormat> QgsWCSSourceSelect::providerFormats()
   QList<SupportedFormat> formats;
 
   QMap<QString, QString> mimes = QgsWcsProvider::supportedMimes();
-  foreach ( QString mime, mimes.keys() )
+  Q_FOREACH ( const QString& mime, mimes.keys() )
   {
     SupportedFormat format = { mime, mimes.value( mime ) };
 
