@@ -52,7 +52,7 @@ class QgsWmsLegendDownloadHandler : public QgsImageFetcher
   public:
 
     QgsWmsLegendDownloadHandler( QgsNetworkAccessManager& networkAccessManager, const QgsWmsSettings& settings, const QUrl& url );
-    ~QgsWmsLegendDownloadHandler( );
+    ~QgsWmsLegendDownloadHandler();
 
     // Make sure to connect to "finish" before starting
     void start() override;
@@ -82,9 +82,9 @@ class QgsWmsLegendDownloadHandler : public QgsImageFetcher
 
 class QgsCachedImageFetcher: public QgsImageFetcher
 {
-    Q_OBJECT;
+    Q_OBJECT
   public:
-    QgsCachedImageFetcher( const QImage& img );
+    explicit QgsCachedImageFetcher( const QImage& img );
     virtual ~QgsCachedImageFetcher();
     virtual void start() override;
   private:
@@ -113,19 +113,19 @@ class QgsWmsProvider : public QgsRasterDataProvider
 
   public:
     /**
-    * Constructor for the provider.
-    *
-    * \param   uri   HTTP URL of the Web Server.  If needed a proxy will be used
-    *                otherwise we contact the host directly.
-    * \param   capabilities   Optionally existing parsed capabilities for the given URI
-    *
-    */
-    QgsWmsProvider( QString const & uri = 0, const QgsWmsCapabilities* capabilities = 0 );
+     * Constructor for the provider.
+     *
+     * \param   uri   HTTP URL of the Web Server.  If needed a proxy will be used
+     *                otherwise we contact the host directly.
+     * \param   capabilities   Optionally existing parsed capabilities for the given URI
+     *
+     */
+    QgsWmsProvider( QString const & uri = nullptr, const QgsWmsCapabilities* capabilities = nullptr );
 
     //! Destructor
     virtual ~QgsWmsProvider();
 
-    QgsRasterInterface * clone() const override;
+    QgsWmsProvider * clone() const override;
 
 
     /** Get the QgsCoordinateReferenceSystem for this layer
@@ -173,7 +173,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
 
 
     /** Return the extent for this data layer
-    */
+     */
     virtual QgsRectangle extent() override;
 
     /** Returns true if layer is valid
@@ -237,7 +237,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
      *
      * \note visibleExtent parameter added in 2.8
      */
-    QImage getLegendGraphic( double scale = 0.0, bool forceRefresh = false, const QgsRectangle * visibleExtent = 0 ) override;
+    QImage getLegendGraphic( double scale = 0.0, bool forceRefresh = false, const QgsRectangle * visibleExtent = nullptr ) override;
 
     /**
      * \class Get an image downloader for the raster legend
@@ -303,37 +303,38 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString lastErrorFormat() override;
 
     /** Return a provider name
-
-    Essentially just returns the provider key.  Should be used to build file
-    dialogs so that providers can be shown with their supported types. Thus
-    if more than one provider supports a given format, the user is able to
-    select a specific provider to open that file.
-
-    @note
-
-    Instead of being pure virtual, might be better to generalize this
-    behavior and presume that none of the sub-classes are going to do
-    anything strange with regards to their name or description?
-
-    */
+     *
+     * Essentially just returns the provider key.  Should be used to build file
+     * dialogs so that providers can be shown with their supported types. Thus
+     * if more than one provider supports a given format, the user is able to
+     * select a specific provider to open that file.
+     *
+     * @note
+     *
+     * Instead of being pure virtual, might be better to generalize this
+     * behavior and presume that none of the sub-classes are going to do
+     * anything strange with regards to their name or description?
+     *
+     */
     QString name() const override;
 
 
     /** Return description
-
-    Return a terse string describing what the provider is.
-
-    @note
-
-    Instead of being pure virtual, might be better to generalize this
-    behavior and presume that none of the sub-classes are going to do
-    anything strange with regards to their name or description?
-
-    */
+     *
+     * Return a terse string describing what the provider is.
+     *
+     * @note
+     *
+     * Instead of being pure virtual, might be better to generalize this
+     * behavior and presume that none of the sub-classes are going to do
+     * anything strange with regards to their name or description?
+     *
+     */
     QString description() const override;
 
     /** Reloads the data from the source. Needs to be implemented by providers with data caches to
-      synchronize with changes in the data source*/
+     * synchronize with changes in the data source
+     */
     virtual void reloadData() override;
 
     static QVector<QgsWmsSupportedFormat> supportedFormats();
@@ -359,9 +360,6 @@ class QgsWmsProvider : public QgsRasterDataProvider
     /** \brief emit a signal to notify of a progress event */
     void progressChanged( int theProgress, int theTotalSteps );
 
-    /** \brief emit a signal to be caught by qgisapp and display a msg on status bar */
-    void statusChanged( QString const &  theStatusQString );
-
     void dataChanged();
 
   private slots:
@@ -378,7 +376,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     bool extentForNonTiledLayer( const QString& layerName, const QString& crs, QgsRectangle& extent );
 
     // case insensitive attribute value lookup
-    static QString nodeAttribute( const QDomElement &e, QString name, QString defValue = QString::null );
+    static QString nodeAttribute( const QDomElement &e, const QString& name, const QString& defValue = QString::null );
 
     /**
      * Add the list of WMS layer names to be rendered by this server
@@ -443,7 +441,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString layerMetadata( QgsWmsLayerProperty &layer );
 
     //! remove query item and replace it with a new value
-    void setQueryItem( QUrl &url, QString key, QString value );
+    void setQueryItem( QUrl &url, const QString& key, const QString& value );
 
     //! add image FORMAT parameter to url
     void setFormatQueryItem( QUrl &url );
@@ -591,7 +589,6 @@ class QgsWmsImageDownloadHandler : public QObject
     QImage* mCachedImage;
 
     QEventLoop* mEventLoop;
-    QgsNetworkAccessManager* mNAM;
 };
 
 
@@ -637,7 +634,6 @@ class QgsWmsTiledImageDownloadHandler : public QObject
     QgsRectangle mCachedViewExtent;
 
     QEventLoop* mEventLoop;
-    QgsNetworkAccessManager* mNAM;
 
     int mTileReqNo;
     bool mSmoothPixmapTransform;
