@@ -32,7 +32,6 @@
 #include "qgsproject.h"
 #include "qgsproviderregistry.h"
 #include "qgswmsconnection.h"
-#include "qgswmsprovider.h"
 #include "qgswmssourceselect.h"
 #include "qgswmtsdimensions.h"
 #include "qgsnetworkaccessmanager.h"
@@ -60,7 +59,7 @@ QgsWMSSourceSelect::QgsWMSSourceSelect( QWidget * parent, Qt::WindowFlags fl, bo
     , mManagerMode( managerMode )
     , mEmbeddedMode( embeddedMode )
     , mDefaultCRS( GEO_EPSG_CRS_AUTHID )
-    , mCurrentTileset( 0 )
+    , mCurrentTileset( nullptr )
 {
   setupUi( this );
 
@@ -97,7 +96,7 @@ QgsWMSSourceSelect::QgsWMSSourceSelect( QWidget * parent, Qt::WindowFlags fl, bo
     {
       mMimeMap.insert( mFormats[i].format, i );
 
-      QRadioButton *btn = new QRadioButton( mFormats[i].label );
+      QRadioButton *btn = new QRadioButton( mFormats.at( i ).label );
       btn->setToolTip( mFormats[i].format );
       btn->setHidden( true );
       mImageFormatGroup->addButton( btn, i );
@@ -214,7 +213,7 @@ void QgsWMSSourceSelect::on_btnSave_clicked()
 
 void QgsWMSSourceSelect::on_btnLoad_clicked()
 {
-  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), ".",
+  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load connections" ), QDir::homePath(),
                      tr( "XML files (*.xml *XML)" ) );
   if ( fileName.isEmpty() )
   {
@@ -265,7 +264,7 @@ void QgsWMSSourceSelect::clear()
 
   mCRSs.clear();
 
-  foreach ( QAbstractButton *b, mImageFormatGroup->buttons() )
+  Q_FOREACH ( QAbstractButton *b, mImageFormatGroup->buttons() )
   {
     b->setHidden( true );
   }
@@ -278,7 +277,7 @@ bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities& capabiliti
   QVector<QgsWmsLayerProperty> layers = capabilities.supportedLayers();
 
   bool first = true;
-  foreach ( QString encoding, capabilities.supportedImageEncodings() )
+  Q_FOREACH ( const QString& encoding, capabilities.supportedImageEncodings() )
   {
     int id = mMimeMap.value( encoding, -1 );
     if ( id < 0 )
@@ -321,17 +320,17 @@ bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities& capabiliti
     // Layer Styles
     for ( int j = 0; j < layer->style.size(); j++ )
     {
-      QgsDebugMsg( QString( "got style name %1 and title '%2'." ).arg( layer->style[j].name ).arg( layer->style[j].title ) );
+      QgsDebugMsg( QString( "got style name %1 and title '%2'." ).arg( layer->style.at( j ).name, layer->style.at( j ).title ) );
 
       QgsNumericSortTreeWidgetItem *lItem2 = new QgsNumericSortTreeWidgetItem( lItem );
       lItem2->setText( 0, QString::number( ++layerAndStyleCount ) );
-      lItem2->setText( 1, layer->style[j].name.simplified() );
-      lItem2->setText( 2, layer->style[j].title.simplified() );
-      lItem2->setText( 3, layer->style[j].abstract.simplified() );
+      lItem2->setText( 1, layer->style.at( j ).name.simplified() );
+      lItem2->setText( 2, layer->style.at( j ).title.simplified() );
+      lItem2->setText( 3, layer->style.at( j ).abstract.simplified() );
 
       lItem2->setData( 0, Qt::UserRole + 0, layer->name );
-      lItem2->setData( 0, Qt::UserRole + 1, layer->style[j].name );
-      lItem2->setData( 0, Qt::UserRole + 3, layer->style[j].title.isEmpty() ? layer->style[j].name : layer->style[j].title );
+      lItem2->setData( 0, Qt::UserRole + 1, layer->style.at( j ).name );
+      lItem2->setData( 0, Qt::UserRole + 3, layer->style.at( j ).title.isEmpty() ? layer->style.at( j ).name : layer->style.at( j ).title );
     }
   }
 
@@ -339,16 +338,16 @@ bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities& capabiliti
 
   mTileLayers = capabilities.supportedTileLayers();
 
-  tabServers->setTabEnabled( tabServers->indexOf( tabTilesets ), mTileLayers.size() > 0 );
+  tabServers->setTabEnabled( tabServers->indexOf( tabTilesets ), !mTileLayers.isEmpty() );
   if ( tabServers->isTabEnabled( tabServers->indexOf( tabTilesets ) ) )
     tabServers->setCurrentWidget( tabTilesets );
 
-  if ( mTileLayers.size() > 0 )
+  if ( !mTileLayers.isEmpty() )
   {
     QHash<QString, QgsWmtsTileMatrixSet> tileMatrixSets = capabilities.supportedTileMatrixSets();
 
     int rows = 0;
-    foreach ( const QgsWmtsTileLayer &l, mTileLayers )
+    Q_FOREACH ( const QgsWmtsTileLayer &l, mTileLayers )
     {
       rows += l.styles.size() * l.setLinks.size() * l.formats.size();
     }
@@ -358,13 +357,13 @@ bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities& capabiliti
     lstTilesets->setSortingEnabled( false );
 
     int row = 0;
-    foreach ( const QgsWmtsTileLayer &l, mTileLayers )
+    Q_FOREACH ( const QgsWmtsTileLayer &l, mTileLayers )
     {
-      foreach ( const QgsWmtsStyle &style, l.styles )
+      Q_FOREACH ( const QgsWmtsStyle &style, l.styles )
       {
-        foreach ( const QgsWmtsTileMatrixSetLink &setLink, l.setLinks )
+        Q_FOREACH ( const QgsWmtsTileMatrixSetLink &setLink, l.setLinks )
         {
-          foreach ( QString format, l.formats )
+          Q_FOREACH ( const QString& format, l.formats )
           {
             QTableWidgetItem *item = new QTableWidgetItem( l.identifier );
             item->setData( Qt::UserRole + 0, l.identifier );
@@ -434,7 +433,6 @@ void QgsWMSSourceSelect::on_btnConnect_clicked()
   mConnName = cmbConnections->currentText();
 
   QgsWMSConnection connection( cmbConnections->currentText() );
-  mConnectionInfo = connection.connectionInfo();
   mUri = connection.uri();
 
   QgsWmsSettings wmsSettings;
@@ -448,7 +446,7 @@ void QgsWMSSourceSelect::on_btnConnect_clicked()
     return;
   }
 
-  QgsWmsCapabilitiesDownload capDownload( wmsSettings.baseUrl(), wmsSettings.authorization() );
+  QgsWmsCapabilitiesDownload capDownload( wmsSettings.baseUrl(), wmsSettings.authorization(), true );
   connect( &capDownload, SIGNAL( statusChanged( QString ) ), this, SLOT( showStatusMessage( QString ) ) );
 
   QApplication::setOverrideCursor( Qt::WaitCursor );
@@ -476,6 +474,8 @@ void QgsWMSSourceSelect::on_btnConnect_clicked()
     return;
   }
 
+  mFeatureCount->setEnabled( caps.identifyCapabilities() != QgsRasterInterface::NoCapabilities );
+
   populateLayerList( caps );
 }
 
@@ -513,9 +513,9 @@ void QgsWMSSourceSelect::addClicked()
 
     uri.setParam( "tileMatrixSet", item->data( Qt::UserRole + 3 ).toStringList() );
 
-    const QgsWmtsTileLayer *layer = 0;
+    const QgsWmtsTileLayer *layer = nullptr;
 
-    foreach ( const QgsWmtsTileLayer &l, mTileLayers )
+    Q_FOREACH ( const QgsWmtsTileLayer &l, mTileLayers )
     {
       if ( l.identifier == layers.join( "," ) )
       {
@@ -524,7 +524,8 @@ void QgsWMSSourceSelect::addClicked()
       }
     }
 
-    Q_ASSERT( layer );
+    if ( !layer )
+      return;
 
     if ( !layer->dimensions.isEmpty() )
     {
@@ -545,8 +546,8 @@ void QgsWMSSourceSelect::addClicked()
             it != dims.constEnd();
             ++it )
       {
-        dimString += delim + it.key() + "=" + it.value();
-        delim = ";";
+        dimString += delim + it.key() + '=' + it.value();
+        delim = ';';
       }
 
       delete dlg;
@@ -566,6 +567,8 @@ void QgsWMSSourceSelect::addClicked()
     uri.setParam( "featureCount", mFeatureCount->text() );
   }
 
+  uri.setParam( "contextualWMSLegend", mContextualLegendCheckbox->isChecked() ? "1" : "0" );
+
   emit addRasterLayer( uri.encodedUri(),
                        leLayerName->text().isEmpty() ? titles.join( "/" ) : leLayerName->text(),
                        "wms" );
@@ -602,7 +605,7 @@ void QgsWMSSourceSelect::enableLayersForCrs( QTreeWidgetItem *item )
 void QgsWMSSourceSelect::on_btnChangeSpatialRefSys_clicked()
 {
   QStringList layers;
-  foreach ( QTreeWidgetItem *item, lstLayers->selectedItems() )
+  Q_FOREACH ( QTreeWidgetItem *item, lstLayers->selectedItems() )
   {
     QString layer = item->data( 0, Qt::UserRole + 0 ).toString();
     if ( !layer.isEmpty() )
@@ -662,8 +665,8 @@ void QgsWMSSourceSelect::applySelectionConstraints( QTreeWidgetItem *item )
       return;
     }
 
-    QTreeWidgetItem *style = 0;
-    QTreeWidgetItem *firstNewStyle = 0;
+    QTreeWidgetItem *style = nullptr;
+    QTreeWidgetItem *firstNewStyle = nullptr;
     for ( int i = 0; i < item->childCount(); i++ )
     {
       QTreeWidgetItem *child = item->child( i );
@@ -681,7 +684,7 @@ void QgsWMSSourceSelect::applySelectionConstraints( QTreeWidgetItem *item )
 
     if ( firstNewStyle || style )
     {
-      // individual style selected => unselect layer and all parent groups
+      // individual style selected => deselect layer and all parent groups
       QTreeWidgetItem *parent = item;
       while ( parent )
       {
@@ -779,7 +782,7 @@ void QgsWMSSourceSelect::on_lstLayers_itemSelectionChanged()
   mCRSs.clear();
 
   // determine selected layers and styles and set of crses that are available for all layers
-  foreach ( QTreeWidgetItem *item, lstLayers->selectedItems() )
+  Q_FOREACH ( QTreeWidgetItem *item, lstLayers->selectedItems() )
   {
     QString layerName = item->data( 0, Qt::UserRole + 0 ).toString();
     QString styleName = item->data( 0, Qt::UserRole + 1 ).toString();
@@ -823,8 +826,8 @@ void QgsWMSSourceSelect::on_lstLayers_itemSelectionChanged()
     // check whether current CRS is supported
     // if not, use one of the available CRS
     QString defaultCRS;
-    QSet<QString>::const_iterator it = mCRSs.begin();
-    for ( ; it != mCRSs.end(); ++it )
+    QSet<QString>::const_iterator it = mCRSs.constBegin();
+    for ( ; it != mCRSs.constEnd(); ++it )
     {
       if ( it->compare( mCRS, Qt::CaseInsensitive ) == 0 )
         break;
@@ -873,7 +876,7 @@ void QgsWMSSourceSelect::on_lstTilesets_itemClicked( QTableWidgetItem *item )
   }
   else
   {
-    mCurrentTileset = 0;
+    mCurrentTileset = nullptr;
   }
   lstTilesets->blockSignals( false );
 
@@ -983,11 +986,6 @@ QString QgsWMSSourceSelect::connName()
   return mConnName;
 }
 
-QString QgsWMSSourceSelect::connectionInfo()
-{
-  return mConnectionInfo;
-}
-
 void QgsWMSSourceSelect::collectSelectedLayers( QStringList &layers, QStringList &styles, QStringList &titles )
 {
   //go through list in layer order tab
@@ -1010,7 +1008,7 @@ QString QgsWMSSourceSelect::selectedImageEncoding()
   }
   else
   {
-    return QUrl::toPercentEncoding( mFormats[ id ].format );
+    return QUrl::toPercentEncoding( mFormats.at( id ).format );
   }
 }
 
@@ -1067,7 +1065,7 @@ void QgsWMSSourceSelect::showError( QgsWmsProvider * wms )
   }
   else
   {
-    mv->setMessageAsPlainText( tr( "Could not understand the response.  The %1 provider said:\n%2" ).arg( wms->name() ).arg( wms->lastError() ) );
+    mv->setMessageAsPlainText( tr( "Could not understand the response. The %1 provider said:\n%2" ).arg( wms->name(), wms->lastError() ) );
   }
   mv->showMessage( true ); // Is deleted when closed
 }
@@ -1083,7 +1081,7 @@ void QgsWMSSourceSelect::on_btnAddDefault_clicked()
   addDefaultServers();
 }
 
-QString QgsWMSSourceSelect::descriptionForAuthId( QString authId )
+QString QgsWMSSourceSelect::descriptionForAuthId( const QString& authId )
 {
   if ( mCrsNames.contains( authId ) )
     return mCrsNames[ authId ];
