@@ -33,12 +33,18 @@
 #include <QStringList>
 #include <QDir>
 
+#ifdef PYTHON2
+#define PYOBJ2QSTRING(obj) PyString_AsString( obj )
+#else
+#define PYOBJ2QSTRING(obj) QString::fromUtf8( PyUnicode_AsUTF8( obj ) )
+#endif
+
 PyThreadState* _mainState;
 
 QgsPythonUtilsImpl::QgsPythonUtilsImpl()
 {
-  mMainModule = NULL;
-  mMainDict = NULL;
+  mMainModule = nullptr;
+  mMainDict = nullptr;
   mPythonEnabled = false;
 }
 
@@ -49,19 +55,8 @@ QgsPythonUtilsImpl::~QgsPythonUtilsImpl()
 #endif
 }
 
-void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
+bool QgsPythonUtilsImpl::checkSystemImports()
 {
-  // initialize python
-  Py_Initialize();
-
-  // initialize threading AND acquire GIL
-  PyEval_InitThreads();
-
-  mPythonEnabled = true;
-
-  mMainModule = PyImport_AddModule( "__main__" ); // borrowed reference
-  mMainDict = PyModule_GetDict( mMainModule ); // borrowed reference
-
   runString( "import sys" ); // import sys module (for display / exception hooks)
   runString( "import os" ); // import os module (for user paths)
 
@@ -84,17 +79,17 @@ void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
   // locally installed plugins have priority over the system plugins
   // use os.path.expanduser to support usernames with special characters (see #2512)
   QStringList pluginpaths;
-  foreach ( QString p, extraPluginsPaths() )
+  Q_FOREACH ( QString p, extraPluginsPaths() )
   {
     if ( !QDir( p ).exists() )
     {
       QgsMessageOutput* msg = QgsMessageOutput::createMessageOutput();
       msg->setTitle( QObject::tr( "Python error" ) );
-      msg->setMessage( QString( QObject::tr( "The extra plugin path '%1' does not exist !" ) ).arg( p ), QgsMessageOutput::MessageText );
+      msg->setMessage( QObject::tr( "The extra plugin path '%1' does not exist!" ).arg( p ), QgsMessageOutput::MessageText );
       msg->showMessage();
     }
 #ifdef Q_OS_WIN
-    p = p.replace( '\\', "\\\\" );
+    p.replace( '\\', "\\\\" );
 #endif
     // we store here paths in unicode strings
     // the str constant will contain utf8 code (through runString)
@@ -114,71 +109,79 @@ void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
 
   // import SIP
   if ( !runString( "import sip",
-                   QObject::tr( "Couldn't load SIP module." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
+                   QObject::tr( "Couldn't load SIP module." ) + '\n' + QObject::tr( "Python support will be disabled." ) ) )
   {
-    exitPython();
-    return;
+    return false;
   }
 
   // set PyQt4 api versions
   QStringList apiV2classes;
   apiV2classes << "QDate" << "QDateTime" << "QString" << "QTextStream" << "QTime" << "QUrl" << "QVariant";
-  foreach ( const QString& clsName, apiV2classes )
+  Q_FOREACH ( const QString& clsName, apiV2classes )
   {
     if ( !runString( QString( "sip.setapi('%1', 2)" ).arg( clsName ),
-                     QObject::tr( "Couldn't set SIP API versions." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
+                     QObject::tr( "Couldn't set SIP API versions." ) + '\n' + QObject::tr( "Python support will be disabled." ) ) )
     {
-      exitPython();
-      return;
+      return false;
     }
   }
-
+#ifdef PYTHON2
   // import Qt bindings
   if ( !runString( "from PyQt4 import QtCore, QtGui",
-                   QObject::tr( "Couldn't load PyQt4." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
+                   QObject::tr( "Couldn't load PyQt." ) + '\n' + QObject::tr( "Python support will be disabled." ) ) )
   {
-    exitPython();
-    return;
+    return false;
   }
+#else
+  // import Qt bindings
+  if ( !runString( "from PyQt5 import QtCore, QtGui",
+                   QObject::tr( "Couldn't load PyQt." ) + '\n' + QObject::tr( "Python support will be disabled." ) ) )
+  {
+    return false;
+  }
+#endif
 
   // import QGIS bindings
-  QString error_msg = QObject::tr( "Couldn't load PyQGIS." ) + "\n" + QObject::tr( "Python support will be disabled." );
+  QString error_msg = QObject::tr( "Couldn't load PyQGIS." ) + '\n' + QObject::tr( "Python support will be disabled." );
   if ( !runString( "from qgis.core import *", error_msg ) || !runString( "from qgis.gui import *", error_msg ) )
   {
-    exitPython();
-    return;
+    return false;
   }
 
   // import QGIS utils
-  error_msg = QObject::tr( "Couldn't load QGIS utils." ) + "\n" + QObject::tr( "Python support will be disabled." );
+  error_msg = QObject::tr( "Couldn't load QGIS utils." ) + '\n' + QObject::tr( "Python support will be disabled." );
   if ( !runString( "import qgis.utils", error_msg ) )
   {
-    exitPython();
-    return;
+    return false;
   }
 
-
   // tell the utils script where to look for the plugins
-  runString( "qgis.utils.plugin_paths = [" + pluginpaths.join( "," ) + "]" );
-  runString( "qgis.utils.sys_plugin_path = \"" + pluginsPath() + "\"" );
+  runString( "qgis.utils.plugin_paths = [" + pluginpaths.join( "," ) + ']' );
+  runString( "qgis.utils.sys_plugin_path = \"" + pluginsPath() + '\"' );
   runString( "qgis.utils.home_plugin_path = " + homePluginsPath() );
 
 #ifdef Q_OS_WIN
   runString( "if oldhome: os.environ['HOME']=oldhome\n" );
 #endif
 
-  // initialize 'iface' object
-  runString( "qgis.utils.initInterface(" + QString::number(( unsigned long ) interface ) + ")" );
+  return true;
+}
 
-  // import QGIS user
-  error_msg = QObject::tr( "Couldn't load qgis.user." ) + "\n" + QObject::tr( "Python support will be disabled." );
-  if ( !runString( "import qgis.user", error_msg ) )
-  {
-    // Should we really bail because of this?!
-    exitPython();
-    return;
-  }
+void QgsPythonUtilsImpl::init()
+{
+  // initialize python
+  Py_Initialize();
+  // initialize threading AND acquire GIL
+  PyEval_InitThreads();
 
+  mPythonEnabled = true;
+
+  mMainModule = PyImport_AddModule( "__main__" ); // borrowed reference
+  mMainDict = PyModule_GetDict( mMainModule ); // borrowed reference
+}
+
+void QgsPythonUtilsImpl::finish()
+{
   // release GIL!
   // Later on, we acquire GIL just before doing some Python calls and
   // release GIL again when the work with Python API is done.
@@ -187,103 +190,42 @@ void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
   _mainState = PyEval_SaveThread();
 }
 
+bool QgsPythonUtilsImpl::checkQgisUser()
+{
+  // import QGIS user
+  QString error_msg = QObject::tr( "Couldn't load qgis.user." ) + '\n' + QObject::tr( "Python support will be disabled." );
+  if ( !runString( "import qgis.user", error_msg ) )
+  {
+    // Should we really bail because of this?!
+    return false;
+  }
+  return true;
+}
+
+void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
+{
+  init();
+  if ( !checkSystemImports() )
+  {
+    exitPython();
+    return;
+  }
+  // initialize 'iface' object
+  runString( "qgis.utils.initInterface(" + QString::number(( unsigned long ) interface ) + ')' );
+  if ( !checkQgisUser() )
+  {
+    exitPython();
+    return;
+  }
+  finish();
+}
+
 
 #ifdef HAVE_SERVER_PYTHON_PLUGINS
 void QgsPythonUtilsImpl::initServerPython( QgsServerInterface* interface )
 {
-  // initialize python
-  Py_Initialize();
-
-  // initialize threading AND acquire GIL
-  PyEval_InitThreads();
-
-  mPythonEnabled = true;
-
-  mMainModule = PyImport_AddModule( "__main__" ); // borrowed reference
-  mMainDict = PyModule_GetDict( mMainModule ); // borrowed reference
-
-  runString( "import sys" ); // import sys module (for display / exception hooks)
-  runString( "import os" ); // import os module (for user paths)
-
-  // support for PYTHONSTARTUP-like environment variable: PYQGIS_STARTUP
-  // (unlike PYTHONHOME and PYTHONPATH, PYTHONSTARTUP is not supported for embedded interpreter by default)
-  // this is different than user's 'startup.py' (below), since it is loaded just after Py_Initialize
-  // it is very useful for cleaning sys.path, which may have undesireable paths, or for
-  // isolating/loading the initial environ without requiring a virt env, e.g. homebrew or MacPorts installs on Mac
-  runString( "pyqgstart = os.getenv('PYQGIS_STARTUP')\n" );
-  runString( "if pyqgstart is not None and os.path.exists(pyqgstart): execfile(pyqgstart)\n" );
-
-#ifdef Q_OS_WIN
-  runString( "oldhome=None" );
-  runString( "if os.environ.has_key('HOME'): oldhome=os.environ['HOME']\n" );
-  runString( "os.environ['HOME']=os.environ['USERPROFILE']\n" );
-#endif
-
-  // construct a list of plugin paths
-  // plugin dirs passed in QGIS_PLUGINPATH env. variable have highest priority (usually empty)
-  // locally installed plugins have priority over the system plugins
-  // use os.path.expanduser to support usernames with special characters (see #2512)
-  QStringList pluginpaths;
-  foreach ( QString p, extraPluginsPaths() )
-  {
-    if ( !QDir( p ).exists() )
-    {
-      QgsMessageOutput* msg = QgsMessageOutput::createMessageOutput();
-      msg->setTitle( QObject::tr( "Python error" ) );
-      msg->setMessage( QString( QObject::tr( "The extra plugin path '%1' does not exist !" ) ).arg( p ), QgsMessageOutput::MessageText );
-      msg->showMessage();
-    }
-#ifdef Q_OS_WIN
-    p = p.replace( '\\', "\\\\" );
-#endif
-    // we store here paths in unicode strings
-    // the str constant will contain utf8 code (through runString)
-    // so we call '...'.decode('utf-8') to make a unicode string
-    pluginpaths << '"' + p + "\".decode('utf-8')";
-  }
-  pluginpaths << homePluginsPath();
-  pluginpaths << '"' + pluginsPath() + '"';
-
-  // expect that bindings are installed locally, so add the path to modules
-  // also add path to plugins
-  QStringList newpaths;
-  newpaths << '"' + pythonPath() + '"';
-  newpaths << homePythonPath();
-  newpaths << pluginpaths;
-  runString( "sys.path = [" + newpaths.join( "," ) + "] + sys.path" );
-
-  // import SIP
-  if ( !runString( "import sip",
-                   QObject::tr( "Couldn't load SIP module." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
-  {
-    exitPython();
-    return;
-  }
-
-  // set PyQt4 api versions
-  QStringList apiV2classes;
-  apiV2classes << "QDate" << "QDateTime" << "QString" << "QTextStream" << "QTime" << "QUrl" << "QVariant";
-  foreach ( const QString& clsName, apiV2classes )
-  {
-    if ( !runString( QString( "sip.setapi('%1', 2)" ).arg( clsName ),
-                     QObject::tr( "Couldn't set SIP API versions." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
-    {
-      exitPython();
-      return;
-    }
-  }
-
-  // import Qt bindings
-  if ( !runString( "from PyQt4 import QtCore, QtGui",
-                   QObject::tr( "Couldn't load PyQt4." ) + "\n" + QObject::tr( "Python support will be disabled." ) ) )
-  {
-    exitPython();
-    return;
-  }
-
-  // import QGIS bindings
-  QString error_msg = QObject::tr( "Couldn't load PyQGIS." ) + "\n" + QObject::tr( "Python support will be disabled." );
-  if ( !runString( "from qgis.core import *", error_msg ) || !runString( "from qgis.gui import *", error_msg ) )
+  init();
+  if ( !checkSystemImports() )
   {
     exitPython();
     return;
@@ -291,43 +233,16 @@ void QgsPythonUtilsImpl::initServerPython( QgsServerInterface* interface )
 
   // This is the main difference with initInterface() for desktop plugins
   // import QGIS Server bindings
-  error_msg = QObject::tr( "Couldn't load PyQGIS Server." ) + "\n" + QObject::tr( "Python support will be disabled." );
+  QString error_msg = QObject::tr( "Couldn't load PyQGIS Server." ) + '\n' + QObject::tr( "Python support will be disabled." );
   if ( !runString( "from qgis.server import *", error_msg ) )
   {
-    exitPython();
     return;
   }
 
-
-  // import QGIS utils
-  error_msg = QObject::tr( "Couldn't load QGIS utils." ) + "\n" + QObject::tr( "Python support will be disabled." );
-  if ( !runString( "import qgis.utils", error_msg ) )
-  {
-    exitPython();
-    return;
-  }
-
-  // tell the utils script where to look for the plugins
-  runString( "qgis.utils.plugin_paths = [" + pluginpaths.join( "," ) + "]" );
-  runString( "qgis.utils.sys_plugin_path = \"" + pluginsPath() + "\"" );
-  runString( "qgis.utils.home_plugin_path = " + homePluginsPath() );
-
-#ifdef Q_OS_WIN
-  runString( "if oldhome: os.environ['HOME']=oldhome\n" );
-#endif
-
   // This is the other main difference with initInterface() for desktop plugins
-  runString( "qgis.utils.initServerInterface(" + QString::number(( unsigned long ) interface ) + ")" );
+  runString( "qgis.utils.initServerInterface(" + QString::number(( unsigned long ) interface ) + ')' );
 
-  QString startuppath = homePythonPath() + " + \"/startup.py\"";
-  runString( "if os.path.exists(" + startuppath + "): from startup import *\n" );
-
-  // release GIL!
-  // Later on, we acquire GIL just before doing some Python calls and
-  // release GIL again when the work with Python API is done.
-  // (i.e. there must be PyGILState_Ensure + PyGILState_Release pair
-  // around any calls to Python API, otherwise we may segfault!)
-  _mainState = PyEval_SaveThread();
+  finish();
 }
 
 bool QgsPythonUtilsImpl::startServerPlugin( QString packageName )
@@ -342,8 +257,8 @@ bool QgsPythonUtilsImpl::startServerPlugin( QString packageName )
 void QgsPythonUtilsImpl::exitPython()
 {
   Py_Finalize();
-  mMainModule = NULL;
-  mMainDict = NULL;
+  mMainModule = nullptr;
+  mMainDict = nullptr;
   mPythonEnabled = false;
 }
 
@@ -374,9 +289,9 @@ bool QgsPythonUtilsImpl::runStringUnsafe( const QString& command, bool single )
   // TODO: convert special characters from unicode strings u"..." to \uXXXX
   // so that they're not mangled to utf-8
   // (non-unicode strings can be mangled)
-  PyRun_String( command.toUtf8().data(), single ? Py_single_input : Py_file_input, mMainDict, mMainDict );
-
-  bool res = ( PyErr_Occurred() == 0 );
+  PyObject* obj = PyRun_String( command.toUtf8().data(), single ? Py_single_input : Py_file_input, mMainDict, mMainDict );
+  bool res = nullptr == PyErr_Occurred();
+  Py_XDECREF( obj );
 
   // we are done calling python API, release global interpreter lock
   PyGILState_Release( gstate );
@@ -393,7 +308,7 @@ bool QgsPythonUtilsImpl::runString( const QString& command, QString msgOnError,
   if ( msgOnError.isEmpty() )
   {
     // use some default message if custom hasn't been specified
-    msgOnError = QObject::tr( "An error occured during execution of following code:" ) + "\n<tt>" + command + "</tt>";
+    msgOnError = QObject::tr( "An error occurred during execution of following code:" ) + "\n<tt>" + command + "</tt>";
   }
 
   // TODO: use python implementation
@@ -405,9 +320,9 @@ bool QgsPythonUtilsImpl::runString( const QString& command, QString msgOnError,
 
   QString str = "<font color=\"red\">" + msgOnError + "</font><br><pre>\n" + traceback + "\n</pre>"
                 + QObject::tr( "Python version:" ) + "<br>" + version + "<br><br>"
-                + QObject::tr( "QGIS version:" ) + "<br>" + QString( "%1 '%2', %3" ).arg( QGis::QGIS_VERSION ).arg( QGis::QGIS_RELEASE_NAME ).arg( QGis::QGIS_DEV_VERSION ) + "<br><br>"
+                + QObject::tr( "QGIS version:" ) + "<br>" + QString( "%1 '%2', %3" ).arg( QGis::QGIS_VERSION, QGis::QGIS_RELEASE_NAME, QGis::QGIS_DEV_VERSION ) + "<br><br>"
                 + QObject::tr( "Python path:" ) + "<br>" + path;
-  str.replace( "\n", "<br>" ).replace( "  ", "&nbsp; " );
+  str.replace( '\n', "<br>" ).replace( "  ", "&nbsp; " );
 
   QgsMessageOutput* msg = QgsMessageOutput::createMessageOutput();
   msg->setTitle( QObject::tr( "Python error" ) );
@@ -429,28 +344,34 @@ QString QgsPythonUtilsImpl::getTraceback()
   QString errMsg;
   QString result;
 
-  PyObject *modStringIO = NULL;
-  PyObject *modTB = NULL;
-  PyObject *obStringIO = NULL;
-  PyObject *obResult = NULL;
+  PyObject *modStringIO = nullptr;
+  PyObject *modTB = nullptr;
+  PyObject *obStringIO = nullptr;
+  PyObject *obResult = nullptr;
 
   PyObject *type, *value, *traceback;
 
   PyErr_Fetch( &type, &value, &traceback );
   PyErr_NormalizeException( &type, &value, &traceback );
 
-  modStringIO = PyImport_ImportModule( "cStringIO" );
-  if ( modStringIO == NULL )
-    TRACEBACK_FETCH_ERROR( "can't import cStringIO" );
+#ifdef PYTHON2
+  const char* iomod = "cStringIO";
+#else
+  const char* iomod = "io";
+#endif
 
-  obStringIO = PyObject_CallMethod( modStringIO, ( char* ) "StringIO", NULL );
+  modStringIO = PyImport_ImportModule( iomod );
+  if ( !modStringIO )
+    TRACEBACK_FETCH_ERROR( QString( "can't import %1" ).arg( iomod ) );
+
+  obStringIO = PyObject_CallMethod( modStringIO, ( char* ) "StringIO", nullptr );
 
   /* Construct a cStringIO object */
-  if ( obStringIO == NULL )
+  if ( !obStringIO )
     TRACEBACK_FETCH_ERROR( "cStringIO.StringIO() failed" );
 
   modTB = PyImport_ImportModule( "traceback" );
-  if ( modTB == NULL )
+  if ( !modTB )
     TRACEBACK_FETCH_ERROR( "can't import traceback" );
 
   obResult = PyObject_CallMethod( modTB, ( char* ) "print_exception",
@@ -460,19 +381,26 @@ QString QgsPythonUtilsImpl::getTraceback()
                                   Py_None,
                                   obStringIO );
 
-  if ( obResult == NULL )
+  if ( !obResult )
     TRACEBACK_FETCH_ERROR( "traceback.print_exception() failed" );
+
   Py_DECREF( obResult );
 
-  obResult = PyObject_CallMethod( obStringIO, ( char* ) "getvalue", NULL );
-  if ( obResult == NULL )
+  obResult = PyObject_CallMethod( obStringIO, ( char* ) "getvalue", nullptr );
+  if ( !obResult )
     TRACEBACK_FETCH_ERROR( "getvalue() failed." );
 
   /* And it should be a string all ready to go - duplicate it. */
-  if ( !PyString_Check( obResult ) )
+  if ( !
+#ifdef PYTHON2
+       PyString_Check( obResult )
+#else
+       PyUnicode_Check( obResult )
+#endif
+     )
     TRACEBACK_FETCH_ERROR( "getvalue() did not return a string" );
 
-  result = PyString_AsString( obResult );
+  result = PYOBJ2QSTRING( obResult );
 
 done:
 
@@ -498,24 +426,27 @@ done:
 
 QString QgsPythonUtilsImpl::getTypeAsString( PyObject* obj )
 {
-  if ( obj == NULL )
-    return NULL;
+  if ( !obj )
+    return nullptr;
 
+#ifdef PYTHON2
   if ( PyClass_Check( obj ) )
   {
     QgsDebugMsg( "got class" );
     return QString( PyString_AsString((( PyClassObject* )obj )->cl_name ) );
   }
-  else if ( PyType_Check( obj ) )
-  {
-    QgsDebugMsg( "got type" );
-    return QString((( PyTypeObject* )obj )->tp_name );
-  }
   else
-  {
-    QgsDebugMsg( "got object" );
-    return PyObjectToQString( obj );
-  }
+#endif
+    if ( PyType_Check( obj ) )
+    {
+      QgsDebugMsg( "got type" );
+      return QString((( PyTypeObject* )obj )->tp_name );
+    }
+    else
+    {
+      QgsDebugMsg( "got object" );
+      return PyObjectToQString( obj );
+    }
 }
 
 bool QgsPythonUtilsImpl::getError( QString& errorClassName, QString& errorText )
@@ -541,7 +472,7 @@ bool QgsPythonUtilsImpl::getError( QString& errorClassName, QString& errorText )
   errorClassName = getTypeAsString( err_type );
 
   // get exception's text
-  if ( err_value != NULL && err_value != Py_None )
+  if ( nullptr != err_value && err_value != Py_None )
   {
     errorText = PyObjectToQString( err_value );
   }
@@ -573,15 +504,20 @@ QString QgsPythonUtilsImpl::PyObjectToQString( PyObject* obj )
   // check whether the object is already a unicode string
   if ( PyUnicode_Check( obj ) )
   {
+#ifdef PYTHON2
     PyObject* utf8 = PyUnicode_AsUTF8String( obj );
     if ( utf8 )
       result = QString::fromUtf8( PyString_AS_STRING( utf8 ) );
     else
       result = "(qgis error)";
     Py_XDECREF( utf8 );
+#else
+    result = PYOBJ2QSTRING( obj );
+#endif
     return result;
   }
 
+#if PYTHON2
   // check whether the object is a classical (8-bit) string
   if ( PyString_Check( obj ) )
   {
@@ -590,7 +526,6 @@ QString QgsPythonUtilsImpl::PyObjectToQString( PyObject* obj )
 
   // it's some other type of object:
   // convert object to unicode string (equivalent to calling unicode(obj) )
-
   PyObject* obj_uni = PyObject_Unicode( obj ); // obj_uni is new reference
   if ( obj_uni )
   {
@@ -601,16 +536,18 @@ QString QgsPythonUtilsImpl::PyObjectToQString( PyObject* obj )
       result = QString::fromUtf8( PyString_AsString( obj_utf8 ) );
     else
       result = "(qgis error)";
+
     Py_XDECREF( obj_utf8 );
     Py_XDECREF( obj_uni );
     return result;
   }
+#endif
 
   // if conversion to unicode failed, try to convert it to classic string, i.e. str(obj)
   PyObject* obj_str = PyObject_Str( obj ); // new reference
   if ( obj_str )
   {
-    result = QString::fromUtf8( PyString_AS_STRING( obj_str ) );
+    result = PYOBJ2QSTRING( obj_str );
     Py_XDECREF( obj_str );
     return result;
   }
@@ -628,13 +565,15 @@ bool QgsPythonUtilsImpl::evalString( const QString& command, QString& result )
   gstate = PyGILState_Ensure();
 
   PyObject* res = PyRun_String( command.toUtf8().data(), Py_eval_input, mMainDict, mMainDict );
-  bool success = ( res != NULL );
+  bool success = nullptr != res;
 
   // TODO: error handling
 
   if ( success )
     result = PyObjectToQString( res );
 
+  Py_XDECREF( res );
+
   // we are done calling python API, release global interpreter lock
   PyGILState_Release( gstate );
 
@@ -659,11 +598,11 @@ QString QgsPythonUtilsImpl::homePythonPath()
   QString settingsDir = QgsApplication::qgisSettingsDirPath();
   if ( QDir::cleanPath( settingsDir ) == QDir::homePath() + QString( "/.qgis%1" ).arg( QGis::QGIS_VERSION_INT / 10000 ) )
   {
-    return QString( "\"%1/.qgis%2/python\".decode('utf-8')" ).arg( QDir::homePath() ).arg( QGis::QGIS_VERSION_INT / 10000 );
+    return QString( "b\"%1/.qgis%2/python\".decode('utf-8')" ).arg( QDir::homePath() ).arg( QGis::QGIS_VERSION_INT / 10000 );
   }
   else
   {
-    return '"' + settingsDir.replace( '\\', "\\\\" ) + "python\".decode('utf-8')";
+    return "b\"" + settingsDir.replace( '\\', "\\\\" ) + "python\".decode('utf-8')";
   }
 }
 
@@ -675,7 +614,7 @@ QString QgsPythonUtilsImpl::homePluginsPath()
 QStringList QgsPythonUtilsImpl::extraPluginsPaths()
 {
   const char* cpaths = getenv( "QGIS_PLUGINPATH" );
-  if ( cpaths == NULL )
+  if ( !cpaths )
     return QStringList();
 
   QString paths = QString::fromLocal8Bit( cpaths );
@@ -699,7 +638,7 @@ QStringList QgsPythonUtilsImpl::pluginList()
   return output.split( QChar( '\n' ), QString::SkipEmptyParts );
 }
 
-QString QgsPythonUtilsImpl::getPluginMetadata( QString pluginName, QString function )
+QString QgsPythonUtilsImpl::getPluginMetadata( const QString& pluginName, const QString& function )
 {
   QString res;
   QString str = "qgis.utils.pluginMetadata('" + pluginName + "', '" + function + "')";
@@ -708,35 +647,35 @@ QString QgsPythonUtilsImpl::getPluginMetadata( QString pluginName, QString funct
   return res;
 }
 
-bool QgsPythonUtilsImpl::loadPlugin( QString packageName )
+bool QgsPythonUtilsImpl::loadPlugin( const QString& packageName )
 {
   QString output;
   evalString( "qgis.utils.loadPlugin('" + packageName + "')", output );
   return ( output == "True" );
 }
 
-bool QgsPythonUtilsImpl::startPlugin( QString packageName )
+bool QgsPythonUtilsImpl::startPlugin( const QString& packageName )
 {
   QString output;
   evalString( "qgis.utils.startPlugin('" + packageName + "')", output );
   return ( output == "True" );
 }
 
-bool QgsPythonUtilsImpl::canUninstallPlugin( QString packageName )
+bool QgsPythonUtilsImpl::canUninstallPlugin( const QString& packageName )
 {
   QString output;
   evalString( "qgis.utils.canUninstallPlugin('" + packageName + "')", output );
   return ( output == "True" );
 }
 
-bool QgsPythonUtilsImpl::unloadPlugin( QString packageName )
+bool QgsPythonUtilsImpl::unloadPlugin( const QString& packageName )
 {
   QString output;
   evalString( "qgis.utils.unloadPlugin('" + packageName + "')", output );
   return ( output == "True" );
 }
 
-bool QgsPythonUtilsImpl::isPluginLoaded( QString packageName )
+bool QgsPythonUtilsImpl::isPluginLoaded( const QString& packageName )
 {
   QString output;
   evalString( "qgis.utils.isPluginLoaded('" + packageName + "')", output );
