@@ -198,18 +198,18 @@ void QgsServer::printRequestParameters( const QMap< QString, QString>& parameter
  */
 void QgsServer::printRequestInfos()
 {
-  QgsMessageLog::logMessage( "********************new request***************", "Server", QgsMessageLog::INFO );
+  QgsMessageLog::logMessage( QStringLiteral( "******************** New request ***************" ), QStringLiteral( "Server" ), QgsMessageLog::INFO );
   if ( getenv( "REMOTE_ADDR" ) )
   {
-    QgsMessageLog::logMessage( "remote ip: " + QString( getenv( "REMOTE_ADDR" ) ), "Server", QgsMessageLog::INFO );
+    QgsMessageLog::logMessage( "REMOTE_ADDR: " + QString( getenv( "REMOTE_ADDR" ) ), QStringLiteral( "Server" ), QgsMessageLog::INFO );
   }
   if ( getenv( "REMOTE_HOST" ) )
   {
-    QgsMessageLog::logMessage( "remote ip: " + QString( getenv( "REMOTE_HOST" ) ), "Server", QgsMessageLog::INFO );
+    QgsMessageLog::logMessage( "REMOTE_HOST: " + QString( getenv( "REMOTE_HOST" ) ), QStringLiteral( "Server" ), QgsMessageLog::INFO );
   }
   if ( getenv( "REMOTE_USER" ) )
   {
-    QgsMessageLog::logMessage( "remote user: " + QString( getenv( "REMOTE_USER" ) ), "Server", QgsMessageLog::INFO );
+    QgsMessageLog::logMessage( "REMOTE_USER: " + QString( getenv( "REMOTE_USER" ) ), QStringLiteral( "Server" ), QgsMessageLog::INFO );
   }
   if ( getenv( "REMOTE_IDENT" ) )
   {
@@ -239,6 +239,10 @@ void QgsServer::printRequestInfos()
   {
     QgsMessageLog::logMessage( "NO_PROXY: " + QString( getenv( "NO_PROXY" ) ), "Server", QgsMessageLog::INFO );
   }
+  if ( getenv( "HTTP_AUTHORIZATION" ) )
+  {
+    QgsMessageLog::logMessage( "HTTP_AUTHORIZATION: " + QString( getenv( "HTTP_AUTHORIZATION" ) ), QStringLiteral( "Server" ), QgsMessageLog::INFO );
+  }
 }
 
 void QgsServer::dummyMessageHandler( QtMsgType type, const char *msg )
@@ -441,9 +445,9 @@ bool QgsServer::init( int & argc, char ** argv )
 void QgsServer::putenv( const QString &var, const QString &val )
 {
 #ifdef _MSC_VER
-  _putenv_s( var.toUtf8().data(), val.toUtf8().data() );
+  _putenv_s( var.toStdString().c_str(), val.toStdString().c_str() );
 #else
-  setenv( var.toUtf8().data(), val.toUtf8().data(), 1 );
+  setenv( var.toStdString().c_str(), val.toStdString().c_str(), 1 );
 #endif
 }
 
