@@ -25,6 +25,7 @@
 #include "qgsmaplayerregistry.h"
 #include "qgsmslayercache.h"
 #include "qgsrasterlayer.h"
+#include "qgsvectorlayerjoinbuffer.h"
 #include "qgseditorwidgetregistry.h"
 #include "qgslayertreegroup.h"
 
@@ -79,6 +80,17 @@ QgsServerProjectParser::QgsServerProjectParser( QDomDocument* xmlDoc, const QStr
   {
     QgsProject::instance()->setFileName( mProjectPath );
   }
+
+  // Set the project scope variables
+  QStringList variableNames = readListEntry( "Variables", "variableNames" );
+  QStringList variableValues = readListEntry( "Variables", "variableValues" );
+
+  //read values
+
+  //append standard values
+
+  QgsProject::instance()->writeEntry( "Variables", "/variableNames", variableNames );
+  QgsProject::instance()->writeEntry( "Variables", "/variableValues", variableValues );
 }
 
 QgsServerProjectParser::QgsServerProjectParser()
@@ -234,7 +246,12 @@ QgsMapLayer* QgsServerProjectParser::createLayerFromElement( const QDomElement&
     if ( !QgsMapLayerRegistry::instance()->mapLayer( id ) )
       QgsMapLayerRegistry::instance()->addMapLayer( layer, false, false );
     if ( layer->type() == QgsMapLayer::VectorLayer )
-      addValueRelationLayersForLayer( dynamic_cast<QgsVectorLayer *>( layer ) );
+    {
+      QgsVectorLayer* vlayer = qobject_cast<QgsVectorLayer *>( layer );
+      addValueRelationLayersForLayer( vlayer );
+      QgsVectorLayerJoinBuffer* joinBuffer = vlayer->joinBuffer();
+      joinBuffer->readXml( const_cast<QDomElement&>( elem ) );
+    }
 
     return layer;
   }
@@ -272,6 +289,10 @@ QgsMapLayer* QgsServerProjectParser::createLayerFromElement( const QDomElement&
     layer->readLayerXML( const_cast<QDomElement&>( elem ) ); //should be changed to const in QgsMapLayer
     //layer->setLayerName( layerName( elem ) );
 
+    if ( !layer->isValid() )
+    {
+      return nullptr;
+    }
     // Insert layer in registry and cache before addValueRelationLayersForLayer
     if ( !QgsMapLayerRegistry::instance()->mapLayer( id ) )
       QgsMapLayerRegistry::instance()->addMapLayer( layer, false, false );
@@ -287,7 +308,7 @@ QgsMapLayer* QgsServerProjectParser::createLayerFromElement( const QDomElement&
 
     if ( layer->type() == QgsMapLayer::VectorLayer )
     {
-      addValueRelationLayersForLayer( dynamic_cast<QgsVectorLayer *>( layer ) );
+      addValueRelationLayersForLayer( qobject_cast<QgsVectorLayer *>( layer ) );
     }
   }
   return layer;
@@ -574,7 +595,7 @@ void QgsServerProjectParser::serviceCapabilities( QDomElement& parentElement, QD
   //Fees
   QDomElement feesElem = propertiesElement.firstChildElement( "WMSFees" );
   QDomElement wmsFeesElem = doc.createElement( "Fees" );
-  QDomText wmsFeesText = doc.createTextNode( "conditions unknown" ); // default value if access conditions are unknown
+  QDomText wmsFeesText = doc.createTextNode( "None" ); // default value if access conditions are unknown
   if ( !feesElem.isNull() && !feesElem.text().isEmpty() )
   {
     wmsFeesText = doc.createTextNode( feesElem.text() );
@@ -1163,6 +1184,27 @@ QSet<QString> QgsServerProjectParser::findRestrictedLayers() const
       }
     }
   }
+  // Add short name in restricted layers
+  else
+  {
+    QDomNodeList layerNodeList = mXMLDoc->elementsByTagName( "maplayer" );
+    for ( int i = 0; i < layerNodeList.size(); ++i )
+    {
+      QDomElement layerElem = layerNodeList.at( i ).toElement();
+      // get name
+      QString lName = layerName( layerElem );
+      if ( restrictedLayerSet.contains( lName ) )
+      {
+        // get short name
+        lName = layerShortName( layerElem );
+        if ( !lName.isEmpty() )
+        {
+          // add short name
+          restrictedLayerSet.insert( lName );
+        }
+      }
+    }
+  }
   return restrictedLayerSet;
 }
 
@@ -1535,7 +1577,7 @@ void QgsServerProjectParser::addJoinLayersForElement( const QDomElement& layerEl
   {
     QString id = joinNodeList.at( i ).toElement().attribute( "joinLayerId" );
     QgsMapLayer* layer = mapLayerFromLayerId( id );
-    if ( layer )
+    if ( layer && !QgsMapLayerRegistry::instance()->mapLayer( id ) )
     {
       QgsMapLayerRegistry::instance()->addMapLayer( layer, false, false );
     }
@@ -1602,4 +1644,34 @@ void QgsServerProjectParser::addGetFeatureLayers( const QDomElement& layerElem )
   }
 }
 
+QStringList QgsServerProjectParser::readListEntry( const QString& scope, const QString& key ) const
+{
+  QStringList entryList;
+  QDomElement propertiesElement = propertiesElem();
+  if ( propertiesElement.isNull() )
+  {
+    return entryList;
+  }
+
+  QDomElement scopeElem = propertiesElement.firstChildElement( scope );
+  if ( scopeElem.isNull() )
+  {
+    return entryList;
+  }
+
+  QDomElement keyElem = scopeElem.firstChildElement( key );
+  if ( keyElem.isNull() )
+  {
+    return entryList;
+  }
+
+  QDomNodeList valueNodeList = keyElem.elementsByTagName( "value" );
+  for ( int i = 0; i < valueNodeList.size(); ++i )
+  {
+    entryList.append( valueNodeList.at( i ).toElement().text() );
+  }
+
+  return entryList;
+}
+
 
