@@ -111,6 +111,7 @@ class SERVER_EXPORT QgsServerProjectParser
     QStringList wfsLayers() const;
     QStringList wcsLayers() const;
 
+    /** Add layers for vector joins */
     void addJoinLayersForElement( const QDomElement& layerElem ) const;
 
     void addValueRelationLayersForLayer( const QgsVectorLayer *vl ) const;
@@ -174,6 +175,9 @@ class SERVER_EXPORT QgsServerProjectParser
 
     /** Adds sublayers of an embedded group to layer set*/
     static void sublayersOfEmbeddedGroup( const QString& projectFilePath, const QString& groupName, QSet<QString>& layerSet );
+
+    /** Reads list entry from project properties*/
+    QStringList readListEntry( const QString& scope, const QString& key ) const;
 };
 
 #endif // QGSSERVERPROJECTPARSER_H
