@@ -18,6 +18,8 @@
 #include "qgscomposition.h"
 #include "qgscomposermodel.h"
 #include "qgscomposerlabel.h"
+#include "qgsapplication.h"
+
 #include <QObject>
 #include <QtTest/QtTest>
 #include <QList>
@@ -29,6 +31,7 @@ class TestQgsComposerModel : public QObject
   public:
     TestQgsComposerModel()
         : mComposition( 0 )
+        , mMapSettings( 0 )
         , mItem1( 0 )
         , mItem2( 0 )
         , mItem3( 0 )
@@ -60,22 +63,32 @@ class TestQgsComposerModel : public QObject
     void reorderToBottomWithRemoved(); //test reordering to bottom with removed items
 
   private:
-    QgsComposition* mComposition;
-    QgsMapSettings mMapSettings;
-    QgsComposerLabel* mItem1;
-    QgsComposerLabel* mItem2;
-    QgsComposerLabel* mItem3;
+    QgsComposition *mComposition;
+    QgsMapSettings *mMapSettings;
+    QgsComposerLabel *mItem1;
+    QgsComposerLabel *mItem2;
+    QgsComposerLabel *mItem3;
 };
 
 void TestQgsComposerModel::initTestCase()
 {
-  mComposition = new QgsComposition( mMapSettings );
+  QgsApplication::init();
+  QgsApplication::initQgis();
+
+  mMapSettings = new QgsMapSettings();
+  mComposition = new QgsComposition( *mMapSettings );
+
   mComposition->setPaperSize( 297, 210 ); //A4 landscape
 }
 
 void TestQgsComposerModel::cleanupTestCase()
 {
+  delete mItem1;
+  delete mItem2;
+  delete mItem3;
   delete mComposition;
+  delete mMapSettings;
+  QgsApplication::exitQgis();
 }
 
 void TestQgsComposerModel::init()
@@ -223,6 +236,9 @@ void TestQgsComposerModel::removeItem()
   QCOMPARE( mComposition->itemsModel()->rowCount(), 0 );
   //also check scene list
   QCOMPARE( mComposition->itemsModel()->mItemsInScene.size(), 0 );
+
+  delete item1;
+  delete item2;
 }
 
 void TestQgsComposerModel::reorderUp()
@@ -450,6 +466,7 @@ void TestQgsComposerModel::setItemRemoved()
   QCOMPARE( mComposition->itemsModel()->mItemsInScene.size(), 2 );
   QCOMPARE( mComposition->itemsModel()->mItemsInScene.at( 0 ), mItem2 );
   QCOMPARE( mComposition->itemsModel()->mItemsInScene.at( 1 ), mItem1 );
+  delete label;
 }
 
 void TestQgsComposerModel::rebuildZListWithRemoved()
