@@ -25,6 +25,8 @@
 #include "qgsmultibandcolorrenderer.h"
 #include "qgsrasterlayer.h"
 #include "qgsfontutils.h"
+
+#include <QLocale>
 #include <QObject>
 #include <QtTest/QtTest>
 
@@ -38,6 +40,7 @@ class TestQgsComposerScaleBar : public QObject
         , mComposerMap( 0 )
         , mComposerScaleBar( 0 )
         , mRasterLayer( 0 )
+        , mMapSettings( 0 )
     {}
 
   private slots:
@@ -56,8 +59,9 @@ class TestQgsComposerScaleBar : public QObject
     QgsComposerMap* mComposerMap;
     QgsComposerScaleBar* mComposerScaleBar;
     QgsRasterLayer* mRasterLayer;
-    QgsMapSettings mMapSettings;
+    QgsMapSettings *mMapSettings;
     QString mReport;
+    QPen mPrevPen;
 };
 
 void TestQgsComposerScaleBar::initTestCase()
@@ -65,8 +69,16 @@ void TestQgsComposerScaleBar::initTestCase()
   QgsApplication::init();
   QgsApplication::initQgis();
 
+  // the scale denominator is formatted in a locale aware manner
+  // so 10000 is rendered as "10,000" in C (or en_US) locale, however
+  // other locales may render the number differently (e.g. "10 000" in cs_CZ)
+  // so we enforce C locale to make sure we get expected result
+  QLocale::setDefault( QLocale::c() );
+
+  mMapSettings = new QgsMapSettings();
+
   //create maplayers from testdata and add to layer registry
-  QFileInfo rasterFileInfo( QString( TEST_DATA_DIR ) + QDir::separator() +  "landsat.tif" );
+  QFileInfo rasterFileInfo( QString( TEST_DATA_DIR ) + "/landsat.tif" );
   mRasterLayer = new QgsRasterLayer( rasterFileInfo.filePath(),
                                      rasterFileInfo.completeBaseName() );
   QgsMultiBandColorRenderer* rasterRenderer = new QgsMultiBandColorRenderer( mRasterLayer->dataProvider(), 2, 3, 4 );
@@ -75,15 +87,15 @@ void TestQgsComposerScaleBar::initTestCase()
   QgsMapLayerRegistry::instance()->addMapLayers( QList<QgsMapLayer*>() << mRasterLayer );
 
   //create composition with composer map
-  mMapSettings.setLayers( QStringList() << mRasterLayer->id() );
+  mMapSettings->setLayers( QStringList() << mRasterLayer->id() );
 
   //reproject to WGS84
   QgsCoordinateReferenceSystem destCRS;
   destCRS.createFromId( 4326, QgsCoordinateReferenceSystem::EpsgCrsId );
-  mMapSettings.setDestinationCrs( destCRS );
-  mMapSettings.setCrsTransformEnabled( true );
+  mMapSettings->setDestinationCrs( destCRS );
+  mMapSettings->setCrsTransformEnabled( true );
 
-  mComposition = new QgsComposition( mMapSettings );
+  mComposition = new QgsComposition( *mMapSettings );
   mComposition->setPaperSize( 297, 210 ); //A4 landscape
   mComposerMap = new QgsComposerMap( mComposition, 20, 20, 150, 150 );
   mComposerMap->setFrameEnabled( true );
@@ -100,7 +112,9 @@ void TestQgsComposerScaleBar::initTestCase()
   mComposerScaleBar->setNumSegmentsLeft( 0 );
   mComposerScaleBar->setNumSegments( 2 );
   mComposerScaleBar->setHeight( 5 );
-
+  QPen scalePen = mComposerScaleBar->pen();
+  scalePen.setWidthF( 1.0 );
+  mComposerScaleBar->setPen( scalePen );
 
   qWarning() << "scalebar font: " << mComposerScaleBar->font().toString() << " exactMatch:" << mComposerScaleBar->font().exactMatch();
 
@@ -110,8 +124,9 @@ void TestQgsComposerScaleBar::initTestCase()
 void TestQgsComposerScaleBar::cleanupTestCase()
 {
   delete mComposition;
+  delete mMapSettings;
 
-  QString myReportFile = QDir::tempPath() + QDir::separator() + "qgistest.html";
+  QString myReportFile = QDir::tempPath() + "/qgistest.html";
   QFile myFile( myReportFile );
   if ( myFile.open( QIODevice::WriteOnly | QIODevice::Append ) )
   {
@@ -121,64 +136,71 @@ void TestQgsComposerScaleBar::cleanupTestCase()
   }
 
   QgsApplication::exitQgis();
-};
+}
 
 void TestQgsComposerScaleBar::init()
 {
 
-};
+}
 
 void TestQgsComposerScaleBar::cleanup()
 {
 
-};
+}
 
 void TestQgsComposerScaleBar::singleBox()
 {
   mComposerScaleBar->setStyle( "Single Box" );
   QgsCompositionChecker checker( "composerscalebar_singlebox", mComposition );
+  checker.setControlPathPrefix( "composer_scalebar" );
   QVERIFY( checker.testComposition( mReport, 0, 0 ) );
-};
+}
 
 void TestQgsComposerScaleBar::singleBoxAlpha()
 {
   mComposerScaleBar->setStyle( "Single Box" );
   mComposerScaleBar->setBrush( QBrush( QColor( 255, 0, 0, 100 ) ) );
   mComposerScaleBar->setBrush2( QBrush( QColor( 0, 255, 0, 50 ) ) );
-  QPen prevPen = mComposerScaleBar->pen();
-  QPen newPen = prevPen;
+  mPrevPen = mComposerScaleBar->pen();
+  QPen newPen = mPrevPen;
   newPen.setColor( QColor( 0, 0, 255, 150 ) );
   mComposerScaleBar->setPen( newPen );
   mComposerScaleBar->setFontColor( QColor( 255, 0, 255, 100 ) );
   QgsCompositionChecker checker( "composerscalebar_singlebox_alpha", mComposition );
+  checker.setControlPathPrefix( "composer_scalebar" );
   QVERIFY( checker.testComposition( mReport, 0, 0 ) );
-  mComposerScaleBar->setBrush( QBrush( Qt::black ) );
-  mComposerScaleBar->setBrush2( QBrush( Qt::white ) );
-  mComposerScaleBar->setPen( prevPen );
-  mComposerScaleBar->setFontColor( Qt::black );
-};
+}
 
 void TestQgsComposerScaleBar::doubleBox()
 {
+  // cleanup singleBoxAlpha
+  mComposerScaleBar->setBrush( QBrush( Qt::black ) );
+  mComposerScaleBar->setBrush2( QBrush( Qt::white ) );
+  mComposerScaleBar->setPen( mPrevPen );
+  mComposerScaleBar->setFontColor( Qt::black );
   mComposerScaleBar->setStyle( "Double Box" );
+
   QgsCompositionChecker checker( "composerscalebar_doublebox", mComposition );
+  checker.setControlPathPrefix( "composer_scalebar" );
   QVERIFY( checker.testComposition( mReport, 0, 0 ) );
-};
+}
 
 void TestQgsComposerScaleBar::numeric()
 {
   mComposerScaleBar->setStyle( "Numeric" );
   mComposerScaleBar->setSceneRect( QRectF( 20, 180, 50, 20 ) );
   QgsCompositionChecker checker( "composerscalebar_numeric", mComposition );
+  checker.setControlPathPrefix( "composer_scalebar" );
   QVERIFY( checker.testComposition( mReport, 0, 0 ) );
-};
+}
 
 void TestQgsComposerScaleBar::tick()
 {
   mComposerScaleBar->setStyle( "Line Ticks Up" );
   QgsCompositionChecker checker( "composerscalebar_tick", mComposition );
+  checker.setControlPathPrefix( "composer_scalebar" );
   QVERIFY( checker.testComposition( mReport, 0, 0 ) );
-};
+}
 
 QTEST_MAIN( TestQgsComposerScaleBar )
 #include "testqgscomposerscalebar.moc"
