@@ -13,8 +13,6 @@ Email                : sherman at mrcc dot com
  *                                                                         *
  ***************************************************************************/
 #include <QtTest/QtTest>
-#include <iostream>
-
 #include <QPixmap>
 
 #include <qgsapplication.h>
@@ -63,6 +61,7 @@ class TestQgsCoordinateReferenceSystem: public QObject
     void mapUnits();
     void setValidationHint();
     void axisInverted();
+    void createFromProj4Invalid();
   private:
     void debugPrint( QgsCoordinateReferenceSystem &theCrs );
     // these used by createFromESRIWkt()
@@ -150,7 +149,7 @@ void TestQgsCoordinateReferenceSystem::createFromOgcWmsCrs()
   QgsCoordinateReferenceSystem myCrs;
   //@todo implement this - for now we just check that if fails
   //if passed an empty string
-  QVERIFY( !myCrs.createFromOgcWmsCrs( QString( "" ) ) );
+  QVERIFY( !myCrs.createFromOgcWmsCrs( QString() ) );
 }
 void TestQgsCoordinateReferenceSystem::createFromSrid()
 {
@@ -180,10 +179,10 @@ QString TestQgsCoordinateReferenceSystem::testESRIWkt( int i, QgsCoordinateRefer
 #endif
   if ( myCrs.toProj4().indexOf( myTOWGS84Strings[i] ) == -1 )
     return QString( "test %1 [%2] not found, PROJ.4 = [%3] expecting [%4]"
-                  ).arg( i ).arg( myTOWGS84Strings[i] ).arg( myCrs.toProj4() ).arg( myProj4Strings[i] );
+                  ).arg( i ).arg( myTOWGS84Strings[i], myCrs.toProj4(), myProj4Strings[i] );
   if ( myCrs.authid() !=  myAuthIdStrings[i] )
     return QString( "test %1 AUTHID = [%2] expecting [%3]"
-                  ).arg( i ).arg( myCrs.authid() ).arg( myAuthIdStrings[i] );
+                  ).arg( i ).arg( myCrs.authid(), myAuthIdStrings[i] );
 
   return "";
 }
@@ -215,8 +214,14 @@ void TestQgsCoordinateReferenceSystem::createFromESRIWkt()
   myWktStrings << "GEOGCS[\"GCS_South_American_1969\",DATUM[\"D_South_American_1969\",SPHEROID[\"GRS_1967_Truncated\",6378160.0,298.25]],PRIMEM[\"Greenwich\",0.0],UNIT[\"Degree\",0.0174532925199433]]";
   myFiles << "";
   myGdalVersionOK << 1900;
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 2000000
+  //proj definition for EPSG:4618 was updated in GDAL 2.0 - see https://github.com/OSGeo/proj.4/issues/241
+  myProj4Strings << "+proj=longlat +ellps=aust_SA +towgs84=-66.87,4.37,-38.52,0,0,0,0 +no_defs";
+  myTOWGS84Strings << "+towgs84=-66.87,4.37,-38.52,0,0,0,0";
+#else
   myProj4Strings << "+proj=longlat +ellps=aust_SA +towgs84=-57,1,-41,0,0,0,0 +no_defs";
   myTOWGS84Strings << "+towgs84=-57,1,-41,0,0,0,0";
+#endif
   myAuthIdStrings << "EPSG:4618";
 
   // do test with WKT definitions
@@ -227,7 +232,7 @@ void TestQgsCoordinateReferenceSystem::createFromESRIWkt()
     CPLSetConfigOption( "GDAL_FIX_ESRI_WKT", configOld );
     myCrs.createFromUserInput( "ESRI::" + myWktStrings[i] );
     msg = testESRIWkt( i, myCrs );
-    if ( GDAL_VERSION_NUM < myGdalVersionOK[i] )
+    if ( GDAL_VERSION_NUM < myGdalVersionOK.at( i ) )
     {
       QEXPECT_FAIL( "", QString( "expected failure with GDAL %1 : %2"
                                ).arg( GDAL_VERSION_NUM ).arg( msg ).toLocal8Bit().constData(),
@@ -243,7 +248,7 @@ void TestQgsCoordinateReferenceSystem::createFromESRIWkt()
     {
       // use ogr to open file, make sure CRS is ok
       // this probably could be in another test, but leaving it here since it deals with CRS
-      QString fileStr = QString( TEST_DATA_DIR ) + QDir::separator() + myFiles[i];
+      QString fileStr = QString( TEST_DATA_DIR ) + '/' + myFiles[i];
       QgsDebugMsg( QString( "i=%1 file=%2" ).arg( i ).arg( fileStr ) );
 
       QgsVectorLayer *myLayer = new QgsVectorLayer( fileStr, "", "ogr" );
@@ -256,10 +261,10 @@ void TestQgsCoordinateReferenceSystem::createFromESRIWkt()
       {
         myCrs = myLayer->crs();
         msg = testESRIWkt( i, myCrs );
-        if ( GDAL_VERSION_NUM < myGdalVersionOK[i] )
+        if ( GDAL_VERSION_NUM < myGdalVersionOK.at( i ) )
         {
           QEXPECT_FAIL( "", QString( "expected failure with GDAL %1 : %2 using layer %3"
-                                   ).arg( GDAL_VERSION_NUM ).arg( msg ).arg( fileStr ).toLocal8Bit().constData(),
+                                   ).arg( GDAL_VERSION_NUM ).arg( msg, fileStr ).toLocal8Bit().constData(),
                         Continue );
         }
         if ( !msg.isEmpty() )
@@ -420,7 +425,7 @@ void TestQgsCoordinateReferenceSystem::setValidationHint()
 {
   QgsCoordinateReferenceSystem myCrs;
   myCrs.setValidationHint( "<head>" );
-  QVERIFY( myCrs.validationHint() == QString( "<head>" ) );
+  QVERIFY( myCrs.validationHint() == "<head>" );
   debugPrint( myCrs );
 }
 
@@ -466,5 +471,11 @@ void TestQgsCoordinateReferenceSystem::debugPrint(
   }
 }
 
+void TestQgsCoordinateReferenceSystem::createFromProj4Invalid()
+{
+  QgsCoordinateReferenceSystem myCrs;
+  QVERIFY( !myCrs.createFromProj4( "+proj=longlat +no_defs" ) );
+}
+
 QTEST_MAIN( TestQgsCoordinateReferenceSystem )
 #include "testqgscoordinatereferencesystem.moc"
