@@ -15,13 +15,11 @@
 #include <QtTest/QtTest>
 #include <QObject>
 #include <QString>
-#include <QObject>
 #include <QApplication>
 #include <QFileInfo>
 #include <QDir>
 #include <QDesktopServices>
 
-#include <iostream>
 //qgis includes...
 #include <qgsapplication.h>
 #include <qgsgeometry.h>
@@ -51,6 +49,8 @@ class TestQgsPoint: public QObject
     void sqrDist();
     void multiply();
     void onSegment();
+    void compare();
+
   private:
     QgsPoint mPoint1;
     QgsPoint mPoint2;
@@ -146,7 +146,7 @@ void TestQgsPoint::cleanupTestCase()
   //
   // Runs once after all tests are run
   //
-  QString myReportFile = QDir::tempPath() + QDir::separator() + "qgistest.html";
+  QString myReportFile = QDir::tempPath() + "/qgistest.html";
   QFile myFile( myReportFile );
   if ( myFile.open( QIODevice::WriteOnly | QIODevice::Append ) )
   {
@@ -623,5 +623,16 @@ void TestQgsPoint::onSegment()
 
 }
 
+void TestQgsPoint::compare()
+{
+  QgsPoint point1( 5.000000000001, 9.0 );
+  QgsPoint point2( 5.0, 8.999999999999999 );
+  QVERIFY( point1.compare( point2, 0.00000001 ) );
+  QgsPoint point3( 5.0, 6.0 );
+  QVERIFY( !( point3.compare( point1 ) ) );
+  QgsPoint point4( 10 / 3.0, 12 / 7.0 );
+  QVERIFY( point4.compare( QgsPoint( 10 / 3.0, 12 / 7.0 ) ) );
+}
+
 QTEST_MAIN( TestQgsPoint )
 #include "testqgspoint.moc"
