@@ -17,7 +17,6 @@
 #include <QtTest/QtTest>
 #include <QObject>
 #include <QString>
-#include <QObject>
 #include <QSharedPointer>
 #include <QCoreApplication>
 #include <QWidget>
@@ -46,7 +45,8 @@ class TestQgsRubberband : public QObject
 
     void testAddSingleMultiGeometries(); //test for #7728
     void testBoundingRect(); //test for #12392
-    void testVisibility(); //test for #12486
+    void testVisibility(); //test for 12486
+    void testClose(); //test closing geometry
 
   private:
     QgsMapCanvas* mCanvas;
@@ -63,7 +63,7 @@ void TestQgsRubberband::initTestCase()
 
   // Setup a map canvas with a vector layer loaded...
   QString myDataDir( TEST_DATA_DIR ); //defined in CmakeLists.txt
-  mTestDataDir = myDataDir + QDir::separator();
+  mTestDataDir = myDataDir + '/';
 
   //
   // load a vector layer
@@ -117,40 +117,40 @@ void TestQgsRubberband::testBoundingRect()
   // Set extent to match canvas size.
   // This is to ensure a 1:1 scale
   mCanvas->setExtent( QgsRectangle( QRectF(
-      QPointF(0,0), mapSize
-  ) ) );
-  QCOMPARE( mCanvas->mapUnitsPerPixel (), 1.0 );
+                                      QPointF( 0, 0 ), mapSize
+                                    ) ) );
+  QCOMPARE( mCanvas->mapUnitsPerPixel(), 1.0 );
 
   // Polygon extent is 10,10 to 30,30
   QSharedPointer<QgsGeometry> geom( QgsGeometry::fromWkt(
-      "POLYGON((10 10,10 30,30 30,30 10,10 10))"
-  ) );
+                                      "POLYGON((10 10,10 30,30 30,30 10,10 10))"
+                                    ) );
   mRubberband = new QgsRubberBand( mCanvas, mPolygonLayer->geometryType() );
   mRubberband->setIconSize( 5 ); // default, but better be explicit
   mRubberband->setWidth( 1 );    // default, but better be explicit
   mRubberband->addGeometry( geom.data(), mPolygonLayer );
 
   // 20 pixels for the extent + 3 for pen & icon per side + 2 of padding
-  QCOMPARE( mRubberband->boundingRect(), QRectF(QPointF(-1,-1),QSizeF(28,28)) );
+  QCOMPARE( mRubberband->boundingRect(), QRectF( QPointF( -1, -1 ), QSizeF( 28, 28 ) ) );
   QCOMPARE( mRubberband->pos(), QPointF(
-    // 10 for extent minx - 3 for pen & icon
-    7,
-    // 30 for extent maxy - 3 for pen & icon
-    mapSize.height() - 30 - 3
-  ) );
+              // 10 for extent minx - 3 for pen & icon
+              7,
+              // 30 for extent maxy - 3 for pen & icon
+              mapSize.height() - 30 - 3
+            ) );
 
   mCanvas->setExtent( QgsRectangle( QRectF(
-      QPointF(0,0), mapSize/2
-  ) ) );
+                                      QPointF( 0, 0 ), mapSize / 2
+                                    ) ) );
 
   // 40 pixels for the extent + 6 for pen & icon per side + 2 of padding
-  QCOMPARE( mRubberband->boundingRect(), QRectF(QPointF(-1,-1),QSizeF(54,54)) );
+  QCOMPARE( mRubberband->boundingRect(), QRectF( QPointF( -1, -1 ), QSizeF( 54, 54 ) ) );
   QCOMPARE( mRubberband->pos(), QPointF(
-    // 10 for extent minx - 3 for pen & icon
-    7 * 2,
-    // 30 for extent maxy - 3 for pen & icon
-    mapSize.height() - ( 30 + 3 ) * 2
-  ) );
+              // 10 for extent minx - 3 for pen & icon
+              7 * 2,
+              // 30 for extent maxy - 3 for pen & icon
+              mapSize.height() - ( 30 + 3 ) * 2
+            ) );
 
 }
 
@@ -173,13 +173,13 @@ void TestQgsRubberband::testVisibility()
 
   // Check visibility after setting to valid geometry
   QSharedPointer<QgsGeometry> geom( QgsGeometry::fromWkt(
-      "POLYGON((10 10,10 30,30 30,30 10,10 10))"
-  ) );
+                                      "POLYGON((10 10,10 30,30 30,30 10,10 10))"
+                                    ) );
   mRubberband->setToGeometry( geom.data(), mPolygonLayer );
   QCOMPARE( mRubberband->isVisible(), true );
 
   // Add point without update
-  mRubberband->reset( true );
+  mRubberband->reset( QGis::Polygon );
   mRubberband->addPoint( QgsPoint( 10, 10 ), false );
   QCOMPARE( mRubberband->isVisible(), false );
 
@@ -194,6 +194,34 @@ void TestQgsRubberband::testVisibility()
 
 }
 
+void TestQgsRubberband::testClose()
+{
+  QgsRubberBand r( mCanvas, QGis::Polygon );
+
+  // try closing empty rubber band, don't want to crash
+  r.closePoints();
+  QCOMPARE( r.partSize( 0 ), 0 );
+
+  r.addPoint( QgsPoint( 1, 2 ) );
+  r.addPoint( QgsPoint( 1, 3 ) );
+  r.addPoint( QgsPoint( 2, 3 ) );
+  QCOMPARE( r.partSize( 0 ), 3 );
+
+  // test with some bad geometry indexes - don't want to crash!
+  r.closePoints( true, -1 );
+  QCOMPARE( r.partSize( 0 ), 3 );
+  r.closePoints( true, 100 );
+  QCOMPARE( r.partSize( 0 ), 3 );
+
+  // valid close
+  r.closePoints();
+  QCOMPARE( r.partSize( 0 ), 4 );
+
+  // close already closed polygon, should be no change
+  r.closePoints();
+  QCOMPARE( r.partSize( 0 ), 4 );
+}
+
 
 QTEST_MAIN( TestQgsRubberband )
 #include "testqgsrubberband.moc"
