@@ -38,7 +38,7 @@
 #include <qgswcscapabilities.h>
 #include <testqgswcspublicservers.h>
 
-#ifdef WIN32
+#ifdef Q_OS_WIN
 // Open files in binary mode
 #include <fcntl.h> /*  _O_BINARY */
 #ifdef MSVC
@@ -78,7 +78,7 @@ void TestQgsWcsPublicServers::init()
   // Unfortunately this seems to be the only way to set timeout, we try to reset it
   // at the end but it can be canceled before ...
   QSettings settings;
-  mOrigTimeout = settings.value( "/qgis/networkAndProxy/networkTimeout", "20000" ).toInt();
+  mOrigTimeout = settings.value( "/qgis/networkAndProxy/networkTimeout", "60000" ).toInt();
   settings.setValue( "/qgis/networkAndProxy/networkTimeout", mTimeout );
 
   //mCacheDir = QDir( "./wcstestcache" );
@@ -97,7 +97,7 @@ void TestQgsWcsPublicServers::init()
 
   QStringList providers;
   providers << "wcs" << "gdal";
-  foreach ( QString provider, providers )
+  Q_FOREACH ( const QString& provider, providers )
   {
     QString prefix = provider == "gdal" ? "GDAL " : "";
     mHead << prefix + "CRS";
@@ -189,7 +189,7 @@ void TestQgsWcsPublicServers::init()
 
 TestQgsWcsPublicServers::Server TestQgsWcsPublicServers::getServer( const QString & url )
 {
-  foreach ( Server server, mServers )
+  Q_FOREACH ( const Server& server, mServers )
   {
     if ( server.url == url ) return server;
   }
@@ -199,14 +199,14 @@ TestQgsWcsPublicServers::Server TestQgsWcsPublicServers::getServer( const QStrin
 QList<TestQgsWcsPublicServers::Issue> TestQgsWcsPublicServers::issues( const QString & url, const QString & coverage, const QString &version )
 {
   QList<Issue> issues;
-  foreach ( Server server, mServers )
+  Q_FOREACH ( const Server& server, mServers )
   {
     if ( server.url == url )
     {
-      foreach ( Issue issue, server.issues )
+      Q_FOREACH ( const Issue& issue, server.issues )
       {
-        if (( issue.coverages.size() == 0 || issue.coverages.contains( coverage ) ) &&
-            ( issue.versions.size() == 0 || issue.versions.contains( version ) ) )
+        if (( issue.coverages.isEmpty() || issue.coverages.contains( coverage ) ) &&
+            ( issue.versions.isEmpty() || issue.versions.contains( version ) ) )
         {
           issues << issue;
         }
@@ -219,7 +219,7 @@ QList<TestQgsWcsPublicServers::Issue> TestQgsWcsPublicServers::issues( const QSt
 QStringList TestQgsWcsPublicServers::issueDescriptions( const QString & url, const QString & coverage, const QString &version )
 {
   QStringList descriptions;
-  foreach ( Issue myIssue, issues( url, coverage, version ) )
+  Q_FOREACH ( const Issue& myIssue, issues( url, coverage, version ) )
   {
     descriptions << myIssue.description;
   }
@@ -229,7 +229,7 @@ QStringList TestQgsWcsPublicServers::issueDescriptions( const QString & url, con
 int TestQgsWcsPublicServers::issueOffender( const QString & url, const QString & coverage, const QString &version )
 {
   int offender = NoOffender;
-  foreach ( Issue myIssue, issues( url, coverage, version ) )
+  Q_FOREACH ( const Issue& myIssue, issues( url, coverage, version ) )
   {
     if ( myIssue.offender == "server" )
     {
@@ -268,41 +268,41 @@ void TestQgsWcsPublicServers::test()
   }
   else
   {
-    foreach ( Server server, mServers )
+    Q_FOREACH ( const Server& server, mServers )
     {
       serverUrls << server.url;
     }
   }
 
-  foreach ( QString serverUrl, serverUrls )
+  Q_FOREACH ( const QString& serverUrl, serverUrls )
   {
     Server myServer = getServer( serverUrl );
     QStringList myServerLog;
     myServerLog << "server:" + serverUrl;
     QString myServerDirName = serverUrl;
     myServerDirName.replace( QRegExp( "[:/]+" ), "." );
-    myServerDirName.replace( QRegExp( "\\.$" ), "" );
+    myServerDirName.remove( QRegExp( "\\.$" ) );
     QgsDebugMsg( "myServerDirName = " + myServerDirName );
 
-    QDir myServerDir( mCacheDir.absolutePath() + QDir::separator() + myServerDirName );
+    QDir myServerDir( mCacheDir.absolutePath() + '/' + myServerDirName );
 
     if ( !myServerDir.exists() )
     {
       mCacheDir.mkdir( myServerDirName );
     }
 
-    QString myServerLogPath = myServerDir.absolutePath() + QDir::separator() + "server.log";
+    QString myServerLogPath = myServerDir.absolutePath() + "/server.log";
 
-    foreach ( QString version, versions )
+    Q_FOREACH ( const QString& version, versions )
     {
       QgsDebugMsg( "server: " + serverUrl + " version: " + version );
       QStringList myVersionLog;
       myVersionLog << "version:" + version;
 
       QString myVersionDirName = "v" + version;
-      QString myVersionDirPath = myServerDir.absolutePath() + QDir::separator() + myVersionDirName;
+      QString myVersionDirPath = myServerDir.absolutePath() + '/' + myVersionDirName;
 
-      QString myVersionLogPath = myVersionDirPath + QDir::separator() + "version.log";
+      QString myVersionLogPath = myVersionDirPath + "/version.log";
 
       QDir myVersionDir( myVersionDirPath );
       if ( !myVersionDir.exists() )
@@ -319,7 +319,7 @@ void TestQgsWcsPublicServers::test()
       }
       myServerUri.setParam( "cache", "AlwaysNetwork" );
 
-      foreach ( QString key, myServer.params.keys() )
+      Q_FOREACH ( const QString& key, myServer.params.keys() )
       {
         myServerUri.setParam( key, myServer.params.value( key ) );
       }
@@ -331,7 +331,7 @@ void TestQgsWcsPublicServers::test()
       if ( !myCapabilities.lastError().isEmpty() )
       {
         QgsDebugMsg( myCapabilities.lastError() );
-        myVersionLog << "error:" +  myCapabilities.lastError().replace( "\n", " " );
+        myVersionLog << "error:" +  myCapabilities.lastError().replace( '\n', ' ' );
         continue;
       }
 
@@ -351,7 +351,7 @@ void TestQgsWcsPublicServers::test()
       int myStep = myCoverages.size() / qMin( mMaxCoverages, myCoverages.size() );
       int myStepCount = -1;
       bool myCoverageFound = false;
-      foreach ( QgsWcsCoverageSummary myCoverage, myCoverages )
+      Q_FOREACH ( QgsWcsCoverageSummary myCoverage, myCoverages )
       {
         QgsDebugMsg( "coverage: " + myCoverage.identifier );
         if ( !mCoverage.isEmpty() && myCoverage.identifier != mCoverage ) continue;
@@ -372,7 +372,7 @@ void TestQgsWcsPublicServers::test()
         if ( myCoverageCount > mMaxCoverages ) break;
 
 
-        QString myPath = myVersionDirPath + QDir::separator() + myCoverage.identifier;
+        QString myPath = myVersionDirPath + '/' + myCoverage.identifier;
         QString myLogPath = myPath + ".log";
 
         if ( QFileInfo( myLogPath ).exists() && !mForce )
@@ -388,7 +388,7 @@ void TestQgsWcsPublicServers::test()
         myCoverage = myCapabilities.coverage( myCoverage.identifier ); // get described
         QgsDataSourceURI myUri = myServerUri;
         myUri.setParam( "identifier", myCoverage.identifier );
-        if ( myCoverage.times.size() > 0 )
+        if ( !myCoverage.times.isEmpty() )
         {
           myUri.setParam( "time", myCoverage.times.value( 0 ) );
         }
@@ -410,7 +410,7 @@ void TestQgsWcsPublicServers::test()
         QStringList providers;
         providers << "wcs" << "gdal";
 
-        foreach ( QString provider, providers )
+        Q_FOREACH ( const QString& provider, providers )
         {
           QTime time;
           time.start();
@@ -426,7 +426,7 @@ void TestQgsWcsPublicServers::test()
             Q_ASSERT( myGdalXmlFile.open( QIODevice::WriteOnly | QIODevice::Text ) );
             QTextStream myStream( &myGdalXmlFile );
             myStream << "<WCS_GDAL>\n";
-            myStream << "  <ServiceURL>" + serverUrl + "?" + "</ServiceURL>\n";
+            myStream << "  <ServiceURL>" + serverUrl + '?' + "</ServiceURL>\n";
             myStream << "  <CoverageName>" + myCoverage.identifier + "</CoverageName>\n";
             myStream << "  <Version>" + version + "</Version>\n";
             myStream << QString( "  <Timeout>%1</Timeout>\n" ).arg( mTimeout / 1000., 0, 'd' );
@@ -443,10 +443,10 @@ void TestQgsWcsPublicServers::test()
             myLog << provider + "_height:" + QString::number( myLayer->dataProvider()->ySize() );
             QgsRectangle extent = myLayer->dataProvider()->extent();
             myLog << provider + "_extent:"
-            + QgsRasterBlock::printValue( extent.xMinimum() ) + ","
-            + QgsRasterBlock::printValue( extent.yMinimum() ) + ","
-            + QgsRasterBlock::printValue( extent.xMaximum() ) + ","
-            + QgsRasterBlock::printValue( extent.yMaximum() ) + ",";
+            + QgsRasterBlock::printValue( extent.xMinimum() ) + ','
+            + QgsRasterBlock::printValue( extent.yMinimum() ) + ','
+            + QgsRasterBlock::printValue( extent.xMaximum() ) + ','
+            + QgsRasterBlock::printValue( extent.yMaximum() ) + ',';
             int myBandCount = myLayer->dataProvider()->bandCount();
             myLog << provider + "_bandCount:" + QString::number( myBandCount );
             if ( myBandCount > 0 )
@@ -553,9 +553,9 @@ void TestQgsWcsPublicServers::test()
   }
 }
 
-void TestQgsWcsPublicServers::writeReport( QString theReport )
+void TestQgsWcsPublicServers::writeReport( const QString& theReport )
 {
-  QString myReportFile = mCacheDir.absolutePath() + QDir::separator() + "index.html";
+  QString myReportFile = mCacheDir.absolutePath() + "/index.html";
   QFile myFile( myReportFile );
   if ( myFile.open( QIODevice::WriteOnly ) )
   {
@@ -577,12 +577,12 @@ void TestQgsWcsPublicServers::report()
   int myCoverageErrCount = 0;
   int myCoverageWarnCount = 0;
 
-  foreach ( QString myServerDirName, mCacheDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) )
+  Q_FOREACH ( const QString& myServerDirName, mCacheDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) )
   {
     myServerCount++;
-    QDir myServerDir( mCacheDir.absolutePath() + QDir::separator() + myServerDirName );
+    QDir myServerDir( mCacheDir.absolutePath() + '/' + myServerDirName );
 
-    QString myServerLogPath = myServerDir.absolutePath() + QDir::separator() + "server.log";
+    QString myServerLogPath = myServerDir.absolutePath() + "/server.log";
     QMap<QString, QString> myServerLog = readLog( myServerLogPath );
 
     myReport += QString( "<h2>Server: %1</h2>" ).arg( myServerLog.value( "server" ) );
@@ -591,12 +591,12 @@ void TestQgsWcsPublicServers::report()
     {
       myReport += myServer.description + "<br>\n";
     }
-    if ( myServer.params.size() > 0 )
+    if ( !myServer.params.isEmpty() )
     {
       myReport += "<br>Additional params: ";
-      foreach ( QString key, myServer.params.keys() )
+      Q_FOREACH ( const QString& key, myServer.params.keys() )
       {
-        myReport += key + "=" + myServer.params.value( key ) + " ";
+        myReport += key + '=' + myServer.params.value( key ) + " ";
       }
       myReport += "<br>\n";
     }
@@ -605,25 +605,25 @@ void TestQgsWcsPublicServers::report()
 
     bool myServerErr = false;
     bool myServerWarn = false;
-    foreach ( QString myVersionDirName, myServerDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) )
+    Q_FOREACH ( const QString& myVersionDirName, myServerDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) )
     {
       QString myVersionReport;
       int myVersionCoverageCount = 0;
       int myVersionErrCount = 0;
       int myVersionWarnCount = 0;
 
-      QString myVersionDirPath = myServerDir.absolutePath() + QDir::separator() + myVersionDirName;
-      QString myVersionLogPath = myVersionDirPath + QDir::separator() + "version.log";
+      QString myVersionDirPath = myServerDir.absolutePath() + '/' + myVersionDirName;
+      QString myVersionLogPath = myVersionDirPath + "/version.log";
       QMap<QString, QString> myVersionLog = readLog( myVersionLogPath );
       QDir myVersionDir( myVersionDirPath );
 
       QString myVersion = myVersionLog.value( "version" );
-      myServerReport += QString( "<h3><a href='%1'>Version: %2</a></h3>" ).arg( myVersionLog.value( "getCapabilitiesUrl" ) ).arg( myVersion.isEmpty() ? "(empty)" : myVersion );
+      myServerReport += QString( "<h3><a href='%1'>Version: %2</a></h3>" ).arg( myVersionLog.value( "getCapabilitiesUrl" ), myVersion.isEmpty() ? "(empty)" : myVersion );
 
       if ( !myVersionLog.value( "error" ).isEmpty() )
       {
         // Server may have more errors, for each version
-        //foreach ( QString err, myServerLog.values( "error" ) )
+        //Q_FOREACH ( QString err, myServerLog.values( "error" ) )
         //{
         //myVersionReport += error( err );
         //}
@@ -637,18 +637,18 @@ void TestQgsWcsPublicServers::report()
         QStringList filters;
         filters << "*.log";
         myVersionDir.setNameFilters( filters );
-        foreach ( QString myLogFileName, myVersionDir.entryList( QDir::Files ) )
+        Q_FOREACH ( const QString& myLogFileName, myVersionDir.entryList( QDir::Files ) )
         {
           if ( myLogFileName == "version.log" ) continue;
           myVersionCoverageCount++;
           myCoverageCount++;
 
-          QString myLogPath = myVersionDir.absolutePath() + QDir::separator() + myLogFileName;
+          QString myLogPath = myVersionDir.absolutePath() + '/' + myLogFileName;
           QMap<QString, QString>myLog = readLog( myLogPath );
           myVersionReport += "<tr>";
 
           QStringList myValues;
-          myValues << QString( "<a href='%1'>%2</a>" ).arg( myLog.value( "describeCoverageUrl" ) ).arg( myLog.value( "identifier" ) );
+          myValues << QString( "<a href='%1'>%2</a>" ).arg( myLog.value( "describeCoverageUrl" ), myLog.value( "identifier" ) );
           //myValues << myLog.value( "hasSize" );
           myVersionReport += cells( myValues, "", 1, 2 );
           myValues.clear();
@@ -660,9 +660,9 @@ void TestQgsWcsPublicServers::report()
           providers << "wcs" << "gdal";
 
           bool hasErr = false;
-          foreach ( QString provider, providers )
+          Q_FOREACH ( const QString& provider, providers )
           {
-            QString imgPath = myVersionDir.absolutePath() + QDir::separator() + QFileInfo( myLogPath ).completeBaseName() + "-" + provider + ".png";
+            QString imgPath = myVersionDir.absolutePath() + '/' + QFileInfo( myLogPath ).completeBaseName() + "-" + provider + ".png";
 
 
             if ( !myLog.value( provider + "_error" ).isEmpty() )
@@ -671,7 +671,7 @@ void TestQgsWcsPublicServers::report()
               int offender = NoOffender;
               if ( provider == "wcs" )
               {
-                myValues << issuesString;;
+                myValues << issuesString;
 
                 offender = issueOffender( myServerLog.value( "server" ), myLog.value( "identifier" ), myLog.value( "version" ) );
                 myVersionErrCount++;
@@ -698,7 +698,7 @@ void TestQgsWcsPublicServers::report()
               myValues << myLog.value( provider + "_crs" );
               myValues << myLog.value( provider + "_width" );
               myValues << myLog.value( provider + "_height" );
-              myValues << QString( myLog.value( provider + "_extent" ) ).replace( ",", "<br>" );
+              myValues << QString( myLog.value( provider + "_extent" ) ).replace( ',', "<br>" );
               myValues << "<img src='" + imgPath + "'>";
               myValues << myLog.value( provider + "_bandCount" );
               myValues << myLog.value( provider + "_srcType" );
@@ -765,7 +765,7 @@ void TestQgsWcsPublicServers::report()
   } // servers
 
   QString mySettings = QgsApplication::showSettings();
-  mySettings = mySettings.replace( "\n", "<br />" );
+  mySettings = mySettings.replace( '\n', "<br />" );
   QString myRep = "<h1>WCS public servers test</h1>\n";
   myRep += "<p>" + mySettings + "</p>";
 
@@ -793,7 +793,7 @@ void TestQgsWcsPublicServers::report()
   writeReport( myRep );
 }
 
-QMap<QString, QString> TestQgsWcsPublicServers::readLog( QString theFileName )
+QMap<QString, QString> TestQgsWcsPublicServers::readLog( const QString& theFileName )
 {
   QMap<QString, QString> myMap;
 
@@ -801,9 +801,9 @@ QMap<QString, QString> TestQgsWcsPublicServers::readLog( QString theFileName )
   if ( myFile.open( QIODevice::ReadOnly ) )
   {
     QTextStream myStream( &myFile );
-    foreach ( QString row, myStream.readAll().split( "\n" ) )
+    Q_FOREACH ( const QString& row, myStream.readAll().split( '\n' ) )
     {
-      int sepIdx = row.indexOf( ":" );
+      int sepIdx = row.indexOf( ':' );
       myMap.insert( row.left( sepIdx ), row.mid( sepIdx + 1 ) );
     }
     myFile.close();
@@ -811,7 +811,7 @@ QMap<QString, QString> TestQgsWcsPublicServers::readLog( QString theFileName )
   return myMap;
 }
 
-QString TestQgsWcsPublicServers::error( QString theMessage )
+QString TestQgsWcsPublicServers::error( const QString& theMessage )
 {
   QString myRow = "<font class='errmsg'>Error: ";
   myRow += theMessage;
@@ -819,7 +819,7 @@ QString TestQgsWcsPublicServers::error( QString theMessage )
   return myRow;
 }
 
-QString TestQgsWcsPublicServers::cells( QStringList theValues, QString theClass, int colspan, int rowspan )
+QString TestQgsWcsPublicServers::cells( const QStringList& theValues, const QString& theClass, int colspan, int rowspan )
 {
   QString myRow;
   for ( int i = 0; i < theValues.size(); i++ )
@@ -834,12 +834,12 @@ QString TestQgsWcsPublicServers::cells( QStringList theValues, QString theClass,
     {
       rowspanStr = QString( "rowspan=%1" ).arg( rowspan );
     }
-    myRow += QString( "<td class='cell %1' %2 %3>%4</td>" ).arg( theClass ).arg( colspanStr ).arg( rowspanStr ).arg( val );
+    myRow += QString( "<td class='cell %1' %2 %3>%4</td>" ).arg( theClass, colspanStr, rowspanStr, val );
   }
   return myRow;
 }
 
-QString TestQgsWcsPublicServers::row( QStringList theValues, QString theClass )
+QString TestQgsWcsPublicServers::row( const QStringList& theValues, const QString& theClass )
 {
   QString myRow = "<tr>";
   for ( int i = 0; i < theValues.size(); i++ )
@@ -850,7 +850,7 @@ QString TestQgsWcsPublicServers::row( QStringList theValues, QString theClass )
     {
       colspan = QString( "colspan=%1" ).arg( mHead.size() - theValues.size() + 1 );
     }
-    myRow += QString( "<td class='cell %1' %2>%3</td>" ).arg( theClass ).arg( colspan ).arg( val );
+    myRow += QString( "<td class='cell %1' %2>%3</td>" ).arg( theClass, colspan, val );
   }
   myRow += "</tr>\n";
   return myRow;
@@ -875,13 +875,13 @@ void usage( std::string const & appName )
 
 int main( int argc, char *argv[] )
 {
-#ifdef WIN32  // Windows
+#ifdef Q_OS_WIN // Windows
 #ifdef _MSC_VER
   _set_fmode( _O_BINARY );
 #else //MinGW
   _fmode = _O_BINARY;
 #endif  // _MSC_VER
-#endif  // WIN32
+#endif  // Q_OS_WIN
 
   QString myServer;
   QString myCoverage;
@@ -889,7 +889,7 @@ int main( int argc, char *argv[] )
   int myMaxCoverages = 2;
   bool myForce = false;
 
-#ifndef WIN32
+#ifndef Q_OS_WIN
   int optionChar;
   static struct option long_options[] =
   {
@@ -949,7 +949,6 @@ int main( int argc, char *argv[] )
       case 'h':
         usage( argv[0] );
         return 2;   // XXX need standard exit codes
-        break;
 
       default:
         QgsDebugMsg( QString( "%1: getopt returned character code %2" ).arg( argv[0] ).arg( optionChar ) );
