@@ -33,7 +33,7 @@ from utilities import (
 try:
     # noinspection PyUnresolvedReferences
     from qgis.core import QgsRectangle, QgsCoordinateReferenceSystem
-except ImportError, e:
+except ImportError as e:
     raise ImportError(str(e) + '\n\nPlace path to pyqgis modules on sys.path,'
                                ' or assign to PYTHONPATH')
 
@@ -357,7 +357,7 @@ class QgisLocalServer(object):
                     shutil.copy2(path, self._web_dir)
                 elif os.path.isdir(path):
                     shutil.copytree(path, self._web_dir)
-            except Exception, err:
+            except Exception as err:
                 raise ServerProcessError('Failed to copy to web directory:',
                                          item,
                                          str(err))
@@ -370,7 +370,7 @@ class QgisLocalServer(object):
                     os.unlink(path)
                 else:
                     shutil.rmtree(path)
-            except Exception, err:
+            except Exception as err:
                 raise ServerProcessError('Failed to clear web directory', err)
 
     def temp_dir(self):
@@ -443,7 +443,7 @@ class QgisLocalServer(object):
         params = self._params_to_upper(params)
         try:
             proj = params['MAP']
-        except KeyError, err:
+        except KeyError as err:
             raise KeyError(str(err) + '\nMAP not found in parameters dict')
 
         if not os.path.exists(proj):
@@ -564,10 +564,10 @@ class QgisLocalServer(object):
         self._web_dir = os.path.join(self._temp_dir, 'www', 'htdocs')
         cgi_bin = os.path.join(self._temp_dir, 'cgi-bin')
 
-        os.makedirs(cgi_bin, mode=0755)
-        os.makedirs(os.path.join(self._temp_dir, 'log'), mode=0755)
-        os.makedirs(os.path.join(self._temp_dir, 'var', 'run'), mode=0755)
-        os.makedirs(self._web_dir, mode=0755)
+        os.makedirs(cgi_bin, mode=0o755)
+        os.makedirs(os.path.join(self._temp_dir, 'log'), mode=0o755)
+        os.makedirs(os.path.join(self._temp_dir, 'var', 'run'), mode=0o755)
+        os.makedirs(self._web_dir, mode=0o755)
 
         # symlink or copy in components
         shutil.copy2(os.path.join(self._conf_dir, 'index.html'), self._web_dir)
@@ -756,7 +756,7 @@ def getLocalServer():
             assert not os.path.exists(srv.temp_dir()), msg
 
             MAPSERV = srv
-        except AssertionError, err:
+        except AssertionError as err:
             srv.shutdown()
             raise AssertionError(err)
 
