@@ -35,23 +35,29 @@ from qgis.core import (QgsVectorLayer,
                        QgsMultiRenderChecker,
                        QgsRasterLayer,
                        QgsMultiBandColorRenderer,
+                       QgsRectangle
                        )
 
-from utilities import (unitTestDataPath,
-                       getQgisTestApp,
-                       TestCase,
-                       unittest
-                       )
+from qgis.testing import (start_app,
+                          unittest
+                          )
+
+from qgis.testing.mocked import get_iface
+
+from utilities import unitTestDataPath
 # Convenience instances in case you may need them
-QGISAPP, CANVAS, IFACE, PARENT = getQgisTestApp()
+start_app()
 TEST_DATA_DIR = unitTestDataPath()
 
-class TestQgsBlendModes(TestCase):
+
+class TestQgsBlendModes(unittest.TestCase):
 
     def __init__(self, methodName):
-        """Run once on class initialisation."""
+        """Run once on class initialization."""
         unittest.TestCase.__init__(self, methodName)
 
+        self.iface = get_iface()
+
         # initialize class MapRegistry, Canvas, MapRenderer, Map and PAL
         self.mMapRegistry = QgsMapLayerRegistry.instance()
 
@@ -76,93 +82,96 @@ class TestQgsBlendModes(TestCase):
         self.mMapRegistry.addMapLayer(self.mLineLayer)
 
         # create two raster layers
-        myRasterFile = os.path.join(TEST_DATA_DIR, 'landsat.tif')
+        myRasterFile = os.path.join(TEST_DATA_DIR, 'rgb256x256.png')
         self.mRasterLayer1 = QgsRasterLayer(myRasterFile, "raster1")
         self.mRasterLayer2 = QgsRasterLayer(myRasterFile, "raster2")
-        myMultiBandRenderer1 = QgsMultiBandColorRenderer(self.mRasterLayer1.dataProvider(), 2, 3, 4)
+        myMultiBandRenderer1 = QgsMultiBandColorRenderer(self.mRasterLayer1.dataProvider(), 1, 2, 3)
         self.mRasterLayer1.setRenderer(myMultiBandRenderer1)
         self.mMapRegistry.addMapLayer(self.mRasterLayer1)
-        myMultiBandRenderer2 = QgsMultiBandColorRenderer(self.mRasterLayer2.dataProvider(), 2, 3, 4)
+        myMultiBandRenderer2 = QgsMultiBandColorRenderer(self.mRasterLayer2.dataProvider(), 1, 2, 3)
         self.mRasterLayer2.setRenderer(myMultiBandRenderer2)
         self.mMapRegistry.addMapLayer(self.mRasterLayer2)
 
         # to match blend modes test comparisons background
-        self.mCanvas = CANVAS
+        self.mCanvas = self.iface.mapCanvas()
         self.mCanvas.setCanvasColor(QColor(152, 219, 249))
         self.mMap = self.mCanvas.map()
         self.mMap.resize(QSize(400, 400))
         self.mapSettings = self.mCanvas.mapSettings()
         self.mapSettings.setOutputSize(QSize(400, 400))
+        self.mapSettings.setOutputDpi(96)
+
+        self.extent = QgsRectangle(-118.8888888888887720, 22.8002070393376783, -83.3333333333331581, 46.8719806763287536)
 
     def testVectorBlending(self):
         """Test that blend modes work for vector layers."""
 
-        #Add vector layers to map
+        # Add vector layers to map
         myLayers = []
         myLayers.append(self.mLineLayer.id())
         myLayers.append(self.mPolygonLayer.id())
         self.mapSettings.setLayers(myLayers)
-        self.mapSettings.setExtent(self.mPointLayer.extent())
+        self.mapSettings.setExtent(self.extent)
 
-        #Set blending modes for both layers
+        # Set blending modes for both layers
         self.mLineLayer.setBlendMode(QPainter.CompositionMode_Difference)
         self.mPolygonLayer.setBlendMode(QPainter.CompositionMode_Difference)
 
         checker = QgsMultiRenderChecker()
         checker.setControlName("expected_vector_blendmodes")
         checker.setMapSettings(self.mapSettings)
-        checker.setColorTolerance( 1 )
+        checker.setColorTolerance(1)
 
         myResult = checker.runTest("vector_blendmodes", 20)
         myMessage = ('vector blending failed')
         assert myResult, myMessage
 
-        #Reset layers
+        # Reset layers
         self.mLineLayer.setBlendMode(QPainter.CompositionMode_SourceOver)
         self.mPolygonLayer.setBlendMode(QPainter.CompositionMode_SourceOver)
 
     def testVectorFeatureBlending(self):
         """Test that feature blend modes work for vector layers."""
 
-        #Add vector layers to map
+        # Add vector layers to map
         myLayers = []
         myLayers.append(self.mLineLayer.id())
         myLayers.append(self.mPolygonLayer.id())
         self.mapSettings.setLayers(myLayers)
-        self.mapSettings.setExtent(self.mPointLayer.extent())
+        self.mapSettings.setExtent(self.extent)
 
-        #Set feature blending for line layer
+        # Set feature blending for line layer
         self.mLineLayer.setFeatureBlendMode(QPainter.CompositionMode_Plus)
 
         checker = QgsMultiRenderChecker()
         checker.setControlName("expected_vector_featureblendmodes")
         checker.setMapSettings(self.mapSettings)
-        checker.setColorTolerance( 1 )
+        checker.setColorTolerance(1)
 
         myResult = checker.runTest("vector_featureblendmodes", 20)
         myMessage = ('vector feature blending failed')
         assert myResult, myMessage
 
-        #Reset layers
+        # Reset layers
         self.mLineLayer.setFeatureBlendMode(QPainter.CompositionMode_SourceOver)
 
     def testVectorLayerTransparency(self):
         """Test that layer transparency works for vector layers."""
 
-        #Add vector layers to map
+        # Add vector layers to map
         myLayers = []
         myLayers.append(self.mLineLayer.id())
         myLayers.append(self.mPolygonLayer.id())
         self.mapSettings.setLayers(myLayers)
-        self.mapSettings.setExtent(self.mPointLayer.extent())
+        self.mapSettings.setExtent(self.extent)
 
-        #Set feature blending for line layer
-        self.mLineLayer.setLayerTransparency( 50 )
+        # Set feature blending for line layer
+        self.mLineLayer.setLayerTransparency(50)
 
         checker = QgsMultiRenderChecker()
         checker.setControlName("expected_vector_layertransparency")
         checker.setMapSettings(self.mapSettings)
-        checker.setColorTolerance( 1 )
+        checker.setColorTolerance(1)
 
         myResult = checker.runTest("vector_layertransparency", 20)
         myMessage = ('vector layer transparency failed')
@@ -170,20 +179,20 @@ class TestQgsBlendModes(TestCase):
 
     def testRasterBlending(self):
         """Test that blend modes work for raster layers."""
-        #Add raster layers to map
+        # Add raster layers to map
         myLayers = []
         myLayers.append(self.mRasterLayer1.id())
         myLayers.append(self.mRasterLayer2.id())
         self.mapSettings.setLayers(myLayers)
         self.mapSettings.setExtent(self.mRasterLayer1.extent())
 
-        #Set blending mode for top layer
-        self.mRasterLayer1.setBlendMode(QPainter.CompositionMode_Plus)
+        # Set blending mode for top layer
+        self.mRasterLayer1.setBlendMode(QPainter.CompositionMode_Difference)
         checker = QgsMultiRenderChecker()
         checker.setControlName("expected_raster_blendmodes")
         checker.setMapSettings(self.mapSettings)
-        checker.setColorTolerance( 1 )
-        checker.setColorTolerance( 1 )
+        checker.setColorTolerance(1)
+        checker.setColorTolerance(1)
 
         myResult = checker.runTest("raster_blendmodes", 20)
         myMessage = ('raster blending failed')
