@@ -13,7 +13,6 @@ __copyright__ = 'Copyright 2012, The QGIS Project'
 __revision__ = '$Format:%H$'
 
 import qgis
-import unittest
 import os
 
 from PyQt4.QtCore import QUrl, qDebug
@@ -22,22 +21,30 @@ from qgis.core import (QgsComposition,
                        QgsComposerHtml,
                        QgsComposerFrame,
                        QgsComposerMultiFrame,
-                       QgsMapSettings)
+                       QgsMapSettings
+                       )
 
 from qgscompositionchecker import QgsCompositionChecker
 
-from utilities import (unitTestDataPath,
-                       getQgisTestApp,
-                       TestCase
-                       )
-QGISAPP, CANVAS, IFACE, PARENT = getQgisTestApp()
+from qgis.testing import (
+    start_app,
+    unittest
+)
+
+from qgis.testing.mocked import get_iface
+
+from utilities import unitTestDataPath
+
+start_app()
+
 TEST_DATA_DIR = unitTestDataPath()
 
 
-class TestQgsComposerHtml(TestCase):
+class TestQgsComposerHtml(unittest.TestCase):
 
     def setUp(self):
         """Run before each test."""
+        self.iface = get_iface()
         self.mapSettings = QgsMapSettings()
         self.mComposition = QgsComposition(self.mapSettings)
         self.mComposition.setPaperSize(297, 210)  # A4 landscape
@@ -62,10 +69,11 @@ class TestQgsComposerHtml(TestCase):
         composerHtml.setUrl(self.htmlUrl())
 
         checker = QgsCompositionChecker('composerhtml_table', self.mComposition)
+        checker.setControlPathPrefix("composer_html")
         myTestResult, myMessage = checker.testComposition()
 
         qDebug(myMessage)
-        self.mComposition.removeMultiFrame( composerHtml )
+        self.mComposition.removeMultiFrame(composerHtml)
         composerHtml = None
 
         assert myTestResult, myMessage
@@ -78,23 +86,25 @@ class TestQgsComposerHtml(TestCase):
         composerHtml.addFrame(htmlFrame)
         composerHtml.setResizeMode(
             QgsComposerMultiFrame.RepeatUntilFinished)
-        composerHtml.setUseSmartBreaks( False )
+        composerHtml.setUseSmartBreaks(False)
         composerHtml.setUrl(self.htmlUrl())
         composerHtml.frame(0).setFrameEnabled(True)
 
         print "Checking page 1"
         myPage = 0
         checker1 = QgsCompositionChecker('composerhtml_multiframe1', self.mComposition)
-        myTestResult, myMessage = checker1.testComposition( myPage )
+        checker1.setControlPathPrefix("composer_html")
+        myTestResult, myMessage = checker1.testComposition(myPage)
         assert myTestResult, myMessage
 
         print "Checking page 2"
         myPage = 1
         checker2 = QgsCompositionChecker('composerhtml_multiframe2', self.mComposition)
-        myTestResult, myMessage = checker2.testComposition( myPage )
+        checker2.setControlPathPrefix("composer_html")
+        myTestResult, myMessage = checker2.testComposition(myPage)
         assert myTestResult, myMessage
 
-        self.mComposition.removeMultiFrame( composerHtml )
+        self.mComposition.removeMultiFrame(composerHtml)
         composerHtml = None
 
         assert myTestResult, myMessage
@@ -107,23 +117,25 @@ class TestQgsComposerHtml(TestCase):
         composerHtml.addFrame(htmlFrame)
         composerHtml.setResizeMode(
             QgsComposerMultiFrame.RepeatUntilFinished)
-        composerHtml.setUseSmartBreaks( True )
+        composerHtml.setUseSmartBreaks(True)
         composerHtml.setUrl(self.htmlUrl())
         composerHtml.frame(0).setFrameEnabled(True)
 
         print "Checking page 1"
         myPage = 0
         checker1 = QgsCompositionChecker('composerhtml_smartbreaks1', self.mComposition)
-        myTestResult, myMessage = checker1.testComposition( myPage, 200 )
+        checker1.setControlPathPrefix("composer_html")
+        myTestResult, myMessage = checker1.testComposition(myPage, 200)
         assert myTestResult, myMessage
 
         print "Checking page 2"
         myPage = 1
         checker2 = QgsCompositionChecker('composerhtml_smartbreaks2', self.mComposition)
-        myTestResult, myMessage = checker2.testComposition( myPage, 200 )
+        checker2.setControlPathPrefix("composer_html")
+        myTestResult, myMessage = checker2.testComposition(myPage, 200)
         assert myTestResult, myMessage
 
-        self.mComposition.removeMultiFrame( composerHtml )
+        self.mComposition.removeMultiFrame(composerHtml)
         composerHtml = None
 
         assert myTestResult, myMessage
@@ -131,7 +143,7 @@ class TestQgsComposerHtml(TestCase):
     def testComposerHtmlAccessor(self):
         """Test that we can retrieve the ComposerHtml instance given an item.
         """
-        myComposition = QgsComposition(CANVAS.mapRenderer())
+        myComposition = QgsComposition(self.iface.mapCanvas().mapRenderer())
         mySubstitutionMap = {'replace-me': 'Foo bar'}
         myFile = os.path.join(TEST_DATA_DIR, 'template.qpt')
         myTemplateFile = file(myFile, 'rt')
