@@ -17,27 +17,27 @@ import os
 
 from qgis.core import QgsPoint, QgsVectorLayer
 
-from utilities import (getQgisTestApp,
-                       TestCase,
-                       unittest
-                       )
+from qgis.testing import (start_app,
+                          unittest
+                          )
 
 from pyspatialite import dbapi2 as sqlite3
 
 # Convenience instances in case you may need them
-QGISAPP, CANVAS, IFACE, PARENT = getQgisTestApp()
+start_app()
 
 
 def die(error_message):
     raise Exception(error_message)
 
-class TestQgsSpatialiteProvider(TestCase):
+
+class TestQgsSpatialiteProvider(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
         """Run before all tests"""
         # create test db
-        if os.path.exists("test.sqlite") :
+        if os.path.exists("test.sqlite"):
             os.remove("test.sqlite")
         con = sqlite3.connect("test.sqlite", isolation_level=None)
         cur = con.cursor()
@@ -51,15 +51,15 @@ class TestQgsSpatialiteProvider(TestCase):
         sql = "SELECT AddGeometryColumn('test_mpg', 'geometry', 4326, 'MULTIPOLYGON', 'XY')"
         cur.execute(sql)
         sql = "INSERT INTO test_mpg (name, geometry) "
-        sql +=    "VALUES ('multipolygon with 8 squares', GeomFromText('MULTIPOLYGON("
-        for i in range(0,4,2):
-            for j in range (0,4,2):
+        sql += "VALUES ('multipolygon with 8 squares', GeomFromText('MULTIPOLYGON("
+        for i in range(0, 4, 2):
+            for j in range(0, 4, 2):
                 sql += "(("
-                sql += str(i)   + " " + str(j)   + ","
-                sql += str(i+1) + " " + str(j)   + ","
-                sql += str(i+1) + " " + str(j+1) + ","
-                sql += str(i)   + " " + str(j+1) + ","
-                sql += str(i)   + " " + str(j)
+                sql += str(i) + " " + str(j) + ","
+                sql += str(i + 1) + " " + str(j) + ","
+                sql += str(i + 1) + " " + str(j + 1) + ","
+                sql += str(i) + " " + str(j + 1) + ","
+                sql += str(i) + " " + str(j)
                 sql += ")),"
         sql = sql[:-1] # remove last comma
         sql += ")', 4326))"
@@ -70,7 +70,7 @@ class TestQgsSpatialiteProvider(TestCase):
         sql = "SELECT AddGeometryColumn('test_pg', 'geometry', 4326, 'POLYGON', 'XY')"
         cur.execute(sql)
         sql = "INSERT INTO test_pg (name, geometry) "
-        sql +=    "VALUES ('polygon with interior ring', GeomFromText('POLYGON((0 0,3 0,3 3,0 3,0 0),(1 1,1 2,2 2,2 1,1 1))', 4326))"
+        sql += "VALUES ('polygon with interior ring', GeomFromText('POLYGON((0 0,3 0,3 3,0 3,0 0),(1 1,1 2,2 2,2 1,1 1))', 4326))"
         cur.execute(sql)
         cur.execute("COMMIT")
         con.close()
@@ -79,7 +79,7 @@ class TestQgsSpatialiteProvider(TestCase):
     def tearDownClass(cls):
         """Run after all tests"""
         # for the time beeing, keep the file to check with qgis
-        #if os.path.exists("test.sqlite") :
+        # if os.path.exists("test.sqlite") :
         #    os.remove("test.sqlite")
         pass
 
@@ -98,8 +98,8 @@ class TestQgsSpatialiteProvider(TestCase):
         assert(layer.hasGeometryType())
         layer.featureCount() == 1 or die("wrong number of features")
         layer.startEditing()
-        layer.splitFeatures([QgsPoint(0.5, -0.5), QgsPoint(0.5, 1.5)], 0)==0 or die("error in split of one polygon of multipolygon")
-        layer.splitFeatures([QgsPoint(2.5, -0.5), QgsPoint(2.5, 4)], 0)==0 or die("error in split of two polygons of multipolygon at a time")
+        layer.splitFeatures([QgsPoint(0.5, -0.5), QgsPoint(0.5, 1.5)], 0) == 0 or die("error in split of one polygon of multipolygon")
+        layer.splitFeatures([QgsPoint(2.5, -0.5), QgsPoint(2.5, 4)], 0) == 0 or die("error in split of two polygons of multipolygon at a time")
         layer.commitChanges() or die("this commit should work")
         layer.featureCount() == 7 or die("wrong number of features after 2 split")
 
@@ -110,7 +110,7 @@ class TestQgsSpatialiteProvider(TestCase):
         assert(layer.hasGeometryType())
         layer.featureCount() == 1 or die("wrong number of features")
         layer.startEditing()
-        layer.splitFeatures([QgsPoint(1.5, -0.5), QgsPoint(1.5, 1.5)], 0)==0 or die("error when trying to create an invalid polygon in split")
+        layer.splitFeatures([QgsPoint(1.5, -0.5), QgsPoint(1.5, 1.5)], 0) == 0 or die("error when trying to create an invalid polygon in split")
         layer.commitChanges() or die("this commit should work")
         layer.featureCount() == 1 or die("wrong number of features, polygon should be unafected by cut")
 
