@@ -34,6 +34,7 @@ from qgis_local_server import getLocalServer
 from test_qgspallabeling_base import TestQgsPalLabeling, runSuite
 from test_qgspallabeling_tests import (
     TestPointBase,
+    TestLineBase,
     suiteTests
 )
 
@@ -55,10 +56,11 @@ class TestServerBase(TestQgsPalLabeling):
             TestQgsPalLabeling.setUpClass()
         MAPSERV.startup()
         MAPSERV.web_dir_install(glob.glob(cls._PalDataDir + os.sep + '*.qml'))
+        MAPSERV.web_dir_install(glob.glob(cls._PalDataDir + os.sep + '*.qgs'))
 
         # noinspection PyArgumentList
         cls._TestProj = QgsProject.instance()
-        cls._TestProjName = 'pal_test.qgs'
+        cls._TestProjName = 'test-labeling.qgs'
         cls._TestProj.setFileName(
             os.path.join(MAPSERV.web_dir(), cls._TestProjName))
 
@@ -212,6 +214,28 @@ class TestServerVsCanvasPoint(TestServerBasePoint, TestPointBase):
         self.configTest('pal_canvas', 'sp')
 
 
+class TestServerBaseLine(TestServerBase):
+
+    @classmethod
+    def setUpClass(cls):
+        TestServerBase.setUpClass()
+        cls.layer = TestQgsPalLabeling.loadFeatureLayer('line')
+
+
+class TestServerLine(TestServerBaseLine, TestLineBase):
+
+    def setUp(self):
+        """Run before each test."""
+        super(TestServerLine, self).setUp()
+        self.configTest('pal_server_line', 'sp')
+
+
+class TestServerVsCanvasLine(TestServerBaseLine, TestLineBase):
+
+    def setUp(self):
+        super(TestServerVsCanvasLine, self).setUp()
+        self.configTest('pal_canvas_line', 'sp')
+
 if __name__ == '__main__':
     # NOTE: unless PAL_SUITE env var is set all test class methods will be run
     # SEE: test_qgspallabeling_tests.suiteTests() to define suite
