@@ -12,16 +12,14 @@ __copyright__ = 'Copyright 2012, The QGIS Project'
 # This will get replaced with a git SHA1 when you do a git archive
 __revision__ = '$Format:%H$'
 
-import os
-import unittest
 import qgis
+import os
 
 from PyQt4.QtCore import QFileInfo, QObject, SIGNAL
 from PyQt4 import QtGui
 
 from qgis.core import (QgsRaster,
                        QgsRasterLayer,
-                       QgsRasterDataProvider,
                        QgsColorRampShader,
                        QgsContrastEnhancement,
                        QgsMapLayerRegistry,
@@ -32,17 +30,17 @@ from qgis.core import (QgsRaster,
                        QgsRenderChecker,
                        QgsSingleBandGrayRenderer,
                        QgsSingleBandPseudoColorRenderer)
-from utilities import (unitTestDataPath,
-                       getQgisTestApp,
-                       TestCase,
-                       unittest)
-                       #expectedFailure)
+from utilities import unitTestDataPath
+from qgis.testing import (start_app,
+                          unittest
+                          )
+
 # Convenience instances in case you may need them
 # not used in this test
-QGISAPP, CANVAS, IFACE, PARENT = getQgisTestApp()
+start_app()
 
 
-class TestQgsRasterLayer(TestCase):
+class TestQgsRasterLayer(unittest.TestCase):
 
     def testIdentify(self):
         myPath = os.path.join(unitTestDataPath(), 'landsat.tif')
@@ -52,16 +50,16 @@ class TestQgsRasterLayer(TestCase):
         myMessage = 'Raster not loaded: %s' % myPath
         assert myRasterLayer.isValid(), myMessage
         myPoint = QgsPoint(786690, 3345803)
-        #print 'Extents: %s' % myRasterLayer.extent().toString()
+        # print 'Extents: %s' % myRasterLayer.extent().toString()
         #myResult, myRasterValues = myRasterLayer.identify(myPoint)
         #assert myResult
-        myRasterValues =  myRasterLayer.dataProvider().identify(myPoint, QgsRaster.IdentifyFormatValue ).results()
+        myRasterValues = myRasterLayer.dataProvider().identify(myPoint, QgsRaster.IdentifyFormatValue).results()
 
-        assert len( myRasterValues ) > 0
+        assert len(myRasterValues) > 0
 
         # Get the name of the first band
         myBand = myRasterValues.keys()[0]
-        #myExpectedName = 'Band 1
+        # myExpectedName = 'Band 1
         myExpectedBand = 1
         myMessage = 'Expected "%s" got "%s" for first raster band name' % (
                     myExpectedBand, myBand)
@@ -72,7 +70,7 @@ class TestQgsRasterLayer(TestCase):
         myValues = myRasterValues.values()
         myIntValues = []
         for myValue in myValues:
-          myIntValues.append( int(myValue) )
+            myIntValues.append(int(myValue))
         myValues = str(myIntValues)
         myExpectedValues = '[127, 141, 112, 72, 86, 126, 156, 211, 170]'
         myMessage = 'Expected: %s\nGot: %s' % (myValues, myExpectedValues)
@@ -94,9 +92,9 @@ class TestQgsRasterLayer(TestCase):
             QgsRaster.ContrastEnhancementMinMax)
 
         myContrastEnhancement = myRasterLayer.renderer().contrastEnhancement()
-        #print ("myContrastEnhancement.minimumValue = %.17g" %
+        # print ("myContrastEnhancement.minimumValue = %.17g" %
         #       myContrastEnhancement.minimumValue())
-        #print ("myContrastEnhancement.maximumValue = %.17g" %
+        # print ("myContrastEnhancement.maximumValue = %.17g" %
         #        myContrastEnhancement.maximumValue())
 
         # Unfortunately the minimum/maximum values calculated in C++ and Python
@@ -109,8 +107,6 @@ class TestQgsRasterLayer(TestCase):
         #myType = myRasterLayer.dataProvider().dataType(1);
         #myEnhancement = QgsContrastEnhancement(myType);
 
-
-
         myTransparentSingleValuePixelList = []
         rasterTransparency = QgsRasterTransparency()
 
@@ -136,7 +132,7 @@ class TestQgsRasterLayer(TestCase):
 
         rasterRenderer.setRasterTransparency(rasterTransparency)
 
-        QgsMapLayerRegistry.instance().addMapLayers([ myRasterLayer, ])
+        QgsMapLayerRegistry.instance().addMapLayers([myRasterLayer, ])
 
         myMapRenderer = QgsMapRenderer()
 
@@ -149,7 +145,7 @@ class TestQgsRasterLayer(TestCase):
         myChecker.setControlName("expected_raster_transparency")
         myChecker.setMapRenderer(myMapRenderer)
 
-        myResultFlag = myChecker.runTest("raster_transparency_python");
+        myResultFlag = myChecker.runTest("raster_transparency_python")
         assert myResultFlag, "Raster transparency rendering test failed"
 
     def testIssue7023(self):
@@ -190,7 +186,7 @@ class TestQgsRasterLayer(TestCase):
         myColorRampShader.setColorRampItemList(myItems)
         myRasterShader.setRasterShaderFunction(myColorRampShader)
         myPseudoRenderer = QgsSingleBandPseudoColorRenderer(
-            myRasterLayer.dataProvider(), 1,  myRasterShader)
+            myRasterLayer.dataProvider(), 1, myRasterShader)
         myRasterLayer.setRenderer(myPseudoRenderer)
 
         return
@@ -213,12 +209,13 @@ class TestQgsRasterLayer(TestCase):
         myRasterShader.setRasterShaderFunction(myColorRampShader)
         ######## crash on next line (fixed now)##################
         myPseudoRenderer = QgsSingleBandPseudoColorRenderer(
-            myRasterLayer.dataProvider(), 1,  myRasterShader)
+            myRasterLayer.dataProvider(), 1, myRasterShader)
         myRasterLayer.setRenderer(myPseudoRenderer)
 
-    def onRendererChanged( self ):
+    def onRendererChanged(self):
         self.rendererChanged = True
-    def test_setRenderer( self ):
+
+    def test_setRenderer(self):
         myPath = os.path.join(unitTestDataPath('raster'),
                               'band1_float32_noct_epsg4326.tif')
         myFileInfo = QFileInfo(myPath)
@@ -226,14 +223,14 @@ class TestQgsRasterLayer(TestCase):
         layer = QgsRasterLayer(myPath, myBaseName)
 
         self.rendererChanged = False
-        QObject.connect( layer, SIGNAL( "rendererChanged()" ),
-                         self.onRendererChanged )
+        QObject.connect(layer, SIGNAL("rendererChanged()"),
+                        self.onRendererChanged)
 
         rShader = QgsRasterShader()
-        r = QgsSingleBandPseudoColorRenderer( layer.dataProvider(), 1, rShader )
+        r = QgsSingleBandPseudoColorRenderer(layer.dataProvider(), 1, rShader)
 
-        layer.setRenderer( r )
-        assert self.rendererChanged == True
+        layer.setRenderer(r)
+        assert self.rendererChanged
         assert layer.renderer() == r
 
 if __name__ == '__main__':
