@@ -294,7 +294,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Country,Hello",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -311,7 +311,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Hello",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -327,7 +327,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Country",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -351,7 +351,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Hello",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -390,7 +390,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Hello",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -420,7 +420,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Country",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -842,7 +842,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Country,Hello_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -859,7 +859,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Hello_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -882,7 +882,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "FILTER": "Hello_Filter_SubsetString:\"pkuid\" IN ( 7 , 8 )",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -900,7 +900,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "FILTER": "Hello_Filter_SubsetString:\"pkuid\" IN ( 7 , 8 )",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -919,7 +919,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Hello_Project_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -936,7 +936,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "LAYERS": "Hello_Project_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857"
@@ -954,7 +954,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Hello_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -990,7 +990,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Hello_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -1026,7 +1026,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "QUERY_LAYERS": "Hello_Project_SubsetString",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
@@ -1135,7 +1135,7 @@ class TestQgsServerAccessControl(unittest.TestCase):
             "FILTER": "Hello_Filter_SubsetString:\"pkuid\" IN ( 7 , 8 )",
             "STYLES": "",
             "FORMAT": "image/png",
-            "BBOX": "-16817707,-4710778,5696513,14587125",
+            "BBOX": "-16817707,-6318936.5,5696513,16195283.5",
             "HEIGHT": "500",
             "WIDTH": "500",
             "SRS": "EPSG:3857",
